@@ -49,6 +49,7 @@ func (s *controllerSuite) SetUpTest(c *gc.C) {
 	server.AddGetResponse("/api/2.0/machines/", http.StatusOK, machinesResponse)
 	server.AddGetResponse("/api/2.0/machines/?hostname=untasted-markita", http.StatusOK, "["+machineResponse+"]")
 	server.AddGetResponse("/api/2.0/spaces/", http.StatusOK, spacesResponse)
+	server.AddGetResponse("/api/2.0/static-routes/", http.StatusOK, staticRoutesResponse)
 	server.AddGetResponse("/api/2.0/users/?op=whoami", http.StatusOK, `"captain awesome"`)
 	server.AddGetResponse("/api/2.0/version/", http.StatusOK, versionResponse)
 	server.AddGetResponse("/api/2.0/zones/", http.StatusOK, zoneResponse)
@@ -131,6 +132,112 @@ func (s *controllerSuite) TestNewControllerUnexpected(c *gc.C) {
 	c.Assert(err, jc.Satisfies, IsUnexpectedError)
 }
 
+func (s *controllerSuite) TestNewControllerKnownVersion(c *gc.C) {
+	// Using a server URL including the version should work.
+	officialController, err := NewController(ControllerArgs{
+		BaseURL: s.server.URL + "/api/2.0/",
+		APIKey:  "fake:as:key",
+	})
+	c.Assert(err, jc.ErrorIsNil)
+	rawController, ok := officialController.(*controller)
+	c.Assert(ok, jc.IsTrue)
+	c.Assert(rawController.apiVersion, gc.Equals, version.Number{
+		Major: 2,
+		Minor: 0,
+	})
+}
+
+func (s *controllerSuite) TestNewControllerUnsupportedVersionSpecified(c *gc.C) {
+	// Ensure the server would actually respond to the version if it
+	// was asked.
+	s.server.AddGetResponse("/api/3.0/users/?op=whoami", http.StatusOK, `"captain awesome"`)
+	s.server.AddGetResponse("/api/3.0/version/", http.StatusOK, versionResponse)
+	// Using a server URL including a version that isn't in the known
+	// set should be denied.
+	controller, err := NewController(ControllerArgs{
+		BaseURL: s.server.URL + "/api/3.0/",
+		APIKey:  "fake:as:key",
+	})
+	c.Assert(controller, gc.IsNil)
+	c.Assert(err, jc.Satisfies, IsUnsupportedVersionError)
+}
+
+func (s *controllerSuite) TestNewControllerNotHidingErrors(c *gc.C) {
+	// We should only treat 404 and 410 as "this version isn't
+	// supported". Other errors should be returned up the stack
+	// unchanged, so we don't confuse transient network errors with
+	// version mismatches. lp:1667095
+	server := NewSimpleServer()
+	server.AddGetResponse("/api/2.0/users/?op=whoami", http.StatusOK, "underwater woman")
+	server.AddGetResponse("/api/2.0/version/", http.StatusInternalServerError, "kablooey")
+	server.Start()
+	defer server.Close()
+
+	controller, err := NewController(ControllerArgs{
+		BaseURL: server.URL,
+		APIKey:  "fake:as:key",
+	})
+	c.Assert(controller, gc.IsNil)
+	c.Assert(err, gc.ErrorMatches, `ServerError: 500 Internal Server Error \(kablooey\)`)
+}
+
+func (s *controllerSuite) TestNewController410(c *gc.C) {
+	// We should only treat 404 and 410 as "this version isn't
+	// supported". Other errors should be returned up the stack
+	// unchanged, so we don't confuse transient network errors with
+	// version mismatches. lp:1667095
+	server := NewSimpleServer()
+	server.AddGetResponse("/api/2.0/users/?op=whoami", http.StatusOK, "the answer to all your prayers")
+	server.AddGetResponse("/api/2.0/version/", http.StatusGone, "cya")
+	server.Start()
+	defer server.Close()
+
+	controller, err := NewController(ControllerArgs{
+		BaseURL: server.URL,
+		APIKey:  "fake:as:key",
+	})
+	c.Assert(controller, gc.IsNil)
+	c.Assert(err, jc.Satisfies, IsUnsupportedVersionError)
+}
+
+func (s *controllerSuite) TestNewController404(c *gc.C) {
+	// We should only treat 404 and 410 as "this version isn't
+	// supported". Other errors should be returned up the stack
+	// unchanged, so we don't confuse transient network errors with
+	// version mismatches. lp:1667095
+	server := NewSimpleServer()
+	server.AddGetResponse("/api/2.0/users/?op=whoami", http.StatusOK, "the answer to all your prayers")
+	server.AddGetResponse("/api/2.0/version/", http.StatusNotFound, "huh?")
+	server.Start()
+	defer server.Close()
+
+	controller, err := NewController(ControllerArgs{
+		BaseURL: server.URL,
+		APIKey:  "fake:as:key",
+	})
+	c.Assert(controller, gc.IsNil)
+	c.Assert(err, jc.Satisfies, IsUnsupportedVersionError)
+}
+
+func (s *controllerSuite) TestNewControllerWith194Bug(c *gc.C) {
+	// 1.9.4 has a bug where if you ask for /api/2.0/version/ without
+	// being logged in (rather than OAuth connection) it redirects you
+	// to the login page. This is fixed in 1.9.5, but we should work
+	// around it anyway. https://bugs.launchpad.net/maas/+bug/1583715
+	server := NewSimpleServer()
+	server.AddGetResponse("/api/2.0/users/?op=whoami", http.StatusOK, "the answer to all your prayers")
+	server.AddGetResponse("/api/2.0/version/", http.StatusOK, "<html><head>")
+	server.Start()
+	defer server.Close()
+
+	controller, err := NewController(ControllerArgs{
+		BaseURL: server.URL,
+		APIKey:  "fake:as:key",
+	})
+	c.Assert(controller, gc.IsNil)
+	c.Assert(err, jc.Satisfies, IsUnsupportedVersionError)
+}
+
 func (s *controllerSuite) TestBootResources(c *gc.C) {
 	controller := s.getController(c)
 	resources, err := controller.BootResources()
@@ -222,6 +329,13 @@ func (s *controllerSuite) TestSpaces(c *gc.C) {
 	c.Assert(spaces, gc.HasLen, 1)
 }
 
+func (s *controllerSuite) TestStaticRoutes(c *gc.C) {
+	controller := s.getController(c)
+	staticRoutes, err := controller.StaticRoutes()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(staticRoutes, gc.HasLen, 1)
+}
+
 func (s *controllerSuite) TestZones(c *gc.C) {
 	controller := s.getController(c)
 	zones, err := controller.Zones()
