@@ -5,28 +5,30 @@ package api
 
 import (
 	"bufio"
+	"context"
 	"crypto/tls"
 	"crypto/x509"
 	"encoding/json"
 	"fmt"
 	"io"
+	"net"
 	"net/http"
 	"net/url"
 	"strings"
 	"sync/atomic"
 	"time"
 
+	"github.com/gorilla/websocket"
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
-	"github.com/juju/retry"
 	"github.com/juju/utils"
 	"github.com/juju/utils/clock"
 	"github.com/juju/utils/parallel"
 	"github.com/juju/version"
-	"golang.org/x/net/websocket"
 	"gopkg.in/juju/names.v2"
 	"gopkg.in/macaroon-bakery.v1/httpbakery"
 	"gopkg.in/macaroon.v1"
+	"gopkg.in/retry.v1"
 
 	"github.com/juju/juju/api/base"
 	"github.com/juju/juju/apiserver/observer"
@@ -34,6 +36,7 @@ import (
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/rpc"
 	"github.com/juju/juju/rpc/jsoncodec"
+	"github.com/juju/juju/utils/proxy"
 )
 
 // PingPeriod defines how often the internal connection health check
@@ -44,6 +47,14 @@ const PingPeriod = 1 * time.Minute
 // consider it to have failed.
 const pingTimeout = 30 * time.Second
 
+// modelRoot is the prefix that all model API paths begin with.
+const modelRoot = "/model/"
+
+// Use a 64k frame size for the websockets while we need to deal
+// with x/net/websocket connections that don't deal with recieving
+// fragmented messages.
+const websocketFrameSize = 65536
+
 var logger = loggo.GetLogger("juju.api")
 
 type rpcConnection interface {
@@ -55,12 +66,15 @@ type rpcConnection interface {
 // state is the internal implementation of the Connection interface.
 type state struct {
 	client rpcConnection
-	conn   *websocket.Conn
+	conn   jsoncodec.JSONConn
 	clock  clock.Clock
 
 	// addr is the address used to connect to the API server.
 	addr string
 
+	// ipAddr is the IP address used to connect to the API server.
+	ipAddr string
+
 	// cookieURL is the URL that HTTP cookies for the API
 	// will be associated with (specifically macaroon auth cookies).
 	cookieURL *url.URL
@@ -81,6 +95,11 @@ type state struct {
 	// which the client may cache and use for failover.
 	hostPorts [][]network.HostPort
 
+	// publicDNSName is the public host name returned from Login
+	// which the client can use to make a connection verified
+	// by an officially signed certificate.
+	publicDNSName string
+
 	// facadeVersions holds the versions of all facades as reported by
 	// Login
 	facadeVersions map[string][]int
@@ -129,10 +148,6 @@ type state struct {
 	// connections to the API endpoints.
 	tlsConfig *tls.Config
 
-	// certPool holds the cert pool that is used to authenticate the tls
-	// connections to the API.
-	certPool *x509.CertPool
-
 	// bakeryClient holds the client that will be used to
 	// authorize macaroon based login requests.
 	bakeryClient *httpbakery.Client
@@ -164,28 +179,24 @@ func (e *RedirectError) Error() string {
 //
 // See Connect for details of the connection mechanics.
 func Open(info *Info, opts DialOpts) (Connection, error) {
-	return open(info, opts, clock.WallClock)
-}
-
-// open is the unexported version of open that also includes
-// an explicit clock instance argument.
-func open(
-	info *Info,
-	opts DialOpts,
-	clock clock.Clock,
-) (Connection, error) {
 	if err := info.Validate(); err != nil {
 		return nil, errors.Annotate(err, "validating info for opening an API connection")
 	}
-	if clock == nil {
-		return nil, errors.NotValidf("nil clock")
+	if opts.Clock == nil {
+		opts.Clock = clock.WallClock
 	}
-	conn, tlsConfig, err := dialAPI(info, opts)
+	ctx := context.TODO()
+	if opts.Timeout > 0 {
+		ctx1, cancel := utils.ContextWithTimeout(ctx, opts.Clock, opts.Timeout)
+		defer cancel()
+		ctx = ctx1
+	}
+	dialResult, err := dialAPI(ctx, info, opts)
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
 
-	client := rpc.NewConn(jsoncodec.NewWebsocket(conn), observer.None())
+	client := rpc.NewConn(jsoncodec.New(dialResult.conn), observer.None())
 	client.Start()
 
 	bakeryClient := opts.BakeryClient
@@ -198,29 +209,30 @@ func open(
 		httpc := *bakeryClient.Client
 		bakeryClient.Client = &httpc
 	}
-	apiHost := conn.Config().Location.Host
+
 	// Technically when there's no CACert, we don't need this
 	// machinery, because we could just use http.DefaultTransport
 	// for everything, but it's easier just to leave it in place.
 	bakeryClient.Client.Transport = &hostSwitchingTransport{
-		primaryHost: apiHost,
-		primary:     utils.NewHttpTLSTransport(tlsConfig),
+		primaryHost: dialResult.addr,
+		primary:     utils.NewHttpTLSTransport(dialResult.tlsConfig),
 		fallback:    http.DefaultTransport,
 	}
 
 	st := &state{
 		client: client,
-		conn:   conn,
-		clock:  clock,
-		addr:   apiHost,
+		conn:   dialResult.conn,
+		clock:  opts.Clock,
+		addr:   dialResult.addr,
+		ipAddr: dialResult.ipAddr,
 		cookieURL: &url.URL{
 			Scheme: "https",
-			Host:   conn.Config().Location.Host,
+			Host:   dialResult.addr,
 			Path:   "/",
 		},
 		pingerFacadeVersion: facadeVersions["Pinger"],
 		serverScheme:        "https",
-		serverRootAddress:   conn.Config().Location.Host,
+		serverRootAddress:   dialResult.addr,
 		// We populate the username and password before
 		// login because, when doing HTTP requests, we'll want
 		// to use the same username and password for authenticating
@@ -229,13 +241,13 @@ func open(
 		password:     info.Password,
 		macaroons:    info.Macaroons,
 		nonce:        info.Nonce,
-		tlsConfig:    tlsConfig,
+		tlsConfig:    dialResult.tlsConfig,
 		bakeryClient: bakeryClient,
 		modelTag:     info.ModelTag,
 	}
 	if !info.SkipLogin {
-		if err := st.Login(info.Tag, info.Password, info.Nonce, info.Macaroons); err != nil {
-			conn.Close()
+		if err := loginWithContext(ctx, st, info); err != nil {
+			dialResult.conn.Close()
 			return nil, errors.Trace(err)
 		}
 	}
@@ -244,7 +256,7 @@ func open(
 	st.closed = make(chan struct{})
 
 	go (&monitor{
-		clock:       clock,
+		clock:       opts.Clock,
 		ping:        st.Ping,
 		pingPeriod:  PingPeriod,
 		pingTimeout: pingTimeout,
@@ -255,6 +267,42 @@ func open(
 	return st, nil
 }
 
+// loginWithContext wraps st.Login with code that terminates
+// if the context is cancelled.
+// TODO(rogpeppe) pass Context into Login (and all API calls) so
+// that this becomes unnecessary.
+func loginWithContext(ctx context.Context, st *state, info *Info) error {
+	result := make(chan error, 1)
+	go func() {
+		result <- st.Login(info.Tag, info.Password, info.Nonce, info.Macaroons)
+	}()
+	select {
+	case err := <-result:
+		return errors.Trace(err)
+	case <-ctx.Done():
+		return errors.Annotatef(ctx.Err(), "cannot log in")
+	}
+}
+
+type NewConnectionForModelFunc func(*Info) (func(string) (Connection, error), error)
+
+// NewConnectionForModel returns a function which returns a model API
+// connection for a specified model UUID, based on the specified api info.
+// Currently, such a connection will always be to a single controller.
+func NewConnectionForModel(apiInfo *Info) (func(string) (Connection, error), error) {
+	return func(modelUUID string) (Connection, error) {
+		apiInfo.ModelTag = names.NewModelTag(modelUUID)
+		conn, err := Open(apiInfo, DialOpts{
+			Timeout:    time.Second,
+			RetryDelay: 200 * time.Millisecond,
+		})
+		if err != nil {
+			return nil, errors.Annotatef(err, "failed to open API to model %s", modelUUID)
+		}
+		return conn, nil
+	}, nil
+}
+
 // hostSwitchingTransport provides an http.RoundTripper
 // that chooses an actual RoundTripper to use
 // depending on the destination host.
@@ -277,6 +325,34 @@ func (t *hostSwitchingTransport) RoundTrip(req *http.Request) (*http.Response, e
 
 // ConnectStream implements StreamConnector.ConnectStream.
 func (st *state) ConnectStream(path string, attrs url.Values) (base.Stream, error) {
+	path, err := apiPath(st.modelTag, path)
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	conn, err := st.connectStreamWithRetry(path, attrs, nil)
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	return conn, nil
+}
+
+// ConnectControllerStream creates a stream connection to an API path
+// that isn't prefixed with /model/uuid.
+func (st *state) ConnectControllerStream(path string, attrs url.Values, headers http.Header) (base.Stream, error) {
+	if !strings.HasPrefix(path, "/") {
+		return nil, errors.Errorf("path %q is not absolute", path)
+	}
+	if strings.HasPrefix(path, modelRoot) {
+		return nil, errors.Errorf("path %q is model-specific", path)
+	}
+	conn, err := st.connectStreamWithRetry(path, attrs, headers)
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	return conn, nil
+}
+
+func (st *state) connectStreamWithRetry(path string, attrs url.Values, headers http.Header) (base.Stream, error) {
 	if !st.isLoggedIn() {
 		return nil, errors.New("cannot use ConnectStream without logging in")
 	}
@@ -286,7 +362,7 @@ func (st *state) ConnectStream(path string, attrs url.Values) (base.Stream, erro
 	// error, the response will contain a macaroon that, when discharged,
 	// may allow access, so we discharge it (using bakery.Client.HandleError)
 	// and try the request again.
-	conn, err := st.connectStream(path, attrs)
+	conn, err := st.connectStream(path, attrs, headers)
 	if err == nil {
 		return conn, err
 	}
@@ -297,7 +373,7 @@ func (st *state) ConnectStream(path string, attrs url.Values) (base.Stream, erro
 		return nil, errors.Trace(err)
 	}
 	// Try again with the discharged macaroon.
-	conn, err = st.connectStream(path, attrs)
+	conn, err = st.connectStream(path, attrs, headers)
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
@@ -307,34 +383,53 @@ func (st *state) ConnectStream(path string, attrs url.Values) (base.Stream, erro
 // connectStream is the internal version of ConnectStream. It differs from
 // ConnectStream only in that it will not retry the connection if it encounters
 // discharge-required error.
-func (st *state) connectStream(path string, attrs url.Values) (base.Stream, error) {
-	path, err := apiPath(st.modelTag, path)
-	if err != nil {
-		return nil, errors.Trace(err)
-	}
+func (st *state) connectStream(path string, attrs url.Values, extraHeaders http.Header) (base.Stream, error) {
 	target := url.URL{
 		Scheme:   "wss",
 		Host:     st.addr,
 		Path:     path,
 		RawQuery: attrs.Encode(),
 	}
-	cfg, err := websocket.NewConfig(target.String(), "http://localhost/")
-	if st.tag != "" {
-		cfg.Header = utils.BasicAuthHeader(st.tag, st.password)
+	// TODO(macgreagoir) IPv6. Ubuntu still always provides IPv4 loopback,
+	// and when/if this changes localhost should resolve to IPv6 loopback
+	// in any case (lp:1644009). Review.
+
+	dialer := &websocket.Dialer{
+		Proxy:           proxy.DefaultConfig.GetProxy,
+		TLSClientConfig: st.tlsConfig,
+		// In order to deal with the remote side not handling message
+		// fragmentation, we default to largeish frames.
+		ReadBufferSize:  websocketFrameSize,
+		WriteBufferSize: websocketFrameSize,
 	}
+	var requestHeader http.Header
+	if st.tag != "" {
+		requestHeader = utils.BasicAuthHeader(st.tag, st.password)
+	} else {
+		requestHeader = make(http.Header)
+	}
+	requestHeader.Set("Origin", "http://localhost/")
 	if st.nonce != "" {
-		cfg.Header.Set(params.MachineNonceHeader, st.nonce)
+		requestHeader.Set(params.MachineNonceHeader, st.nonce)
 	}
 	// Add any cookies because they will not be sent to websocket
 	// connections by default.
-	st.addCookiesToHeader(cfg.Header)
+	err := st.addCookiesToHeader(requestHeader)
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	for header, values := range extraHeaders {
+		for _, value := range values {
+			requestHeader.Add(header, value)
+		}
+	}
 
-	cfg.TlsConfig = st.tlsConfig
-	connection, err := websocketDialConfig(cfg)
+	connection, err := websocketDial(dialer, target.String(), requestHeader)
 	if err != nil {
 		return nil, err
 	}
 	if err := readInitialStreamError(connection); err != nil {
+		connection.Close()
 		return nil, errors.Trace(err)
 	}
 	return connection, nil
@@ -342,14 +437,21 @@ func (st *state) connectStream(path string, attrs url.Values) (base.Stream, erro
 
 // readInitialStreamError reads the initial error response
 // from a stream connection and returns it.
-func readInitialStreamError(conn io.Reader) error {
+func readInitialStreamError(ws base.Stream) error {
 	// We can use bufio here because the websocket guarantees that a
 	// single read will not read more than a single frame; there is
 	// no guarantee that a single read might not read less than the
 	// whole frame though, so using a single Read call is not
 	// correct. By using ReadSlice rather than ReadBytes, we
 	// guarantee that the error can't be too big (>4096 bytes).
-	line, err := bufio.NewReader(conn).ReadSlice('\n')
+	messageType, reader, err := ws.NextReader()
+	if err != nil {
+		return errors.Annotate(err, "unable to get reader")
+	}
+	if messageType != websocket.TextMessage {
+		return errors.Errorf("unexpected message type %v", messageType)
+	}
+	line, err := bufio.NewReader(reader).ReadSlice('\n')
 	if err != nil {
 		return errors.Annotate(err, "unable to read initial response")
 	}
@@ -366,7 +468,7 @@ func readInitialStreamError(conn io.Reader) error {
 // addCookiesToHeader adds any cookies associated with the
 // API host to the given header. This is necessary because
 // otherwise cookies are not sent to websocket endpoints.
-func (st *state) addCookiesToHeader(h http.Header) {
+func (st *state) addCookiesToHeader(h http.Header) error {
 	// net/http only allows adding cookies to a request,
 	// but when it sends a request to a non-http endpoint,
 	// it doesn't add the cookies, so make a request, starting
@@ -379,6 +481,20 @@ func (st *state) addCookiesToHeader(h http.Header) {
 	for _, c := range cookies {
 		req.AddCookie(c)
 	}
+	if len(cookies) == 0 && len(st.macaroons) > 0 {
+		// These macaroons must have been added directly rather than
+		// obtained from a request. Add them. (For example in the
+		// logtransfer connection for a migration.)
+		// See https://bugs.launchpad.net/juju/+bug/1650451
+		for _, macaroon := range st.macaroons {
+			cookie, err := httpbakery.NewCookie(macaroon)
+			if err != nil {
+				return errors.Trace(err)
+			}
+			req.AddCookie(cookie)
+		}
+	}
+	return nil
 }
 
 // apiEndpoint returns a URL that refers to the given API slash-prefixed
@@ -411,7 +527,7 @@ func apiPath(modelTag names.ModelTag, path string) (string, error) {
 	if modelUUID == "" {
 		return path, nil
 	}
-	return "/model/" + modelUUID + path, nil
+	return modelRoot + modelUUID + path, nil
 }
 
 // tagToString returns the value of a tag's String method, or "" if the tag is nil.
@@ -422,70 +538,199 @@ func tagToString(tag names.Tag) string {
 	return tag.String()
 }
 
+// dialResult holds a dialled connection, the URL
+// and TLS configuration used to connect to it.
+type dialResult struct {
+	conn      jsoncodec.JSONConn
+	addr      string
+	urlStr    string
+	ipAddr    string
+	tlsConfig *tls.Config
+}
+
+// Close implements io.Closer by closing the websocket
+// connection. It is implemented so that a *dialResult
+// value can be used as the result of a parallel.Try.
+func (c *dialResult) Close() error {
+	return c.conn.Close()
+}
+
+// dialOpts holds the original dial options
+// but adds some information for the local dial logic.
+type dialOpts struct {
+	DialOpts
+	sniHostName string
+	deadline    time.Time
+	// certPool holds a cert pool containing the CACert
+	// if there is one.
+	certPool *x509.CertPool
+}
+
 // dialAPI establishes a websocket connection to the RPC
 // API websocket on the API server using Info. If multiple API addresses
 // are provided in Info they will be tried concurrently - the first successful
 // connection wins.
 //
 // It also returns the TLS configuration that it has derived from the Info.
-func dialAPI(info *Info, opts DialOpts) (*websocket.Conn, *tls.Config, error) {
-	// Set opts.DialWebsocket here rather than in open because
-	// some tests call dialAPI directly.
-	if opts.DialWebsocket == nil {
-		opts.DialWebsocket = websocket.DialConfig
-	}
+func dialAPI(ctx context.Context, info *Info, opts0 DialOpts) (*dialResult, error) {
 	if len(info.Addrs) == 0 {
-		return nil, nil, errors.New("no API addresses to connect to")
+		return nil, errors.New("no API addresses to connect to")
+	}
+	opts := dialOpts{
+		DialOpts:    opts0,
+		sniHostName: info.SNIHostName,
 	}
-	tlsConfig := utils.SecureTLSConfig()
-	tlsConfig.InsecureSkipVerify = opts.InsecureSkipVerify
-
 	if info.CACert != "" {
-		// We want to be specific here (rather than just using "anything".
-		// See commit 7fc118f015d8480dfad7831788e4b8c0432205e8 (PR 899).
-		tlsConfig.ServerName = "juju-apiserver"
 		certPool, err := CreateCertPool(info.CACert)
 		if err != nil {
-			return nil, nil, errors.Annotate(err, "cert pool creation failed")
+			return nil, errors.Annotate(err, "cert pool creation failed")
 		}
-		tlsConfig.RootCAs = certPool
-	} else {
-		// No CA certificate so use the SNI host name for all
-		// connections (if SNIHostName is empty, the host
-		// name in the address will be used as usual).
-		tlsConfig.ServerName = info.SNIHostName
+		opts.certPool = certPool
+	}
+	// Set opts.DialWebsocket and opts.Clock here rather than in open because
+	// some tests call dialAPI directly.
+	if opts.DialWebsocket == nil {
+		opts.DialWebsocket = gorillaDialWebsocket
+	}
+	if opts.IPAddrResolver == nil {
+		opts.IPAddrResolver = net.DefaultResolver
+	}
+	if opts.Clock == nil {
+		opts.Clock = clock.WallClock
+	}
+	if opts.DNSCache == nil {
+		opts.DNSCache = nopDNSCache{}
 	}
 	path, err := apiPath(info.ModelTag, "/api")
 	if err != nil {
-		return nil, nil, errors.Trace(err)
+		return nil, errors.Trace(err)
 	}
-	conn, err := dialWebsocketMulti(info.Addrs, path, tlsConfig, opts)
+	if opts.DialTimeout > 0 {
+		ctx1, cancel := utils.ContextWithTimeout(ctx, opts.Clock, opts.DialTimeout)
+		defer cancel()
+		ctx = ctx1
+	}
+	dialInfo, err := dialWebsocketMulti(ctx, info.Addrs, path, opts)
 	if err != nil {
-		return nil, nil, errors.Trace(err)
+		return nil, errors.Trace(err)
 	}
-	logger.Infof("connection established to %q", conn.RemoteAddr())
-	return conn, tlsConfig, nil
+	logger.Infof("connection established to %q", dialInfo.urlStr)
+	return dialInfo, nil
+}
+
+// gorillaDialWebsocket makes a websocket connection using the
+// gorilla websocket package. The ipAddr parameter holds the
+// actual IP address that will be contacted - the host in urlStr
+// is used only for TLS verification when tlsConfig.ServerName
+// is empty.
+func gorillaDialWebsocket(ctx context.Context, urlStr string, tlsConfig *tls.Config, ipAddr string) (jsoncodec.JSONConn, error) {
+	url, err := url.Parse(urlStr)
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	// TODO(rogpeppe) We'd like to set Deadline here
+	// but that would break lots of tests that rely on
+	// setting a zero timeout.
+	netDialer := net.Dialer{}
+	dialer := &websocket.Dialer{
+		NetDial: func(netw, addr string) (net.Conn, error) {
+			if addr == url.Host {
+				// Use pre-resolved IP address. The address
+				// may be different if a proxy is in use.
+				addr = ipAddr
+			}
+			return netDialer.DialContext(ctx, netw, addr)
+		},
+		Proxy:           proxy.DefaultConfig.GetProxy,
+		TLSClientConfig: tlsConfig,
+		// In order to deal with the remote side not handling message
+		// fragmentation, we default to largeish frames.
+		ReadBufferSize:  websocketFrameSize,
+		WriteBufferSize: websocketFrameSize,
+	}
+	// Note: no extra headers.
+	c, _, err := dialer.Dial(urlStr, nil)
+	if err != nil {
+		return nil, err
+	}
+	return jsoncodec.NewWebsocketConn(c), nil
 }
 
 // dialWebsocketMulti dials a websocket with one of the provided addresses, the
 // specified URL path, TLS configuration, and dial options. Each of the
 // specified addresses will be attempted concurrently, and the first
 // successful connection will be returned.
-func dialWebsocketMulti(addrs []string, path string, tlsConfig *tls.Config, opts DialOpts) (*websocket.Conn, error) {
+func dialWebsocketMulti(ctx context.Context, addrs []string, path string, opts dialOpts) (*dialResult, error) {
 	// Dial all addresses at reasonable intervals.
 	try := parallel.NewTry(0, nil)
 	defer try.Kill()
-	for _, addr := range addrs {
-		err := startDialWebsocket(try, addr, path, opts, tlsConfig)
-		if err == parallel.ErrStopped {
+	// Make a context that's cancelled when the try
+	// completes so that (for example) a slow DNS
+	// query will be cancelled if a previous try succeeds.
+	ctx, cancel := context.WithCancel(ctx)
+	go func() {
+		<-try.Dead()
+		cancel()
+	}()
+	tried := make(map[string]bool)
+	var cacheUsed []string
+	for {
+		if len(addrs) == 0 && len(cacheUsed) > 0 {
+			// We've tried all the addresses but for some
+			// of them we used cached values which might
+			// have become out of date, so retry them
+			// with no cache.
+			addrs = cacheUsed
+			cacheUsed = nil
+			opts.DNSCache = emptyDNSCache{opts.DNSCache}
+		}
+		if len(addrs) == 0 {
 			break
 		}
+		addr := addrs[0]
+		addrs = addrs[1:]
+		host, port, err := net.SplitHostPort(addr)
 		if err != nil {
-			return nil, errors.Trace(err)
+			// Defensive - this should never happen because
+			// the addresses are checked with Info.Validate
+			// beforehand.
+			err := errors.Errorf("invalid address %q: %v", addr, err)
+			recordTryError(try, err)
+			continue
 		}
-		select {
-		case <-time.After(opts.DialAddressInterval):
-		case <-try.Dead():
+		ips := opts.DNSCache.Lookup(host)
+		if len(ips) > 0 {
+			cacheUsed = append(cacheUsed, addr)
+		} else if isNumericHost(host) {
+			ips = []string{host}
+		} else {
+			var err error
+			ips, err = lookupIPAddr(ctx, host, opts.IPAddrResolver)
+			if err != nil {
+				err := errors.Errorf("cannot resolve %q: %v", host, err)
+				recordTryError(try, err)
+				continue
+			}
+			opts.DNSCache.Add(host, ips)
+			logger.Debugf("looked up %v -> %v", host, ips)
+		}
+		for _, ip := range ips {
+			ipStr := net.JoinHostPort(ip, port)
+			if tried[ipStr] {
+				continue
+			}
+			tried[ipStr] = true
+			err := startDialWebsocket(ctx, try, ipStr, addr, path, opts)
+			if err == parallel.ErrStopped {
+				break
+			}
+			if err != nil {
+				return nil, errors.Trace(err)
+			}
+			select {
+			case <-opts.Clock.After(opts.DialAddressInterval):
+			case <-try.Dead():
+			}
 		}
 	}
 	try.Close()
@@ -493,84 +738,194 @@ func dialWebsocketMulti(addrs []string, path string, tlsConfig *tls.Config, opts
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
-	return result.(*websocket.Conn), nil
+	return result.(*dialResult), nil
 }
 
+func lookupIPAddr(ctx context.Context, host string, resolver IPAddrResolver) ([]string, error) {
+	addrs, err := resolver.LookupIPAddr(ctx, host)
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	ips := make([]string, 0, len(addrs))
+	for _, addr := range addrs {
+		if addr.Zone != "" {
+			// Ignore IPv6 zone. Hopefully this shouldn't
+			// cause any problems in practice.
+			logger.Infof("ignoring IP address with zone %q", addr)
+			continue
+		}
+		ips = append(ips, addr.IP.String())
+	}
+	return ips, nil
+}
+
+// recordTryError starts a try that just returns the given error.
+// This is so that we can use the usual Try error combination
+// logic even for errors that happen before we start a try.
+func recordTryError(try *parallel.Try, err error) {
+	logger.Infof("%v", err)
+	try.Start(func(_ <-chan struct{}) (io.Closer, error) {
+		return nil, errors.Trace(err)
+	})
+}
+
+var oneAttempt = retry.LimitCount(1, retry.Regular{
+	Min: 1,
+})
+
 // startDialWebsocket starts websocket connection to a single address
 // on the given try instance.
-func startDialWebsocket(try *parallel.Try, addr, path string, opts DialOpts, tlsConfig *tls.Config) error {
-	// origin is required by the WebSocket API, used for "origin policy"
-	// in websockets. We pass localhost to satisfy the API; it is
-	// inconsequential to us.
-	const origin = "http://localhost/"
-	cfg, err := websocket.NewConfig("wss://"+addr+path, origin)
-	if err != nil {
-		return errors.Trace(err)
+func startDialWebsocket(ctx context.Context, try *parallel.Try, ipAddr, addr, path string, opts dialOpts) error {
+	var openAttempt retry.Strategy
+	if opts.RetryDelay > 0 {
+		openAttempt = retry.Regular{
+			Total: opts.Timeout,
+			Delay: opts.RetryDelay,
+			Min:   int(opts.Timeout / opts.RetryDelay),
+		}
+	} else {
+		// Zero retry delay implies exactly one try.
+		openAttempt = oneAttempt
 	}
-	cfg.TlsConfig = tlsConfig
-	return try.Start(newWebsocketDialer(cfg, opts))
+	d := dialer{
+		ctx:         ctx,
+		openAttempt: openAttempt,
+		serverName:  opts.sniHostName,
+		ipAddr:      ipAddr,
+		urlStr:      "wss://" + addr + path,
+		addr:        addr,
+		opts:        opts,
+	}
+	return try.Start(d.dial)
 }
 
-// newWebsocketDialer0 returns a function that dials the websocket represented
-// by the given configuration with the given dial options, suitable for passing
-// to utils/parallel.Try.Start.
-func newWebsocketDialer(cfg *websocket.Config, opts DialOpts) func(<-chan struct{}) (io.Closer, error) {
-	// TODO(katco): 2016-08-09: lp:1611427
-	openAttempt := utils.AttemptStrategy{
-		Total: opts.Timeout,
-		Delay: opts.RetryDelay,
-	}
-	return func(stop <-chan struct{}) (io.Closer, error) {
-		for a := openAttempt.Start(); a.Next(); {
-			select {
-			case <-stop:
-				return nil, parallel.ErrStopped
-			default:
-			}
-			logger.Infof("dialing %q", cfg.Location)
-			conn, err := opts.DialWebsocket(cfg)
-			if err == nil {
-				return conn, nil
-			}
-			if !a.HasNext() || isX509Error(err) {
-				// We won't reconnect when there's an X509 error
-				// because we're not going to succeed if we retry
-				// in that case.
-				logger.Infof("error dialing %q: %v", cfg.Location, err)
-				return nil, errors.Annotatef(err, "unable to connect to API")
-			}
+type dialer struct {
+	ctx         context.Context
+	openAttempt retry.Strategy
+
+	// serverName holds the SNI name to use
+	// when connecting with a public certificate.
+	serverName string
+
+	// addr holds the host:port that is being dialed.
+	addr string
+
+	// addr holds the ipaddr:port (one of the addresses
+	// that addr resolves to) that is being dialed.
+	ipAddr string
+
+	// urlStr holds the URL that is being dialed.
+	urlStr string
+
+	// opts holds the dial options.
+	opts dialOpts
+}
+
+// dial implements the function value expected by Try.Start
+// by dialing the websocket as specified in d and retrying
+// when appropriate.
+func (d dialer) dial(_ <-chan struct{}) (io.Closer, error) {
+	a := retry.StartWithCancel(d.openAttempt, d.opts.Clock, d.ctx.Done())
+	for a.Next() {
+		conn, tlsConfig, err := d.dial1()
+		if err == nil {
+			return &dialResult{
+				conn:      conn,
+				addr:      d.addr,
+				ipAddr:    d.ipAddr,
+				urlStr:    d.urlStr,
+				tlsConfig: tlsConfig,
+			}, nil
+		}
+		if isX509Error(err) || !a.More() {
+			// certificate errors don't improve with retries.
+			logger.Debugf("error dialing websocket: %v", err)
+			return nil, errors.Annotatef(err, "unable to connect to API")
 		}
-		panic("unreachable")
 	}
+	return nil, parallel.ErrStopped
+}
+
+// dial1 makes a single dial attempt.
+func (d dialer) dial1() (jsoncodec.JSONConn, *tls.Config, error) {
+	tlsConfig := utils.SecureTLSConfig()
+	tlsConfig.InsecureSkipVerify = d.opts.InsecureSkipVerify
+	if d.opts.certPool != nil {
+		// We want to be specific here (rather than just using "anything").
+		// See commit 7fc118f015d8480dfad7831788e4b8c0432205e8 (PR 899).
+		tlsConfig.RootCAs = d.opts.certPool
+		tlsConfig.ServerName = "juju-apiserver"
+	} else {
+		tlsConfig.ServerName = d.serverName
+	}
+	conn, err := d.opts.DialWebsocket(d.ctx, d.urlStr, tlsConfig, d.ipAddr)
+	if err == nil {
+		logger.Debugf("successfully dialed %q", d.urlStr)
+		return conn, tlsConfig, nil
+	}
+	if !isX509Error(err) {
+		return nil, nil, errors.Trace(err)
+	}
+	if tlsConfig.RootCAs == nil || d.serverName == "" {
+		// There's no private certificate or we don't have a
+		// public hostname. In the former case, we've already
+		// tried public certificates; in the latter, public cert
+		// validation won't help, because you generally can't
+		// obtain a public cert for a numeric IP address. In
+		// both those cases, we won't succeed when trying again
+		// because a cert error isn't temporary, so return
+		// immediately.
+		//
+		// Note that the error returned from
+		// websocket.DialConfig always includes the location in
+		// the message.
+		return nil, nil, errors.Trace(err)
+	}
+	// It's possible we're inappropriately using the private
+	// CA certificate, so retry immediately with the public one.
+	tlsConfig.RootCAs = nil
+	tlsConfig.ServerName = d.serverName
+	conn, rootCAErr := d.opts.DialWebsocket(d.ctx, d.urlStr, tlsConfig, d.ipAddr)
+	if rootCAErr != nil {
+		logger.Debugf("failed to dial websocket using fallback public CA: %v", rootCAErr)
+		// We return the original error as it's usually more meaningful.
+		return nil, nil, errors.Trace(err)
+	}
+	return conn, tlsConfig, nil
+}
+
+// isNumericHost reports whether the given host name is
+// a numeric IP address.
+func isNumericHost(host string) bool {
+	return net.ParseIP(host) != nil
 }
 
 // isX509Error reports whether the given websocket error
 // results from an X509 problem.
 func isX509Error(err error) bool {
-	wsErr, ok := errors.Cause(err).(*websocket.DialError)
-	if !ok {
-		return false
-	}
-	switch wsErr.Err.(type) {
-	case x509.HostnameError,
+	switch errType := errors.Cause(err).(type) {
+	case *websocket.CloseError:
+		return errType.Code == websocket.CloseTLSHandshake
+	case x509.CertificateInvalidError,
+		x509.HostnameError,
 		x509.InsecureAlgorithmError,
 		x509.UnhandledCriticalExtension,
 		x509.UnknownAuthorityError,
 		x509.ConstraintViolationError,
 		x509.SystemRootsError:
 		return true
+	default:
+		return false
 	}
-	switch err {
-	case x509.ErrUnsupportedAlgorithm,
-		x509.IncorrectPasswordError:
-		return true
-	}
-	return false
 }
 
-type hasErrorCode interface {
-	ErrorCode() string
-}
+var apiCallRetryStrategy = retry.LimitTime(10*time.Second,
+	retry.Exponential{
+		Initial:  100 * time.Millisecond,
+		Factor:   2,
+		MaxDelay: 1500 * time.Millisecond,
+	},
+)
 
 // APICall places a call to the remote machine.
 //
@@ -578,31 +933,21 @@ type hasErrorCode interface {
 // object id, and the specific RPC method. It marshalls the Arguments, and will
 // unmarshall the result into the response object that is supplied.
 func (s *state) APICall(facade string, version int, id, method string, args, response interface{}) error {
-	retrySpec := retry.CallArgs{
-		Func: func() error {
-			return s.client.Call(rpc.Request{
-				Type:    facade,
-				Version: version,
-				Id:      id,
-				Action:  method,
-			}, args, response)
-		},
-		IsFatalError: func(err error) bool {
-			err = errors.Cause(err)
-			ec, ok := err.(hasErrorCode)
-			if !ok {
-				return true
-			}
-			return ec.ErrorCode() != params.CodeRetry
-		},
-		Delay:       100 * time.Millisecond,
-		MaxDelay:    1500 * time.Millisecond,
-		MaxDuration: 10 * time.Second,
-		BackoffFunc: retry.DoubleDelay,
-		Clock:       s.clock,
+	for a := retry.Start(apiCallRetryStrategy, s.clock); a.Next(); {
+		err := s.client.Call(rpc.Request{
+			Type:    facade,
+			Version: version,
+			Id:      id,
+			Action:  method,
+		}, args, response)
+		if params.ErrCode(err) != params.CodeRetry {
+			return errors.Trace(err)
+		}
+		if !a.More() {
+			return errors.Annotatef(err, "too many retries")
+		}
 	}
-	err := retry.Call(retrySpec)
-	return errors.Trace(err)
+	panic("unreachable")
 }
 
 func (s *state) Close() error {
@@ -640,6 +985,12 @@ func (s *state) Addr() string {
 	return s.addr
 }
 
+// IPAddr returns the resolved IP address that was used to
+// connect to the API server.
+func (s *state) IPAddr() string {
+	return s.ipAddr
+}
+
 // ModelTag implements base.APICaller.ModelTag.
 func (s *state) ModelTag() (names.ModelTag, bool) {
 	return s.modelTag, s.modelTag.Id() != ""
@@ -668,6 +1019,14 @@ func (s *state) APIHostPorts() [][]network.HostPort {
 	return hostPorts
 }
 
+// PublicDNSName returns the host name for which an officially
+// signed certificate will be used for TLS connection to the server.
+// If empty, the private Juju CA certificate must be used to verify
+// the connection.
+func (s *state) PublicDNSName() string {
+	return s.publicDNSName
+}
+
 // AllFacadeVersions returns what versions we know about for all facades
 func (s *state) AllFacadeVersions() map[string][]int {
 	facades := make(map[string][]int, len(s.facadeVersions))
@@ -700,3 +1059,23 @@ func (s *state) isLoggedIn() bool {
 func (s *state) setLoggedIn() {
 	atomic.StoreInt32(&s.loggedIn, 1)
 }
+
+// emptyDNSCache implements DNSCache by
+// never returning any entries but writing any
+// added entries to the embedded DNSCache object.
+type emptyDNSCache struct {
+	DNSCache
+}
+
+func (emptyDNSCache) Lookup(host string) []string {
+	return nil
+}
+
+type nopDNSCache struct{}
+
+func (nopDNSCache) Lookup(host string) []string {
+	return nil
+}
+
+func (nopDNSCache) Add(host string, ips []string) {
+}
