@@ -13,19 +13,21 @@ import (
 	"net/url"
 	"path"
 	"strings"
+	"time"
 
+	"github.com/gorilla/websocket"
 	"github.com/juju/errors"
 	"github.com/juju/httprequest"
 	"github.com/juju/loggo"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/version"
-	"golang.org/x/net/websocket"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/charm.v6-unstable"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/api"
 	"github.com/juju/juju/api/base"
+	"github.com/juju/juju/api/common"
 	"github.com/juju/juju/apiserver/params"
 	jujunames "github.com/juju/juju/juju/names"
 	jujutesting "github.com/juju/juju/juju/testing"
@@ -356,7 +358,7 @@ func (s *clientSuite) TestWatchDebugLogConnected(c *gc.C) {
 	// Use the no tail option so we don't try to start a tailing cursor
 	// on the oplog when there is no oplog configured in mongo as the tests
 	// don't set up mongo in replicaset mode.
-	messages, err := client.WatchDebugLog(api.DebugLogParams{NoTail: true})
+	messages, err := client.WatchDebugLog(common.DebugLogParams{NoTail: true})
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(messages, gc.NotNil)
 }
@@ -368,7 +370,7 @@ func (s *clientSuite) TestConnectStreamRequiresSlashPathPrefix(c *gc.C) {
 }
 
 func (s *clientSuite) TestConnectStreamErrorBadConnection(c *gc.C) {
-	s.PatchValue(api.WebsocketDialConfig, func(_ *websocket.Config) (base.Stream, error) {
+	s.PatchValue(api.WebsocketDial, func(_ *websocket.Dialer, _ string, _ http.Header) (base.Stream, error) {
 		return nil, fmt.Errorf("bad connection")
 	})
 	reader, err := s.APIState.ConnectStream("/", nil)
@@ -377,7 +379,7 @@ func (s *clientSuite) TestConnectStreamErrorBadConnection(c *gc.C) {
 }
 
 func (s *clientSuite) TestConnectStreamErrorNoData(c *gc.C) {
-	s.PatchValue(api.WebsocketDialConfig, func(_ *websocket.Config) (base.Stream, error) {
+	s.PatchValue(api.WebsocketDial, func(_ *websocket.Dialer, _ string, _ http.Header) (base.Stream, error) {
 		return fakeStreamReader{&bytes.Buffer{}}, nil
 	})
 	reader, err := s.APIState.ConnectStream("/", nil)
@@ -386,7 +388,7 @@ func (s *clientSuite) TestConnectStreamErrorNoData(c *gc.C) {
 }
 
 func (s *clientSuite) TestConnectStreamErrorBadData(c *gc.C) {
-	s.PatchValue(api.WebsocketDialConfig, func(_ *websocket.Config) (base.Stream, error) {
+	s.PatchValue(api.WebsocketDial, func(_ *websocket.Dialer, _ string, _ http.Header) (base.Stream, error) {
 		return fakeStreamReader{strings.NewReader("junk\n")}, nil
 	})
 	reader, err := s.APIState.ConnectStream("/", nil)
@@ -395,7 +397,7 @@ func (s *clientSuite) TestConnectStreamErrorBadData(c *gc.C) {
 }
 
 func (s *clientSuite) TestConnectStreamErrorReadError(c *gc.C) {
-	s.PatchValue(api.WebsocketDialConfig, func(_ *websocket.Config) (base.Stream, error) {
+	s.PatchValue(api.WebsocketDial, func(_ *websocket.Dialer, _ string, _ http.Header) (base.Stream, error) {
 		err := fmt.Errorf("bad read")
 		return fakeStreamReader{&badReader{err}}, nil
 	})
@@ -404,11 +406,37 @@ func (s *clientSuite) TestConnectStreamErrorReadError(c *gc.C) {
 	c.Assert(reader, gc.IsNil)
 }
 
+func (s *clientSuite) TestConnectControllerStreamRejectsRelativePaths(c *gc.C) {
+	reader, err := s.APIState.ConnectControllerStream("foo", nil, nil)
+	c.Assert(err, gc.ErrorMatches, `path "foo" is not absolute`)
+	c.Assert(reader, gc.IsNil)
+}
+
+func (s *clientSuite) TestConnectControllerStreamRejectsModelPaths(c *gc.C) {
+	reader, err := s.APIState.ConnectControllerStream("/model/foo", nil, nil)
+	c.Assert(err, gc.ErrorMatches, `path "/model/foo" is model-specific`)
+	c.Assert(reader, gc.IsNil)
+}
+
+func (s *clientSuite) TestConnectControllerStreamAppliesHeaders(c *gc.C) {
+	catcher := urlCatcher{}
+	headers := http.Header{}
+	headers.Add("thomas", "cromwell")
+	headers.Add("anne", "boleyn")
+	s.PatchValue(api.WebsocketDial, catcher.recordLocation)
+
+	_, err := s.APIState.ConnectControllerStream("/something", nil, headers)
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Assert(catcher.headers.Get("thomas"), gc.Equals, "cromwell")
+	c.Assert(catcher.headers.Get("anne"), gc.Equals, "boleyn")
+}
+
 func (s *clientSuite) TestWatchDebugLogParamsEncoded(c *gc.C) {
 	catcher := urlCatcher{}
-	s.PatchValue(api.WebsocketDialConfig, catcher.recordLocation)
+	s.PatchValue(api.WebsocketDial, catcher.recordLocation)
 
-	params := api.DebugLogParams{
+	params := common.DebugLogParams{
 		IncludeEntity: []string{"a", "b"},
 		IncludeModule: []string{"c", "d"},
 		ExcludeEntity: []string{"e", "f"},
@@ -418,13 +446,16 @@ func (s *clientSuite) TestWatchDebugLogParamsEncoded(c *gc.C) {
 		Level:         loggo.ERROR,
 		Replay:        true,
 		NoTail:        true,
+		StartTime:     time.Date(2016, 11, 30, 11, 48, 0, 100, time.UTC),
 	}
 
 	client := s.APIState.Client()
 	_, err := client.WatchDebugLog(params)
 	c.Assert(err, jc.ErrorIsNil)
 
-	connectURL := catcher.location
+	connectURL, err := url.Parse(catcher.location)
+	c.Assert(err, jc.ErrorIsNil)
+
 	values := connectURL.Query()
 	c.Assert(values, jc.DeepEquals, url.Values{
 		"includeEntity": params.IncludeEntity,
@@ -436,12 +467,13 @@ func (s *clientSuite) TestWatchDebugLogParamsEncoded(c *gc.C) {
 		"level":         {"ERROR"},
 		"replay":        {"true"},
 		"noTail":        {"true"},
+		"startTime":     {"2016-11-30T11:48:00.0000001Z"},
 	})
 }
 
 func (s *clientSuite) TestConnectStreamAtUUIDPath(c *gc.C) {
 	catcher := urlCatcher{}
-	s.PatchValue(api.WebsocketDialConfig, catcher.recordLocation)
+	s.PatchValue(api.WebsocketDial, catcher.recordLocation)
 	model, err := s.State.Model()
 	c.Assert(err, jc.ErrorIsNil)
 	info := s.APIInfo(c)
@@ -451,7 +483,8 @@ func (s *clientSuite) TestConnectStreamAtUUIDPath(c *gc.C) {
 	defer apistate.Close()
 	_, err = apistate.ConnectStream("/path", nil)
 	c.Assert(err, jc.ErrorIsNil)
-	connectURL := catcher.location
+	connectURL, err := url.Parse(catcher.location)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(connectURL.Path, gc.Matches, fmt.Sprintf("/model/%s/path", model.UUID()))
 }
 
@@ -494,7 +527,7 @@ func (s *clientSuite) TestSetModelAgentVersionDuringUpgrade(c *gc.C) {
 	_, err = s.State.EnsureUpgradeInfo(machine.Id(), agentVersion, nextVersion)
 	c.Assert(err, jc.ErrorIsNil)
 
-	err = s.APIState.Client().SetModelAgentVersion(nextVersion)
+	err = s.APIState.Client().SetModelAgentVersion(nextVersion, false)
 
 	// Expect an error with a error code that indicates this specific
 	// situation. The client needs to be able to reliably identify
@@ -529,11 +562,13 @@ func (r *badReader) Read(p []byte) (n int, err error) {
 }
 
 type urlCatcher struct {
-	location *url.URL
+	location string
+	headers  http.Header
 }
 
-func (u *urlCatcher) recordLocation(config *websocket.Config) (base.Stream, error) {
-	u.location = config.Location
+func (u *urlCatcher) recordLocation(d *websocket.Dialer, urlStr string, header http.Header) (base.Stream, error) {
+	u.location = urlStr
+	u.headers = header
 	pr, pw := io.Pipe()
 	go func() {
 		fmt.Fprintf(pw, "null\n")
@@ -552,6 +587,10 @@ func (s fakeStreamReader) Close() error {
 	return nil
 }
 
+func (s fakeStreamReader) NextReader() (messageType int, r io.Reader, err error) {
+	return websocket.TextMessage, s.Reader, nil
+}
+
 func (s fakeStreamReader) Write([]byte) (int, error) {
 	return 0, errors.NotImplementedf("Write")
 }
