@@ -12,15 +12,15 @@ package api
 // Facades that existed before versioning start at 0.
 var facadeVersions = map[string]int{
 	"Action":                       2,
+	"ActionPruner":                 1,
 	"Agent":                        2,
 	"AgentTools":                   1,
 	"AllModelWatcher":              2,
 	"AllWatcher":                   1,
 	"Annotations":                  2,
-	"Application":                  3,
-	"ApplicationScaler":            1,
+	"Application":                  4,
 	"ApplicationOffers":            1,
-	"ApplicationRelationsWatcher":  1,
+	"ApplicationScaler":            1,
 	"Backups":                      1,
 	"Block":                        2,
 	"Bundle":                       1,
@@ -30,9 +30,7 @@ var facadeVersions = map[string]int{
 	"Client":                       1,
 	"Cloud":                        1,
 	"Controller":                   3,
-	"CrossModelRelations":          1,
 	"Deployer":                     1,
-	"DiscoverSpaces":               2,
 	"DiskManager":                  2,
 	"EntityWatcher":                2,
 	"FilesystemAttachmentsWatcher": 2,
@@ -49,7 +47,7 @@ var facadeVersions = map[string]int{
 	"LogForwarding":                1,
 	"Logger":                       1,
 	"MachineActions":               1,
-	"MachineManager":               2,
+	"MachineManager":               4,
 	"MachineUndertaker":            1,
 	"Machiner":                     1,
 	"MeterStatus":                  1,
@@ -62,23 +60,24 @@ var facadeVersions = map[string]int{
 	"MigrationStatusWatcher":       1,
 	"MigrationTarget":              1,
 	"ModelConfig":                  1,
-	"ModelManager":                 2,
+	"ModelManager":                 3,
+	"ModelUpgrader":                1,
 	"NotifyWatcher":                1,
-	"OfferedApplications":          1,
 	"Payloads":                     1,
 	"PayloadsHookContext":          1,
 	"Pinger":                       1,
-	"Provisioner":                  3,
+	"Provisioner":                  4,
 	"ProxyUpdater":                 1,
 	"Reboot":                       2,
 	"RelationUnitsWatcher":         1,
+	"RemoteFirewaller":             1,
 	"RemoteRelations":              1,
 	"Resources":                    1,
 	"ResourcesHookContext":         1,
 	"Resumer":                      2,
 	"RetryStrategy":                1,
 	"Singular":                     1,
-	"Spaces":                       2,
+	"Spaces":                       3,
 	"SSHClient":                    2,
 	"StatusHistory":                2,
 	"Storage":                      3,
@@ -87,7 +86,7 @@ var facadeVersions = map[string]int{
 	"Subnets":                      2,
 	"Undertaker":                   1,
 	"UnitAssigner":                 1,
-	"Uniter":                       4,
+	"Uniter":                       6,
 	"Upgrader":                     1,
 	"UserManager":                  1,
 	"VolumeAttachmentsWatcher":     2,
