@@ -4,12 +4,16 @@
 package api
 
 import (
+	"context"
+	"crypto/tls"
+	"net"
 	"net/url"
 	"time"
 
 	"github.com/juju/errors"
+	"github.com/juju/utils/clock"
+	"github.com/juju/utils/set"
 	"github.com/juju/version"
-	"golang.org/x/net/websocket"
 	"gopkg.in/juju/names.v2"
 	"gopkg.in/macaroon-bakery.v1/httpbakery"
 	"gopkg.in/macaroon.v1"
@@ -17,7 +21,6 @@ import (
 	"github.com/juju/juju/api/base"
 	"github.com/juju/juju/api/charmrevisionupdater"
 	"github.com/juju/juju/api/cleaner"
-	"github.com/juju/juju/api/discoverspaces"
 	"github.com/juju/juju/api/imagemetadata"
 	"github.com/juju/juju/api/instancepoller"
 	"github.com/juju/juju/api/reboot"
@@ -25,7 +28,7 @@ import (
 	"github.com/juju/juju/api/uniter"
 	"github.com/juju/juju/api/upgrader"
 	"github.com/juju/juju/network"
-	"github.com/juju/utils/set"
+	"github.com/juju/juju/rpc/jsoncodec"
 )
 
 // Info encapsulates information about a server holding juju state and
@@ -121,12 +124,18 @@ type DialOpts struct {
 	// before starting to dial another address.
 	DialAddressInterval time.Duration
 
-	// Timeout is the amount of time to wait contacting
-	// a controller.
+	// DialTimeout is the amount of time to wait for the dial
+	// portion only of the api.Open to succeed. If this is zero,
+	// there is no dial timeout.
+	DialTimeout time.Duration
+
+	// Timeout is the amount of time to wait for the entire
+	// api.Open to succeed. If this is zero, there is no timeout.
 	Timeout time.Duration
 
 	// RetryDelay is the amount of time to wait between
-	// unsuccessful connection attempts.
+	// unsuccessful connection attempts. If this is
+	// zero, only one attempt will be made.
 	RetryDelay time.Duration
 
 	// BakeryClient is the httpbakery Client, which
@@ -145,11 +154,41 @@ type DialOpts struct {
 	// DialWebsocket is used to make connections to API servers.
 	// It will be called with a websocket URL to connect to,
 	// and the TLS configuration to use to secure the connection.
+	// If ipAddr is non-empty, the actual net.Dial should use
+	// that IP address, regardless of the URL host.
 	//
-	// If DialWebsocket is nil, webaocket.DialConfig will be used.
-	//
-	// This field is provided for testing purposes only.
-	DialWebsocket func(cfg *websocket.Config) (*websocket.Conn, error)
+	// If DialWebsocket is nil, a default implementation using
+	// gorilla websockets will be used.
+	DialWebsocket func(ctx context.Context, urlStr string, tlsConfig *tls.Config, ipAddr string) (jsoncodec.JSONConn, error)
+
+	// IPAddrResolver is used to resolve host names to IP addresses.
+	// If it is nil, net.DefaultResolver will be used.
+	IPAddrResolver IPAddrResolver
+
+	// DNSCache is consulted to find and store cached DNS lookups.
+	// If it is nil, no cache will be used or updated.
+	DNSCache DNSCache
+
+	// Clock is used as a time source for retries.
+	// If it is nil, clock.WallClock will be used.
+	Clock clock.Clock
+}
+
+// IPAddrResolver implements a resolved from host name to the
+// set of IP addresses associated with it. It is notably
+// implemented by net.Resolver.
+type IPAddrResolver interface {
+	LookupIPAddr(ctx context.Context, host string) ([]net.IPAddr, error)
+}
+
+// DNSCache implements a cache of DNS lookup results.
+type DNSCache interface {
+	// Lookup returns the IP addresses associated
+	// with the given host.
+	Lookup(host string) []string
+	// Add sets the IP addresses associated with
+	// the given host name.
+	Add(host string, ips []string)
 }
 
 // DefaultDialOpts returns a DialOpts representing the default
@@ -173,8 +212,24 @@ type OpenFunc func(*Info, DialOpts) (Connection, error)
 type Connection interface {
 
 	// This first block of methods is pretty close to a sane Connection interface.
+
+	// Close closes the connection.
 	Close() error
+
+	// Addr returns the address used to connect to the API server.
 	Addr() string
+
+	// IPAddr returns the IP address used to connect to the API server.
+	IPAddr() string
+
+	// APIHostPorts returns addresses that may be used to connect
+	// to the API server, including the address used to connect.
+	//
+	// The addresses are scoped (public, cloud-internal, etc.), so
+	// the client may choose which addresses to attempt. For the
+	// Juju CLI, all addresses must be attempted, as the CLI may
+	// be invoked both within and outside the model (think
+	// private clouds).
 	APIHostPorts() [][]network.HostPort
 
 	// Broken returns a channel which will be closed if the connection
@@ -187,6 +242,12 @@ type Connection interface {
 	// ping.
 	IsBroken() bool
 
+	// PublicDNSName returns the host name for which an officially
+	// signed certificate will be used for TLS connection to the server.
+	// If empty, the private Juju CA certificate must be used to verify
+	// the connection.
+	PublicDNSName() string
+
 	// These are a bit off -- ServerVersion is apparently not known until after
 	// Login()? Maybe evidence of need for a separate AuthenticatedConnection..?
 	Login(name names.Tag, password, nonce string, ms []macaroon.Slice) error
@@ -235,7 +296,6 @@ type Connection interface {
 	Uniter() (*uniter.State, error)
 	Upgrader() *upgrader.State
 	Reboot() (reboot.State, error)
-	DiscoverSpaces() *discoverspaces.API
 	InstancePoller() *instancepoller.API
 	CharmRevisionUpdater() *charmrevisionupdater.State
 	Cleaner() *cleaner.API
