@@ -9,9 +9,11 @@
 package provisioner_test
 
 import (
+	"fmt"
 	"time"
 
 	"github.com/juju/errors"
+	gitjujutesting "github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
 	"github.com/juju/utils/arch"
@@ -21,6 +23,8 @@ import (
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/api"
+	"github.com/juju/juju/api/base"
+	apibasetesting "github.com/juju/juju/api/base/testing"
 	"github.com/juju/juju/api/provisioner"
 	apitesting "github.com/juju/juju/api/testing"
 	"github.com/juju/juju/apiserver/common"
@@ -120,11 +124,11 @@ func (s *provisionerSuite) TestGetSetInstanceStatus(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(instanceStatus, gc.Equals, status.Pending)
 	c.Assert(info, gc.Equals, "")
-	err = apiMachine.SetInstanceStatus(status.Started, "blah", nil)
+	err = apiMachine.SetInstanceStatus(status.Running, "blah", nil)
 	c.Assert(err, jc.ErrorIsNil)
 	instanceStatus, info, err = apiMachine.InstanceStatus()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(instanceStatus, gc.Equals, status.Started)
+	c.Assert(instanceStatus, gc.Equals, status.Running)
 	c.Assert(info, gc.Equals, "blah")
 	statusInfo, err := s.machine.InstanceStatus()
 	c.Assert(err, jc.ErrorIsNil)
@@ -152,12 +156,12 @@ func (s *provisionerSuite) TestMachinesWithTransientErrors(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	now := time.Now()
 	sInfo := status.StatusInfo{
-		Status:  status.Error,
+		Status:  status.ProvisioningError,
 		Message: "blah",
 		Data:    map[string]interface{}{"transient": true},
 		Since:   &now,
 	}
-	err = machine.SetStatus(sInfo)
+	err = machine.SetInstanceStatus(sInfo)
 	c.Assert(err, jc.ErrorIsNil)
 	machines, info, err := s.provisioner.MachinesWithTransientErrors()
 	c.Assert(err, jc.ErrorIsNil)
@@ -167,7 +171,7 @@ func (s *provisionerSuite) TestMachinesWithTransientErrors(c *gc.C) {
 	c.Assert(info[0], gc.DeepEquals, params.StatusResult{
 		Id:     "1",
 		Life:   "alive",
-		Status: "error",
+		Status: "provisioning error",
 		Info:   "blah",
 		Data:   map[string]interface{}{"transient": true},
 	})
@@ -354,6 +358,46 @@ func (s *provisionerSuite) TestSeries(c *gc.C) {
 	c.Assert(series, gc.Equals, "quantal")
 }
 
+func (s *provisionerSuite) TestAvailabilityZone(c *gc.C) {
+	// Create a fresh machine, since machine 0 is already provisioned.
+	template := state.MachineTemplate{
+		Series: "xenial",
+		Jobs:   []state.MachineJob{state.JobHostUnits},
+	}
+	notProvisionedMachine, err := s.State.AddOneMachine(template)
+	c.Assert(err, jc.ErrorIsNil)
+
+	apiMachine, err := s.provisioner.Machine(notProvisionedMachine.Tag().(names.MachineTag))
+	c.Assert(err, jc.ErrorIsNil)
+
+	instanceId, err := apiMachine.InstanceId()
+	c.Assert(err, jc.Satisfies, params.IsCodeNotProvisioned)
+	c.Assert(err, gc.ErrorMatches, "machine 1 not provisioned")
+	c.Assert(instanceId, gc.Equals, instance.Id(""))
+
+	availabilityZone := "ru-north-siberia"
+	hwChars := instance.MustParseHardware(fmt.Sprintf("availability-zone=%s", availabilityZone))
+
+	err = apiMachine.SetInstanceInfo(
+		"azinst", "nonce", &hwChars, nil, nil, nil,
+	)
+	c.Assert(err, jc.ErrorIsNil)
+
+	retAvailabilityZone, err := apiMachine.AvailabilityZone()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(availabilityZone, gc.Equals, retAvailabilityZone)
+}
+
+func (s *provisionerSuite) TestKeepInstance(c *gc.C) {
+	err := s.machine.SetKeepInstance(true)
+	c.Assert(err, jc.ErrorIsNil)
+	apiMachine, err := s.provisioner.Machine(s.machine.Tag().(names.MachineTag))
+	c.Assert(err, jc.ErrorIsNil)
+	keep, err := apiMachine.KeepInstance()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(keep, jc.IsTrue)
+}
+
 func (s *provisionerSuite) TestDistributionGroup(c *gc.C) {
 	apiMachine, err := s.provisioner.Machine(s.machine.Tag().(names.MachineTag))
 	c.Assert(err, jc.ErrorIsNil)
@@ -633,7 +677,7 @@ func (s *provisionerSuite) testFindTools(c *gc.C, matchArch bool, apiError, logi
 	current := version.Binary{
 		Number: jujuversion.Current,
 		Arch:   arch.HostArch(),
-		Series: series.HostSeries(),
+		Series: series.MustHostSeries(),
 	}
 	var toolsList = coretools.List{&coretools.Tools{Version: current}}
 	var called bool
@@ -649,7 +693,7 @@ func (s *provisionerSuite) testFindTools(c *gc.C, matchArch bool, apiError, logi
 		c.Assert(request, gc.Equals, "FindTools")
 		expected := params.FindToolsParams{
 			Number:       jujuversion.Current,
-			Series:       series.HostSeries(),
+			Series:       series.MustHostSeries(),
 			Arch:         a,
 			MinorVersion: -1,
 			MajorVersion: -1,
@@ -662,7 +706,7 @@ func (s *provisionerSuite) testFindTools(c *gc.C, matchArch bool, apiError, logi
 		}
 		return apiError
 	})
-	apiList, err := s.provisioner.FindTools(jujuversion.Current, series.HostSeries(), a)
+	apiList, err := s.provisioner.FindTools(jujuversion.Current, series.MustHostSeries(), a)
 	c.Assert(called, jc.IsTrue)
 	if apiError != nil {
 		c.Assert(err, gc.Equals, apiError)
@@ -673,3 +717,179 @@ func (s *provisionerSuite) testFindTools(c *gc.C, matchArch bool, apiError, logi
 		c.Assert(apiList, jc.SameContents, toolsList)
 	}
 }
+
+func (s *provisionerSuite) TestHostChangesForContainer(c *gc.C) {
+	// Create a machine, put it in "default" space with a single NIC. Create
+	// a container that is also in the "default" space, and request the
+	// HostChangesForContainer to see that it wants to bridge that NIC
+	_, err := s.State.AddSpace("default", network.Id("default"), nil, true)
+	c.Assert(err, jc.ErrorIsNil)
+	_, err = s.State.AddSubnet(state.SubnetInfo{
+		CIDR:      "10.0.0.0/24",
+		SpaceName: "default",
+	})
+	c.Assert(err, jc.ErrorIsNil)
+	err = s.machine.SetLinkLayerDevices(
+		state.LinkLayerDeviceArgs{
+			Name:       "ens3",
+			Type:       state.EthernetDevice,
+			ParentName: "",
+			IsUp:       true,
+		},
+	)
+	c.Assert(err, jc.ErrorIsNil)
+	err = s.machine.SetDevicesAddresses(
+		state.LinkLayerDeviceAddress{
+			DeviceName:   "ens3",
+			CIDRAddress:  "10.0.0.10/24",
+			ConfigMethod: state.StaticAddress,
+		},
+	)
+	c.Assert(err, jc.ErrorIsNil)
+	containerTemplate := state.MachineTemplate{
+		Series: "quantal",
+		Jobs:   []state.MachineJob{state.JobHostUnits},
+	}
+	container, err := s.State.AddMachineInsideMachine(containerTemplate, s.machine.Id(), instance.LXD)
+	c.Assert(err, jc.ErrorIsNil)
+
+	changes, reconfigureDelay, err := s.provisioner.HostChangesForContainer(container.MachineTag())
+	c.Assert(err, gc.ErrorMatches, "dummy provider network config not supported.*")
+	c.Skip("can't test without network support")
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(changes, gc.DeepEquals, []network.DeviceToBridge{{
+		BridgeName: "br-ens3",
+		DeviceName: "ens3",
+	}})
+	c.Check(reconfigureDelay, gc.Equals, 0)
+}
+
+var _ = gc.Suite(&prepareContainerSuite{})
+
+type prepareContainerSuite struct {
+	coretesting.BaseSuite
+
+	*gitjujutesting.Stub
+}
+
+type prepareFacade struct {
+	*gitjujutesting.Stub
+
+	prepareResult []params.NetworkConfig
+	prepareError  error
+}
+
+func (f *prepareFacade) FacadeCall(objType string, version int, id, request string, requestParams, response interface{}) error {
+	// We only support PrepareContainerInterfaceInfo
+	if objType != "Provisioner" {
+		return errors.Errorf("bad facade name: %q", objType)
+	}
+	if version != 0 {
+		return errors.Errorf("bad version: %d", version)
+	}
+	if request != "PrepareContainerInterfaceInfo" {
+		return errors.Errorf("bad method name: %q", request)
+	}
+	entities, ok := requestParams.(params.Entities)
+	if !ok {
+		return errors.Errorf("unknown request type: %t", requestParams)
+	}
+	f.Stub.AddCall("api.PrepareContainerInterfaceInfo", entities.Entities)
+	if len(entities.Entities) != 1 {
+		return errors.Errorf("only support a single entity not: %v", entities.Entities)
+	}
+	result, ok := response.(*params.MachineNetworkConfigResults)
+	if !ok {
+		return errors.Errorf("invalid return type: %t", response)
+	}
+	result.Results = make([]params.MachineNetworkConfigResult, len(entities.Entities))
+	if f.prepareError != nil {
+		result.Results[0].Error = common.ServerError(f.prepareError)
+	} else {
+		result.Results[0].Config = f.prepareResult
+	}
+	return nil
+}
+
+func (s *prepareContainerSuite) SetUpTest(c *gc.C) {
+	s.BaseSuite.SetUpTest(c)
+
+	s.Stub = &gitjujutesting.Stub{}
+}
+
+func (s *prepareContainerSuite) apiForPrepareContainer(config []params.NetworkConfig, err error) base.APICaller {
+	facade := &prepareFacade{
+		Stub:          s.Stub,
+		prepareResult: config,
+		prepareError:  err,
+	}
+	return apibasetesting.APICallerFunc(facade.FacadeCall)
+}
+
+func (s *prepareContainerSuite) TestPrepareContainerInterfaceInfoNoValues(c *gc.C) {
+	apicaller := s.apiForPrepareContainer(nil, nil)
+	st := provisioner.NewState(apicaller)
+	networkInfo, err := st.PrepareContainerInterfaceInfo(names.NewMachineTag("machine-0/lxd/0"))
+	c.Assert(err, gc.IsNil)
+	c.Check(networkInfo, jc.DeepEquals, []network.InterfaceInfo{})
+}
+
+func (s *prepareContainerSuite) TestPrepareContainerInterfaceInfoSingleNIC(c *gc.C) {
+	apicaller := s.apiForPrepareContainer([]params.NetworkConfig{{
+		DeviceIndex:         1,
+		MACAddress:          "de:ad:be:ff:11:22",
+		CIDR:                "192.168.0.5/24",
+		MTU:                 9000,
+		ProviderId:          "prov-id",
+		ProviderSubnetId:    "prov-sub-id",
+		ProviderSpaceId:     "prov-space-id",
+		ProviderAddressId:   "prov-address-id",
+		ProviderVLANId:      "prov-vlan-id",
+		VLANTag:             25,
+		InterfaceName:       "eth5",
+		ParentInterfaceName: "parent#br-eth5",
+		InterfaceType:       "ethernet",
+		Disabled:            false,
+		NoAutoStart:         false,
+		ConfigType:          "static",
+		Address:             "192.168.0.6",
+		DNSServers:          []string{"8.8.8.8"},
+		DNSSearchDomains:    []string{"mydomain"},
+		GatewayAddress:      "192.168.0.1",
+		Routes: []params.NetworkRoute{{
+			DestinationCIDR: "10.0.0.0/16",
+			GatewayIP:       "192.168.0.1",
+			Metric:          55,
+		}},
+	}}, nil)
+	st := provisioner.NewState(apicaller)
+	networkInfo, err := st.PrepareContainerInterfaceInfo(names.NewMachineTag("machine-0/lxd/0"))
+	c.Assert(err, gc.IsNil)
+	c.Check(networkInfo, jc.DeepEquals, []network.InterfaceInfo{{
+		DeviceIndex:         1,
+		MACAddress:          "de:ad:be:ff:11:22",
+		CIDR:                "192.168.0.5/24",
+		MTU:                 9000,
+		ProviderId:          "prov-id",
+		ProviderSubnetId:    "prov-sub-id",
+		ProviderSpaceId:     "prov-space-id",
+		ProviderAddressId:   "prov-address-id",
+		ProviderVLANId:      "prov-vlan-id",
+		VLANTag:             25,
+		InterfaceName:       "eth5",
+		ParentInterfaceName: "parent#br-eth5",
+		InterfaceType:       "ethernet",
+		Disabled:            false,
+		NoAutoStart:         false,
+		ConfigType:          "static",
+		Address:             network.NewAddress("192.168.0.6"),
+		DNSServers:          network.NewAddresses("8.8.8.8"),
+		DNSSearchDomains:    []string{"mydomain"},
+		GatewayAddress:      network.NewAddress("192.168.0.1"),
+		Routes: []network.Route{{
+			DestinationCIDR: "10.0.0.0/16",
+			GatewayIP:       "192.168.0.1",
+			Metric:          55,
+		}},
+	}})
+}
