@@ -13,8 +13,9 @@ import (
 	"github.com/juju/juju/api/base"
 	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/core/migration"
-	"github.com/juju/juju/state/multiwatcher"
+	"github.com/juju/juju/rpc"
 	"github.com/juju/juju/watcher"
+	"github.com/juju/juju/worker"
 )
 
 var logger = loggo.GetLogger("juju.api.watcher")
@@ -84,7 +85,10 @@ func (w *commonWatcher) commonLoop() {
 		defer wg.Done()
 		<-w.tomb.Dying()
 		if err := w.call("Stop", nil); err != nil {
-			logger.Errorf("error trying to stop watcher: %v", err)
+			// Don't log an error if a watcher is stopped due to an agent restart.
+			if err.Error() != worker.ErrRestartAgent.Error() && err.Error() != rpc.ErrShutdown.Error() {
+				logger.Errorf("error trying to stop watcher: %v", err)
+			}
 		}
 	}()
 	wg.Add(1)
@@ -387,61 +391,6 @@ func (w *machineAttachmentsWatcher) Changes() watcher.MachineStorageIdsChannel {
 	return w.out
 }
 
-// EntitiesWatcher will send events when something changes.
-// The content for the changes is a list of tag strings.
-type entitiesWatcher struct {
-	commonWatcher
-	caller            base.APICaller
-	entitiesWatcherId string
-	out               chan []string
-}
-
-func NewEntitiesWatcher(caller base.APICaller, result params.EntitiesWatchResult) watcher.EntitiesWatcher {
-	w := &entitiesWatcher{
-		caller:            caller,
-		entitiesWatcherId: result.EntitiesWatcherId,
-		out:               make(chan []string),
-	}
-	go func() {
-		defer w.tomb.Done()
-		w.tomb.Kill(w.loop(result.Changes))
-	}()
-	return w
-}
-
-func (w *entitiesWatcher) loop(initialChanges []string) error {
-	changes := initialChanges
-	w.newResult = func() interface{} { return new(params.EntitiesWatchResult) }
-	w.call = makeWatcherAPICaller(w.caller, "EntityWatcher", w.entitiesWatcherId)
-	w.commonWatcher.init()
-	go w.commonLoop()
-
-	for {
-		select {
-		// Send the initial event or subsequent change.
-		case w.out <- changes:
-		case <-w.tomb.Dying():
-			return nil
-		}
-		// Read the next change.
-		data, ok := <-w.in
-		if !ok {
-			// The tomb is already killed with the correct error
-			// at this point, so just return.
-			return nil
-		}
-		// Changes have been transformed at the server side already.
-		changes = data.(*params.EntitiesWatchResult).Changes
-	}
-}
-
-// Changes returns a channel that receives a list of changes
-// as tags (converted to strings) of the watched entities
-// with changes.
-func (w *entitiesWatcher) Changes() watcher.StringsChannel {
-	return w.out
-}
-
 // NewMigrationStatusWatcher takes the NotifyWatcherId returns by the
 // MigrationSlave.Watch API and returns a watcher which will report
 // status changes for any migration of the model associated with the
@@ -514,85 +463,3 @@ func (w *migrationStatusWatcher) loop() error {
 func (w *migrationStatusWatcher) Changes() <-chan watcher.MigrationStatus {
 	return w.out
 }
-
-// applicationRelationsWatcher will sends changes to relations an application
-// is involved in, including changes to the units involved in those
-// relations, and their settings.
-type applicationRelationsWatcher struct {
-	commonWatcher
-	caller                        base.APICaller
-	applicationRelationsWatcherId string
-	out                           chan watcher.ApplicationRelationsChange
-}
-
-// NewApplicationRelationsWatcher returns an ApplicationRelationsWatcher which
-// communicates with the ApplicationRelationsWatcher API facade to watch
-// application relations.
-func NewApplicationRelationsWatcher(caller base.APICaller, result params.ApplicationRelationsWatchResult) watcher.ApplicationRelationsWatcher {
-	w := &applicationRelationsWatcher{
-		caller: caller,
-		applicationRelationsWatcherId: result.ApplicationRelationsWatcherId,
-		out: make(chan watcher.ApplicationRelationsChange),
-	}
-	go func() {
-		defer w.tomb.Done()
-		defer close(w.out)
-		w.tomb.Kill(w.loop(*result.Changes))
-	}()
-	return w
-}
-
-func (w *applicationRelationsWatcher) loop(initialChanges params.ApplicationRelationsChange) error {
-	changes := copyApplicationRelationsChange(initialChanges)
-	w.newResult = func() interface{} { return new(params.ApplicationRelationsWatchResult) }
-	w.call = makeWatcherAPICaller(w.caller, "ApplicationRelationsWatcher", w.applicationRelationsWatcherId)
-	w.commonWatcher.init()
-	go w.commonLoop()
-
-	for {
-		select {
-		// Send the initial event or subsequent change.
-		case w.out <- changes:
-		case <-w.tomb.Dying():
-			return nil
-		}
-		// Read the next change.
-		data, ok := <-w.in
-		if !ok {
-			// The tomb is already killed with the correct error
-			// at this point, so just return.
-			return nil
-		}
-		changes = copyApplicationRelationsChange(*data.(*params.ApplicationRelationsWatchResult).Changes)
-	}
-}
-
-func copyApplicationRelationsChange(src params.ApplicationRelationsChange) watcher.ApplicationRelationsChange {
-	dst := watcher.ApplicationRelationsChange{
-		RemovedRelations: src.RemovedRelations,
-	}
-	if src.ChangedRelations != nil {
-		dst.ChangedRelations = make([]watcher.RelationChange, len(src.ChangedRelations))
-		for i, change := range src.ChangedRelations {
-			cr := watcher.RelationChange{
-				RelationId:    change.RelationId,
-				Life:          multiwatcher.Life(change.Life),
-				DepartedUnits: change.DepartedUnits,
-			}
-			cr.ChangedUnits = make(map[string]watcher.RelationUnitChange)
-			for name, relationChange := range change.ChangedUnits {
-				cr.ChangedUnits[name] = watcher.RelationUnitChange{Settings: relationChange.Settings}
-			}
-			dst.ChangedRelations[i] = cr
-		}
-	}
-	return dst
-}
-
-// Changes returns a channel that will receive the changes to
-// relations an application is involved in. The first event on the channel
-// holds the initial state of the application's relations in its Changed
-// field.
-func (w *applicationRelationsWatcher) Changes() watcher.ApplicationRelationsChannel {
-	return w.out
-}
