@@ -14,10 +14,6 @@ import (
 	"github.com/juju/juju/state"
 )
 
-func init() {
-	common.RegisterStandardFacade("Action", 2, NewActionAPI)
-}
-
 // ActionAPI implements the client API for interacting with Actions
 type ActionAPI struct {
 	state      *state.State
@@ -243,8 +239,10 @@ func (a *ActionAPI) Cancel(arg params.Entities) (params.ActionResults, error) {
 	}
 
 	response := params.ActionResults{Results: make([]params.ActionResult, len(arg.Entities))}
+
 	for i, entity := range arg.Entities {
 		currentResult := &response.Results[i]
+		currentResult.Action = &params.Action{Tag: entity.Tag}
 		tag, err := names.ParseTag(entity.Tag)
 		if err != nil {
 			currentResult.Error = common.ServerError(common.ErrBadId)
