@@ -38,30 +38,26 @@ import (
 
 type baseLoginSuite struct {
 	jujutesting.JujuConnSuite
-	setAdminAPI func(*apiserver.Server)
+	pool *state.StatePool
 }
 
 type loginSuite struct {
 	baseLoginSuite
 }
 
-var _ = gc.Suite(&loginSuite{
-	baseLoginSuite{
-		setAdminAPI: func(srv *apiserver.Server) {
-			apiserver.SetAdminAPIVersions(srv, 3)
-		},
-	},
-})
+var _ = gc.Suite(&loginSuite{})
 
 func (s *baseLoginSuite) SetUpTest(c *gc.C) {
 	s.JujuConnSuite.SetUpTest(c)
 	loggo.GetLogger("juju.apiserver").SetLogLevel(loggo.TRACE)
+	s.pool = state.NewStatePool(s.State)
+	s.AddCleanup(func(*gc.C) { s.pool.Close() })
 }
 
 func (s *baseLoginSuite) newMachineAndServer(c *gc.C) (*api.Info, *apiserver.Server) {
 	machine, password := s.Factory.MakeMachineReturningPassword(
 		c, &factory.MachineParams{Nonce: "fake_nonce"})
-	info, srv := newServer(c, s.State)
+	info, srv := newServer(c, s.pool)
 	info.Tag = machine.Tag()
 	info.Password = password
 	info.Nonce = "fake_nonce"
@@ -88,7 +84,7 @@ func (s *loginSuite) TestBadLogin(c *gc.C) {
 	// Start our own server so we can control when the first login
 	// happens. Otherwise in JujuConnSuite.SetUpTest api.Open is
 	// called with user-admin permissions automatically.
-	info, srv := newServer(c, s.State)
+	info, srv := newServer(c, s.pool)
 	defer assertStop(c, srv)
 	info.ModelTag = s.State.ModelTag()
 
@@ -143,7 +139,7 @@ func (s *loginSuite) TestBadLogin(c *gc.C) {
 }
 
 func (s *loginSuite) TestLoginAsDeactivatedUser(c *gc.C) {
-	info, srv := newServer(c, s.State)
+	info, srv := newServer(c, s.pool)
 	defer assertStop(c, srv)
 	info.ModelTag = s.State.ModelTag()
 
@@ -169,7 +165,7 @@ func (s *loginSuite) TestLoginAsDeactivatedUser(c *gc.C) {
 }
 
 func (s *baseLoginSuite) runLoginSetsLogIdentifier(c *gc.C) {
-	info, srv := newServer(c, s.State)
+	info, srv := newServer(c, s.pool)
 	defer assertStop(c, srv)
 
 	machine, password := s.Factory.MakeMachineReturningPassword(
@@ -339,7 +335,7 @@ func (s *loginSuite) TestLoginRateLimited(c *gc.C) {
 	select {
 	case err := <-errResults:
 		c.Check(err, jc.Satisfies, params.IsCodeTryAgain)
-	case <-time.After(coretesting.LongWait):
+	case <-time.After(apiserver.LoginRetyPause + coretesting.LongWait):
 		c.Fatalf("timed out waiting for login to get rejected.")
 	}
 
@@ -429,7 +425,7 @@ func (s *loginSuite) TestUsersLoginWhileRateLimited(c *gc.C) {
 }
 
 func (s *loginSuite) TestUsersAreNotRateLimited(c *gc.C) {
-	info, srv := newServer(c, s.State)
+	info, srv := newServer(c, s.pool)
 	defer assertStop(c, srv)
 
 	info.Tag = s.AdminUserTag(c)
@@ -459,7 +455,7 @@ func (s *loginSuite) TestUsersAreNotRateLimited(c *gc.C) {
 }
 
 func (s *loginSuite) TestNonModelUserLoginFails(c *gc.C) {
-	info, srv := newServer(c, s.State)
+	info, srv := newServer(c, s.pool)
 	defer assertStop(c, srv)
 	info.ModelTag = s.State.ModelTag()
 	user := s.Factory.MakeUser(c, &factory.UserParams{Password: "dummy-password", NoModelUser: true})
@@ -520,7 +516,7 @@ func (s *loginSuite) TestFailedLoginDuringMaintenance(c *gc.C) {
 	cfg.Validator = func(params.LoginRequest) error {
 		return errors.New("something")
 	}
-	info, srv := newServerWithConfig(c, s.State, cfg)
+	info, srv := newServerWithConfig(c, s.pool, cfg)
 	defer assertStop(c, srv)
 	info.ModelTag = s.State.ModelTag()
 
@@ -538,7 +534,7 @@ type validationChecker func(c *gc.C, err error, st api.Connection)
 func (s *baseLoginSuite) checkLoginWithValidator(c *gc.C, validator apiserver.LoginValidator, checker validationChecker) {
 	cfg := defaultServerConfig(c)
 	cfg.Validator = validator
-	info, srv := newServerWithConfig(c, s.State, cfg)
+	info, srv := newServerWithConfig(c, s.pool, cfg)
 	defer assertStop(c, srv)
 	info.ModelTag = s.State.ModelTag()
 
@@ -570,7 +566,7 @@ func (s *baseLoginSuite) openAPIWithoutLogin(c *gc.C, info0 *api.Info) api.Conne
 }
 
 func (s *loginSuite) TestControllerModel(c *gc.C) {
-	info, srv := newServer(c, s.State)
+	info, srv := newServer(c, s.pool)
 	defer assertStop(c, srv)
 
 	info.ModelTag = s.State.ModelTag()
@@ -584,7 +580,7 @@ func (s *loginSuite) TestControllerModel(c *gc.C) {
 }
 
 func (s *loginSuite) TestControllerModelBadCreds(c *gc.C) {
-	info, srv := newServer(c, s.State)
+	info, srv := newServer(c, s.pool)
 	defer assertStop(c, srv)
 
 	info.ModelTag = s.State.ModelTag()
@@ -596,7 +592,7 @@ func (s *loginSuite) TestControllerModelBadCreds(c *gc.C) {
 }
 
 func (s *loginSuite) TestNonExistentModel(c *gc.C) {
-	info, srv := newServer(c, s.State)
+	info, srv := newServer(c, s.pool)
 	defer assertStop(c, srv)
 
 	uuid, err := utils.NewUUID()
@@ -613,7 +609,7 @@ func (s *loginSuite) TestNonExistentModel(c *gc.C) {
 }
 
 func (s *loginSuite) TestInvalidModel(c *gc.C) {
-	info, srv := newServer(c, s.State)
+	info, srv := newServer(c, s.pool)
 	defer assertStop(c, srv)
 	info.ModelTag = names.NewModelTag("rubbish")
 
@@ -628,7 +624,7 @@ func (s *loginSuite) TestInvalidModel(c *gc.C) {
 }
 
 func (s *loginSuite) TestOtherModel(c *gc.C) {
-	info, srv := newServer(c, s.State)
+	info, srv := newServer(c, s.pool)
 	defer assertStop(c, srv)
 
 	envOwner := s.Factory.MakeUser(c, nil)
@@ -649,7 +645,7 @@ func (s *loginSuite) TestMachineLoginOtherModel(c *gc.C) {
 	// Machine credentials are checked against environment specific
 	// machines, so this makes sure that the credential checking is
 	// using the correct state connection.
-	info, srv := newServer(c, s.State)
+	info, srv := newServer(c, s.pool)
 	defer assertStop(c, srv)
 
 	envOwner := s.Factory.MakeUser(c, nil)
@@ -674,7 +670,7 @@ func (s *loginSuite) TestMachineLoginOtherModel(c *gc.C) {
 }
 
 func (s *loginSuite) TestMachineLoginOtherModelNotProvisioned(c *gc.C) {
-	info, srv := newServer(c, s.State)
+	info, srv := newServer(c, s.pool)
 	defer assertStop(c, srv)
 
 	envOwner := s.Factory.MakeUser(c, nil)
@@ -701,7 +697,7 @@ func (s *loginSuite) TestMachineLoginOtherModelNotProvisioned(c *gc.C) {
 }
 
 func (s *loginSuite) TestOtherEnvironmentFromController(c *gc.C) {
-	info, srv := newServer(c, s.State)
+	info, srv := newServer(c, s.pool)
 	defer assertStop(c, srv)
 
 	machine, password := s.Factory.MakeMachineReturningPassword(c, &factory.MachineParams{
@@ -718,7 +714,7 @@ func (s *loginSuite) TestOtherEnvironmentFromController(c *gc.C) {
 }
 
 func (s *loginSuite) TestOtherEnvironmentFromControllerOtherNotProvisioned(c *gc.C) {
-	info, srv := newServer(c, s.State)
+	info, srv := newServer(c, s.pool)
 	defer assertStop(c, srv)
 
 	managerMachine, password := s.Factory.MakeMachineReturningPassword(c, &factory.MachineParams{
@@ -744,7 +740,7 @@ func (s *loginSuite) TestOtherEnvironmentFromControllerOtherNotProvisioned(c *gc
 }
 
 func (s *loginSuite) TestOtherEnvironmentWhenNotController(c *gc.C) {
-	info, srv := newServer(c, s.State)
+	info, srv := newServer(c, s.pool)
 	defer assertStop(c, srv)
 
 	machine, password := s.Factory.MakeMachineReturningPassword(c, nil)
@@ -777,7 +773,7 @@ func (s *loginSuite) loginLocalUser(c *gc.C, info *api.Info) (*state.User, param
 }
 
 func (s *loginSuite) TestLoginResultLocalUser(c *gc.C) {
-	info, srv := newServer(c, s.State)
+	info, srv := newServer(c, s.pool)
 	defer assertStop(c, srv)
 	info.ModelTag = s.State.ModelTag()
 
@@ -788,7 +784,7 @@ func (s *loginSuite) TestLoginResultLocalUser(c *gc.C) {
 }
 
 func (s *loginSuite) TestLoginResultLocalUserEveryoneCreateOnlyNonLocal(c *gc.C) {
-	info, srv := newServer(c, s.State)
+	info, srv := newServer(c, s.pool)
 	defer assertStop(c, srv)
 	info.ModelTag = s.State.ModelTag()
 
@@ -868,6 +864,13 @@ var _ = gc.Suite(&macaroonLoginSuite{})
 
 type macaroonLoginSuite struct {
 	apitesting.MacaroonSuite
+	pool *state.StatePool
+}
+
+func (s *macaroonLoginSuite) SetUpTest(c *gc.C) {
+	s.MacaroonSuite.SetUpTest(c)
+	s.pool = state.NewStatePool(s.State)
+	s.AddCleanup(func(*gc.C) { s.pool.Close() })
 }
 
 func (s *macaroonLoginSuite) TestLoginToController(c *gc.C) {
@@ -1007,7 +1010,7 @@ func (s *macaroonLoginSuite) testRemoteUserLoginToModelWithExplicitAccess(c *gc.
 	cfg := defaultServerConfig(c)
 	cfg.AllowModelAccess = allowModelAccess
 
-	info, srv := newServerWithConfig(c, s.State, cfg)
+	info, srv := newServerWithConfig(c, s.pool, cfg)
 	defer assertStop(c, srv)
 	info.ModelTag = s.State.ModelTag()
 
@@ -1109,13 +1112,7 @@ func setEveryoneAccess(c *gc.C, st *state.State, adminUser names.UserTag, access
 	c.Assert(err, jc.ErrorIsNil)
 }
 
-var _ = gc.Suite(&migrationSuite{
-	baseLoginSuite{
-		setAdminAPI: func(srv *apiserver.Server) {
-			apiserver.SetAdminAPIVersions(srv, 3)
-		},
-	},
-})
+var _ = gc.Suite(&migrationSuite{})
 
 type migrationSuite struct {
 	baseLoginSuite
@@ -1143,3 +1140,23 @@ func (s *migrationSuite) TestImportingModel(c *gc.C) {
 	_, err = apimachiner.NewState(machineConn).Machine(m.MachineTag())
 	c.Check(err, jc.ErrorIsNil)
 }
+
+func (s *migrationSuite) TestExportingModel(c *gc.C) {
+	model, err := s.State.Model()
+	c.Assert(err, jc.ErrorIsNil)
+	err = model.SetMigrationMode(state.MigrationModeExporting)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Users should be able to log in but RPC requests should fail.
+	info := s.APIInfo(c)
+	userConn := s.OpenAPIAs(c, info.Tag, info.Password)
+	defer userConn.Close()
+
+	// Status is fine.
+	_, err = userConn.Client().Status(nil)
+	c.Check(err, jc.ErrorIsNil)
+
+	// Modifying commands like destroy machines are not.
+	err = userConn.Client().DestroyMachines("42")
+	c.Check(err, gc.ErrorMatches, "model migration in progress")
+}
