@@ -7,20 +7,25 @@ import (
 	"crypto/tls"
 	"crypto/x509"
 	"io"
+	"log"
 	"net"
 	"net/http"
+	"path"
+	"path/filepath"
 	"strings"
 	"sync"
 	"sync/atomic"
+	"time"
 
 	"github.com/bmizerany/pat"
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
+	"github.com/juju/pubsub"
 	"github.com/juju/utils"
 	"github.com/juju/utils/clock"
+	"github.com/prometheus/client_golang/prometheus"
 	"golang.org/x/crypto/acme"
 	"golang.org/x/crypto/acme/autocert"
-	"golang.org/x/net/websocket"
 	"gopkg.in/juju/names.v2"
 	"gopkg.in/macaroon-bakery.v1/httpbakery"
 	"gopkg.in/tomb.v1"
@@ -28,8 +33,13 @@ import (
 	"github.com/juju/juju/apiserver/authentication"
 	"github.com/juju/juju/apiserver/common"
 	"github.com/juju/juju/apiserver/common/apihttp"
+	"github.com/juju/juju/apiserver/facade"
+	"github.com/juju/juju/apiserver/logsink"
 	"github.com/juju/juju/apiserver/observer"
 	"github.com/juju/juju/apiserver/params"
+	"github.com/juju/juju/apiserver/websocket"
+	"github.com/juju/juju/resource"
+	"github.com/juju/juju/resource/resourceadapters"
 	"github.com/juju/juju/rpc"
 	"github.com/juju/juju/rpc/jsoncodec"
 	"github.com/juju/juju/state"
@@ -37,43 +47,74 @@ import (
 
 var logger = loggo.GetLogger("juju.apiserver")
 
-// loginRateLimit defines how many concurrent Login requests we will
-// accept
-const loginRateLimit = 10
+var defaultHTTPMethods = []string{"GET", "POST", "HEAD", "PUT", "DELETE", "OPTIONS"}
+
+// These vars define how we rate limit incoming connections.
+const (
+	defaultLoginRateLimit         = 10 // concurrent login operations
+	defaultLoginMinPause          = 100 * time.Millisecond
+	defaultLoginMaxPause          = 1 * time.Second
+	defaultLoginRetryPause        = 5 * time.Second
+	defaultConnMinPause           = 0 * time.Millisecond
+	defaultConnMaxPause           = 5 * time.Second
+	defaultConnLookbackWindow     = 1 * time.Second
+	defaultConnLowerThreshold     = 1000   // connections per second
+	defaultConnUpperThreshold     = 100000 // connections per second
+	defaultLogSinkRateLimitBurst  = 1000
+	defaultLogSinkRateLimitRefill = time.Millisecond
+)
 
 // Server holds the server side of the API.
 type Server struct {
-	tomb              tomb.Tomb
-	clock             clock.Clock
-	pingClock         clock.Clock
-	wg                sync.WaitGroup
-	state             *state.State
-	statePool         *state.StatePool
-	lis               net.Listener
-	tag               names.Tag
-	dataDir           string
-	logDir            string
-	limiter           utils.Limiter
-	validator         LoginValidator
-	adminAPIFactories map[int]adminAPIFactory
-	modelUUID         string
-	authCtxt          *authContext
-	lastConnectionID  uint64
-	newObserver       observer.ObserverFactory
-	connCount         int64
-	certChanged       <-chan params.StateServingInfo
-	tlsConfig         *tls.Config
-	allowModelAccess  bool
-	logSinkWriter     io.WriteCloser
+	tomb                   tomb.Tomb
+	clock                  clock.Clock
+	pingClock              clock.Clock
+	wg                     sync.WaitGroup
+	statePool              *state.StatePool
+	lis                    net.Listener
+	tag                    names.Tag
+	dataDir                string
+	logDir                 string
+	limiter                utils.Limiter
+	loginRetryPause        time.Duration
+	validator              LoginValidator
+	facades                *facade.Registry
+	modelUUID              string
+	authCtxt               *authContext
+	lastConnectionID       uint64
+	centralHub             *pubsub.StructuredHub
+	newObserver            observer.ObserverFactory
+	connCount              int64
+	totalConn              int64
+	loginAttempts          int64
+	certChanged            <-chan params.StateServingInfo
+	tlsConfig              *tls.Config
+	allowModelAccess       bool
+	logSinkWriter          io.WriteCloser
+	logsinkRateLimitConfig logsink.RateLimitConfig
+	dbloggers              dbloggers
 
 	// mu guards the fields below it.
 	mu sync.Mutex
 
+	// publicDNSName_ holds the value that will be returned in
+	// LoginResult.PublicDNSName. Currently this is set once from
+	// AutocertDNSName and does not change but in the future it
+	// may change when a server certificate is explicitly set,
+	// hence it's here guarded by the mutex.
+	publicDNSName_ string
+
 	// cert holds the current certificate used for tls.Config.
 	cert *tls.Certificate
 
 	// certDNSNames holds the DNS names associated with cert.
 	certDNSNames []string
+
+	// registerIntrospectionHandlers is a function that will
+	// call a function with (path, http.Handler) tuples. This
+	// is to support registering the handlers underneath the
+	// "/introspection" prefix.
+	registerIntrospectionHandlers func(func(string, http.Handler))
 }
 
 // LoginValidator functions are used to decide whether login requests
@@ -91,6 +132,7 @@ type ServerConfig struct {
 	DataDir     string
 	LogDir      string
 	Validator   LoginValidator
+	Hub         *pubsub.StructuredHub
 	CertChanged <-chan params.StateServingInfo
 
 	// AutocertDNSName holds the DNS name for which
@@ -113,34 +155,172 @@ type ServerConfig struct {
 	// notified of key events during API requests.
 	NewObserver observer.ObserverFactory
 
-	// StatePool only exists to support testing.
-	StatePool *state.StatePool
+	// RegisterIntrospectionHandlers is a function that will
+	// call a function with (path, http.Handler) tuples. This
+	// is to support registering the handlers underneath the
+	// "/introspection" prefix.
+	RegisterIntrospectionHandlers func(func(string, http.Handler))
+
+	// RateLimitConfig holds paramaters to control
+	// aspects of rate limiting connections and logins.
+	RateLimitConfig RateLimitConfig
+
+	// LogSinkConfig holds parameters to control the API server's
+	// logsink endpoint behaviour. If this is nil, the values from
+	// DefaultLogSinkConfig() will be used.
+	LogSinkConfig *LogSinkConfig
+
+	// PrometheusRegisterer registers Prometheus collectors.
+	PrometheusRegisterer prometheus.Registerer
 }
 
-func (c *ServerConfig) Validate() error {
+// Validate validates the API server configuration.
+func (c ServerConfig) Validate() error {
+	if c.Hub == nil {
+		return errors.NotValidf("missing Hub")
+	}
 	if c.Clock == nil {
 		return errors.NotValidf("missing Clock")
 	}
 	if c.NewObserver == nil {
 		return errors.NotValidf("missing NewObserver")
 	}
-
+	if err := c.RateLimitConfig.Validate(); err != nil {
+		return errors.Annotate(err, "validating rate limit configuration")
+	}
+	if c.LogSinkConfig != nil {
+		if err := c.LogSinkConfig.Validate(); err != nil {
+			return errors.Annotate(err, "validating logsink configuration")
+		}
+	}
 	return nil
 }
 
-func (c *ServerConfig) pingClock() clock.Clock {
+func (c ServerConfig) pingClock() clock.Clock {
 	if c.PingClock == nil {
 		return c.Clock
 	}
 	return c.PingClock
 }
 
+// RateLimitConfig holds parameters to control
+// aspects of rate limiting connections and logins.
+type RateLimitConfig struct {
+	LoginRateLimit     int
+	LoginMinPause      time.Duration
+	LoginMaxPause      time.Duration
+	LoginRetryPause    time.Duration
+	ConnMinPause       time.Duration
+	ConnMaxPause       time.Duration
+	ConnLookbackWindow time.Duration
+	ConnLowerThreshold int
+	ConnUpperThreshold int
+}
+
+// DefaultRateLimitConfig returns a RateLimtConfig struct with
+// all attributes set to their default values.
+func DefaultRateLimitConfig() RateLimitConfig {
+	return RateLimitConfig{
+		LoginRateLimit:     defaultLoginRateLimit,
+		LoginMinPause:      defaultLoginMinPause,
+		LoginMaxPause:      defaultLoginMaxPause,
+		LoginRetryPause:    defaultLoginRetryPause,
+		ConnMinPause:       defaultConnMinPause,
+		ConnMaxPause:       defaultConnMaxPause,
+		ConnLookbackWindow: defaultConnLookbackWindow,
+		ConnLowerThreshold: defaultConnLowerThreshold,
+		ConnUpperThreshold: defaultConnUpperThreshold,
+	}
+}
+
+// Validate validates the rate limit configuration.
+// We apply arbitrary but sensible upper limits to prevent
+// typos from introducing obviously bad config.
+func (c RateLimitConfig) Validate() error {
+	if c.LoginRateLimit <= 0 || c.LoginRateLimit > 100 {
+		return errors.NotValidf("login-rate-limit %d <= 0 or > 100", c.LoginRateLimit)
+	}
+	if c.LoginMinPause < 0 || c.LoginMinPause > 100*time.Millisecond {
+		return errors.NotValidf("login-min-pause %d < 0 or > 100ms", c.LoginMinPause)
+	}
+	if c.LoginMaxPause < 0 || c.LoginMaxPause > 5*time.Second {
+		return errors.NotValidf("login-max-pause %d < 0 or > 5s", c.LoginMaxPause)
+	}
+	if c.LoginRetryPause < 0 || c.LoginRetryPause > 10*time.Second {
+		return errors.NotValidf("login-retry-pause %d < 0 or > 10s", c.LoginRetryPause)
+	}
+	if c.ConnMinPause < 0 || c.ConnMinPause > 100*time.Millisecond {
+		return errors.NotValidf("conn-min-pause %d < 0 or > 100ms", c.ConnMinPause)
+	}
+	if c.ConnMaxPause < 0 || c.ConnMaxPause > 10*time.Second {
+		return errors.NotValidf("conn-max-pause %d < 0 or > 10s", c.ConnMaxPause)
+	}
+	if c.ConnLookbackWindow < 0 || c.ConnLookbackWindow > 5*time.Second {
+		return errors.NotValidf("conn-lookback-window %d < 0 or > 5s", c.ConnMaxPause)
+	}
+	return nil
+}
+
+// LogSinkConfig holds parameters to control the API server's
+// logsink endpoint behaviour.
+type LogSinkConfig struct {
+	// DBLoggerBufferSize is the capacity of the database logger's buffer.
+	DBLoggerBufferSize int
+
+	// DBLoggerFlushInterval is the amount of time to allow a log record
+	// to sit in the buffer before being flushed to the database.
+	DBLoggerFlushInterval time.Duration
+
+	// RateLimitBurst defines the number of log messages that will be let
+	// through before we start rate limiting.
+	RateLimitBurst int64
+
+	// RateLimitRefill defines the rate at which log messages will be let
+	// through once the initial burst amount has been depleted.
+	RateLimitRefill time.Duration
+}
+
+// Validate validates the logsink endpoint configuration.
+func (cfg LogSinkConfig) Validate() error {
+	if cfg.DBLoggerBufferSize <= 0 || cfg.DBLoggerBufferSize > 1000 {
+		return errors.NotValidf("DBLoggerBufferSize %d <= 0 or > 1000", cfg.DBLoggerBufferSize)
+	}
+	if cfg.DBLoggerFlushInterval <= 0 || cfg.DBLoggerFlushInterval > 10*time.Second {
+		return errors.NotValidf("DBLoggerFlushInterval %s <= 0 or > 10 seconds", cfg.DBLoggerFlushInterval)
+	}
+	if cfg.RateLimitBurst <= 0 {
+		return errors.NotValidf("RateLimitBurst %d <= 0", cfg.RateLimitBurst)
+	}
+	if cfg.RateLimitRefill <= 0 {
+		return errors.NotValidf("RateLimitRefill %s <= 0", cfg.RateLimitRefill)
+	}
+	return nil
+}
+
+// DefaultLogSinkConfig returns a LogSinkConfig with default values.
+func DefaultLogSinkConfig() LogSinkConfig {
+	return LogSinkConfig{
+		DBLoggerBufferSize:    defaultDBLoggerBufferSize,
+		DBLoggerFlushInterval: defaultDBLoggerFlushInterval,
+		RateLimitBurst:        defaultLogSinkRateLimitBurst,
+		RateLimitRefill:       defaultLogSinkRateLimitRefill,
+	}
+}
+
 // NewServer serves the given state by accepting requests on the given
 // listener, using the given certificate and key (in PEM format) for
 // authentication.
 //
-// The Server will close the listener when it exits, even if returns an error.
-func NewServer(s *state.State, lis net.Listener, cfg ServerConfig) (*Server, error) {
+// The Server will not close the StatePool; the caller is responsible
+// for closing it after the Server has been stopped.
+//
+// The Server will close the listener when it exits, even if returns
+// an error.
+func NewServer(stPool *state.StatePool, lis net.Listener, cfg ServerConfig) (*Server, error) {
+	if cfg.LogSinkConfig == nil {
+		logSinkConfig := DefaultLogSinkConfig()
+		cfg.LogSinkConfig = &logSinkConfig
+	}
 	if err := cfg.Validate(); err != nil {
 		return nil, errors.Trace(err)
 	}
@@ -150,7 +330,7 @@ func NewServer(s *state.State, lis net.Listener, cfg ServerConfig) (*Server, err
 	// server needs to run before mongo upgrades have happened and
 	// any state manipulation may be be relying on features of the
 	// database added by upgrades. Here be dragons.
-	srv, err := newServer(s, lis, cfg)
+	srv, err := newServer(stPool, lis, cfg)
 	if err != nil {
 		// There is no running server around to close the listener.
 		lis.Close()
@@ -159,51 +339,90 @@ func NewServer(s *state.State, lis net.Listener, cfg ServerConfig) (*Server, err
 	return srv, nil
 }
 
-func newServer(s *state.State, lis net.Listener, cfg ServerConfig) (_ *Server, err error) {
-	stPool := cfg.StatePool
-	if stPool == nil {
-		stPool = state.NewStatePool(s)
-	}
-
+func newServer(stPool *state.StatePool, lis net.Listener, cfg ServerConfig) (_ *Server, err error) {
+	limiter := utils.NewLimiterWithPause(
+		cfg.RateLimitConfig.LoginRateLimit, cfg.RateLimitConfig.LoginMinPause,
+		cfg.RateLimitConfig.LoginMaxPause, clock.WallClock)
 	srv := &Server{
-		clock:       cfg.Clock,
-		pingClock:   cfg.pingClock(),
-		lis:         lis,
-		newObserver: cfg.NewObserver,
-		state:       s,
-		statePool:   stPool,
-		tag:         cfg.Tag,
-		dataDir:     cfg.DataDir,
-		logDir:      cfg.LogDir,
-		limiter:     utils.NewLimiter(loginRateLimit),
-		validator:   cfg.Validator,
-		adminAPIFactories: map[int]adminAPIFactory{
-			3: newAdminAPIV3,
+		clock:                         cfg.Clock,
+		pingClock:                     cfg.pingClock(),
+		lis:                           lis,
+		newObserver:                   cfg.NewObserver,
+		statePool:                     stPool,
+		tag:                           cfg.Tag,
+		dataDir:                       cfg.DataDir,
+		logDir:                        cfg.LogDir,
+		limiter:                       limiter,
+		loginRetryPause:               cfg.RateLimitConfig.LoginRetryPause,
+		validator:                     cfg.Validator,
+		facades:                       AllFacades(),
+		centralHub:                    cfg.Hub,
+		certChanged:                   cfg.CertChanged,
+		allowModelAccess:              cfg.AllowModelAccess,
+		publicDNSName_:                cfg.AutocertDNSName,
+		registerIntrospectionHandlers: cfg.RegisterIntrospectionHandlers,
+		logsinkRateLimitConfig: logsink.RateLimitConfig{
+			Refill: cfg.LogSinkConfig.RateLimitRefill,
+			Burst:  cfg.LogSinkConfig.RateLimitBurst,
+			Clock:  cfg.Clock,
+		},
+		dbloggers: dbloggers{
+			clock:                 cfg.Clock,
+			dbLoggerBufferSize:    cfg.LogSinkConfig.DBLoggerBufferSize,
+			dbLoggerFlushInterval: cfg.LogSinkConfig.DBLoggerFlushInterval,
 		},
-		certChanged:      cfg.CertChanged,
-		allowModelAccess: cfg.AllowModelAccess,
 	}
 
 	srv.tlsConfig = srv.newTLSConfig(cfg)
-	srv.lis = tls.NewListener(lis, srv.tlsConfig)
+	srv.lis = newThrottlingListener(
+		tls.NewListener(lis, srv.tlsConfig), cfg.RateLimitConfig, clock.WallClock)
 
-	srv.authCtxt, err = newAuthContext(s)
+	srv.authCtxt, err = newAuthContext(stPool.SystemState())
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
 	if err := srv.updateCertificate(cfg.Cert, cfg.Key); err != nil {
 		return nil, errors.Annotatef(err, "cannot set initial certificate")
 	}
-	logSinkWriter, err := newLogSinkWriter(srv.logDir)
+
+	logSinkWriter, err := logsink.NewFileWriter(filepath.Join(srv.logDir, "logsink.log"))
 	if err != nil {
 		return nil, errors.Annotate(err, "creating logsink writer")
 	}
 	srv.logSinkWriter = logSinkWriter
 
+	if cfg.PrometheusRegisterer != nil {
+		apiserverCollectior := NewMetricsCollector(&metricAdaptor{srv})
+		cfg.PrometheusRegisterer.Unregister(apiserverCollectior)
+		if err := cfg.PrometheusRegisterer.Register(apiserverCollectior); err != nil {
+			return nil, errors.Annotate(err, "registering apiserver metrics collector")
+		}
+	}
+
 	go srv.run()
 	return srv, nil
 }
 
+type metricAdaptor struct {
+	srv *Server
+}
+
+func (a *metricAdaptor) TotalConnections() int64 {
+	return a.srv.TotalConnections()
+}
+
+func (a *metricAdaptor) ConnectionCount() int64 {
+	return a.srv.ConnectionCount()
+}
+
+func (a *metricAdaptor) ConcurrentLoginAttempts() int64 {
+	return a.srv.LoginAttempts()
+}
+
+func (a *metricAdaptor) ConnectionPauseTime() time.Duration {
+	return a.srv.lis.(*throttlingListener).pauseTime()
+}
+
 func (srv *Server) newTLSConfig(cfg ServerConfig) *tls.Config {
 	tlsConfig := utils.SecureTLSConfig()
 	if cfg.AutocertDNSName == "" {
@@ -216,7 +435,7 @@ func (srv *Server) newTLSConfig(cfg ServerConfig) *tls.Config {
 	}
 	m := autocert.Manager{
 		Prompt:     autocert.AcceptTOS,
-		Cache:      srv.state.AutocertCache(),
+		Cache:      srv.statePool.SystemState().AutocertCache(),
 		HostPolicy: autocert.HostWhitelist(cfg.AutocertDNSName),
 	}
 	if cfg.AutocertURL != "" {
@@ -243,10 +462,21 @@ func (srv *Server) newTLSConfig(cfg ServerConfig) *tls.Config {
 	return tlsConfig
 }
 
+// TotalConnections returns the total number of connections ever made.
+func (srv *Server) TotalConnections() int64 {
+	return atomic.LoadInt64(&srv.totalConn)
+}
+
+// ConnectionCount returns the number of current connections.
 func (srv *Server) ConnectionCount() int64 {
 	return atomic.LoadInt64(&srv.connCount)
 }
 
+// LoginAttempts returns the number of current login attempts.
+func (srv *Server) LoginAttempts() int64 {
+	return atomic.LoadInt64(&srv.loginAttempts)
+}
+
 // Dead returns a channel that signals when the server has exited.
 func (srv *Server) Dead() <-chan struct{} {
 	return srv.tomb.Dead()
@@ -269,6 +499,21 @@ func (srv *Server) Wait() error {
 	return srv.tomb.Wait()
 }
 
+// loggoWrapper is an io.Writer() that forwards the messages to a loggo.Logger.
+// Unfortunately http takes a concrete stdlib log.Logger struct, and not an
+// interface, so we can't just proxy all of the log levels without inspecting
+// the string content. For now, we just want to get the messages into the log
+// file.
+type loggoWrapper struct {
+	logger loggo.Logger
+	level  loggo.Level
+}
+
+func (w *loggoWrapper) Write(content []byte) (int, error) {
+	w.logger.Logf(w.level, "%s", string(content))
+	return len(content), nil
+}
+
 func (srv *Server) run() {
 	logger.Infof("listening on %q", srv.lis.Addr())
 
@@ -279,12 +524,11 @@ func (srv *Server) run() {
 
 		// Break deadlocks caused by leadership BlockUntil... calls.
 		srv.statePool.KillWorkers()
-		srv.state.KillWorkers()
+		srv.statePool.SystemState().KillWorkers()
 
 		srv.wg.Wait() // wait for any outstanding requests to complete.
 		srv.tomb.Done()
-		srv.statePool.Close()
-		srv.state.Close()
+		srv.dbloggers.dispose()
 		srv.logSinkWriter.Close()
 	}()
 
@@ -325,6 +569,10 @@ func (srv *Server) run() {
 		httpSrv := &http.Server{
 			Handler:   mux,
 			TLSConfig: srv.tlsConfig,
+			ErrorLog: log.New(&loggoWrapper{
+				level:  loggo.WARNING,
+				logger: logger,
+			}, "", 0), // no prefix and no flags so log.Logger doesn't add extra prefixes
 		}
 		err := httpSrv.Serve(srv.lis)
 		// Normally logging an error at debug level would be grounds for a beating,
@@ -337,20 +585,14 @@ func (srv *Server) run() {
 }
 
 func (srv *Server) endpoints() []apihttp.Endpoint {
-	httpCtxt := httpContext{
-		srv: srv,
-	}
-
-	endpoints := common.ResolveAPIEndpoints(srv.newHandlerArgs)
-
-	// TODO(ericsnow) Add the following to the registry instead.
+	var endpoints []apihttp.Endpoint
 
 	add := func(pattern string, handler http.Handler) {
 		// TODO: We can switch from all methods to specific ones for entries
 		// where we only want to support specific request methods. However, our
 		// tests currently assert that errors come back as application/json and
 		// pat only does "text/plain" responses.
-		for _, method := range common.DefaultHTTPMethods {
+		for _, method := range defaultHTTPMethods {
 			endpoints = append(endpoints, apihttp.Endpoint{
 				Pattern: pattern,
 				Method:  method,
@@ -359,31 +601,122 @@ func (srv *Server) endpoints() []apihttp.Endpoint {
 		}
 	}
 
+	httpCtxt := httpContext{
+		srv: srv,
+	}
+
 	strictCtxt := httpCtxt
 	strictCtxt.strictValidation = true
 	strictCtxt.controllerModelOnly = true
 
 	mainAPIHandler := srv.trackRequests(http.HandlerFunc(srv.apiHandler))
-	logSinkHandler := newLogSinkHandler(httpCtxt, srv.logSinkWriter)
-	logStreamHandler := srv.trackRequests(newLogStreamEndpointHandler(strictCtxt))
+	logStreamHandler := srv.trackRequests(newLogStreamEndpointHandler(httpCtxt))
 	debugLogHandler := srv.trackRequests(newDebugLogDBHandler(httpCtxt))
+	pubsubHandler := srv.trackRequests(newPubSubHandler(httpCtxt, srv.centralHub))
 
-	add("/model/:modeluuid/logsink", srv.trackRequests(logSinkHandler))
+	// This handler is model specific even though it only ever makes sense
+	// for a controller because the API caller that is handed to the worker
+	// that is forwarding the messages between controllers is bound to the
+	// /model/:modeluuid namespace.
+	add("/model/:modeluuid/pubsub", pubsubHandler)
 	add("/model/:modeluuid/logstream", logStreamHandler)
 	add("/model/:modeluuid/log", debugLogHandler)
 
-	charmsHandler := &charmsHandler{
-		ctxt:    httpCtxt,
-		dataDir: srv.dataDir,
+	logSinkHandler := logsink.NewHTTPHandler(
+		newAgentLogWriteCloserFunc(httpCtxt, srv.logSinkWriter, &srv.dbloggers),
+		httpCtxt.stop(),
+		&srv.logsinkRateLimitConfig,
+	)
+	add("/model/:modeluuid/logsink", srv.trackRequests(logSinkHandler))
+
+	// We don't need to save the migrated logs to a logfile as well as to the DB.
+	logTransferHandler := logsink.NewHTTPHandler(
+		newMigrationLogWriteCloserFunc(httpCtxt, &srv.dbloggers),
+		httpCtxt.stop(),
+		nil, // no rate-limiting
+	)
+	add("/migrate/logtransfer", srv.trackRequests(logTransferHandler))
+
+	modelRestHandler := &modelRestHandler{
+		ctxt:          httpCtxt,
+		dataDir:       srv.dataDir,
+		stateAuthFunc: httpCtxt.stateForRequestAuthenticatedUser,
+	}
+	modelRestServer := &RestHTTPHandler{
+		GetHandler: modelRestHandler.ServeGet,
+	}
+	add("/model/:modeluuid/rest/1.0/:entity/:name/:attribute", modelRestServer)
+
+	modelCharmsHandler := &charmsHandler{
+		ctxt:          httpCtxt,
+		dataDir:       srv.dataDir,
+		stateAuthFunc: httpCtxt.stateForRequestAuthenticatedUser,
 	}
 	charmsServer := &CharmsHTTPHandler{
-		PostHandler: charmsHandler.ServePost,
-		GetHandler:  charmsHandler.ServeGet,
+		PostHandler: modelCharmsHandler.ServePost,
+		GetHandler:  modelCharmsHandler.ServeGet,
 	}
 	add("/model/:modeluuid/charms", charmsServer)
 	add("/model/:modeluuid/tools",
 		&toolsUploadHandler{
-			ctxt: httpCtxt,
+			ctxt:          httpCtxt,
+			stateAuthFunc: httpCtxt.stateForRequestAuthenticatedUser,
+		},
+	)
+
+	add("/model/:modeluuid/applications/:application/resources/:resource", &ResourcesHandler{
+		StateAuthFunc: func(req *http.Request, tagKinds ...string) (ResourcesBackend, state.StatePoolReleaser, names.Tag, error) {
+			st, closer, entity, err := httpCtxt.stateForRequestAuthenticatedTag(req, tagKinds...)
+			if err != nil {
+				return nil, nil, nil, errors.Trace(err)
+			}
+			rst, err := st.Resources()
+			if err != nil {
+				return nil, nil, nil, errors.Trace(err)
+			}
+			return rst, closer, entity.Tag(), nil
+		},
+	})
+	add("/model/:modeluuid/units/:unit/resources/:resource", &UnitResourcesHandler{
+		NewOpener: func(req *http.Request, tagKinds ...string) (resource.Opener, state.StatePoolReleaser, error) {
+			st, closer, _, err := httpCtxt.stateForRequestAuthenticatedTag(req, tagKinds...)
+			if err != nil {
+				return nil, nil, errors.Trace(err)
+			}
+			tagStr := req.URL.Query().Get(":unit")
+			tag, err := names.ParseUnitTag(tagStr)
+			if err != nil {
+				return nil, nil, errors.Trace(err)
+			}
+			opener, err := resourceadapters.NewResourceOpener(st, tag.Id())
+			if err != nil {
+				return nil, nil, errors.Trace(err)
+			}
+			return opener, closer, nil
+		},
+	})
+
+	migrateCharmsHandler := &charmsHandler{
+		ctxt:          httpCtxt,
+		dataDir:       srv.dataDir,
+		stateAuthFunc: httpCtxt.stateForMigrationImporting,
+	}
+	add("/migrate/charms",
+		&CharmsHTTPHandler{
+			PostHandler: migrateCharmsHandler.ServePost,
+			GetHandler:  migrateCharmsHandler.ServeUnsupported,
+		},
+	)
+	add("/migrate/tools",
+		&toolsUploadHandler{
+			ctxt:          httpCtxt,
+			stateAuthFunc: httpCtxt.stateForMigrationImporting,
+		},
+	)
+	add("/migrate/resources",
+		&resourcesMigrationUploadHandler{
+			ctxt:          httpCtxt,
+			stateAuthFunc: httpCtxt.stateForMigrationImporting,
 		},
 	)
 	add("/model/:modeluuid/tools/:version",
@@ -398,7 +731,8 @@ func (srv *Server) endpoints() []apihttp.Endpoint {
 	)
 	add("/model/:modeluuid/api", mainAPIHandler)
 
-	endpoints = append(endpoints, guiEndpoints("/gui/:modeluuid/", srv.dataDir, httpCtxt)...)
+	// GUI related paths.
+	endpoints = append(endpoints, guiEndpoints(guiURLPathPrefix, srv.dataDir, httpCtxt)...)
 	add("/gui-archive", &guiArchiveHandler{
 		ctxt: httpCtxt,
 	})
@@ -412,7 +746,8 @@ func (srv *Server) endpoints() []apihttp.Endpoint {
 	add("/charms", charmsServer)
 	add("/tools",
 		&toolsUploadHandler{
-			ctxt: httpCtxt,
+			ctxt:          httpCtxt,
+			stateAuthFunc: httpCtxt.stateForRequestAuthenticatedUser,
 		},
 	)
 	add("/tools/:version",
@@ -431,8 +766,21 @@ func (srv *Server) endpoints() []apihttp.Endpoint {
 	// possible endpoints, but only / itself.
 	add("/", mainAPIHandler)
 
+	// Register the introspection endpoints.
+	if srv.registerIntrospectionHandlers != nil {
+		handle := func(subpath string, handler http.Handler) {
+			add(path.Join("/introspection/", subpath),
+				introspectionHandler{
+					httpCtxt,
+					handler,
+				},
+			)
+		}
+		srv.registerIntrospectionHandlers(handle)
+	}
+
 	// Add HTTP handlers for local-user macaroon authentication.
-	localLoginHandlers := &localLoginHandlers{srv.authCtxt, srv.state}
+	localLoginHandlers := &localLoginHandlers{srv.authCtxt, srv.statePool.SystemState()}
 	dischargeMux := http.NewServeMux()
 	httpbakery.AddDischargeHandler(
 		dischargeMux,
@@ -468,35 +816,6 @@ func (srv *Server) expireLocalLoginInteractions() error {
 	}
 }
 
-func (srv *Server) newHandlerArgs(spec apihttp.HandlerConstraints) apihttp.NewHandlerArgs {
-	ctxt := httpContext{
-		srv:                 srv,
-		strictValidation:    spec.StrictValidation,
-		controllerModelOnly: spec.ControllerModelOnly,
-	}
-
-	var args apihttp.NewHandlerArgs
-	switch spec.AuthKind {
-	case names.UserTagKind:
-		args.Connect = ctxt.stateForRequestAuthenticatedUser
-	case names.UnitTagKind:
-		args.Connect = ctxt.stateForRequestAuthenticatedAgent
-	case "":
-		logger.Tracef(`no access level specified; proceeding with "unauthenticated"`)
-		args.Connect = func(req *http.Request) (*state.State, state.Entity, error) {
-			st, err := ctxt.stateForRequestUnauthenticated(req)
-			return st, nil, err
-		}
-	default:
-		logger.Infof(`unrecognized access level %q; proceeding with "unauthenticated"`, spec.AuthKind)
-		args.Connect = func(req *http.Request) (*state.State, state.Entity, error) {
-			st, err := ctxt.stateForRequestUnauthenticated(req)
-			return st, nil, err
-		}
-	}
-	return args
-}
-
 // trackRequests wraps a http.Handler, incrementing and decrementing
 // the apiserver's WaitGroup and blocking request when the apiserver
 // is shutting down.
@@ -540,6 +859,7 @@ func registerEndpoint(ep apihttp.Endpoint, mux *pat.PatternServeMux) {
 }
 
 func (srv *Server) apiHandler(w http.ResponseWriter, req *http.Request) {
+	atomic.AddInt64(&srv.totalConn, 1)
 	addCount := func(delta int64) {
 		atomic.AddInt64(&srv.connCount, delta)
 	}
@@ -553,21 +873,17 @@ func (srv *Server) apiHandler(w http.ResponseWriter, req *http.Request) {
 	apiObserver.Join(req, connectionID)
 	defer apiObserver.Leave()
 
-	wsServer := websocket.Server{
-		Handler: func(conn *websocket.Conn) {
-			modelUUID := req.URL.Query().Get(":modeluuid")
-			logger.Tracef("got a request for model %q", modelUUID)
-			if err := srv.serveConn(conn, modelUUID, apiObserver, req.Host); err != nil {
-				logger.Errorf("error serving RPCs: %v", err)
-			}
-		},
-	}
-	wsServer.ServeHTTP(w, req)
+	websocket.Serve(w, req, func(conn *websocket.Conn) {
+		modelUUID := req.URL.Query().Get(":modeluuid")
+		logger.Tracef("got a request for model %q", modelUUID)
+		if err := srv.serveConn(conn, modelUUID, apiObserver, req.Host); err != nil {
+			logger.Errorf("error serving RPCs: %v", err)
+		}
+	})
 }
 
 func (srv *Server) serveConn(wsConn *websocket.Conn, modelUUID string, apiObserver observer.Observer, host string) error {
-	codec := jsoncodec.NewWebsocket(wsConn)
-
+	codec := jsoncodec.NewWebsocket(wsConn.Conn)
 	conn := rpc.NewConn(codec, apiObserver)
 
 	// Note that we don't overwrite modelUUID here because
@@ -578,20 +894,16 @@ func (srv *Server) serveConn(wsConn *websocket.Conn, modelUUID string, apiObserv
 		modelUUID: modelUUID,
 	})
 	var (
-		st *state.State
-		h  *apiHandler
+		st       *state.State
+		h        *apiHandler
+		releaser state.StatePoolReleaser
 	)
 	if err == nil {
-		st, err = srv.statePool.Get(resolvedModelUUID)
+		st, releaser, err = srv.statePool.Get(resolvedModelUUID)
 	}
 
 	if err == nil {
-		defer func() {
-			err := srv.statePool.Release(resolvedModelUUID)
-			if err != nil {
-				logger.Errorf("error releasing %v back into the state pool:", err)
-			}
-		}()
+		defer releaser()
 		h, err = newAPIHandler(srv, st, conn, modelUUID, host)
 	}
 
@@ -599,7 +911,7 @@ func (srv *Server) serveConn(wsConn *websocket.Conn, modelUUID string, apiObserv
 		conn.ServeRoot(&errRoot{errors.Trace(err)}, serverError)
 	} else {
 		adminAPIs := make(map[int]interface{})
-		for apiVersion, factory := range srv.adminAPIFactories {
+		for apiVersion, factory := range adminAPIFactories {
 			adminAPIs[apiVersion] = factory(srv, h, apiObserver)
 		}
 		conn.ServeRoot(newAnonRoot(h, adminAPIs), serverError)
@@ -613,7 +925,7 @@ func (srv *Server) serveConn(wsConn *websocket.Conn, modelUUID string, apiObserv
 }
 
 func (srv *Server) mongoPinger() error {
-	session := srv.state.MongoSession().Copy()
+	session := srv.statePool.SystemState().MongoSession().Copy()
 	defer session.Close()
 	for {
 		if err := session.Ping(); err != nil {
@@ -628,6 +940,13 @@ func (srv *Server) mongoPinger() error {
 	}
 }
 
+// publicDNSName returns the current public hostname.
+func (srv *Server) publicDNSName() string {
+	srv.mu.Lock()
+	defer srv.mu.Unlock()
+	return srv.publicDNSName_
+}
+
 // localCertificate returns the local server certificate and reports
 // whether it should be used to serve a connection addressed to the
 // given server name.
@@ -704,7 +1023,8 @@ func serverError(err error) error {
 }
 
 func (srv *Server) processModelRemovals() error {
-	w := srv.state.WatchModels()
+	st := srv.statePool.SystemState()
+	w := st.WatchModelLives()
 	defer w.Stop()
 	for {
 		select {
@@ -712,7 +1032,7 @@ func (srv *Server) processModelRemovals() error {
 			return tomb.ErrDying
 		case modelUUIDs := <-w.Changes():
 			for _, modelUUID := range modelUUIDs {
-				model, err := srv.state.GetModel(names.NewModelTag(modelUUID))
+				model, err := st.GetModel(names.NewModelTag(modelUUID))
 				gone := errors.IsNotFound(err)
 				dead := err == nil && model.Life() == state.Dead
 				if err != nil && !gone {
@@ -726,8 +1046,7 @@ func (srv *Server) processModelRemovals() error {
 				// Model's gone away - ensure that it gets removed
 				// from from the state pool once people are finished
 				// with it.
-				err = srv.statePool.Remove(modelUUID)
-				if err != nil {
+				if _, err := srv.statePool.Remove(modelUUID); err != nil {
 					return errors.Trace(err)
 				}
 			}
