@@ -8,6 +8,8 @@ import (
 
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
+	"github.com/juju/utils/os"
+	"github.com/juju/utils/series"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/apiserver/application"
@@ -17,7 +19,8 @@ import (
 	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/config"
-	"github.com/juju/juju/environs/manual"
+	"github.com/juju/juju/environs/manual/sshprovisioner"
+	"github.com/juju/juju/environs/manual/winrmprovisioner"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/permission"
@@ -26,10 +29,6 @@ import (
 	jujuversion "github.com/juju/juju/version"
 )
 
-func init() {
-	common.RegisterStandardFacade("Client", 1, newClient)
-}
-
 var logger = loggo.GetLogger("juju.apiserver.client")
 
 type API struct {
@@ -50,11 +49,18 @@ func (api *API) state() *state.State {
 	return api.stateAccessor.(stateShim).State
 }
 
+// caCerter implements the subset of common.APIAddresser
+// methods that we choose to expose in the client facade.
+type caCerter interface {
+	CACert() params.BytesResult
+}
+
 // Client serves client-specific API methods.
 type Client struct {
 	// TODO(wallyworld) - we'll retain model config facade methods
 	// on the client facade until GUI and Python client library are updated.
 	*modelconfig.ModelConfigAPI
+	caCerter
 
 	api        *API
 	newEnviron func() (environs.Environ, error)
@@ -62,28 +68,39 @@ type Client struct {
 }
 
 func (c *Client) checkCanRead() error {
+	isAdmin, err := c.api.auth.HasPermission(permission.SuperuserAccess, c.api.stateAccessor.ControllerTag())
+	if err != nil {
+		return errors.Trace(err)
+	}
+
 	canRead, err := c.api.auth.HasPermission(permission.ReadAccess, c.api.stateAccessor.ModelTag())
 	if err != nil {
 		return errors.Trace(err)
 	}
-	if !canRead {
+	if !canRead && !isAdmin {
 		return common.ErrPerm
 	}
 	return nil
 }
 
 func (c *Client) checkCanWrite() error {
+	isAdmin, err := c.api.auth.HasPermission(permission.SuperuserAccess, c.api.stateAccessor.ControllerTag())
+	if err != nil {
+		return errors.Trace(err)
+	}
+
 	canWrite, err := c.api.auth.HasPermission(permission.WriteAccess, c.api.stateAccessor.ModelTag())
 	if err != nil {
 		return errors.Trace(err)
 	}
-	if !canWrite {
+	if !canWrite && !isAdmin {
 		return common.ErrPerm
 	}
 	return nil
 }
 
-func newClient(st *state.State, resources facade.Resources, authorizer facade.Authorizer) (*Client, error) {
+// NewFacade provides the required signature for facade registration.
+func NewFacade(st *state.State, resources facade.Resources, authorizer facade.Authorizer) (*Client, error) {
 	urlGetter := common.NewToolsURLGetter(st.ModelUUID(), st)
 	configGetter := stateenvirons.EnvironConfigGetter{st}
 	statusSetter := common.NewStatusSetter(st, common.AuthAlways())
@@ -96,6 +113,7 @@ func newClient(st *state.State, resources facade.Resources, authorizer facade.Au
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
+	addresser := common.NewAPIAddresser(st, resources)
 	return NewClient(
 		NewStateBackend(st),
 		modelConfigAPI,
@@ -105,6 +123,7 @@ func newClient(st *state.State, resources facade.Resources, authorizer facade.Au
 		toolsFinder,
 		newEnviron,
 		blockChecker,
+		addresser,
 	)
 }
 
@@ -118,30 +137,47 @@ func NewClient(
 	toolsFinder *common.ToolsFinder,
 	newEnviron func() (environs.Environ, error),
 	blockChecker *common.BlockChecker,
+	caCerter caCerter,
 ) (*Client, error) {
 	if !authorizer.AuthClient() {
 		return nil, common.ErrPerm
 	}
 	client := &Client{
-		modelConfigAPI,
-		&API{
+		ModelConfigAPI: modelConfigAPI,
+		caCerter:       caCerter,
+		api: &API{
 			stateAccessor: st,
 			auth:          authorizer,
 			resources:     resources,
 			statusSetter:  statusSetter,
 			toolsFinder:   toolsFinder,
 		},
-		newEnviron,
-		blockChecker,
+		newEnviron: newEnviron,
+		check:      blockChecker,
 	}
 	return client, nil
 }
 
+// WatchAll initiates a watcher for entities in the connected model.
 func (c *Client) WatchAll() (params.AllWatcherId, error) {
 	if err := c.checkCanRead(); err != nil {
 		return params.AllWatcherId{}, err
 	}
-	w := c.api.stateAccessor.Watch()
+	model, err := c.api.stateAccessor.Model()
+	if err != nil {
+		return params.AllWatcherId{}, errors.Trace(err)
+	}
+
+	// Since we know this is a user tag (because AuthClient is true),
+	// we just do the type assertion to the UserTag.
+	apiUser, _ := c.api.auth.GetAuthTag().(names.UserTag)
+	isAdmin, err := common.HasModelAdmin(c.api.auth, apiUser, c.api.stateAccessor.ControllerTag(), model)
+	if err != nil {
+		return params.AllWatcherId{}, errors.Trace(err)
+	}
+	watchParams := state.WatchParams{IncludeOffers: isAdmin}
+
+	w := c.api.stateAccessor.Watch(watchParams)
 	return params.AllWatcherId{
 		AllWatcherId: c.api.resources.Register(w),
 	}, nil
@@ -374,7 +410,6 @@ func (c *Client) ProvisioningScript(args params.ProvisioningScriptParams) (param
 			err, "getting instance config",
 		))
 	}
-
 	// Until DisablePackageCommands is retired, for backwards
 	// compatibility, we must respect the client's request and
 	// override any model settings the user may have specified.
@@ -393,12 +428,24 @@ func (c *Client) ProvisioningScript(args params.ProvisioningScriptParams) (param
 		icfg.EnableOSRefreshUpdate = cfg.EnableOSRefreshUpdate()
 	}
 
-	result.Script, err = manual.ProvisioningScript(icfg)
+	osSeries, err := series.GetOSFromSeries(icfg.Series)
+	if err != nil {
+		return result, common.ServerError(errors.Annotatef(err,
+			"cannot decide which provisioning script to generate based on this series %q", icfg.Series))
+	}
+
+	getProvisioningScript := sshprovisioner.ProvisioningScript
+	if osSeries == os.Windows {
+		getProvisioningScript = winrmprovisioner.ProvisioningScript
+	}
+
+	result.Script, err = getProvisioningScript(icfg)
 	if err != nil {
 		return result, common.ServerError(errors.Annotate(
 			err, "getting provisioning script",
 		))
 	}
+
 	return result, nil
 }
 
@@ -417,7 +464,7 @@ func (c *Client) DestroyMachines(args params.DestroyMachines) error {
 
 // ModelInfo returns information about the current model.
 func (c *Client) ModelInfo() (params.ModelInfo, error) {
-	if err := c.checkCanWrite(); err != nil {
+	if err := c.checkCanRead(); err != nil {
 		return params.ModelInfo{}, err
 	}
 	state := c.api.stateAccessor
@@ -439,6 +486,9 @@ func (c *Client) ModelInfo() (params.ModelInfo, error) {
 		OwnerTag:      model.Owner().String(),
 		Life:          params.Life(model.Life().String()),
 	}
+	if agentVersion, exists := conf.AgentVersion(); exists {
+		info.AgentVersion = &agentVersion
+	}
 	if tag, ok := model.CloudCredential(); ok {
 		info.CloudCredentialTag = tag.String()
 	}
@@ -505,7 +555,21 @@ func (c *Client) SetModelAgentVersion(args params.SetModelAgentVersion) error {
 	if err := environs.CheckProviderAPI(env); err != nil {
 		return err
 	}
-	return c.api.stateAccessor.SetModelAgentVersion(args.Version)
+	// If this is the controller model, also check to make sure that there are
+	// no running migrations.  All models should have migration mode of None.
+	if c.api.stateAccessor.IsController() {
+		models, err := c.api.stateAccessor.AllModels()
+		if err != nil {
+			return errors.Trace(err)
+		}
+		for _, model := range models {
+			if mode := model.MigrationMode(); mode != state.MigrationModeNone {
+				return errors.Errorf("model \"%s/%s\" is %s, upgrade blocked", model.Owner().Name(), model.Name(), mode)
+			}
+		}
+	}
+
+	return c.api.stateAccessor.SetModelAgentVersion(args.Version, args.IgnoreAgentVersions)
 }
 
 // AbortCurrentUpgrade aborts and archives the current upgrade
