@@ -18,17 +18,18 @@ import (
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/charm.v6-unstable"
 	"gopkg.in/juju/names.v2"
+	worker "gopkg.in/juju/worker.v1"
 
 	"github.com/juju/juju/agent"
 	"github.com/juju/juju/apiserver/client"
 	"github.com/juju/juju/apiserver/common"
-	"github.com/juju/juju/apiserver/modelconfig"
+	"github.com/juju/juju/apiserver/facade"
 	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/apiserver/testing"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/config"
-	"github.com/juju/juju/environs/manual"
+	"github.com/juju/juju/environs/manual/sshprovisioner"
 	toolstesting "github.com/juju/juju/environs/tools/testing"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/network"
@@ -42,7 +43,6 @@ import (
 	coretesting "github.com/juju/juju/testing"
 	"github.com/juju/juju/testing/factory"
 	jujuversion "github.com/juju/juju/version"
-	"github.com/juju/juju/worker"
 )
 
 type serverSuite struct {
@@ -58,36 +58,26 @@ func (s *serverSuite) SetUpTest(c *gc.C) {
 		"authorized-keys": coretesting.FakeAuthKeys,
 	}
 	s.baseSuite.SetUpTest(c)
+	s.client = s.clientForState(c, s.State)
+}
 
-	var err error
-	auth := testing.FakeAuthorizer{
-		Tag:            s.AdminUserTag(c),
-		EnvironManager: true,
-	}
-	urlGetter := common.NewToolsURLGetter(s.State.ModelUUID(), s.State)
-	configGetter := stateenvirons.EnvironConfigGetter{s.State}
-	statusSetter := common.NewStatusSetter(s.State, common.AuthAlways())
-	toolsFinder := common.NewToolsFinder(configGetter, s.State, urlGetter)
+func (s *serverSuite) authClientForState(c *gc.C, st *state.State, auth facade.Authorizer) *client.Client {
+	apiserverClient, err := client.NewFacade(st, common.NewResources(), auth)
+	c.Assert(err, jc.ErrorIsNil)
 	s.newEnviron = func() (environs.Environ, error) {
-		return environs.GetEnviron(configGetter, environs.New)
+		return environs.GetEnviron(stateenvirons.EnvironConfigGetter{st}, environs.New)
 	}
-	newEnviron := func() (environs.Environ, error) {
+	client.SetNewEnviron(apiserverClient, func() (environs.Environ, error) {
 		return s.newEnviron()
-	}
-	blockChecker := common.NewBlockChecker(s.State)
-	modelConfigAPI, err := modelconfig.NewModelConfigAPI(s.State, auth)
-	c.Assert(err, jc.ErrorIsNil)
-	s.client, err = client.NewClient(
-		client.NewStateBackend(s.State),
-		modelConfigAPI,
-		common.NewResources(),
-		auth,
-		statusSetter,
-		toolsFinder,
-		newEnviron,
-		blockChecker,
-	)
-	c.Assert(err, jc.ErrorIsNil)
+	})
+	return apiserverClient
+}
+
+func (s *serverSuite) clientForState(c *gc.C, st *state.State) *client.Client {
+	return s.authClientForState(c, st, testing.FakeAuthorizer{
+		Tag:        s.AdminUserTag(c),
+		Controller: true,
+	})
 }
 
 func (s *serverSuite) setAgentPresence(c *gc.C, machineId string) *presence.Pinger {
@@ -108,7 +98,12 @@ func (s *serverSuite) TestModelInfo(c *gc.C) {
 	model, err := s.State.Model()
 	c.Assert(err, jc.ErrorIsNil)
 	conf, _ := s.State.ModelConfig()
-	info, err := s.client.ModelInfo()
+	// Model info is available to read-only users.
+	client := s.authClientForState(c, s.State, testing.FakeAuthorizer{
+		Tag:        names.NewUserTag("read"),
+		Controller: true,
+	})
+	info, err := client.ModelInfo()
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(info.DefaultSeries, gc.Equals, config.PreferredSeries(conf))
 	c.Assert(info.CloudRegion, gc.Equals, model.CloudRegion())
@@ -117,6 +112,8 @@ func (s *serverSuite) TestModelInfo(c *gc.C) {
 	c.Assert(info.UUID, gc.Equals, model.UUID())
 	c.Assert(info.OwnerTag, gc.Equals, model.Owner().String())
 	c.Assert(info.Life, gc.Equals, params.Alive)
+	expectedAgentVersion, _ := conf.AgentVersion()
+	c.Assert(info.AgentVersion, gc.DeepEquals, &expectedAgentVersion)
 	// The controller UUID is not returned by the ModelInfo endpoint on the
 	// Client facade.
 	c.Assert(info.ControllerUUID, gc.Equals, "")
@@ -212,18 +209,114 @@ func (s *serverSuite) makeLocalModelUser(c *gc.C, username, displayname string)
 	return modelUser
 }
 
-func (s *serverSuite) TestSetEnvironAgentVersion(c *gc.C) {
+func (s *serverSuite) assertModelVersion(c *gc.C, st *state.State, expected string) {
+	modelConfig, err := st.ModelConfig()
+	c.Assert(err, jc.ErrorIsNil)
+	agentVersion, found := modelConfig.AllAttrs()["agent-version"]
+	c.Assert(found, jc.IsTrue)
+	c.Assert(agentVersion, gc.Equals, expected)
+}
+
+func (s *serverSuite) TestSetModelAgentVersion(c *gc.C) {
 	args := params.SetModelAgentVersion{
 		Version: version.MustParse("9.8.7"),
 	}
 	err := s.client.SetModelAgentVersion(args)
 	c.Assert(err, jc.ErrorIsNil)
+	s.assertModelVersion(c, s.State, "9.8.7")
+}
 
-	modelConfig, err := s.State.ModelConfig()
+func (s *serverSuite) TestSetModelAgentVersionForced(c *gc.C) {
+	// Get the agent-version set in the model.
+	cfg, err := s.State.ModelConfig()
 	c.Assert(err, jc.ErrorIsNil)
-	agentVersion, found := modelConfig.AllAttrs()["agent-version"]
-	c.Assert(found, jc.IsTrue)
-	c.Assert(agentVersion, gc.Equals, "9.8.7")
+	agentVersion, ok := cfg.AgentVersion()
+	c.Assert(ok, jc.IsTrue)
+	currentVersion := agentVersion.String()
+
+	// Add a machine with the current version and a unit with a different version
+	machine, err := s.State.AddMachine("series", state.JobHostUnits)
+	c.Assert(err, jc.ErrorIsNil)
+	service, err := s.State.AddApplication(state.AddApplicationArgs{Name: "wordpress", Charm: s.AddTestingCharm(c, "wordpress")})
+	c.Assert(err, jc.ErrorIsNil)
+	unit, err := service.AddUnit()
+	c.Assert(err, jc.ErrorIsNil)
+
+	err = machine.SetAgentVersion(version.MustParseBinary(currentVersion + "-quantal-amd64"))
+	c.Assert(err, jc.ErrorIsNil)
+	err = unit.SetAgentVersion(version.MustParseBinary("1.0.2-quantal-amd64"))
+	c.Assert(err, jc.ErrorIsNil)
+
+	// This should be refused because an agent doesn't match "currentVersion"
+	args := params.SetModelAgentVersion{
+		Version: version.MustParse("9.8.7"),
+	}
+	err = s.client.SetModelAgentVersion(args)
+	c.Check(err, gc.ErrorMatches, "some agents have not upgraded to the current model version .*: unit-wordpress-0")
+	// Version hasn't changed
+	s.assertModelVersion(c, s.State, currentVersion)
+	// But we can force it
+	args = params.SetModelAgentVersion{
+		Version:             version.MustParse("7.8.6"),
+		IgnoreAgentVersions: true,
+	}
+	err = s.client.SetModelAgentVersion(args)
+	c.Assert(err, jc.ErrorIsNil)
+	s.assertModelVersion(c, s.State, "7.8.6")
+}
+
+func (s *serverSuite) makeMigratingModel(c *gc.C, name string, mode state.MigrationMode) {
+	otherSt := s.Factory.MakeModel(c, &factory.ModelParams{
+		Name:  name,
+		Owner: names.NewUserTag("some-user"),
+	})
+	defer otherSt.Close()
+	model, err := otherSt.Model()
+	c.Assert(err, jc.ErrorIsNil)
+	err = model.SetMigrationMode(mode)
+	c.Assert(err, jc.ErrorIsNil)
+}
+
+func (s *serverSuite) TestControllerModelSetModelAgentVersionBlockedByImportingModel(c *gc.C) {
+	s.Factory.MakeUser(c, &factory.UserParams{Name: "some-user"})
+	s.makeMigratingModel(c, "to-migrate", state.MigrationModeImporting)
+	args := params.SetModelAgentVersion{
+		Version: version.MustParse("9.8.7"),
+	}
+	err := s.client.SetModelAgentVersion(args)
+	c.Assert(err, gc.ErrorMatches, `model "some-user/to-migrate" is importing, upgrade blocked`)
+}
+
+func (s *serverSuite) TestControllerModelSetModelAgentVersionBlockedByExportingModel(c *gc.C) {
+	s.Factory.MakeUser(c, &factory.UserParams{Name: "some-user"})
+	s.makeMigratingModel(c, "to-migrate", state.MigrationModeExporting)
+	args := params.SetModelAgentVersion{
+		Version: version.MustParse("9.8.7"),
+	}
+	err := s.client.SetModelAgentVersion(args)
+	c.Assert(err, gc.ErrorMatches, `model "some-user/to-migrate" is exporting, upgrade blocked`)
+}
+
+func (s *serverSuite) TestUserModelSetModelAgentVersionNotAffectedByMigration(c *gc.C) {
+	s.Factory.MakeUser(c, &factory.UserParams{Name: "some-user"})
+	otherSt := s.Factory.MakeModel(c, nil)
+	defer otherSt.Close()
+
+	s.makeMigratingModel(c, "exporting-model", state.MigrationModeExporting)
+	s.makeMigratingModel(c, "importing-model", state.MigrationModeImporting)
+	args := params.SetModelAgentVersion{
+		Version: version.MustParse("2.0.4"),
+	}
+	client := s.clientForState(c, otherSt)
+
+	s.newEnviron = func() (environs.Environ, error) {
+		return &mockEnviron{}, nil
+	}
+
+	err := client.SetModelAgentVersion(args)
+	c.Assert(err, jc.ErrorIsNil)
+
+	s.assertModelVersion(c, otherSt, "2.0.4")
 }
 
 type mockEnviron struct {
@@ -264,7 +357,7 @@ func (s *serverSuite) TestCheckProviderAPIFail(c *gc.C) {
 	s.assertCheckProviderAPI(c, errors.New("instances error"), "cannot make API call to provider: instances error")
 }
 
-func (s *serverSuite) assertSetEnvironAgentVersion(c *gc.C) {
+func (s *serverSuite) assertSetModelAgentVersion(c *gc.C) {
 	args := params.SetModelAgentVersion{
 		Version: version.MustParse("9.8.7"),
 	}
@@ -277,7 +370,7 @@ func (s *serverSuite) assertSetEnvironAgentVersion(c *gc.C) {
 	c.Assert(agentVersion, gc.Equals, "9.8.7")
 }
 
-func (s *serverSuite) assertSetEnvironAgentVersionBlocked(c *gc.C, msg string) {
+func (s *serverSuite) assertSetModelAgentVersionBlocked(c *gc.C, msg string) {
 	args := params.SetModelAgentVersion{
 		Version: version.MustParse("9.8.7"),
 	}
@@ -285,19 +378,19 @@ func (s *serverSuite) assertSetEnvironAgentVersionBlocked(c *gc.C, msg string) {
 	s.AssertBlocked(c, err, msg)
 }
 
-func (s *serverSuite) TestBlockDestroySetEnvironAgentVersion(c *gc.C) {
-	s.BlockDestroyModel(c, "TestBlockDestroySetEnvironAgentVersion")
-	s.assertSetEnvironAgentVersion(c)
+func (s *serverSuite) TestBlockDestroySetModelAgentVersion(c *gc.C) {
+	s.BlockDestroyModel(c, "TestBlockDestroySetModelAgentVersion")
+	s.assertSetModelAgentVersion(c)
 }
 
-func (s *serverSuite) TestBlockRemoveSetEnvironAgentVersion(c *gc.C) {
-	s.BlockRemoveObject(c, "TestBlockRemoveSetEnvironAgentVersion")
-	s.assertSetEnvironAgentVersion(c)
+func (s *serverSuite) TestBlockRemoveSetModelAgentVersion(c *gc.C) {
+	s.BlockRemoveObject(c, "TestBlockRemoveSetModelAgentVersion")
+	s.assertSetModelAgentVersion(c)
 }
 
-func (s *serverSuite) TestBlockChangesSetEnvironAgentVersion(c *gc.C) {
-	s.BlockAllChanges(c, "TestBlockChangesSetEnvironAgentVersion")
-	s.assertSetEnvironAgentVersionBlocked(c, "TestBlockChangesSetEnvironAgentVersion")
+func (s *serverSuite) TestBlockChangesSetModelAgentVersion(c *gc.C) {
+	s.BlockAllChanges(c, "TestBlockChangesSetModelAgentVersion")
+	s.assertSetModelAgentVersionBlocked(c, "TestBlockChangesSetModelAgentVersion")
 }
 
 func (s *serverSuite) TestAbortCurrentUpgrade(c *gc.C) {
@@ -386,8 +479,8 @@ var _ = gc.Suite(&clientSuite{})
 // clearSinceTimes zeros out the updated timestamps inside status
 // so we can easily check the results.
 func clearSinceTimes(status *params.FullStatus) {
-	for applicationId, service := range status.Applications {
-		for unitId, unit := range service.Units {
+	for applicationId, application := range status.Applications {
+		for unitId, unit := range application.Units {
 			unit.WorkloadStatus.Since = nil
 			unit.AgentStatus.Since = nil
 			for id, subord := range unit.Subordinates {
@@ -395,16 +488,21 @@ func clearSinceTimes(status *params.FullStatus) {
 				subord.AgentStatus.Since = nil
 				unit.Subordinates[id] = subord
 			}
-			service.Units[unitId] = unit
+			application.Units[unitId] = unit
 		}
-		service.Status.Since = nil
-		status.Applications[applicationId] = service
+		application.Status.Since = nil
+		status.Applications[applicationId] = application
+	}
+	for applicationId, application := range status.RemoteApplications {
+		application.Status.Since = nil
+		status.RemoteApplications[applicationId] = application
 	}
 	for id, machine := range status.Machines {
 		machine.AgentStatus.Since = nil
 		machine.InstanceStatus.Since = nil
 		status.Machines[id] = machine
 	}
+	status.Model.ModelStatus.Since = nil
 }
 
 func (s *clientSuite) TestClientStatus(c *gc.C) {
@@ -518,6 +616,12 @@ func (s *clientSuite) assertResolvedBlocked(c *gc.C, u *state.Unit, msg string)
 	s.AssertBlocked(c, err, msg)
 }
 
+func (s *serverSuite) TestCACert(c *gc.C) {
+	r, err := s.APIState.Client().CACert()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(r, gc.Equals, coretesting.CACert)
+}
+
 func (s *clientSuite) TestBlockDestroyUnitResolved(c *gc.C) {
 	u := s.setupResolved(c)
 	s.BlockDestroyModel(c, "TestBlockDestroyUnitResolved")
@@ -567,7 +671,7 @@ func (s *clientRepoSuite) TearDownTest(c *gc.C) {
 	s.baseSuite.TearDownTest(c)
 }
 
-func (s *clientSuite) TestClientWatchAll(c *gc.C) {
+func (s *clientSuite) TestClientWatchAllReadPermission(c *gc.C) {
 	loggo.GetLogger("juju.apiserver").SetLogLevel(loggo.TRACE)
 	// A very simple end-to-end test, because
 	// all the logic is tested elsewhere.
@@ -575,7 +679,15 @@ func (s *clientSuite) TestClientWatchAll(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	err = m.SetProvisioned("i-0", agent.BootstrapNonce, nil)
 	c.Assert(err, jc.ErrorIsNil)
-	watcher, err := s.APIState.Client().WatchAll()
+
+	user := s.Factory.MakeUser(c, &factory.UserParams{
+		Password: "ro-password",
+	})
+	c.Assert(err, jc.ErrorIsNil)
+	roClient := s.OpenAPIAs(c, user.UserTag(), "ro-password").Client()
+	defer roClient.Close()
+
+	watcher, err := roClient.WatchAll()
 	c.Assert(err, jc.ErrorIsNil)
 	defer func() {
 		err := watcher.Stop()
@@ -615,6 +727,101 @@ func (s *clientSuite) TestClientWatchAll(c *gc.C) {
 	}
 }
 
+func (s *clientSuite) TestClientWatchAllAdminPermission(c *gc.C) {
+	loggo.GetLogger("juju.apiserver").SetLogLevel(loggo.TRACE)
+	// A very simple end-to-end test, because
+	// all the logic is tested elsewhere.
+	m, err := s.State.AddMachine("quantal", state.JobManageModel)
+	c.Assert(err, jc.ErrorIsNil)
+	err = m.SetProvisioned("i-0", agent.BootstrapNonce, nil)
+	c.Assert(err, jc.ErrorIsNil)
+	// Include a remote app that needs admin access to see.
+	_, err = s.State.AddRemoteApplication(state.AddRemoteApplicationParams{
+		Name:        "remote-db2",
+		OfferName:   "hosted-db2",
+		URL:         "admin/prod.db2",
+		SourceModel: coretesting.ModelTag,
+		Endpoints: []charm.Relation{
+			{
+				Name:      "database",
+				Interface: "db2",
+				Role:      charm.RoleProvider,
+				Scope:     charm.ScopeGlobal,
+			},
+		},
+	})
+	c.Assert(err, jc.ErrorIsNil)
+
+	watcher, err := s.APIState.Client().WatchAll()
+	c.Assert(err, jc.ErrorIsNil)
+	defer func() {
+		err := watcher.Stop()
+		c.Assert(err, jc.ErrorIsNil)
+	}()
+	deltas, err := watcher.Next()
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Assert(len(deltas), gc.Equals, 2)
+	mIndex := 0
+	aIndex := 1
+	dMachine, ok0 := deltas[mIndex].Entity.(*multiwatcher.MachineInfo)
+	dApp, ok1 := deltas[aIndex].Entity.(*multiwatcher.RemoteApplicationInfo)
+	if !ok0 {
+		mIndex = 1
+		aIndex = 0
+		dMachine, ok0 = deltas[mIndex].Entity.(*multiwatcher.MachineInfo)
+		dApp, ok1 = deltas[aIndex].Entity.(*multiwatcher.RemoteApplicationInfo)
+	}
+	c.Assert(ok0, jc.IsTrue)
+	c.Assert(ok1, jc.IsTrue)
+	dMachine.AgentStatus.Since = nil
+	dMachine.InstanceStatus.Since = nil
+	dApp.Status.Since = nil
+
+	if !c.Check(deltas[mIndex], jc.DeepEquals, multiwatcher.Delta{
+		Entity: &multiwatcher.MachineInfo{
+			ModelUUID:  s.State.ModelUUID(),
+			Id:         m.Id(),
+			InstanceId: "i-0",
+			AgentStatus: multiwatcher.StatusInfo{
+				Current: status.Pending,
+			},
+			InstanceStatus: multiwatcher.StatusInfo{
+				Current: status.Pending,
+			},
+			Life:                    multiwatcher.Life("alive"),
+			Series:                  "quantal",
+			Jobs:                    []multiwatcher.MachineJob{state.JobManageModel.ToParams()},
+			Addresses:               []multiwatcher.Address{},
+			HardwareCharacteristics: &instance.HardwareCharacteristics{},
+			HasVote:                 false,
+			WantsVote:               true,
+		},
+	}) {
+		c.Logf("got:")
+		for _, d := range deltas {
+			c.Logf("%#v\n", d.Entity)
+		}
+	}
+	if !c.Check(deltas[aIndex], jc.DeepEquals, multiwatcher.Delta{
+		Entity: &multiwatcher.RemoteApplicationInfo{
+			Name:           "remote-db2",
+			ModelUUID:      s.State.ModelUUID(),
+			ApplicationURL: "admin/prod.db2",
+			Life:           "alive",
+			Status: multiwatcher.StatusInfo{
+				Current: status.Unknown,
+				Message: "waiting for remote connection",
+			},
+		},
+	}) {
+		c.Logf("got:")
+		for _, d := range deltas {
+			c.Logf("%#v\n", d.Entity)
+		}
+	}
+}
+
 func (s *clientSuite) TestClientSetModelConstraints(c *gc.C) {
 	// Set constraints for the model.
 	cons, err := constraints.Parse("mem=4096", "cores=2")
@@ -879,7 +1086,7 @@ func (s *clientSuite) TestClientAddMachineInsideMachine(c *gc.C) {
 // updateConfig sets config variable with given key to a given value
 // Asserts that no errors were encountered.
 func (s *baseSuite) updateConfig(c *gc.C, key string, block bool) {
-	err := s.State.UpdateModelConfig(map[string]interface{}{key: block}, nil, nil)
+	err := s.State.UpdateModelConfig(map[string]interface{}{key: block}, nil)
 	c.Assert(err, jc.ErrorIsNil)
 }
 
@@ -1050,7 +1257,7 @@ func (s *clientSuite) TestProvisioningScript(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	icfg, err := client.InstanceConfig(s.State, machineId, apiParams.Nonce, "")
 	c.Assert(err, jc.ErrorIsNil)
-	provisioningScript, err := manual.ProvisioningScript(icfg)
+	provisioningScript, err := sshprovisioner.ProvisioningScript(icfg)
 	c.Assert(err, jc.ErrorIsNil)
 	// ProvisioningScript internally calls MachineConfig,
 	// which allocates a new, random password. Everything
@@ -1091,7 +1298,6 @@ func (s *clientSuite) TestProvisioningScriptDisablePackageCommands(c *gc.C) {
 				"enable-os-refresh-update": update,
 			},
 			nil,
-			nil,
 		)
 	}
 
