@@ -3,16 +3,18 @@
 
 package client
 
+import (
+	"github.com/juju/juju/environs"
+)
+
 // Filtering exports
 var (
 	MatchPortRanges = matchPortRanges
 	MatchSubnet     = matchSubnet
 )
 
-// Status exports
-var (
-	ProcessMachines   = processMachines
-	MakeMachineStatus = makeMachineStatus
-)
+func SetNewEnviron(c *Client, newEnviron func() (environs.Environ, error)) {
+	c.newEnviron = newEnviron
+}
 
 type MachineAndContainers machineAndContainers
