@@ -109,9 +109,10 @@ func (c *Client) StatusHistory(request params.StatusHistoryRequests) params.Stat
 	// a oneHistory method for clarity.
 	for _, request := range request.Requests {
 		filter := status.StatusHistoryFilter{
-			Size:  request.Filter.Size,
-			Date:  request.Filter.Date,
-			Delta: request.Filter.Delta,
+			Size:     request.Filter.Size,
+			FromDate: request.Filter.Date,
+			Delta:    request.Filter.Delta,
+			Exclude:  set.NewStrings(request.Filter.Exclude...),
 		}
 		if err := c.checkCanRead(); err != nil {
 			history := params.StatusHistoryResult{
@@ -171,8 +172,14 @@ func (c *Client) FullStatus(args params.StatusParams) (params.FullStatus, error)
 	var noStatus params.FullStatus
 	var context statusContext
 	var err error
+	if context.model, err = c.api.stateAccessor.Model(); err != nil {
+		return noStatus, errors.Annotate(err, "could not fetch model")
+	}
+	if context.status, err = context.model.LoadModelStatus(); err != nil {
+		return noStatus, errors.Annotate(err, "could not load model status values")
+	}
 	if context.applications, context.units, context.latestCharms, err =
-		fetchAllApplicationsAndUnits(c.api.stateAccessor, len(args.Patterns) <= 0); err != nil {
+		fetchAllApplicationsAndUnits(c.api.stateAccessor, context.model, len(args.Patterns) <= 0); err != nil {
 		return noStatus, errors.Annotate(err, "could not fetch applications and units")
 	}
 	if featureflag.Enabled(feature.CrossModelRelations) {
@@ -184,6 +191,11 @@ func (c *Client) FullStatus(args params.StatusParams) (params.FullStatus, error)
 	if context.machines, err = fetchMachines(c.api.stateAccessor, nil); err != nil {
 		return noStatus, errors.Annotate(err, "could not fetch machines")
 	}
+	// These may be empty when machines have not finished deployment.
+	if context.ipAddresses, context.spaces, context.linkLayerDevices, err =
+		fetchNetworkInterfaces(c.api.stateAccessor); err != nil {
+		return noStatus, errors.Annotate(err, "could not fetch IP addresses and link layer devices")
+	}
 	if context.relations, err = fetchRelations(c.api.stateAccessor); err != nil {
 		return noStatus, errors.Annotate(err, "could not fetch relations")
 	}
@@ -290,7 +302,7 @@ func (c *Client) FullStatus(args params.StatusParams) (params.FullStatus, error)
 	}
 	return params.FullStatus{
 		Model:              modelStatus,
-		Machines:           processMachines(context.machines),
+		Machines:           context.processMachines(),
 		Applications:       context.processApplications(),
 		RemoteApplications: context.processRemoteApplications(),
 		Relations:          context.processRelations(),
@@ -312,7 +324,7 @@ func (c *Client) modelStatus() (params.ModelStatusInfo, error) {
 
 	cfg, err := m.Config()
 	if err != nil {
-		return info, errors.Annotate(err, "cannot obtain current model config")
+		return params.ModelStatusInfo{}, errors.Annotate(err, "cannot obtain current model config")
 	}
 
 	latestVersion := m.LatestToolsVersion()
@@ -324,47 +336,46 @@ func (c *Client) modelStatus() (params.ModelStatusInfo, error) {
 		}
 	}
 
-	migStatus, err := c.getMigrationStatus()
+	status, err := m.Status()
 	if err != nil {
-		// It's not worth killing the entire status out if migration
-		// status can't be retrieved.
-		logger.Errorf("error retrieving migration status: %v", err)
-		info.Migration = "error retrieving migration status"
-	} else {
-		info.Migration = migStatus
+		return params.ModelStatusInfo{}, errors.Annotate(err, "cannot obtain model status info")
+	}
+
+	info.SLA = m.SLALevel()
+
+	info.ModelStatus = params.DetailedStatus{
+		Status: status.Status.String(),
+		Info:   status.Message,
+		Since:  status.Since,
+		Data:   status.Data,
+	}
+	ms := m.MeterStatus()
+	if isColorStatus(ms.Code) {
+		info.MeterStatus = params.MeterStatus{Color: strings.ToLower(ms.Code.String()), Message: ms.Info}
 	}
 
 	return info, nil
 }
 
-func (c *Client) getMigrationStatus() (string, error) {
-	mig, err := c.api.stateAccessor.LatestMigration()
-	if err != nil {
-		if errors.IsNotFound(err) {
-			return "", nil
-		}
-		return "", errors.Trace(err)
-	}
-
-	phase, err := mig.Phase()
-	if err != nil {
-		return "", errors.Trace(err)
-	}
-	if phase.IsTerminal() {
-		// There has been a migration attempt but it's no longer
-		// active - don't include this in status.
-		return "", nil
-	}
-
-	return mig.StatusMessage(), nil
-}
-
 type statusContext struct {
+	model  *state.Model
+	status *state.ModelStatus
 	// machines: top-level machine id -> list of machines nested in
 	// this machine.
 	machines map[string][]*state.Machine
+
+	// ipAddresses: machine id -> list of ip.addresses
+	ipAddresses map[string][]*state.Address
+
+	// spaces: machine id -> deviceName -> list of spaceNames
+	spaces map[string]map[string]set.Strings
+
+	// linkLayerDevices: machine id -> list of linkLayerDevices
+	linkLayerDevices map[string][]*state.LinkLayerDevice
+
 	// applications: application name -> application
 	applications map[string]*state.Application
+
 	// remote applications: application name -> application
 	remoteApplications map[string]*state.RemoteApplication
 	relations          map[string][]*state.Relation
@@ -405,10 +416,90 @@ func fetchMachines(st Backend, machineIds set.Strings) (map[string][]*state.Mach
 	return v, nil
 }
 
+// fetchNetworkInterfaces returns maps from machine id to ip.addresses, machine
+// id to a map of interface names from space names, and machine id to
+// linklayerdevices.
+//
+// All are required to determine a machine's network interfaces configuration,
+// so we want all or none.
+func fetchNetworkInterfaces(st Backend) (map[string][]*state.Address, map[string]map[string]set.Strings, map[string][]*state.LinkLayerDevice, error) {
+	ipAddresses := make(map[string][]*state.Address)
+	spaces := make(map[string]map[string]set.Strings)
+	subnets, err := st.AllSubnets()
+	if err != nil {
+		return nil, nil, nil, err
+	}
+	subnetsByCIDR := make(map[string]*state.Subnet)
+	for _, subnet := range subnets {
+		subnetsByCIDR[subnet.CIDR()] = subnet
+	}
+	// For every machine, track what devices have addresses so we can filter linklayerdevices later
+	devicesWithAddresses := make(map[string]set.Strings)
+	ipAddrs, err := st.AllIPAddresses()
+	if err != nil {
+		return nil, nil, nil, err
+	}
+	for _, ipAddr := range ipAddrs {
+		if ipAddr.LoopbackConfigMethod() {
+			continue
+		}
+		machineID := ipAddr.MachineID()
+		ipAddresses[machineID] = append(ipAddresses[machineID], ipAddr)
+		if subnet, ok := subnetsByCIDR[ipAddr.SubnetCIDR()]; ok {
+			if spaceName := subnet.SpaceName(); spaceName != "" {
+				devices, ok := spaces[machineID]
+				if !ok {
+					devices = make(map[string]set.Strings)
+					spaces[machineID] = devices
+				}
+				deviceName := ipAddr.DeviceName()
+				spacesSet, ok := devices[deviceName]
+				if !ok {
+					spacesSet = make(set.Strings)
+					devices[deviceName] = spacesSet
+				}
+				spacesSet.Add(spaceName)
+			}
+		}
+		deviceSet, ok := devicesWithAddresses[machineID]
+		if ok {
+			deviceSet.Add(ipAddr.DeviceName())
+		} else {
+			devicesWithAddresses[machineID] = set.NewStrings(ipAddr.DeviceName())
+		}
+	}
+
+	linkLayerDevices := make(map[string][]*state.LinkLayerDevice)
+	llDevs, err := st.AllLinkLayerDevices()
+	if err != nil {
+		return nil, nil, nil, err
+	}
+	for _, llDev := range llDevs {
+		if llDev.IsLoopbackDevice() {
+			continue
+		}
+		machineID := llDev.MachineID()
+		machineDevs, ok := devicesWithAddresses[machineID]
+		if !ok {
+			// This machine ID doesn't seem to have any devices with IP Addresses
+			continue
+		}
+		if !machineDevs.Contains(llDev.Name()) {
+			// this device did not have any IP Addresses
+			continue
+		}
+		// This device had an IP Address, so include it in the list of devices for this machine
+		linkLayerDevices[machineID] = append(linkLayerDevices[machineID], llDev)
+	}
+
+	return ipAddresses, spaces, linkLayerDevices, nil
+}
+
 // fetchAllApplicationsAndUnits returns a map from application name to application,
 // a map from application name to unit name to unit, and a map from base charm URL to latest URL.
 func fetchAllApplicationsAndUnits(
 	st Backend,
+	model *state.Model,
 	matchAny bool,
 ) (map[string]*state.Application, map[string]map[string]*state.Unit, map[charm.URL]*state.Charm, error) {
 
@@ -419,21 +510,30 @@ func fetchAllApplicationsAndUnits(
 	if err != nil {
 		return nil, nil, nil, err
 	}
-	for _, s := range applications {
-		units, err := s.AllUnits()
-		if err != nil {
-			return nil, nil, nil, err
+	units, err := model.AllUnits()
+	if err != nil {
+		return nil, nil, nil, err
+	}
+	allUnitsByApp := make(map[string]map[string]*state.Unit)
+	for _, unit := range units {
+		appName := unit.ApplicationName()
+
+		if inner, found := allUnitsByApp[appName]; found {
+			inner[unit.Name()] = unit
+		} else {
+			allUnitsByApp[appName] = map[string]*state.Unit{
+				unit.Name(): unit,
+			}
 		}
-		appUnitMap := make(map[string]*state.Unit)
-		for _, u := range units {
-			appUnitMap[u.Name()] = u
-		}
-		if matchAny || len(appUnitMap) > 0 {
-			unitMap[s.Name()] = appUnitMap
-			appMap[s.Name()] = s
+	}
+	for _, app := range applications {
+		appUnits := allUnitsByApp[app.Name()]
+		if matchAny || len(appUnits) > 0 {
+			unitMap[app.Name()] = appUnits
+			appMap[app.Name()] = app
 			// Record the base URL for the application's charm so that
 			// the latest store revision can be looked up.
-			charmURL, _ := s.CharmURL()
+			charmURL, _ := app.CharmURL()
 			if charmURL.Schema == "cs" {
 				latestCharms[*charmURL.WithRevision(-1)] = nil
 			}
@@ -460,8 +560,11 @@ func fetchRemoteApplications(st Backend) (map[string]*state.RemoteApplication, e
 	if err != nil {
 		return nil, err
 	}
-	for _, s := range applications {
-		appMap[s.Name()] = s
+	for _, a := range applications {
+		if _, ok := a.URL(); !ok {
+			continue
+		}
+		appMap[a.Name()] = a
 	}
 	return appMap, nil
 }
@@ -497,10 +600,10 @@ func (m machineAndContainers) Containers(id string) []*state.Machine {
 	return m[id][1:]
 }
 
-func processMachines(idToMachines map[string][]*state.Machine) map[string]params.MachineStatus {
+func (c *statusContext) processMachines() map[string]params.MachineStatus {
 	machinesMap := make(map[string]params.MachineStatus)
 	cache := make(map[string]params.MachineStatus)
-	for id, machines := range idToMachines {
+	for id, machines := range c.machines {
 
 		if len(machines) <= 0 {
 			continue
@@ -508,17 +611,18 @@ func processMachines(idToMachines map[string][]*state.Machine) map[string]params
 
 		// Element 0 is assumed to be the top-level machine.
 		tlMachine := machines[0]
-		hostStatus := makeMachineStatus(tlMachine)
+		hostStatus := c.makeMachineStatus(tlMachine)
 		machinesMap[id] = hostStatus
 		cache[id] = hostStatus
 
 		for _, machine := range machines[1:] {
 			parent, ok := cache[state.ParentId(machine.Id())]
 			if !ok {
-				panic("We've broken an assumpution.")
+				logger.Errorf("programmer error, please file a bug, reference this whole log line: %q, %q", id, machine.Id())
+				continue
 			}
 
-			status := makeMachineStatus(machine)
+			status := c.makeMachineStatus(machine)
 			parent.Containers[machine.Id()] = status
 			cache[machine.Id()] = status
 		}
@@ -526,18 +630,24 @@ func processMachines(idToMachines map[string][]*state.Machine) map[string]params
 	return machinesMap
 }
 
-func makeMachineStatus(machine *state.Machine) (status params.MachineStatus) {
+func (c *statusContext) makeMachineStatus(machine *state.Machine) (status params.MachineStatus) {
+	machineID := machine.Id()
+	ipAddresses := c.ipAddresses[machineID]
+	spaces := c.spaces[machineID]
+	linkLayerDevices := c.linkLayerDevices[machineID]
+
 	var err error
 	status.Id = machine.Id()
-	agentStatus := processMachine(machine)
+	agentStatus := c.processMachine(machine)
 	status.AgentStatus = agentStatus
 
 	status.Series = machine.Series()
 	status.Jobs = paramsJobsFromJobs(machine.Jobs())
 	status.WantsVote = machine.WantsVote()
 	status.HasVote = machine.HasVote()
-	sInfo, err := machine.InstanceStatus()
+	sInfo, err := c.status.MachineInstance(machineID)
 	populateStatusFromStatusInfoAndErr(&status.InstanceStatus, sInfo, err)
+	// TODO: fetch all instance data for machines in one go.
 	instid, err := machine.InstanceId()
 	if err == nil {
 		status.InstanceId = instid
@@ -549,7 +659,6 @@ func makeMachineStatus(machine *state.Machine) (status params.MachineStatus) {
 			logger.Debugf("error fetching public address: %q", err)
 		}
 		status.DNSName = addr.Value
-
 		mAddrs := machine.Addresses()
 		if len(mAddrs) == 0 {
 			logger.Debugf("no IP addresses fetched for machine %q", instid)
@@ -565,6 +674,48 @@ func makeMachineStatus(machine *state.Machine) (status params.MachineStatus) {
 			}
 			status.IPAddresses = append(status.IPAddresses, mAddr.Value)
 		}
+		status.NetworkInterfaces = make(map[string]params.NetworkInterface, len(linkLayerDevices))
+		for _, llDev := range linkLayerDevices {
+			device := llDev.Name()
+			ips := []string{}
+			gw := []string{}
+			ns := []string{}
+			sp := make(set.Strings)
+			for _, ipAddress := range ipAddresses {
+				if ipAddress.DeviceName() != device {
+					continue
+				}
+				ips = append(ips, ipAddress.Value())
+				// We don't expect to find more than one
+				// ipAddress on a device with a list of
+				// nameservers, but append in any case.
+				if len(ipAddress.DNSServers()) > 0 {
+					ns = append(ns, ipAddress.DNSServers()...)
+				}
+				// There should only be one gateway per device
+				// (per machine, in fact, as we don't store
+				// metrics). If we find more than one we should
+				// show them all.
+				if ipAddress.GatewayAddress() != "" {
+					gw = append(gw, ipAddress.GatewayAddress())
+				}
+				// There should only be one space per address,
+				// but it's technically possible to have more
+				// than one address on an interface. If we find
+				// that happens, we need to show all spaces, to
+				// be safe.
+				sp = spaces[device]
+			}
+			status.NetworkInterfaces[device] = params.NetworkInterface{
+				IPAddresses:    ips,
+				MACAddress:     llDev.MACAddress(),
+				Gateway:        strings.Join(gw, " "),
+				DNSNameservers: ns,
+				Space:          strings.Join(sp.Values(), " "),
+				IsUp:           llDev.IsUp(),
+			}
+		}
+		logger.Debugf("NetworkInterfaces: %+v", status.NetworkInterfaces)
 	} else {
 		if errors.IsNotProvisioned(err) {
 			status.InstanceId = "pending"
@@ -572,6 +723,16 @@ func makeMachineStatus(machine *state.Machine) (status params.MachineStatus) {
 			status.InstanceId = "error"
 		}
 	}
+	// TODO: preload all constraints.
+	constraints, err := machine.Constraints()
+	if err != nil {
+		if !errors.IsNotFound(err) {
+			status.Constraints = "error"
+		}
+	} else {
+		status.Constraints = constraints.String()
+	}
+	// TODO: preload all hardware characteristics.
 	hc, err := machine.HardwareCharacteristics()
 	if err != nil {
 		if !errors.IsNotFound(err) {
@@ -687,7 +848,11 @@ func (context *statusContext) processApplication(application *state.Application)
 	if application.IsPrincipal() {
 		processedStatus.Units = context.processUnits(units, applicationCharm.URL().String())
 	}
-	applicationStatus, err := application.Status()
+	var unitNames []string
+	for _, unit := range units {
+		unitNames = append(unitNames, unit.Name())
+	}
+	applicationStatus, err := context.status.Application(application.Name(), unitNames)
 	if err != nil {
 		processedStatus.Err = common.ServerError(err)
 		return processedStatus
@@ -705,20 +870,12 @@ func (context *statusContext) processApplication(application *state.Application)
 
 	versions := make([]status.StatusInfo, 0, len(units))
 	for _, unit := range units {
-		statuses, err := unit.WorkloadVersionHistory().StatusHistory(
-			status.StatusHistoryFilter{Size: 1},
-		)
+		workloadVersion, err := context.status.FullUnitWorkloadVersion(unit.Name())
 		if err != nil {
 			processedStatus.Err = common.ServerError(err)
 			return processedStatus
 		}
-		// Even though we fully expect there to be historical values there,
-		// even the first should be the empty string, the status history
-		// collection is not added to in a transactional manner, so it may be
-		// not there even though we'd really like it to be. Such is mongo.
-		if len(statuses) > 0 {
-			versions = append(versions, statuses[0])
-		}
+		versions = append(versions, workloadVersion)
 	}
 	if len(versions) > 0 {
 		sort.Sort(bySinceDescending(versions))
@@ -737,7 +894,7 @@ func (context *statusContext) processRemoteApplications() map[string]params.Remo
 }
 
 func (context *statusContext) processRemoteApplication(application *state.RemoteApplication) (status params.RemoteApplicationStatus) {
-	status.ApplicationURL = application.URL()
+	status.ApplicationURL, _ = application.URL()
 	status.ApplicationName = application.Name()
 	eps, err := application.Endpoints()
 	if err != nil {
@@ -750,6 +907,7 @@ func (context *statusContext) processRemoteApplication(application *state.Remote
 			Name:      ep.Name,
 			Interface: ep.Interface,
 			Role:      ep.Role,
+			Scope:     ep.Scope,
 		}
 	}
 	status.Life = processLife(application)
@@ -814,14 +972,14 @@ func (context *statusContext) processUnit(unit *state.Unit, applicationCharm str
 	if applicationCharm != "" && curl != nil && curl.String() != applicationCharm {
 		result.Charm = curl.String()
 	}
-	workloadVersion, err := unit.WorkloadVersion()
+	workloadVersion, err := context.status.UnitWorkloadVersion(unit.Name())
 	if err == nil {
 		result.WorkloadVersion = workloadVersion
 	} else {
 		logger.Debugf("error fetching workload version: %v", err)
 	}
 
-	processUnitAndAgentStatus(unit, &result)
+	result.AgentStatus, result.WorkloadStatus = context.processUnitAndAgentStatus(unit)
 
 	if subUnits := unit.SubordinateNames(); len(subUnits) > 0 {
 		result.Subordinates = make(map[string]params.UnitStatus)
@@ -900,13 +1058,42 @@ type lifer interface {
 	Life() state.Life
 }
 
+// contextUnit overloads the AgentStatus and Status calls to use the cached
+// status values, and delegates everything else to the Unit.
+// TODO: cache presence as well.
+type contextUnit struct {
+	*state.Unit
+	context *statusContext
+}
+
+// AgentStatus implements UnitStatusGetter.
+func (c *contextUnit) AgentStatus() (status.StatusInfo, error) {
+	return c.context.status.UnitAgent(c.Name())
+}
+
+// Status implements UnitStatusGetter.
+func (c *contextUnit) Status() (status.StatusInfo, error) {
+	return c.context.status.UnitWorkload(c.Name())
+}
+
 // processUnitAndAgentStatus retrieves status information for both unit and unitAgents.
-func processUnitAndAgentStatus(unit *state.Unit, unitStatus *params.UnitStatus) {
-	unitStatus.AgentStatus, unitStatus.WorkloadStatus = processUnit(unit)
+func (c *statusContext) processUnitAndAgentStatus(unit *state.Unit) (agentStatus, workloadStatus params.DetailedStatus) {
+	wrapped := &contextUnit{unit, c}
+	agent, workload := common.UnitStatus(wrapped)
+	populateStatusFromStatusInfoAndErr(&agentStatus, agent.Status, agent.Err)
+	populateStatusFromStatusInfoAndErr(&workloadStatus, workload.Status, workload.Err)
+
+	agentStatus.Life = processLife(unit)
+
+	if t, err := unit.AgentTools(); err == nil {
+		agentStatus.Version = t.Version.Number.String()
+	}
+	return
 }
 
 // populateStatusFromStatusInfoAndErr creates AgentStatus from the typical output
 // of a status getter.
+// TODO: make this a function that just returns a type.
 func populateStatusFromStatusInfoAndErr(agent *params.DetailedStatus, statusInfo status.StatusInfo, err error) {
 	agent.Err = err
 	agent.Status = statusInfo.Status.String()
@@ -915,10 +1102,24 @@ func populateStatusFromStatusInfoAndErr(agent *params.DetailedStatus, statusInfo
 	agent.Since = statusInfo.Since
 }
 
+// contextMachine overloads the Status call to use the cached status values,
+// and delegates everything else to the Machine.
+// TODO: cache presence as well.
+type contextMachine struct {
+	*state.Machine
+	context *statusContext
+}
+
+// Return the agent status for the machine.
+func (c *contextMachine) Status() (status.StatusInfo, error) {
+	return c.context.status.MachineAgent(c.Id())
+}
+
 // processMachine retrieves version and status information for the given machine.
 // It also returns deprecated legacy status information.
-func processMachine(machine *state.Machine) (out params.DetailedStatus) {
-	statusInfo, err := common.MachineStatus(machine)
+func (c *statusContext) processMachine(machine *state.Machine) (out params.DetailedStatus) {
+	wrapped := &contextMachine{machine, c}
+	statusInfo, err := common.MachineStatus(wrapped)
 	populateStatusFromStatusInfoAndErr(&out, statusInfo, err)
 
 	out.Life = processLife(machine)
@@ -929,20 +1130,6 @@ func processMachine(machine *state.Machine) (out params.DetailedStatus) {
 	return
 }
 
-// processUnit retrieves version and status information for the given unit.
-func processUnit(unit *state.Unit) (agentStatus, workloadStatus params.DetailedStatus) {
-	agent, workload := common.UnitStatus(unit)
-	populateStatusFromStatusInfoAndErr(&agentStatus, agent.Status, agent.Err)
-	populateStatusFromStatusInfoAndErr(&workloadStatus, workload.Status, workload.Err)
-
-	agentStatus.Life = processLife(unit)
-
-	if t, err := unit.AgentTools(); err == nil {
-		agentStatus.Version = t.Version.Number.String()
-	}
-	return
-}
-
 // filterStatusData limits what agent StatusData data is passed over
 // the API. This prevents unintended leakage of internal-only data.
 func filterStatusData(status map[string]interface{}) map[string]interface{} {
