@@ -10,6 +10,7 @@ import (
 	"github.com/juju/utils/set"
 	"gopkg.in/juju/names.v2"
 
+	"github.com/juju/juju/apiserver/common"
 	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/network"
@@ -26,6 +27,7 @@ type BackingSubnet interface {
 	CIDR() string
 	VLANTag() int
 	ProviderId() network.Id
+	ProviderNetworkId() network.Id
 	AvailabilityZones() []string
 	Status() string
 	SpaceName() string
@@ -41,8 +43,6 @@ type BackingSubnet interface {
 // * subnetDoc.AvailabilityZone becomes subnetDoc.AvailabilityZones,
 //   adding an upgrade step to migrate existing non empty zones on
 //   subnet docs. Also change state.Subnet.AvailabilityZone to
-// * add subnetDoc.SpaceName - no upgrade step needed, as it will only
-//   be used for new space-aware subnets.
 // * Subnets need a reference count to calculate Status.
 // * ensure EC2 and MAAS providers accept empty IDs as Subnets() args
 //   and return all subnets, including the AvailabilityZones (for EC2;
@@ -51,6 +51,11 @@ type BackingSubnetInfo struct {
 	// ProviderId is a provider-specific network id. This may be empty.
 	ProviderId network.Id
 
+	// ProviderNetworkId is the id of the network containing this
+	// subnet from the provider's perspective. It can be empty if the
+	// provider doesn't support distinct networks.
+	ProviderNetworkId network.Id
+
 	// CIDR of the network, in 123.45.67.89/24 format.
 	CIDR string
 
@@ -86,17 +91,9 @@ type BackingSpace interface {
 
 	// ProviderId returns the network ID of the provider
 	ProviderId() network.Id
-
-	// Zones returns a list of availability zone(s) that this
-	// space is in. It can be empty if the provider does not support
-	// availability zones.
-	Zones() []string
-
-	// Life returns the lifecycle state of the space
-	Life() params.Life
 }
 
-// Backing defines the methods needed by the API facade to store and
+// NetworkBacking defines the methods needed by the API facade to store and
 // retrieve information from the underlying persistency layer (state
 // DB).
 type NetworkBacking interface {
@@ -124,6 +121,9 @@ type NetworkBacking interface {
 
 	// ModelTag returns the tag of the model this state is associated to.
 	ModelTag() names.ModelTag
+
+	// ReloadSpaces loads spaces from backing environ
+	ReloadSpaces(environ environs.Environ) error
 }
 
 func BackingSubnetToParamsSubnet(subnet BackingSubnet) params.Subnet {
@@ -157,6 +157,14 @@ func NetworkConfigFromInterfaceInfo(interfaceInfos []network.InterfaceInfo) []pa
 		for _, nameserver := range v.DNSServers {
 			dnsServers = append(dnsServers, nameserver.Value)
 		}
+		routes := make([]params.NetworkRoute, len(v.Routes))
+		for j, route := range v.Routes {
+			routes[j] = params.NetworkRoute{
+				DestinationCIDR: route.DestinationCIDR,
+				GatewayIP:       route.GatewayIP,
+				Metric:          route.Metric,
+			}
+		}
 		result[i] = params.NetworkConfig{
 			DeviceIndex:         v.DeviceIndex,
 			MACAddress:          v.MACAddress,
@@ -178,6 +186,7 @@ func NetworkConfigFromInterfaceInfo(interfaceInfos []network.InterfaceInfo) []pa
 			DNSServers:          dnsServers,
 			DNSSearchDomains:    v.DNSSearchDomains,
 			GatewayAddress:      v.GatewayAddress.Value,
+			Routes:              routes,
 		}
 	}
 	return result
@@ -307,199 +316,6 @@ type NetworkConfigSource interface {
 	InterfaceAddresses(name string) ([]net.Addr, error)
 }
 
-type netPackageConfigSource struct{}
-
-// SysClassNetPath implements NetworkConfigSource.
-func (n *netPackageConfigSource) SysClassNetPath() string {
-	return network.SysClassNetPath
-}
-
-// Interfaces implements NetworkConfigSource.
-func (n *netPackageConfigSource) Interfaces() ([]net.Interface, error) {
-	return net.Interfaces()
-}
-
-// InterfaceAddresses implements NetworkConfigSource.
-func (n *netPackageConfigSource) InterfaceAddresses(name string) ([]net.Addr, error) {
-	iface, err := net.InterfaceByName(name)
-	if err != nil {
-		return nil, errors.Trace(err)
-	}
-	return iface.Addrs()
-}
-
-// DefaultNetworkConfigSource returns a NetworkConfigSource backed by the net
-// package, to be used with GetObservedNetworkConfig().
-func DefaultNetworkConfigSource() NetworkConfigSource {
-	return &netPackageConfigSource{}
-}
-
-// GetObservedNetworkConfig uses the given source to find all available network
-// interfaces and their assigned addresses, and returns the result as
-// []params.NetworkConfig. In addition to what the source returns, a few
-// additional transformations are done:
-//
-// * On any OS, the state (UP/DOWN) of each interface and the DeviceIndex field,
-//   will be correctly populated. Loopback interfaces are also properly detected
-//   and will have InterfaceType set LoopbackInterface.
-// * On Linux only, the InterfaceType field will be reliably detected for a few
-//   types: BondInterface, BridgeInterface, VLAN_8021QInterface.
-// * Also on Linux, for interfaces that are discovered to be ports on a bridge,
-//   the ParentInterfaceName will be populated with the name of the bridge.
-// * ConfigType fields will be set to ConfigManual when no address is detected,
-//   or ConfigStatic when it is.
-// * TODO: any IPv6 addresses found will be ignored and treated as empty ATM.
-//
-// Result entries will be grouped by InterfaceName, in the same order they are
-// returned by the given source.
-func GetObservedNetworkConfig(source NetworkConfigSource) ([]params.NetworkConfig, error) {
-	logger.Tracef("discovering observed machine network config...")
-
-	interfaces, err := source.Interfaces()
-	if err != nil {
-		return nil, errors.Annotate(err, "cannot get network interfaces")
-	}
-
-	var namesOrder []string
-	nameToConfigs := make(map[string][]params.NetworkConfig)
-	sysClassNetPath := source.SysClassNetPath()
-	for _, nic := range interfaces {
-		nicType := network.ParseInterfaceType(sysClassNetPath, nic.Name)
-		nicConfig := interfaceToNetworkConfig(nic, nicType)
-
-		if nicType == network.BridgeInterface {
-			updateParentForBridgePorts(nic.Name, sysClassNetPath, nameToConfigs)
-		}
-
-		seenSoFar := false
-		if existing, ok := nameToConfigs[nic.Name]; ok {
-			nicConfig.ParentInterfaceName = existing[0].ParentInterfaceName
-			// If only ParentInterfaceName was set in a previous iteration (e.g.
-			// if the bridge appeared before the port), treat the interface as
-			// not yet seen.
-			seenSoFar = existing[0].InterfaceName != ""
-		}
-
-		if !seenSoFar {
-			nameToConfigs[nic.Name] = []params.NetworkConfig(nil)
-			namesOrder = append(namesOrder, nic.Name)
-		}
-
-		addrs, err := source.InterfaceAddresses(nic.Name)
-		if err != nil {
-			return nil, errors.Annotatef(err, "cannot get interface %q addresses", nic.Name)
-		}
-
-		if len(addrs) == 0 {
-			logger.Infof("no addresses observed on interface %q", nic.Name)
-			nameToConfigs[nic.Name] = append(nameToConfigs[nic.Name], nicConfig)
-			continue
-		}
-
-		for _, addr := range addrs {
-			addressConfig, err := interfaceAddressToNetworkConfig(nic.Name, nicConfig.ConfigType, addr)
-			if err != nil {
-				return nil, errors.Trace(err)
-			}
-
-			// Need to copy nicConfig so only the fields relevant for the
-			// current address are updated.
-			nicConfigCopy := nicConfig
-			nicConfigCopy.Address = addressConfig.Address
-			nicConfigCopy.CIDR = addressConfig.CIDR
-			nicConfigCopy.ConfigType = addressConfig.ConfigType
-			nameToConfigs[nic.Name] = append(nameToConfigs[nic.Name], nicConfigCopy)
-		}
-	}
-
-	// Return all interfaces configs in input order.
-	var observedConfig []params.NetworkConfig
-	for _, name := range namesOrder {
-		observedConfig = append(observedConfig, nameToConfigs[name]...)
-	}
-	logger.Tracef("observed network config: %+v", observedConfig)
-	return observedConfig, nil
-}
-
-func interfaceToNetworkConfig(nic net.Interface, nicType network.InterfaceType) params.NetworkConfig {
-	configType := network.ConfigManual // assume manual initially, until we parse the address.
-	isUp := nic.Flags&net.FlagUp > 0
-	isLoopback := nic.Flags&net.FlagLoopback > 0
-	isUnknown := nicType == network.UnknownInterface
-
-	switch {
-	case isUnknown && isLoopback:
-		nicType = network.LoopbackInterface
-		configType = network.ConfigLoopback
-	case isUnknown:
-		nicType = network.EthernetInterface
-	}
-
-	return params.NetworkConfig{
-		DeviceIndex:   nic.Index,
-		MACAddress:    nic.HardwareAddr.String(),
-		ConfigType:    string(configType),
-		MTU:           nic.MTU,
-		InterfaceName: nic.Name,
-		InterfaceType: string(nicType),
-		NoAutoStart:   !isUp,
-		Disabled:      !isUp,
-	}
-}
-
-func updateParentForBridgePorts(bridgeName, sysClassNetPath string, nameToConfigs map[string][]params.NetworkConfig) {
-	ports := network.GetBridgePorts(sysClassNetPath, bridgeName)
-	for _, portName := range ports {
-		portConfigs, ok := nameToConfigs[portName]
-		if ok {
-			portConfigs[0].ParentInterfaceName = bridgeName
-		} else {
-			portConfigs = []params.NetworkConfig{{ParentInterfaceName: bridgeName}}
-		}
-		nameToConfigs[portName] = portConfigs
-	}
-}
-
-func interfaceAddressToNetworkConfig(interfaceName, configType string, address net.Addr) (params.NetworkConfig, error) {
-	config := params.NetworkConfig{
-		ConfigType: configType,
-	}
-
-	cidrAddress := address.String()
-	if cidrAddress == "" {
-		return config, nil
-	}
-
-	ip, ipNet, err := net.ParseCIDR(cidrAddress)
-	if err != nil {
-		logger.Infof("cannot parse %q on interface %q as CIDR, trying as IP address: %v", cidrAddress, interfaceName, err)
-		if ip = net.ParseIP(cidrAddress); ip == nil {
-			return config, errors.Errorf("cannot parse IP address %q on interface %q", cidrAddress, interfaceName)
-		} else {
-			ipNet = &net.IPNet{IP: ip}
-		}
-	}
-	if ip.To4() == nil {
-		logger.Debugf("skipping observed IPv6 address %q on %q: not fully supported yet", ip, interfaceName)
-		// TODO(dimitern): Treat IPv6 addresses as empty until we can handle
-		// them reliably.
-		return config, nil
-	}
-
-	if ipNet.Mask != nil {
-		config.CIDR = ipNet.String()
-	}
-	config.Address = ip.String()
-	if configType != string(network.ConfigLoopback) {
-		config.ConfigType = string(network.ConfigStatic)
-	}
-
-	// TODO(dimitern): Add DNS servers, search domains, and gateway
-	// later.
-
-	return config, nil
-}
-
 // MergeProviderAndObservedNetworkConfigs returns the effective network configs,
 // using observedConfigs as a base and selectively updating it using the
 // matching providerConfigs for each interface.
@@ -621,3 +437,31 @@ func mergeObservedAndProviderAddressConfig(observedConfig, providerConfig params
 
 	return finalConfig
 }
+
+func networkToParamsNetworkInfo(info network.NetworkInfo) params.NetworkInfo {
+	addresses := make([]params.InterfaceAddress, len(info.Addresses))
+	for i, addr := range info.Addresses {
+		addresses[i] = params.InterfaceAddress{
+			Address: addr.Address,
+			CIDR:    addr.CIDR,
+		}
+	}
+	return params.NetworkInfo{
+		MACAddress:    info.MACAddress,
+		InterfaceName: info.InterfaceName,
+		Addresses:     addresses,
+	}
+}
+
+func MachineNetworkInfoResultToNetworkInfoResult(inResult state.MachineNetworkInfoResult) params.NetworkInfoResult {
+	if inResult.Error != nil {
+		return params.NetworkInfoResult{Error: common.ServerError(*inResult.Error)}
+	}
+	infos := make([]params.NetworkInfo, len(inResult.NetworkInfos))
+	for i, info := range inResult.NetworkInfos {
+		infos[i] = networkToParamsNetworkInfo(info)
+	}
+	return params.NetworkInfoResult{
+		Info: infos,
+	}
+}
