@@ -31,10 +31,6 @@ import (
 
 var logger = loggo.GetLogger("juju.apiserver.controller")
 
-func init() {
-	common.RegisterStandardFacade("Controller", 3, NewControllerAPI)
-}
-
 // Controller defines the methods on the controller API end point.
 type Controller interface {
 	AllModels() (params.UserModelList, error)
@@ -59,6 +55,7 @@ type ControllerAPI struct {
 	cloudspec.CloudSpecAPI
 
 	state      *state.State
+	statePool  *state.StatePool
 	authorizer facade.Authorizer
 	apiUser    names.UserTag
 	resources  facade.Resources
@@ -68,11 +65,8 @@ var _ Controller = (*ControllerAPI)(nil)
 
 // NewControllerAPI creates a new api server endpoint for managing
 // environments.
-func NewControllerAPI(
-	st *state.State,
-	resources facade.Resources,
-	authorizer facade.Authorizer,
-) (*ControllerAPI, error) {
+func NewControllerAPI(ctx facade.Context) (*ControllerAPI, error) {
+	authorizer := ctx.Auth()
 	if !authorizer.AuthClient() {
 		return nil, errors.Trace(common.ErrPerm)
 	}
@@ -81,15 +75,22 @@ func NewControllerAPI(
 	// we just do the type assertion to the UserTag.
 	apiUser, _ := authorizer.GetAuthTag().(names.UserTag)
 
+	st := ctx.State()
 	environConfigGetter := stateenvirons.EnvironConfigGetter{st}
 	return &ControllerAPI{
 		ControllerConfigAPI: common.NewControllerConfig(st),
-		ModelStatusAPI:      common.NewModelStatusAPI(common.NewModelManagerBackend(st), authorizer, apiUser),
-		CloudSpecAPI:        cloudspec.NewCloudSpec(environConfigGetter.CloudSpec, common.AuthFuncForTag(st.ModelTag())),
-		state:               st,
-		authorizer:          authorizer,
-		apiUser:             apiUser,
-		resources:           resources,
+		ModelStatusAPI: common.NewModelStatusAPI(
+			common.NewModelManagerBackend(st),
+			common.NewBackendPool(ctx.StatePool()),
+			authorizer,
+			apiUser,
+		),
+		CloudSpecAPI: cloudspec.NewCloudSpec(environConfigGetter.CloudSpec, common.AuthFuncForTag(st.ModelTag())),
+		state:        st,
+		statePool:    ctx.StatePool(),
+		authorizer:   authorizer,
+		apiUser:      apiUser,
+		resources:    ctx.Resources(),
 	}, nil
 }
 
@@ -105,51 +106,52 @@ func (s *ControllerAPI) checkHasAdmin() error {
 }
 
 // AllModels allows controller administrators to get the list of all the
-// environments in the controller.
+// models in the controller.
 func (s *ControllerAPI) AllModels() (params.UserModelList, error) {
 	result := params.UserModelList{}
 	if err := s.checkHasAdmin(); err != nil {
 		return result, errors.Trace(err)
 	}
 
-	// Get all the environments that the authenticated user can see, and
-	// supplement that with the other environments that exist that the user
-	// cannot see. The reason we do this is to get the LastConnection time for
-	// the environments that the user is able to see, so we have consistent
-	// output when listing with or without --all when an admin user.
-	environments, err := s.state.ModelsForUser(s.apiUser)
+	// Get all the models that the authenticated user can see, and
+	// supplement that with the other models that exist that the user
+	// cannot see. The reason we do this is to get the LastConnection
+	// time for the models that the user is able to see, so we have
+	// consistent output when listing with or without --all when an
+	// admin user.
+	models, err := s.state.ModelsForUser(s.apiUser)
 	if err != nil {
 		return result, errors.Trace(err)
 	}
-	visibleEnvironments := set.NewStrings()
-	for _, env := range environments {
-		lastConn, err := env.LastConnection()
+	visibleModels := set.NewStrings()
+	for _, model := range models {
+		lastConn, err := model.LastConnection()
 		if err != nil && !state.IsNeverConnectedError(err) {
 			return result, errors.Trace(err)
 		}
-		visibleEnvironments.Add(env.UUID())
+		visibleModels.Add(model.UUID())
 		result.UserModels = append(result.UserModels, params.UserModel{
 			Model: params.Model{
-				Name:     env.Name(),
-				UUID:     env.UUID(),
-				OwnerTag: env.Owner().String(),
+				Name:     model.Name(),
+				UUID:     model.UUID(),
+				OwnerTag: model.Owner().String(),
 			},
 			LastConnection: &lastConn,
 		})
 	}
 
-	allEnvs, err := s.state.AllModels()
+	allModels, err := s.state.AllModels()
 	if err != nil {
 		return result, errors.Trace(err)
 	}
 
-	for _, env := range allEnvs {
-		if !visibleEnvironments.Contains(env.UUID()) {
+	for _, model := range allModels {
+		if !visibleModels.Contains(model.UUID()) {
 			result.UserModels = append(result.UserModels, params.UserModel{
 				Model: params.Model{
-					Name:     env.Name(),
-					UUID:     env.UUID(),
-					OwnerTag: env.Owner().String(),
+					Name:     model.Name(),
+					UUID:     model.UUID(),
+					OwnerTag: model.Owner().String(),
 				},
 				// No LastConnection as this user hasn't.
 			})
@@ -298,7 +300,7 @@ func (c *ControllerAPI) WatchAllModels() (params.AllWatcherId, error) {
 	if err := c.checkHasAdmin(); err != nil {
 		return params.AllWatcherId{}, errors.Trace(err)
 	}
-	w := c.state.WatchAllModels()
+	w := c.state.WatchAllModels(c.statePool)
 	return params.AllWatcherId{
 		AllWatcherId: c.resources.Register(w),
 	}, nil
@@ -352,13 +354,13 @@ func (c *ControllerAPI) GetControllerAccess(req params.Entities) (params.UserAcc
 			results.Results[i].Error = common.ServerError(common.ErrPerm)
 			continue
 		}
-		accessInfo, err := c.state.UserAccess(userTag, c.state.ControllerTag())
+		access, err := c.state.UserPermission(userTag, c.state.ControllerTag())
 		if err != nil {
 			results.Results[i].Error = common.ServerError(err)
 			continue
 		}
 		results.Results[i].Result = &params.UserAccess{
-			Access:  string(accessInfo.Access),
+			Access:  string(access),
 			UserTag: userTag.String()}
 	}
 	return results, nil
@@ -432,17 +434,14 @@ func (c *ControllerAPI) initiateOneMigration(spec params.MigrationSpec) (string,
 	}
 
 	// Check if the migration is likely to succeed.
-	if !(spec.ExternalControl && spec.SkipInitialPrechecks) {
-		if err := runMigrationPrechecks(hostedState, targetInfo); err != nil {
-			return "", errors.Trace(err)
-		}
+	if err := runMigrationPrechecks(hostedState, &targetInfo); err != nil {
+		return "", errors.Trace(err)
 	}
 
 	// Trigger the migration.
 	mig, err := hostedState.CreateMigration(state.MigrationSpec{
-		InitiatedBy:     c.apiUser,
-		TargetInfo:      targetInfo,
-		ExternalControl: spec.ExternalControl,
+		InitiatedBy: c.apiUser,
+		TargetInfo:  targetInfo,
 	})
 	if err != nil {
 		return "", errors.Trace(err)
@@ -488,9 +487,16 @@ func (c *ControllerAPI) ModifyControllerAccess(args params.ModifyControllerAcces
 	return result, nil
 }
 
-var runMigrationPrechecks = func(st *state.State, targetInfo coremigration.TargetInfo) error {
+// runMigrationPrechecks runs prechecks on the migration and updates
+// information in targetInfo as needed based on information
+// retrieved from the target controller.
+var runMigrationPrechecks = func(st *state.State, targetInfo *coremigration.TargetInfo) error {
 	// Check model and source controller.
-	if err := migration.SourcePrecheck(migration.PrecheckShim(st)); err != nil {
+	backend, err := migration.PrecheckShim(st)
+	if err != nil {
+		return errors.Annotate(err, "creating backend")
+	}
+	if err := migration.SourcePrecheck(backend); err != nil {
 		return errors.Annotate(err, "source prechecks failed")
 	}
 
@@ -504,7 +510,19 @@ var runMigrationPrechecks = func(st *state.State, targetInfo coremigration.Targe
 	if err != nil {
 		return errors.Trace(err)
 	}
-	err = migrationtarget.NewClient(conn).Prechecks(modelInfo)
+	client := migrationtarget.NewClient(conn)
+	if targetInfo.CACert == "" {
+		targetInfo.CACert, err = client.CACert()
+		if err != nil {
+			if !params.IsCodeNotImplemented(err) {
+				return errors.Annotatef(err, "cannot retrieve CA certificate")
+			}
+			// If the call's not implemented, it indicates an earlier version
+			// of the controller, which we can't migrate to.
+			return errors.New("controller API version is too old")
+		}
+	}
+	err = client.Prechecks(modelInfo)
 	return errors.Annotate(err, "target prechecks failed")
 }
 
@@ -515,20 +533,35 @@ func makeModelInfo(st *state.State) (coremigration.ModelInfo, error) {
 	if err != nil {
 		return empty, errors.Trace(err)
 	}
+
+	// Retrieve agent version for the model.
 	conf, err := st.ModelConfig()
 	if err != nil {
 		return empty, errors.Trace(err)
 	}
 	agentVersion, _ := conf.AgentVersion()
+
+	// Retrieve agent version for the controller.
+	controllerModel, err := st.ControllerModel()
+	if err != nil {
+		return empty, errors.Trace(err)
+	}
+	controllerConfig, err := controllerModel.Config()
+	if err != nil {
+		return empty, errors.Trace(err)
+	}
+	controllerVersion, _ := controllerConfig.AgentVersion()
+
 	return coremigration.ModelInfo{
-		UUID:         model.UUID(),
-		Name:         model.Name(),
-		Owner:        model.Owner(),
-		AgentVersion: agentVersion,
+		UUID:                   model.UUID(),
+		Name:                   model.Name(),
+		Owner:                  model.Owner(),
+		AgentVersion:           agentVersion,
+		ControllerAgentVersion: controllerVersion,
 	}, nil
 }
 
-func targetToAPIInfo(ti coremigration.TargetInfo) *api.Info {
+func targetToAPIInfo(ti *coremigration.TargetInfo) *api.Info {
 	return &api.Info{
 		Addrs:     ti.Addrs,
 		CACert:    ti.CACert,
