@@ -36,6 +36,7 @@ import (
 type controllerSuite struct {
 	statetesting.StateSuite
 
+	statePool  *state.StatePool
 	controller *controller.ControllerAPI
 	resources  *common.Resources
 	authorizer apiservertesting.FakeAuthorizer
@@ -50,6 +51,13 @@ func (s *controllerSuite) SetUpTest(c *gc.C) {
 	})
 
 	s.StateSuite.SetUpTest(c)
+
+	s.statePool = state.NewStatePool(s.State)
+	s.AddCleanup(func(c *gc.C) {
+		err := s.statePool.Close()
+		c.Assert(err, jc.ErrorIsNil)
+	})
+
 	s.resources = common.NewResources()
 	s.AddCleanup(func(_ *gc.C) { s.resources.StopAll() })
 
@@ -58,7 +66,13 @@ func (s *controllerSuite) SetUpTest(c *gc.C) {
 		AdminTag: s.Owner,
 	}
 
-	controller, err := controller.NewControllerAPI(s.State, s.resources, s.authorizer)
+	controller, err := controller.NewControllerAPI(
+		facadetest.Context{
+			State_:     s.State,
+			StatePool_: s.statePool,
+			Resources_: s.resources,
+			Auth_:      s.authorizer,
+		})
 	c.Assert(err, jc.ErrorIsNil)
 	s.controller = controller
 
@@ -69,7 +83,12 @@ func (s *controllerSuite) TestNewAPIRefusesNonClient(c *gc.C) {
 	anAuthoriser := apiservertesting.FakeAuthorizer{
 		Tag: names.NewUnitTag("mysql/0"),
 	}
-	endPoint, err := controller.NewControllerAPI(s.State, s.resources, anAuthoriser)
+	endPoint, err := controller.NewControllerAPI(
+		facadetest.Context{
+			State_:     s.State,
+			Resources_: s.resources,
+			Auth_:      anAuthoriser,
+		})
 	c.Assert(endPoint, gc.IsNil)
 	c.Assert(err, gc.ErrorMatches, "permission denied")
 }
@@ -240,7 +259,13 @@ func (s *controllerSuite) TestModelConfigFromNonController(c *gc.C) {
 		Tag:      s.Owner,
 		AdminTag: s.Owner,
 	}
-	controller, err := controller.NewControllerAPI(st, common.NewResources(), authorizer)
+	controller, err := controller.NewControllerAPI(
+		facadetest.Context{
+			State_:     st,
+			Resources_: common.NewResources(),
+			Auth_:      authorizer,
+		})
+
 	c.Assert(err, jc.ErrorIsNil)
 	cfg, err := controller.ModelConfig()
 	c.Assert(err, jc.ErrorIsNil)
@@ -263,7 +288,12 @@ func (s *controllerSuite) TestControllerConfigFromNonController(c *gc.C) {
 	defer st.Close()
 
 	authorizer := &apiservertesting.FakeAuthorizer{Tag: s.Owner}
-	controller, err := controller.NewControllerAPI(st, common.NewResources(), authorizer)
+	controller, err := controller.NewControllerAPI(
+		facadetest.Context{
+			State_:     st,
+			Resources_: common.NewResources(),
+			Auth_:      authorizer,
+		})
 	c.Assert(err, jc.ErrorIsNil)
 	cfg, err := controller.ControllerConfig()
 	c.Assert(err, jc.ErrorIsNil)
@@ -301,17 +331,20 @@ func (s *controllerSuite) TestWatchAllModels(c *gc.C) {
 	watcherId, err := s.controller.WatchAllModels()
 	c.Assert(err, jc.ErrorIsNil)
 
+	var disposed bool
 	watcherAPI_, err := apiserver.NewAllWatcher(facadetest.Context{
 		State_:     s.State,
 		Resources_: s.resources,
 		Auth_:      s.authorizer,
 		ID_:        watcherId.AllWatcherId,
+		Dispose_:   func() { disposed = true },
 	})
 	c.Assert(err, jc.ErrorIsNil)
 	watcherAPI := watcherAPI_.(*apiserver.SrvAllWatcher)
 	defer func() {
 		err := watcherAPI.Stop()
 		c.Assert(err, jc.ErrorIsNil)
+		c.Assert(disposed, jc.IsTrue)
 	}()
 
 	resultC := make(chan params.AllWatcherNextResults)
@@ -370,7 +403,6 @@ func (s *controllerSuite) TestInitiateMigration(c *gc.C) {
 					Macaroons:     string(macsJSON),
 					Password:      "secret2",
 				},
-				ExternalControl: true,
 			},
 		},
 	}
@@ -395,7 +427,6 @@ func (s *controllerSuite) TestInitiateMigration(c *gc.C) {
 		c.Check(mig.Id(), gc.Equals, expectedId)
 		c.Check(mig.ModelUUID(), gc.Equals, st.ModelUUID())
 		c.Check(mig.InitiatedBy(), gc.Equals, s.Owner.Id())
-		c.Check(mig.ExternalControl(), gc.Equals, args.Specs[i].ExternalControl)
 
 		targetInfo, err := mig.TargetInfo()
 		c.Assert(err, jc.ErrorIsNil)
@@ -519,34 +550,6 @@ func (s *controllerSuite) TestInitiateMigrationPrecheckFail(c *gc.C) {
 	c.Check(active, jc.IsFalse)
 }
 
-func (s *controllerSuite) TestInitiateMigrationSkipPrechecks(c *gc.C) {
-	st := s.Factory.MakeModel(c, nil)
-	defer st.Close()
-	controller.SetPrecheckResult(s, errors.New("should not happen"))
-
-	args := params.InitiateMigrationArgs{
-		Specs: []params.MigrationSpec{
-			{
-				ModelTag: st.ModelTag().String(),
-				TargetInfo: params.MigrationTargetInfo{
-					ControllerTag: randomControllerTag(),
-					Addrs:         []string{"1.1.1.1:1111", "2.2.2.2:2222"},
-					CACert:        "cert",
-					AuthTag:       names.NewUserTag("admin").String(),
-					Password:      "secret",
-				},
-				ExternalControl:      true,
-				SkipInitialPrechecks: true,
-			},
-		},
-	}
-	out, err := s.controller.InitiateMigration(args)
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(out.Results, gc.HasLen, 1)
-	c.Check(out.Results[0].ModelTag, gc.Equals, st.ModelTag().String())
-	c.Check(out.Results[0].Error, gc.IsNil)
-}
-
 func randomControllerTag() string {
 	uuid := utils.MustNewUUID().String()
 	return names.NewControllerTag(uuid).String()
@@ -786,7 +789,12 @@ func (s *controllerSuite) TestGetControllerAccessPermissions(c *gc.C) {
 	anAuthoriser := apiservertesting.FakeAuthorizer{
 		Tag: user.Tag(),
 	}
-	endpoint, err := controller.NewControllerAPI(s.State, s.resources, anAuthoriser)
+	endpoint, err := controller.NewControllerAPI(
+		facadetest.Context{
+			State_:     s.State,
+			Resources_: s.resources,
+			Auth_:      anAuthoriser,
+		})
 	c.Assert(err, jc.ErrorIsNil)
 	args := params.ModifyControllerAccessRequest{
 		Changes: []params.ModifyControllerAccess{{
