@@ -13,7 +13,7 @@ type patcher interface {
 }
 
 func SetPrecheckResult(p patcher, err error) {
-	p.PatchValue(&runMigrationPrechecks, func(*state.State, migration.TargetInfo) error {
+	p.PatchValue(&runMigrationPrechecks, func(*state.State, *migration.TargetInfo) error {
 		return err
 	})
 }
