@@ -4,25 +4,19 @@
 package diskmanager
 
 import (
-	"github.com/juju/loggo"
-	"github.com/juju/names"
+	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/apiserver/common"
+	"github.com/juju/juju/apiserver/facade"
 	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/storage"
 )
 
-func init() {
-	common.RegisterStandardFacade("DiskManager", 1, NewDiskManagerAPI)
-}
-
-var logger = loggo.GetLogger("juju.apiserver.diskmanager")
-
 // DiskManagerAPI provides access to the DiskManager API facade.
 type DiskManagerAPI struct {
 	st          stateInterface
-	authorizer  common.Authorizer
+	authorizer  facade.Authorizer
 	getAuthFunc common.GetAuthFunc
 }
 
@@ -33,8 +27,8 @@ var getState = func(st *state.State) stateInterface {
 // NewDiskManagerAPI creates a new server-side DiskManager API facade.
 func NewDiskManagerAPI(
 	st *state.State,
-	resources *common.Resources,
-	authorizer common.Authorizer,
+	resources facade.Resources,
+	authorizer facade.Authorizer,
 ) (*DiskManagerAPI, error) {
 
 	if !authorizer.AuthMachineAgent() {
@@ -97,9 +91,11 @@ func stateBlockDeviceInfo(devices []storage.BlockDevice) []state.BlockDeviceInfo
 	for i, dev := range devices {
 		result[i] = state.BlockDeviceInfo{
 			dev.DeviceName,
+			dev.DeviceLinks,
 			dev.Label,
 			dev.UUID,
 			dev.HardwareId,
+			dev.WWN,
 			dev.BusAddress,
 			dev.Size,
 			dev.FilesystemType,
