@@ -24,10 +24,11 @@ import (
 type firewallerBaseSuite struct {
 	testing.JujuConnSuite
 
-	machines []*state.Machine
-	service  *state.Application
-	charm    *state.Charm
-	units    []*state.Unit
+	machines  []*state.Machine
+	service   *state.Application
+	charm     *state.Charm
+	units     []*state.Unit
+	relations []*state.Relation
 
 	authorizer apiservertesting.FakeAuthorizer
 	resources  *common.Resources
@@ -59,10 +60,19 @@ func (s *firewallerBaseSuite) setUpTest(c *gc.C) {
 		s.units = append(s.units, unit)
 	}
 
+	// Create a relation.
+	s.AddTestingService(c, "mysql", s.AddTestingCharm(c, "mysql"))
+	eps, err := s.State.InferEndpoints("wordpress", "mysql")
+	c.Assert(err, jc.ErrorIsNil)
+
+	s.relations = make([]*state.Relation, 1)
+	s.relations[0], err = s.State.AddRelation(eps...)
+	c.Assert(err, jc.ErrorIsNil)
+
 	// Create a FakeAuthorizer so we can check permissions,
 	// set up assuming we logged in as the environment manager.
 	s.authorizer = apiservertesting.FakeAuthorizer{
-		EnvironManager: true,
+		Controller: true,
 	}
 
 	// Create the resource registry separately to track invocations to
@@ -75,7 +85,7 @@ func (s *firewallerBaseSuite) testFirewallerFailsWithNonEnvironManagerUser(
 	factory func(_ *state.State, _ facade.Resources, _ facade.Authorizer) error,
 ) {
 	anAuthorizer := s.authorizer
-	anAuthorizer.EnvironManager = false
+	anAuthorizer.Controller = false
 	err := factory(s.State, s.resources, anAuthorizer)
 	c.Assert(err, gc.NotNil)
 	c.Assert(err, gc.ErrorMatches, "permission denied")
@@ -101,6 +111,7 @@ func (s *firewallerBaseSuite) testLife(
 		{Tag: s.machines[0].Tag().String()},
 		{Tag: s.machines[1].Tag().String()},
 		{Tag: s.machines[2].Tag().String()},
+		{Tag: s.relations[0].Tag().String()},
 	}})
 	result, err := facade.Life(args)
 	c.Assert(err, jc.ErrorIsNil)
@@ -109,6 +120,7 @@ func (s *firewallerBaseSuite) testLife(
 			{Life: "alive"},
 			{Life: "dead"},
 			{Life: "alive"},
+			{Life: "alive"},
 			{Error: apiservertesting.NotFoundError("machine 42")},
 			{Error: apiservertesting.NotFoundError(`unit "foo/0"`)},
 			{Error: apiservertesting.NotFoundError(`application "bar"`)},
