@@ -5,18 +5,16 @@
 package machineactions
 
 import (
+	"gopkg.in/juju/names.v2"
+
 	"github.com/juju/juju/apiserver/common"
 	"github.com/juju/juju/apiserver/facade"
 	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/state"
-	"gopkg.in/juju/names.v2"
 )
 
-func init() {
-	common.RegisterStandardFacade("MachineActions", 1, newFacade)
-}
-
-func newFacade(st *state.State, res facade.Resources, auth facade.Authorizer) (*Facade, error) {
+// NewExternalFacade is used for API registration.
+func NewExternalFacade(st *state.State, res facade.Resources, auth facade.Authorizer) (*Facade, error) {
 	return NewFacade(backendShim{st}, res, auth)
 }
 
