@@ -7,6 +7,7 @@ import (
 	"strings"
 	"time"
 
+	"github.com/juju/description"
 	"github.com/juju/errors"
 	gitjujutesting "github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
@@ -15,13 +16,11 @@ import (
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/apiserver/common"
-	"github.com/juju/juju/apiserver/metricsender"
 	"github.com/juju/juju/apiserver/modelmanager"
 	"github.com/juju/juju/apiserver/params"
 	apiservertesting "github.com/juju/juju/apiserver/testing"
 	"github.com/juju/juju/cloud"
 	"github.com/juju/juju/controller"
-	"github.com/juju/juju/core/description"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/config"
 	"github.com/juju/juju/instance"
@@ -50,7 +49,6 @@ func (s *modelInfoSuite) SetUpTest(c *gc.C) {
 		Tag: names.NewUserTag("admin@local"),
 	}
 	s.st = &mockState{
-		modelUUID:      coretesting.ModelTag.Id(),
 		controllerUUID: coretesting.ControllerTag.Id(),
 		cloud: cloud.Cloud{
 			Type:      "dummy",
@@ -75,6 +73,10 @@ func (s *modelInfoSuite) SetUpTest(c *gc.C) {
 		owner: names.NewUserTag("admin@local"),
 		life:  state.Alive,
 		cfg:   coretesting.ModelConfig(c),
+		// This feels kind of wrong as both controller model and
+		// default model will end up with the same model tag.
+		tag:            coretesting.ModelTag,
+		controllerUUID: s.st.controllerUUID,
 		status: status.StatusInfo{
 			Status: status.Available,
 			Since:  &time.Time{},
@@ -89,14 +91,15 @@ func (s *modelInfoSuite) SetUpTest(c *gc.C) {
 	}
 
 	s.st.model = &mockModel{
-		owner: names.NewUserTag("bob@local"),
-		cfg:   coretesting.ModelConfig(c),
-		life:  state.Dying,
+		owner:          names.NewUserTag("bob@local"),
+		cfg:            coretesting.ModelConfig(c),
+		tag:            coretesting.ModelTag,
+		controllerUUID: s.st.controllerUUID,
+		life:           state.Dying,
 		status: status.StatusInfo{
 			Status: status.Destroying,
 			Since:  &time.Time{},
 		},
-
 		users: []*mockModelUser{{
 			userName: "admin",
 			access:   permission.AdminAccess,
@@ -133,19 +136,21 @@ func (s *modelInfoSuite) SetUpTest(c *gc.C) {
 	}
 
 	var err error
-	s.modelmanager, err = modelmanager.NewModelManagerAPI(s.st, nil, &s.authorizer)
+	s.modelmanager, err = modelmanager.NewModelManagerAPI(s.st, nil, nil, &s.authorizer)
 	c.Assert(err, jc.ErrorIsNil)
 }
 
 func (s *modelInfoSuite) setAPIUser(c *gc.C, user names.UserTag) {
 	s.authorizer.Tag = user
 	var err error
-	s.modelmanager, err = modelmanager.NewModelManagerAPI(s.st, nil, s.authorizer)
+	s.modelmanager, err = modelmanager.NewModelManagerAPI(s.st, nil, nil, s.authorizer)
 	c.Assert(err, jc.ErrorIsNil)
 }
 
 func (s *modelInfoSuite) TestModelInfo(c *gc.C) {
-	info := s.getModelInfo(c)
+	info := s.getModelInfo(c, s.st.model.cfg.UUID())
+	expectedAgentVersion, exists := s.st.model.cfg.AgentVersion()
+	c.Assert(exists, jc.IsTrue)
 	c.Assert(info, jc.DeepEquals, params.ModelInfo{
 		Name:               "testenv",
 		UUID:               s.st.model.cfg.UUID(),
@@ -187,39 +192,51 @@ func (s *modelInfoSuite) TestModelInfo(c *gc.C) {
 		}, {
 			Id: "2",
 		}},
+		SLA: &params.ModelSLAInfo{
+			Level: "essential",
+			Owner: "user",
+		},
+		AgentVersion: &expectedAgentVersion,
 	})
 	s.st.CheckCalls(c, []gitjujutesting.StubCall{
 		{"ControllerTag", nil},
 		{"ModelUUID", nil},
 		{"ForModel", []interface{}{names.NewModelTag(s.st.model.cfg.UUID())}},
 		{"Model", nil},
-		{"ControllerConfig", nil},
 		{"LastModelConnection", []interface{}{names.NewUserTag("admin")}},
 		{"LastModelConnection", []interface{}{names.NewLocalUserTag("bob")}},
 		{"LastModelConnection", []interface{}{names.NewLocalUserTag("charlotte")}},
 		{"LastModelConnection", []interface{}{names.NewLocalUserTag("mary")}},
 		{"AllMachines", nil},
+		{"LatestMigration", nil},
 		{"Close", nil},
 	})
 	s.st.model.CheckCalls(c, []gitjujutesting.StubCall{
+		{"UUID", nil},
+		{"Owner", nil},
+		{"Name", nil},
+		{"UUID", nil},
+		{"ControllerUUID", nil},
+		{"Life", nil},
+		{"Cloud", nil},
+		{"CloudRegion", nil},
+		{"CloudCredential", nil},
+		{"SLALevel", nil},
+		{"SLAOwner", nil},
+		{"Life", nil},
 		{"Config", nil},
+		{"Status", nil},
 		{"Users", nil},
 		{"ModelTag", nil},
 		{"ModelTag", nil},
 		{"ModelTag", nil},
 		{"ModelTag", nil},
-		{"Status", nil},
-		{"Owner", nil},
-		{"Life", nil},
-		{"Cloud", nil},
-		{"CloudRegion", nil},
-		{"CloudCredential", nil},
 	})
 }
 
 func (s *modelInfoSuite) TestModelInfoOwner(c *gc.C) {
 	s.setAPIUser(c, names.NewUserTag("bob@local"))
-	info := s.getModelInfo(c)
+	info := s.getModelInfo(c, s.st.model.cfg.UUID())
 	c.Assert(info.Users, gc.HasLen, 4)
 	c.Assert(info.Machines, gc.HasLen, 2)
 }
@@ -228,7 +245,7 @@ func (s *modelInfoSuite) TestModelInfoWriteAccess(c *gc.C) {
 	mary := names.NewUserTag("mary@local")
 	s.authorizer.HasWriteTag = mary
 	s.setAPIUser(c, mary)
-	info := s.getModelInfo(c)
+	info := s.getModelInfo(c, s.st.model.cfg.UUID())
 	c.Assert(info.Users, gc.HasLen, 1)
 	c.Assert(info.Users[0].UserName, gc.Equals, "mary")
 	c.Assert(info.Machines, gc.HasLen, 2)
@@ -236,22 +253,22 @@ func (s *modelInfoSuite) TestModelInfoWriteAccess(c *gc.C) {
 
 func (s *modelInfoSuite) TestModelInfoNonOwner(c *gc.C) {
 	s.setAPIUser(c, names.NewUserTag("charlotte@local"))
-	info := s.getModelInfo(c)
+	info := s.getModelInfo(c, s.st.model.cfg.UUID())
 	c.Assert(info.Users, gc.HasLen, 1)
 	c.Assert(info.Users[0].UserName, gc.Equals, "charlotte")
 	c.Assert(info.Machines, gc.HasLen, 0)
 }
 
-func (s *modelInfoSuite) getModelInfo(c *gc.C) params.ModelInfo {
+func (s *modelInfoSuite) getModelInfo(c *gc.C, modelUUID string) params.ModelInfo {
 	results, err := s.modelmanager.ModelInfo(params.Entities{
 		Entities: []params.Entity{{
-			names.NewModelTag(s.st.model.cfg.UUID()).String(),
+			names.NewModelTag(modelUUID).String(),
 		}},
 	})
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(results.Results, gc.HasLen, 1)
-	c.Assert(results.Results[0].Result, gc.NotNil)
-	c.Assert(results.Results[0].Error, gc.IsNil)
+	c.Check(results.Results[0].Result, gc.NotNil)
+	c.Check(results.Results[0].Error, gc.IsNil)
 	return *results.Results[0].Result
 }
 
@@ -270,7 +287,11 @@ func (s *modelInfoSuite) TestModelInfoErrorModelConfig(c *gc.C) {
 }
 
 func (s *modelInfoSuite) TestModelInfoErrorModelUsers(c *gc.C) {
-	s.st.model.SetErrors(errors.Errorf("no users for you"))
+	s.st.model.SetErrors(
+		nil, //Config
+		nil, //Status
+		errors.Errorf("no users for you"), // Users
+	)
 	s.testModelInfoError(c, coretesting.ModelTag.String(), `no users for you`)
 }
 
@@ -284,6 +305,206 @@ func (s *modelInfoSuite) TestModelInfoErrorNoAccess(c *gc.C) {
 	s.testModelInfoError(c, coretesting.ModelTag.String(), `permission denied`)
 }
 
+func (s *modelInfoSuite) TestRunningMigration(c *gc.C) {
+	start := time.Now().Add(-20 * time.Minute)
+	s.st.migration = &mockMigration{
+		status: "computing optimal bin packing",
+		start:  start,
+	}
+
+	results, err := s.modelmanager.ModelInfo(params.Entities{
+		Entities: []params.Entity{{coretesting.ModelTag.String()}},
+	})
+
+	c.Assert(err, jc.ErrorIsNil)
+	migrationResult := results.Results[0].Result.Migration
+	c.Assert(migrationResult.Status, gc.Equals, "computing optimal bin packing")
+	c.Assert(*migrationResult.Start, gc.Equals, start)
+	c.Assert(migrationResult.End, gc.IsNil)
+}
+
+func (s *modelInfoSuite) TestFailedMigration(c *gc.C) {
+	start := time.Now().Add(-20 * time.Minute)
+	end := time.Now().Add(-10 * time.Minute)
+	s.st.migration = &mockMigration{
+		status: "couldn't realign alternate time frames",
+		start:  start,
+		end:    end,
+	}
+
+	results, err := s.modelmanager.ModelInfo(params.Entities{
+		Entities: []params.Entity{{coretesting.ModelTag.String()}},
+	})
+
+	c.Assert(err, jc.ErrorIsNil)
+	migrationResult := results.Results[0].Result.Migration
+	c.Assert(migrationResult.Status, gc.Equals, "couldn't realign alternate time frames")
+	c.Assert(*migrationResult.Start, gc.Equals, start)
+	c.Assert(*migrationResult.End, gc.Equals, end)
+}
+
+func (s *modelInfoSuite) TestNoMigration(c *gc.C) {
+	results, err := s.modelmanager.ModelInfo(params.Entities{
+		Entities: []params.Entity{{coretesting.ModelTag.String()}},
+	})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(results.Results[0].Result.Migration, gc.IsNil)
+}
+
+func (s *modelInfoSuite) TestAliveModelGetsAllInfo(c *gc.C) {
+	s.assertSuccess(c, s.st.model.cfg.UUID(), state.Alive, params.Alive)
+}
+
+func (s *modelInfoSuite) TestAliveModelWithConfigFailure(c *gc.C) {
+	s.st.model.life = state.Alive
+	s.setModelConfigError()
+	s.testModelInfoError(c, s.st.model.tag.String(), "config not found")
+}
+
+func (s *modelInfoSuite) TestAliveModelWithStatusFailure(c *gc.C) {
+	s.st.model.life = state.Alive
+	s.setModelStatusError()
+	s.testModelInfoError(c, s.st.model.tag.String(), "status not found")
+}
+
+func (s *modelInfoSuite) TestAliveModelWithUsersFailure(c *gc.C) {
+	s.st.model.life = state.Alive
+	s.setModelUsersError()
+	s.testModelInfoError(c, s.st.model.tag.String(), "users not found")
+}
+
+func (s *modelInfoSuite) TestDeadModelGetsAllInfo(c *gc.C) {
+	s.assertSuccess(c, s.st.model.cfg.UUID(), state.Dead, params.Dead)
+}
+
+func (s *modelInfoSuite) TestDeadModelWithConfigFailure(c *gc.C) {
+	testData := incompleteModelInfoTest{
+		failModel:    s.setModelConfigError,
+		desiredLife:  state.Dead,
+		expectedLife: params.Dead,
+	}
+	s.assertSuccessWithMissingData(c, testData)
+}
+
+func (s *modelInfoSuite) TestDeadModelWithStatusFailure(c *gc.C) {
+	testData := incompleteModelInfoTest{
+		failModel:    s.setModelStatusError,
+		desiredLife:  state.Dead,
+		expectedLife: params.Dead,
+	}
+	s.assertSuccessWithMissingData(c, testData)
+}
+
+func (s *modelInfoSuite) TestDeadModelWithUsersFailure(c *gc.C) {
+	testData := incompleteModelInfoTest{
+		failModel:    s.setModelUsersError,
+		desiredLife:  state.Dead,
+		expectedLife: params.Dead,
+	}
+	s.assertSuccessWithMissingData(c, testData)
+}
+
+func (s *modelInfoSuite) TestDyingModelWithConfigFailure(c *gc.C) {
+	testData := incompleteModelInfoTest{
+		failModel:    s.setModelConfigError,
+		desiredLife:  state.Dying,
+		expectedLife: params.Dying,
+	}
+	s.assertSuccessWithMissingData(c, testData)
+}
+
+func (s *modelInfoSuite) TestDyingModelWithStatusFailure(c *gc.C) {
+	testData := incompleteModelInfoTest{
+		failModel:    s.setModelStatusError,
+		desiredLife:  state.Dying,
+		expectedLife: params.Dying,
+	}
+	s.assertSuccessWithMissingData(c, testData)
+}
+
+func (s *modelInfoSuite) TestDyingModelWithUsersFailure(c *gc.C) {
+	testData := incompleteModelInfoTest{
+		failModel:    s.setModelUsersError,
+		desiredLife:  state.Dying,
+		expectedLife: params.Dying,
+	}
+	s.assertSuccessWithMissingData(c, testData)
+}
+
+func (s *modelInfoSuite) TestImportingModelGetsAllInfo(c *gc.C) {
+	s.st.model.migrationStatus = state.MigrationModeImporting
+	s.assertSuccess(c, s.st.model.cfg.UUID(), state.Alive, params.Alive)
+}
+
+func (s *modelInfoSuite) TestImportingModelWithConfigFailure(c *gc.C) {
+	s.st.model.migrationStatus = state.MigrationModeImporting
+	testData := incompleteModelInfoTest{
+		failModel:    s.setModelConfigError,
+		desiredLife:  state.Alive,
+		expectedLife: params.Alive,
+	}
+	s.assertSuccessWithMissingData(c, testData)
+}
+
+func (s *modelInfoSuite) TestImportingModelWithStatusFailure(c *gc.C) {
+	s.st.model.migrationStatus = state.MigrationModeImporting
+	testData := incompleteModelInfoTest{
+		failModel:    s.setModelStatusError,
+		desiredLife:  state.Alive,
+		expectedLife: params.Alive,
+	}
+	s.assertSuccessWithMissingData(c, testData)
+}
+
+func (s *modelInfoSuite) TestImportingModelWithUsersFailure(c *gc.C) {
+	s.st.model.migrationStatus = state.MigrationModeImporting
+	testData := incompleteModelInfoTest{
+		failModel:    s.setModelUsersError,
+		desiredLife:  state.Alive,
+		expectedLife: params.Alive,
+	}
+	s.assertSuccessWithMissingData(c, testData)
+}
+
+type incompleteModelInfoTest struct {
+	failModel    func()
+	desiredLife  state.Life
+	expectedLife params.Life
+}
+
+func (s *modelInfoSuite) setModelConfigError() {
+	s.st.model.SetErrors(errors.NotFoundf("config"))
+}
+
+func (s *modelInfoSuite) setModelStatusError() {
+	s.st.model.SetErrors(
+		nil, //Config
+		errors.NotFoundf("status"), //Status
+	)
+}
+
+func (s *modelInfoSuite) setModelUsersError() {
+	s.st.model.SetErrors(
+		nil, //Config
+		nil, //Status
+		errors.NotFoundf("users"), //Users
+	)
+}
+
+func (s *modelInfoSuite) assertSuccessWithMissingData(c *gc.C, test incompleteModelInfoTest) {
+	test.failModel()
+	// We do not expect any errors to surface and still want to get basic model info.
+	s.assertSuccess(c, s.st.model.cfg.UUID(), test.desiredLife, test.expectedLife)
+}
+
+func (s *modelInfoSuite) assertSuccess(c *gc.C, modelUUID string, desiredLife state.Life, expectedLife params.Life) {
+	s.st.model.life = desiredLife
+	// should get no errors
+	info := s.getModelInfo(c, modelUUID)
+	c.Assert(info.UUID, gc.Equals, modelUUID)
+	c.Assert(info.Life, gc.Equals, expectedLife)
+}
+
 func (s *modelInfoSuite) testModelInfoError(c *gc.C, modelTag, expectedErr string) {
 	results, err := s.modelmanager.ModelInfo(params.Entities{
 		Entities: []params.Entity{{modelTag}},
@@ -298,6 +519,16 @@ type unitRetriever interface {
 	Unit(name string) (*state.Unit, error)
 }
 
+// metricSender defines methods required by the metricsender package.
+type metricSender interface {
+	MetricsManager() (*state.MetricsManager, error)
+	MetricsToSend(batchSize int) ([]*state.MetricBatch, error)
+	SetMetricBatchesSent(batchUUIDs []string) error
+	CountOfUnsentMetrics() (int, error)
+	CountOfSentMetrics() (int, error)
+	CleanupOldMetrics() error
+}
+
 type mockState struct {
 	gitjujutesting.Stub
 
@@ -305,10 +536,9 @@ type mockState struct {
 	common.APIHostPortsGetter
 	common.ToolsStorageGetter
 	common.BlockGetter
-	metricsender.MetricsSenderBackend
+	metricSender
 	unitRetriever
 
-	modelUUID       string
 	controllerUUID  string
 	cloud           cloud.Cloud
 	clouds          map[names.CloudTag]cloud.Cloud
@@ -320,6 +550,7 @@ type mockState struct {
 	cfgDefaults     config.ModelDefaultAttributes
 	blockMsg        string
 	block           state.BlockType
+	migration       *mockMigration
 }
 
 type fakeModelDescription struct {
@@ -329,12 +560,16 @@ type fakeModelDescription struct {
 }
 
 func (st *mockState) Export() (description.Model, error) {
-	return &fakeModelDescription{UUID: st.modelUUID}, nil
+	return &fakeModelDescription{UUID: st.model.UUID()}, nil
+}
+
+func (st *mockState) ExportPartial(state.ExportConfig) (description.Model, error) {
+	return st.Export()
 }
 
 func (st *mockState) ModelUUID() string {
 	st.MethodCall(st, "ModelUUID")
-	return st.modelUUID
+	return st.model.UUID()
 }
 
 func (st *mockState) ModelsForUser(user names.UserTag) ([]*state.UserModel, error) {
@@ -473,6 +708,16 @@ func (st *mockState) RemoveModelUser(tag names.UserTag) error {
 
 func (st *mockState) UserAccess(tag names.UserTag, target names.Tag) (permission.UserAccess, error) {
 	st.MethodCall(st, "ModelUser", tag, target)
+	for _, user := range st.users {
+		if user.UserTag != tag {
+			continue
+		}
+		nextErr := st.NextErr()
+		if nextErr != nil {
+			return permission.UserAccess{}, nextErr
+		}
+		return user, nil
+	}
 	return permission.UserAccess{}, st.NextErr()
 }
 
@@ -540,6 +785,11 @@ func (st *mockState) GetBlockForType(t state.BlockType) (state.Block, bool, erro
 	}
 }
 
+func (st *mockState) ReloadSpaces(environ environs.Environ) error {
+	st.MethodCall(st, "ReloadSpaces", environ)
+	return st.NextErr()
+}
+
 func (st *mockState) DumpAll() (map[string]interface{}, error) {
 	st.MethodCall(st, "DumpAll")
 	return map[string]interface{}{
@@ -547,6 +797,21 @@ func (st *mockState) DumpAll() (map[string]interface{}, error) {
 	}, st.NextErr()
 }
 
+func (st *mockState) LatestMigration() (state.ModelMigration, error) {
+	st.MethodCall(st, "LatestMigration")
+	if st.migration == nil {
+		// Handle nil->notfound directly here rather than having to
+		// count errors.
+		return nil, errors.NotFoundf("")
+	}
+	return st.migration, st.NextErr()
+}
+
+func (st *mockState) SetModelMeterStatus(level, message string) error {
+	st.MethodCall(st, "SetModelMeterStatus", level, message)
+	return st.NextErr()
+}
+
 type mockBlock struct {
 	state.Block
 	t state.BlockType
@@ -609,12 +874,14 @@ func (m *mockMachine) Status() (status.StatusInfo, error) {
 
 type mockModel struct {
 	gitjujutesting.Stub
-	owner  names.UserTag
-	life   state.Life
-	tag    names.ModelTag
-	status status.StatusInfo
-	cfg    *config.Config
-	users  []*mockModelUser
+	owner           names.UserTag
+	life            state.Life
+	tag             names.ModelTag
+	status          status.StatusInfo
+	cfg             *config.Config
+	users           []*mockModelUser
+	migrationStatus state.MigrationMode
+	controllerUUID  string
 }
 
 func (m *mockModel) Config() (*config.Config, error) {
@@ -624,19 +891,16 @@ func (m *mockModel) Config() (*config.Config, error) {
 
 func (m *mockModel) Owner() names.UserTag {
 	m.MethodCall(m, "Owner")
-	m.PopNoErr()
 	return m.owner
 }
 
 func (m *mockModel) ModelTag() names.ModelTag {
 	m.MethodCall(m, "ModelTag")
-	m.PopNoErr()
 	return m.tag
 }
 
 func (m *mockModel) Life() state.Life {
 	m.MethodCall(m, "Life")
-	m.PopNoErr()
 	return m.life
 }
 
@@ -647,19 +911,16 @@ func (m *mockModel) Status() (status.StatusInfo, error) {
 
 func (m *mockModel) Cloud() string {
 	m.MethodCall(m, "Cloud")
-	m.PopNoErr()
 	return "some-cloud"
 }
 
 func (m *mockModel) CloudRegion() string {
 	m.MethodCall(m, "CloudRegion")
-	m.PopNoErr()
 	return "some-region"
 }
 
 func (m *mockModel) CloudCredential() (names.CloudCredentialTag, bool) {
 	m.MethodCall(m, "CloudCredential")
-	m.PopNoErr()
 	return names.NewCloudCredentialTag("some-cloud/bob/some-credential"), true
 }
 
@@ -692,6 +953,36 @@ func (m *mockModel) DestroyIncludingHosted() error {
 	return m.NextErr()
 }
 
+func (m *mockModel) SLALevel() string {
+	m.MethodCall(m, "SLALevel")
+	return "essential"
+}
+
+func (m *mockModel) SLAOwner() string {
+	m.MethodCall(m, "SLAOwner")
+	return "user"
+}
+
+func (m *mockModel) ControllerUUID() string {
+	m.MethodCall(m, "ControllerUUID")
+	return m.controllerUUID
+}
+
+func (m *mockModel) UUID() string {
+	m.MethodCall(m, "UUID")
+	return m.cfg.UUID()
+}
+
+func (m *mockModel) Name() string {
+	m.MethodCall(m, "Name")
+	return m.cfg.Name()
+}
+
+func (m *mockModel) MigrationMode() state.MigrationMode {
+	m.MethodCall(m, "MigrationMode")
+	return m.migrationStatus
+}
+
 type mockModelUser struct {
 	gitjujutesting.Stub
 	userName       string
@@ -699,3 +990,32 @@ type mockModelUser struct {
 	lastConnection time.Time
 	access         permission.Access
 }
+
+type mockMigration struct {
+	state.ModelMigration
+
+	status string
+	start  time.Time
+	end    time.Time
+}
+
+func (m *mockMigration) StatusMessage() string {
+	return m.status
+}
+
+func (m *mockMigration) StartTime() time.Time {
+	return m.start
+}
+
+func (m *mockMigration) EndTime() time.Time {
+	return m.end
+}
+
+type mockPool struct {
+	st *mockState
+}
+
+func (p *mockPool) Get(modelUUID string) (common.ModelManagerBackend, func(), error) {
+	p.st.MethodCall(p, "Get", modelUUID)
+	return p.st, func() {}, p.st.NextErr()
+}
