@@ -15,6 +15,8 @@ import (
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/names.v2"
 
+	// Register the providers for the field check test
+	"github.com/juju/juju/apiserver/common"
 	"github.com/juju/juju/apiserver/modelmanager"
 	"github.com/juju/juju/apiserver/params"
 	apiservertesting "github.com/juju/juju/apiserver/testing"
@@ -23,11 +25,6 @@ import (
 	"github.com/juju/juju/environs/config"
 	jujutesting "github.com/juju/juju/juju/testing"
 	"github.com/juju/juju/permission"
-	"github.com/juju/juju/state/stateenvirons"
-	"github.com/juju/juju/status"
-	jujuversion "github.com/juju/juju/version"
-	// Register the providers for the field check test
-	"github.com/juju/juju/apiserver/common"
 	_ "github.com/juju/juju/provider/azure"
 	"github.com/juju/juju/provider/dummy"
 	_ "github.com/juju/juju/provider/ec2"
@@ -35,16 +32,29 @@ import (
 	_ "github.com/juju/juju/provider/maas"
 	_ "github.com/juju/juju/provider/openstack"
 	"github.com/juju/juju/state"
+	"github.com/juju/juju/state/stateenvirons"
+	"github.com/juju/juju/status"
 	coretesting "github.com/juju/juju/testing"
 	"github.com/juju/juju/testing/factory"
+	jujuversion "github.com/juju/juju/version"
 )
 
-type modelManagerBaseSuite struct {
+func createArgs(owner names.UserTag) params.ModelCreateArgs {
+	return params.ModelCreateArgs{
+		Name:     "test-model",
+		OwnerTag: owner.String(),
+		Config: map[string]interface{}{
+			"authorized-keys": "ssh-key",
+			// And to make it a valid dummy config
+			"controller": false,
+		},
+	}
 }
 
 type modelManagerSuite struct {
 	gitjujutesting.IsolationSuite
 	st         *mockState
+	pool       *mockPool
 	authoriser apiservertesting.FakeAuthorizer
 	api        *modelmanager.ModelManagerAPI
 }
@@ -69,13 +79,12 @@ func (s *modelManagerSuite) SetUpTest(c *gc.C) {
 	}
 
 	s.st = &mockState{
-		modelUUID: coretesting.ModelTag.Id(),
-		cloud:     dummyCloud,
+		cloud: dummyCloud,
 		clouds: map[names.CloudTag]cloud.Cloud{
 			names.NewCloudTag("some-cloud"): dummyCloud,
 		},
 		controllerModel: &mockModel{
-			owner: names.NewUserTag("admin@local"),
+			owner: names.NewUserTag("admin"),
 			life:  state.Alive,
 			cfg:   cfg,
 			status: status.StatusInfo{
@@ -86,12 +95,16 @@ func (s *modelManagerSuite) SetUpTest(c *gc.C) {
 				userName: "admin",
 				access:   permission.AdminAccess,
 			}, {
+				userName: "add-model",
+				access:   permission.AdminAccess,
+			}, {
+
 				userName: "otheruser",
 				access:   permission.WriteAccess,
 			}},
 		},
 		model: &mockModel{
-			owner: names.NewUserTag("admin@local"),
+			owner: names.NewUserTag("admin"),
 			life:  state.Alive,
 			tag:   coretesting.ModelTag,
 			cfg:   cfg,
@@ -103,6 +116,10 @@ func (s *modelManagerSuite) SetUpTest(c *gc.C) {
 				userName: "admin",
 				access:   permission.AdminAccess,
 			}, {
+				userName: "add-model",
+				access:   permission.AdminAccess,
+			}, {
+
 				userName: "otheruser",
 				access:   permission.WriteAccess,
 			}},
@@ -124,16 +141,17 @@ func (s *modelManagerSuite) SetUpTest(c *gc.C) {
 		},
 	}
 	s.authoriser = apiservertesting.FakeAuthorizer{
-		Tag: names.NewUserTag("admin@local"),
+		Tag: names.NewUserTag("admin"),
 	}
-	api, err := modelmanager.NewModelManagerAPI(s.st, nil, s.authoriser)
+	s.pool = &mockPool{s.st}
+	api, err := modelmanager.NewModelManagerAPI(s.st, s.pool, nil, s.authoriser)
 	c.Assert(err, jc.ErrorIsNil)
 	s.api = api
 }
 
 func (s *modelManagerSuite) setAPIUser(c *gc.C, user names.UserTag) {
 	s.authoriser.Tag = user
-	mm, err := modelmanager.NewModelManagerAPI(s.st, nil, s.authoriser)
+	mm, err := modelmanager.NewModelManagerAPI(s.st, s.pool, nil, s.authoriser)
 	c.Assert(err, jc.ErrorIsNil)
 	s.api = mm
 }
@@ -173,12 +191,14 @@ func (s *modelManagerSuite) TestCreateModelArgs(c *gc.C) {
 		"ControllerConfig",
 		"ComposeNewModelConfig",
 		"NewModel",
+		"ReloadSpaces",
 		"ForModel",
 		"Model",
-		"ControllerConfig",
+		"LastModelConnection",
 		"LastModelConnection",
 		"LastModelConnection",
 		"AllMachines",
+		"LatestMigration",
 		"Close",
 		"Close",
 	)
@@ -207,7 +227,7 @@ func (s *modelManagerSuite) TestCreateModelArgs(c *gc.C) {
 	newModelArgs.StorageProviderRegistry = nil
 
 	c.Assert(newModelArgs, jc.DeepEquals, state.ModelArgs{
-		Owner:       names.NewUserTag("admin@local"),
+		Owner:       names.NewUserTag("admin"),
 		CloudName:   "some-cloud",
 		CloudRegion: "qux",
 		CloudCredential: names.NewCloudCredentialTag(
@@ -428,14 +448,14 @@ func (s *modelManagerSuite) TestUnsetModelDefaultsMissing(c *gc.C) {
 }
 
 func (s *modelManagerSuite) TestModelDefaultsAsNormalUser(c *gc.C) {
-	s.setAPIUser(c, names.NewUserTag("charlie@local"))
+	s.setAPIUser(c, names.NewUserTag("charlie"))
 	got, err := s.api.ModelDefaults()
 	c.Assert(err, gc.ErrorMatches, "permission denied")
 	c.Assert(got, gc.DeepEquals, params.ModelDefaultsResult{})
 }
 
 func (s *modelManagerSuite) TestSetModelDefaultsAsNormalUser(c *gc.C) {
-	s.setAPIUser(c, names.NewUserTag("charlie@local"))
+	s.setAPIUser(c, names.NewUserTag("charlie"))
 	got, err := s.api.SetModelDefaults(params.SetModelDefaults{
 		Config: []params.ModelDefaultValues{{
 			Config: map[string]interface{}{
@@ -450,14 +470,14 @@ func (s *modelManagerSuite) TestSetModelDefaultsAsNormalUser(c *gc.C) {
 					Code:    "unauthorized access"}}}})
 
 	// Make sure it didn't change.
-	s.setAPIUser(c, names.NewUserTag("admin@local"))
+	s.setAPIUser(c, names.NewUserTag("admin"))
 	cfg, err := s.api.ModelDefaults()
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(cfg.Config["ftp-proxy"].Controller, gc.IsNil)
 }
 
 func (s *modelManagerSuite) TestUnsetModelDefaultsAsNormalUser(c *gc.C) {
-	s.setAPIUser(c, names.NewUserTag("charlie@local"))
+	s.setAPIUser(c, names.NewUserTag("charlie"))
 	got, err := s.api.UnsetModelDefaults(params.UnsetModelDefaults{
 		Keys: []params.ModelUnsetKeys{{
 			Keys: []string{"attr2"}}}})
@@ -468,14 +488,16 @@ func (s *modelManagerSuite) TestUnsetModelDefaultsAsNormalUser(c *gc.C) {
 				Error: nil}}})
 
 	// Make sure it didn't change.
-	s.setAPIUser(c, names.NewUserTag("admin@local"))
+	s.setAPIUser(c, names.NewUserTag("admin"))
 	cfg, err := s.api.ModelDefaults()
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(cfg.Config["attr2"].Controller.(string), gc.Equals, "val3")
 }
 
-func (s *modelManagerSuite) TestDumpModel(c *gc.C) {
-	results := s.api.DumpModels(params.Entities{[]params.Entity{{
+func (s *modelManagerSuite) TestDumpModelV2(c *gc.C) {
+	api := &modelmanager.ModelManagerAPIV2{s.api}
+
+	results := api.DumpModels(params.Entities{[]params.Entity{{
 		Tag: "bad-tag",
 	}, {
 		Tag: "application-foo",
@@ -497,27 +519,48 @@ func (s *modelManagerSuite) TestDumpModel(c *gc.C) {
 	})
 }
 
+func (s *modelManagerSuite) TestDumpModel(c *gc.C) {
+	results := s.api.DumpModels(params.DumpModelRequest{
+		Entities: []params.Entity{{
+			Tag: "bad-tag",
+		}, {
+			Tag: "application-foo",
+		}, {
+			Tag: s.st.ModelTag().String(),
+		}}})
+
+	c.Assert(results.Results, gc.HasLen, 3)
+	bad, notApp, good := results.Results[0], results.Results[1], results.Results[2]
+	c.Check(bad.Result, gc.Equals, "")
+	c.Check(bad.Error.Message, gc.Equals, `"bad-tag" is not a valid tag`)
+
+	c.Check(notApp.Result, gc.Equals, "")
+	c.Check(notApp.Error.Message, gc.Equals, `"application-foo" is not a valid model tag`)
+
+	c.Check(good.Error, gc.IsNil)
+	c.Check(good.Result, jc.DeepEquals, "model-uuid: deadbeef-0bad-400d-8000-4b1d0d06f00d\n")
+}
+
 func (s *modelManagerSuite) TestDumpModelMissingModel(c *gc.C) {
 	s.st.SetErrors(errors.NotFoundf("boom"))
 	tag := names.NewModelTag("deadbeef-0bad-400d-8000-4b1d0d06f000")
-	models := params.Entities{[]params.Entity{{Tag: tag.String()}}}
+	models := params.DumpModelRequest{Entities: []params.Entity{{Tag: tag.String()}}}
 	results := s.api.DumpModels(models)
-
-	calls := s.st.Calls()
-	c.Logf("%#v", calls)
-	lastCall := calls[len(calls)-1]
-	c.Check(lastCall.FuncName, gc.Equals, "ForModel")
-
+	s.st.CheckCalls(c, []gitjujutesting.StubCall{
+		{"ControllerTag", nil},
+		{"ModelUUID", nil},
+		{"Get", []interface{}{tag.Id()}},
+	})
 	c.Assert(results.Results, gc.HasLen, 1)
 	result := results.Results[0]
-	c.Assert(result.Result, gc.IsNil)
+	c.Assert(result.Result, gc.Equals, "")
 	c.Assert(result.Error, gc.NotNil)
 	c.Check(result.Error.Code, gc.Equals, `not found`)
 	c.Check(result.Error.Message, gc.Equals, `id not found`)
 }
 
 func (s *modelManagerSuite) TestDumpModelUsers(c *gc.C) {
-	models := params.Entities{[]params.Entity{{Tag: s.st.ModelTag().String()}}}
+	models := params.DumpModelRequest{Entities: []params.Entity{{Tag: s.st.ModelTag().String()}}}
 	for _, user := range []names.UserTag{
 		names.NewUserTag("otheruser"),
 		names.NewUserTag("unknown"),
@@ -526,7 +569,7 @@ func (s *modelManagerSuite) TestDumpModelUsers(c *gc.C) {
 		results := s.api.DumpModels(models)
 		c.Assert(results.Results, gc.HasLen, 1)
 		result := results.Results[0]
-		c.Assert(result.Result, gc.IsNil)
+		c.Assert(result.Result, gc.Equals, "")
 		c.Assert(result.Error, gc.NotNil)
 		c.Check(result.Error.Message, gc.Equals, `permission denied`)
 	}
@@ -561,11 +604,12 @@ func (s *modelManagerSuite) TestDumpModelsDBMissingModel(c *gc.C) {
 	models := params.Entities{[]params.Entity{{Tag: tag.String()}}}
 	results := s.api.DumpModelsDB(models)
 
-	calls := s.st.Calls()
-	c.Logf("%#v", calls)
-	lastCall := calls[len(calls)-1]
-	c.Check(lastCall.FuncName, gc.Equals, "ForModel")
-
+	s.st.CheckCalls(c, []gitjujutesting.StubCall{
+		{"ControllerTag", nil},
+		{"ModelUUID", nil},
+		{"ModelTag", nil},
+		{"Get", []interface{}{tag.Id()}},
+	})
 	c.Assert(results.Results, gc.HasLen, 1)
 	result := results.Results[0]
 	c.Assert(result.Result, gc.IsNil)
@@ -590,6 +634,45 @@ func (s *modelManagerSuite) TestDumpModelsDBUsers(c *gc.C) {
 	}
 }
 
+func (s *modelManagerSuite) TestAddModelCanCreateModel(c *gc.C) {
+	addModelUser := names.NewUserTag("add-model")
+	userAccess := permission.UserAccess{
+		UserID:      addModelUser.Id(),
+		UserTag:     addModelUser,
+		Object:      s.st.ControllerTag(),
+		Access:      permission.AddModelAccess,
+		CreatedBy:   names.NewUserTag("admin"),
+		DisplayName: addModelUser.Name(),
+		UserName:    addModelUser.Name(),
+	}
+	s.st.users = []permission.UserAccess{userAccess}
+	_, err := s.st.UserAccess(addModelUser, s.st.ControllerTag())
+	c.Assert(err, jc.ErrorIsNil)
+	s.setAPIUser(c, addModelUser)
+	_, err = s.api.CreateModel(createArgs(addModelUser))
+	c.Assert(err, jc.ErrorIsNil)
+}
+
+func (s *modelManagerSuite) TestAddModelCantCreateModelForSomeoneElse(c *gc.C) {
+	addModelUser := names.NewUserTag("add-model")
+	userAccess := permission.UserAccess{
+		UserID:      addModelUser.Id(),
+		UserTag:     addModelUser,
+		Object:      s.st.ControllerTag(),
+		Access:      permission.AddModelAccess,
+		CreatedBy:   names.NewUserTag("admin"),
+		DisplayName: addModelUser.Name(),
+		UserName:    addModelUser.Name(),
+	}
+	s.st.users = []permission.UserAccess{userAccess}
+	_, err := s.st.UserAccess(addModelUser, s.st.ControllerTag())
+	c.Assert(err, jc.ErrorIsNil)
+	s.setAPIUser(c, addModelUser)
+	nonAdminUser := names.NewUserTag("non-admin")
+	_, err = s.api.CreateModel(createArgs(nonAdminUser))
+	c.Assert(err, gc.ErrorMatches, "\"add-model\" permission does not permit creation of models for different owners: permission denied")
+}
+
 // modelManagerStateSuite contains end-to-end tests.
 // Prefer adding tests to modelManagerSuite above.
 type modelManagerStateSuite struct {
@@ -620,6 +703,7 @@ func (s *modelManagerStateSuite) setAPIUser(c *gc.C, user names.UserTag) {
 	s.authoriser.Tag = user
 	modelmanager, err := modelmanager.NewModelManagerAPI(
 		common.NewModelManagerBackend(s.State),
+		nil,
 		stateenvirons.EnvironConfigGetter{s.State},
 		s.authoriser,
 	)
@@ -631,7 +715,7 @@ func (s *modelManagerStateSuite) TestNewAPIAcceptsClient(c *gc.C) {
 	anAuthoriser := s.authoriser
 	anAuthoriser.Tag = names.NewUserTag("external@remote")
 	endPoint, err := modelmanager.NewModelManagerAPI(
-		common.NewModelManagerBackend(s.State), nil, anAuthoriser,
+		common.NewModelManagerBackend(s.State), nil, nil, anAuthoriser,
 	)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(endPoint, gc.NotNil)
@@ -641,34 +725,22 @@ func (s *modelManagerStateSuite) TestNewAPIRefusesNonClient(c *gc.C) {
 	anAuthoriser := s.authoriser
 	anAuthoriser.Tag = names.NewUnitTag("mysql/0")
 	endPoint, err := modelmanager.NewModelManagerAPI(
-		common.NewModelManagerBackend(s.State), nil, anAuthoriser,
+		common.NewModelManagerBackend(s.State), nil, nil, anAuthoriser,
 	)
 	c.Assert(endPoint, gc.IsNil)
 	c.Assert(err, gc.ErrorMatches, "permission denied")
 }
 
-func (s *modelManagerStateSuite) createArgs(c *gc.C, owner names.UserTag) params.ModelCreateArgs {
-	return params.ModelCreateArgs{
-		Name:     "test-model",
-		OwnerTag: owner.String(),
-		Config: map[string]interface{}{
-			"authorized-keys": "ssh-key",
-			// And to make it a valid dummy config
-			"controller": false,
-		},
-	}
-}
-
 func (s *modelManagerStateSuite) createArgsForVersion(c *gc.C, owner names.UserTag, ver interface{}) params.ModelCreateArgs {
-	params := s.createArgs(c, owner)
+	params := createArgs(owner)
 	params.Config["agent-version"] = ver
 	return params
 }
 
 func (s *modelManagerStateSuite) TestUserCanCreateModel(c *gc.C) {
-	owner := names.NewUserTag("admin@local")
+	owner := names.NewUserTag("admin")
 	s.setAPIUser(c, owner)
-	model, err := s.modelmanager.CreateModel(s.createArgs(c, owner))
+	model, err := s.modelmanager.CreateModel(createArgs(owner))
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(model.OwnerTag, gc.Equals, owner.String())
 	c.Assert(model.Name, gc.Equals, "test-model")
@@ -677,7 +749,7 @@ func (s *modelManagerStateSuite) TestUserCanCreateModel(c *gc.C) {
 func (s *modelManagerStateSuite) TestAdminCanCreateModelForSomeoneElse(c *gc.C) {
 	s.setAPIUser(c, s.AdminUserTag(c))
 	owner := names.NewUserTag("external@remote")
-	model, err := s.modelmanager.CreateModel(s.createArgs(c, owner))
+	model, err := s.modelmanager.CreateModel(createArgs(owner))
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(model.OwnerTag, gc.Equals, owner.String())
 	c.Assert(model.Name, gc.Equals, "test-model")
@@ -697,21 +769,21 @@ func (s *modelManagerStateSuite) TestAdminCanCreateModelForSomeoneElse(c *gc.C)
 func (s *modelManagerStateSuite) TestNonAdminCannotCreateModelForSomeoneElse(c *gc.C) {
 	s.setAPIUser(c, names.NewUserTag("non-admin@remote"))
 	owner := names.NewUserTag("external@remote")
-	_, err := s.modelmanager.CreateModel(s.createArgs(c, owner))
+	_, err := s.modelmanager.CreateModel(createArgs(owner))
 	c.Assert(err, gc.ErrorMatches, "permission denied")
 }
 
 func (s *modelManagerStateSuite) TestNonAdminCannotCreateModelForSelf(c *gc.C) {
 	owner := names.NewUserTag("non-admin@remote")
 	s.setAPIUser(c, owner)
-	_, err := s.modelmanager.CreateModel(s.createArgs(c, owner))
+	_, err := s.modelmanager.CreateModel(createArgs(owner))
 	c.Assert(err, gc.ErrorMatches, "permission denied")
 }
 
 func (s *modelManagerStateSuite) TestCreateModelValidatesConfig(c *gc.C) {
 	admin := s.AdminUserTag(c)
 	s.setAPIUser(c, admin)
-	args := s.createArgs(c, admin)
+	args := createArgs(admin)
 	args.Config["controller"] = "maybe"
 	_, err := s.modelmanager.CreateModel(args)
 	c.Assert(err, gc.ErrorMatches,
@@ -720,7 +792,7 @@ func (s *modelManagerStateSuite) TestCreateModelValidatesConfig(c *gc.C) {
 }
 
 func (s *modelManagerStateSuite) TestCreateModelBadConfig(c *gc.C) {
-	owner := names.NewUserTag("admin@local")
+	owner := names.NewUserTag("admin")
 	s.setAPIUser(c, owner)
 	for i, test := range []struct {
 		key      string
@@ -738,7 +810,7 @@ func (s *modelManagerStateSuite) TestCreateModelBadConfig(c *gc.C) {
 		},
 	} {
 		c.Logf("%d: %s", i, test.key)
-		args := s.createArgs(c, owner)
+		args := createArgs(owner)
 		args.Config[test.key] = test.value
 		_, err := s.modelmanager.CreateModel(args)
 		c.Assert(err, gc.ErrorMatches, test.errMatch)
@@ -755,7 +827,7 @@ func (s *modelManagerStateSuite) TestCreateModelSameAgentVersion(c *gc.C) {
 }
 
 func (s *modelManagerStateSuite) TestCreateModelBadAgentVersion(c *gc.C) {
-	err := s.BackingState.SetModelAgentVersion(coretesting.FakeVersionNumber)
+	err := s.BackingState.SetModelAgentVersion(coretesting.FakeVersionNumber, false)
 	c.Assert(err, jc.ErrorIsNil)
 
 	admin := s.AdminUserTag(c)
@@ -804,7 +876,7 @@ func (s *modelManagerStateSuite) TestListModelsForSelfLocalUser(c *gc.C) {
 	// When the user's credentials cache stores the simple name, but the
 	// api server converts it to a fully qualified name.
 	user := names.NewUserTag("local-user")
-	s.setAPIUser(c, names.NewUserTag("local-user@local"))
+	s.setAPIUser(c, names.NewUserTag("local-user"))
 	result, err := s.modelmanager.ListModels(params.Entity{Tag: user.String()})
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(result.UserModels, gc.HasLen, 0)
@@ -830,10 +902,10 @@ func (s *modelManagerStateSuite) TestListModelsAdminSelf(c *gc.C) {
 func (s *modelManagerStateSuite) TestListModelsAdminListsOther(c *gc.C) {
 	user := s.AdminUserTag(c)
 	s.setAPIUser(c, user)
-	other := names.NewUserTag("external@remote")
+	other := names.NewUserTag("admin")
 	result, err := s.modelmanager.ListModels(params.Entity{Tag: other.String()})
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(result.UserModels, gc.HasLen, 0)
+	c.Assert(result.UserModels, gc.HasLen, 1)
 }
 
 func (s *modelManagerStateSuite) TestListModelsDenied(c *gc.C) {
@@ -860,16 +932,16 @@ func (s *modelManagerStateSuite) TestDestroyOwnModel(c *gc.C) {
 	// TODO(perrito666) this test is not valid until we have
 	// proper controller permission since the only users that
 	// can create models are controller admins.
-	owner := names.NewUserTag("admin@local")
+	owner := names.NewUserTag("admin")
 	s.setAPIUser(c, owner)
-	m, err := s.modelmanager.CreateModel(s.createArgs(c, owner))
+	m, err := s.modelmanager.CreateModel(createArgs(owner))
 	c.Assert(err, jc.ErrorIsNil)
 	st, err := s.State.ForModel(names.NewModelTag(m.UUID))
 	c.Assert(err, jc.ErrorIsNil)
 	defer st.Close()
 
 	s.modelmanager, err = modelmanager.NewModelManagerAPI(
-		common.NewModelManagerBackend(st), nil, s.authoriser,
+		common.NewModelManagerBackend(st), nil, nil, s.authoriser,
 	)
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -888,16 +960,16 @@ func (s *modelManagerStateSuite) TestDestroyOwnModel(c *gc.C) {
 func (s *modelManagerStateSuite) TestAdminDestroysOtherModel(c *gc.C) {
 	// TODO(perrito666) Both users are admins in this case, this tesst is of dubious
 	// usefulness until proper controller permissions are in place.
-	owner := names.NewUserTag("admin@local")
+	owner := names.NewUserTag("admin")
 	s.setAPIUser(c, owner)
-	m, err := s.modelmanager.CreateModel(s.createArgs(c, owner))
+	m, err := s.modelmanager.CreateModel(createArgs(owner))
 	c.Assert(err, jc.ErrorIsNil)
 	st, err := s.State.ForModel(names.NewModelTag(m.UUID))
 	c.Assert(err, jc.ErrorIsNil)
 	defer st.Close()
 
 	s.modelmanager, err = modelmanager.NewModelManagerAPI(
-		common.NewModelManagerBackend(st), nil, s.authoriser,
+		common.NewModelManagerBackend(st), nil, nil, s.authoriser,
 	)
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -918,16 +990,16 @@ func (s *modelManagerStateSuite) TestAdminDestroysOtherModel(c *gc.C) {
 }
 
 func (s *modelManagerStateSuite) TestDestroyModelErrors(c *gc.C) {
-	owner := names.NewUserTag("admin@local")
+	owner := names.NewUserTag("admin")
 	s.setAPIUser(c, owner)
-	m, err := s.modelmanager.CreateModel(s.createArgs(c, owner))
+	m, err := s.modelmanager.CreateModel(createArgs(owner))
 	c.Assert(err, jc.ErrorIsNil)
 	st, err := s.State.ForModel(names.NewModelTag(m.UUID))
 	c.Assert(err, jc.ErrorIsNil)
 	defer st.Close()
 
 	s.modelmanager, err = modelmanager.NewModelManagerAPI(
-		common.NewModelManagerBackend(st), nil, s.authoriser,
+		common.NewModelManagerBackend(st), nil, nil, s.authoriser,
 	)
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -1210,7 +1282,7 @@ func (s *modelManagerStateSuite) TestGrantModelInvalidUserTag(c *gc.C) {
 		tag:      "machine-0",
 		validTag: true,
 	}, {
-		tag:      "user@local",
+		tag:      "user",
 		validTag: false,
 	}, {
 		tag:      "user-Mua^h^h^h^arh",
