@@ -294,9 +294,26 @@ type RelationUnitsSettings struct {
 	RelationUnits []RelationUnitSettings `json:"relation-units"`
 }
 
+// RelationResults holds the result of an API call that returns
+// information about multiple relations.
+type RelationResults struct {
+	Results []RelationResult `json:"results"`
+}
+
 // RelationResult returns information about a single relation,
 // or an error.
 type RelationResult struct {
+	Error            *Error                `json:"error,omitempty"`
+	Life             Life                  `json:"life"`
+	Id               int                   `json:"id"`
+	Key              string                `json:"key"`
+	Endpoint         multiwatcher.Endpoint `json:"endpoint"`
+	OtherApplication string                `json:"other-application,omitempty"`
+}
+
+// RelationResultV5 returns information about a single relation,
+// or an error, but doesn't include the other application name.
+type RelationResultV5 struct {
 	Error    *Error                `json:"error,omitempty"`
 	Life     Life                  `json:"life"`
 	Id       int                   `json:"id"`
@@ -304,10 +321,10 @@ type RelationResult struct {
 	Endpoint multiwatcher.Endpoint `json:"endpoint"`
 }
 
-// RelationResults holds the result of an API call that returns
-// information about multiple relations.
-type RelationResults struct {
-	Results []RelationResult `json:"results"`
+// RelationResultsV5 holds the result of an API call that returns
+// information about multiple V5 relations.
+type RelationResultsV5 struct {
+	Results []RelationResultV5 `json:"results"`
 }
 
 // EntityCharmURL holds an entity's tag and a charm URL.
@@ -432,6 +449,23 @@ type VersionResults struct {
 	Results []VersionResult `json:"results"`
 }
 
+// SetModelEnvironVersions holds the tags and associated environ versions
+// of a collection of models.
+type SetModelEnvironVersions struct {
+	Models []SetModelEnvironVersion `json:"models,omitempty"`
+}
+
+// SetModelEnvironVersions holds the tag and associated environ version
+// of a model.
+type SetModelEnvironVersion struct {
+	// ModelTag is the string representation of a model tag, which
+	// should be parseable using names.ParseModelTag.
+	ModelTag string `json:"model-tag"`
+
+	// Version is the environ version to set for the model.
+	Version int `json:"version"`
+}
+
 // ToolsResult holds the tools and possibly error for a given
 // Tools() API call.
 type ToolsResult struct {
@@ -537,70 +571,6 @@ type RelationUnitsWatchResults struct {
 	Results []RelationUnitsWatchResult `json:"results"`
 }
 
-// ApplicationRelationsWatchResult holds a ApplicationRelationsWatcher id,
-// changes and an error (if any).
-type ApplicationRelationsWatchResult struct {
-	ApplicationRelationsWatcherId string
-	Changes                       *ApplicationRelationsChange `json:"changes,omitempty"`
-	Error                         *Error                      `json:"error,omitempty"`
-}
-
-// ApplicationRelationsWatchResults holds the results for any API call which ends
-// up returning a list of ApplicationRelationsWatchers.
-type ApplicationRelationsWatchResults struct {
-	Results []ApplicationRelationsWatchResult `json:"results"`
-}
-
-// ApplicationChange describes changes to a service.
-type ApplicationChange struct {
-	ApplicationTag string                     `json:"application-tag"`
-	Life           Life                       `json:"life"`
-	Relations      ApplicationRelationsChange `json:"relations"`
-}
-
-// ApplicationChanges describes a set of changes to services.
-type ApplicationChanges struct {
-	Changes []ApplicationChange `json:"changes,omitempty"`
-}
-
-// ApplicationRelationsChange describes changes to the relations that a service
-// is involved in.
-type ApplicationRelationsChange struct {
-	// ChangedRelations maps relation IDs to relation changes.
-	ChangedRelations []RelationChange `json:"changed,omitempty"`
-
-	// RemovedRelations contains the IDs of relations removed
-	// since the last change.
-	RemovedRelations []int `json:"removed,omitempty"`
-}
-
-// ApplicationRelationsChanges holds a set of ApplicationRelationsChange structures.
-type ApplicationRelationsChanges struct {
-	Changes []ApplicationRelationsChange `json:"changes,omitempty"`
-}
-
-// RelationChange describes changes to a relation.
-type RelationChange struct {
-	// RelationId is the numeric ID of the relation.
-	RelationId int `json:"id"`
-
-	// Life is the current lifecycle state of the relation.
-	Life Life `json:"life"`
-
-	// ChangedUnits maps unit names to relation unit changes.
-	ChangedUnits map[string]RelationUnitChange `json:"changedunits,omitempty"`
-
-	// DepartedUnits contains the names of units that have departed
-	// the relation since the last change.
-	DepartedUnits []string `json:"departedunits,omitempty"`
-}
-
-// RelationUnitChange describes a relation unit change.
-type RelationUnitChange struct {
-	// Settings is the current settings for the relation unit.
-	Settings map[string]interface{} `json:"settings,omitempty"`
-}
-
 // MachineStorageIdsWatchResult holds a MachineStorageIdsWatcher id,
 // changes and an error (if any).
 type MachineStorageIdsWatchResult struct {
@@ -786,3 +756,16 @@ type LogMessage struct {
 	Location  string    `json:"loc"`
 	Message   string    `json:"msg"`
 }
+
+// ResourceUploadResult is used to return some details about an
+// uploaded resource.
+type ResourceUploadResult struct {
+	// Error will contain details about a failed upload attempt.
+	Error *Error `json:"error,omitempty"`
+
+	// ID uniquely identifies a resource-application pair within the model.
+	ID string `json:"id"`
+
+	// Timestamp indicates when the resource was added to the model.
+	Timestamp time.Time `json:"timestamp"`
+}
