@@ -19,6 +19,17 @@ type Subnet struct {
 	// ProviderId is the provider-specific subnet ID (if applicable).
 	ProviderId string `json:"provider-id,omitempty"`
 
+	// ProviderNetworkId is the id of the network containing this
+	// subnet from the provider's perspective. It can be empty if the
+	// provider doesn't support distinct networks.
+	ProviderNetworkId string `json:"provider-network-id,omitempty"`
+
+	// ProviderSpaceId is the id of the space containing this subnet
+	// from the provider's perspective. It can be empty if the
+	// provider doesn't support spaces (in which case all subnets are
+	// effectively in the default space).
+	ProviderSpaceId string `json:"provider-space-id,omitempty"`
+
 	// VLANTag needs to be between 1 and 4094 for VLANs and 0 for
 	// normal networks. It's defined by IEEE 802.1Q standard.
 	VLANTag int `json:"vlan-tag"`
@@ -41,6 +52,17 @@ type Subnet struct {
 	Status string `json:"status,omitempty"`
 }
 
+// NetworkRoute describes a special route that should be added for a given
+// network interface.
+type NetworkRoute struct {
+	// DestinationCIDR is the Subnet CIDR of traffic that needs a custom route.
+	DestinationCIDR string `json:"destination-cidr"`
+	// GatewayIP is the target IP to use as the next-hop when sending traffic to DestinationCIDR
+	GatewayIP string `json:"gateway-ip"`
+	// Metric is the cost for this particular route.
+	Metric int `json:"metric"`
+}
+
 // NetworkConfig describes the necessary information to configure
 // a single network interface on a machine. This mostly duplicates
 // network.InterfaceInfo type and it's defined here so it can be kept
@@ -130,12 +152,17 @@ type NetworkConfig struct {
 	// configure for this network interface. For containers this
 	// usually (one of) the host address(es).
 	GatewayAddress string `json:"gateway-address,omitempty"`
+
+	// Routes is a list of routes that should be applied when this interface is
+	// active.
+	Routes []NetworkRoute `json:"routes,omitempty"`
 }
 
-// NetworkConfigs holds the network configuration for multiple networks
-type NetworkConfigs struct {
-	Results []NetworkConfig `json:"results"`
-	Errors  []error         `json:"errors,omitempty"`
+// DeviceBridgeInfo lists the host device and the expected bridge to be
+// created.
+type DeviceBridgeInfo struct {
+	HostDeviceName string `json:"host-device-name"`
+	BridgeName     string `json:"bridge-name"`
 }
 
 // ProviderInterfaceInfoResults holds the results of a
@@ -359,6 +386,7 @@ func NetworkHostsPorts(hpm [][]HostPort) [][]network.HostPort {
 	return nhpm
 }
 
+// TODO (wpk) Uniter.NetworkConfig API is obsolete, use NetworkInfo instead
 // UnitsNetworkConfig holds the parameters for calling Uniter.NetworkConfig()
 // API.
 type UnitsNetworkConfig struct {
@@ -443,7 +471,7 @@ type UnitNetworkConfigResult struct {
 	Config []NetworkConfig `json:"info"`
 }
 
-// UnitNetworkConfigResults holds network configuration for multiple machines.
+// UnitNetworkConfigResults holds network configuration for multiple units.
 type UnitNetworkConfigResults struct {
 	Results []UnitNetworkConfigResult `json:"results"`
 }
@@ -461,6 +489,25 @@ type MachineNetworkConfigResults struct {
 	Results []MachineNetworkConfigResult `json:"results"`
 }
 
+// HostNetworkChange holds the information about how a host machine should be
+// modified to prepare for a container.
+type HostNetworkChange struct {
+	Error *Error `json:"error,omitempty"`
+
+	// NewBridges lists the bridges that need to be created and what host
+	// device they should be connected to.
+	NewBridges []DeviceBridgeInfo `json:"new-bridges"`
+
+	// ReconfigureDelay is the duration in seconds to sleep before
+	// raising the bridged interface
+	ReconfigureDelay int `json:"reconfigure-delay"`
+}
+
+// HostNetworkChangeResults holds the network changes that are necessary for multiple containers to be created.
+type HostNetworkChangeResults struct {
+	Results []HostNetworkChange `json:"results"`
+}
+
 // MachinePortsParams holds the arguments for making a
 // FirewallerAPIV1.GetMachinePorts() API call.
 type MachinePortsParams struct {
@@ -542,10 +589,12 @@ type AddSubnetsParams struct {
 // SubnetProviderId must be set, but not both. Zones can be empty if
 // they can be discovered
 type AddSubnetParams struct {
-	SubnetTag        string   `json:"subnet-tag,omitempty"`
-	SubnetProviderId string   `json:"subnet-provider-id,omitempty"`
-	SpaceTag         string   `json:"space-tag"`
-	Zones            []string `json:"zones,omitempty"`
+	SubnetTag         string   `json:"subnet-tag,omitempty"`
+	SubnetProviderId  string   `json:"subnet-provider-id,omitempty"`
+	ProviderNetworkId string   `json:"provider-network-id,omitempty"`
+	SpaceTag          string   `json:"space-tag"`
+	VLANTag           int      `json:"vlan-tag,omitempty"`
+	Zones             []string `json:"zones,omitempty"`
 }
 
 // CreateSubnetsParams holds the arguments of CreateSubnets API call.
@@ -589,11 +638,6 @@ type Space struct {
 	Error   *Error   `json:"error,omitempty"`
 }
 
-// DiscoverSpacesResults holds the list of all provider spaces.
-type DiscoverSpacesResults struct {
-	Results []ProviderSpace `json:"results"`
-}
-
 // ProviderSpace holds the information about a single space and its associated subnets.
 type ProviderSpace struct {
 	Name       string   `json:"name"`
@@ -620,3 +664,40 @@ type ProxyConfigResult struct {
 type ProxyConfigResults struct {
 	Results []ProxyConfigResult `json:"results"`
 }
+
+// InterfaceAddress represents a single address attached to the interface.
+type InterfaceAddress struct {
+	Address string `json:"value"`
+	CIDR    string `json:"cidr"`
+}
+
+// NetworkInfo describes one interface with IP addresses.
+type NetworkInfo struct {
+	// MACAddress is the network interface's hardware MAC address
+	// (e.g. "aa:bb:cc:dd:ee:ff").
+	MACAddress string `json:"mac-address"`
+
+	// InterfaceName is the raw OS-specific network device name (e.g.
+	// "eth1", even for a VLAN eth1.42 virtual interface).
+	InterfaceName string `json:"interface-name"`
+
+	// Addresses contains a list of addresses configured on the interface.
+	Addresses []InterfaceAddress `json:"addresses"`
+}
+
+// NetworkInfoResult holds either and error or a list of NetworkInfos for given binding.
+type NetworkInfoResult struct {
+	Error *Error        `json:"error,omitempty" yaml:"error,omitempty"`
+	Info  []NetworkInfo `json:"network-info" yaml:"info"`
+}
+
+// NetworkInfoResults holds a mapping from binding name to NetworkInfoResult.
+type NetworkInfoResults struct {
+	Results map[string]NetworkInfoResult `json:"results"`
+}
+
+// NetworkInfoParams holds a name of the unit and list of bindings for which we want to get NetworkInfos.
+type NetworkInfoParams struct {
+	Unit     string   `json:"unit"`
+	Bindings []string `json:"bindings"`
+}
