@@ -21,6 +21,7 @@ import (
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/network"
+	"github.com/juju/juju/network/containerizer"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/state/stateenvirons"
 	"github.com/juju/juju/state/watcher"
@@ -31,10 +32,6 @@ import (
 
 var logger = loggo.GetLogger("juju.apiserver.provisioner")
 
-func init() {
-	common.RegisterStandardFacade("Provisioner", 3, NewProvisionerAPI)
-}
-
 // ProvisionerAPI provides access to the Provisioner API facade.
 type ProvisionerAPI struct {
 	*common.ControllerConfigAPI
@@ -51,6 +48,7 @@ type ProvisionerAPI struct {
 	*common.InstanceIdGetter
 	*common.ToolsFinder
 	*common.ToolsGetter
+	*networkingcommon.NetworkConfigAPI
 
 	st                      *state.State
 	resources               facade.Resources
@@ -59,15 +57,16 @@ type ProvisionerAPI struct {
 	storagePoolManager      poolmanager.PoolManager
 	configGetter            environs.EnvironConfigGetter
 	getAuthFunc             common.GetAuthFunc
+	getCanModify            common.GetAuthFunc
 }
 
 // NewProvisionerAPI creates a new server-side ProvisionerAPI facade.
 func NewProvisionerAPI(st *state.State, resources facade.Resources, authorizer facade.Authorizer) (*ProvisionerAPI, error) {
-	if !authorizer.AuthMachineAgent() && !authorizer.AuthModelManager() {
+	if !authorizer.AuthMachineAgent() && !authorizer.AuthController() {
 		return nil, common.ErrPerm
 	}
 	getAuthFunc := func() (common.AuthFunc, error) {
-		isModelManager := authorizer.AuthModelManager()
+		isModelManager := authorizer.AuthController()
 		isMachineAgent := authorizer.AuthMachineAgent()
 		authEntityTag := authorizer.GetAuthTag()
 
@@ -95,6 +94,9 @@ func NewProvisionerAPI(st *state.State, resources facade.Resources, authorizer f
 			}
 		}, nil
 	}
+	getCanModify := func() (common.AuthFunc, error) {
+		return authorizer.AuthOwner, nil
+	}
 	getAuthOwner := func() (common.AuthFunc, error) {
 		return authorizer.AuthOwner, nil
 	}
@@ -124,6 +126,7 @@ func NewProvisionerAPI(st *state.State, resources facade.Resources, authorizer f
 		InstanceIdGetter:        common.NewInstanceIdGetter(st, getAuthFunc),
 		ToolsFinder:             common.NewToolsFinder(configGetter, st, urlGetter),
 		ToolsGetter:             common.NewToolsGetter(st, configGetter, st, urlGetter, getAuthOwner),
+		NetworkConfigAPI:        networkingcommon.NewNetworkConfigAPI(st, getCanModify),
 		st:                      st,
 		resources:               resources,
 		authorizer:              authorizer,
@@ -131,6 +134,7 @@ func NewProvisionerAPI(st *state.State, resources facade.Resources, authorizer f
 		storageProviderRegistry: storageProviderRegistry,
 		storagePoolManager:      poolmanager.New(state.NewStateSettings(st), storageProviderRegistry),
 		getAuthFunc:             getAuthFunc,
+		getCanModify:            getCanModify,
 	}, nil
 }
 
@@ -213,6 +217,7 @@ func (p *ProvisionerAPI) SetSupportedContainers(args params.MachineContainersPar
 	for i, arg := range args.Params {
 		tag, err := names.ParseMachineTag(arg.MachineTag)
 		if err != nil {
+			logger.Warningf("SetSupportedContainers called with %q which is not a valid machine tag: %v", arg.MachineTag, err)
 			result.Results[i].Error = common.ServerError(common.ErrPerm)
 			continue
 		}
@@ -298,14 +303,14 @@ func (p *ProvisionerAPI) MachinesWithTransientErrors() (params.StatusResults, er
 			continue
 		}
 		var result params.StatusResult
-		statusInfo, err := machine.Status()
+		statusInfo, err := machine.InstanceStatus()
 		if err != nil {
 			continue
 		}
 		result.Status = statusInfo.Status.String()
 		result.Info = statusInfo.Message
 		result.Data = statusInfo.Data
-		if statusInfo.Status != status.Error {
+		if statusInfo.Status != status.Error && statusInfo.Status != status.ProvisioningError {
 			continue
 		}
 		// Transient errors are marked as such in the status data.
@@ -343,6 +348,65 @@ func (p *ProvisionerAPI) Series(args params.Entities) (params.StringResults, err
 	return result, nil
 }
 
+// AvailabilityZone returns a provider-specific availability zone for each given machine entity
+func (p *ProvisionerAPI) AvailabilityZone(args params.Entities) (params.StringResults, error) {
+	result := params.StringResults{
+		Results: make([]params.StringResult, len(args.Entities)),
+	}
+	canAccess, err := p.getAuthFunc()
+	if err != nil {
+		return result, err
+	}
+	for i, entity := range args.Entities {
+		tag, err := names.ParseMachineTag(entity.Tag)
+		if err != nil {
+			result.Results[i].Error = common.ServerError(common.ErrPerm)
+			continue
+		}
+		machine, err := p.getMachine(canAccess, tag)
+		if err == nil {
+			hc, err := machine.HardwareCharacteristics()
+			if err == nil {
+				if hc.AvailabilityZone != nil {
+					result.Results[i].Result = *hc.AvailabilityZone
+				} else {
+					result.Results[i].Result = ""
+				}
+			} else {
+				result.Results[i].Error = common.ServerError(err)
+			}
+		}
+	}
+	return result, nil
+}
+
+// KeepInstance returns the keep-instance value for each given machine entity.
+func (p *ProvisionerAPI) KeepInstance(args params.Entities) (params.BoolResults, error) {
+	result := params.BoolResults{
+
+		Results: make([]params.BoolResult, len(args.Entities)),
+	}
+	canAccess, err := p.getAuthFunc()
+	if err != nil {
+		return result, err
+	}
+	for i, entity := range args.Entities {
+		tag, err := names.ParseMachineTag(entity.Tag)
+		if err != nil {
+			result.Results[i].Error = common.ServerError(common.ErrPerm)
+			continue
+		}
+		machine, err := p.getMachine(canAccess, tag)
+		if err == nil {
+			keep, err := machine.KeepInstance()
+			result.Results[i].Result = keep
+			result.Results[i].Error = common.ServerError(err)
+		}
+		result.Results[i].Error = common.ServerError(err)
+	}
+	return result, nil
+}
+
 // DistributionGroup returns, for each given machine entity,
 // a slice of instance.Ids that belong to the same distribution
 // group as that machine. This information may be used to
@@ -508,7 +572,7 @@ func (p *ProvisionerAPI) SetInstanceInfo(args params.InstancesInfo) (params.Erro
 // the provisioner should retry provisioning machines with transient errors.
 func (p *ProvisionerAPI) WatchMachineErrorRetry() (params.NotifyWatchResult, error) {
 	result := params.NotifyWatchResult{}
-	if !p.authorizer.AuthModelManager() {
+	if !p.authorizer.AuthController() {
 		return result, common.ErrPerm
 	}
 	watch := newWatchMachineErrorRetry()
@@ -588,27 +652,42 @@ func (p *ProvisionerAPI) GetContainerInterfaceInfo(args params.Entities) (
 	return p.prepareOrGetContainerInterfaceInfo(args, true)
 }
 
-func (p *ProvisionerAPI) prepareOrGetContainerInterfaceInfo(args params.Entities, maintain bool) (params.MachineNetworkConfigResults, error) {
-	result := params.MachineNetworkConfigResults{
-		Results: make([]params.MachineNetworkConfigResult, len(args.Entities)),
-	}
+// perContainerHandler is the interface we need to trigger processing on
+// every container passed in as a list of things to process.
+type perContainerHandler interface {
+	// ProcessOneContainer is called once we've assured ourselves that all of
+	// the access permissions are correct and the container is ready to be
+	// processed.
+	// env is the Environment you are working, idx is the index for this
+	// container (used for deciding where to store results), host is the
+	// machine that is hosting the container.
+	// Any errors that are returned from ProcessOneContainer will be turned
+	// into ServerError and handed to SetError
+	ProcessOneContainer(env environs.Environ, idx int, host, container *state.Machine) error
+	// SetError will be called whenever there is a problem with the a given
+	// request. Generally this just does result.Results[i].Error = error
+	// but the Result type is opaque so we can't do it ourselves.
+	SetError(resultIndex int, err *params.Error)
+}
 
-	netEnviron, hostMachine, canAccess, err := p.prepareContainerAccessEnvironment()
+func (p *ProvisionerAPI) processEachContainer(args params.Entities, handler perContainerHandler) error {
+	env, hostMachine, canAccess, err := p.prepareContainerAccessEnvironment()
 	if err != nil {
-		return result, errors.Trace(err)
+		// Overall error
+		return errors.Trace(err)
 	}
-	instId, err := hostMachine.InstanceId()
+	_, err = hostMachine.InstanceId()
 	if errors.IsNotProvisioned(err) {
 		err = errors.NotProvisionedf("cannot prepare container network config: host machine %q", hostMachine)
-		return result, err
+		return err
 	} else if err != nil {
-		return result, errors.Trace(err)
+		return errors.Trace(err)
 	}
 
 	for i, entity := range args.Entities {
 		machineTag, err := names.ParseMachineTag(entity.Tag)
 		if err != nil {
-			result.Results[i].Error = common.ServerError(err)
+			handler.SetError(i, common.ServerError(err))
 			continue
 		}
 		// The auth function (canAccess) checks that the machine is a
@@ -616,120 +695,173 @@ func (p *ProvisionerAPI) prepareOrGetContainerInterfaceInfo(args params.Entities
 		// machine has the host as a parent.
 		container, err := p.getMachine(canAccess, machineTag)
 		if err != nil {
-			result.Results[i].Error = common.ServerError(err)
+			handler.SetError(i, common.ServerError(err))
 			continue
 		} else if !container.IsContainer() {
 			err = errors.Errorf("cannot prepare network config for %q: not a container", machineTag)
-			result.Results[i].Error = common.ServerError(err)
+			handler.SetError(i, common.ServerError(err))
 			continue
-		} else if ciid, cerr := container.InstanceId(); maintain == true && cerr == nil {
+		}
+
+		if err := handler.ProcessOneContainer(env, i, hostMachine, container); err != nil {
+			handler.SetError(i, common.ServerError(err))
+			continue
+		}
+	}
+	return nil
+}
+
+type prepareOrGetContext struct {
+	result   params.MachineNetworkConfigResults
+	maintain bool
+}
+
+func (ctx *prepareOrGetContext) SetError(idx int, err *params.Error) {
+	ctx.result.Results[idx].Error = err
+}
+
+func (ctx *prepareOrGetContext) ProcessOneContainer(env environs.Environ, idx int, host, container *state.Machine) error {
+	containerId, err := container.InstanceId()
+	if ctx.maintain {
+		if err == nil {
 			// Since we want to configure and create NICs on the
 			// container before it starts, it must also be not
 			// provisioned yet.
-			err = errors.Errorf("container %q already provisioned as %q", container, ciid)
-			result.Results[i].Error = common.ServerError(err)
-			continue
-		} else if cerr != nil && !errors.IsNotProvisioned(cerr) {
-			// Any other error needs to be reported.
-			result.Results[i].Error = common.ServerError(cerr)
-			continue
+			return errors.Errorf("container %q already provisioned as %q", container, containerId)
 		}
+	}
+	// The only error we allow is NotProvisioned
+	if err != nil && !errors.IsNotProvisioned(err) {
+		return err
+	}
 
-		if err := hostMachine.SetContainerLinkLayerDevices(container); err != nil {
-			result.Results[i].Error = common.ServerError(err)
-			continue
+	supportContainerAddresses := environs.SupportsContainerAddresses(env)
+	bridgePolicy := containerizer.BridgePolicy{
+		NetBondReconfigureDelay: env.Config().NetBondReconfigureDelay(),
+		UseLocalBridges:         !supportContainerAddresses,
+	}
+
+	// TODO(jam): 2017-01-31 PopulateContainerLinkLayerDevices should really
+	// just be returning the ones we'd like to exist, and then we turn those
+	// into things we'd like to tell the Host machine to create, and then *it*
+	// reports back what actually exists when its done.
+	if err := bridgePolicy.PopulateContainerLinkLayerDevices(host, container); err != nil {
+		return err
+	}
+
+	containerDevices, err := container.AllLinkLayerDevices()
+	if err != nil {
+		return err
+	}
+
+	preparedInfo := make([]network.InterfaceInfo, len(containerDevices))
+	for j, device := range containerDevices {
+		parentDevice, err := device.ParentDevice()
+		if err != nil || parentDevice == nil {
+			return errors.Errorf(
+				"cannot get parent %q of container device %q: %v",
+				device.ParentName(), device.Name(), err,
+			)
 		}
-
-		containerDevices, err := container.AllLinkLayerDevices()
+		parentAddrs, err := parentDevice.Addresses()
 		if err != nil {
-			result.Results[i].Error = common.ServerError(err)
-			continue
+			return err
 		}
 
-		preparedInfo := make([]network.InterfaceInfo, len(containerDevices))
-		preparedOK := true
-		for j, device := range containerDevices {
-			parentDevice, err := device.ParentDevice()
-			if err != nil || parentDevice == nil {
-				err = errors.Errorf(
-					"cannot get parent %q of container device %q: %v",
-					device.ParentName(), device.Name(), err,
-				)
-				result.Results[i].Error = common.ServerError(err)
-				preparedOK = false
-				break
-			}
-			parentAddrs, err := parentDevice.Addresses()
-			if err != nil {
-				result.Results[i].Error = common.ServerError(err)
-				preparedOK = false
-				break
-			}
+		info := network.InterfaceInfo{
+			InterfaceName:       device.Name(),
+			MACAddress:          device.MACAddress(),
+			ConfigType:          network.ConfigManual,
+			InterfaceType:       network.InterfaceType(device.Type()),
+			NoAutoStart:         !device.IsAutoStart(),
+			Disabled:            !device.IsUp(),
+			MTU:                 int(device.MTU()),
+			ParentInterfaceName: parentDevice.Name(),
+		}
 
-			info := network.InterfaceInfo{
-				InterfaceName:       device.Name(),
-				MACAddress:          device.MACAddress(),
-				ConfigType:          network.ConfigManual,
-				InterfaceType:       network.InterfaceType(device.Type()),
-				NoAutoStart:         !device.IsAutoStart(),
-				Disabled:            !device.IsUp(),
-				MTU:                 int(device.MTU()),
-				ParentInterfaceName: parentDevice.Name(),
-			}
-
-			if len(parentAddrs) > 0 {
-				logger.Infof("host machine device %q has addresses %v", parentDevice.Name(), parentAddrs)
-
-				firstAddress := parentAddrs[0]
+		if len(parentAddrs) > 0 {
+			logger.Debugf("host machine device %q has addresses %v", parentDevice.Name(), parentAddrs)
+			firstAddress := parentAddrs[0]
+			if supportContainerAddresses {
 				parentDeviceSubnet, err := firstAddress.Subnet()
 				if err != nil {
-					err = errors.Annotatef(err,
+					return errors.Annotatef(err,
 						"cannot get subnet %q used by address %q of host machine device %q",
 						firstAddress.SubnetCIDR(), firstAddress.Value(), parentDevice.Name(),
 					)
-					result.Results[i].Error = common.ServerError(err)
-					preparedOK = false
-					break
 				}
 				info.ConfigType = network.ConfigStatic
 				info.CIDR = parentDeviceSubnet.CIDR()
 				info.ProviderSubnetId = parentDeviceSubnet.ProviderId()
 				info.VLANTag = parentDeviceSubnet.VLANTag()
 			} else {
-				logger.Infof("host machine device %q has no addresses %v", parentDevice.Name(), parentAddrs)
+				info.ConfigType = network.ConfigDHCP
+				info.CIDR = firstAddress.SubnetCIDR()
+				info.ProviderSubnetId = ""
+				info.VLANTag = 0
 			}
-
-			logger.Tracef("prepared info for container interface %q: %+v", info.InterfaceName, info)
-			preparedOK = true
-			preparedInfo[j] = info
+		} else {
+			logger.Infof("host machine device %q has no addresses %v", parentDevice.Name(), parentAddrs)
+			// TODO(jam): 2017-02-15, have a concrete test for this case, as it
+			// seems to be the common case in the wild.
+			info.ConfigType = network.ConfigDHCP
+			info.ProviderSubnetId = ""
+			info.VLANTag = 0
 		}
 
-		if !preparedOK {
-			// Error result is already set.
-			continue
-		}
+		logger.Tracef("prepared info for container interface %q: %+v", info.InterfaceName, info)
+		preparedInfo[j] = info
+	}
 
-		allocatedInfo, err := netEnviron.AllocateContainerAddresses(instId, machineTag, preparedInfo)
+	hostInstanceId, err := host.InstanceId()
+	if err != nil {
+		// this should have already been checked in the processEachContainer helper
+		return err
+	}
+	allocatedInfo := preparedInfo
+	if supportContainerAddresses {
+		// supportContainerAddresses already checks that we can cast to an environ.Networking
+		networking := env.(environs.Networking)
+		allocatedInfo, err = networking.AllocateContainerAddresses(hostInstanceId, container.MachineTag(), preparedInfo)
 		if err != nil {
-			result.Results[i].Error = common.ServerError(err)
-			continue
+			return err
 		}
 		logger.Debugf("got allocated info from provider: %+v", allocatedInfo)
-
-		allocatedConfig := networkingcommon.NetworkConfigFromInterfaceInfo(allocatedInfo)
-		logger.Tracef("allocated network config: %+v", allocatedConfig)
-		result.Results[i].Config = allocatedConfig
+	} else {
+		logger.Debugf("using dhcp allocated addresses")
 	}
-	return result, nil
+
+	allocatedConfig := networkingcommon.NetworkConfigFromInterfaceInfo(allocatedInfo)
+	logger.Debugf("allocated network config: %+v", allocatedConfig)
+	ctx.result.Results[idx].Config = allocatedConfig
+	return nil
+}
+
+func (p *ProvisionerAPI) prepareOrGetContainerInterfaceInfo(args params.Entities, maintain bool) (params.MachineNetworkConfigResults, error) {
+	ctx := &prepareOrGetContext{
+		result: params.MachineNetworkConfigResults{
+			Results: make([]params.MachineNetworkConfigResult, len(args.Entities)),
+		},
+		maintain: maintain,
+	}
+
+	if err := p.processEachContainer(args, ctx); err != nil {
+		return ctx.result, errors.Trace(err)
+	}
+	return ctx.result, nil
 }
 
 // prepareContainerAccessEnvironment retrieves the environment, host machine, and access
 // for working with containers.
-func (p *ProvisionerAPI) prepareContainerAccessEnvironment() (environs.NetworkingEnviron, *state.Machine, common.AuthFunc, error) {
-	netEnviron, err := networkingcommon.NetworkingEnvironFromModelConfig(p.configGetter)
+func (p *ProvisionerAPI) prepareContainerAccessEnvironment() (environs.Environ, *state.Machine, common.AuthFunc, error) {
+	env, err := environs.GetEnviron(p.configGetter, environs.New)
 	if err != nil {
 		return nil, nil, nil, errors.Trace(err)
 	}
+	// TODO(jam): 2017-02-01 NetworkingEnvironFromModelConfig used to do this, but it doesn't feel good
+	if env.Config().Type() == "dummy" {
+		return nil, nil, nil, errors.NotSupportedf("dummy provider network config")
+	}
 
 	canAccess, err := p.getAuthFunc()
 	if err != nil {
@@ -747,7 +879,52 @@ func (p *ProvisionerAPI) prepareContainerAccessEnvironment() (environs.Networkin
 	if err != nil {
 		return nil, nil, nil, errors.Trace(err)
 	}
-	return netEnviron, host, canAccess, nil
+	return env, host, canAccess, nil
+}
+
+type hostChangesContext struct {
+	result params.HostNetworkChangeResults
+}
+
+func (ctx *hostChangesContext) ProcessOneContainer(env environs.Environ, idx int, host, container *state.Machine) error {
+	bridgePolicy := containerizer.BridgePolicy{
+		NetBondReconfigureDelay: env.Config().NetBondReconfigureDelay(),
+		UseLocalBridges:         !environs.SupportsContainerAddresses(env),
+	}
+	bridges, reconfigureDelay, err := bridgePolicy.FindMissingBridgesForContainer(host, container)
+	if err != nil {
+		return err
+	}
+
+	ctx.result.Results[idx].ReconfigureDelay = reconfigureDelay
+	for _, bridgeInfo := range bridges {
+		ctx.result.Results[idx].NewBridges = append(
+			ctx.result.Results[idx].NewBridges,
+			params.DeviceBridgeInfo{
+				HostDeviceName: bridgeInfo.DeviceName,
+				BridgeName:     bridgeInfo.BridgeName,
+			})
+	}
+	return nil
+}
+
+func (ctx *hostChangesContext) SetError(idx int, err *params.Error) {
+	ctx.result.Results[idx].Error = err
+}
+
+// HostChangesForContainers returns the set of changes that need to be done
+// to the host machine to prepare it for the containers to be created.
+// Pass in a list of the containers that you want the changes for.
+func (p *ProvisionerAPI) HostChangesForContainers(args params.Entities) (params.HostNetworkChangeResults, error) {
+	ctx := &hostChangesContext{
+		result: params.HostNetworkChangeResults{
+			Results: make([]params.HostNetworkChange, len(args.Entities)),
+		},
+	}
+	if err := p.processEachContainer(args, ctx); err != nil {
+		return ctx.result, errors.Trace(err)
+	}
+	return ctx.result, nil
 }
 
 // InstanceStatus returns the instance status for each given entity.
@@ -764,7 +941,8 @@ func (p *ProvisionerAPI) InstanceStatus(args params.Entities) (params.StatusResu
 	for i, arg := range args.Entities {
 		mTag, err := names.ParseMachineTag(arg.Tag)
 		if err != nil {
-			result.Results[i].Error = common.ServerError(err)
+			logger.Warningf("InstanceStatus called with %q which is not a valid machine tag: %v", arg.Tag, err)
+			result.Results[i].Error = common.ServerError(common.ErrPerm)
 			continue
 		}
 		machine, err := p.getMachine(canAccess, mTag)
@@ -781,6 +959,44 @@ func (p *ProvisionerAPI) InstanceStatus(args params.Entities) (params.StatusResu
 	return result, nil
 }
 
+func (p *ProvisionerAPI) setOneInstanceStatus(canAccess common.AuthFunc, arg params.EntityStatusArgs) error {
+	logger.Debugf("SetInstanceStatus called with: %#v", arg)
+	mTag, err := names.ParseMachineTag(arg.Tag)
+	if err != nil {
+		logger.Warningf("SetInstanceStatus called with %q which is not a valid machine tag: %v", arg.Tag, err)
+		return common.ErrPerm
+	}
+	machine, err := p.getMachine(canAccess, mTag)
+	if err != nil {
+		logger.Debugf("SetInstanceStatus unable to get machine %q", mTag)
+		return err
+	}
+	// TODO(perrito666) 2016-05-02 lp:1558657
+	now := time.Now()
+	s := status.StatusInfo{
+		Status:  status.Status(arg.Status),
+		Message: arg.Info,
+		Data:    arg.Data,
+		Since:   &now,
+	}
+
+	// TODO(jam): 2017-01-29 These two status should be set in a single
+	//	transaction, not in two separate transactions. Otherwise you can see
+	//	one toggle, but not the other.
+	if err = machine.SetInstanceStatus(s); err != nil {
+		logger.Debugf("failed to SetInstanceStatus for %q: %v", mTag, err)
+		return err
+	}
+	if status.Status(arg.Status) == status.ProvisioningError {
+		s.Status = status.Error
+		logger.Debugf("SetInstanceStatus triggering SetStatus for %#v", s)
+		if err = machine.SetStatus(s); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
 // SetInstanceStatus updates the instance status for each given
 // entity. Only machine tags are accepted.
 func (p *ProvisionerAPI) SetInstanceStatus(args params.SetStatus) (params.ErrorResults, error) {
@@ -793,29 +1009,7 @@ func (p *ProvisionerAPI) SetInstanceStatus(args params.SetStatus) (params.ErrorR
 		return result, errors.Trace(err)
 	}
 	for i, arg := range args.Entities {
-		mTag, err := names.ParseMachineTag(arg.Tag)
-		if err != nil {
-			result.Results[i].Error = common.ServerError(err)
-			continue
-		}
-		machine, err := p.getMachine(canAccess, mTag)
-		if err == nil {
-			// TODO(perrito666) 2016-05-02 lp:1558657
-			now := time.Now()
-			s := status.StatusInfo{
-				Status:  status.Status(arg.Status),
-				Message: arg.Info,
-				Data:    arg.Data,
-				Since:   &now,
-			}
-			err = machine.SetInstanceStatus(s)
-			if status.Status(arg.Status) == status.ProvisioningError {
-				s.Status = status.Error
-				if err == nil {
-					err = machine.SetStatus(s)
-				}
-			}
-		}
+		err = p.setOneInstanceStatus(canAccess, arg)
 		result.Results[i].Error = common.ServerError(err)
 	}
 	return result, nil
@@ -848,3 +1042,7 @@ func (p *ProvisionerAPI) markOneMachineForRemoval(machineTag string, canAccess c
 	}
 	return machine.MarkForRemoval()
 }
+
+func (p *ProvisionerAPI) SetHostMachineNetworkConfig(args params.SetMachineNetworkConfig) error {
+	return p.SetObservedNetworkConfig(args)
+}
