@@ -30,6 +30,7 @@ import (
 	"github.com/juju/juju/status"
 	"github.com/juju/juju/storage/poolmanager"
 	coretesting "github.com/juju/juju/testing"
+	"github.com/juju/juju/testing/factory"
 )
 
 func TestPackage(t *stdtesting.T) {
@@ -75,7 +76,7 @@ func (s *provisionerSuite) setUpTest(c *gc.C, withController bool) {
 	// Create a FakeAuthorizer so we can check permissions,
 	// set up assuming we logged in as the environment manager.
 	s.authorizer = apiservertesting.FakeAuthorizer{
-		EnvironManager: true,
+		Controller: true,
 	}
 
 	// Create the resource registry separately to track invocations to
@@ -106,14 +107,14 @@ func (s *withoutControllerSuite) SetUpTest(c *gc.C) {
 
 func (s *withoutControllerSuite) TestProvisionerFailsWithNonMachineAgentNonManagerUser(c *gc.C) {
 	anAuthorizer := s.authorizer
-	anAuthorizer.EnvironManager = true
+	anAuthorizer.Controller = true
 	// Works with an environment manager, which is not a machine agent.
 	aProvisioner, err := provisioner.NewProvisionerAPI(s.State, s.resources, anAuthorizer)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(aProvisioner, gc.NotNil)
 
 	// But fails with neither a machine agent or an environment manager.
-	anAuthorizer.EnvironManager = false
+	anAuthorizer.Controller = false
 	aProvisioner, err = provisioner.NewProvisionerAPI(s.State, s.resources, anAuthorizer)
 	c.Assert(err, gc.NotNil)
 	c.Assert(aProvisioner, gc.IsNil)
@@ -183,7 +184,7 @@ func (s *withoutControllerSuite) TestLifeAsMachineAgent(c *gc.C) {
 
 	// Login as a machine agent for machine 0.
 	anAuthorizer := s.authorizer
-	anAuthorizer.EnvironManager = false
+	anAuthorizer.Controller = false
 	anAuthorizer.Tag = s.machines[0].Tag()
 	aProvisioner, err := provisioner.NewProvisionerAPI(s.State, s.resources, anAuthorizer)
 	c.Assert(err, jc.ErrorIsNil)
@@ -371,37 +372,92 @@ func (s *withoutControllerSuite) TestSetStatus(c *gc.C) {
 	s.assertStatus(c, 2, status.Started, "again", map[string]interface{}{})
 }
 
-func (s *withoutControllerSuite) TestMachinesWithTransientErrors(c *gc.C) {
+func (s *withoutControllerSuite) TestSetInstanceStatus(c *gc.C) {
 	now := time.Now()
 	sInfo := status.StatusInfo{
-		Status:  status.Started,
+		Status:  status.Provisioning,
 		Message: "blah",
 		Since:   &now,
 	}
-	err := s.machines[0].SetStatus(sInfo)
+	err := s.machines[0].SetInstanceStatus(sInfo)
+	c.Assert(err, jc.ErrorIsNil)
+	sInfo = status.StatusInfo{
+		Status:  status.Running,
+		Message: "foo",
+		Since:   &now,
+	}
+	err = s.machines[1].SetInstanceStatus(sInfo)
 	c.Assert(err, jc.ErrorIsNil)
 	sInfo = status.StatusInfo{
 		Status:  status.Error,
+		Message: "not really",
+		Since:   &now,
+	}
+	err = s.machines[2].SetInstanceStatus(sInfo)
+	c.Assert(err, jc.ErrorIsNil)
+
+	args := params.SetStatus{
+		Entities: []params.EntityStatusArgs{
+			{Tag: s.machines[0].Tag().String(), Status: status.Provisioning.String(), Info: "not really",
+				Data: map[string]interface{}{"foo": "bar"}},
+			{Tag: s.machines[1].Tag().String(), Status: status.Running.String(), Info: "foobar"},
+			{Tag: s.machines[2].Tag().String(), Status: status.ProvisioningError.String(), Info: "again"},
+			{Tag: "machine-42", Status: status.Provisioning.String(), Info: "blah"},
+			{Tag: "unit-foo-0", Status: status.Error.String(), Info: "foobar"},
+			{Tag: "application-bar", Status: status.ProvisioningError.String(), Info: "foobar"},
+		}}
+	result, err := s.provisioner.SetInstanceStatus(args)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(result, gc.DeepEquals, params.ErrorResults{
+		Results: []params.ErrorResult{
+			{nil},
+			{nil},
+			{nil},
+			{apiservertesting.NotFoundError("machine 42")},
+			{apiservertesting.ErrUnauthorized},
+			{apiservertesting.ErrUnauthorized},
+		},
+	})
+
+	// Verify the changes.
+	s.assertInstanceStatus(c, 0, status.Provisioning, "not really", map[string]interface{}{"foo": "bar"})
+	s.assertInstanceStatus(c, 1, status.Running, "foobar", map[string]interface{}{})
+	s.assertInstanceStatus(c, 2, status.ProvisioningError, "again", map[string]interface{}{})
+	// ProvisioningError also has a special case which is to set the machine to Error
+	s.assertStatus(c, 2, status.Error, "again", map[string]interface{}{})
+}
+
+func (s *withoutControllerSuite) TestMachinesWithTransientErrors(c *gc.C) {
+	now := time.Now()
+	sInfo := status.StatusInfo{
+		Status:  status.Provisioning,
+		Message: "blah",
+		Since:   &now,
+	}
+	err := s.machines[0].SetInstanceStatus(sInfo)
+	c.Assert(err, jc.ErrorIsNil)
+	sInfo = status.StatusInfo{
+		Status:  status.ProvisioningError,
 		Message: "transient error",
 		Data:    map[string]interface{}{"transient": true, "foo": "bar"},
 		Since:   &now,
 	}
-	err = s.machines[1].SetStatus(sInfo)
+	err = s.machines[1].SetInstanceStatus(sInfo)
 	c.Assert(err, jc.ErrorIsNil)
 	sInfo = status.StatusInfo{
-		Status:  status.Error,
+		Status:  status.ProvisioningError,
 		Message: "error",
 		Data:    map[string]interface{}{"transient": false},
 		Since:   &now,
 	}
-	err = s.machines[2].SetStatus(sInfo)
+	err = s.machines[2].SetInstanceStatus(sInfo)
 	c.Assert(err, jc.ErrorIsNil)
 	sInfo = status.StatusInfo{
 		Status:  status.Error,
 		Message: "error",
 		Since:   &now,
 	}
-	err = s.machines[3].SetStatus(sInfo)
+	err = s.machines[3].SetInstanceStatus(sInfo)
 	c.Assert(err, jc.ErrorIsNil)
 	// Machine 4 is provisioned but error not reset yet.
 	sInfo = status.StatusInfo{
@@ -410,7 +466,7 @@ func (s *withoutControllerSuite) TestMachinesWithTransientErrors(c *gc.C) {
 		Data:    map[string]interface{}{"transient": true, "foo": "bar"},
 		Since:   &now,
 	}
-	err = s.machines[4].SetStatus(sInfo)
+	err = s.machines[4].SetInstanceStatus(sInfo)
 	c.Assert(err, jc.ErrorIsNil)
 	hwChars := instance.MustParseHardware("arch=i386", "mem=4G")
 	err = s.machines[4].SetProvisioned("i-am", "fake_nonce", &hwChars)
@@ -420,7 +476,7 @@ func (s *withoutControllerSuite) TestMachinesWithTransientErrors(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(result, gc.DeepEquals, params.StatusResults{
 		Results: []params.StatusResult{
-			{Id: "1", Life: "alive", Status: "error", Info: "transient error",
+			{Id: "1", Life: "alive", Status: "provisioning error", Info: "transient error",
 				Data: map[string]interface{}{"transient": true, "foo": "bar"}},
 		},
 	})
@@ -429,48 +485,50 @@ func (s *withoutControllerSuite) TestMachinesWithTransientErrors(c *gc.C) {
 func (s *withoutControllerSuite) TestMachinesWithTransientErrorsPermission(c *gc.C) {
 	// Machines where there's permission issues are omitted.
 	anAuthorizer := s.authorizer
-	anAuthorizer.EnvironManager = false
+	anAuthorizer.Controller = false
 	anAuthorizer.Tag = names.NewMachineTag("1")
 	aProvisioner, err := provisioner.NewProvisionerAPI(s.State, s.resources,
 		anAuthorizer)
 	now := time.Now()
 	sInfo := status.StatusInfo{
-		Status:  status.Started,
+		Status:  status.Running,
 		Message: "blah",
 		Since:   &now,
 	}
-	err = s.machines[0].SetStatus(sInfo)
+	err = s.machines[0].SetInstanceStatus(sInfo)
 	c.Assert(err, jc.ErrorIsNil)
 	sInfo = status.StatusInfo{
-		Status:  status.Error,
+		Status:  status.ProvisioningError,
 		Message: "transient error",
 		Data:    map[string]interface{}{"transient": true, "foo": "bar"},
 		Since:   &now,
 	}
-	err = s.machines[1].SetStatus(sInfo)
+	err = s.machines[1].SetInstanceStatus(sInfo)
 	c.Assert(err, jc.ErrorIsNil)
 	sInfo = status.StatusInfo{
-		Status:  status.Error,
+		Status:  status.ProvisioningError,
 		Message: "error",
 		Data:    map[string]interface{}{"transient": false},
 		Since:   &now,
 	}
-	err = s.machines[2].SetStatus(sInfo)
+	err = s.machines[2].SetInstanceStatus(sInfo)
 	c.Assert(err, jc.ErrorIsNil)
 	sInfo = status.StatusInfo{
-		Status:  status.Error,
+		Status:  status.ProvisioningError,
 		Message: "error",
 		Since:   &now,
 	}
-	err = s.machines[3].SetStatus(sInfo)
+	err = s.machines[3].SetInstanceStatus(sInfo)
 	c.Assert(err, jc.ErrorIsNil)
 
 	result, err := aProvisioner.MachinesWithTransientErrors()
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(result, gc.DeepEquals, params.StatusResults{
-		Results: []params.StatusResult{
-			{Id: "1", Life: "alive", Status: "error", Info: "transient error",
-				Data: map[string]interface{}{"transient": true, "foo": "bar"}},
+		Results: []params.StatusResult{{
+			Id: "1", Life: "alive", Status: "provisioning error",
+			Info: "transient error",
+			Data: map[string]interface{}{"transient": true, "foo": "bar"},
+		},
 		},
 	})
 }
@@ -525,6 +583,16 @@ func (s *withoutControllerSuite) assertStatus(c *gc.C, index int, expectStatus s
 	c.Assert(statusInfo.Data, gc.DeepEquals, expectData)
 }
 
+func (s *withoutControllerSuite) assertInstanceStatus(c *gc.C, index int, expectStatus status.Status, expectInfo string,
+	expectData map[string]interface{}) {
+
+	statusInfo, err := s.machines[index].InstanceStatus()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(statusInfo.Status, gc.Equals, expectStatus)
+	c.Assert(statusInfo.Message, gc.Equals, expectInfo)
+	c.Assert(statusInfo.Data, gc.DeepEquals, expectData)
+}
+
 func (s *withoutControllerSuite) TestWatchContainers(c *gc.C) {
 	c.Assert(s.resources.Count(), gc.Equals, 0)
 
@@ -604,7 +672,7 @@ func (s *withoutControllerSuite) TestModelConfigNonManager(c *gc.C) {
 	// the secret attributes are masked.
 	anAuthorizer := s.authorizer
 	anAuthorizer.Tag = names.NewMachineTag("1")
-	anAuthorizer.EnvironManager = false
+	anAuthorizer.Controller = false
 	aProvisioner, err := provisioner.NewProvisionerAPI(s.State, s.resources,
 		anAuthorizer)
 	c.Assert(err, jc.ErrorIsNil)
@@ -667,11 +735,65 @@ func (s *withoutControllerSuite) TestStatus(c *gc.C) {
 	})
 }
 
-func (s *withoutControllerSuite) TestSeries(c *gc.C) {
-	// Add a machine with different series.
-	foobarMachine, err := s.State.AddMachine("foobar", state.JobHostUnits)
+func (s *withoutControllerSuite) TestInstanceStatus(c *gc.C) {
+	now := time.Now()
+	sInfo := status.StatusInfo{
+		Status:  status.Provisioning,
+		Message: "blah",
+		Since:   &now,
+	}
+	err := s.machines[0].SetInstanceStatus(sInfo)
+	c.Assert(err, jc.ErrorIsNil)
+	sInfo = status.StatusInfo{
+		Status:  status.Running,
+		Message: "foo",
+		Since:   &now,
+	}
+	err = s.machines[1].SetInstanceStatus(sInfo)
+	c.Assert(err, jc.ErrorIsNil)
+	sInfo = status.StatusInfo{
+		Status:  status.ProvisioningError,
+		Message: "not really",
+		Data:    map[string]interface{}{"foo": "bar"},
+		Since:   &now,
+	}
+	err = s.machines[2].SetInstanceStatus(sInfo)
 	c.Assert(err, jc.ErrorIsNil)
 
+	args := params.Entities{Entities: []params.Entity{
+		{Tag: s.machines[0].Tag().String()},
+		{Tag: s.machines[1].Tag().String()},
+		{Tag: s.machines[2].Tag().String()},
+		{Tag: "machine-42"},
+		{Tag: "unit-foo-0"},
+		{Tag: "application-bar"},
+	}}
+	result, err := s.provisioner.InstanceStatus(args)
+	c.Assert(err, jc.ErrorIsNil)
+	// Zero out the updated timestamps so we can easily check the results.
+	for i, statusResult := range result.Results {
+		r := statusResult
+		if r.Status != "" {
+			c.Assert(r.Since, gc.NotNil)
+		}
+		r.Since = nil
+		result.Results[i] = r
+	}
+	c.Assert(result, gc.DeepEquals, params.StatusResults{
+		Results: []params.StatusResult{
+			{Status: status.Provisioning.String(), Info: "blah", Data: map[string]interface{}{}},
+			{Status: status.Running.String(), Info: "foo", Data: map[string]interface{}{}},
+			{Status: status.ProvisioningError.String(), Info: "not really", Data: map[string]interface{}{"foo": "bar"}},
+			{Error: apiservertesting.NotFoundError("machine 42")},
+			{Error: apiservertesting.ErrUnauthorized},
+			{Error: apiservertesting.ErrUnauthorized},
+		},
+	})
+}
+
+func (s *withoutControllerSuite) TestSeries(c *gc.C) {
+	// Add a machine with different series.
+	foobarMachine := s.Factory.MakeMachine(c, &factory.MachineParams{Series: "foobar"})
 	args := params.Entities{Entities: []params.Entity{
 		{Tag: s.machines[0].Tag().String()},
 		{Tag: foobarMachine.Tag().String()},
@@ -694,6 +816,69 @@ func (s *withoutControllerSuite) TestSeries(c *gc.C) {
 	})
 }
 
+func (s *withoutControllerSuite) TestAvailabilityZone(c *gc.C) {
+	availabilityZone := "ru-north-siberia"
+	emptyAz := ""
+	hcWithAZ := instance.HardwareCharacteristics{AvailabilityZone: &availabilityZone}
+	hcWithEmptyAZ := instance.HardwareCharacteristics{AvailabilityZone: &emptyAz}
+	hcWithNilAz := instance.HardwareCharacteristics{AvailabilityZone: nil}
+
+	// add machines with different availability zones: string, empty string, nil
+	azMachine, _ := s.Factory.MakeMachineReturningPassword(c, &factory.MachineParams{
+		Characteristics: &hcWithAZ,
+	})
+
+	emptyAzMachine, _ := s.Factory.MakeMachineReturningPassword(c, &factory.MachineParams{
+		Characteristics: &hcWithEmptyAZ,
+	})
+
+	nilAzMachine, _ := s.Factory.MakeMachineReturningPassword(c, &factory.MachineParams{
+		Characteristics: &hcWithNilAz,
+	})
+	args := params.Entities{Entities: []params.Entity{
+		{Tag: azMachine.Tag().String()},
+		{Tag: emptyAzMachine.Tag().String()},
+		{Tag: nilAzMachine.Tag().String()},
+	}}
+	result, err := s.provisioner.AvailabilityZone(args)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(result, gc.DeepEquals, params.StringResults{
+		Results: []params.StringResult{
+			{Result: availabilityZone},
+			{Result: emptyAz},
+			{Result: emptyAz},
+		},
+	})
+}
+
+func (s *withoutControllerSuite) TestKeepInstance(c *gc.C) {
+	// Add a machine with keep-instance = true.
+	foobarMachine := s.Factory.MakeMachine(c, &factory.MachineParams{InstanceId: "1234"})
+	err := foobarMachine.SetKeepInstance(true)
+	c.Assert(err, jc.ErrorIsNil)
+
+	args := params.Entities{Entities: []params.Entity{
+		{Tag: s.machines[0].Tag().String()},
+		{Tag: foobarMachine.Tag().String()},
+		{Tag: s.machines[2].Tag().String()},
+		{Tag: "machine-42"},
+		{Tag: "unit-foo-0"},
+		{Tag: "application-bar"},
+	}}
+	result, err := s.provisioner.KeepInstance(args)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(result, gc.DeepEquals, params.BoolResults{
+		Results: []params.BoolResult{
+			{Result: false},
+			{Result: true},
+			{Result: false},
+			{Error: apiservertesting.NotFoundError("machine 42")},
+			{Error: apiservertesting.ErrUnauthorized},
+			{Error: apiservertesting.ErrUnauthorized},
+		},
+	})
+}
+
 func (s *withoutControllerSuite) TestDistributionGroup(c *gc.C) {
 	addUnits := func(name string, machines ...*state.Machine) (units []*state.Unit) {
 		svc := s.AddTestingService(c, name, s.AddTestingCharm(c, name))
@@ -793,7 +978,7 @@ func (s *withoutControllerSuite) TestDistributionGroupEnvironManagerAuth(c *gc.C
 func (s *withoutControllerSuite) TestDistributionGroupMachineAgentAuth(c *gc.C) {
 	anAuthorizer := s.authorizer
 	anAuthorizer.Tag = names.NewMachineTag("1")
-	anAuthorizer.EnvironManager = false
+	anAuthorizer.Controller = false
 	provisioner, err := provisioner.NewProvisionerAPI(s.State, s.resources, anAuthorizer)
 	c.Check(err, jc.ErrorIsNil)
 	args := params.Entities{Entities: []params.Entity{
@@ -860,7 +1045,7 @@ func (s *withoutControllerSuite) TestSetInstanceInfo(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	err = s.State.UpdateModelConfig(map[string]interface{}{
 		"storage-default-block-source": "static-pool",
-	}, nil, nil)
+	}, nil)
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Provision machine 0 first.
@@ -1020,7 +1205,7 @@ func (s *withoutControllerSuite) TestWatchModelMachines(c *gc.C) {
 	// Make sure WatchModelMachines fails with a machine agent login.
 	anAuthorizer := s.authorizer
 	anAuthorizer.Tag = names.NewMachineTag("1")
-	anAuthorizer.EnvironManager = false
+	anAuthorizer.Controller = false
 	aProvisioner, err := provisioner.NewProvisionerAPI(s.State, s.resources, anAuthorizer)
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -1046,13 +1231,21 @@ func (s *withoutControllerSuite) TestContainerManagerConfig(c *gc.C) {
 func (s *withoutControllerSuite) TestContainerConfig(c *gc.C) {
 	attrs := map[string]interface{}{
 		"http-proxy":            "http://proxy.example.com:9000",
+		"apt-https-proxy":       "https://proxy.example.com:9000",
 		"allow-lxd-loop-mounts": true,
 		"apt-mirror":            "http://example.mirror.com",
 	}
-	err := s.State.UpdateModelConfig(attrs, nil, nil)
+	err := s.State.UpdateModelConfig(attrs, nil)
 	c.Assert(err, jc.ErrorIsNil)
+	expectedAPTProxy := proxy.Settings{
+		Http:    "http://proxy.example.com:9000",
+		Https:   "https://proxy.example.com:9000",
+		NoProxy: "127.0.0.1,localhost,::1",
+	}
+
 	expectedProxy := proxy.Settings{
-		Http: "http://proxy.example.com:9000",
+		Http:    "http://proxy.example.com:9000",
+		NoProxy: "127.0.0.1,localhost,::1",
 	}
 
 	results, err := s.provisioner.ContainerConfig()
@@ -1062,7 +1255,7 @@ func (s *withoutControllerSuite) TestContainerConfig(c *gc.C) {
 	c.Check(results.AuthorizedKeys, gc.Equals, s.Environ.Config().AuthorizedKeys())
 	c.Check(results.SSLHostnameVerification, jc.IsTrue)
 	c.Check(results.Proxy, gc.DeepEquals, expectedProxy)
-	c.Check(results.AptProxy, gc.DeepEquals, expectedProxy)
+	c.Check(results.AptProxy, gc.DeepEquals, expectedAPTProxy)
 	c.Check(results.AptMirror, gc.DeepEquals, "http://example.mirror.com")
 }
 
@@ -1095,7 +1288,7 @@ func (s *withoutControllerSuite) TestSetSupportedContainers(c *gc.C) {
 func (s *withoutControllerSuite) TestSetSupportedContainersPermissions(c *gc.C) {
 	// Login as a machine agent for machine 0.
 	anAuthorizer := s.authorizer
-	anAuthorizer.EnvironManager = false
+	anAuthorizer.Controller = false
 	anAuthorizer.Tag = s.machines[0].Tag()
 	aProvisioner, err := provisioner.NewProvisionerAPI(s.State, s.resources, anAuthorizer)
 	c.Assert(err, jc.ErrorIsNil)
@@ -1211,7 +1404,7 @@ func (s *withoutControllerSuite) TestWatchMachineErrorRetry(c *gc.C) {
 	// Make sure WatchMachineErrorRetry fails with a machine agent login.
 	anAuthorizer := s.authorizer
 	anAuthorizer.Tag = names.NewMachineTag("1")
-	anAuthorizer.EnvironManager = false
+	anAuthorizer.Controller = false
 	aProvisioner, err := provisioner.NewProvisionerAPI(s.State, s.resources, anAuthorizer)
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -1270,3 +1463,14 @@ func (s *withoutControllerSuite) TestMarkMachinesForRemoval(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	c.Check(removals, jc.SameContents, []string{"0", "2"})
 }
+
+// TODO(jam): 2017-02-15 We seem to be lacking most of direct unit tests around ProcessOneContainer
+// Some of the use cases we need to be testing are:
+// 1) Provider can allocate addresses, should result in a container with
+//    addresses requested from the provider, and 'static' configuration on those
+//    devices.
+// 2) Provider cannot allocate addresses, currently this should make us use
+//    'lxdbr0' and DHCP allocated addresses.
+// 3) Provider could allocate DHCP based addresses on the host device, which would let us
+//    use a bridge on the device and DHCP. (Currently not supported, but desirable for
+//    vSphere and Manual and probably LXD providers.)
