@@ -10,21 +10,19 @@ import (
 	"github.com/juju/juju/state"
 )
 
-func init() {
-	common.RegisterStandardFacade("StatusHistory", 2, NewAPI)
-}
-
-// API is the concrete implementation of the Pruner endpoint..
+// API is the concrete implementation of the Pruner endpoint.
 type API struct {
+	*common.ModelWatcher
 	st         *state.State
 	authorizer facade.Authorizer
 }
 
 // NewAPI returns an API Instance.
-func NewAPI(st *state.State, _ facade.Resources, auth facade.Authorizer) (*API, error) {
+func NewAPI(st *state.State, r facade.Resources, auth facade.Authorizer) (*API, error) {
 	return &API{
-		st:         st,
-		authorizer: auth,
+		ModelWatcher: common.NewModelWatcher(st, r, auth),
+		st:           st,
+		authorizer:   auth,
 	}, nil
 }
 
@@ -32,7 +30,7 @@ func NewAPI(st *state.State, _ facade.Resources, auth facade.Authorizer) (*API,
 // only the ones newer than now - p.MaxHistoryTime remain and
 // the history is smaller than p.MaxHistoryMB.
 func (api *API) Prune(p params.StatusHistoryPruneArgs) error {
-	if !api.authorizer.AuthModelManager() {
+	if !api.authorizer.AuthController() {
 		return common.ErrPerm
 	}
 	return state.PruneStatusHistory(api.st, p.MaxHistoryTime, p.MaxHistoryMB)
