@@ -341,6 +341,10 @@ func (s *userManagerSuite) TestUserInfo(c *gc.C) {
 		s.State, s.AdminUserTag(c), names.NewUserTag("fred@external"),
 		params.GrantControllerAccess, permission.AddModelAccess)
 	c.Assert(err, jc.ErrorIsNil)
+	err = controller.ChangeControllerAccess(
+		s.State, s.AdminUserTag(c), names.NewUserTag("everyone@external"),
+		params.GrantControllerAccess, permission.AddModelAccess)
+	c.Assert(err, jc.ErrorIsNil)
 
 	args := params.UserInfoRequest{
 		Entities: []params.Entity{
@@ -352,6 +356,8 @@ func (s *userManagerSuite) TestUserInfo(c *gc.C) {
 				Tag: names.NewLocalUserTag("ellie").String(),
 			}, {
 				Tag: names.NewUserTag("fred@external").String(),
+			}, {
+				Tag: names.NewUserTag("mary@external").String(),
 			}, {
 				Tag: "not-a-tag",
 			},
@@ -390,6 +396,11 @@ func (s *userManagerSuite) TestUserInfo(c *gc.C) {
 				Username: "fred@external",
 				Access:   "add-model",
 			},
+		}, {
+			info: &params.UserInfo{
+				Username: "mary@external",
+				Access:   "add-model",
+			},
 		}, {
 			err: &params.Error{
 				Message: `"not-a-tag" is not a valid tag`,
@@ -649,7 +660,7 @@ func (s *userManagerSuite) TestRemoveUser(c *gc.C) {
 	// Create a user to delete.
 	jjam := s.Factory.MakeUser(c, &factory.UserParams{Name: "jimmyjam"})
 
-	expectedError := fmt.Sprintf("%q user not found", jjam.Name())
+	expectedError := fmt.Sprintf("failed to delete user %q: user %q is permanently deleted", jjam.Name(), jjam.Name())
 
 	// Remove the user
 	got, err := s.usermanager.RemoveUser(params.Entities{
@@ -670,7 +681,7 @@ func (s *userManagerSuite) TestRemoveUser(c *gc.C) {
 	c.Check(got.Results, gc.HasLen, 1)
 	c.Check(got.Results[0].Error, jc.DeepEquals, &params.Error{
 		Message: expectedError,
-		Code:    "user not found",
+		Code:    "",
 	})
 	c.Assert(err, jc.ErrorIsNil)
 }
