@@ -18,7 +18,6 @@ import (
 	"gopkg.in/yaml.v2"
 
 	"github.com/juju/juju/juju/osenv"
-	"github.com/juju/juju/provider/lxd/lxdnames"
 )
 
 //go:generate go run ../generate/filetoconst/filetoconst.go fallbackPublicCloudInfo fallback-public-cloud.yaml fallback_public_cloud.go 2015 cloud
@@ -56,6 +55,18 @@ const (
 	// EmptyAuthType is the authentication type used for providers
 	// that require no credentials, e.g. "lxd", and "manual".
 	EmptyAuthType AuthType = "empty"
+
+	// AuthTypesKey is the name of the key in a cloud config or cloud schema
+	// that holds the cloud's auth types.
+	AuthTypesKey = "auth-types"
+
+	// EndpointKey is the name of the key in a cloud config or cloud schema
+	// that holds the cloud's endpoint url.
+	EndpointKey = "endpoint"
+
+	// RegionsKey is the name of the key in a cloud schema that holds the list
+	// of regions a cloud supports.
+	RegionsKey = "regions"
 )
 
 // Attrs serves as a map to hold regions specific configuration attributes.
@@ -71,6 +82,9 @@ type RegionConfig map[string]Attrs
 
 // Cloud is a cloud definition.
 type Cloud struct {
+	// Name of the cloud.
+	Name string
+
 	// Type is the type of cloud, eg ec2, openstack etc.
 	// This is one of the provider names registered with
 	// environs.RegisterProvider.
@@ -141,6 +155,7 @@ type cloudSet struct {
 
 // cloud is equivalent to Cloud, for marshalling and unmarshalling.
 type cloud struct {
+	Name             string                 `yaml:"name,omitempty"`
 	Type             string                 `yaml:"type"`
 	Description      string                 `yaml:"description,omitempty"`
 	AuthTypes        []AuthType             `yaml:"auth-types,omitempty,flow"`
@@ -174,19 +189,6 @@ type region struct {
 	StorageEndpoint  string `yaml:"storage-endpoint,omitempty"`
 }
 
-//DefaultLXD is the name of the default lxd cloud.
-const DefaultLXD = "localhost"
-
-// BuiltInClouds work out of the box.
-var BuiltInClouds = map[string]Cloud{
-	DefaultLXD: {
-		Type:        lxdnames.ProviderType,
-		AuthTypes:   []AuthType{EmptyAuthType},
-		Regions:     []Region{{Name: lxdnames.DefaultRegion}},
-		Description: defaultCloudDescription[lxdnames.ProviderType],
-	},
-}
-
 // CloudByName returns the cloud with the specified name.
 // If there exists no cloud with the specified name, an
 // error satisfying errors.IsNotFound will be returned.
@@ -208,9 +210,6 @@ func CloudByName(name string) (*Cloud, error) {
 	if cloud, ok := clouds[name]; ok {
 		return &cloud, nil
 	}
-	if cloud, ok := BuiltInClouds[name]; ok {
-		return &cloud, nil
-	}
 	return nil, errors.NotFoundf("cloud %s", name)
 }
 
@@ -266,6 +265,15 @@ func PublicCloudMetadata(searchPath ...string) (result map[string]Cloud, fallbac
 	return clouds, true, err
 }
 
+// ParseOneCloud parses the given yaml bytes into a single Cloud metadata.
+func ParseOneCloud(data []byte) (Cloud, error) {
+	c := &cloud{}
+	if err := yaml.Unmarshal(data, &c); err != nil {
+		return Cloud{}, errors.Annotate(err, "cannot unmarshal yaml cloud metadata")
+	}
+	return cloudFromInternal(c), nil
+}
+
 // ParseCloudMetadata parses the given yaml bytes into Clouds metadata.
 func ParseCloudMetadata(data []byte) (map[string]Cloud, error) {
 	var metadata cloudSet
@@ -278,6 +286,7 @@ func ParseCloudMetadata(data []byte) (map[string]Cloud, error) {
 	clouds := make(map[string]Cloud)
 	for name, cloud := range metadata.Clouds {
 		details := cloudFromInternal(cloud)
+		details.Name = name
 		if details.Description == "" {
 			var ok bool
 			if details.Description, ok = defaultCloudDescription[name]; !ok {
@@ -289,6 +298,12 @@ func ParseCloudMetadata(data []byte) (map[string]Cloud, error) {
 	return clouds, nil
 }
 
+// DefaultCloudDescription returns the description for the specified cloud
+// type, or an empty string if the cloud type is unknown.
+func DefaultCloudDescription(cloudType string) string {
+	return defaultCloudDescription[cloudType]
+}
+
 var defaultCloudDescription = map[string]string{
 	"aws":         "Amazon Web Services",
 	"aws-china":   "Amazon China",
@@ -302,6 +317,7 @@ var defaultCloudDescription = map[string]string{
 	"lxd":         "LXD Container Hypervisor",
 	"maas":        "Metal As A Service",
 	"openstack":   "Openstack Cloud",
+	"oracle":      "Oracle Compute Cloud Service",
 }
 
 // WritePublicCloudMetadata marshals to YAML and writes the cloud metadata
@@ -333,7 +349,7 @@ func IsSameCloudMetadata(meta1, meta2 map[string]Cloud) (bool, error) {
 func marshalCloudMetadata(cloudsMap map[string]Cloud) ([]byte, error) {
 	clouds := cloudSet{make(map[string]*cloud)}
 	for name, metadata := range cloudsMap {
-		clouds.Clouds[name] = cloudToInternal(metadata)
+		clouds.Clouds[name] = cloudToInternal(metadata, false)
 	}
 	data, err := yaml.Marshal(clouds)
 	if err != nil {
@@ -344,7 +360,7 @@ func marshalCloudMetadata(cloudsMap map[string]Cloud) ([]byte, error) {
 
 // MarshalCloud marshals a Cloud to an opaque byte array.
 func MarshalCloud(cloud Cloud) ([]byte, error) {
-	return yaml.Marshal(cloudToInternal(cloud))
+	return yaml.Marshal(cloudToInternal(cloud, true))
 }
 
 // UnmarshalCloud unmarshals a Cloud from a byte array produced by MarshalCloud.
@@ -356,7 +372,7 @@ func UnmarshalCloud(in []byte) (Cloud, error) {
 	return cloudFromInternal(&internal), nil
 }
 
-func cloudToInternal(in Cloud) *cloud {
+func cloudToInternal(in Cloud, withName bool) *cloud {
 	var regions regions
 	for _, r := range in.Regions {
 		regions.Slice = append(regions.Slice, yaml.MapItem{
@@ -367,7 +383,12 @@ func cloudToInternal(in Cloud) *cloud {
 			},
 		})
 	}
+	name := in.Name
+	if !withName {
+		name = ""
+	}
 	return &cloud{
+		Name:             name,
 		Type:             in.Type,
 		AuthTypes:        in.AuthTypes,
 		Endpoint:         in.Endpoint,
@@ -400,6 +421,7 @@ func cloudFromInternal(in *cloud) Cloud {
 		}
 	}
 	meta := Cloud{
+		Name:             in.Name,
 		Type:             in.Type,
 		AuthTypes:        in.AuthTypes,
 		Endpoint:         in.Endpoint,
