@@ -12,11 +12,11 @@ import (
 	"strings"
 
 	"github.com/juju/errors"
+	"github.com/juju/utils/cert"
 	"github.com/juju/utils/featureflag"
 	"github.com/juju/utils/series"
 	"gopkg.in/juju/names.v2"
 
-	"github.com/juju/juju/cert"
 	"github.com/juju/juju/juju/osenv"
 	"github.com/juju/juju/juju/paths"
 	"github.com/juju/juju/tools"
