@@ -4,7 +4,9 @@
 package application
 
 import (
+	"encoding/base64"
 	"fmt"
+	"io/ioutil"
 	"os"
 	"path/filepath"
 	"sort"
@@ -12,13 +14,15 @@ import (
 	"time"
 
 	"github.com/juju/bundlechanges"
+	"github.com/juju/cmd"
 	"github.com/juju/errors"
+	"github.com/juju/utils"
 	"gopkg.in/juju/charm.v6-unstable"
 	"gopkg.in/juju/charmrepo.v2-unstable"
 	csparams "gopkg.in/juju/charmrepo.v2-unstable/csclient/params"
 	"gopkg.in/juju/names.v2"
 	"gopkg.in/macaroon.v1"
-	"gopkg.in/yaml.v1"
+	"gopkg.in/yaml.v2"
 
 	"github.com/juju/juju/api"
 	"github.com/juju/juju/api/application"
@@ -53,13 +57,18 @@ type deploymentLogger interface {
 // charm store client. The deployment is not transactional, and its progress is
 // notified using the given deployment logger.
 func deployBundle(
-	bundleFilePath string,
+	bundleDir string,
 	data *charm.BundleData,
+	bundleConfigFile string,
 	channel csparams.Channel,
 	apiRoot DeployAPI,
-	log deploymentLogger,
+	ctx *cmd.Context,
 	bundleStorage map[string]map[string]storage.Constraints,
 ) (map[*charm.URL]*macaroon.Macaroon, error) {
+
+	if err := processBundleConfig(data, bundleConfigFile); err != nil {
+		return nil, err
+	}
 	verifyConstraints := func(s string) error {
 		_, err := constraints.Parse(s)
 		return err
@@ -69,10 +78,18 @@ func deployBundle(
 		return err
 	}
 	var verifyError error
-	if bundleFilePath == "" {
+	if bundleDir == "" {
+		// Process includes in the bundle data.
+		if err := processBundleIncludes(ctx.Dir, data); err != nil {
+			return nil, errors.Annotate(err, "unable to process includes")
+		}
 		verifyError = data.Verify(verifyConstraints, verifyStorage)
 	} else {
-		verifyError = data.VerifyLocal(bundleFilePath, verifyConstraints, verifyStorage)
+		// Process includes in the bundle data.
+		if err := processBundleIncludes(bundleDir, data); err != nil {
+			return nil, errors.Annotate(err, "unable to process includes")
+		}
+		verifyError = data.VerifyLocal(bundleDir, verifyConstraints, verifyStorage)
 	}
 	if verifyError != nil {
 		if verr, ok := verifyError.(*charm.VerificationError); ok {
@@ -110,13 +127,13 @@ func deployBundle(
 
 	// Instantiate the bundle handler.
 	h := &bundleHandler{
-		bundleDir:       bundleFilePath,
+		bundleDir:       bundleDir,
 		changes:         changes,
 		results:         make(map[string]string, numChanges),
 		channel:         channel,
 		api:             apiRoot,
 		bundleStorage:   bundleStorage,
-		log:             log,
+		ctx:             ctx,
 		data:            data,
 		unitStatus:      unitStatus,
 		ignoredMachines: make(map[string]bool, len(data.Applications)),
@@ -199,9 +216,10 @@ type bundleHandler struct {
 	// in the bundle itself.
 	bundleStorage map[string]map[string]storage.Constraints
 
-	// log is used to output messages to the user, so that the user can keep
-	// track of the bundle deployment progress.
-	log deploymentLogger
+	// ctx is the command context, which is used to output messages to the
+	// user, so that the user can keep track of the bundle deployment
+	// progress.
+	ctx *cmd.Context
 
 	// data is the original bundle data that we want to deploy.
 	data *charm.BundleData
@@ -328,6 +346,12 @@ func (h *bundleHandler) addService(
 		}
 	}
 	resources := make(map[string]string)
+	for resName, path := range p.LocalResources {
+		if !filepath.IsAbs(path) {
+			path = filepath.Clean(filepath.Join(h.bundleDir, path))
+		}
+		resources[resName] = path
+	}
 	for resName, revision := range p.Resources {
 		resources[resName] = fmt.Sprint(revision)
 	}
@@ -370,7 +394,7 @@ func (h *bundleHandler) addService(
 
 	// Deploy the application.
 	logger.Debugf("application %s is deploying (charm %s)", p.Application, ch)
-	h.log.Infof("Deploying charm %q", ch)
+	h.ctx.Infof("Deploying charm %q", ch)
 	if err := api.Deploy(application.DeployArgs{
 		CharmID:          chID,
 		Cons:             cons,
@@ -382,7 +406,7 @@ func (h *bundleHandler) addService(
 		EndpointBindings: p.EndpointBindings,
 	}); err == nil {
 		for resName := range resNames2IDs {
-			h.log.Infof("added resource %s", resName)
+			h.ctx.Infof("added resource %s", resName)
 		}
 		return nil
 	} else if !isErrServiceExists(err) {
@@ -405,7 +429,7 @@ func (h *bundleHandler) addService(
 			// by the application Deploy call above.
 			return errors.Annotatef(err, "cannot update options for application %q", p.Application)
 		}
-		h.log.Infof("configuration updated for application %s", p.Application)
+		h.ctx.Infof("configuration updated for application %s", p.Application)
 	}
 	// Update application constraints.
 	if p.Constraints != "" {
@@ -413,7 +437,7 @@ func (h *bundleHandler) addService(
 			// This should never happen, as the bundle is already verified.
 			return errors.Annotatef(err, "cannot update constraints for application %q", p.Application)
 		}
-		h.log.Infof("constraints applied for application %s", p.Application)
+		h.ctx.Infof("constraints applied for application %s", p.Application)
 	}
 	return nil
 }
@@ -450,7 +474,7 @@ func (h *bundleHandler) addMachine(id string, p bundlechanges.AddMachineParams)
 		default:
 			msg = strings.Join(notify[:svcLen-1], ", ") + " and " + notify[svcLen-1]
 		}
-		h.log.Infof("avoid creating other machines to host %s units", msg)
+		h.ctx.Infof("avoid creating other machines to host %s units", msg)
 		return nil
 	}
 	cons, err := constraints.Parse(p.Constraints)
@@ -468,7 +492,7 @@ func (h *bundleHandler) addMachine(id string, p bundlechanges.AddMachineParams)
 		// placement directives as lxd.
 		if ct == "lxc" {
 			if !h.warnedLXC {
-				h.log.Infof("Bundle has one or more containers specified as lxc. lxc containers are deprecated in Juju 2.0. lxd containers will be deployed instead.")
+				h.ctx.Infof("Bundle has one or more containers specified as lxc. lxc containers are deprecated in Juju 2.0. lxd containers will be deployed instead.")
 				h.warnedLXC = true
 			}
 			ct = string(instance.LXD)
@@ -511,7 +535,7 @@ func (h *bundleHandler) addRelation(id string, p bundlechanges.AddRelationParams
 	_, err := h.api.AddRelation(ep1, ep2)
 	if err == nil {
 		// A new relation has been established.
-		h.log.Infof("Related %q and %q", ep1, ep2)
+		h.ctx.Infof("Related %q and %q", ep1, ep2)
 		return nil
 	}
 	if isErrRelationExists(err) {
@@ -539,7 +563,7 @@ func (h *bundleHandler) addUnit(id string, p bundlechanges.AddUnitParams) error
 			} else {
 				msg = fmt.Sprintf("%d units already present", num)
 			}
-			h.log.Infof("avoid adding new units to application %s: %s", application, msg)
+			h.ctx.Infof("avoid adding new units to application %s: %s", application, msg)
 		}
 		return nil
 	}
@@ -585,7 +609,7 @@ func (h *bundleHandler) exposeService(id string, p bundlechanges.ExposeParams) e
 	if err := h.api.Expose(application); err != nil {
 		return errors.Annotatef(err, "cannot expose application %s", application)
 	}
-	h.log.Infof("application %s exposed", application)
+	h.ctx.Infof("application %s exposed", application)
 	return nil
 }
 
@@ -814,7 +838,7 @@ func (h *bundleHandler) upgradeCharm(
 		return errors.Annotatef(err, "cannot retrieve info for application %q", applicationName)
 	}
 	if existing.String() == id {
-		h.log.Infof("reusing application %s (charm: %s)", applicationName, id)
+		h.ctx.Infof("reusing application %s (charm: %s)", applicationName, id)
 		return nil
 	}
 	url, err := charm.ParseURL(id)
@@ -856,9 +880,9 @@ func (h *bundleHandler) upgradeCharm(
 	if err := h.api.SetCharm(cfg); err != nil {
 		return errors.Annotatef(err, "cannot upgrade charm to %q", id)
 	}
-	h.log.Infof("upgraded charm for existing application %s (from %s to %s)", applicationName, existing, id)
+	h.ctx.Infof("upgraded charm for existing application %s (from %s to %s)", applicationName, existing, id)
 	for resName := range resNames2IDs {
-		h.log.Infof("added resource %s", resName)
+		h.ctx.Infof("added resource %s", resName)
 	}
 	return nil
 }
@@ -868,7 +892,7 @@ func (h *bundleHandler) upgradeCharm(
 func isErrServiceExists(err error) bool {
 	// TODO frankban (bug 1495952): do this check using the cause rather than
 	// the string when a specific cause is available.
-	return strings.HasSuffix(err.Error(), "application already exists")
+	return params.IsCodeAlreadyExists(err) || strings.HasSuffix(err.Error(), "application already exists")
 }
 
 // isErrRelationExists reports whether the given error has been generated
@@ -876,5 +900,239 @@ func isErrServiceExists(err error) bool {
 func isErrRelationExists(err error) bool {
 	// TODO frankban (bug 1495952): do this check using the cause rather than
 	// the string when a specific cause is available.
-	return strings.HasSuffix(err.Error(), "relation already exists")
+	return params.IsCodeAlreadyExists(err) || strings.HasSuffix(err.Error(), "relation already exists")
+}
+
+func processBundleIncludes(baseDir string, data *charm.BundleData) error {
+
+	for app, appData := range data.Applications {
+		// A bundle isn't valid if there are no applications, and applications must
+		// specify a charm at least, so we know appData must be non-nil.
+		for key, value := range appData.Options {
+			result, processed, err := processValue(baseDir, value)
+			if err != nil {
+				return errors.Annotatef(err, "processing options value %s for application %s", key, app)
+			}
+			if processed {
+				appData.Options[key] = result
+			}
+		}
+		for key, value := range appData.Annotations {
+			result, processed, err := processValue(baseDir, value)
+			if err != nil {
+				return errors.Annotatef(err, "processing annotation value %s for application %s", key, app)
+			}
+			if processed {
+				appData.Annotations[key] = result.(string)
+			}
+		}
+	}
+
+	for machine, machineData := range data.Machines {
+		if machineData == nil {
+			continue
+		}
+		for key, value := range machineData.Annotations {
+			result, processed, err := processValue(baseDir, value)
+			if err != nil {
+				return errors.Annotatef(err, "processing annotation value %s for machine %s", key, machine)
+			}
+			if processed {
+				machineData.Annotations[key] = result.(string)
+			}
+		}
+	}
+	return nil
+}
+
+func processValue(baseDir string, v interface{}) (interface{}, bool, error) {
+
+	const (
+		includeFile   = "include-file://"
+		includeBase64 = "include-base64://"
+	)
+
+	value, ok := v.(string)
+	if !ok {
+		// Not a string, just return it unchanged.
+		return v, false, nil
+	}
+
+	encode := false
+	readFile := false
+	filename := ""
+
+	if strings.HasPrefix(value, includeFile) {
+		readFile = true
+		filename = value[len(includeFile):]
+	} else if strings.HasPrefix(value, includeBase64) {
+		encode = true
+		readFile = true
+		filename = value[len(includeBase64):]
+	}
+
+	if !readFile {
+		// Unchanged, just return it.
+		return v, false, nil
+	}
+
+	if !filepath.IsAbs(filename) {
+		filename = filepath.Clean(filepath.Join(baseDir, filename))
+	}
+
+	bytes, err := ioutil.ReadFile(filename)
+	if err != nil {
+		return nil, false, errors.Annotate(err, "unable to read file")
+	}
+
+	var result string
+	if encode {
+		result = base64.StdEncoding.EncodeToString(bytes)
+	} else {
+		result = string(bytes)
+	}
+
+	return result, true, nil
+}
+
+type bundleConfig struct {
+	Applications map[string]*charm.ApplicationSpec `yaml:"applications"`
+	// TODO soon, add machine mapping and space mapping.
+}
+
+type bundleConfigValueExists struct {
+	Applications map[string]map[string]interface{} `yaml:"applications"`
+}
+
+func processBundleConfig(data *charm.BundleData, bundleConfigFile string) error {
+	if bundleConfigFile == "" {
+		// Nothing to do here.
+		return nil
+	}
+
+	bundleConfigFile, err := utils.NormalizePath(bundleConfigFile)
+	if err != nil {
+		return errors.Annotate(err, "unable to normalise bundle-config file")
+	}
+	// Make sure the filename is absolute.
+	if !filepath.IsAbs(bundleConfigFile) {
+		cwd, err := os.Getwd()
+		if err != nil {
+			return errors.Trace(err)
+		}
+		bundleConfigFile = filepath.Clean(filepath.Join(cwd, bundleConfigFile))
+	}
+	content, err := ioutil.ReadFile(bundleConfigFile)
+	if err != nil {
+		return errors.Annotate(err, "unable to open bundle-config file")
+	}
+	baseDir := filepath.Dir(bundleConfigFile)
+
+	// Now that we have the content, attempt to deserialize into the bundleConfig.
+	var config bundleConfig
+	if err := yaml.Unmarshal(content, &config); err != nil {
+		return errors.Annotate(err, "unable to deserialize config structure")
+	}
+	// If this works, then this deserialisation should certainly succeed.
+	// Since we are only looking to overwrite the values in the underlying bundle
+	// for config values that are set, we need to know if they were actually set,
+	// and not just zero. The configCheck structure is a map that allows us to check
+	// if the fields were actually in the underlying YAML.
+	var configCheck bundleConfigValueExists
+	if err := yaml.Unmarshal(content, &configCheck); err != nil {
+		return errors.Annotate(err, "unable to deserialize config structure")
+	}
+	// Additional checks to make sure that only things that we know about
+	// are passed in as config.
+	var checkTopLevel map[string]interface{}
+	if err := yaml.Unmarshal(content, &checkTopLevel); err != nil {
+		return errors.Annotate(err, "unable to deserialize config structure")
+	}
+	for key := range checkTopLevel {
+		switch key {
+		case "applications":
+			// no-op, all good
+		default:
+			return errors.Errorf("unexpected key %q in config", key)
+		}
+	}
+
+	// We want to confirm that all the applications mentioned in the config
+	// actually exist in the bundle data.
+	for appName, bc := range config.Applications {
+		app, found := data.Applications[appName]
+		if !found {
+			return errors.Errorf("application %q from config not found in bundle", appName)
+		}
+
+		fieldCheck := configCheck.Applications[appName]
+
+		if _, set := fieldCheck["charm"]; set {
+			app.Charm = bc.Charm
+		}
+		if _, set := fieldCheck["series"]; set {
+			app.Series = bc.Series
+		}
+		if _, set := fieldCheck["resources"]; set {
+			if app.Resources == nil {
+				app.Resources = make(map[string]interface{})
+			}
+			for key, value := range bc.Resources {
+				app.Resources[key] = value
+			}
+		}
+		if _, set := fieldCheck["num_units"]; set {
+			app.NumUnits = bc.NumUnits
+		}
+		if _, set := fieldCheck["to"]; set {
+			app.To = bc.To
+		}
+		if _, set := fieldCheck["expose"]; set {
+			app.Expose = bc.Expose
+		}
+		if _, set := fieldCheck["options"]; set {
+			if app.Options == nil {
+				app.Options = make(map[string]interface{})
+			}
+			for key, value := range bc.Options {
+				result, _, err := processValue(baseDir, value)
+				if err != nil {
+					return errors.Annotatef(err, "processing config options value %s for application %s", key, appName)
+				}
+				app.Options[key] = result
+			}
+		}
+		if _, set := fieldCheck["annotations"]; set {
+			if app.Annotations == nil {
+				app.Annotations = make(map[string]string)
+			}
+			for key, value := range bc.Annotations {
+				result, _, err := processValue(baseDir, value)
+				if err != nil {
+					return errors.Annotatef(err, "processing config annotations value %s for application %s", key, appName)
+				}
+				app.Annotations[key] = result.(string)
+			}
+		}
+		if _, set := fieldCheck["constraints"]; set {
+			app.Constraints = bc.Constraints
+		}
+		if _, set := fieldCheck["storage"]; set {
+			if app.Storage == nil {
+				app.Storage = make(map[string]string)
+			}
+			for key, value := range bc.Storage {
+				app.Storage[key] = value
+			}
+		}
+		if _, set := fieldCheck["bindings"]; set {
+			if app.EndpointBindings == nil {
+				app.EndpointBindings = make(map[string]string)
+			}
+			for key, value := range bc.EndpointBindings {
+				app.EndpointBindings[key] = value
+			}
+		}
+	}
+	return nil
 }
