@@ -4,20 +4,26 @@
 package application
 
 import (
+	"encoding/base64"
 	"fmt"
 	"io/ioutil"
 	"os"
 	"path/filepath"
+	"runtime"
+	"sort"
 	"strings"
 	"time"
 
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/charm.v6-unstable"
+	charmresource "gopkg.in/juju/charm.v6-unstable/resource"
+	"gopkg.in/juju/charmrepo.v2-unstable"
 	"gopkg.in/juju/charmrepo.v2-unstable/csclient"
 
 	"github.com/juju/juju/api"
 	"github.com/juju/juju/constraints"
+	"github.com/juju/juju/resource"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/state/multiwatcher"
 	"github.com/juju/juju/state/watcher"
@@ -30,16 +36,8 @@ import (
 // target in testing/base.go:SetupSuite we'll need to also update the entries
 // herein.
 
-// runDeployCommand executes the deploy command in order to deploy the given
-// charm or bundle. The deployment output and error are returned.
-func runDeployCommand(c *gc.C, id string, args ...string) (string, error) {
-	args = append([]string{id}, args...)
-	ctx, err := coretesting.RunCommand(c, NewDefaultDeployCommand(), args...)
-	return strings.Trim(coretesting.Stderr(ctx), "\n"), err
-}
-
 func (s *BundleDeployCharmStoreSuite) TestDeployBundleNotFoundCharmStore(c *gc.C) {
-	err := runDeploy(c, "bundle/no-such")
+	_, err := runDeploy(c, "bundle/no-such")
 	c.Assert(err, gc.ErrorMatches, `cannot resolve URL "cs:bundle/no-such": bundle not found`)
 }
 
@@ -47,19 +45,19 @@ func (s *BundleDeployCharmStoreSuite) TestDeployBundleInvalidFlags(c *gc.C) {
 	testcharms.UploadCharm(c, s.client, "xenial/mysql-42", "mysql")
 	testcharms.UploadCharm(c, s.client, "xenial/wordpress-47", "wordpress")
 	testcharms.UploadBundle(c, s.client, "bundle/wordpress-simple-1", "wordpress-simple")
-	_, err := runDeployCommand(c, "bundle/wordpress-simple", "--config", "config.yaml")
-	c.Assert(err, gc.ErrorMatches, "Flags provided but not supported when deploying a bundle: --config.")
-	_, err = runDeployCommand(c, "bundle/wordpress-simple", "-n", "2")
-	c.Assert(err, gc.ErrorMatches, "Flags provided but not supported when deploying a bundle: -n.")
-	_, err = runDeployCommand(c, "bundle/wordpress-simple", "--series", "xenial", "--force")
-	c.Assert(err, gc.ErrorMatches, "Flags provided but not supported when deploying a bundle: --force, --series.")
+	_, err := runDeploy(c, "bundle/wordpress-simple", "--config", "config.yaml")
+	c.Assert(err, gc.ErrorMatches, "flags provided but not supported when deploying a bundle: --config")
+	_, err = runDeploy(c, "bundle/wordpress-simple", "-n", "2")
+	c.Assert(err, gc.ErrorMatches, "flags provided but not supported when deploying a bundle: -n")
+	_, err = runDeploy(c, "bundle/wordpress-simple", "--series", "xenial", "--force")
+	c.Assert(err, gc.ErrorMatches, "flags provided but not supported when deploying a bundle: --force, --series")
 }
 
 func (s *BundleDeployCharmStoreSuite) TestDeployBundleSuccess(c *gc.C) {
 	testcharms.UploadCharm(c, s.client, "xenial/mysql-42", "mysql")
 	testcharms.UploadCharm(c, s.client, "xenial/wordpress-47", "wordpress")
 	testcharms.UploadBundle(c, s.client, "bundle/wordpress-simple-1", "wordpress-simple")
-	_, err := runDeployCommand(c, "bundle/wordpress-simple")
+	_, err := runDeploy(c, "bundle/wordpress-simple")
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertCharmsUploaded(c, "cs:xenial/mysql-42", "cs:xenial/wordpress-47")
 	s.assertApplicationsDeployed(c, map[string]serviceInfo{
@@ -77,7 +75,7 @@ func (s *BundleDeployCharmStoreSuite) TestDeployBundleWithTermsSuccess(c *gc.C)
 	testcharms.UploadCharm(c, s.client, "xenial/terms1-17", "terms1")
 	testcharms.UploadCharm(c, s.client, "xenial/terms2-42", "terms2")
 	testcharms.UploadBundle(c, s.client, "bundle/terms-simple-1", "terms-simple")
-	_, err := runDeployCommand(c, "bundle/terms-simple")
+	_, err := runDeploy(c, "bundle/terms-simple")
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertCharmsUploaded(c, "cs:xenial/terms1-17", "cs:xenial/terms2-42")
 	s.assertApplicationsDeployed(c, map[string]serviceInfo{
@@ -95,7 +93,7 @@ func (s *BundleDeployCharmStoreSuite) TestDeployBundleStorage(c *gc.C) {
 	testcharms.UploadCharm(c, s.client, "xenial/mysql-42", "mysql-storage")
 	testcharms.UploadCharm(c, s.client, "xenial/wordpress-47", "wordpress")
 	testcharms.UploadBundle(c, s.client, "bundle/wordpress-with-mysql-storage-1", "wordpress-with-mysql-storage")
-	_, err := runDeployCommand(
+	_, err := runDeploy(
 		c, "bundle/wordpress-with-mysql-storage",
 		"--storage", "mysql:logs=tmpfs,10G", // override logs
 	)
@@ -122,7 +120,7 @@ func (s *BundleDeployCharmStoreSuite) TestDeployBundleEndpointBindingsSpaceMissi
 	testcharms.UploadCharm(c, s.client, "xenial/mysql-42", "mysql")
 	testcharms.UploadCharm(c, s.client, "xenial/wordpress-extra-bindings-47", "wordpress-extra-bindings")
 	testcharms.UploadBundle(c, s.client, "bundle/wordpress-with-endpoint-bindings-1", "wordpress-with-endpoint-bindings")
-	output, err := runDeployCommand(c, "bundle/wordpress-with-endpoint-bindings")
+	output, err := runDeploy(c, "bundle/wordpress-with-endpoint-bindings")
 	c.Assert(err, gc.ErrorMatches, ""+
 		"cannot deploy bundle: cannot deploy application \"mysql\": "+
 		"cannot add application \"mysql\": unknown space \"db\" not valid")
@@ -144,7 +142,7 @@ func (s *BundleDeployCharmStoreSuite) TestDeployBundleEndpointBindingsSuccess(c
 	testcharms.UploadCharm(c, s.client, "xenial/mysql-42", "mysql")
 	testcharms.UploadCharm(c, s.client, "xenial/wordpress-extra-bindings-47", "wordpress-extra-bindings")
 	testcharms.UploadBundle(c, s.client, "bundle/wordpress-with-endpoint-bindings-1", "wordpress-with-endpoint-bindings")
-	_, err = runDeployCommand(c, "bundle/wordpress-with-endpoint-bindings")
+	_, err = runDeploy(c, "bundle/wordpress-with-endpoint-bindings")
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertCharmsUploaded(c, "cs:xenial/mysql-42", "cs:xenial/wordpress-extra-bindings-47")
 
@@ -154,7 +152,7 @@ func (s *BundleDeployCharmStoreSuite) TestDeployBundleEndpointBindingsSuccess(c
 	})
 	s.assertDeployedServiceBindings(c, map[string]serviceInfo{
 		"mysql": {
-			endpointBindings: map[string]string{"server": "db"},
+			endpointBindings: map[string]string{"server": "db", "server-admin": ""},
 		},
 		"wordpress-extra-bindings": {
 			endpointBindings: map[string]string{
@@ -181,9 +179,9 @@ func (s *BundleDeployCharmStoreSuite) TestDeployBundleTwice(c *gc.C) {
 	testcharms.UploadCharm(c, s.client, "xenial/mysql-42", "mysql")
 	testcharms.UploadCharm(c, s.client, "xenial/wordpress-47", "wordpress")
 	testcharms.UploadBundle(c, s.client, "bundle/wordpress-simple-1", "wordpress-simple")
-	_, err := runDeployCommand(c, "bundle/wordpress-simple")
+	_, err := runDeploy(c, "bundle/wordpress-simple")
 	c.Assert(err, jc.ErrorIsNil)
-	_, err = runDeployCommand(c, "bundle/wordpress-simple")
+	_, err = runDeploy(c, "bundle/wordpress-simple")
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertCharmsUploaded(c, "cs:xenial/mysql-42", "cs:xenial/wordpress-47")
 	s.assertApplicationsDeployed(c, map[string]serviceInfo{
@@ -202,7 +200,7 @@ func (s *BundleDeployCharmStoreSuite) TestDeployBundleGatedCharm(c *gc.C) {
 	url, _ := testcharms.UploadCharm(c, s.client, "xenial/wordpress-47", "wordpress")
 	s.changeReadPerm(c, url, clientUserName)
 	testcharms.UploadBundle(c, s.client, "bundle/wordpress-simple-1", "wordpress-simple")
-	_, err := runDeployCommand(c, "bundle/wordpress-simple")
+	_, err := runDeploy(c, "bundle/wordpress-simple")
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertCharmsUploaded(c, "cs:xenial/mysql-42", "cs:xenial/wordpress-47")
 	s.assertApplicationsDeployed(c, map[string]serviceInfo{
@@ -225,7 +223,7 @@ func (s *BundleDeployCharmStoreSuite) TestDeployBundleLocalPath(c *gc.C) {
     `
 	err := ioutil.WriteFile(path, []byte(data), 0644)
 	c.Assert(err, jc.ErrorIsNil)
-	_, err = runDeployCommand(c, path)
+	_, err = runDeploy(c, path)
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertCharmsUploaded(c, "local:xenial/dummy-1")
 	s.assertApplicationsDeployed(c, map[string]serviceInfo{
@@ -233,6 +231,30 @@ func (s *BundleDeployCharmStoreSuite) TestDeployBundleLocalPath(c *gc.C) {
 	})
 }
 
+func (s *BundleDeployCharmStoreSuite) TestDeployBundleLocalResources(c *gc.C) {
+	data := `
+        series: quantal
+        applications:
+            "dummy-resource":
+                charm: ./dummy-resource
+                series: quantal
+                num_units: 1
+                resources:
+                  dummy: ./dummy-resource.zip
+    `
+	dir := s.makeBundleDir(c, data)
+	testcharms.Repo.ClonedDir(dir, "dummy-resource")
+	c.Assert(
+		ioutil.WriteFile(filepath.Join(dir, "dummy-resource.zip"), []byte("zip file"), 0644),
+		jc.ErrorIsNil)
+	_, err := runDeploy(c, dir)
+	c.Assert(err, jc.ErrorIsNil)
+	s.assertCharmsUploaded(c, "local:quantal/dummy-resource-0")
+	s.assertApplicationsDeployed(c, map[string]serviceInfo{
+		"dummy-resource": {charm: "local:quantal/dummy-resource-0"},
+	})
+}
+
 func (s *BundleDeployCharmStoreSuite) TestDeployBundleNoSeriesInCharmURL(c *gc.C) {
 	testcharms.UploadCharmMultiSeries(c, s.client, "~who/multi-series", "multi-series")
 	dir := c.MkDir()
@@ -246,7 +268,7 @@ func (s *BundleDeployCharmStoreSuite) TestDeployBundleNoSeriesInCharmURL(c *gc.C
     `
 	err := ioutil.WriteFile(path, []byte(data), 0644)
 	c.Assert(err, jc.ErrorIsNil)
-	_, err = runDeployCommand(c, path)
+	_, err = runDeploy(c, path)
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertCharmsUploaded(c, "cs:~who/multi-series-0")
 	s.assertApplicationsDeployed(c, map[string]serviceInfo{
@@ -259,8 +281,105 @@ func (s *BundleDeployCharmStoreSuite) TestDeployBundleGatedCharmUnauthorized(c *
 	url, _ := testcharms.UploadCharm(c, s.client, "xenial/wordpress-47", "wordpress")
 	s.changeReadPerm(c, url, "who")
 	testcharms.UploadBundle(c, s.client, "bundle/wordpress-simple-1", "wordpress-simple")
-	_, err := runDeployCommand(c, "bundle/wordpress-simple")
-	c.Assert(err, gc.ErrorMatches, `cannot deploy bundle: .*: unauthorized: access denied for user "client-username"`)
+	_, err := runDeploy(c, "bundle/wordpress-simple")
+	c.Assert(err, gc.ErrorMatches, `cannot deploy bundle: .*: access denied for user "client-username"`)
+}
+
+func (s *BundleDeployCharmStoreSuite) TestDeployBundleResources(c *gc.C) {
+	testcharms.UploadCharm(c, s.Client(), "trusty/starsay-42", "starsay")
+	bundleMeta := `
+        applications:
+            starsay:
+                charm: cs:starsay
+                num_units: 1
+                resources:
+                    store-resource: 0
+                    install-resource: 0
+                    upload-resource: 0
+    `
+	output, err := s.DeployBundleYAML(c, bundleMeta)
+	c.Assert(err, jc.ErrorIsNil)
+
+	lines := strings.Split(output, "\n")
+	expectedLines := strings.Split(strings.TrimSpace(`
+Deploying charm "cs:trusty/starsay-42"
+added resource install-resource
+added resource store-resource
+added resource upload-resource
+Deploy of bundle completed.
+    `), "\n")
+	c.Check(lines, gc.HasLen, len(expectedLines))
+	c.Check(lines[0], gc.Equals, expectedLines[0])
+	// The "added resource" lines are checked after we sort since
+	// the ordering of those lines is unknown.
+	sort.Strings(lines)
+	sort.Strings(expectedLines)
+
+	resourceHash := func(content string) charmresource.Fingerprint {
+		fp, err := charmresource.GenerateFingerprint(strings.NewReader(content))
+		c.Assert(err, jc.ErrorIsNil)
+		return fp
+	}
+
+	c.Check(lines, jc.DeepEquals, expectedLines)
+	s.checkResources(c, "starsay", []resource.Resource{{
+		Resource: charmresource.Resource{
+			Meta: charmresource.Meta{
+				Name:        "install-resource",
+				Type:        charmresource.TypeFile,
+				Path:        "gotta-have-it.txt",
+				Description: "get things started",
+			},
+			Origin:      charmresource.OriginStore,
+			Revision:    0,
+			Fingerprint: resourceHash("install-resource content"),
+			Size:        int64(len("install-resource content")),
+		},
+		ID:            "starsay/install-resource",
+		ApplicationID: "starsay",
+	}, {
+		Resource: charmresource.Resource{
+			Meta: charmresource.Meta{
+				Name:        "store-resource",
+				Type:        charmresource.TypeFile,
+				Path:        "filename.tgz",
+				Description: "One line that is useful when operators need to push it.",
+			},
+			Origin:      charmresource.OriginStore,
+			Fingerprint: resourceHash("store-resource content"),
+			Size:        int64(len("store-resource content")),
+			Revision:    0,
+		},
+		ID:            "starsay/store-resource",
+		ApplicationID: "starsay",
+	}, {
+		Resource: charmresource.Resource{
+			Meta: charmresource.Meta{
+				Name:        "upload-resource",
+				Type:        charmresource.TypeFile,
+				Path:        "somename.xml",
+				Description: "Who uses xml anymore?",
+			},
+			Origin:      charmresource.OriginStore,
+			Fingerprint: resourceHash("upload-resource content"),
+			Size:        int64(len("upload-resource content")),
+			Revision:    0,
+		},
+		ID:            "starsay/upload-resource",
+		ApplicationID: "starsay",
+	}})
+}
+
+func (s *BundleDeployCharmStoreSuite) checkResources(c *gc.C, serviceName string, expected []resource.Resource) {
+	_, err := s.State.Application("starsay")
+	c.Check(err, jc.ErrorIsNil)
+	st, err := s.State.Resources()
+	c.Assert(err, jc.ErrorIsNil)
+	svcResources, err := st.ListResources("starsay")
+	c.Assert(err, jc.ErrorIsNil)
+	resources := svcResources.Resources
+	resource.Sort(resources)
+	c.Assert(resources, jc.DeepEquals, expected)
 }
 
 type BundleDeployCharmStoreSuite struct {
@@ -281,15 +400,20 @@ func (s *BundleDeployCharmStoreSuite) Client() *csclient.Client {
 // DeployBundleYAML uses the given bundle content to create a bundle in the
 // local repository and then deploy it. It returns the bundle deployment output
 // and error.
-func (s *BundleDeployCharmStoreSuite) DeployBundleYAML(c *gc.C, content string) (string, error) {
+func (s *BundleDeployCharmStoreSuite) DeployBundleYAML(c *gc.C, content string, extraArgs ...string) (string, error) {
+	bundlePath := s.makeBundleDir(c, content)
+	args := append([]string{bundlePath}, extraArgs...)
+	return runDeploy(c, args...)
+}
+
+func (s *BundleDeployCharmStoreSuite) makeBundleDir(c *gc.C, content string) string {
 	bundlePath := filepath.Join(c.MkDir(), "example")
 	c.Assert(os.Mkdir(bundlePath, 0777), jc.ErrorIsNil)
-	defer os.RemoveAll(bundlePath)
 	err := ioutil.WriteFile(filepath.Join(bundlePath, "bundle.yaml"), []byte(content), 0644)
 	c.Assert(err, jc.ErrorIsNil)
 	err = ioutil.WriteFile(filepath.Join(bundlePath, "README.md"), []byte("README"), 0644)
 	c.Assert(err, jc.ErrorIsNil)
-	return runDeployCommand(c, bundlePath)
+	return bundlePath
 }
 
 var deployBundleErrorsTests = []struct {
@@ -305,7 +429,7 @@ var deployBundleErrorsTests = []struct {
                 num_units: 1
     `,
 	err: `the provided bundle has the following errors:
-charm path in application "mysql" does not exist: mysql`,
+charm path in application "mysql" does not exist: .*mysql`,
 }, {
 	about: "charm store charm not found",
 	content: `
@@ -414,7 +538,36 @@ func (s *BundleDeployCharmStoreSuite) TestDeployBundleInvalidSeries(c *gc.C) {
 	c.Assert(err, gc.ErrorMatches, `cannot deploy bundle: cannot add unit for application "django": adding new machine to host unit "django/0": cannot assign unit "django/0" to machine 0: series does not match`)
 }
 
+func (s *BundleDeployCharmStoreSuite) TestDeployBundleInvalidBinding(c *gc.C) {
+	testcharms.UploadCharm(c, s.client, "xenial/wordpress-42", "wordpress")
+	_, err := s.DeployBundleYAML(c, `
+        applications:
+            wp:
+                charm: xenial/wordpress-42
+                num_units: 1
+                bindings:
+                  noturl: public
+    `)
+	c.Assert(err, gc.ErrorMatches, `cannot deploy bundle: cannot deploy application "wp": invalid binding\(s\) supplied "noturl", valid binding names are "admin-api",.* "url"`)
+}
+
+func (s *BundleDeployCharmStoreSuite) TestDeployBundleInvalidSpace(c *gc.C) {
+	testcharms.UploadCharm(c, s.client, "xenial/wordpress-42", "wordpress")
+	_, err := s.DeployBundleYAML(c, `
+        applications:
+            wp:
+                charm: xenial/wordpress-42
+                num_units: 1
+                bindings:
+                  url: public
+    `)
+	// TODO(jam): 2017-02-05 double repeating "cannot deploy application" and "cannot add application" is a bit ugly
+	// https://pad.lv/1661937
+	c.Assert(err, gc.ErrorMatches, `cannot deploy bundle: cannot deploy application "wp": cannot add application "wp": unknown space "public" not valid`)
+}
+
 func (s *BundleDeployCharmStoreSuite) TestDeployBundleWatcherTimeout(c *gc.C) {
+	coretesting.SkipFlaky(c, "lp:1625213")
 	// Inject an "AllWatcher" that never delivers a result.
 	ch := make(chan struct{})
 	defer close(ch)
@@ -474,6 +627,69 @@ func (s *BundleDeployCharmStoreSuite) TestDeployBundleLocalDeployment(c *gc.C) {
 	})
 }
 
+func (s *BundleDeployCharmStoreSuite) TestDeployBundleLocalDeploymentBadConfig(c *gc.C) {
+	charmsPath := c.MkDir()
+	mysqlPath := testcharms.Repo.ClonedDirPath(charmsPath, "mysql")
+	wordpressPath := testcharms.Repo.ClonedDirPath(charmsPath, "wordpress")
+	_, err := s.DeployBundleYAML(c, fmt.Sprintf(`
+        series: xenial
+        applications:
+            wordpress:
+                charm: %s
+                num_units: 1
+            mysql:
+                charm: %s
+                num_units: 2
+        relations:
+            - ["wordpress:db", "mysql:server"]
+    `, wordpressPath, mysqlPath),
+		"--bundle-config", "missing-file")
+	c.Assert(err, gc.ErrorMatches, "unable to open bundle-config file: "+missingFileRegex("missing-file"))
+}
+
+func (s *BundleDeployCharmStoreSuite) TestDeployBundleLocalDeploymentWithBundleConfig(c *gc.C) {
+	configDir := c.MkDir()
+	configFile := filepath.Join(configDir, "config.yaml")
+	c.Assert(
+		ioutil.WriteFile(
+			configFile, []byte(`
+                applications:
+                    wordpress:
+                        options:
+                            blog-title: include-file://title
+            `), 0644),
+		jc.ErrorIsNil)
+	c.Assert(
+		ioutil.WriteFile(
+			filepath.Join(configDir, "title"), []byte("magic bundle config"), 0644),
+		jc.ErrorIsNil)
+
+	charmsPath := c.MkDir()
+	mysqlPath := testcharms.Repo.ClonedDirPath(charmsPath, "mysql")
+	wordpressPath := testcharms.Repo.ClonedDirPath(charmsPath, "wordpress")
+	_, err := s.DeployBundleYAML(c, fmt.Sprintf(`
+        series: xenial
+        applications:
+            wordpress:
+                charm: %s
+                num_units: 1
+            mysql:
+                charm: %s
+                num_units: 2
+        relations:
+            - ["wordpress:db", "mysql:server"]
+    `, wordpressPath, mysqlPath),
+		"--bundle-config", configFile)
+
+	c.Assert(err, jc.ErrorIsNil)
+	// Now check the blog-title of the wordpress.	le")
+	wordpress, err := s.State.Application("wordpress")
+	c.Assert(err, jc.ErrorIsNil)
+	settings, err := wordpress.ConfigSettings()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(settings["blog-title"], gc.Equals, "magic bundle config")
+}
+
 func (s *BundleDeployCharmStoreSuite) TestDeployBundleLocalAndCharmStoreCharms(c *gc.C) {
 	charmsPath := c.MkDir()
 	testcharms.UploadCharm(c, s.client, "xenial/wordpress-42", "wordpress")
@@ -1232,3 +1448,343 @@ func (w mockAllWatcher) Next() ([]multiwatcher.Delta, error) {
 func (mockAllWatcher) Stop() error {
 	return nil
 }
+
+type ProcessIncludesSuite struct {
+	coretesting.BaseSuite
+}
+
+var _ = gc.Suite(&ProcessIncludesSuite{})
+
+func (*ProcessIncludesSuite) TestNonString(c *gc.C) {
+	value := 1234
+	result, changed, err := processValue("", value)
+
+	c.Check(err, jc.ErrorIsNil)
+	c.Check(changed, jc.IsFalse)
+	c.Check(result, gc.Equals, value)
+}
+
+func (*ProcessIncludesSuite) TestSimpleString(c *gc.C) {
+	value := "simple"
+	result, changed, err := processValue("", value)
+
+	c.Check(err, jc.ErrorIsNil)
+	c.Check(changed, jc.IsFalse)
+	c.Check(result, gc.Equals, value)
+}
+
+func (*ProcessIncludesSuite) TestMissingFile(c *gc.C) {
+	value := "include-file://simple"
+	result, changed, err := processValue("", value)
+
+	c.Check(err, gc.ErrorMatches, "unable to read file: "+missingFileRegex("simple"))
+	c.Check(changed, jc.IsFalse)
+	c.Check(result, gc.IsNil)
+}
+
+func (*ProcessIncludesSuite) TestFileNameIsInDir(c *gc.C) {
+	dir := c.MkDir()
+	filename := filepath.Join(dir, "content")
+	err := ioutil.WriteFile(filename, []byte("testing"), 0644)
+	c.Assert(err, jc.ErrorIsNil)
+
+	value := "include-file://content"
+	result, changed, err := processValue(dir, value)
+
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(changed, jc.IsTrue)
+	c.Check(result, gc.Equals, "testing")
+}
+
+func (*ProcessIncludesSuite) TestRelativePath(c *gc.C) {
+	dir := c.MkDir()
+	c.Assert(os.Mkdir(filepath.Join(dir, "nested"), 0755), jc.ErrorIsNil)
+
+	filename := filepath.Join(dir, "nested", "content")
+	err := ioutil.WriteFile(filename, []byte("testing"), 0644)
+	c.Assert(err, jc.ErrorIsNil)
+
+	value := "include-file://./nested/content"
+	result, changed, err := processValue(dir, value)
+
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(changed, jc.IsTrue)
+	c.Check(result, gc.Equals, "testing")
+}
+
+func (*ProcessIncludesSuite) TestAbsolutePath(c *gc.C) {
+	dir := c.MkDir()
+	c.Assert(os.Mkdir(filepath.Join(dir, "nested"), 0755), jc.ErrorIsNil)
+
+	filename := filepath.Join(dir, "nested", "content")
+	c.Check(filepath.IsAbs(filename), jc.IsTrue)
+	err := ioutil.WriteFile(filename, []byte("testing"), 0644)
+	c.Assert(err, jc.ErrorIsNil)
+
+	value := "include-file://" + filename
+	result, changed, err := processValue(dir, value)
+
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(changed, jc.IsTrue)
+	c.Check(result, gc.Equals, "testing")
+}
+
+func (*ProcessIncludesSuite) TestBase64Encode(c *gc.C) {
+	dir := c.MkDir()
+	filename := filepath.Join(dir, "content")
+	err := ioutil.WriteFile(filename, []byte("testing"), 0644)
+	c.Assert(err, jc.ErrorIsNil)
+
+	value := "include-base64://content"
+	result, changed, err := processValue(dir, value)
+
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(changed, jc.IsTrue)
+	encoded := base64.StdEncoding.EncodeToString([]byte("testing"))
+	c.Check(result, gc.Equals, encoded)
+}
+
+func (*ProcessIncludesSuite) TestBundleReplacements(c *gc.C) {
+	bundleYAML := `
+        applications:
+            django:
+                charm: cs:django
+                num_units: 1
+                options:
+                    private: include-base64://sekrit.binary
+                annotations:
+                    key1: value1
+                    key2: value2
+                    key3: include-file://annotation
+                to: [1]
+            memcached:
+                charm: xenial/mem-47
+                num_units: 1
+        machines:
+            1:
+                annotations: {foo: bar, baz: "include-file://machine" }
+            2:
+    `
+
+	baseDir := c.MkDir()
+	bundleFile := filepath.Join(baseDir, "bundle.yaml")
+	err := ioutil.WriteFile(bundleFile, []byte(bundleYAML), 0644)
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Assert(
+		ioutil.WriteFile(
+			filepath.Join(baseDir, "sekrit.binary"),
+			[]byte{42, 12, 0, 23, 8}, 0644),
+		jc.ErrorIsNil)
+	c.Assert(
+		ioutil.WriteFile(
+			filepath.Join(baseDir, "annotation"),
+			[]byte("value3"), 0644),
+		jc.ErrorIsNil)
+	c.Assert(
+		ioutil.WriteFile(
+			filepath.Join(baseDir, "machine"),
+			[]byte("wibble"), 0644),
+		jc.ErrorIsNil)
+
+	bundleData, err := charmrepo.ReadBundleFile(bundleFile)
+	c.Assert(err, jc.ErrorIsNil)
+
+	err = processBundleIncludes(baseDir, bundleData)
+	c.Assert(err, jc.ErrorIsNil)
+
+	django := bundleData.Applications["django"]
+	c.Check(django.Annotations["key1"], gc.Equals, "value1")
+	c.Check(django.Annotations["key2"], gc.Equals, "value2")
+	c.Check(django.Annotations["key3"], gc.Equals, "value3")
+	c.Check(django.Options["private"], gc.Equals, "KgwAFwg=")
+	annotations := bundleData.Machines["1"].Annotations
+	c.Check(annotations["foo"], gc.Equals, "bar")
+	c.Check(annotations["baz"], gc.Equals, "wibble")
+}
+
+type ProcessBundleConfigSuite struct {
+	coretesting.BaseSuite
+
+	bundleData *charm.BundleData
+}
+
+var _ = gc.Suite(&ProcessBundleConfigSuite{})
+
+func (s *ProcessBundleConfigSuite) SetUpTest(c *gc.C) {
+	s.BaseSuite.SetUpTest(c)
+
+	baseBundle := `
+        applications:
+            django:
+                expose: true
+                charm: cs:django
+                num_units: 1
+                options:
+                    general: good
+                annotations:
+                    key1: value1
+                    key2: value2
+                to: [1]
+            memcached:
+                charm: xenial/mem-47
+                num_units: 1
+                options:
+                    key: value
+        machines:
+            1:
+                annotations: {foo: bar}`
+
+	baseDir := c.MkDir()
+	bundleFile := filepath.Join(baseDir, "bundle.yaml")
+	err := ioutil.WriteFile(bundleFile, []byte(baseBundle), 0644)
+	c.Assert(err, jc.ErrorIsNil)
+
+	s.bundleData, err = charmrepo.ReadBundleFile(bundleFile)
+	c.Assert(err, jc.ErrorIsNil)
+}
+
+func (s *ProcessBundleConfigSuite) writeFile(c *gc.C, content string) string {
+	// Write the content to a file in a new directoryt and return the file.
+	baseDir := c.MkDir()
+	filename := filepath.Join(baseDir, "config.yaml")
+	err := ioutil.WriteFile(filename, []byte(content), 0644)
+	c.Assert(err, jc.ErrorIsNil)
+	return filename
+}
+
+func (s *ProcessBundleConfigSuite) TestNoFile(c *gc.C) {
+	err := processBundleConfig(s.bundleData, "")
+	c.Assert(err, jc.ErrorIsNil)
+}
+
+func (s *ProcessBundleConfigSuite) TestBadFile(c *gc.C) {
+	err := processBundleConfig(s.bundleData, "bad")
+	c.Assert(err, gc.ErrorMatches, "unable to open bundle-config file: "+missingFileRegex("bad"))
+}
+
+func (s *ProcessBundleConfigSuite) TestGoodYAML(c *gc.C) {
+	filename := s.writeFile(c, "bad:\n\tindent")
+	err := processBundleConfig(s.bundleData, filename)
+	c.Assert(err, gc.ErrorMatches, "unable to deserialize config structure: yaml: line 1: found character that cannot start any token")
+}
+
+func (s *ProcessBundleConfigSuite) TestReplaceZeroValues(c *gc.C) {
+	config := `
+        applications:
+            django:
+                expose: false
+                num_units: 0
+    `
+	filename := s.writeFile(c, config)
+	err := processBundleConfig(s.bundleData, filename)
+	c.Assert(err, jc.ErrorIsNil)
+	django := s.bundleData.Applications["django"]
+
+	c.Check(django.Expose, jc.IsFalse)
+	c.Check(django.NumUnits, gc.Equals, 0)
+}
+
+func (s *ProcessBundleConfigSuite) TestUnknownConfigKey(c *gc.C) {
+	config := `
+        machines:
+            1: 2
+        applications:
+            django:
+                expose: false
+                num_units: 0
+    `
+	filename := s.writeFile(c, config)
+	err := processBundleConfig(s.bundleData, filename)
+	c.Assert(err, gc.ErrorMatches, `unexpected key "machines" in config`)
+}
+
+func (s *ProcessBundleConfigSuite) TestUnknownApplication(c *gc.C) {
+	config := `
+        applications:
+            wordpress:
+                expose: false
+                num_units: 0
+    `
+	filename := s.writeFile(c, config)
+	err := processBundleConfig(s.bundleData, filename)
+	c.Assert(err, gc.ErrorMatches, `application "wordpress" from config not found in bundle`)
+}
+
+func (s *ProcessBundleConfigSuite) TestIncludes(c *gc.C) {
+	config := `
+        applications:
+            django:
+                options:
+                    private: include-base64://sekrit.binary
+                annotations:
+                    key3: include-file://annotation
+    `
+	filename := s.writeFile(c, config)
+	baseDir := filepath.Dir(filename)
+
+	c.Assert(
+		ioutil.WriteFile(
+			filepath.Join(baseDir, "sekrit.binary"),
+			[]byte{42, 12, 0, 23, 8}, 0644),
+		jc.ErrorIsNil)
+	c.Assert(
+		ioutil.WriteFile(
+			filepath.Join(baseDir, "annotation"),
+			[]byte("value3"), 0644),
+		jc.ErrorIsNil)
+
+	err := processBundleConfig(s.bundleData, filename)
+	c.Assert(err, jc.ErrorIsNil)
+	django := s.bundleData.Applications["django"]
+	c.Check(django.Annotations, jc.DeepEquals, map[string]string{
+		"key1": "value1",
+		"key2": "value2",
+		"key3": "value3"})
+	c.Check(django.Options, jc.DeepEquals, map[string]interface{}{
+		"general": "good",
+		"private": "KgwAFwg="})
+}
+
+func (s *ProcessBundleConfigSuite) TestRemainingFields(c *gc.C) {
+	// Note that we don't care about the actual values here
+	// as bundle validation is done after replacement.
+	config := `
+        applications:
+            django:
+                charm: cs:django-23
+                series: wisty
+                resources:
+                    something: or other
+                to:
+                  - 3
+                constraints: big machine
+                storage:
+                  disk: big
+                bindings:
+                  where: dmz
+    `
+	filename := s.writeFile(c, config)
+	err := processBundleConfig(s.bundleData, filename)
+	c.Assert(err, jc.ErrorIsNil)
+	django := s.bundleData.Applications["django"]
+
+	c.Check(django.Charm, gc.Equals, "cs:django-23")
+	c.Check(django.Series, gc.Equals, "wisty")
+	c.Check(django.Resources, jc.DeepEquals, map[string]interface{}{
+		"something": "or other"})
+	c.Check(django.To, jc.DeepEquals, []string{"3"})
+	c.Check(django.Constraints, gc.Equals, "big machine")
+	c.Check(django.Storage, jc.DeepEquals, map[string]string{
+		"disk": "big"})
+	c.Check(django.EndpointBindings, jc.DeepEquals, map[string]string{
+		"where": "dmz"})
+}
+
+func missingFileRegex(filename string) string {
+	text := "no such file or directory"
+	if runtime.GOOS == "windows" {
+		text = "The system cannot find the file specified."
+	}
+	return fmt.Sprintf("open .*%s: %s", filename, text)
+}
