@@ -8,6 +8,7 @@ import (
 	"net/http/httptest"
 
 	"github.com/juju/cmd"
+	"github.com/juju/cmd/cmdtesting"
 	"github.com/juju/errors"
 	"github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
@@ -17,7 +18,6 @@ import (
 
 	apicharms "github.com/juju/juju/api/charms"
 	"github.com/juju/juju/charmstore"
-	coretesting "github.com/juju/juju/testing"
 )
 
 var _ = gc.Suite(&registrationSuite{})
@@ -34,14 +34,20 @@ type registrationSuite struct {
 func (s *registrationSuite) SetUpTest(c *gc.C) {
 	s.CleanupSuite.SetUpTest(c)
 	s.stub = &testing.Stub{}
-	s.handler = &testMetricsRegistrationHandler{Stub: s.stub}
+	s.handler = &testMetricsRegistrationHandler{
+		Stub: s.stub,
+		availablePlans: []availablePlanURL{
+			{URL: "thisplan"},
+			{URL: "thisotherplan"},
+		},
+	}
 	s.server = httptest.NewServer(s.handler)
 	s.register = &RegisterMeteredCharm{
 		Plan:           "someplan",
 		RegisterURL:    s.server.URL,
-		AllocationSpec: "personal:100",
+		IncreaseBudget: 100,
 	}
-	s.ctx = coretesting.Context(c)
+	s.ctx = cmdtesting.Context(c)
 }
 
 func (s *registrationSuite) TearDownTest(c *gc.C) {
@@ -77,8 +83,7 @@ func (s *registrationSuite) TestMeteredCharm(c *gc.C) {
 			CharmURL:        "cs:quantal/metered-1",
 			ApplicationName: "application name",
 			PlanURL:         "someplan",
-			Budget:          "personal",
-			Limit:           "100",
+			IncreaseBudget:  100,
 		}},
 	}, {
 		"SetMetricCredentials", []interface{}{
@@ -116,8 +121,7 @@ func (s *registrationSuite) TestOptionalPlanMeteredCharm(c *gc.C) {
 			CharmURL:        "cs:quantal/metered-1",
 			ApplicationName: "application name",
 			PlanURL:         "someplan",
-			Budget:          "personal",
-			Limit:           "100",
+			IncreaseBudget:  100,
 		}},
 	}, {
 		"SetMetricCredentials", []interface{}{
@@ -155,8 +159,7 @@ func (s *registrationSuite) TestPlanNotSpecifiedCharm(c *gc.C) {
 			CharmURL:        "cs:quantal/metered-1",
 			ApplicationName: "application name",
 			PlanURL:         "someplan",
-			Budget:          "personal",
-			Limit:           "100",
+			IncreaseBudget:  100,
 		}},
 	}, {
 		"SetMetricCredentials", []interface{}{
@@ -191,8 +194,7 @@ func (s *registrationSuite) TestMeteredCharmAPIError(c *gc.C) {
 			CharmURL:        "cs:quantal/metered-1",
 			ApplicationName: "application name",
 			PlanURL:         "someplan",
-			Budget:          "personal",
-			Limit:           "100",
+			IncreaseBudget:  100,
 		}},
 	}})
 }
@@ -214,14 +216,53 @@ func (s *registrationSuite) TestMeteredCharmInvalidAllocation(c *gc.C) {
 	s.register = &RegisterMeteredCharm{
 		Plan:           "someplan",
 		RegisterURL:    s.server.URL,
-		AllocationSpec: "invalid allocation",
+		IncreaseBudget: -1000,
 	}
 
 	err := s.register.RunPre(&mockMeteredDeployAPI{Stub: s.stub}, client, s.ctx, d)
-	c.Assert(err, gc.ErrorMatches, `invalid allocation, expecting <budget>:<limit>`)
+	c.Assert(err, gc.ErrorMatches, `invalid budget increase -1000`)
 	s.stub.CheckNoCalls(c)
 }
 
+func (s *registrationSuite) TestMeteredCharmDefaultBudgetAllocation(c *gc.C) {
+	client := httpbakery.NewClient()
+	d := DeploymentInfo{
+		CharmID: charmstore.CharmID{
+			URL: charm.MustParseURL("cs:quantal/metered-1"),
+		},
+		ApplicationName: "application name",
+		ModelUUID:       "model uuid",
+		CharmInfo: &apicharms.CharmInfo{
+			Metrics: &charm.Metrics{
+				Plan: &charm.Plan{Required: true},
+			},
+		},
+	}
+	s.register = &RegisterMeteredCharm{
+		Plan:           "someplan",
+		RegisterURL:    s.server.URL,
+		IncreaseBudget: 20,
+	}
+
+	err := s.register.RunPre(&mockMeteredDeployAPI{Stub: s.stub}, client, s.ctx, d)
+	c.Assert(err, jc.ErrorIsNil)
+	s.stub.CheckCalls(c, []testing.StubCall{{
+		FuncName: "IsMetered",
+		Args:     []interface{}{"cs:quantal/metered-1"},
+	}, {
+		FuncName: "Authorize",
+		Args: []interface{}{metricRegistrationPost{
+			ModelUUID:       "model uuid",
+			CharmURL:        "cs:quantal/metered-1",
+			ApplicationName: "application name",
+			PlanURL:         "someplan",
+			IncreaseBudget:  20,
+		},
+		},
+	},
+	})
+}
+
 func (s *registrationSuite) TestMeteredCharmDeployError(c *gc.C) {
 	client := httpbakery.NewClient()
 	d := DeploymentInfo{
@@ -252,8 +293,7 @@ func (s *registrationSuite) TestMeteredCharmDeployError(c *gc.C) {
 			CharmURL:        "cs:quantal/metered-1",
 			ApplicationName: "application name",
 			PlanURL:         "someplan",
-			Budget:          "personal",
-			Limit:           "100",
+			IncreaseBudget:  100,
 		}},
 	}})
 }
@@ -286,8 +326,7 @@ func (s *registrationSuite) TestMeteredLocalCharmWithPlan(c *gc.C) {
 			CharmURL:        "local:quantal/metered-1",
 			ApplicationName: "application name",
 			PlanURL:         "someplan",
-			Budget:          "personal",
-			Limit:           "100",
+			IncreaseBudget:  100,
 		}},
 	}, {
 		"SetMetricCredentials", []interface{}{
@@ -301,7 +340,7 @@ func (s *registrationSuite) TestMeteredLocalCharmNoPlan(c *gc.C) {
 	s.register = &RegisterMeteredCharm{
 		RegisterURL:    s.server.URL,
 		QueryURL:       s.server.URL,
-		AllocationSpec: "personal:100",
+		IncreaseBudget: 100,
 	}
 	client := httpbakery.NewClient()
 	d := DeploymentInfo{
@@ -330,8 +369,7 @@ func (s *registrationSuite) TestMeteredLocalCharmNoPlan(c *gc.C) {
 			CharmURL:        "local:quantal/metered-1",
 			ApplicationName: "application name",
 			PlanURL:         "",
-			Budget:          "personal",
-			Limit:           "100",
+			IncreaseBudget:  100,
 		}},
 	}, {
 		"SetMetricCredentials", []interface{}{
@@ -343,7 +381,7 @@ func (s *registrationSuite) TestMeteredLocalCharmNoPlan(c *gc.C) {
 
 func (s *registrationSuite) TestMeteredCharmNoPlanSet(c *gc.C) {
 	s.register = &RegisterMeteredCharm{
-		AllocationSpec: "personal:100",
+		IncreaseBudget: 100,
 		RegisterURL:    s.server.URL,
 		QueryURL:       s.server.URL}
 	client := httpbakery.NewClient()
@@ -376,8 +414,7 @@ func (s *registrationSuite) TestMeteredCharmNoPlanSet(c *gc.C) {
 			CharmURL:        "cs:quantal/metered-1",
 			ApplicationName: "application name",
 			PlanURL:         "thisplan",
-			Budget:          "personal",
-			Limit:           "100",
+			IncreaseBudget:  100,
 		}},
 	}, {
 		"SetMetricCredentials", []interface{}{
@@ -390,7 +427,7 @@ func (s *registrationSuite) TestMeteredCharmNoPlanSet(c *gc.C) {
 func (s *registrationSuite) TestMeteredCharmNoDefaultPlan(c *gc.C) {
 	s.stub.SetErrors(nil, errors.NotFoundf("default plan"))
 	s.register = &RegisterMeteredCharm{
-		AllocationSpec: "personal:100",
+		IncreaseBudget: 100,
 		RegisterURL:    s.server.URL,
 		QueryURL:       s.server.URL}
 	client := httpbakery.NewClient()
@@ -417,10 +454,41 @@ func (s *registrationSuite) TestMeteredCharmNoDefaultPlan(c *gc.C) {
 	}})
 }
 
+func (s *registrationSuite) TestMeteredCharmNoAvailablePlan(c *gc.C) {
+	s.stub.SetErrors(nil, errors.NotFoundf("default plan"))
+	s.handler.availablePlans = []availablePlanURL{}
+	s.register = &RegisterMeteredCharm{
+		IncreaseBudget: 100,
+		RegisterURL:    s.server.URL,
+		QueryURL:       s.server.URL}
+	client := httpbakery.NewClient()
+	d := DeploymentInfo{
+		CharmID: charmstore.CharmID{
+			URL: charm.MustParseURL("cs:quantal/metered-1"),
+		},
+		ApplicationName: "application name",
+		ModelUUID:       "model uuid",
+		CharmInfo: &apicharms.CharmInfo{
+			Metrics: &charm.Metrics{
+				Plan: &charm.Plan{Required: true},
+			},
+		},
+	}
+	err := s.register.RunPre(&mockMeteredDeployAPI{Stub: s.stub}, client, s.ctx, d)
+	c.Assert(err, gc.ErrorMatches, `no plans available for cs:quantal/metered-1.`)
+	s.stub.CheckCalls(c, []testing.StubCall{{
+		"IsMetered", []interface{}{"cs:quantal/metered-1"},
+	}, {
+		"DefaultPlan", []interface{}{"cs:quantal/metered-1"},
+	}, {
+		"ListPlans", []interface{}{"cs:quantal/metered-1"},
+	}})
+}
+
 func (s *registrationSuite) TestMeteredCharmFailToQueryDefaultCharm(c *gc.C) {
 	s.stub.SetErrors(nil, errors.New("something failed"))
 	s.register = &RegisterMeteredCharm{
-		AllocationSpec: "personal:100",
+		IncreaseBudget: 100,
 		RegisterURL:    s.server.URL,
 		QueryURL:       s.server.URL}
 	client := httpbakery.NewClient()
@@ -498,8 +566,7 @@ func (s *registrationSuite) TestFailedAuth(c *gc.C) {
 			CharmURL:        "cs:quantal/metered-1",
 			ApplicationName: "application name",
 			PlanURL:         "someplan",
-			Budget:          "personal",
-			Limit:           "100",
+			IncreaseBudget:  100,
 		}},
 	}})
 }
@@ -558,7 +625,7 @@ func (s *registrationSuite) TestPlanArgumentPlanRequiredInteraction(c *gc.C) {
 		}
 		s.register = &RegisterMeteredCharm{
 			Plan:           test.planArgument,
-			AllocationSpec: "personal:100",
+			IncreaseBudget: 100,
 			RegisterURL:    s.server.URL,
 			QueryURL:       s.server.URL,
 		}
@@ -587,8 +654,13 @@ func (s *registrationSuite) TestPlanArgumentPlanRequiredInteraction(c *gc.C) {
 	}
 }
 
+type availablePlanURL struct {
+	URL string `json:"url"`
+}
+
 type testMetricsRegistrationHandler struct {
 	*testing.Stub
+	availablePlans []availablePlanURL
 }
 
 type respErr struct {
@@ -648,13 +720,7 @@ func (c *testMetricsRegistrationHandler) ServeHTTP(w http.ResponseWriter, req *h
 			http.Error(w, rErr.Error(), http.StatusInternalServerError)
 			return
 		}
-		result := []struct {
-			URL string `json:"url"`
-		}{
-			{"thisplan"},
-			{"thisotherplan"},
-		}
-		err := json.NewEncoder(w).Encode(result)
+		err := json.NewEncoder(w).Encode(c.availablePlans)
 		if err != nil {
 			panic(err)
 		}
@@ -683,9 +749,9 @@ func (s *noPlanRegistrationSuite) SetUpTest(c *gc.C) {
 	s.register = &RegisterMeteredCharm{
 		Plan:           "",
 		RegisterURL:    s.server.URL,
-		AllocationSpec: "personal:100",
+		IncreaseBudget: 100,
 	}
-	s.ctx = coretesting.Context(c)
+	s.ctx = cmdtesting.Context(c)
 }
 
 func (s *noPlanRegistrationSuite) TearDownTest(c *gc.C) {
