@@ -12,12 +12,16 @@ import (
 	"text/template"
 
 	"github.com/juju/cmd"
+	"github.com/juju/errors"
 	"github.com/juju/loggo"
 	"github.com/juju/utils/featureflag"
 	utilsos "github.com/juju/utils/os"
+	proxyutils "github.com/juju/utils/proxy"
 	"github.com/juju/utils/series"
 	"github.com/juju/version"
 
+	// Import the providers.
+	cloudfile "github.com/juju/juju/cloud"
 	jujucmd "github.com/juju/juju/cmd"
 	"github.com/juju/juju/cmd/juju/action"
 	"github.com/juju/juju/cmd/juju/application"
@@ -44,9 +48,9 @@ import (
 	"github.com/juju/juju/juju"
 	"github.com/juju/juju/juju/osenv"
 	"github.com/juju/juju/jujuclient"
-	jujuversion "github.com/juju/juju/version"
-	// Import the providers.
 	_ "github.com/juju/juju/provider/all"
+	"github.com/juju/juju/utils/proxy"
+	jujuversion "github.com/juju/juju/version"
 )
 
 var logger = loggo.GetLogger("juju.cmd.juju.commands")
@@ -127,7 +131,7 @@ type main struct {
 func (m main) Run(args []string) int {
 	ctx, err := cmd.DefaultContext()
 	if err != nil {
-		fmt.Fprintf(os.Stderr, "error: %v\n", err)
+		cmd.WriteError(os.Stderr, err)
 		return 2
 	}
 
@@ -136,7 +140,12 @@ func (m main) Run(args []string) int {
 	newInstall := m.maybeWarnJuju1x()
 
 	if err = juju.InitJujuXDGDataHome(); err != nil {
-		fmt.Fprintf(os.Stderr, "error: %s\n", err)
+		cmd.WriteError(ctx.Stderr, err)
+		return 2
+	}
+
+	if err := installProxy(); err != nil {
+		cmd.WriteError(ctx.Stderr, err)
 		return 2
 	}
 
@@ -144,7 +153,7 @@ func (m main) Run(args []string) int {
 		fmt.Fprintf(ctx.Stderr, "Since Juju %v is being run for the first time, downloading latest cloud information.\n", jujuversion.Current.Major)
 		updateCmd := cloud.NewUpdateCloudsCommand()
 		if err := updateCmd.Run(ctx); err != nil {
-			fmt.Fprintf(ctx.Stderr, "error: %v\n", err)
+			cmd.WriteError(ctx.Stderr, err)
 		}
 	}
 
@@ -160,6 +169,18 @@ func (m main) Run(args []string) int {
 	return cmd.Main(jcmd, ctx, args[1:])
 }
 
+func installProxy() error {
+	// Set the default transport to use the in-process proxy
+	// configuration.
+	if err := proxy.DefaultConfig.Set(proxyutils.DetectProxies()); err != nil {
+		return errors.Trace(err)
+	}
+	if err := proxy.DefaultConfig.InstallInDefaultTransport(); err != nil {
+		return errors.Trace(err)
+	}
+	return nil
+}
+
 func (m main) maybeWarnJuju1x() (newInstall bool) {
 	newInstall = !juju2xConfigDataExists()
 	if !shouldWarnJuju1x() {
@@ -209,7 +230,7 @@ func (m main) juju1xVersion() (ver string, exists bool) {
 
 func shouldWarnJuju1x() bool {
 	// this code only applies to Ubuntu, where we renamed Juju 1.x to juju-1.
-	ostype, err := series.GetOSFromSeries(series.HostSeries())
+	ostype, err := series.GetOSFromSeries(series.MustHostSeries())
 	if err != nil || ostype != utilsos.Ubuntu {
 		return false
 	}
@@ -254,11 +275,12 @@ func registerCommands(r commandRegistry, ctx *cmd.Context) {
 		r.Register(crossmodel.NewShowOfferedEndpointCommand())
 		r.Register(crossmodel.NewListEndpointsCommand())
 		r.Register(crossmodel.NewFindEndpointsCommand())
+		r.Register(application.NewConsumeCommand())
 	}
 
 	// Destruction commands.
 	r.Register(application.NewRemoveRelationCommand())
-	r.Register(application.NewRemoveServiceCommand())
+	r.Register(application.NewRemoveApplicationCommand())
 	r.Register(application.NewRemoveUnitCommand())
 
 	// Reporting commands.
@@ -267,7 +289,7 @@ func registerCommands(r commandRegistry, ctx *cmd.Context) {
 	r.Register(status.NewStatusHistoryCommand())
 
 	// Error resolution and debugging commands.
-	r.Register(newRunCommand())
+	r.Register(newDefaultRunCommand())
 	r.Register(newSCPCommand(nil))
 	r.Register(newSSHCommand(nil))
 	r.Register(newResolvedCommand())
@@ -331,9 +353,7 @@ func registerCommands(r commandRegistry, ctx *cmd.Context) {
 	r.Register(model.NewRevokeCommand())
 	r.Register(model.NewShowCommand())
 
-	if featureflag.Enabled(feature.Migration) {
-		r.Register(newMigrateCommand())
-	}
+	r.Register(newMigrateCommand())
 	if featureflag.Enabled(feature.DeveloperMode) {
 		r.Register(model.NewDumpCommand())
 		r.Register(model.NewDumpDBCommand())
@@ -344,6 +364,7 @@ func registerCommands(r commandRegistry, ctx *cmd.Context) {
 	r.Register(action.NewRunCommand())
 	r.Register(action.NewShowOutputCommand())
 	r.Register(action.NewListCommand())
+	r.Register(action.NewCancelCommand())
 
 	// Manage controller availability
 	r.Register(newEnableHACommand())
@@ -351,7 +372,7 @@ func registerCommands(r commandRegistry, ctx *cmd.Context) {
 	// Manage and control services
 	r.Register(application.NewAddUnitCommand())
 	r.Register(application.NewConfigCommand())
-	r.Register(application.NewDefaultDeployCommand())
+	r.Register(application.NewDeployCommand())
 	r.Register(application.NewExposeCommand())
 	r.Register(application.NewUnexposeCommand())
 	r.Register(application.NewServiceGetConstraintsCommand())
@@ -368,10 +389,16 @@ func registerCommands(r commandRegistry, ctx *cmd.Context) {
 	r.Register(storage.NewPoolCreateCommand())
 	r.Register(storage.NewPoolListCommand())
 	r.Register(storage.NewShowCommand())
+	r.Register(storage.NewRemoveStorageCommandWithAPI())
+	if featureflag.Enabled(feature.PersistentStorage) {
+		r.Register(storage.NewDetachStorageCommandWithAPI())
+		r.Register(storage.NewAttachStorageCommandWithAPI())
+	}
 
 	// Manage spaces
 	r.Register(space.NewAddCommand())
 	r.Register(space.NewListCommand())
+	r.Register(space.NewReloadCommand())
 	if featureflag.Enabled(feature.PostNetCLIMVP) {
 		r.Register(space.NewRemoveCommand())
 		r.Register(space.NewUpdateCommand())
@@ -408,7 +435,7 @@ func registerCommands(r commandRegistry, ctx *cmd.Context) {
 	r.Register(cloud.NewListCloudsCommand())
 	r.Register(cloud.NewListRegionsCommand())
 	r.Register(cloud.NewShowCloudCommand())
-	r.Register(cloud.NewAddCloudCommand())
+	r.Register(cloud.NewAddCloudCommand(&cloudToCommandAdapter{}))
 	r.Register(cloud.NewRemoveCloudCommand())
 	r.Register(cloud.NewListCredentialsCommand())
 	r.Register(cloud.NewDetectCredentialsCommand())
@@ -433,3 +460,21 @@ func registerCommands(r commandRegistry, ctx *cmd.Context) {
 	}
 	rcmd.RegisterAll(r)
 }
+
+type cloudToCommandAdapter struct{}
+
+func (cloudToCommandAdapter) ParseCloudMetadataFile(path string) (map[string]cloudfile.Cloud, error) {
+	return cloudfile.ParseCloudMetadataFile(path)
+}
+func (cloudToCommandAdapter) ParseOneCloud(data []byte) (cloudfile.Cloud, error) {
+	return cloudfile.ParseOneCloud(data)
+}
+func (cloudToCommandAdapter) PublicCloudMetadata(searchPaths ...string) (map[string]cloudfile.Cloud, bool, error) {
+	return cloudfile.PublicCloudMetadata(searchPaths...)
+}
+func (cloudToCommandAdapter) PersonalCloudMetadata() (map[string]cloudfile.Cloud, error) {
+	return cloudfile.PersonalCloudMetadata()
+}
+func (cloudToCommandAdapter) WritePersonalCloudMetadata(cloudsMap map[string]cloudfile.Cloud) error {
+	return cloudfile.WritePersonalCloudMetadata(cloudsMap)
+}
