@@ -13,6 +13,7 @@ import (
 	"strings"
 
 	"github.com/juju/cmd"
+	"github.com/juju/cmd/cmdtesting"
 	gitjujutesting "github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils/arch"
@@ -25,7 +26,6 @@ import (
 	"github.com/juju/juju/cmd/juju/application"
 	"github.com/juju/juju/cmd/juju/cloud"
 	"github.com/juju/juju/cmd/modelcmd"
-	cmdtesting "github.com/juju/juju/cmd/testing"
 	"github.com/juju/juju/feature"
 	"github.com/juju/juju/juju/osenv"
 	_ "github.com/juju/juju/provider/dummy"
@@ -41,7 +41,7 @@ type MainSuite struct {
 var _ = gc.Suite(&MainSuite{})
 
 func deployHelpText() string {
-	return cmdtesting.HelpText(application.NewDefaultDeployCommand(), "juju deploy")
+	return cmdtesting.HelpText(application.NewDeployCommand(), "juju deploy")
 }
 func configHelpText() string {
 	return cmdtesting.HelpText(application.NewConfigCommand(), "juju config")
@@ -102,17 +102,17 @@ func (s *MainSuite) TestRunMain(c *gc.C) {
 		summary: "unknown option before command",
 		args:    []string{"--cheese", "bootstrap"},
 		code:    2,
-		out:     "error: flag provided but not defined: --cheese\n",
+		out:     "ERROR flag provided but not defined: --cheese\n",
 	}, {
 		summary: "unknown option after command",
 		args:    []string{"bootstrap", "--cheese"},
 		code:    2,
-		out:     "error: flag provided but not defined: --cheese\n",
+		out:     "ERROR flag provided but not defined: --cheese\n",
 	}, {
 		summary: "known option, but specified before command",
 		args:    []string{"--model", "blah", "bootstrap"},
 		code:    2,
-		out:     "error: flag provided but not defined: --model\n",
+		out:     "ERROR flag provided but not defined: --model\n",
 	}, {
 		summary: "juju sync-tools registered properly",
 		args:    []string{"sync-tools", "--help"},
@@ -125,7 +125,7 @@ func (s *MainSuite) TestRunMain(c *gc.C) {
 		out: version.Binary{
 			Number: jujuversion.Current,
 			Arch:   arch.HostArch(),
-			Series: series.HostSeries(),
+			Series: series.MustHostSeries(),
 		}.String() + "\n",
 	}} {
 		c.Logf("test %d: %s", i, t.summary)
@@ -144,9 +144,9 @@ func (s *MainSuite) TestActualRunJujuArgOrder(c *gc.C) {
 	s.PatchEnvironment(osenv.JujuModelEnvKey, "current")
 	logpath := filepath.Join(c.MkDir(), "log")
 	tests := [][]string{
-		{"--log-file", logpath, "--debug", "controllers"}, // global flags before
-		{"controllers", "--log-file", logpath, "--debug"}, // after
-		{"--log-file", logpath, "controllers", "--debug"}, // mixed
+		{"--log-file", logpath, "--debug", "help"}, // global flags before
+		{"help", "--log-file", logpath, "--debug"}, // after
+		{"--log-file", logpath, "help", "--debug"}, // mixed
 	}
 	for i, test := range tests {
 		c.Logf("test %d: %v", i, test)
@@ -167,7 +167,7 @@ func (s *MainSuite) TestFirstRun2xFrom1xOnUbuntu(c *gc.C) {
 
 	// Code should only run on ubuntu series, so patch out the series for
 	// when non-ubuntu OSes run this test.
-	s.PatchValue(&series.HostSeries, func() string { return "trusty" })
+	s.PatchValue(&series.MustHostSeries, func() string { return "trusty" })
 
 	argChan := make(chan []string, 1)
 
@@ -215,7 +215,7 @@ Since Juju 2 is being run for the first time, downloading latest cloud informati
 
 func (s *MainSuite) TestFirstRun2xFrom1xNotUbuntu(c *gc.C) {
 	// Code should only run on ubuntu series, so pretend to be something else.
-	s.PatchValue(&series.HostSeries, func() string { return "win8" })
+	s.PatchValue(&series.MustHostSeries, func() string { return "win8" })
 
 	argChan := make(chan []string, 1)
 
@@ -255,7 +255,7 @@ Since Juju 2 is being run for the first time, downloading latest cloud informati
 func (s *MainSuite) TestNoWarn1xWith2xData(c *gc.C) {
 	// Code should only rnu on ubuntu series, so patch out the series for
 	// when non-ubuntu OSes run this test.
-	s.PatchValue(&series.HostSeries, func() string { return "trusty" })
+	s.PatchValue(&series.MustHostSeries, func() string { return "trusty" })
 
 	argChan := make(chan []string, 1)
 
@@ -288,7 +288,7 @@ func (s *MainSuite) TestNoWarn1xWith2xData(c *gc.C) {
 func (s *MainSuite) TestNoWarnWithNo1xOr2xData(c *gc.C) {
 	// Code should only rnu on ubuntu series, so patch out the series for
 	// when non-ubuntu OSes run this test.
-	s.PatchValue(&series.HostSeries, func() string { return "trusty" })
+	s.PatchValue(&series.MustHostSeries, func() string { return "trusty" })
 
 	argChan := make(chan []string, 1)
 	// we shouldn't actually be running anything, but if we do, this will
@@ -370,7 +370,7 @@ func checkVersionOutput(c *gc.C, output string) {
 	ver := version.Binary{
 		Number: jujuversion.Current,
 		Arch:   arch.HostArch(),
-		Series: series.HostSeries(),
+		Series: series.MustHostSeries(),
 	}
 
 	c.Check(output, gc.Equals, ver.String()+"\n")
@@ -401,26 +401,26 @@ var commandNames = []string{
 	"add-user",
 	"agree",
 	"agreements",
-	"allocate",
+	"attach",
 	"autoload-credentials",
 	"backups",
 	"bootstrap",
-	"budgets",
+	"budget",
 	"cached-images",
+	"cancel-action",
 	"change-user-password",
 	"charm",
 	"clouds",
-	"config",
 	"collect-metrics",
+	"config",
+	"controller-config",
 	"controllers",
 	"create-backup",
-	"create-budget",
 	"create-storage-pool",
+	"create-wallet",
 	"credentials",
-	"controller-config",
 	"debug-hooks",
 	"debug-log",
-	"remove-user",
 	"deploy",
 	"destroy-controller",
 	"destroy-model",
@@ -428,9 +428,9 @@ var commandNames = []string{
 	"disable-user",
 	"disabled-commands",
 	"download-backup",
-	"enable-ha",
 	"enable-command",
 	"enable-destroy-controller",
+	"enable-ha",
 	"enable-user",
 	"expose",
 	"get-constraints",
@@ -444,7 +444,6 @@ var commandNames = []string{
 	"list-actions",
 	"list-agreements",
 	"list-backups",
-	"list-budgets",
 	"list-cached-images",
 	"list-clouds",
 	"list-controllers",
@@ -452,25 +451,31 @@ var commandNames = []string{
 	"list-disabled-commands",
 	"list-machines",
 	"list-models",
+	"list-payloads",
 	"list-plans",
 	"list-regions",
-	"list-ssh-keys",
+	"list-resources",
 	"list-spaces",
+	"list-ssh-keys",
 	"list-storage",
 	"list-storage-pools",
 	"list-subnets",
 	"list-users",
+	"list-wallets",
 	"login",
 	"logout",
 	"machines",
 	"metrics",
+	"migrate",
 	"model-config",
 	"model-defaults",
 	"models",
+	"payloads",
 	"plans",
 	"regions",
 	"register",
 	"relate", //alias for add-relation
+	"reload-spaces",
 	"remove-application",
 	"remove-backup",
 	"remove-cached-images",
@@ -479,25 +484,27 @@ var commandNames = []string{
 	"remove-machine",
 	"remove-relation",
 	"remove-ssh-key",
+	"remove-storage",
 	"remove-unit",
+	"remove-user",
 	"resolved",
+	"resources",
 	"restore-backup",
 	"retry-provisioning",
 	"revoke",
 	"run",
 	"run-action",
 	"scp",
-	"set-budget",
 	"set-constraints",
 	"set-default-credential",
 	"set-default-region",
 	"set-meter-status",
 	"set-model-constraints",
 	"set-plan",
+	"set-wallet",
 	"show-action-output",
 	"show-action-status",
 	"show-backup",
-	"show-budget",
 	"show-cloud",
 	"show-controller",
 	"show-machine",
@@ -506,6 +513,8 @@ var commandNames = []string{
 	"show-status-log",
 	"show-storage",
 	"show-user",
+	"show-wallet",
+	"sla",
 	"spaces",
 	"ssh",
 	"ssh-keys",
@@ -516,27 +525,32 @@ var commandNames = []string{
 	"switch",
 	"sync-tools",
 	"unexpose",
-	"update-allocation",
-	"upload-backup",
 	"unregister",
 	"update-clouds",
 	"update-credential",
 	"upgrade-charm",
 	"upgrade-gui",
 	"upgrade-juju",
+	"upload-backup",
 	"users",
 	"version",
+	"wallets",
 	"whoami",
 }
 
 // devFeatures are feature flags that impact registration of commands.
-var devFeatures = []string{feature.Migration, feature.CrossModelRelations}
+var devFeatures = []string{
+	feature.CrossModelRelations,
+	feature.PersistentStorage,
+}
 
 // These are the commands that are behind the `devFeatures`.
 var commandNamesBehindFlags = set.NewStrings(
+	"attach-storage",
+	"consume",
+	"detach-storage",
 	"find-endpoints",
 	"list-offers",
-	"migrate",
 	"offer",
 	"offers",
 	"show-endpoints",
@@ -643,7 +657,7 @@ func (s *MainSuite) TestRegisterCommands(c *gc.C) {
 
 	registry := &stubRegistry{stub: stub}
 	registry.names = append(registry.names, "help", "version") // implicit
-	registerCommands(registry, testing.Context(c))
+	registerCommands(registry, cmdtesting.Context(c))
 	sort.Strings(registry.names)
 
 	expected := make([]string, len(commandNames))
@@ -671,7 +685,7 @@ func (r *commands) RegisterSuperAlias(name, super, forName string, check cmd.Dep
 
 func (s *MainSuite) TestModelCommands(c *gc.C) {
 	var commands commands
-	registerCommands(&commands, testing.Context(c))
+	registerCommands(&commands, cmdtesting.Context(c))
 	// There should not be any ModelCommands registered.
 	// ModelCommands must be wrapped using modelcmd.Wrap.
 	for _, cmd := range commands {
@@ -680,22 +694,23 @@ func (s *MainSuite) TestModelCommands(c *gc.C) {
 	}
 }
 
-func (s *MainSuite) TestAllCommandsPurposeDocCapitalization(c *gc.C) {
+func (s *MainSuite) TestAllCommandsPurpose(c *gc.C) {
 	// Verify each command that:
-	// - the Purpose field is not empty
+	// - the Purpose field is not empty.
+	// - the Purpose ends with a full stop.
 	// - if set, the Doc field either begins with the name of the
 	// command or and uppercase letter.
 	//
-	// The first makes Purpose a required documentation. Also, makes
-	// both "help commands"'s output and "help <cmd>"'s header more
-	// uniform. The second makes the Doc content either start like a
-	// sentence, or start godoc-like by using the command's name in
-	// lowercase.
+	// This:
+	// - makes Purpose a required documentation.
+	// - Standardises Purpose formatting across all commands.
+	// - Brings "help commands"'s output in line with "help <cmd>"'s header.
+	// - Makes the Doc content either start like a sentence, or start
+	//   godoc-like by using the command's name in lowercase.
 	var commands commands
-	registerCommands(&commands, testing.Context(c))
+	registerCommands(&commands, cmdtesting.Context(c))
 	for _, cmd := range commands {
 		info := cmd.Info()
-		c.Logf("%v", info.Name)
 		purpose := strings.TrimSpace(info.Purpose)
 		doc := strings.TrimSpace(info.Doc)
 		comment := func(message string) interface{} {
@@ -706,8 +721,9 @@ func (s *MainSuite) TestAllCommandsPurposeDocCapitalization(c *gc.C) {
 		if purpose != "" {
 			prefix := string(purpose[0])
 			c.Check(prefix, gc.Equals, strings.ToUpper(prefix),
-				comment("expected uppercase first-letter Purpose"),
-			)
+				comment("expected uppercase first-letter Purpose"))
+			c.Check(strings.HasSuffix(purpose, "."), jc.IsTrue,
+				comment("is missing full stop in Purpose"))
 		}
 		if doc != "" && !strings.HasPrefix(doc, info.Name) {
 			prefix := string(doc[0])
