@@ -11,6 +11,7 @@ import (
 	"io/ioutil"
 	"strings"
 
+	"github.com/juju/cmd/cmdtesting"
 	"github.com/juju/errors"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils/arch"
@@ -183,35 +184,35 @@ var upgradeJujuTests = []struct {
 	currentVersion: "3.0.0-quantal-amd64",
 	agentVersion:   "3.0.0",
 	args:           []string{"--agent-version", "3.2.0"},
-	expectErr:      "no tools available",
+	expectErr:      "no agent binaries available",
 }, {
 	about:          "specified version, no matching major version",
 	tools:          []string{"4.2.0-quantal-amd64"},
 	currentVersion: "3.0.0-quantal-amd64",
 	agentVersion:   "3.0.0",
 	args:           []string{"--agent-version", "3.2.0"},
-	expectErr:      "no matching tools available",
+	expectErr:      "no matching agent binaries available",
 }, {
 	about:          "specified version, no matching minor version",
 	tools:          []string{"3.4.0-quantal-amd64"},
 	currentVersion: "3.0.0-quantal-amd64",
 	agentVersion:   "3.0.0",
 	args:           []string{"--agent-version", "3.2.0"},
-	expectErr:      "no matching tools available",
+	expectErr:      "no matching agent binaries available",
 }, {
 	about:          "specified version, no matching patch version",
 	tools:          []string{"3.2.5-quantal-amd64"},
 	currentVersion: "3.0.0-quantal-amd64",
 	agentVersion:   "3.0.0",
 	args:           []string{"--agent-version", "3.2.0"},
-	expectErr:      "no matching tools available",
+	expectErr:      "no matching agent binaries available",
 }, {
 	about:          "specified version, no matching build version",
 	tools:          []string{"3.2.0.2-quantal-amd64"},
 	currentVersion: "3.0.0-quantal-amd64",
 	agentVersion:   "3.0.0",
 	args:           []string{"--agent-version", "3.2.0"},
-	expectErr:      "no matching tools available",
+	expectErr:      "no matching agent binaries available",
 }, {
 	about:          "incompatible version (minor != 0)",
 	tools:          []string{"3.2.0-quantal-amd64"},
@@ -314,9 +315,9 @@ func (s *UpgradeJujuSuite) TestUpgradeJuju(c *gc.C) {
 		current := version.MustParseBinary(test.currentVersion)
 		s.PatchValue(&jujuversion.Current, current.Number)
 		s.PatchValue(&arch.HostArch, func() string { return current.Arch })
-		s.PatchValue(&series.HostSeries, func() string { return current.Series })
+		s.PatchValue(&series.MustHostSeries, func() string { return current.Series })
 		com := newUpgradeJujuCommand(test.upgradeMap)
-		if err := coretesting.InitCommand(com, test.args); err != nil {
+		if err := cmdtesting.InitCommand(com, test.args); err != nil {
 			if test.expectInitErr != "" {
 				c.Check(err, gc.ErrorMatches, test.expectInitErr)
 			} else {
@@ -331,7 +332,7 @@ func (s *UpgradeJujuSuite) TestUpgradeJuju(c *gc.C) {
 			"agent-version":      test.agentVersion,
 			"agent-metadata-url": "file://" + toolsDir + "/tools",
 		}
-		err := s.State.UpdateModelConfig(updateAttrs, nil, nil)
+		err := s.State.UpdateModelConfig(updateAttrs, nil)
 		c.Assert(err, jc.ErrorIsNil)
 		versions := make([]version.Binary, len(test.tools))
 		for i, v := range test.tools {
@@ -343,7 +344,7 @@ func (s *UpgradeJujuSuite) TestUpgradeJuju(c *gc.C) {
 			envtesting.MustUploadFakeToolsVersions(stor, s.Environ.Config().AgentStream(), versions...)
 		}
 
-		err = com.Run(coretesting.Context(c))
+		err = com.Run(cmdtesting.Context(c))
 		if test.expectErr != "" {
 			c.Check(err, gc.ErrorMatches, test.expectErr)
 			continue
@@ -381,7 +382,7 @@ func (s *UpgradeJujuSuite) checkToolsUploaded(c *gc.C, vers version.Binary, agen
 	expectContent := version.Binary{
 		Number: agentVersion,
 		Arch:   arch.HostArch(),
-		Series: series.HostSeries(),
+		Series: series.MustHostSeries(),
 	}
 	checkToolsContent(c, data, "jujud contents "+expectContent.String())
 }
@@ -423,7 +424,7 @@ func (s *UpgradeJujuSuite) Reset(c *gc.C) {
 		"default-series": "raring",
 		"agent-version":  "1.2.3",
 	}
-	err := s.State.UpdateModelConfig(updateAttrs, nil, nil)
+	err := s.State.UpdateModelConfig(updateAttrs, nil)
 	c.Assert(err, jc.ErrorIsNil)
 	s.PatchValue(&sync.BuildAgentTarball, toolstesting.GetMockBuildTools(c))
 
@@ -443,12 +444,12 @@ func (s *UpgradeJujuSuite) TestUpgradeJujuWithRealUpload(c *gc.C) {
 	s.Reset(c)
 	s.PatchValue(&jujuversion.Current, version.MustParse("1.99.99"))
 	cmd := newUpgradeJujuCommand(map[int]version.Number{2: version.MustParse("1.99.99")})
-	_, err := coretesting.RunCommand(c, cmd, "--build-agent")
+	_, err := cmdtesting.RunCommand(c, cmd, "--build-agent")
 	c.Assert(err, jc.ErrorIsNil)
 	vers := version.Binary{
 		Number: jujuversion.Current,
 		Arch:   arch.HostArch(),
-		Series: series.HostSeries(),
+		Series: series.MustHostSeries(),
 	}
 	vers.Build = 1
 	s.checkToolsUploaded(c, vers, vers.Number)
@@ -470,7 +471,7 @@ func (s *UpgradeJujuSuite) TestUpgradeJujuWithImplicitUploadDevAgent(c *gc.C) {
 	})
 	s.PatchValue(&jujuversion.Current, version.MustParse("1.99.99"))
 	cmd := newUpgradeJujuCommand(nil)
-	_, err := coretesting.RunCommand(c, cmd)
+	_, err := cmdtesting.RunCommand(c, cmd)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(fakeAPI.tools, gc.Not(gc.HasLen), 0)
 	c.Assert(fakeAPI.tools[0].Version.Number, gc.Equals, version.MustParse("1.99.99.1"))
@@ -492,11 +493,12 @@ func (s *UpgradeJujuSuite) TestUpgradeJujuWithImplicitUploadNewerClient(c *gc.C)
 	})
 	s.PatchValue(&jujuversion.Current, version.MustParse("1.100.0"))
 	cmd := newUpgradeJujuCommand(nil)
-	_, err := coretesting.RunCommand(c, cmd)
+	_, err := cmdtesting.RunCommand(c, cmd)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(fakeAPI.tools, gc.Not(gc.HasLen), 0)
 	c.Assert(fakeAPI.tools[0].Version.Number, gc.Equals, version.MustParse("1.100.0.1"))
 	c.Assert(fakeAPI.modelAgentVersion, gc.Equals, fakeAPI.tools[0].Version.Number)
+	c.Assert(fakeAPI.ignoreAgentVersions, jc.IsFalse)
 }
 
 func (s *UpgradeJujuSuite) TestUpgradeJujuWithImplicitUploadNonController(c *gc.C) {
@@ -515,8 +517,9 @@ func (s *UpgradeJujuSuite) TestUpgradeJujuWithImplicitUploadNonController(c *gc.
 	})
 	s.PatchValue(&jujuversion.Current, version.MustParse("1.99.99"))
 	cmd := newUpgradeJujuCommand(nil)
-	_, err := coretesting.RunCommand(c, cmd)
+	_, err := cmdtesting.RunCommand(c, cmd)
 	c.Assert(err, gc.ErrorMatches, "no more recent supported versions available")
+	c.Assert(fakeAPI.ignoreAgentVersions, jc.IsFalse)
 }
 
 func (s *UpgradeJujuSuite) TestBlockUpgradeJujuWithRealUpload(c *gc.C) {
@@ -525,7 +528,7 @@ func (s *UpgradeJujuSuite) TestBlockUpgradeJujuWithRealUpload(c *gc.C) {
 	cmd := newUpgradeJujuCommand(map[int]version.Number{2: version.MustParse("1.99.99")})
 	// Block operation
 	s.BlockAllChanges(c, "TestBlockUpgradeJujuWithRealUpload")
-	_, err := coretesting.RunCommand(c, cmd, "--build-agent")
+	_, err := cmdtesting.RunCommand(c, cmd, "--build-agent")
 	coretesting.AssertOperationWasBlocked(c, err, ".*TestBlockUpgradeJujuWithRealUpload.*")
 }
 
@@ -543,10 +546,34 @@ func (s *UpgradeJujuSuite) TestFailUploadOnNonController(c *gc.C) {
 		return fakeAPI, nil
 	})
 	cmd := newUpgradeJujuCommand(nil)
-	_, err := coretesting.RunCommand(c, cmd, "--build-agent", "-m", "dummy-model")
+	_, err := cmdtesting.RunCommand(c, cmd, "--build-agent", "-m", "dummy-model")
 	c.Assert(err, gc.ErrorMatches, "--build-agent can only be used with the controller model")
 }
 
+func (s *UpgradeJujuSuite) TestUpgradeJujuWithIgnoreAgentVersions(c *gc.C) {
+	s.Reset(c)
+	fakeAPI := &fakeUpgradeJujuAPINoState{
+		name:           "dummy-model",
+		uuid:           "deadbeef-0bad-400d-8000-4b1d0d06f00d",
+		controllerUUID: "deadbeef-1bad-500d-9000-4b1d0d06f00d",
+		agentVersion:   "1.99.99",
+	}
+	s.PatchValue(&getUpgradeJujuAPI, func(*upgradeJujuCommand) (upgradeJujuAPI, error) {
+		return fakeAPI, nil
+	})
+	s.PatchValue(&getModelConfigAPI, func(*upgradeJujuCommand) (modelConfigAPI, error) {
+		return fakeAPI, nil
+	})
+	s.PatchValue(&jujuversion.Current, version.MustParse("1.100.0"))
+	cmd := newUpgradeJujuCommand(nil)
+	_, err := cmdtesting.RunCommand(c, cmd, "--ignore-agent-versions")
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(fakeAPI.tools, gc.Not(gc.HasLen), 0)
+	c.Assert(fakeAPI.tools[0].Version.Number, gc.Equals, version.MustParse("1.100.0.1"))
+	c.Assert(fakeAPI.modelAgentVersion, gc.Equals, fakeAPI.tools[0].Version.Number)
+	c.Assert(fakeAPI.ignoreAgentVersions, jc.IsTrue)
+}
+
 type DryRunTest struct {
 	about             string
 	cmdArgs           []string
@@ -598,10 +625,10 @@ func (s *UpgradeJujuSuite) TestUpgradeDryRun(c *gc.C) {
 		s.setUpEnvAndTools(c, test.currentVersion, test.agentVersion, test.tools)
 
 		com := newUpgradeJujuCommand(nil)
-		err := coretesting.InitCommand(com, test.cmdArgs)
+		err := cmdtesting.InitCommand(com, test.cmdArgs)
 		c.Assert(err, jc.ErrorIsNil)
 
-		ctx := coretesting.Context(c)
+		ctx := cmdtesting.Context(c)
 		err = com.Run(ctx)
 		c.Assert(err, jc.ErrorIsNil)
 
@@ -611,7 +638,7 @@ func (s *UpgradeJujuSuite) TestUpgradeDryRun(c *gc.C) {
 		agentVer, ok := cfg.AgentVersion()
 		c.Assert(ok, jc.IsTrue)
 		c.Assert(agentVer, gc.Equals, version.MustParse(test.agentVersion))
-		output := coretesting.Stderr(ctx)
+		output := cmdtesting.Stderr(ctx)
 		c.Assert(output, gc.Equals, test.expectedCmdOutput)
 	}
 }
@@ -620,7 +647,7 @@ func (s *UpgradeJujuSuite) setUpEnvAndTools(c *gc.C, currentVersion string, agen
 	current := version.MustParseBinary(currentVersion)
 	s.PatchValue(&jujuversion.Current, current.Number)
 	s.PatchValue(&arch.HostArch, func() string { return current.Arch })
-	s.PatchValue(&series.HostSeries, func() string { return current.Series })
+	s.PatchValue(&series.MustHostSeries, func() string { return current.Series })
 
 	toolsDir := c.MkDir()
 	updateAttrs := map[string]interface{}{
@@ -628,7 +655,7 @@ func (s *UpgradeJujuSuite) setUpEnvAndTools(c *gc.C, currentVersion string, agen
 		"agent-metadata-url": "file://" + toolsDir + "/tools",
 	}
 
-	err := s.State.UpdateModelConfig(updateAttrs, nil, nil)
+	err := s.State.UpdateModelConfig(updateAttrs, nil)
 	c.Assert(err, jc.ErrorIsNil)
 	versions := make([]version.Binary, len(tools))
 	for i, v := range tools {
@@ -742,10 +769,10 @@ func (s *UpgradeJujuSuite) TestUpgradesDifferentMajor(c *gc.C) {
 		s.setUpEnvAndTools(c, test.currentVersion, test.agentVersion, test.tools)
 
 		com := newUpgradeJujuCommand(test.upgradeMap)
-		err := coretesting.InitCommand(com, test.cmdArgs)
+		err := cmdtesting.InitCommand(com, test.cmdArgs)
 		c.Assert(err, jc.ErrorIsNil)
 
-		ctx := coretesting.Context(c)
+		ctx := cmdtesting.Context(c)
 		err = com.Run(ctx)
 		if test.expectedErr != "" {
 			c.Check(err, gc.ErrorMatches, test.expectedErr)
@@ -759,7 +786,7 @@ func (s *UpgradeJujuSuite) TestUpgradesDifferentMajor(c *gc.C) {
 		agentVer, ok := cfg.AgentVersion()
 		c.Assert(ok, jc.IsTrue)
 		c.Check(agentVer, gc.Equals, version.MustParse(test.expectedVersion))
-		output := coretesting.Stderr(ctx)
+		output := cmdtesting.Stderr(ctx)
 		if test.expectedCmdOutput != "" {
 			c.Check(output, gc.Equals, test.expectedCmdOutput)
 		}
@@ -778,10 +805,10 @@ func (s *UpgradeJujuSuite) TestUpgradeUnknownSeriesInStreams(c *gc.C) {
 	fakeAPI.patch(s)
 
 	cmd := &upgradeJujuCommand{}
-	err := coretesting.InitCommand(modelcmd.Wrap(cmd), []string{})
+	err := cmdtesting.InitCommand(modelcmd.Wrap(cmd), []string{})
 	c.Assert(err, jc.ErrorIsNil)
 
-	err = modelcmd.Wrap(cmd).Run(coretesting.Context(c))
+	err = modelcmd.Wrap(cmd).Run(cmdtesting.Context(c))
 	c.Assert(err, gc.IsNil)
 
 	// ensure find tools was called
@@ -797,10 +824,10 @@ func (s *UpgradeJujuSuite) TestUpgradeInProgress(c *gc.C) {
 	}
 	fakeAPI.patch(s)
 	cmd := &upgradeJujuCommand{}
-	err := coretesting.InitCommand(modelcmd.Wrap(cmd), []string{})
+	err := cmdtesting.InitCommand(modelcmd.Wrap(cmd), []string{})
 	c.Assert(err, jc.ErrorIsNil)
 
-	err = modelcmd.Wrap(cmd).Run(coretesting.Context(c))
+	err = modelcmd.Wrap(cmd).Run(cmdtesting.Context(c))
 	c.Assert(err, gc.ErrorMatches, "a message from the server about the problem\n"+
 		"\n"+
 		"Please wait for the upgrade to complete or if there was a problem with\n"+
@@ -814,12 +841,12 @@ func (s *UpgradeJujuSuite) TestBlockUpgradeInProgress(c *gc.C) {
 	fakeAPI.setVersionErr = common.OperationBlockedError("the operation has been blocked")
 	fakeAPI.patch(s)
 	cmd := &upgradeJujuCommand{}
-	err := coretesting.InitCommand(modelcmd.Wrap(cmd), []string{})
+	err := cmdtesting.InitCommand(modelcmd.Wrap(cmd), []string{})
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Block operation
 	s.BlockAllChanges(c, "TestBlockUpgradeInProgress")
-	err = modelcmd.Wrap(cmd).Run(coretesting.Context(c))
+	err = modelcmd.Wrap(cmd).Run(cmdtesting.Context(c))
 	s.AssertBlocked(c, err, ".*To enable changes.*")
 }
 
@@ -827,7 +854,7 @@ func (s *UpgradeJujuSuite) TestResetPreviousUpgrade(c *gc.C) {
 	fakeAPI := NewFakeUpgradeJujuAPI(c, s.State)
 	fakeAPI.patch(s)
 
-	ctx := coretesting.Context(c)
+	ctx := cmdtesting.Context(c)
 	var stdin bytes.Buffer
 	ctx.Stdin = &stdin
 
@@ -840,7 +867,7 @@ func (s *UpgradeJujuSuite) TestResetPreviousUpgrade(c *gc.C) {
 		fakeAPI.reset()
 
 		cmd := &upgradeJujuCommand{}
-		err := coretesting.InitCommand(modelcmd.Wrap(cmd),
+		err := cmdtesting.InitCommand(modelcmd.Wrap(cmd),
 			append([]string{"--reset-previous-upgrade"}, args...))
 		c.Assert(err, jc.ErrorIsNil)
 		err = modelcmd.Wrap(cmd).Run(ctx)
@@ -856,6 +883,7 @@ func (s *UpgradeJujuSuite) TestResetPreviousUpgrade(c *gc.C) {
 			expectedVersion = fakeAPI.nextVersion.Number
 		}
 		c.Assert(fakeAPI.setVersionCalledWith, gc.Equals, expectedVersion)
+		c.Assert(fakeAPI.setIgnoreCalledWith, gc.Equals, false)
 	}
 
 	const expectUpgrade = true
@@ -885,7 +913,7 @@ func NewFakeUpgradeJujuAPI(c *gc.C, st *state.State) *fakeUpgradeJujuAPI {
 	nextVersion := version.Binary{
 		Number: jujuversion.Current,
 		Arch:   arch.HostArch(),
-		Series: series.HostSeries(),
+		Series: series.MustHostSeries(),
 	}
 	nextVersion.Minor++
 	return &fakeUpgradeJujuAPI{
@@ -902,6 +930,7 @@ type fakeUpgradeJujuAPI struct {
 	setVersionErr             error
 	abortCurrentUpgradeCalled bool
 	setVersionCalledWith      version.Number
+	setIgnoreCalledWith       bool
 	tools                     []string
 	findToolsCalled           bool
 }
@@ -910,6 +939,7 @@ func (a *fakeUpgradeJujuAPI) reset() {
 	a.setVersionErr = nil
 	a.abortCurrentUpgradeCalled = false
 	a.setVersionCalledWith = version.Number{}
+	a.setIgnoreCalledWith = false
 	a.tools = []string{}
 	a.findToolsCalled = false
 }
@@ -971,8 +1001,9 @@ func (a *fakeUpgradeJujuAPI) AbortCurrentUpgrade() error {
 	return nil
 }
 
-func (a *fakeUpgradeJujuAPI) SetModelAgentVersion(v version.Number) error {
+func (a *fakeUpgradeJujuAPI) SetModelAgentVersion(v version.Number, ignoreAgentVersions bool) error {
 	a.setVersionCalledWith = v
+	a.setIgnoreCalledWith = ignoreAgentVersions
 	return a.setVersionErr
 }
 
@@ -983,12 +1014,13 @@ func (a *fakeUpgradeJujuAPI) Close() error {
 // Mock an API with no state
 type fakeUpgradeJujuAPINoState struct {
 	upgradeJujuAPI
-	name              string
-	uuid              string
-	controllerUUID    string
-	agentVersion      string
-	tools             coretools.List
-	modelAgentVersion version.Number
+	name                string
+	uuid                string
+	controllerUUID      string
+	agentVersion        string
+	tools               coretools.List
+	modelAgentVersion   version.Number
+	ignoreAgentVersions bool
 }
 
 func (a *fakeUpgradeJujuAPINoState) Close() error {
@@ -1015,8 +1047,9 @@ func (a *fakeUpgradeJujuAPINoState) UploadTools(r io.ReadSeeker, vers version.Bi
 	return a.tools, nil
 }
 
-func (a *fakeUpgradeJujuAPINoState) SetModelAgentVersion(version version.Number) error {
+func (a *fakeUpgradeJujuAPINoState) SetModelAgentVersion(version version.Number, ignoreAgentVersions bool) error {
 	a.modelAgentVersion = version
+	a.ignoreAgentVersions = ignoreAgentVersions
 	return nil
 }
 
