@@ -23,6 +23,7 @@ import (
 	"github.com/juju/juju/cmd/juju/common"
 	"github.com/juju/juju/cmd/modelcmd"
 	"github.com/juju/juju/cmd/output"
+	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/config"
 	"github.com/juju/juju/jujuclient"
 )
@@ -36,40 +37,54 @@ func NewAddModelCommand() cmd.Command {
 		newCloudAPI: func(caller base.APICallCloser) CloudAPI {
 			return cloudapi.NewClient(caller)
 		},
+		providerRegistry: environs.GlobalProviderRegistry(),
 	})
 }
 
 // addModelCommand calls the API to add a new model.
 type addModelCommand struct {
 	modelcmd.ControllerCommandBase
-	apiRoot        api.Connection
-	newAddModelAPI func(base.APICallCloser) AddModelAPI
-	newCloudAPI    func(base.APICallCloser) CloudAPI
+	apiRoot          api.Connection
+	newAddModelAPI   func(base.APICallCloser) AddModelAPI
+	newCloudAPI      func(base.APICallCloser) CloudAPI
+	providerRegistry environs.ProviderRegistry
 
 	Name           string
 	Owner          string
 	CredentialName string
 	CloudRegion    string
 	Config         common.ConfigFlag
+	noSwitch       bool
 }
 
 const addModelHelpDoc = `
 Adding a model is typically done in order to run a specific workload.
-To add a model, you must at a minimum specify a model name. You may
-also supply model-specific configuration, a credential, and which
-cloud/region to deploy the model to. The cloud/region and credentials
+
+To add a model, you must specify a model name. Model names can be duplicated 
+across controllers but must be unique per user for any given controller. 
+In other words, Alice and Bob can each have their own model called "secret" but 
+Alice can have only one model called "secret" in a controller.
+Model names may only contain lowercase letters, digits and hyphens, and 
+may not start with a hyphen.
+
+To add a model, Juju requires a credential:
+
+    * if you have a default (or just one) credential defined at client
+     (i.e. in credentials.yaml), then juju will use that;
+    * if you have no default (and multiple) credentials defined at the client, 
+     then you must specify one using --credential;
+    * as the admin user you can omit the credential, 
+     and the credential used to bootstrap will be used.
+
+To add a credential for add-model, use one of the "juju add-credential" or 
+"juju autoload-credentials" commands. These will add credentials 
+to the Juju client, which "juju add-model" will upload to the controller 
+as necessary.
+
+You may also supply model-specific configuration as well as a
+cloud/region to which this model will be deployed. The cloud/region and credentials
 are the ones used to create any future resources within the model.
 
-Model names can be duplicated across controllers but must be unique for
-any given controller. Model names may only contain lowercase letters,
-digits and hyphens, and may not start with a hyphen.
-
-Credential names are specified either in the form "credential-name", or
-"credential-owner/credential-name". There is currently no way to acquire
-access to another user's credentials, so the only valid value for
-credential-owner is your own user name. This may change in a future
-release.
-
 If no cloud/region is specified, then the model will be deployed to
 the same cloud/region as the controller model. If a region is specified
 without a cloud qualifier, then it is assumed to be in the same cloud
@@ -101,6 +116,7 @@ func (c *addModelCommand) SetFlags(f *gnuflag.FlagSet) {
 	f.StringVar(&c.Owner, "owner", "", "The owner of the new model if not the current user")
 	f.StringVar(&c.CredentialName, "credential", "", "Credential used to add the model")
 	f.Var(&c.Config, "config", "Path to YAML model configuration file or individual options (--config config.yaml [--config key=value ...])")
+	f.BoolVar(&c.noSwitch, "no-switch", false, "Do not switch to the newly created controller")
 }
 
 func (c *addModelCommand) Init(args []string) error {
@@ -148,6 +164,10 @@ func (c *addModelCommand) newAPIRoot() (api.Connection, error) {
 }
 
 func (c *addModelCommand) Run(ctx *cmd.Context) error {
+	controllerName, err := c.ControllerName()
+	if err != nil {
+		return errors.Trace(err)
+	}
 	api, err := c.newAPIRoot()
 	if err != nil {
 		return errors.Annotate(err, "opening API connection")
@@ -155,7 +175,6 @@ func (c *addModelCommand) Run(ctx *cmd.Context) error {
 	defer api.Close()
 
 	store := c.ClientStore()
-	controllerName := c.ControllerName()
 	accountDetails, err := store.AccountDetails(controllerName)
 	if err != nil {
 		return errors.Trace(err)
@@ -184,20 +203,27 @@ func (c *addModelCommand) Run(ctx *cmd.Context) error {
 		if err != nil {
 			return errors.Trace(err)
 		}
+	} else {
+		if cloudTag, cloud, err = defaultCloud(cloudClient); err != nil {
+			return errors.Trace(err)
+		}
 	}
 
-	// If the user has specified a credential, then we will upload it if
-	// it doesn't already exist in the controller, and it exists locally.
-	var credentialTag names.CloudCredentialTag
-	if c.CredentialName != "" {
-		var err error
-		if c.CloudRegion == "" {
-			if cloudTag, cloud, err = defaultCloud(cloudClient); err != nil {
-				return errors.Trace(err)
-			}
-		}
-		credentialTag, err = c.maybeUploadCredential(ctx, cloudClient, cloudTag, cloudRegion, cloud, modelOwner)
-		if err != nil {
+	// Find a credential to use with the new model.
+	credential, credentialTag, cloudRegion, err := c.findCredential(ctx, cloudClient, &findCredentialParams{
+		cloudTag:    cloudTag,
+		cloudRegion: cloudRegion,
+		cloud:       cloud,
+		modelOwner:  modelOwner,
+	})
+	if err != nil {
+		return errors.Trace(err)
+	}
+
+	// Upload the credential if it was found locally.
+	if credential != nil {
+		ctx.Infof("Uploading credential '%s' to controller", credentialTag.Id())
+		if err := cloudClient.UpdateCredential(credentialTag, *credential); err != nil {
 			return errors.Trace(err)
 		}
 	}
@@ -215,14 +241,15 @@ func (c *addModelCommand) Run(ctx *cmd.Context) error {
 	messageArgs := []interface{}{c.Name}
 
 	if modelOwner == accountDetails.User {
-		controllerName := c.ControllerName()
 		if err := store.UpdateModel(controllerName, c.Name, jujuclient.ModelDetails{
 			model.UUID,
 		}); err != nil {
 			return errors.Trace(err)
 		}
-		if err := store.SetCurrentModel(controllerName, c.Name); err != nil {
-			return errors.Trace(err)
+		if !c.noSwitch {
+			if err := store.SetCurrentModel(controllerName, c.Name); err != nil {
+				return errors.Trace(err)
+			}
 		}
 	}
 
@@ -320,9 +347,6 @@ func (c *addModelCommand) getCloudRegion(cloudClient CloudAPI) (cloudTag names.C
 			}
 			return names.CloudTag{}, jujucloud.Cloud{}, "", errors.Trace(err)
 		}
-	} else if len(cloud.Regions) > 0 {
-		// The first region in the list is the default.
-		cloudRegion = cloud.Regions[0].Name
 	}
 	return cloudTag, cloud, cloudRegion, nil
 }
@@ -385,31 +409,127 @@ there is no default cloud defined, please specify one using:
 	return cloudTag, cloud, nil
 }
 
-func (c *addModelCommand) maybeUploadCredential(
-	ctx *cmd.Context,
-	cloudClient CloudAPI,
-	cloudTag names.CloudTag,
-	cloudRegion string,
-	cloud jujucloud.Cloud,
-	modelOwner string,
-) (names.CloudCredentialTag, error) {
+var ambiguousDetectedCredentialError = errors.New(`
+more than one credential detected. Add all detected credentials
+to the client with:
 
-	modelOwnerTag := names.NewUserTag(modelOwner)
-	credentialTag, err := common.ResolveCloudCredentialTag(
-		modelOwnerTag, cloudTag, c.CredentialName,
-	)
-	if err != nil {
-		return names.CloudCredentialTag{}, errors.Trace(err)
+    juju autoload-credentials
+
+and then run the add-model command again with the --credential flag.`[1:],
+)
+
+var ambiguousCredentialError = errors.New(`
+more than one credential is available. List credentials with:
+
+    juju credentials
+
+and then run the add-model command again with the --credential flag.`[1:],
+)
+
+type findCredentialParams struct {
+	cloudTag    names.CloudTag
+	cloud       jujucloud.Cloud
+	cloudRegion string
+	modelOwner  string
+}
+
+// findCredential finds a suitable credential to use for the new model.
+// The credential will first be searched for locally and then on the
+// controller. If a credential is found locally then it's value will be
+// returned as the first return value. If it is found on the controller
+// this will be nil as there is no need to upload it in that case.
+func (c *addModelCommand) findCredential(ctx *cmd.Context, cloudClient CloudAPI, p *findCredentialParams) (_ *jujucloud.Credential, _ names.CloudCredentialTag, cloudRegion string, _ error) {
+	if c.CredentialName == "" {
+		return c.findUnspecifiedCredential(ctx, cloudClient, p)
+	}
+	return c.findSpecifiedCredential(ctx, cloudClient, p)
+}
+
+func (c *addModelCommand) findUnspecifiedCredential(ctx *cmd.Context, cloudClient CloudAPI, p *findCredentialParams) (_ *jujucloud.Credential, _ names.CloudCredentialTag, cloudRegion string, _ error) {
+	fail := func(err error) (*jujucloud.Credential, names.CloudCredentialTag, string, error) {
+		return nil, names.CloudCredentialTag{}, "", err
+	}
+	// If the user has not specified a credential, and the cloud advertises
+	// itself as supporting the "empty" auth-type, then return immediately.
+	for _, authType := range p.cloud.AuthTypes {
+		if authType == jujucloud.EmptyAuthType {
+			return nil, names.CloudCredentialTag{}, p.cloudRegion, nil
+		}
 	}
 
-	// Check if the credential is already in the controller.
-	//
-	// TODO(axw) consider implementing a call that can check
-	// that the credential exists without fetching all of the
-	// names.
-	credentialTags, err := cloudClient.UserCredentials(modelOwnerTag, cloudTag)
+	// No credential has been specified, so see if there is one already on the controller we can use.
+	modelOwnerTag := names.NewUserTag(p.modelOwner)
+	credentialTags, err := cloudClient.UserCredentials(modelOwnerTag, p.cloudTag)
 	if err != nil {
-		return names.CloudCredentialTag{}, errors.Trace(err)
+		return fail(errors.Trace(err))
+	}
+	var credentialTag names.CloudCredentialTag
+	if len(credentialTags) == 1 {
+		credentialTag = credentialTags[0]
+	}
+
+	if (credentialTag != names.CloudCredentialTag{}) {
+		// If the controller already has a credential, see if
+		// there is a local version that has an associated
+		// region.
+		credential, _, cloudRegion, err := c.findLocalCredential(ctx, p, credentialTag.Name())
+		if err == nil || errors.IsNotFound(err) {
+			// If there is a credential in the controller use it even if we don't have a local version.
+			return credential, credentialTag, cloudRegion, nil
+		}
+		if err != nil {
+			return fail(errors.Trace(err))
+		}
+	}
+	// There is not a default credential on the controller (either
+	// there are no credentials, or there is more than one). Look for
+	// a local credential we might use.
+	credential, credentialName, cloudRegion, err := c.findLocalCredential(ctx, p, "")
+	if err != nil {
+		return fail(errors.Trace(err))
+	}
+	// We've got a local credential to use.
+	credentialTag, err = common.ResolveCloudCredentialTag(
+		modelOwnerTag, p.cloudTag, credentialName,
+	)
+	if err != nil {
+		return fail(errors.Trace(err))
+	}
+	return credential, credentialTag, cloudRegion, nil
+}
+
+func (c *addModelCommand) findSpecifiedCredential(ctx *cmd.Context, cloudClient CloudAPI, p *findCredentialParams) (_ *jujucloud.Credential, _ names.CloudCredentialTag, cloudRegion string, _ error) {
+	fail := func(err error) (*jujucloud.Credential, names.CloudCredentialTag, string, error) {
+		return nil, names.CloudCredentialTag{}, "", err
+	}
+	// Look for a local credential with the specified name
+	credential, credentialName, cloudRegion, err := c.findLocalCredential(ctx, p, c.CredentialName)
+	if err != nil {
+		return fail(errors.Trace(err))
+	}
+	if credential != nil {
+		// We found a local credential with the specified name.
+		modelOwnerTag := names.NewUserTag(p.modelOwner)
+		credentialTag, err := common.ResolveCloudCredentialTag(
+			modelOwnerTag, p.cloudTag, credentialName,
+		)
+		if err != nil {
+			return fail(errors.Trace(err))
+		}
+		return credential, credentialTag, cloudRegion, nil
+	}
+
+	// There was no local credential with that name, check the controller
+	modelOwnerTag := names.NewUserTag(p.modelOwner)
+	credentialTags, err := cloudClient.UserCredentials(modelOwnerTag, p.cloudTag)
+	if err != nil {
+		return fail(errors.Trace(err))
+	}
+	credentialTag, err := common.ResolveCloudCredentialTag(
+		modelOwnerTag, p.cloudTag, c.CredentialName,
+	)
+	if err != nil {
+		return fail(errors.Trace(err))
 	}
 	credentialId := credentialTag.Id()
 	for _, tag := range credentialTags {
@@ -417,34 +537,40 @@ func (c *addModelCommand) maybeUploadCredential(
 			continue
 		}
 		ctx.Infof("Using credential '%s' cached in controller", c.CredentialName)
-		return credentialTag, nil
+		return nil, credentialTag, "", nil
 	}
+	// Cannot find a credential with the correct name
+	return fail(errors.NotFoundf("credential '%s'", c.CredentialName))
+}
 
-	if credentialTag.Owner().Id() != modelOwner {
-		// Another user's credential was specified, so
-		// we cannot automatically upload.
-		return names.CloudCredentialTag{}, errors.NotFoundf(
-			"credential '%s'", c.CredentialName,
-		)
+func (c *addModelCommand) findLocalCredential(ctx *cmd.Context, p *findCredentialParams, name string) (_ *jujucloud.Credential, credentialName, cloudRegion string, _ error) {
+	fail := func(err error) (*jujucloud.Credential, string, string, error) {
+		return nil, "", "", err
 	}
-
-	// Upload the credential from the client, if it exists locally.
-	credential, _, _, err := modelcmd.GetCredentials(
-		ctx, c.ClientStore(), modelcmd.GetCredentialsParams{
-			Cloud:          cloud,
-			CloudName:      cloudTag.Id(),
-			CloudRegion:    cloudRegion,
-			CredentialName: credentialTag.Name(),
+	provider, err := c.providerRegistry.Provider(p.cloud.Type)
+	if err != nil {
+		return fail(errors.Trace(err))
+	}
+	credential, credentialName, cloudRegion, _, err := common.GetOrDetectCredential(
+		ctx, c.ClientStore(), provider, modelcmd.GetCredentialsParams{
+			Cloud:          p.cloud,
+			CloudRegion:    p.cloudRegion,
+			CredentialName: name,
 		},
 	)
-	if err != nil {
-		return names.CloudCredentialTag{}, errors.Trace(err)
+	if err == nil {
+		return credential, credentialName, cloudRegion, nil
 	}
-	ctx.Infof("Uploading credential '%s' to controller", credentialTag.Id())
-	if err := cloudClient.UpdateCredential(credentialTag, *credential); err != nil {
-		return names.CloudCredentialTag{}, errors.Trace(err)
+	if errors.IsNotFound(err) {
+		return nil, "", "", nil
 	}
-	return credentialTag, nil
+	switch errors.Cause(err) {
+	case modelcmd.ErrMultipleCredentials:
+		return fail(ambiguousCredentialError)
+	case common.ErrMultipleDetectedCredentials:
+		return fail(ambiguousDetectedCredentialError)
+	}
+	return fail(errors.Trace(err))
 }
 
 func (c *addModelCommand) getConfigValues(ctx *cmd.Context) (map[string]interface{}, error) {
