@@ -9,10 +9,12 @@ import (
 	"strings"
 
 	"github.com/juju/cmd"
+	"github.com/juju/cmd/cmdtesting"
 	"github.com/juju/errors"
 	gitjujutesting "github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
+	"github.com/juju/version"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/names.v2"
 	"gopkg.in/yaml.v2"
@@ -22,36 +24,57 @@ import (
 	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/cloud"
 	"github.com/juju/juju/cmd/juju/controller"
+	"github.com/juju/juju/environs"
 	"github.com/juju/juju/jujuclient"
-	"github.com/juju/juju/jujuclient/jujuclienttesting"
 	_ "github.com/juju/juju/provider/ec2"
 	"github.com/juju/juju/testing"
 )
 
 type AddModelSuite struct {
 	testing.FakeJujuXDGDataHomeSuite
-	fakeAddModelAPI *fakeAddClient
-	fakeCloudAPI    *fakeCloudAPI
-	store           *jujuclienttesting.MemStore
+	fakeAddModelAPI      *fakeAddClient
+	fakeCloudAPI         *fakeCloudAPI
+	fakeProvider         *fakeProvider
+	fakeProviderRegistry *fakeProviderRegistry
+	store                *jujuclient.MemStore
 }
 
 var _ = gc.Suite(&AddModelSuite{})
 
 func (s *AddModelSuite) SetUpTest(c *gc.C) {
 	s.FakeJujuXDGDataHomeSuite.SetUpTest(c)
+
+	agentVersion, err := version.Parse("2.55.5")
+	c.Assert(err, jc.ErrorIsNil)
 	s.fakeAddModelAPI = &fakeAddClient{
 		model: base.ModelInfo{
-			Name:  "test",
-			UUID:  "fake-model-uuid",
-			Owner: "ignored-for-now",
+			Name:         "test",
+			UUID:         "fake-model-uuid",
+			Owner:        "ignored-for-now",
+			AgentVersion: &agentVersion,
 		},
 	}
-	s.fakeCloudAPI = &fakeCloudAPI{}
+	s.fakeCloudAPI = &fakeCloudAPI{
+		authTypes: []cloud.AuthType{
+			cloud.EmptyAuthType,
+			cloud.AccessKeyAuthType,
+		},
+		credentials: []names.CloudCredentialTag{
+			names.NewCloudCredentialTag("cloud/admin/default"),
+			names.NewCloudCredentialTag("aws/other/secrets"),
+		},
+	}
+	s.fakeProvider = &fakeProvider{
+		detected: cloud.NewEmptyCloudCredential(),
+	}
+	s.fakeProviderRegistry = &fakeProviderRegistry{
+		provider: s.fakeProvider,
+	}
 
 	// Set up the current controller, and write just enough info
 	// so we don't try to refresh
 	controllerName := "test-master"
-	s.store = jujuclienttesting.NewMemStore()
+	s.store = jujuclient.NewMemStore()
 	s.store.CurrentControllerName = controllerName
 	s.store.Controllers[controllerName] = jujuclient.ControllerDetails{}
 	s.store.Accounts[controllerName] = jujuclient.AccountDetails{
@@ -76,8 +99,14 @@ func (*fakeAPIConnection) Close() error {
 }
 
 func (s *AddModelSuite) run(c *gc.C, args ...string) (*cmd.Context, error) {
-	command, _ := controller.NewAddModelCommandForTest(&fakeAPIConnection{}, s.fakeAddModelAPI, s.fakeCloudAPI, s.store)
-	return testing.RunCommand(c, command, args...)
+	command, _ := controller.NewAddModelCommandForTest(
+		&fakeAPIConnection{},
+		s.fakeAddModelAPI,
+		s.fakeCloudAPI,
+		s.store,
+		s.fakeProviderRegistry,
+	)
+	return cmdtesting.RunCommand(c, command, args...)
 }
 
 func (s *AddModelSuite) TestInit(c *gc.C) {
@@ -131,8 +160,8 @@ func (s *AddModelSuite) TestInit(c *gc.C) {
 		},
 	} {
 		c.Logf("test %d", i)
-		wrappedCommand, command := controller.NewAddModelCommandForTest(nil, nil, nil, s.store)
-		err := testing.InitCommand(wrappedCommand, test.args)
+		wrappedCommand, command := controller.NewAddModelCommandForTest(nil, nil, nil, s.store, nil)
+		err := cmdtesting.InitCommand(wrappedCommand, test.args)
 		if test.err != "" {
 			c.Assert(err, gc.ErrorMatches, test.err)
 			continue
@@ -176,7 +205,7 @@ func (s *AddModelSuite) TestAddModelUnauthorizedMentionsJujuGrant(c *gc.C) {
 		Code:    params.CodeUnauthorized,
 	}
 	ctx, _ := s.run(c, "test")
-	errString := strings.Replace(testing.Stderr(ctx), "\n", " ", -1)
+	errString := strings.Replace(cmdtesting.Stderr(ctx), "\n", " ", -1)
 	c.Assert(errString, gc.Matches, `.*juju grant.*`)
 }
 
@@ -201,6 +230,17 @@ func (s *AddModelSuite) TestCredentialsOtherUserPassedThroughWhenCloud(c *gc.C)
 	c.Assert(s.fakeAddModelAPI.cloudCredential, gc.Equals, names.NewCloudCredentialTag("aws/other/secrets"))
 }
 
+func (s *AddModelSuite) TestCredentialsOtherUserCredentialNotFound(c *gc.C) {
+	// Have the API respond with no credentials.
+	s.PatchValue(&s.fakeCloudAPI.credentials, []names.CloudCredentialTag{})
+
+	_, err := s.run(c, "test", "--credential", "other/secrets")
+	c.Assert(err, gc.ErrorMatches, "credential 'other/secrets' not found")
+
+	// There should be no detection or UpdateCredentials call.
+	s.fakeCloudAPI.CheckCallNames(c, "DefaultCloud", "Cloud", "UserCredentials")
+}
+
 func (s *AddModelSuite) TestCredentialsNoDefaultCloud(c *gc.C) {
 	s.fakeCloudAPI.SetErrors(&params.Error{Code: params.CodeNotFound})
 	_, err := s.run(c, "test", "--credential", "secrets")
@@ -209,6 +249,76 @@ func (s *AddModelSuite) TestCredentialsNoDefaultCloud(c *gc.C) {
     juju add-model \[flags\] \<model-name\> cloud\[/region\]`)
 }
 
+func (s *AddModelSuite) TestCredentialsOneCached(c *gc.C) {
+	// Disable empty auth and clear the local credentials,
+	// forcing a check for credentials in the controller.
+	s.PatchValue(&s.fakeCloudAPI.authTypes, []cloud.AuthType{cloud.AccessKeyAuthType})
+	delete(s.store.Credentials, "aws")
+
+	// Cache just a single credential in the controller,
+	// so it is selected automatically.
+	credentialTag := names.NewCloudCredentialTag("aws/foo/secrets")
+	s.PatchValue(&s.fakeCloudAPI.credentials, []names.CloudCredentialTag{credentialTag})
+
+	_, err := s.run(c, "test")
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Assert(s.fakeAddModelAPI.cloudCredential, gc.Equals, credentialTag)
+}
+
+func (s *AddModelSuite) TestCredentialsDetected(c *gc.C) {
+	// Disable empty auth and clear the local credentials,
+	// forcing a check for credentials in the controller.
+	// There are multiple credentials in the controller,
+	// so none of them will be chosen by default.
+	s.PatchValue(&s.fakeCloudAPI.authTypes, []cloud.AuthType{cloud.AccessKeyAuthType})
+
+	// Delete all local credentials, so we don't choose
+	// any of them to upload. This will force credential
+	// detection.
+	delete(s.store.Credentials, "aws")
+
+	_, err := s.run(c, "test")
+	c.Assert(err, jc.ErrorIsNil)
+
+	credentialTag := names.NewCloudCredentialTag("aws/bob/default")
+	credential := cloud.NewCredential(cloud.AccessKeyAuthType, map[string]string{})
+	credential.Label = "finalized"
+
+	c.Assert(s.fakeAddModelAPI.cloudCredential, gc.Equals, credentialTag)
+	s.fakeCloudAPI.CheckCallNames(c, "DefaultCloud", "Cloud", "UserCredentials", "UpdateCredential")
+	s.fakeCloudAPI.CheckCall(c, 3, "UpdateCredential", credentialTag, credential)
+}
+
+func (s *AddModelSuite) TestCredentialsDetectedAmbiguous(c *gc.C) {
+	// Disable empty auth and clear the local credentials,
+	// forcing a check for credentials in the controller.
+	// There are multiple credentials in the controller,
+	// so none of them will be chosen by default.
+	s.PatchValue(&s.fakeCloudAPI.authTypes, []cloud.AuthType{cloud.AccessKeyAuthType})
+
+	// Delete all local credentials, so we don't choose
+	// any of them to upload. This will force credential
+	// detection.
+	delete(s.store.Credentials, "aws")
+
+	s.PatchValue(&s.fakeProvider.detected, &cloud.CloudCredential{
+		AuthCredentials: map[string]cloud.Credential{
+			"one": {},
+			"two": {},
+		},
+	})
+
+	_, err := s.run(c, "test")
+	c.Assert(err, gc.ErrorMatches, `
+more than one credential detected. Add all detected credentials
+to the client with:
+
+    juju autoload-credentials
+
+and then run the add-model command again with the --credential flag.`[1:])
+}
+
 func (s *AddModelSuite) TestCloudRegionPassedThrough(c *gc.C) {
 	_, err := s.run(c, "test", "aws/us-west-1")
 	c.Assert(err, jc.ErrorIsNil)
@@ -221,8 +331,8 @@ func (s *AddModelSuite) TestDefaultCloudPassedThrough(c *gc.C) {
 	_, err := s.run(c, "test")
 	c.Assert(err, jc.ErrorIsNil)
 
-	s.fakeCloudAPI.CheckCallNames(c /*none*/)
-	c.Assert(s.fakeAddModelAPI.cloudName, gc.Equals, "")
+	s.fakeCloudAPI.CheckCallNames(c, "DefaultCloud", "Cloud")
+	c.Assert(s.fakeAddModelAPI.cloudName, gc.Equals, "aws")
 	c.Assert(s.fakeAddModelAPI.cloudRegion, gc.Equals, "")
 }
 
@@ -260,12 +370,50 @@ lxd
 	})
 }
 
-func (s *AddModelSuite) TestCloudDefaultRegionPassedThrough(c *gc.C) {
+func (s *AddModelSuite) TestCloudUnspecifiedRegionPassedThrough(c *gc.C) {
+	// Use a cloud that doesn't support empty authorization.
+	s.fakeCloudAPI = &fakeCloudAPI{
+		authTypes: []cloud.AuthType{
+			cloud.AccessKeyAuthType,
+		},
+		credentials: []names.CloudCredentialTag{
+			names.NewCloudCredentialTag("cloud/admin/default"),
+			names.NewCloudCredentialTag("aws/other/secrets"),
+		},
+	}
 	_, err := s.run(c, "test", "aws")
 	c.Assert(err, jc.ErrorIsNil)
 
 	c.Assert(s.fakeAddModelAPI.cloudName, gc.Equals, "aws")
-	c.Assert(s.fakeAddModelAPI.cloudRegion, gc.Equals, "us-east-1")
+	c.Assert(s.fakeAddModelAPI.cloudRegion, gc.Equals, "")
+}
+
+func (s *AddModelSuite) TestCloudDefaultRegionUsedIfSet(c *gc.C) {
+	// Overwrite the credentials with a default region.
+	s.store.Credentials["aws"] = cloud.CloudCredential{
+		DefaultRegion: "us-west-1",
+		AuthCredentials: map[string]cloud.Credential{
+			"secrets": cloud.NewCredential(cloud.AccessKeyAuthType, map[string]string{
+				"access-key": "key",
+				"secret-key": "sekret",
+			}),
+		},
+	}
+	// Use a cloud that doesn't support empty authorization.
+	s.fakeCloudAPI = &fakeCloudAPI{
+		authTypes: []cloud.AuthType{
+			cloud.AccessKeyAuthType,
+		},
+		credentials: []names.CloudCredentialTag{
+			names.NewCloudCredentialTag("cloud/admin/default"),
+			names.NewCloudCredentialTag("aws/other/secrets"),
+		},
+	}
+	_, err := s.run(c, "test", "aws")
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Assert(s.fakeAddModelAPI.cloudName, gc.Equals, "aws")
+	c.Assert(s.fakeAddModelAPI.cloudRegion, gc.Equals, "us-west-1")
 }
 
 func (s *AddModelSuite) TestInvalidCloudOrRegionName(c *gc.C) {
@@ -394,10 +542,36 @@ func (s *AddModelSuite) TestAddErrorRemoveConfigstoreInfo(c *gc.C) {
 }
 
 func (s *AddModelSuite) TestAddStoresValues(c *gc.C) {
+	const controllerName = "test-master"
+
 	_, err := s.run(c, "test")
 	c.Assert(err, jc.ErrorIsNil)
 
-	model, err := s.store.ModelByName("test-master", "bob/test")
+	c.Check(s.store.CurrentControllerName, gc.Equals, controllerName)
+	modelName, err := s.store.CurrentModel(controllerName)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(modelName, gc.Equals, "bob/test")
+
+	model, err := s.store.ModelByName(controllerName, modelName)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(model, jc.DeepEquals, &jujuclient.ModelDetails{"fake-model-uuid"})
+}
+
+func (s *AddModelSuite) TestNoSwitch(c *gc.C) {
+	const controllerName = "test-master"
+	checkNoModelSelected := func() {
+		_, err := s.store.CurrentModel(controllerName)
+		c.Check(err, jc.Satisfies, errors.IsNotFound)
+	}
+	checkNoModelSelected()
+
+	_, err := s.run(c, "test", "--no-switch")
+	c.Assert(err, jc.ErrorIsNil)
+
+	// New model should not be selected by should still exist in the
+	// store.
+	checkNoModelSelected()
+	model, err := s.store.ModelByName(controllerName, "bob/test")
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(model, jc.DeepEquals, &jujuclient.ModelDetails{"fake-model-uuid"})
 }
@@ -445,6 +619,8 @@ func (f *fakeAddClient) CreateModel(name, owner, cloudName, cloudRegion string,
 type fakeCloudAPI struct {
 	controller.CloudAPI
 	gitjujutesting.Stub
+	authTypes   []cloud.AuthType
+	credentials []names.CloudCredentialTag
 }
 
 func (c *fakeCloudAPI) DefaultCloud() (names.CloudTag, error) {
@@ -456,6 +632,8 @@ func (c *fakeCloudAPI) Clouds() (map[names.CloudTag]cloud.Cloud, error) {
 	c.MethodCall(c, "Clouds")
 	return map[names.CloudTag]cloud.Cloud{
 		names.NewCloudTag("aws"): {
+			Name:      "aws",
+			AuthTypes: c.authTypes,
 			Regions: []cloud.Region{
 				{Name: "us-east-1"},
 				{Name: "us-west-1"},
@@ -471,8 +649,9 @@ func (c *fakeCloudAPI) Cloud(tag names.CloudTag) (cloud.Cloud, error) {
 		return cloud.Cloud{}, &params.Error{Code: params.CodeNotFound}
 	}
 	return cloud.Cloud{
+		Name:      "aws",
 		Type:      "ec2",
-		AuthTypes: []cloud.AuthType{cloud.AccessKeyAuthType},
+		AuthTypes: c.authTypes,
 		Regions: []cloud.Region{
 			{Name: "us-east-1"},
 			{Name: "us-west-1"},
@@ -482,13 +661,42 @@ func (c *fakeCloudAPI) Cloud(tag names.CloudTag) (cloud.Cloud, error) {
 
 func (c *fakeCloudAPI) UserCredentials(user names.UserTag, cloud names.CloudTag) ([]names.CloudCredentialTag, error) {
 	c.MethodCall(c, "UserCredentials", user, cloud)
-	return []names.CloudCredentialTag{
-		names.NewCloudCredentialTag("cloud/admin/default"),
-		names.NewCloudCredentialTag("aws/other/secrets"),
-	}, c.NextErr()
+	return c.credentials, c.NextErr()
 }
 
 func (c *fakeCloudAPI) UpdateCredential(credentialTag names.CloudCredentialTag, credential cloud.Credential) error {
 	c.MethodCall(c, "UpdateCredential", credentialTag, credential)
 	return c.NextErr()
 }
+
+type fakeProviderRegistry struct {
+	gitjujutesting.Stub
+	environs.ProviderRegistry
+	provider environs.EnvironProvider
+}
+
+func (r *fakeProviderRegistry) Provider(providerType string) (environs.EnvironProvider, error) {
+	r.MethodCall(r, "Provider", providerType)
+	return r.provider, r.NextErr()
+}
+
+type fakeProvider struct {
+	gitjujutesting.Stub
+	environs.EnvironProvider
+	detected *cloud.CloudCredential
+}
+
+func (p *fakeProvider) DetectCredentials() (*cloud.CloudCredential, error) {
+	p.MethodCall(p, "DetectCredentials")
+	return p.detected, p.NextErr()
+}
+
+func (p *fakeProvider) FinalizeCredential(
+	ctx environs.FinalizeCredentialContext,
+	args environs.FinalizeCredentialParams,
+) (*cloud.Credential, error) {
+	p.MethodCall(p, "FinalizeCredential", ctx, args)
+	out := cloud.NewCredential(cloud.AccessKeyAuthType, map[string]string{})
+	out.Label = "finalized"
+	return &out, p.NextErr()
+}
