@@ -129,7 +129,11 @@ type ModelSet struct {
 
 // Run implements Command.Run
 func (c *modelsCommand) Run(ctx *cmd.Context) error {
-	accountDetails, err := c.ClientStore().AccountDetails(c.ControllerName())
+	controllerName, err := c.ControllerName()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	accountDetails, err := c.CurrentAccountDetails()
 	if err != nil {
 		return err
 	}
@@ -163,12 +167,12 @@ func (c *modelsCommand) Run(ctx *cmd.Context) error {
 		if err != nil {
 			return errors.Trace(err)
 		}
-		model.ControllerName = c.ControllerName()
+		model.ControllerName = controllerName
 		modelInfo = append(modelInfo, model)
 	}
 
 	modelSet := ModelSet{Models: modelInfo}
-	current, err := c.ClientStore().CurrentModel(c.ControllerName())
+	current, err := c.ClientStore().CurrentModel(controllerName)
 	if err != nil && !errors.IsNotFound(err) {
 		return err
 	}
@@ -212,7 +216,7 @@ func (c *modelsCommand) getModelInfo(userModels []base.UserModel) ([]params.Mode
 		return nil, errors.Trace(err)
 	}
 
-	info := make([]params.ModelInfo, len(tags))
+	info := []params.ModelInfo{}
 	for i, result := range results {
 		if result.Error != nil {
 			if params.IsCodeUnauthorized(result.Error) {
@@ -226,7 +230,7 @@ func (c *modelsCommand) getModelInfo(userModels []base.UserModel) ([]params.Mode
 				userModels[i].UUID, userModels[i].Name,
 			)
 		}
-		info[i] = *result.Result
+		info = append(info, *result.Result)
 	}
 	return info, nil
 }
@@ -255,21 +259,24 @@ func (c *modelsCommand) formatTabular(writer io.Writer, value interface{}) error
 	if !ok {
 		return errors.Errorf("expected value of type %T, got %T", modelSet, value)
 	}
-
 	// We need the tag of the user for which we're listing models,
 	// and for the logged-in user. We use these below when formatting
 	// the model display names.
 	loggedInUser := names.NewUserTag(c.loggedInUser)
 	userForLastConn := loggedInUser
-	var userForListing names.UserTag
+	var currentUser names.UserTag
 	if c.user != "" {
-		userForListing = names.NewUserTag(c.user)
-		userForLastConn = userForListing
+		currentUser = names.NewUserTag(c.user)
+		userForLastConn = currentUser
 	}
 
 	tw := output.TabWriter(writer)
 	w := output.Wrapper{tw}
-	w.Println("Controller: " + c.ControllerName())
+	controllerName, err := c.ControllerName()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	w.Println("Controller: " + controllerName)
 	w.Println()
 	w.Print("Model")
 	if c.listUUID {
@@ -296,8 +303,12 @@ func (c *modelsCommand) formatTabular(writer io.Writer, value interface{}) error
 	for _, model := range modelSet.Models {
 		cloudRegion := strings.Trim(model.Cloud+"/"+model.CloudRegion, "/")
 		owner := names.NewUserTag(model.Owner)
-		name := common.OwnerQualifiedModelName(model.Name, owner, userForListing)
-		if jujuclient.JoinOwnerModelName(owner, model.Name) == modelSet.CurrentModelQualified {
+		name := model.Name
+		if currentUser == owner {
+			// No need to display fully qualified model name to its owner.
+			name = model.ShortName
+		}
+		if model.Name == modelSet.CurrentModelQualified {
 			name += "*"
 			w.PrintColor(output.CurrentHighlight, name)
 		} else {
@@ -306,16 +317,15 @@ func (c *modelsCommand) formatTabular(writer io.Writer, value interface{}) error
 		if c.listUUID {
 			w.Print(model.UUID)
 		}
-		lastConnection := model.Users[userForLastConn.Id()].LastConnection
-		if lastConnection == "" {
-			lastConnection = "never connected"
-		}
 		userForAccess := loggedInUser
 		if c.user != "" {
 			userForAccess = names.NewUserTag(c.user)
 		}
-		access := model.Users[userForAccess.Id()].Access
-		w.Print(cloudRegion, model.Status.Current)
+		status := "-"
+		if model.Status != nil {
+			status = model.Status.Current.String()
+		}
+		w.Print(cloudRegion, status)
 		if haveMachineInfo {
 			machineInfo := fmt.Sprintf("%d", len(model.Machines))
 			cores := uint64(0)
@@ -328,6 +338,14 @@ func (c *modelsCommand) formatTabular(writer io.Writer, value interface{}) error
 			}
 			w.Print(machineInfo, coresInfo)
 		}
+		access := model.Users[userForAccess.Id()].Access
+		if access == "" {
+			access = "-"
+		}
+		lastConnection := model.Users[userForLastConn.Id()].LastConnection
+		if lastConnection == "" {
+			lastConnection = "never connected"
+		}
 		w.Println(access, lastConnection)
 	}
 	tw.Flush()
