@@ -18,10 +18,20 @@ var (
 	BreakOneWord = breakOneWord
 )
 
-func NewOfferCommandForTest(store jujuclient.ClientStore, api OfferAPI) cmd.Command {
-	aCmd := &offerCommand{newAPIFunc: func() (OfferAPI, error) {
-		return api, nil
-	}}
+func noOpRefresh(jujuclient.ClientStore, string) error {
+	return nil
+}
+
+func NewOfferCommandForTest(
+	store jujuclient.ClientStore,
+	api OfferAPI,
+) cmd.Command {
+	aCmd := &offerCommand{
+		newAPIFunc: func() (OfferAPI, error) {
+			return api, nil
+		},
+		refreshModels: noOpRefresh,
+	}
 	aCmd.SetClientStore(store)
 	return modelcmd.WrapController(aCmd)
 }
