@@ -10,13 +10,14 @@ import (
 	"strings"
 
 	"github.com/juju/errors"
+
 	"github.com/juju/juju/cmd/output"
 )
 
 // formatFindTabular returns a tabular summary of remote applications or
 // errors out if parameter is not of expected type.
 func formatFindTabular(writer io.Writer, value interface{}) error {
-	endpoints, ok := value.(map[string]RemoteApplicationResult)
+	endpoints, ok := value.(map[string]ApplicationOfferResult)
 	if !ok {
 		return errors.Errorf("expected value of type %T, got %T", endpoints, value)
 	}
@@ -24,10 +25,10 @@ func formatFindTabular(writer io.Writer, value interface{}) error {
 }
 
 // formatFoundEndpointsTabular returns a tabular summary of offered applications' endpoints.
-func formatFoundEndpointsTabular(writer io.Writer, all map[string]RemoteApplicationResult) error {
+func formatFoundEndpointsTabular(writer io.Writer, all map[string]ApplicationOfferResult) error {
 	tw := output.TabWriter(writer)
 	w := output.Wrapper{tw}
-	w.Println("URL", "Interfaces")
+	w.Println("URL", "Access", "Interfaces")
 
 	for url, one := range all {
 		applicationURL := url
@@ -37,7 +38,7 @@ func formatFoundEndpointsTabular(writer io.Writer, all map[string]RemoteApplicat
 			interfaces = append(interfaces, fmt.Sprintf("%s:%s", ep.Interface, name))
 		}
 		sort.Strings(interfaces)
-		w.Println(applicationURL, strings.Join(interfaces, ", "))
+		w.Println(applicationURL, one.Access, strings.Join(interfaces, ", "))
 	}
 	tw.Flush()
 
