@@ -6,15 +6,14 @@ import (
 	"bytes"
 	"fmt"
 	"io"
+	"os"
 	"sort"
 	"strings"
 
-	"gopkg.in/juju/names.v2"
-
 	"github.com/juju/cmd"
 	"github.com/juju/errors"
 	"github.com/juju/gnuflag"
-	"github.com/juju/utils/keyvalues"
+	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/api"
 	"github.com/juju/juju/api/base"
@@ -22,6 +21,7 @@ import (
 	"github.com/juju/juju/api/modelmanager"
 	jujucloud "github.com/juju/juju/cloud"
 	"github.com/juju/juju/cmd/juju/block"
+	"github.com/juju/juju/cmd/juju/common"
 	"github.com/juju/juju/cmd/modelcmd"
 	"github.com/juju/juju/cmd/output"
 	"github.com/juju/juju/environs/config"
@@ -33,6 +33,7 @@ const (
 By default, all default configuration (keys and values) are
 displayed if a key is not specified. Supplying key=value will set the
 supplied key to the supplied value. This can be repeated for multiple keys.
+You can also specify a yaml file containing key values.
 By default, the model is the current model.
 
 
@@ -45,6 +46,8 @@ Examples:
     juju model-defaults ftp-proxy=10.0.0.1:8000
     juju model-defaults aws/us-east-1 ftp-proxy=10.0.0.1:8000
     juju model-defaults us-east-1 ftp-proxy=10.0.0.1:8000
+    juju model-defaults us-east-1 ftp-proxy=10.0.0.1:8000 path/to/file.yaml
+    juju model-defaults us-east-1 path/to/file.yaml    
     juju model-defaults -m othercontroller:mymodel default-series=yakkety test-mode=false
     juju model-defaults --reset default-series test-mode
     juju model-defaults aws/us-east-1 --reset http-proxy
@@ -80,12 +83,16 @@ type defaultsCommand struct {
 	newDefaultsAPI func(base.APICallCloser) defaultsCommandAPI
 	newCloudAPI    func(base.APICallCloser) cloudAPI
 
+	// args holds all the command-line arguments before
+	// they've been parsed.
+	args []string
+
 	action                func(defaultsCommandAPI, *cmd.Context) error // The function handling the input, set in Init.
 	key                   string
 	resetKeys             []string // Holds the keys to be reset once parsed.
 	cloudName, regionName string
 	reset                 []string // Holds the keys to be reset until parsed.
-	values                attributes
+	setOptions            common.ConfigFlag
 }
 
 // cloudAPI defines an API to be passed in for testing.
@@ -134,9 +141,49 @@ func (c *defaultsCommand) SetFlags(f *gnuflag.FlagSet) {
 	f.Var(cmd.NewAppendStringsValue(&c.reset), "reset", "Reset the provided comma delimited keys")
 }
 
-// Init implements part of the cmd.Command interface.
+// Init implements cmd.Command.Init.
+func (c *defaultsCommand) Init(args []string) error {
+	// There's no way of distinguishing a cloud name
+	// from a model configuration setting without contacting the
+	// API, but we aren't allowed to contact the API at Init time,
+	// so we defer parsing the arguments until Run is called.
+	c.args = args
+	return nil
+}
+
+// Run implements part of the cmd.Command interface.
+func (c *defaultsCommand) Run(ctx *cmd.Context) error {
+	if err := c.parseArgs(c.args); err != nil {
+		return errors.Trace(err)
+	}
+	root, err := c.newAPIRoot()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	client := c.newDefaultsAPI(root)
+	if err != nil {
+		return errors.Trace(err)
+	}
+	defer client.Close()
+
+	if len(c.resetKeys) > 0 {
+		err := c.resetDefaults(client, ctx)
+		if err != nil {
+			// We return this error naked as it is almost certainly going to be
+			// cmd.ErrSilent and the cmd.Command framework expects that back
+			// from cmd.Run if the process is blocked.
+			return err
+		}
+	}
+	if c.action == nil {
+		// If we are reset only we end up here, only we've already done that.
+		return nil
+	}
+	return c.action(client, ctx)
+}
+
 // This needs to parse a command line invocation to reset and set, or get
-// model-default values. The arguments may be interspersed as demosntrated in
+// model-default values. The arguments may be interspersed as demonstrated in
 // the examples.
 //
 // This sets foo=baz and unsets bar in aws/us-east-1
@@ -180,7 +227,7 @@ func (c *defaultsCommand) SetFlags(f *gnuflag.FlagSet) {
 // same key, that is to say keys to be mutated must be unique.
 //
 // Here we go...
-func (c *defaultsCommand) Init(args []string) error {
+func (c *defaultsCommand) parseArgs(args []string) error {
 	var err error
 	//  If there's nothing to reset and no args we're returning everything. So
 	//  we short circuit immediately.
@@ -210,8 +257,16 @@ func (c *defaultsCommand) Init(args []string) error {
 
 	// Remember we *might* have one less arg at this point if we chopped the
 	// first off because it was a valid cloud/region option.
+	wantSet := false
+	if len(args) > 0 {
+		lastArg := args[len(args)-1]
+		// We may have a config.yaml file
+		_, err := os.Stat(lastArg)
+		wantSet = err == nil || strings.Contains(lastArg, "=")
+	}
+
 	switch {
-	case len(args) > 0 && strings.Contains(args[len(args)-1], "="):
+	case wantSet:
 		// In the event that we are setting values, the final positional arg
 		// will always have an "=" in it. So if we see that we know we want to
 		// set args.
@@ -234,7 +289,6 @@ func (c *defaultsCommand) Init(args []string) error {
 		// certainly invalid, but in different possible ways.
 		return c.handleExtraArgs(args)
 	}
-
 }
 
 // parseResetKeys splits the keys provided to --reset after trimming any
@@ -354,7 +408,9 @@ func (c *defaultsCommand) validCloudRegion(cloudName, region string) (bool, erro
 
 // handleSetArgs parses args for setting defaults.
 func (c *defaultsCommand) handleSetArgs(args []string) error {
-	argZeroKeyOnly := !strings.Contains(args[0], "=")
+	// We may have a config.yaml file
+	_, err := os.Stat(args[0])
+	argZeroKeyOnly := err != nil && !strings.Contains(args[0], "=")
 	// If an invalid region was specified, the first positional arg won't have
 	// an "=". If we see one here we know it is invalid.
 	switch {
@@ -374,22 +430,9 @@ func (c *defaultsCommand) handleSetArgs(args []string) error {
 // parseSetKeys iterates over the args and make sure that the key=value pairs
 // are valid. It also checks that the same key isn't also being reset.
 func (c *defaultsCommand) parseSetKeys(args []string) error {
-	options, err := keyvalues.Parse(args, true)
-	if err != nil {
-		return errors.Trace(err)
-	}
-
-	c.values = make(attributes)
-	for k, v := range options {
-		if k == config.AgentVersionKey {
-			return errors.Errorf(`%q must be set via "upgrade-juju"`, config.AgentVersionKey)
-		}
-		c.values[k] = v
-	}
-	for _, k := range c.resetKeys {
-		if _, ok := c.values[k]; ok {
-			return errors.Errorf(
-				"key %q cannot be both set and unset in the same command", k)
+	for _, arg := range args {
+		if err := c.setOptions.Set(arg); err != nil {
+			return errors.Trace(err)
 		}
 	}
 	return nil
@@ -431,7 +474,9 @@ func (c *defaultsCommand) handleExtraArgs(args []string) error {
 	// last positional arg is not one. We assume the user intended to get a
 	// value after setting them.
 	for _, arg := range args {
-		if strings.Contains(arg, "=") {
+		// We may have a config.yaml file
+		_, err := os.Stat(arg)
+		if err == nil || strings.Contains(arg, "=") {
 			return errors.New("cannot set and retrieve default values simultaneously")
 		}
 	}
@@ -454,34 +499,6 @@ func (c *defaultsCommand) handleExtraArgs(args []string) error {
 	return errors.New("invalid input")
 }
 
-// Run implements part of the cmd.Command interface.
-func (c *defaultsCommand) Run(ctx *cmd.Context) error {
-	root, err := c.newAPIRoot()
-	if err != nil {
-		return errors.Trace(err)
-	}
-	client := c.newDefaultsAPI(root)
-	if err != nil {
-		return errors.Trace(err)
-	}
-	defer client.Close()
-
-	if len(c.resetKeys) > 0 {
-		err := c.resetDefaults(client, ctx)
-		if err != nil {
-			// We return this error naked as it is almost certainly going to be
-			// cmd.ErrSilent and the cmd.Command framework expects that back
-			// from cmd.Run if the process is blocked.
-			return err
-		}
-	}
-	if c.action == nil {
-		// If we are reset only we end up here, only we've already done that.
-		return nil
-	}
-	return c.action(client, ctx)
-}
-
 // getDefaults writes out the value for a single key or the full tree of
 // defaults.
 func (c *defaultsCommand) getDefaults(client defaultsCommandAPI, ctx *cmd.Context) error {
@@ -536,19 +553,39 @@ func (c *defaultsCommand) getDefaults(client defaultsCommandAPI, ctx *cmd.Contex
 
 // setDefaults sets defaults as provided in c.values.
 func (c *defaultsCommand) setDefaults(client defaultsCommandAPI, ctx *cmd.Context) error {
-	// ctx unused in this method.
-	if err := c.verifyKnownKeys(client); err != nil {
+	attrs, err := c.setOptions.ReadAttrs(ctx)
+	if err != nil {
+		return errors.Trace(err)
+	}
+	var keys []string
+	values := make(attributes)
+	for k, v := range attrs {
+		if k == config.AgentVersionKey {
+			return errors.Errorf(`"agent-version" must be set via "upgrade-juju"`)
+		}
+		values[k] = v
+		keys = append(keys, k)
+	}
+
+	for _, k := range c.resetKeys {
+		if _, ok := values[k]; ok {
+			return errors.Errorf(
+				"key %q cannot be both set and unset in the same command", k)
+		}
+	}
+
+	if err := c.verifyKnownKeys(client, keys); err != nil {
 		return errors.Trace(err)
 	}
 	return block.ProcessBlockedError(
 		client.SetModelDefaults(
-			c.cloudName, c.regionName, c.values), block.BlockChange)
+			c.cloudName, c.regionName, values), block.BlockChange)
 }
 
 // resetDefaults resets the keys in resetKeys.
 func (c *defaultsCommand) resetDefaults(client defaultsCommandAPI, ctx *cmd.Context) error {
 	// ctx unused in this method.
-	if err := c.verifyKnownKeys(client); err != nil {
+	if err := c.verifyKnownKeys(client, c.resetKeys); err != nil {
 		return errors.Trace(err)
 	}
 	return block.ProcessBlockedError(
@@ -559,14 +596,14 @@ func (c *defaultsCommand) resetDefaults(client defaultsCommandAPI, ctx *cmd.Cont
 
 // verifyKnownKeys is a helper to validate the keys we are operating with
 // against the set of known attributes from the model.
-func (c *defaultsCommand) verifyKnownKeys(client defaultsCommandAPI) error {
+func (c *defaultsCommand) verifyKnownKeys(client defaultsCommandAPI, keys []string) error {
 	known, err := client.ModelDefaults()
 	if err != nil {
 		return errors.Trace(err)
 	}
 
 	allKeys := c.resetKeys[:]
-	for k := range c.values {
+	for _, k := range keys {
 		allKeys = append(allKeys, k)
 	}
 
