@@ -18,6 +18,7 @@ type formattedStatus struct {
 	Machines           map[string]machineStatus           `json:"machines"`
 	Applications       map[string]applicationStatus       `json:"applications"`
 	RemoteApplications map[string]remoteApplicationStatus `json:"application-endpoints,omitempty" yaml:"application-endpoints,omitempty"`
+	Relations          []relationStatus                   `json:"-" yaml:"-"`
 }
 
 type formattedMachineStatus struct {
@@ -30,27 +31,40 @@ type errorStatus struct {
 }
 
 type modelStatus struct {
-	Name             string `json:"name" yaml:"name"`
-	Controller       string `json:"controller" yaml:"controller"`
-	Cloud            string `json:"cloud" yaml:"cloud"`
-	CloudRegion      string `json:"region,omitempty" yaml:"region,omitempty"`
-	Version          string `json:"version" yaml:"version"`
-	AvailableVersion string `json:"upgrade-available,omitempty" yaml:"upgrade-available,omitempty"`
-	Migration        string `json:"migration,omitempty" yaml:"migration,omitempty"`
+	Name             string             `json:"name" yaml:"name"`
+	Controller       string             `json:"controller" yaml:"controller"`
+	Cloud            string             `json:"cloud" yaml:"cloud"`
+	CloudRegion      string             `json:"region,omitempty" yaml:"region,omitempty"`
+	Version          string             `json:"version" yaml:"version"`
+	AvailableVersion string             `json:"upgrade-available,omitempty" yaml:"upgrade-available,omitempty"`
+	Status           statusInfoContents `json:"model-status,omitempty" yaml:"model-status,omitempty"`
+	MeterStatus      *meterStatus       `json:"meter-status,omitempty" yaml:"meter-status,omitempty"`
+	SLA              string             `json:"sla,omitempty" yaml:"sla,omitempty"`
+}
+
+type networkInterface struct {
+	IPAddresses    []string `json:"ip-addresses" yaml:"ip-addresses"`
+	MACAddress     string   `json:"mac-address" yaml:"mac-address"`
+	Gateway        string   `json:"gateway,omitempty" yaml:"gateway,omitempty"`
+	DNSNameservers []string `json:"dns-nameservers,omitempty" yaml:"dns-nameservers,omitempty"`
+	Space          string   `json:"space,omitempty" yaml:"space,omitempty"`
+	IsUp           bool     `json:"is-up" yaml:"is-up"`
 }
 
 type machineStatus struct {
-	Err           error                    `json:"-" yaml:",omitempty"`
-	JujuStatus    statusInfoContents       `json:"juju-status,omitempty" yaml:"juju-status,omitempty"`
-	DNSName       string                   `json:"dns-name,omitempty" yaml:"dns-name,omitempty"`
-	IPAddresses   []string                 `json:"ip-addresses,omitempty" yaml:"ip-addresses,omitempty"`
-	InstanceId    instance.Id              `json:"instance-id,omitempty" yaml:"instance-id,omitempty"`
-	MachineStatus statusInfoContents       `json:"machine-status,omitempty" yaml:"machine-status,omitempty"`
-	Series        string                   `json:"series,omitempty" yaml:"series,omitempty"`
-	Id            string                   `json:"-" yaml:"-"`
-	Containers    map[string]machineStatus `json:"containers,omitempty" yaml:"containers,omitempty"`
-	Hardware      string                   `json:"hardware,omitempty" yaml:"hardware,omitempty"`
-	HAStatus      string                   `json:"controller-member-status,omitempty" yaml:"controller-member-status,omitempty"`
+	Err               error                       `json:"-" yaml:",omitempty"`
+	JujuStatus        statusInfoContents          `json:"juju-status,omitempty" yaml:"juju-status,omitempty"`
+	DNSName           string                      `json:"dns-name,omitempty" yaml:"dns-name,omitempty"`
+	IPAddresses       []string                    `json:"ip-addresses,omitempty" yaml:"ip-addresses,omitempty"`
+	InstanceId        instance.Id                 `json:"instance-id,omitempty" yaml:"instance-id,omitempty"`
+	MachineStatus     statusInfoContents          `json:"machine-status,omitempty" yaml:"machine-status,omitempty"`
+	Series            string                      `json:"series,omitempty" yaml:"series,omitempty"`
+	Id                string                      `json:"-" yaml:"-"`
+	NetworkInterfaces map[string]networkInterface `json:"network-interfaces,omitempty" yaml:"network-interfaces,omitempty"`
+	Containers        map[string]machineStatus    `json:"containers,omitempty" yaml:"containers,omitempty"`
+	Constraints       string                      `json:"constraints,omitempty" yaml:"constraints,omitempty"`
+	Hardware          string                      `json:"hardware,omitempty" yaml:"hardware,omitempty"`
+	HAStatus          string                      `json:"controller-member-status,omitempty" yaml:"controller-member-status,omitempty"`
 }
 
 // A goyaml bug means we can't declare these types
@@ -230,3 +244,10 @@ func (s unitStatus) MarshalYAML() (interface{}, error) {
 	}
 	return unitStatusNoMarshal(s), nil
 }
+
+type relationStatus struct {
+	Provider  string
+	Requirer  string
+	Interface string
+	Type      string
+}
