@@ -4,6 +4,7 @@
 package status
 
 import (
+	"fmt"
 	"strings"
 
 	"github.com/juju/utils/series"
@@ -58,11 +59,19 @@ func (sf *statusFormatter) format() (formattedStatus, error) {
 			CloudRegion:      sf.status.Model.CloudRegion,
 			Version:          sf.status.Model.Version,
 			AvailableVersion: sf.status.Model.AvailableVersion,
-			Migration:        sf.status.Model.Migration,
+			Status:           sf.getStatusInfoContents(sf.status.Model.ModelStatus),
+			SLA:              sf.status.Model.SLA,
 		},
 		Machines:           make(map[string]machineStatus),
 		Applications:       make(map[string]applicationStatus),
 		RemoteApplications: make(map[string]remoteApplicationStatus),
+		Relations:          make([]relationStatus, len(sf.relations)),
+	}
+	if sf.status.Model.MeterStatus.Color != "" {
+		out.Model.MeterStatus = &meterStatus{
+			Color:   sf.status.Model.MeterStatus.Color,
+			Message: sf.status.Model.MeterStatus.Message,
+		}
 	}
 	for k, m := range sf.status.Machines {
 		out.Machines[k] = sf.formatMachine(m)
@@ -73,6 +82,11 @@ func (sf *statusFormatter) format() (formattedStatus, error) {
 	for sn, s := range sf.status.RemoteApplications {
 		out.RemoteApplications[sn] = sf.formatRemoteApplication(sn, s)
 	}
+	i := 0
+	for _, rel := range sf.relations {
+		out.Relations[i] = sf.formatRelation(rel)
+		i++
+	}
 	return out, nil
 }
 
@@ -103,17 +117,29 @@ func (sf *statusFormatter) formatMachine(machine params.MachineStatus) machineSt
 	var out machineStatus
 
 	out = machineStatus{
-		JujuStatus:    sf.getStatusInfoContents(machine.AgentStatus),
-		DNSName:       machine.DNSName,
-		IPAddresses:   machine.IPAddresses,
-		InstanceId:    machine.InstanceId,
-		MachineStatus: sf.getStatusInfoContents(machine.InstanceStatus),
-		Series:        machine.Series,
-		Id:            machine.Id,
-		Containers:    make(map[string]machineStatus),
-		Hardware:      machine.Hardware,
+		JujuStatus:        sf.getStatusInfoContents(machine.AgentStatus),
+		DNSName:           machine.DNSName,
+		IPAddresses:       machine.IPAddresses,
+		InstanceId:        machine.InstanceId,
+		MachineStatus:     sf.getStatusInfoContents(machine.InstanceStatus),
+		Series:            machine.Series,
+		Id:                machine.Id,
+		NetworkInterfaces: make(map[string]networkInterface),
+		Containers:        make(map[string]machineStatus),
+		Constraints:       machine.Constraints,
+		Hardware:          machine.Hardware,
 	}
 
+	for k, d := range machine.NetworkInterfaces {
+		out.NetworkInterfaces[k] = networkInterface{
+			IPAddresses:    d.IPAddresses,
+			MACAddress:     d.MACAddress,
+			Gateway:        d.Gateway,
+			DNSNameservers: d.DNSNameservers,
+			Space:          d.Space,
+			IsUp:           d.IsUp,
+		}
+	}
 	for k, m := range machine.Containers {
 		out.Containers[k] = sf.formatMachine(m)
 	}
@@ -197,6 +223,37 @@ func (sf *statusFormatter) formatRemoteApplication(name string, application para
 	return out
 }
 
+func (sf *statusFormatter) formatRelation(rel params.RelationStatus) relationStatus {
+	var provider, requirer params.EndpointStatus
+	for _, ep := range rel.Endpoints {
+		switch charm.RelationRole(ep.Role) {
+		case charm.RolePeer:
+			provider = ep
+			requirer = ep
+		case charm.RoleProvider:
+			provider = ep
+		case charm.RoleRequirer:
+			requirer = ep
+		}
+	}
+	var relType string
+	switch {
+	case rel.Scope == "container":
+		relType = "subordinate"
+	case provider.ApplicationName == requirer.ApplicationName:
+		relType = "peer"
+	default:
+		relType = "regular"
+	}
+	out := relationStatus{
+		Provider:  fmt.Sprintf("%s:%s", provider.ApplicationName, provider.Name),
+		Requirer:  fmt.Sprintf("%s:%s", requirer.ApplicationName, requirer.Name),
+		Interface: rel.Interface,
+		Type:      relType,
+	}
+	return out
+}
+
 func (sf *statusFormatter) getApplicationStatusInfo(application params.ApplicationStatus) statusInfoContents {
 	// TODO(perrito66) add status validation.
 	info := statusInfoContents{
