@@ -13,7 +13,6 @@ import (
 	"github.com/juju/ansiterm"
 	"github.com/juju/errors"
 	"github.com/juju/utils"
-	"github.com/juju/utils/set"
 	"gopkg.in/juju/charm.v6-unstable/hooks"
 
 	"github.com/juju/juju/cmd/output"
@@ -22,61 +21,6 @@ import (
 	"github.com/juju/juju/status"
 )
 
-type statusRelation struct {
-	application1 string
-	application2 string
-	relation     string
-	subordinate  bool
-}
-
-func (s *statusRelation) relationType() string {
-	if s.subordinate {
-		return "subordinate"
-	} else if s.application1 == s.application2 {
-		return "peer"
-	}
-	return "regular"
-}
-
-type relationFormatter struct {
-	relationIndex set.Strings
-	relations     map[string]*statusRelation
-}
-
-func newRelationFormatter() *relationFormatter {
-	return &relationFormatter{
-		relationIndex: set.NewStrings(),
-		relations:     make(map[string]*statusRelation),
-	}
-}
-
-func (r *relationFormatter) len() int {
-	return r.relationIndex.Size()
-}
-
-func (r *relationFormatter) add(rel1, rel2, relation string, is2SubOf1 bool) {
-	rel := []string{rel1, rel2}
-	if !is2SubOf1 {
-		sort.Sort(sort.StringSlice(rel))
-	}
-	k := strings.Join(rel, "\t")
-	r.relations[k] = &statusRelation{
-		application1: rel[0],
-		application2: rel[1],
-		relation:     relation,
-		subordinate:  is2SubOf1,
-	}
-	r.relationIndex.Add(k)
-}
-
-func (r *relationFormatter) sorted() []string {
-	return r.relationIndex.SortedValues()
-}
-
-func (r *relationFormatter) get(k string) *statusRelation {
-	return r.relations[k]
-}
-
 // FormatTabular writes a tabular summary of machines, applications, and
 // units. Any subordinate items are indented by two spaces beneath
 // their superior.
@@ -106,6 +50,8 @@ func FormatTabular(writer io.Writer, forceColor bool, value interface{}) error {
 		cloudRegion += "/" + fs.Model.CloudRegion
 	}
 
+	metering := fs.Model.MeterStatus != nil
+
 	header := []interface{}{"Model", "Controller", "Cloud/Region", "Version"}
 	values := []interface{}{fs.Model.Name, fs.Model.Controller, cloudRegion, fs.Model.Version}
 	message := getModelMessage(fs.Model)
@@ -113,39 +59,39 @@ func FormatTabular(writer io.Writer, forceColor bool, value interface{}) error {
 		header = append(header, "Notes")
 		values = append(values, message)
 	}
+	if fs.Model.SLA != "" {
+		header = append(header, "SLA")
+		values = append(values, fs.Model.SLA)
+	}
 
 	// The first set of headers don't use outputHeaders because it adds the blank line.
 	p(header...)
 	p(values...)
 
 	if len(fs.RemoteApplications) > 0 {
-		outputHeaders("SAAS name", "Status", "Store", "URL", "Interfaces")
-		for _, svcName := range utils.SortStringsNaturally(stringKeysFromMap(fs.RemoteApplications)) {
-			svc := fs.RemoteApplications[svcName]
+		outputHeaders("SAAS name", "Status", "Store", "URL")
+		for _, appName := range utils.SortStringsNaturally(stringKeysFromMap(fs.RemoteApplications)) {
+			app := fs.RemoteApplications[appName]
 			var store, urlPath string
-			url, err := crossmodel.ParseApplicationURL(svc.ApplicationURL)
+			url, err := crossmodel.ParseApplicationURL(app.ApplicationURL)
 			if err == nil {
-				store = url.Directory
+				store = url.Source
 				urlPath = url.Path()
+				if store == "" {
+					store = "local"
+				}
 			} else {
 				// This is not expected.
-				logger.Errorf("invalid application URL %q: %v", svc.ApplicationURL, err)
+				logger.Errorf("invalid application URL %q: %v", app.ApplicationURL, err)
 				store = "unknown"
-				urlPath = svc.ApplicationURL
+				urlPath = app.ApplicationURL
 			}
-			interfaces := make([]string, len(svc.Endpoints))
-			for i, name := range utils.SortStringsNaturally(stringKeysFromMap(svc.Endpoints)) {
-				ep := svc.Endpoints[name]
-				interfaces[i] = fmt.Sprintf("%s:%s", ep.Interface, name)
-			}
-			p(svcName, svc.StatusInfo.Current, store, urlPath, strings.Join(interfaces, ", "))
+			p(appName, app.StatusInfo.Current, store, urlPath)
 		}
 		tw.Flush()
 	}
 
 	units := make(map[string]unitStatus)
-	metering := false
-	relations := newRelationFormatter()
 	outputHeaders("App", "Version", "Status", "Scale", "Charm", "Store", "Rev", "OS", "Notes")
 	tw.SetColumnAlignRight(3)
 	tw.SetColumnAlignRight(6)
@@ -181,20 +127,6 @@ func FormatTabular(writer io.Writer, forceColor bool, value interface{}) error {
 				metering = true
 			}
 		}
-		// Ensure that we pick a consistent name for peer relations.
-		sortedRelTypes := make([]string, 0, len(app.Relations))
-		for relType := range app.Relations {
-			sortedRelTypes = append(sortedRelTypes, relType)
-		}
-		sort.Strings(sortedRelTypes)
-
-		subs := set.NewStrings(app.SubordinateTo...)
-		for _, relType := range sortedRelTypes {
-			for _, related := range app.Relations[relType] {
-				relations.add(related, appName, relType, subs.Contains(related))
-			}
-		}
-
 	}
 
 	pUnit := func(name string, u unitStatus, level int) {
@@ -226,11 +158,22 @@ func FormatTabular(writer io.Writer, forceColor bool, value interface{}) error {
 	}
 
 	if metering {
-		outputHeaders("Meter", "Status", "Message")
+		outputHeaders("Entity", "Meter status", "Message")
+		if fs.Model.MeterStatus != nil {
+			w.Print("model")
+			outputColor := fromMeterStatusColor(fs.Model.MeterStatus.Color)
+			w.PrintColor(outputColor, fs.Model.MeterStatus.Color)
+			w.PrintColor(outputColor, fs.Model.MeterStatus.Message)
+			w.Println()
+		}
 		for _, name := range utils.SortStringsNaturally(stringKeysFromMap(units)) {
 			u := units[name]
 			if u.MeterStatus != nil {
-				p(name, u.MeterStatus.Color, u.MeterStatus.Message)
+				w.Print(name)
+				outputColor := fromMeterStatusColor(u.MeterStatus.Color)
+				w.PrintColor(outputColor, u.MeterStatus.Color)
+				w.PrintColor(outputColor, u.MeterStatus.Message)
+				w.Println()
 			}
 		}
 	}
@@ -238,13 +181,17 @@ func FormatTabular(writer io.Writer, forceColor bool, value interface{}) error {
 	p()
 	printMachines(tw, fs.Machines)
 
-	if relations.len() > 0 {
-		outputHeaders("Relation", "Provides", "Consumes", "Type")
-		for _, k := range relations.sorted() {
-			r := relations.get(k)
-			if r != nil {
-				p(r.relation, r.application1, r.application2, r.relationType())
+	if len(fs.Relations) > 0 {
+		sort.Slice(fs.Relations, func(i, j int) bool {
+			a, b := fs.Relations[i], fs.Relations[j]
+			if a.Provider == b.Provider {
+				return a.Requirer < b.Requirer
 			}
+			return a.Provider < b.Provider
+		})
+		outputHeaders("Relation provider", "Requirer", "Interface", "Type")
+		for _, r := range fs.Relations {
+			p(r.Provider, r.Requirer, r.Interface, r.Type)
 		}
 	}
 
@@ -252,11 +199,23 @@ func FormatTabular(writer io.Writer, forceColor bool, value interface{}) error {
 	return nil
 }
 
+func fromMeterStatusColor(msColor string) *ansiterm.Context {
+	switch msColor {
+	case "green":
+		return output.GoodHighlight
+	case "amber":
+		return output.WarningHighlight
+	case "red":
+		return output.ErrorHighlight
+	}
+	return nil
+}
+
 func getModelMessage(model modelStatus) string {
 	// Select the most important message about the model (if any).
 	switch {
-	case model.Migration != "":
-		return "migrating: " + model.Migration
+	case model.Status.Message != "":
+		return model.Status.Message
 	case model.AvailableVersion != "":
 		return "upgrade available: " + model.AvailableVersion
 	default:
@@ -266,7 +225,7 @@ func getModelMessage(model modelStatus) string {
 
 func printMachines(tw *ansiterm.TabWriter, machines map[string]machineStatus) {
 	w := output.Wrapper{tw}
-	w.Println("Machine", "State", "DNS", "Inst id", "Series", "AZ")
+	w.Println("Machine", "State", "DNS", "Inst id", "Series", "AZ", "Message")
 	for _, name := range utils.SortStringsNaturally(stringKeysFromMap(machines)) {
 		printMachine(w, machines[name])
 	}
@@ -284,7 +243,7 @@ func printMachine(w output.Wrapper, m machineStatus) {
 	}
 	w.Print(m.Id)
 	w.PrintStatus(m.JujuStatus.Current)
-	w.Println(m.DNSName, m.InstanceId, m.Series, az)
+	w.Println(m.DNSName, m.InstanceId, m.Series, az, m.MachineStatus.Message)
 	for _, name := range utils.SortStringsNaturally(stringKeysFromMap(m.Containers)) {
 		printMachine(w, m.Containers[name])
 	}
