@@ -27,15 +27,24 @@ type VolumeInfo struct {
 	// Attachments is the set of entities attached to the volume.
 	Attachments *VolumeAttachments `yaml:"attachments,omitempty" json:"attachments,omitempty"`
 
+	// Pool is the name of the storage pool that the volume came from.
+	Pool string `yaml:"pool,omitempty" json:"pool,omitempty"`
+
 	// from params.Volume
 	HardwareId string `yaml:"hardware-id,omitempty" json:"hardware-id,omitempty"`
 
+	// from params.Volume
+	WWN string `yaml:"wwn,omitempty" json:"wwn,omitempty"`
+
 	// from params.Volume
 	Size uint64 `yaml:"size" json:"size"`
 
 	// from params.Volume
 	Persistent bool `yaml:"persistent" json:"persistent"`
 
+	// Life is the lifecycle state of the volume.
+	Life string `yaml:"life,omitempty" json:"life,omitempty"`
+
 	// from params.Volume
 	Status EntityStatus `yaml:"status,omitempty" json:"status,omitempty"`
 }
@@ -56,13 +65,14 @@ type MachineVolumeAttachment struct {
 	DeviceLink string `yaml:"device-link,omitempty" json:"device-link,omitempty"`
 	BusAddress string `yaml:"bus-address,omitempty" json:"bus-address,omitempty"`
 	ReadOnly   bool   `yaml:"read-only" json:"read-only"`
+	Life       string `yaml:"life,omitempty" json:"life,omitempty"`
 	// TODO(axw) add machine volume attachment status when we have it
 }
 
 //generateListVolumeOutput returns a map of volume info
-func (c *listCommand) generateListVolumeOutput(ctx *cmd.Context, api StorageListAPI) (output interface{}, err error) {
+func generateListVolumeOutput(ctx *cmd.Context, api StorageListAPI, ids []string) (map[string]VolumeInfo, error) {
 
-	results, err := api.ListVolumes(c.ids)
+	results, err := api.ListVolumes(ids)
 	if err != nil {
 		return nil, err
 	}
@@ -79,17 +89,7 @@ func (c *listCommand) generateListVolumeOutput(ctx *cmd.Context, api StorageList
 	if len(valid) == 0 {
 		return nil, nil
 	}
-	info, err := convertToVolumeInfo(valid)
-	if err != nil {
-		return nil, err
-	}
-	switch c.out.Name() {
-	case "yaml", "json":
-		output = map[string]map[string]VolumeInfo{"volumes": info}
-	default:
-		output = info
-	}
-	return output, nil
+	return convertToVolumeInfo(valid)
 }
 
 // convertToVolumeInfo returns a map of volume IDs to volume info.
@@ -122,8 +122,11 @@ func createVolumeInfo(details params.VolumeDetails) (names.VolumeTag, VolumeInfo
 	var info VolumeInfo
 	info.ProviderVolumeId = details.Info.VolumeId
 	info.HardwareId = details.Info.HardwareId
+	info.WWN = details.Info.WWN
+	info.Pool = details.Info.Pool
 	info.Size = details.Info.Size
 	info.Persistent = details.Info.Persistent
+	info.Life = string(details.Life)
 	info.Status = EntityStatus{
 		details.Status.Status,
 		details.Status.Info,
@@ -143,6 +146,7 @@ func createVolumeInfo(details params.VolumeDetails) (names.VolumeTag, VolumeInfo
 				attachment.DeviceLink,
 				attachment.BusAddress,
 				attachment.ReadOnly,
+				string(attachment.Life),
 			}
 		}
 		info.Attachments = &VolumeAttachments{
