@@ -8,8 +8,13 @@ import (
 	"time"
 
 	"github.com/juju/errors"
+	"github.com/juju/pubsub"
+	"github.com/juju/utils/clock"
 	"github.com/juju/utils/proxy"
 	"github.com/juju/utils/voyeur"
+	"github.com/juju/version"
+	"github.com/prometheus/client_golang/prometheus"
+	"gopkg.in/juju/worker.v1"
 
 	coreagent "github.com/juju/juju/agent"
 	"github.com/juju/juju/api"
@@ -18,12 +23,14 @@ import (
 	"github.com/juju/juju/cmd/jujud/agent/engine"
 	"github.com/juju/juju/container/lxd"
 	"github.com/juju/juju/state"
-	"github.com/juju/juju/worker"
+	proxyconfig "github.com/juju/juju/utils/proxy"
+	jworker "github.com/juju/juju/worker"
 	"github.com/juju/juju/worker/agent"
 	"github.com/juju/juju/worker/apiaddressupdater"
 	"github.com/juju/juju/worker/apicaller"
 	"github.com/juju/juju/worker/apiconfigwatcher"
 	"github.com/juju/juju/worker/authenticationworker"
+	"github.com/juju/juju/worker/centralhub"
 	"github.com/juju/juju/worker/dependency"
 	"github.com/juju/juju/worker/deployer"
 	"github.com/juju/juju/worker/diskmanager"
@@ -31,8 +38,6 @@ import (
 	"github.com/juju/juju/worker/gate"
 	"github.com/juju/juju/worker/hostkeyreporter"
 	"github.com/juju/juju/worker/identityfilewriter"
-	"github.com/juju/juju/worker/logforwarder"
-	"github.com/juju/juju/worker/logforwarder/sinks"
 	"github.com/juju/juju/worker/logger"
 	"github.com/juju/juju/worker/logsender"
 	"github.com/juju/juju/worker/machineactions"
@@ -49,8 +54,6 @@ import (
 	"github.com/juju/juju/worker/toolsversionchecker"
 	"github.com/juju/juju/worker/upgrader"
 	"github.com/juju/juju/worker/upgradesteps"
-	"github.com/juju/utils/clock"
-	"github.com/juju/version"
 )
 
 // ManifoldsConfig allows specialisation of the result of Manifolds.
@@ -126,6 +129,20 @@ type ManifoldsConfig struct {
 	// migration process to check that the agent will be ok when
 	// connected to the new target controller.
 	ValidateMigration func(base.APICaller) error
+
+	// PrometheusRegisterer is a prometheus.Registerer that may be used
+	// by workers to register Prometheus metric collectors.
+	PrometheusRegisterer prometheus.Registerer
+
+	// CentralHub is the primary hub that exists in the apiserver.
+	CentralHub *pubsub.StructuredHub
+
+	// DepEngineReporter is a dependency engine reporter.
+	DepEngineReporter dependency.Reporter
+
+	// UpdateLoggerConfig is a function that will save the specified
+	// config value as the logging config in the agent.conf file.
+	UpdateLoggerConfig func(string) error
 }
 
 // Manifolds returns a set of co-configured manifolds covering the
@@ -147,7 +164,7 @@ func Manifolds(config ManifoldsConfig) dependency.Manifolds {
 			if err2 != nil {
 				return errors.Trace(err2)
 			}
-			return worker.ErrTerminateAgent
+			return jworker.ErrTerminateAgent
 		} else if cause == apicaller.ErrChangedPassword {
 			return dependency.ErrBounce
 		}
@@ -182,6 +199,17 @@ func Manifolds(config ManifoldsConfig) dependency.Manifolds {
 			AgentConfigChanged: config.AgentConfigChanged,
 		}),
 
+		// The centralhub manifold watches the state config to make sure it
+		// only starts for machines that are api servers. Currently the hub is
+		// passed in as config, but when the apiserver and peergrouper are
+		// updated to use the dependency engine, the centralhub manifold
+		// should also take the agentName so the worker can get the machine ID
+		// for the creation of the hub.
+		centralHubName: centralhub.Manifold(centralhub.ManifoldConfig{
+			StateConfigWatcherName: stateConfigWatcherName,
+			Hub: config.CentralHub,
+		}),
+
 		// The state manifold creates a *state.State and makes it
 		// available to other manifolds. It pings the mongodb session
 		// regularly and will die if pings fail.
@@ -336,8 +364,9 @@ func Manifolds(config ManifoldsConfig) dependency.Manifolds {
 		// according to changes in environment config. We should only need
 		// one of these in a consolidated agent.
 		loggingConfigUpdaterName: ifNotMigrating(logger.Manifold(logger.ManifoldConfig{
-			AgentName:     agentName,
-			APICallerName: apiCallerName,
+			AgentName:       agentName,
+			APICallerName:   apiCallerName,
+			UpdateAgentFunc: config.UpdateLoggerConfig,
 		})),
 
 		// The diskmanager worker periodically lists block devices on the
@@ -351,10 +380,11 @@ func Manifolds(config ManifoldsConfig) dependency.Manifolds {
 		// The proxy config updater is a leaf worker that sets http/https/apt/etc
 		// proxy settings.
 		proxyConfigUpdater: ifNotMigrating(proxyupdater.Manifold(proxyupdater.ManifoldConfig{
-			AgentName:      agentName,
-			APICallerName:  apiCallerName,
-			WorkerFunc:     proxyupdater.NewWorker,
-			ExternalUpdate: externalUpdateProxyFunc,
+			AgentName:       agentName,
+			APICallerName:   apiCallerName,
+			WorkerFunc:      proxyupdater.NewWorker,
+			ExternalUpdate:  externalUpdateProxyFunc,
+			InProcessUpdate: proxyconfig.DefaultConfig.Set,
 		})),
 
 		// The api address updater is a leaf worker that rewrites agent config
@@ -443,14 +473,6 @@ func Manifolds(config ManifoldsConfig) dependency.Manifolds {
 			NewFacade:     hostkeyreporter.NewFacade,
 			NewWorker:     hostkeyreporter.NewWorker,
 		})),
-		logForwarderName: ifFullyUpgraded(logforwarder.Manifold(logforwarder.ManifoldConfig{
-			StateName:     stateName,
-			APICallerName: apiCallerName,
-			Sinks: []logforwarder.LogSinkSpec{{
-				Name:   "juju-log-forward",
-				OpenFn: sinks.OpenSyslog,
-			}},
-		})),
 	}
 }
 
@@ -476,6 +498,7 @@ const (
 	stateWorkersName       = "unconverted-state-workers"
 	apiCallerName          = "api-caller"
 	apiConfigWatcherName   = "api-config-watcher"
+	centralHubName         = "central-hub"
 
 	upgraderName         = "upgrader"
 	upgradeStepsName     = "upgrade-steps-runner"
@@ -505,5 +528,4 @@ const (
 	toolsVersionCheckerName  = "tools-version-checker"
 	machineActionName        = "machine-action-runner"
 	hostKeyReporterName      = "host-key-reporter"
-	logForwarderName         = "log-forwarder"
 )
