@@ -129,7 +129,6 @@ func (c *BootstrapCommand) Run(_ *cmd.Context) error {
 	// Get the bootstrap machine's addresses from the provider.
 	cloudSpec, err := environs.MakeCloudSpec(
 		args.ControllerCloud,
-		args.ControllerCloudName,
 		args.ControllerCloudRegion,
 		args.ControllerCloudCredential,
 	)
@@ -152,20 +151,24 @@ func (c *BootstrapCommand) Run(_ *cmd.Context) error {
 		// If we have been asked for a newer version, ensure the newer
 		// tools can actually be found, or else bootstrap won't complete.
 		stream := envtools.PreferredStream(&desiredVersion, args.ControllerModelConfig.Development(), args.ControllerModelConfig.AgentStream())
-		logger.Infof("newer tools requested, looking for %v in stream %v", desiredVersion, stream)
+		logger.Infof("newer agent binaries requested, looking for %v in stream %v", desiredVersion, stream)
+		hostSeries, err := series.HostSeries()
+		if err != nil {
+			return errors.Trace(err)
+		}
 		filter := tools.Filter{
 			Number: desiredVersion,
 			Arch:   arch.HostArch(),
-			Series: series.HostSeries(),
+			Series: hostSeries,
 		}
 		_, toolsErr := envtools.FindTools(env, -1, -1, stream, filter)
 		if toolsErr == nil {
-			logger.Infof("tools are available, upgrade will occur after bootstrap")
+			logger.Infof("agent binaries are available, upgrade will occur after bootstrap")
 		}
 		if errors.IsNotFound(toolsErr) {
 			// Newer tools not available, so revert to using the tools
 			// matching the current agent version.
-			logger.Warningf("newer tools for %q not available, sticking with version %q", desiredVersion, jujuversion.Current)
+			logger.Warningf("newer agent binaries for %q not available, sticking with version %q", desiredVersion, jujuversion.Current)
 			newConfigAttrs["agent-version"] = jujuversion.Current.String()
 		} else if toolsErr != nil {
 			logger.Errorf("cannot find newer tools: %v", toolsErr)
@@ -212,6 +215,12 @@ func (c *BootstrapCommand) Run(_ *cmd.Context) error {
 	info.SystemIdentity = privateKey
 	err = c.ChangeConfig(func(agentConfig agent.ConfigSetter) error {
 		agentConfig.SetStateServingInfo(info)
+		mmprof, err := mongo.NewMemoryProfile(args.ControllerConfig.MongoMemoryProfile())
+		if err != nil {
+			logger.Errorf("could not set requested memory profile: %v", err)
+		} else {
+			agentConfig.SetMongoMemoryProfile(mmprof)
+		}
 		return nil
 	})
 	if err != nil {
@@ -277,6 +286,15 @@ func (c *BootstrapCommand) Run(_ *cmd.Context) error {
 	}
 	defer st.Close()
 
+	// Fetch spaces from substrate
+	if err = st.ReloadSpaces(env); err != nil {
+		if errors.IsNotSupported(err) {
+			logger.Debugf("Not performing spaces load on a non-networking environment")
+		} else {
+			return err
+		}
+	}
+
 	// Populate the tools catalogue.
 	if err := c.populateTools(st, env); err != nil {
 		return err
@@ -323,8 +341,11 @@ func (c *BootstrapCommand) startMongo(addrs []network.Address, agentConfig agent
 	// Use localhost to dial the mongo server, because it's running in
 	// auth mode and will refuse to perform any operations unless
 	// we dial that address.
+	// TODO(macgreagoir) IPv6. Ubuntu still always provides IPv4 loopback,
+	// and when/if this changes localhost should resolve to IPv6 loopback
+	// in any case (lp:1644009). Review.
 	dialInfo.Addrs = []string{
-		net.JoinHostPort("127.0.0.1", fmt.Sprint(servingInfo.StatePort)),
+		net.JoinHostPort("localhost", fmt.Sprint(servingInfo.StatePort)),
 	}
 
 	logger.Debugf("calling ensureMongoServer")
@@ -359,10 +380,14 @@ func (c *BootstrapCommand) populateTools(st *state.State, env environs.Environ)
 	agentConfig := c.CurrentConfig()
 	dataDir := agentConfig.DataDir()
 
+	hostSeries, err := series.HostSeries()
+	if err != nil {
+		return errors.Trace(err)
+	}
 	current := version.Binary{
 		Number: jujuversion.Current,
 		Arch:   arch.HostArch(),
-		Series: series.HostSeries(),
+		Series: hostSeries,
 	}
 	tools, err := agenttools.ReadTools(dataDir, current)
 	if err != nil {
@@ -407,7 +432,7 @@ func (c *BootstrapCommand) populateTools(st *state.State, env environs.Environ)
 			Size:    tools.Size,
 			SHA256:  tools.SHA256,
 		}
-		logger.Debugf("Adding tools: %v", toolsVersion)
+		logger.Debugf("Adding agent binaries: %v", toolsVersion)
 		if err := toolstorage.Add(bytes.NewReader(data), metadata); err != nil {
 			return errors.Trace(err)
 		}
