@@ -7,14 +7,17 @@ import (
 	"gopkg.in/juju/charm.v6-unstable"
 )
 
-// OfferedApplicationDetails represents a remote application used when vendor
+// ApplicationOfferDetails represents a remote application used when vendor
 // lists their own applications.
-type OfferedApplicationDetails struct {
-	// ApplicationName is the application name.
+type ApplicationOfferDetails struct {
+	// OfferName is the name of the offer
+	OfferName string
+
+	// ApplicationName is the application name to which the offer pertains.
 	ApplicationName string
 
-	// ApplicationURL is the URl where the application can be located.
-	ApplicationURL string
+	// OfferURL is the URL where the offer can be located.
+	OfferURL string
 
 	// CharmName is a name of a charm for remote application.
 	CharmName string
@@ -27,10 +30,10 @@ type OfferedApplicationDetails struct {
 	ConnectedCount int
 }
 
-// OfferedApplicationDetailsResult is a result of listing a remote application.
-type OfferedApplicationDetailsResult struct {
+// ApplicationOfferDetailsResult is a result of listing a remote application.
+type ApplicationOfferDetailsResult struct {
 	// Result contains remote application information.
-	Result *OfferedApplicationDetails
+	Result *ApplicationOfferDetails
 
 	// Error contains error related to this item.
 	Error error
