@@ -15,9 +15,11 @@ import (
 	"strings"
 	"time"
 
+	"github.com/juju/cmd/cmdtesting"
 	"github.com/juju/errors"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils/arch"
+	utilscert "github.com/juju/utils/cert"
 	jujuos "github.com/juju/utils/os"
 	"github.com/juju/utils/series"
 	"github.com/juju/version"
@@ -193,7 +195,7 @@ func intPtr(i uint64) *uint64 {
 }
 
 func (s *bootstrapSuite) TestBootstrapImage(c *gc.C) {
-	s.PatchValue(&series.HostSeries, func() string { return "precise" })
+	s.PatchValue(&series.MustHostSeries, func() string { return "precise" })
 	s.PatchValue(&arch.HostArch, func() string { return arch.AMD64 })
 
 	metadataDir, metadata := createImageMetadata(c)
@@ -237,6 +239,7 @@ func (s *bootstrapSuite) TestBootstrapImage(c *gc.C) {
 	expectedCons := bootstrapCons
 	expectedCons.Mem = intPtr(3584)
 	c.Assert(env.instanceConfig.Bootstrap.BootstrapMachineConstraints, jc.DeepEquals, expectedCons)
+	c.Assert(env.instanceConfig.Bootstrap.ControllerModelEnvironVersion, gc.Equals, 123)
 }
 
 func (s *bootstrapSuite) TestBootstrapAddsArchFromImageToExistingProviderSupportedArches(c *gc.C) {
@@ -271,7 +274,7 @@ type testImageMetadata struct {
 // setupImageMetadata returns architecture for which metadata was setup
 func (s *bootstrapSuite) setupImageMetadata(c *gc.C) testImageMetadata {
 	testArch := arch.S390X
-	s.PatchValue(&series.HostSeries, func() string { return "precise" })
+	s.PatchValue(&series.MustHostSeries, func() string { return "precise" })
 	s.PatchValue(&arch.HostArch, func() string { return testArch })
 
 	metadataDir, metadata := createImageMetadataForArch(c, testArch)
@@ -373,7 +376,7 @@ func (s *bootstrapSuite) setupProviderWithNoSupportedArches(c *gc.C) bootstrapEn
 // despite image metadata in other data sources compatible with the same configuration as well.
 // Related to bug#1560625.
 func (s *bootstrapSuite) TestBootstrapImageMetadataFromAllSources(c *gc.C) {
-	s.PatchValue(&series.HostSeries, func() string { return "raring" })
+	s.PatchValue(&series.MustHostSeries, func() string { return "raring" })
 	s.PatchValue(&arch.HostArch, func() string { return arch.AMD64 })
 
 	// Ensure that we can find at least one image metadata
@@ -698,7 +701,7 @@ func (s *bootstrapSuite) TestBootstrapGUISuccessRemote(c *gc.C) {
 		}}, nil
 	})
 	env := newEnviron("foo", useDefaultKeys, nil)
-	ctx := coretesting.Context(c)
+	ctx := cmdtesting.Context(c)
 	err := bootstrap.Bootstrap(modelcmd.BootstrapContext(ctx), env, bootstrap.BootstrapParams{
 		ControllerConfig:     coretesting.FakeControllerConfig(),
 		AdminSecret:          "admin-secret",
@@ -706,7 +709,7 @@ func (s *bootstrapSuite) TestBootstrapGUISuccessRemote(c *gc.C) {
 		GUIDataSourceBaseURL: "https://1.2.3.4/gui/sources",
 	})
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(coretesting.Stderr(ctx), jc.Contains, "Fetching Juju GUI 2.0.42\n")
+	c.Assert(cmdtesting.Stderr(ctx), jc.Contains, "Fetching Juju GUI 2.0.42\n")
 
 	// The most recent GUI release info has been stored.
 	c.Assert(env.instanceConfig.Bootstrap.GUI.URL, gc.Equals, "https://1.2.3.4/juju-gui-2.0.42.tar.bz2")
@@ -719,14 +722,14 @@ func (s *bootstrapSuite) TestBootstrapGUISuccessLocal(c *gc.C) {
 	path := makeGUIArchive(c, "jujugui-2.2.0")
 	s.PatchEnvironment("JUJU_GUI", path)
 	env := newEnviron("foo", useDefaultKeys, nil)
-	ctx := coretesting.Context(c)
+	ctx := cmdtesting.Context(c)
 	err := bootstrap.Bootstrap(modelcmd.BootstrapContext(ctx), env, bootstrap.BootstrapParams{
 		ControllerConfig: coretesting.FakeControllerConfig(),
 		AdminSecret:      "admin-secret",
 		CAPrivateKey:     coretesting.CAKey,
 	})
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(coretesting.Stderr(ctx), jc.Contains, "Fetching Juju GUI 2.2.0 from local archive\n")
+	c.Assert(cmdtesting.Stderr(ctx), jc.Contains, "Fetching Juju GUI 2.2.0 from local archive\n")
 
 	// Check GUI URL and version.
 	c.Assert(env.instanceConfig.Bootstrap.GUI.URL, gc.Equals, "file://"+path)
@@ -749,14 +752,14 @@ func (s *bootstrapSuite) TestBootstrapGUISuccessLocal(c *gc.C) {
 
 func (s *bootstrapSuite) TestBootstrapGUISuccessNoGUI(c *gc.C) {
 	env := newEnviron("foo", useDefaultKeys, nil)
-	ctx := coretesting.Context(c)
+	ctx := cmdtesting.Context(c)
 	err := bootstrap.Bootstrap(modelcmd.BootstrapContext(ctx), env, bootstrap.BootstrapParams{
 		ControllerConfig: coretesting.FakeControllerConfig(),
 		AdminSecret:      "admin-secret",
 		CAPrivateKey:     coretesting.CAKey,
 	})
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(coretesting.Stderr(ctx), jc.Contains, "Juju GUI installation has been disabled\n")
+	c.Assert(cmdtesting.Stderr(ctx), jc.Contains, "Juju GUI installation has been disabled\n")
 	c.Assert(env.instanceConfig.Bootstrap.GUI, gc.IsNil)
 }
 
@@ -765,7 +768,7 @@ func (s *bootstrapSuite) TestBootstrapGUINoStreams(c *gc.C) {
 		return nil, nil
 	})
 	env := newEnviron("foo", useDefaultKeys, nil)
-	ctx := coretesting.Context(c)
+	ctx := cmdtesting.Context(c)
 	err := bootstrap.Bootstrap(modelcmd.BootstrapContext(ctx), env, bootstrap.BootstrapParams{
 		ControllerConfig:     coretesting.FakeControllerConfig(),
 		AdminSecret:          "admin-secret",
@@ -773,7 +776,7 @@ func (s *bootstrapSuite) TestBootstrapGUINoStreams(c *gc.C) {
 		GUIDataSourceBaseURL: "https://1.2.3.4/gui/sources",
 	})
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(coretesting.Stderr(ctx), jc.Contains, "No available Juju GUI archives found\n")
+	c.Assert(cmdtesting.Stderr(ctx), jc.Contains, "No available Juju GUI archives found\n")
 	c.Assert(env.instanceConfig.Bootstrap.GUI, gc.IsNil)
 }
 
@@ -782,7 +785,7 @@ func (s *bootstrapSuite) TestBootstrapGUIStreamsFailure(c *gc.C) {
 		return nil, errors.New("bad wolf")
 	})
 	env := newEnviron("foo", useDefaultKeys, nil)
-	ctx := coretesting.Context(c)
+	ctx := cmdtesting.Context(c)
 	err := bootstrap.Bootstrap(modelcmd.BootstrapContext(ctx), env, bootstrap.BootstrapParams{
 		ControllerConfig:     coretesting.FakeControllerConfig(),
 		AdminSecret:          "admin-secret",
@@ -790,21 +793,21 @@ func (s *bootstrapSuite) TestBootstrapGUIStreamsFailure(c *gc.C) {
 		GUIDataSourceBaseURL: "https://1.2.3.4/gui/sources",
 	})
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(coretesting.Stderr(ctx), jc.Contains, "Unable to fetch Juju GUI info: bad wolf\n")
+	c.Assert(cmdtesting.Stderr(ctx), jc.Contains, "Unable to fetch Juju GUI info: bad wolf\n")
 	c.Assert(env.instanceConfig.Bootstrap.GUI, gc.IsNil)
 }
 
 func (s *bootstrapSuite) TestBootstrapGUIErrorNotFound(c *gc.C) {
 	s.PatchEnvironment("JUJU_GUI", "/no/such/file")
 	env := newEnviron("foo", useDefaultKeys, nil)
-	ctx := coretesting.Context(c)
+	ctx := cmdtesting.Context(c)
 	err := bootstrap.Bootstrap(modelcmd.BootstrapContext(ctx), env, bootstrap.BootstrapParams{
 		ControllerConfig: coretesting.FakeControllerConfig(),
 		AdminSecret:      "admin-secret",
 		CAPrivateKey:     coretesting.CAKey,
 	})
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(coretesting.Stderr(ctx), jc.Contains, `Cannot use Juju GUI at "/no/such/file": cannot open Juju GUI archive:`)
+	c.Assert(cmdtesting.Stderr(ctx), jc.Contains, `Cannot use Juju GUI at "/no/such/file": cannot open Juju GUI archive:`)
 }
 
 func (s *bootstrapSuite) TestBootstrapGUIErrorInvalidArchive(c *gc.C) {
@@ -813,42 +816,42 @@ func (s *bootstrapSuite) TestBootstrapGUIErrorInvalidArchive(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	s.PatchEnvironment("JUJU_GUI", path)
 	env := newEnviron("foo", useDefaultKeys, nil)
-	ctx := coretesting.Context(c)
+	ctx := cmdtesting.Context(c)
 	err = bootstrap.Bootstrap(modelcmd.BootstrapContext(ctx), env, bootstrap.BootstrapParams{
 		ControllerConfig: coretesting.FakeControllerConfig(),
 		AdminSecret:      "admin-secret",
 		CAPrivateKey:     coretesting.CAKey,
 	})
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(coretesting.Stderr(ctx), jc.Contains, fmt.Sprintf("Cannot use Juju GUI at %q: cannot read Juju GUI archive", path))
+	c.Assert(cmdtesting.Stderr(ctx), jc.Contains, fmt.Sprintf("Cannot use Juju GUI at %q: cannot read Juju GUI archive", path))
 }
 
 func (s *bootstrapSuite) TestBootstrapGUIErrorInvalidVersion(c *gc.C) {
 	path := makeGUIArchive(c, "jujugui-invalid")
 	s.PatchEnvironment("JUJU_GUI", path)
 	env := newEnviron("foo", useDefaultKeys, nil)
-	ctx := coretesting.Context(c)
+	ctx := cmdtesting.Context(c)
 	err := bootstrap.Bootstrap(modelcmd.BootstrapContext(ctx), env, bootstrap.BootstrapParams{
 		ControllerConfig: coretesting.FakeControllerConfig(),
 		AdminSecret:      "admin-secret",
 		CAPrivateKey:     coretesting.CAKey,
 	})
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(coretesting.Stderr(ctx), jc.Contains, fmt.Sprintf(`Cannot use Juju GUI at %q: cannot parse version "invalid"`, path))
+	c.Assert(cmdtesting.Stderr(ctx), jc.Contains, fmt.Sprintf(`Cannot use Juju GUI at %q: cannot parse version "invalid"`, path))
 }
 
 func (s *bootstrapSuite) TestBootstrapGUIErrorUnexpectedArchive(c *gc.C) {
 	path := makeGUIArchive(c, "not-a-gui")
 	s.PatchEnvironment("JUJU_GUI", path)
 	env := newEnviron("foo", useDefaultKeys, nil)
-	ctx := coretesting.Context(c)
+	ctx := cmdtesting.Context(c)
 	err := bootstrap.Bootstrap(modelcmd.BootstrapContext(ctx), env, bootstrap.BootstrapParams{
 		ControllerConfig: coretesting.FakeControllerConfig(),
 		AdminSecret:      "admin-secret",
 		CAPrivateKey:     coretesting.CAKey,
 	})
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(coretesting.Stderr(ctx), jc.Contains, fmt.Sprintf("Cannot use Juju GUI at %q: cannot find Juju GUI version", path))
+	c.Assert(cmdtesting.Stderr(ctx), jc.Contains, fmt.Sprintf("Cannot use Juju GUI at %q: cannot find Juju GUI version", path))
 }
 
 func makeGUIArchive(c *gc.C, dir string) string {
@@ -892,6 +895,9 @@ func createImageMetadataForArch(c *gc.C, arch string) (dir string, _ []*imagemet
 	return sourceDir, im
 }
 
+// TestBootstrapMetadata tests:
+// `juju bootstrap --metadata-source <dir>` where <dir>/images
+// and <dir>/tools exist
 func (s *bootstrapSuite) TestBootstrapMetadata(c *gc.C) {
 	environs.UnregisterImageDataSourceFunc("bootstrap metadata")
 
@@ -929,6 +935,80 @@ func (s *bootstrapSuite) TestBootstrapMetadata(c *gc.C) {
 	c.Assert(env.instanceConfig.Bootstrap.CustomImageMetadata[0], gc.DeepEquals, metadata[0])
 }
 
+func (s *bootstrapSuite) TestBootstrapMetadataDirNonexistend(c *gc.C) {
+	env := newEnviron("foo", useDefaultKeys, nil)
+	nonExistentFileName := "/tmp/TestBootstrapMetadataDirNonexistend"
+	err := bootstrap.Bootstrap(envtesting.BootstrapContext(c), env, bootstrap.BootstrapParams{
+		ControllerConfig: coretesting.FakeControllerConfig(),
+		AdminSecret:      "admin-secret",
+		CAPrivateKey:     coretesting.CAKey,
+		MetadataDir:      nonExistentFileName,
+	})
+	c.Assert(err, gc.NotNil)
+	c.Assert(err, gc.ErrorMatches, fmt.Sprintf("simplestreams metadata source: %s not found", nonExistentFileName))
+}
+
+// TestBootstrapMetadataImagesNoTools tests 2 cases:
+// juju bootstrap --metadata-source <dir>
+// juju bootstrap --metadata-source <dir>/images
+// where <dir>/tools doesn't exist
+func (s *bootstrapSuite) TestBootstrapMetadataImagesNoTools(c *gc.C) {
+
+	metadataDir, _ := createImageMetadata(c)
+	env := newEnviron("foo", useDefaultKeys, nil)
+	s.setDummyStorage(c, env)
+
+	startingDefaultBaseURL := envtools.DefaultBaseURL
+	for i, suffix := range []string{"", "images"} {
+		environs.UnregisterImageDataSourceFunc("bootstrap metadata")
+		err := bootstrap.Bootstrap(envtesting.BootstrapContext(c), env, bootstrap.BootstrapParams{
+			ControllerConfig: coretesting.FakeControllerConfig(),
+			AdminSecret:      "admin-secret",
+			CAPrivateKey:     coretesting.CAKey,
+			MetadataDir:      filepath.Join(metadataDir, suffix),
+		})
+		c.Assert(err, jc.ErrorIsNil)
+		c.Assert(env.bootstrapCount, gc.Equals, i+1)
+		c.Assert(envtools.DefaultBaseURL, gc.Equals, startingDefaultBaseURL)
+
+		datasources, err := environs.ImageMetadataSources(env)
+		c.Assert(err, jc.ErrorIsNil)
+		c.Assert(datasources, gc.HasLen, 3)
+		c.Assert(datasources[0].Description(), gc.Equals, "bootstrap metadata")
+	}
+}
+
+// TestBootstrapMetadataToolsNoImages tests 2 cases:
+// juju bootstrap --metadata-source <dir>
+// juju bootstrap --metadata-source <dir>/tools
+// where <dir>/images doesn't exist
+func (s *bootstrapSuite) TestBootstrapMetadataToolsNoImages(c *gc.C) {
+	environs.UnregisterImageDataSourceFunc("bootstrap metadata")
+
+	metadataDir := c.MkDir()
+	stor, err := filestorage.NewFileStorageWriter(metadataDir)
+	c.Assert(err, jc.ErrorIsNil)
+	envtesting.UploadFakeTools(c, stor, "released", "released")
+
+	env := newEnviron("foo", useDefaultKeys, nil)
+	s.setDummyStorage(c, env)
+	for i, suffix := range []string{"", "tools"} {
+		err = bootstrap.Bootstrap(envtesting.BootstrapContext(c), env, bootstrap.BootstrapParams{
+			ControllerConfig: coretesting.FakeControllerConfig(),
+			AdminSecret:      "admin-secret",
+			CAPrivateKey:     coretesting.CAKey,
+			MetadataDir:      filepath.Join(metadataDir, suffix),
+		})
+		c.Assert(err, jc.ErrorIsNil)
+		c.Assert(env.bootstrapCount, gc.Equals, i+1)
+		c.Assert(envtools.DefaultBaseURL, gc.Equals, metadataDir)
+		datasources, err := environs.ImageMetadataSources(env)
+		c.Assert(err, jc.ErrorIsNil)
+		c.Assert(datasources, gc.HasLen, 2)
+		c.Assert(datasources[0].Description(), gc.Not(gc.Equals), "bootstrap metadata")
+	}
+}
+
 func (s *bootstrapSuite) TestBootstrapCloudCredential(c *gc.C) {
 	env := newEnviron("foo", useDefaultKeys, nil)
 	s.setDummyStorage(c, env)
@@ -938,11 +1018,11 @@ func (s *bootstrapSuite) TestBootstrapCloudCredential(c *gc.C) {
 		AdminSecret:      "admin-secret",
 		CAPrivateKey:     coretesting.CAKey,
 		Cloud: cloud.Cloud{
+			Name:      "cloud-name",
 			Type:      "dummy",
 			AuthTypes: []cloud.AuthType{cloud.EmptyAuthType},
 			Regions:   []cloud.Region{{Name: "region-name"}},
 		},
-		CloudName:           "cloud-name",
 		CloudRegion:         "region-name",
 		CloudCredentialName: "credential-name",
 		CloudCredential:     &credential,
@@ -952,7 +1032,6 @@ func (s *bootstrapSuite) TestBootstrapCloudCredential(c *gc.C) {
 	c.Assert(env.bootstrapCount, gc.Equals, 1)
 	c.Assert(env.instanceConfig, gc.NotNil)
 	c.Assert(env.instanceConfig.Bootstrap.ControllerCloud, jc.DeepEquals, args.Cloud)
-	c.Assert(env.instanceConfig.Bootstrap.ControllerCloudName, jc.DeepEquals, args.CloudName)
 	c.Assert(env.instanceConfig.Bootstrap.ControllerCloudRegion, jc.DeepEquals, args.CloudRegion)
 	c.Assert(env.instanceConfig.Bootstrap.ControllerCloudCredential, jc.DeepEquals, args.CloudCredential)
 	c.Assert(env.instanceConfig.Bootstrap.ControllerCloudCredentialName, jc.DeepEquals, args.CloudCredentialName)
@@ -980,8 +1059,8 @@ func (s *bootstrapSuite) TestFinishBootstrapConfig(c *gc.C) {
 
 	password := "lisboan-pork"
 
-	cloudName := "dummy"
 	dummyCloud := cloud.Cloud{
+		Name: "dummy",
 		RegionConfig: cloud.RegionConfig{
 			"a-region": cloud.Attrs{
 				"a-key": "a-value",
@@ -996,10 +1075,9 @@ func (s *bootstrapSuite) TestFinishBootstrapConfig(c *gc.C) {
 	err := bootstrap.Bootstrap(envtesting.BootstrapContext(c), env, bootstrap.BootstrapParams{
 		ControllerConfig:          coretesting.FakeControllerConfig(),
 		ControllerInheritedConfig: map[string]interface{}{"ftp-proxy": "http://proxy"},
-		CloudName:                 cloudName,
-		Cloud:                     dummyCloud,
-		AdminSecret:               password,
-		CAPrivateKey:              coretesting.CAKey,
+		Cloud:        dummyCloud,
+		AdminSecret:  password,
+		CAPrivateKey: coretesting.CAKey,
 	})
 	c.Assert(err, jc.ErrorIsNil)
 	icfg := env.instanceConfig
@@ -1029,7 +1107,7 @@ func (s *bootstrapSuite) TestFinishBootstrapConfig(c *gc.C) {
 
 	srvCertPEM := icfg.Bootstrap.StateServingInfo.Cert
 	srvKeyPEM := icfg.Bootstrap.StateServingInfo.PrivateKey
-	_, _, err = cert.ParseCertAndKey(srvCertPEM, srvKeyPEM)
+	_, _, err = utilscert.ParseCertAndKey(srvCertPEM, srvKeyPEM)
 	c.Check(err, jc.ErrorIsNil)
 
 	// TODO(perrito666) 2016-05-02 lp:1558657
@@ -1073,7 +1151,7 @@ func (s *bootstrapSuite) setupBootstrapSpecificVersion(c *gc.C, clientMajor, cli
 	currentVersion.Minor = clientMinor
 	currentVersion.Tag = ""
 	s.PatchValue(&jujuversion.Current, currentVersion)
-	s.PatchValue(&series.HostSeries, func() string { return "trusty" })
+	s.PatchValue(&series.MustHostSeries, func() string { return "trusty" })
 	s.PatchValue(&arch.HostArch, func() string { return arch.AMD64 })
 
 	env := newEnviron("foo", useDefaultKeys, nil)
@@ -1239,7 +1317,7 @@ func (e *bootstrapEnviron) Bootstrap(ctx environs.BootstrapContext, args environ
 		e.instanceConfig = icfg
 		return nil
 	}
-	series := series.HostSeries()
+	series := series.MustHostSeries()
 	if args.BootstrapSeries != "" {
 		series = args.BootstrapSeries
 	}
@@ -1266,6 +1344,18 @@ func (e *bootstrapEnviron) ConstraintsValidator() (constraints.Validator, error)
 	return v, nil
 }
 
+func (e *bootstrapEnviron) Provider() environs.EnvironProvider {
+	return bootstrapEnvironProvider{}
+}
+
+type bootstrapEnvironProvider struct {
+	environs.EnvironProvider
+}
+
+func (p bootstrapEnvironProvider) Version() int {
+	return 123
+}
+
 type bootstrapEnvironWithRegion struct {
 	*bootstrapEnviron
 	region simplestreams.CloudSpec
