@@ -7,6 +7,7 @@ import (
 	"fmt"
 	"os"
 	"strings"
+	"time"
 
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
@@ -84,6 +85,9 @@ const (
 	// FTPProxyKey stores the key for this setting.
 	FTPProxyKey = "ftp-proxy"
 
+	// NoProxyKey stores the key for this setting.
+	NoProxyKey = "no-proxy"
+
 	// AptHTTPProxyKey stores the key for this setting.
 	AptHTTPProxyKey = "apt-http-proxy"
 
@@ -93,8 +97,12 @@ const (
 	// AptFTPProxyKey stores the key for this setting.
 	AptFTPProxyKey = "apt-ftp-proxy"
 
-	// NoProxyKey stores the key for this setting.
-	NoProxyKey = "no-proxy"
+	// AptNoProxyKey stores the key for this setting.
+	AptNoProxyKey = "apt-no-proxy"
+
+	// NetBondReconfigureDelay is the key to pass when bridging
+	// the network for containers.
+	NetBondReconfigureDelayKey = "net-bond-reconfigure-delay"
 
 	// The default block storage source.
 	StorageDefaultBlockSourceKey = "storage-default-block-source"
@@ -129,6 +137,29 @@ const (
 	// metrics collected in this model for anonymized aggregate analytics.
 	TransmitVendorMetricsKey = "transmit-vendor-metrics"
 
+	// ExtraInfoKey is the key for arbitrary user specified string data that
+	// is stored against the model.
+	ExtraInfoKey = "extra-info"
+
+	// MaxStatusHistoryAge is the maximum age of status history values
+	// to keep when pruning, eg "72h"
+	MaxStatusHistoryAge = "max-status-history-age"
+
+	// MaxStatusHistorySize is the maximum size the status history
+	// collection can grow to before it is pruned, eg "5M"
+	MaxStatusHistorySize = "max-status-history-size"
+
+	// MaxActionResultsAge is the maximum age of actions to keep when pruning, eg
+	// "72h"
+	MaxActionResultsAge = "max-action-results-age"
+
+	// MaxActionResultsSize is the maximum size the actions collection can
+	// grow to before it is pruned, eg "5M"
+	MaxActionResultsSize = "max-action-results-size"
+
+	// UpdateStatusHookInterval is how often to run the update-status hook.
+	UpdateStatusHookInterval = "update-status-hook-interval"
+
 	//
 	// Deprecated Settings Attributes
 	//
@@ -257,6 +288,7 @@ func New(withDefaults Defaulting, attrs map[string]interface{}) (*Config, error)
 	if err != nil {
 		return nil, err
 	}
+
 	c := &Config{
 		defined: defined.(map[string]interface{}),
 		unknown: make(map[string]interface{}),
@@ -277,6 +309,21 @@ func New(withDefaults Defaulting, attrs map[string]interface{}) (*Config, error)
 	return c, nil
 }
 
+const (
+	// DefaultStatusHistoryAge is the default value for MaxStatusHistoryAge.
+	DefaultStatusHistoryAge = "336h" // 2 weeks
+
+	// DefaultStatusHistorySize is the default value for MaxStatusHistorySize.
+	DefaultStatusHistorySize = "5G"
+
+	// DefaultUpdateStatusHookInterval is the default value for UpdateStatusHookInterval
+	DefaultUpdateStatusHookInterval = "5m"
+
+	DefaultActionResultsAge = "336h" // 2 weeks
+
+	DefaultActionResultsSize = "5G"
+)
+
 var defaultConfigValues = map[string]interface{}{
 	// Network.
 	"firewall-mode":              FwInstance,
@@ -285,6 +332,25 @@ var defaultConfigValues = map[string]interface{}{
 	"ssl-hostname-verification":  true,
 	"proxy-ssh":                  false,
 
+	// Why is net-bond-reconfigure-delay set to 17 seconds?
+	//
+	// The value represents the amount of time in seconds to sleep
+	// between ifdown and ifup when bridging bonded interfaces;
+	// this is a platform bug and all of this can go away when bug
+	// #1657579 (and #1594855 and #1269921) are fixed.
+	//
+	// For a long time the bridge script hardcoded a value of 3s
+	// but some setups now require an even longer period. The last
+	// reported issue was fixed with a 10s timeout, however, we're
+	// increasing that because this issue (and solution) is not
+	// very discoverable and we would like bridging to work
+	// out-of-the-box.
+	//
+	// This value can be further tweaked via:
+	//
+	// $ juju model-config net-bond-reconfigure-delay=30
+	NetBondReconfigureDelayKey: 17,
+
 	"default-series":           series.LatestLts(),
 	ProvisionerHarvestModeKey:  HarvestDestroyed.String(),
 	ResourceTagsKey:            "",
@@ -295,6 +361,7 @@ var defaultConfigValues = map[string]interface{}{
 	"development":              false,
 	"test-mode":                false,
 	TransmitVendorMetricsKey:   true,
+	UpdateStatusHookInterval:   DefaultUpdateStatusHookInterval,
 
 	// Image and agent streams and URLs.
 	"image-stream":       "released",
@@ -309,11 +376,18 @@ var defaultConfigValues = map[string]interface{}{
 	HTTPProxyKey:     "",
 	HTTPSProxyKey:    "",
 	FTPProxyKey:      "",
-	NoProxyKey:       "",
+	NoProxyKey:       "127.0.0.1,localhost,::1",
 	AptHTTPProxyKey:  "",
 	AptHTTPSProxyKey: "",
 	AptFTPProxyKey:   "",
+	AptNoProxyKey:    "",
 	"apt-mirror":     "",
+
+	// Status history settings
+	MaxStatusHistoryAge:  DefaultStatusHistoryAge,
+	MaxStatusHistorySize: DefaultStatusHistorySize,
+	MaxActionResultsAge:  DefaultActionResultsAge,
+	MaxActionResultsSize: DefaultActionResultsSize,
 }
 
 // ConfigDefaults returns the config default values
@@ -451,6 +525,43 @@ func Validate(cfg, old *Config) error {
 		return errors.Annotate(err, "validating resource tags")
 	}
 
+	if v, ok := cfg.defined[MaxStatusHistoryAge].(string); ok {
+		if _, err := time.ParseDuration(v); err != nil {
+			return errors.Annotate(err, "invalid max status history age in model configuration")
+		}
+	}
+
+	if v, ok := cfg.defined[MaxStatusHistorySize].(string); ok {
+		if _, err := utils.ParseSize(v); err != nil {
+			return errors.Annotate(err, "invalid max status history size in model configuration")
+		}
+	}
+
+	if v, ok := cfg.defined[MaxActionResultsAge].(string); ok {
+		if _, err := time.ParseDuration(v); err != nil {
+			return errors.Annotate(err, "invalid max action age in model configuration")
+		}
+	}
+
+	if v, ok := cfg.defined[MaxActionResultsSize].(string); ok {
+		if _, err := utils.ParseSize(v); err != nil {
+			return errors.Annotate(err, "invalid max action size in model configuration")
+		}
+	}
+
+	if v, ok := cfg.defined[UpdateStatusHookInterval].(string); ok {
+		if f, err := time.ParseDuration(v); err != nil {
+			return errors.Annotate(err, "invalid update status hook interval in model configuration")
+		} else {
+			if f < 1*time.Minute {
+				return errors.Annotatef(err, "update status hook frequency %v cannot be less than 1m", f)
+			}
+			if f > 60*time.Minute {
+				return errors.Annotatef(err, "update status hook frequency %v cannot be greater than 60m", f)
+			}
+		}
+	}
+
 	// Check the immutable config values.  These can't change
 	if old != nil {
 		for _, attr := range immutableAttributes {
@@ -512,17 +623,6 @@ func (c *Config) mustString(name string) string {
 	return value
 }
 
-// mustInt returns the named attribute as an integer, panicking if
-// it is not found or is zero. Zero values should have been
-// diagnosed at Validate time.
-func (c *Config) mustInt(name string) int {
-	value, _ := c.defined[name].(int)
-	if value == 0 {
-		panic(fmt.Errorf("empty value for %q found in configuration", name))
-	}
-	return value
-}
-
 // Type returns the model's cloud provider type.
 func (c *Config) Type() string {
 	return c.mustString(TypeKey)
@@ -567,6 +667,13 @@ func (c *Config) ProxySSH() bool {
 	return value
 }
 
+// NetBondReconfigureDelay returns the duration in seconds that should be
+// passed to the bridge script when bridging bonded interfaces.
+func (c *Config) NetBondReconfigureDelay() int {
+	value, _ := c.defined[NetBondReconfigureDelayKey].(int)
+	return value
+}
+
 // ProxySettings returns all four proxy settings; http, https, ftp, and no
 // proxy.
 func (c *Config) ProxySettings() proxy.Settings {
@@ -593,7 +700,7 @@ func (c *Config) FTPProxy() string {
 	return c.asString(FTPProxyKey)
 }
 
-// NoProxy returns the 'no proxy' for the environment.
+// NoProxy returns the 'no-proxy' for the environment.
 func (c *Config) NoProxy() string {
 	return c.asString(NoProxyKey)
 }
@@ -617,9 +724,10 @@ func addSchemeIfMissing(defaultScheme string, url string) string {
 // AptProxySettings returns all three proxy settings; http, https and ftp.
 func (c *Config) AptProxySettings() proxy.Settings {
 	return proxy.Settings{
-		Http:  c.AptHTTPProxy(),
-		Https: c.AptHTTPSProxy(),
-		Ftp:   c.AptFTPProxy(),
+		Http:    c.AptHTTPProxy(),
+		Https:   c.AptHTTPSProxy(),
+		Ftp:     c.AptFTPProxy(),
+		NoProxy: c.AptNoProxy(),
 	}
 }
 
@@ -641,6 +749,11 @@ func (c *Config) AptFTPProxy() string {
 	return addSchemeIfMissing("ftp", c.getWithFallback(AptFTPProxyKey, FTPProxyKey))
 }
 
+// AptNoProxy returns the 'apt-no-proxy' for the environment.
+func (c *Config) AptNoProxy() string {
+	return c.getWithFallback(AptNoProxyKey, NoProxyKey)
+}
+
 // AptMirror sets the apt mirror for the environment.
 func (c *Config) AptMirror() string {
 	return c.asString("apt-mirror")
@@ -870,6 +983,50 @@ func (c *Config) resourceTags() (map[string]string, error) {
 	return v, nil
 }
 
+// MaxStatusHistoryAge is the maximum age of status history entries
+// before being pruned.
+func (c *Config) MaxStatusHistoryAge() time.Duration {
+	// Value has already been validated.
+	val, _ := time.ParseDuration(c.mustString(MaxStatusHistoryAge))
+	return val
+}
+
+// MaxStatusHistorySizeMB is the maximum size in MiB which the status history
+// collection can grow to before being pruned.
+func (c *Config) MaxStatusHistorySizeMB() uint {
+	// Value has already been validated.
+	val, _ := utils.ParseSize(c.mustString(MaxStatusHistorySize))
+	return uint(val)
+}
+
+func (c *Config) MaxActionResultsAge() time.Duration {
+	// Value has already been validated.
+	val, _ := time.ParseDuration(c.mustString(MaxActionResultsAge))
+	return val
+}
+
+func (c *Config) MaxActionResultsSizeMB() uint {
+	// Value has already been validated.
+	val, _ := utils.ParseSize(c.mustString(MaxActionResultsSize))
+	return uint(val)
+}
+
+// UpdateStatusHookInterval is how often to run the charm
+// update-status hook.
+func (c *Config) UpdateStatusHookInterval() time.Duration {
+	// TODO(wallyworld) - remove this work around when possible as
+	// we already have a defaulting mechanism for config.
+	// It's only here to guard against using Juju clients >= 2.2
+	// with Juju controllers running 2.1.x
+	raw := c.asString(UpdateStatusHookInterval)
+	if raw == "" {
+		raw = DefaultUpdateStatusHookInterval
+	}
+	// Value has already been validated.
+	val, _ := time.ParseDuration(raw)
+	return val
+}
+
 // UnknownAttrs returns a copy of the raw configuration attributes
 // that are supposedly specific to the environment type. They could
 // also be wrong attributes, though. Only the specific environment
@@ -933,6 +1090,7 @@ var fields = func() schema.Fields {
 var alwaysOptional = schema.Defaults{
 	AgentVersionKey:   schema.Omit,
 	AuthorizedKeysKey: schema.Omit,
+	ExtraInfoKey:      schema.Omit,
 
 	LogForwardEnabled:      schema.Omit,
 	LogFwdSyslogHost:       schema.Omit,
@@ -954,6 +1112,7 @@ var alwaysOptional = schema.Defaults{
 	AptHTTPProxyKey:              schema.Omit,
 	AptHTTPSProxyKey:             schema.Omit,
 	AptFTPProxyKey:               schema.Omit,
+	AptNoProxyKey:                schema.Omit,
 	"apt-mirror":                 schema.Omit,
 	AgentStreamKey:               schema.Omit,
 	ResourceTagsKey:              schema.Omit,
@@ -972,6 +1131,12 @@ var alwaysOptional = schema.Defaults{
 	AutomaticallyRetryHooks:      schema.Omit,
 	"test-mode":                  schema.Omit,
 	TransmitVendorMetricsKey:     schema.Omit,
+	NetBondReconfigureDelayKey:   schema.Omit,
+	MaxStatusHistoryAge:          schema.Omit,
+	MaxStatusHistorySize:         schema.Omit,
+	MaxActionResultsAge:          schema.Omit,
+	MaxActionResultsSize:         schema.Omit,
+	UpdateStatusHookInterval:     schema.Omit,
 }
 
 func allowEmpty(attr string) bool {
@@ -1019,9 +1184,9 @@ var (
 // of juju that does recognise the fields, but that their presence is still
 // anomalous to some degree and should be flagged (and that there is thereby
 // a mechanism for observing fields that really are typos etc).
-func (cfg *Config) ValidateUnknownAttrs(fields schema.Fields, defaults schema.Defaults) (map[string]interface{}, error) {
+func (cfg *Config) ValidateUnknownAttrs(extrafields schema.Fields, defaults schema.Defaults) (map[string]interface{}, error) {
 	attrs := cfg.UnknownAttrs()
-	checker := schema.FieldMap(fields, defaults)
+	checker := schema.FieldMap(extrafields, defaults)
 	coerced, err := checker.Coerce(attrs, nil)
 	if err != nil {
 		// TODO(ericsnow) Drop this?
@@ -1030,10 +1195,19 @@ func (cfg *Config) ValidateUnknownAttrs(fields schema.Fields, defaults schema.De
 	}
 	result := coerced.(map[string]interface{})
 	for name, value := range attrs {
-		if fields[name] == nil {
+		if extrafields[name] == nil {
+			// We know this name isn't in the global fields, or it wouldn't be
+			// an UnknownAttr, it also appears to not be in the extra fields
+			// that are provider specific.  Check to see if an alternative
+			// spelling is in either the extra fields or the core fields.
 			if val, isString := value.(string); isString && val != "" {
 				// only warn about attributes with non-empty string values
-				logger.Warningf("unknown config field %q", name)
+				altName := strings.Replace(name, "_", "-", -1)
+				if extrafields[altName] != nil || fields[altName] != nil {
+					logger.Warningf("unknown config field %q, did you mean %q?", name, altName)
+				} else {
+					logger.Warningf("unknown config field %q", name)
+				}
 			}
 			result[name] = value
 		}
@@ -1079,6 +1253,7 @@ func AptProxyConfigMap(proxySettings proxy.Settings) map[string]interface{} {
 	addIfNotEmpty(settings, AptHTTPProxyKey, proxySettings.Http)
 	addIfNotEmpty(settings, AptHTTPSProxyKey, proxySettings.Https)
 	addIfNotEmpty(settings, AptFTPProxyKey, proxySettings.Ftp)
+	addIfNotEmpty(settings, AptNoProxyKey, proxySettings.NoProxy)
 	return settings
 }
 
@@ -1144,6 +1319,11 @@ var configSchema = environschema.Fields{
 		Type:        environschema.Tstring,
 		Group:       environschema.EnvironGroup,
 	},
+	AptNoProxyKey: {
+		Description: "List of domain addresses not to be proxied for APT (comma-separated)",
+		Type:        environschema.Tstring,
+		Group:       environschema.EnvironGroup,
+	},
 	"apt-mirror": {
 		// TODO document acceptable format
 		Description: "The APT mirror for the model",
@@ -1185,6 +1365,11 @@ var configSchema = environschema.Fields{
 		Type:        environschema.Tbool,
 		Group:       environschema.EnvironGroup,
 	},
+	ExtraInfoKey: {
+		Description: "Arbitrary user specified string data that is stored against the model.",
+		Type:        environschema.Tstring,
+		Group:       environschema.EnvironGroup,
+	},
 	"firewall-mode": {
 		Description: `The mode to use for network firewalling.
 
@@ -1327,4 +1512,34 @@ data of the store. (default false)`,
 		Type:        environschema.Tbool,
 		Group:       environschema.EnvironGroup,
 	},
+	NetBondReconfigureDelayKey: {
+		Description: "The amount of time in seconds to sleep between ifdown and ifup when bridging",
+		Type:        environschema.Tint,
+		Group:       environschema.EnvironGroup,
+	},
+	MaxStatusHistoryAge: {
+		Description: "The maximum age for status history entries before they are pruned, in human-readable time format",
+		Type:        environschema.Tstring,
+		Group:       environschema.EnvironGroup,
+	},
+	MaxStatusHistorySize: {
+		Description: "The maximum size for the status history collection, in human-readable memory format",
+		Type:        environschema.Tstring,
+		Group:       environschema.EnvironGroup,
+	},
+	MaxActionResultsAge: {
+		Description: "The maximum age for action entries before they are pruned, in human-readable time format",
+		Type:        environschema.Tstring,
+		Group:       environschema.EnvironGroup,
+	},
+	MaxActionResultsSize: {
+		Description: "The maximum size for the action collection, in human-readable memory format",
+		Type:        environschema.Tstring,
+		Group:       environschema.EnvironGroup,
+	},
+	UpdateStatusHookInterval: {
+		Description: "How often to run the charm update-status hook, in human-readable time format (default 5m, range 1-60m)",
+		Type:        environschema.Tstring,
+		Group:       environschema.EnvironGroup,
+	},
 }
