@@ -6,11 +6,14 @@ package environs
 import (
 	"io"
 
+	"github.com/juju/jsonschema"
+	"github.com/juju/version"
 	"gopkg.in/juju/environschema.v1"
 
 	"github.com/juju/juju/cloud"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs/config"
+	"github.com/juju/juju/environs/instances"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/storage"
@@ -21,7 +24,19 @@ type EnvironProvider interface {
 	config.Validator
 	ProviderCredentials
 
-	// TODO(wallyworld) - embed config.ConfigSchemaSource and make all providers implement it
+	// Version returns the version of the provider. This is recorded as the
+	// environ version for each model, and used to identify which upgrade
+	// operations to run when upgrading a model's environ. Providers should
+	// start out at version 0.
+	Version() int
+
+	// CloudSchema returns the schema used to validate input for add-cloud.  If
+	// a provider does not suppport custom clouds, CloudSchema should return
+	// nil.
+	CloudSchema() *jsonschema.Schema
+
+	// Ping tests the connection to the cloud, to verify the endpoint is valid.
+	Ping(endpoint string) error
 
 	// PrepareConfig prepares the configuration for a new model, based on
 	// the provided arguments. PrepareConfig is expected to produce a
@@ -112,7 +127,7 @@ type FinalizeCredentialContext interface {
 // FinalizeCredentialParams contains the parameters for
 // ProviderCredentials.FinalizeCredential.
 type FinalizeCredentialParams struct {
-	// Credential is the credential that the provider should finalize.`
+	// Credential is the credential that the provider should finalize.
 	Credential cloud.Credential
 
 	// CloudEndpoint is the endpoint for the cloud that the credentials are
@@ -120,16 +135,60 @@ type FinalizeCredentialParams struct {
 	// to finalize the credentials.
 	CloudEndpoint string
 
+	// CloudStorageEndpoint is the storage endpoint for the cloud that the
+	// credentials are for. This may be used by the provider to communicate
+	// with the cloud to finalize the credentials.
+	CloudStorageEndpoint string
+
 	// CloudIdentityEndpoint is the identity endpoint for the cloud that the
 	// credentials are for. This may be used by the provider to communicate
 	// with the cloud to finalize the credentials.
 	CloudIdentityEndpoint string
 }
 
+// FinalizeCloudContext is an interface passed into FinalizeCloud
+// to provide a means of interacting with the user when finalizing
+// a cloud definition.
+type FinalizeCloudContext interface {
+	// Verbosef will write the formatted string to Stderr if the
+	// verbose flag is true, and to the logger if not.
+	Verbosef(string, ...interface{})
+}
+
+// CloudFinalizer is an interface that an EnvironProvider implements
+// in order to finalize a cloud.Cloud definition before bootstrapping.
+type CloudFinalizer interface {
+	// FinalizeCloud finalizes a cloud definition, updating any attributes
+	// as necessary. This is always done client-side, before bootstrapping.
+	FinalizeCloud(FinalizeCloudContext, cloud.Cloud) (cloud.Cloud, error)
+}
+
+// CloudDetector is an interface that an EnvironProvider implements
+// in order to automatically detect clouds from the environment.
+type CloudDetector interface {
+	// DetectCloud attempts to detect a cloud with the given name
+	// from the environment. This may involve, for example,
+	// inspecting environment variables, or returning special
+	// hard-coded regions (e.g. "localhost" for lxd).
+	//
+	// If no cloud can be detected, DetectCloud should return
+	// an error satisfying errors.IsNotFound.
+	//
+	// DetectCloud should be used in preference to DetectClouds
+	// when a specific cloud is identified, as this may be more
+	// efficient.
+	DetectCloud(name string) (cloud.Cloud, error)
+
+	// DetectClouds detects clouds from the environment. This may
+	// involve, for example, inspecting environment variables, or
+	// returning special hard-coded regions (e.g. "localhost" for lxd).
+	DetectClouds() ([]cloud.Cloud, error)
+}
+
 // CloudRegionDetector is an interface that an EnvironProvider implements
 // in order to automatically detect cloud regions from the environment.
 type CloudRegionDetector interface {
-	// DetectRetions automatically detects one or more regions
+	// DetectRegions automatically detects one or more regions
 	// from the environment. This may involve, for example, inspecting
 	// environment variables, or returning special hard-coded regions
 	// (e.g. "localhost" for lxd). The first item in the list will be
@@ -205,10 +264,6 @@ type Environ interface {
 	// architecture.
 	Bootstrap(ctx BootstrapContext, params BootstrapParams) (*BootstrapResult, error)
 
-	// BootstrapMessage optionally provides a message to be displayed to
-	// the user at bootstrap time.
-	BootstrapMessage() string
-
 	// Create creates the environment for a new hosted model.
 	//
 	// This will be called before any workers begin operating on the
@@ -219,6 +274,19 @@ type Environ interface {
 	// the Bootstrap method's job to create the controller model.
 	Create(CreateParams) error
 
+	// AdoptResources is called when the model is moved from one
+	// controller to another using model migration. Some providers tag
+	// instances, disks, and cloud storage with the controller UUID to
+	// aid in clean destruction. This method will be called on the
+	// environ for the target controller so it can update the
+	// controller tags for all of those things. For providers that do
+	// not track the controller UUID, a simple method returning nil
+	// will suffice. The version number of the source controller is
+	// provided for backwards compatibility - if the technique used to
+	// tag items changes, the version number can be used to decide how
+	// to remove the old tags correctly.
+	AdoptResources(controllerUUID string, fromVersion version.Number) error
+
 	// InstanceBroker defines methods for starting and stopping
 	// instances.
 	InstanceBroker
@@ -288,6 +356,10 @@ type Environ interface {
 	// constraints package? Can't be instance, because constraints
 	// import instance...
 	PrecheckInstance(series string, cons constraints.Value, placement string) error
+
+	// InstanceTypesFetcher represents an environment that can return
+	// information about the available instance types.
+	InstanceTypesFetcher
 }
 
 // CreateParams contains the parameters for Environ.Create.
@@ -302,17 +374,20 @@ type Firewaller interface {
 	// OpenPorts opens the given port ranges for the whole environment.
 	// Must only be used if the environment was setup with the
 	// FwGlobal firewall mode.
-	OpenPorts(ports []network.PortRange) error
+	OpenPorts(rules []network.IngressRule) error
 
 	// ClosePorts closes the given port ranges for the whole environment.
 	// Must only be used if the environment was setup with the
 	// FwGlobal firewall mode.
-	ClosePorts(ports []network.PortRange) error
+	ClosePorts(rules []network.IngressRule) error
 
-	// Ports returns the port ranges opened for the whole environment.
+	// IngressRules returns the ingress rules applied to the whole environment.
 	// Must only be used if the environment was setup with the
 	// FwGlobal firewall mode.
-	Ports() ([]network.PortRange, error)
+	// It is expected that there be only one ingress rule result for a given
+	// port range - the rule's SourceCIDRs will contain all applicable source
+	// address rules for that port range.
+	IngressRules() ([]network.IngressRule, error)
 }
 
 // InstanceTagger is an interface that can be used for tagging instances.
@@ -323,3 +398,52 @@ type InstanceTagger interface {
 	// same names, but other existing tags will be left alone.
 	TagInstance(id instance.Id, tags map[string]string) error
 }
+
+// InstanceTypesFetcher is an interface that allows for instance information from
+// a provider to be obtained.
+type InstanceTypesFetcher interface {
+	InstanceTypes(constraints.Value) (instances.InstanceTypesWithCostMetadata, error)
+}
+
+// Upgrader is an interface that can be used for upgrading Environs. If an
+// Environ implements this interface, its UpgradeOperations method will be
+// invoked to identify operations that should be run on upgrade.
+type Upgrader interface {
+	// UpgradeOperations returns a list of UpgradeOperations for upgrading
+	// an Environ.
+	UpgradeOperations(UpgradeOperationsParams) []UpgradeOperation
+}
+
+// UpgradeOperationsParams contains the parameters for
+// Upgrader.UpgradeOperations.
+type UpgradeOperationsParams struct {
+	// ControllerUUID is the UUID of the controller that manages
+	// the Environ being upgraded.
+	ControllerUUID string
+}
+
+// UpgradeOperation contains a target agent version and sequence of upgrade
+// steps to apply to get to that version.
+type UpgradeOperation struct {
+	// TargetVersion is the target environ provider version number to
+	// which the upgrade steps pertain. When a model is upgraded, all
+	// upgrade operations will be run for versions greater than the
+	// recorded environ version. This version number is independent of
+	// the agent and controller versions.
+	TargetVersion int
+
+	// Steps contains the sequence of upgrade steps to apply when
+	// upgrading to the accompanying target version number.
+	Steps []UpgradeStep
+}
+
+// UpgradeStep defines an idempotent operation that is run to perform a
+// specific upgrade step on an Environ.
+type UpgradeStep interface {
+	// Description is a human readable description of what the upgrade
+	// step does.
+	Description() string
+
+	// Run executes the upgrade business logic.
+	Run() error
+}
