@@ -5,23 +5,28 @@ package jujutest
 
 import (
 	"fmt"
+	"path/filepath"
 	"time"
 
 	"github.com/juju/errors"
 	gitjujutesting "github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
+	"github.com/juju/utils/arch"
+	"github.com/juju/utils/series"
+	"github.com/juju/version"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v5/charmrepo"
+	"gopkg.in/juju/charmrepo.v2-unstable"
 
 	"github.com/juju/juju/api"
+	"github.com/juju/juju/cloud"
 	"github.com/juju/juju/cloudconfig/instancecfg"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/bootstrap"
 	"github.com/juju/juju/environs/config"
-	"github.com/juju/juju/environs/configstore"
 	"github.com/juju/juju/environs/filestorage"
+	sstesting "github.com/juju/juju/environs/simplestreams/testing"
 	"github.com/juju/juju/environs/storage"
 	"github.com/juju/juju/environs/sync"
 	envtesting "github.com/juju/juju/environs/testing"
@@ -29,15 +34,22 @@ import (
 	envtoolstesting "github.com/juju/juju/environs/tools/testing"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju"
+	"github.com/juju/juju/juju/keys"
 	jujutesting "github.com/juju/juju/juju/testing"
+	"github.com/juju/juju/jujuclient"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/provider/dummy"
 	"github.com/juju/juju/state"
 	statetesting "github.com/juju/juju/state/testing"
+	"github.com/juju/juju/status"
 	"github.com/juju/juju/testcharms"
 	coretesting "github.com/juju/juju/testing"
 	coretools "github.com/juju/juju/tools"
-	"github.com/juju/juju/version"
+	jujuversion "github.com/juju/juju/version"
+)
+
+const (
+	AdminSecret = "admin-secret"
 )
 
 // LiveTests contains tests that are designed to run against a live server
@@ -47,12 +59,25 @@ type LiveTests struct {
 	gitjujutesting.CleanupSuite
 
 	envtesting.ToolsFixture
+	sstesting.TestDataSuite
 
 	// TestConfig contains the configuration attributes for opening an environment.
 	TestConfig coretesting.Attrs
 
+	// Credential contains the credential for preparing an environment for
+	// bootstrapping. If this is unset, empty credentials will be used.
+	Credential cloud.Credential
+
+	// CloudRegion contains the cloud region name to create resources in.
+	CloudRegion string
+
+	// CloudEndpoint contains the cloud API endpoint to communicate with.
+	CloudEndpoint string
+
 	// Attempt holds a strategy for waiting until the environment
 	// becomes logically consistent.
+	//
+	// TODO(katco): 2016-08-09: lp:1611427
 	Attempt utils.AttemptStrategy
 
 	// CanOpenState should be true if the testing environment allows
@@ -67,10 +92,13 @@ type LiveTests struct {
 	// This is set by PrepareOnce and BootstrapOnce.
 	Env environs.Environ
 
-	// ConfigStore holds the configuration storage
-	// used when preparing the environment.
-	// This is initialized by SetUpSuite.
-	ConfigStore configstore.Storage
+	// ControllerStore holds the controller related informtion
+	// such as controllers, accounts, etc., used when preparing
+	// the environment. This is initialized by SetUpSuite.
+	ControllerStore jujuclient.ClientStore
+
+	// ControllerUUID is the uuid of the bootstrapped controller.
+	ControllerUUID string
 
 	prepared     bool
 	bootstrapped bool
@@ -79,36 +107,28 @@ type LiveTests struct {
 
 func (t *LiveTests) SetUpSuite(c *gc.C) {
 	t.CleanupSuite.SetUpSuite(c)
-	t.ConfigStore = configstore.NewMem()
+	t.TestDataSuite.SetUpSuite(c)
+	t.ControllerStore = jujuclient.NewMemStore()
+	t.PatchValue(&keys.JujuPublicKey, sstesting.SignedMetadataPublicKey)
 }
 
 func (t *LiveTests) SetUpTest(c *gc.C) {
 	t.CleanupSuite.SetUpTest(c)
+	t.PatchValue(&jujuversion.Current, coretesting.FakeVersionNumber)
 	storageDir := c.MkDir()
-	t.DefaultBaseURL = "file://" + storageDir + "/tools"
+	baseURLPath := filepath.Join(storageDir, "tools")
+	t.DefaultBaseURL = utils.MakeFileURL(baseURLPath)
 	t.ToolsFixture.SetUpTest(c)
 	stor, err := filestorage.NewFileStorageWriter(storageDir)
 	c.Assert(err, jc.ErrorIsNil)
 	t.UploadFakeTools(c, stor, "released", "released")
 	t.toolsStorage = stor
-	t.CleanupSuite.PatchValue(&envtools.BundleTools, envtoolstesting.GetMockBundleTools(c))
-}
-
-func publicAttrs(e environs.Environ) map[string]interface{} {
-	cfg := e.Config()
-	secrets, err := e.Provider().SecretAttrs(cfg)
-	if err != nil {
-		panic(err)
-	}
-	attrs := cfg.AllAttrs()
-	for attr := range secrets {
-		delete(attrs, attr)
-	}
-	return attrs
+	t.CleanupSuite.PatchValue(&envtools.BundleTools, envtoolstesting.GetMockBundleTools(c, nil))
 }
 
 func (t *LiveTests) TearDownSuite(c *gc.C) {
 	t.Destroy(c)
+	t.TestDataSuite.TearDownSuite(c)
 	t.CleanupSuite.TearDownSuite(c)
 }
 
@@ -123,13 +143,62 @@ func (t *LiveTests) PrepareOnce(c *gc.C) {
 	if t.prepared {
 		return
 	}
-	cfg, err := config.New(config.NoDefaults, t.TestConfig)
-	c.Assert(err, jc.ErrorIsNil)
-	e, err := environs.Prepare(cfg, envtesting.BootstrapContext(c), t.ConfigStore)
+	args := t.prepareForBootstrapParams(c)
+	e, err := bootstrap.Prepare(envtesting.BootstrapContext(c), t.ControllerStore, args)
 	c.Assert(err, gc.IsNil, gc.Commentf("preparing environ %#v", t.TestConfig))
 	c.Assert(e, gc.NotNil)
 	t.Env = e
 	t.prepared = true
+	t.ControllerUUID = coretesting.FakeControllerConfig().ControllerUUID()
+}
+
+func (t *LiveTests) prepareForBootstrapParams(c *gc.C) bootstrap.PrepareParams {
+	credential := t.Credential
+	if credential.AuthType() == "" {
+		credential = cloud.NewEmptyCredential()
+	}
+	return bootstrap.PrepareParams{
+		ControllerConfig: coretesting.FakeControllerConfig(),
+		ModelConfig:      t.TestConfig,
+		Cloud: environs.CloudSpec{
+			Type:       t.TestConfig["type"].(string),
+			Name:       t.TestConfig["type"].(string),
+			Region:     t.CloudRegion,
+			Endpoint:   t.CloudEndpoint,
+			Credential: &credential,
+		},
+		ControllerName: t.TestConfig["name"].(string),
+		AdminSecret:    AdminSecret,
+	}
+}
+
+func (t *LiveTests) bootstrapParams() bootstrap.BootstrapParams {
+	credential := t.Credential
+	if credential.AuthType() == "" {
+		credential = cloud.NewEmptyCredential()
+	}
+	var regions []cloud.Region
+	if t.CloudRegion != "" {
+		regions = []cloud.Region{{
+			Name:     t.CloudRegion,
+			Endpoint: t.CloudEndpoint,
+		}}
+	}
+	return bootstrap.BootstrapParams{
+		ControllerConfig: coretesting.FakeControllerConfig(),
+		Cloud: cloud.Cloud{
+			Name:      t.TestConfig["type"].(string),
+			Type:      t.TestConfig["type"].(string),
+			AuthTypes: []cloud.AuthType{credential.AuthType()},
+			Regions:   regions,
+			Endpoint:  t.CloudEndpoint,
+		},
+		CloudRegion:         t.CloudRegion,
+		CloudCredential:     &credential,
+		CloudCredentialName: "credential",
+		AdminSecret:         AdminSecret,
+		CAPrivateKey:        coretesting.CAKey,
+	}
 }
 
 func (t *LiveTests) BootstrapOnce(c *gc.C) {
@@ -141,12 +210,13 @@ func (t *LiveTests) BootstrapOnce(c *gc.C) {
 	// we could connect to (actual live tests, rather than local-only)
 	cons := constraints.MustParse("mem=2G")
 	if t.CanOpenState {
-		_, err := sync.Upload(t.toolsStorage, "released", nil, coretesting.FakeDefaultSeries)
+		_, err := sync.Upload(t.toolsStorage, "released", nil, series.LatestLts())
 		c.Assert(err, jc.ErrorIsNil)
 	}
-	err := bootstrap.EnsureNotBootstrapped(t.Env)
-	c.Assert(err, jc.ErrorIsNil)
-	err = bootstrap.Bootstrap(envtesting.BootstrapContext(c), t.Env, bootstrap.BootstrapParams{Constraints: cons})
+	args := t.bootstrapParams()
+	args.BootstrapConstraints = cons
+	args.ModelConstraints = cons
+	err := bootstrap.Bootstrap(envtesting.BootstrapContext(c), t.Env, args)
 	c.Assert(err, jc.ErrorIsNil)
 	t.bootstrapped = true
 }
@@ -155,10 +225,11 @@ func (t *LiveTests) Destroy(c *gc.C) {
 	if t.Env == nil {
 		return
 	}
-	err := environs.Destroy(t.Env, t.ConfigStore)
+	err := environs.Destroy(t.Env.Config().Name(), t.Env, t.ControllerStore)
 	c.Assert(err, jc.ErrorIsNil)
 	t.bootstrapped = false
 	t.prepared = false
+	t.ControllerUUID = ""
 	t.Env = nil
 }
 
@@ -179,7 +250,7 @@ func (t *LiveTests) TestPrechecker(c *gc.C) {
 func (t *LiveTests) TestStartStop(c *gc.C) {
 	t.BootstrapOnce(c)
 
-	inst, _ := jujutesting.AssertStartInstance(c, t.Env, "0")
+	inst, _ := jujutesting.AssertStartInstance(c, t.Env, t.ControllerUUID, "0")
 	c.Assert(inst, gc.NotNil)
 	id0 := inst.Id()
 
@@ -232,91 +303,172 @@ func (t *LiveTests) TestStartStop(c *gc.C) {
 func (t *LiveTests) TestPorts(c *gc.C) {
 	t.BootstrapOnce(c)
 
-	inst1, _ := jujutesting.AssertStartInstance(c, t.Env, "1")
+	inst1, _ := jujutesting.AssertStartInstance(c, t.Env, t.ControllerUUID, "1")
 	c.Assert(inst1, gc.NotNil)
 	defer t.Env.StopInstances(inst1.Id())
-	ports, err := inst1.Ports("1")
+	rules, err := inst1.IngressRules("1")
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ports, gc.HasLen, 0)
+	c.Assert(rules, gc.HasLen, 0)
 
-	inst2, _ := jujutesting.AssertStartInstance(c, t.Env, "2")
+	inst2, _ := jujutesting.AssertStartInstance(c, t.Env, t.ControllerUUID, "2")
 	c.Assert(inst2, gc.NotNil)
-	ports, err = inst2.Ports("2")
+	rules, err = inst2.IngressRules("2")
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ports, gc.HasLen, 0)
+	c.Assert(rules, gc.HasLen, 0)
 	defer t.Env.StopInstances(inst2.Id())
 
 	// Open some ports and check they're there.
-	err = inst1.OpenPorts("1", []network.PortRange{{67, 67, "udp"}, {45, 45, "tcp"}, {80, 100, "tcp"}})
-	c.Assert(err, jc.ErrorIsNil)
-	ports, err = inst1.Ports("1")
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ports, gc.DeepEquals, []network.PortRange{{45, 45, "tcp"}, {80, 100, "tcp"}, {67, 67, "udp"}})
-	ports, err = inst2.Ports("2")
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ports, gc.HasLen, 0)
+	err = inst1.OpenPorts(
+		"1", []network.IngressRule{
+			network.MustNewIngressRule("udp", 67, 67),
+			network.MustNewIngressRule("tcp", 45, 45),
+			network.MustNewIngressRule("tcp", 80, 100),
+		})
 
-	err = inst2.OpenPorts("2", []network.PortRange{{89, 89, "tcp"}, {45, 45, "tcp"}, {20, 30, "tcp"}})
+	c.Assert(err, jc.ErrorIsNil)
+	rules, err = inst1.IngressRules("1")
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(
+		rules, jc.DeepEquals,
+		[]network.IngressRule{
+			network.MustNewIngressRule("tcp", 45, 45, "0.0.0.0/0"),
+			network.MustNewIngressRule("tcp", 80, 100, "0.0.0.0/0"),
+			network.MustNewIngressRule("udp", 67, 67, "0.0.0.0/0"),
+		},
+	)
+	rules, err = inst2.IngressRules("2")
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(rules, gc.HasLen, 0)
+
+	err = inst2.OpenPorts(
+		"2", []network.IngressRule{
+			network.MustNewIngressRule("tcp", 89, 89),
+			network.MustNewIngressRule("tcp", 45, 45),
+			network.MustNewIngressRule("tcp", 20, 30),
+		})
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Check there's no crosstalk to another machine
-	ports, err = inst2.Ports("2")
+	rules, err = inst2.IngressRules("2")
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ports, gc.DeepEquals, []network.PortRange{{20, 30, "tcp"}, {45, 45, "tcp"}, {89, 89, "tcp"}})
-	ports, err = inst1.Ports("1")
+	c.Assert(
+		rules, jc.DeepEquals,
+		[]network.IngressRule{
+			network.MustNewIngressRule("tcp", 20, 30, "0.0.0.0/0"),
+			network.MustNewIngressRule("tcp", 45, 45, "0.0.0.0/0"),
+			network.MustNewIngressRule("tcp", 89, 89, "0.0.0.0/0"),
+		},
+	)
+	rules, err = inst1.IngressRules("1")
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ports, gc.DeepEquals, []network.PortRange{{45, 45, "tcp"}, {80, 100, "tcp"}, {67, 67, "udp"}})
+	c.Assert(
+		rules, jc.DeepEquals,
+		[]network.IngressRule{
+			network.MustNewIngressRule("tcp", 45, 45, "0.0.0.0/0"),
+			network.MustNewIngressRule("tcp", 80, 100, "0.0.0.0/0"),
+			network.MustNewIngressRule("udp", 67, 67, "0.0.0.0/0"),
+		},
+	)
 
 	// Check that opening the same port again is ok.
-	oldPorts, err := inst2.Ports("2")
+	oldRules, err := inst2.IngressRules("2")
 	c.Assert(err, jc.ErrorIsNil)
-	err = inst2.OpenPorts("2", []network.PortRange{{45, 45, "tcp"}})
+	err = inst2.OpenPorts(
+		"2", []network.IngressRule{
+			network.MustNewIngressRule("tcp", 45, 45),
+		})
 	c.Assert(err, jc.ErrorIsNil)
-	err = inst2.OpenPorts("2", []network.PortRange{{20, 30, "tcp"}})
+	err = inst2.OpenPorts(
+		"2", []network.IngressRule{
+			network.MustNewIngressRule("tcp", 20, 30),
+		})
 	c.Assert(err, jc.ErrorIsNil)
-	ports, err = inst2.Ports("2")
+	rules, err = inst2.IngressRules("2")
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ports, gc.DeepEquals, oldPorts)
+	c.Assert(rules, jc.DeepEquals, oldRules)
 
 	// Check that opening the same port again and another port is ok.
-	err = inst2.OpenPorts("2", []network.PortRange{{45, 45, "tcp"}, {99, 99, "tcp"}})
+	err = inst2.OpenPorts(
+		"2", []network.IngressRule{
+			network.MustNewIngressRule("tcp", 45, 45),
+			network.MustNewIngressRule("tcp", 99, 99),
+		})
 	c.Assert(err, jc.ErrorIsNil)
-	ports, err = inst2.Ports("2")
+	rules, err = inst2.IngressRules("2")
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ports, gc.DeepEquals, []network.PortRange{{20, 30, "tcp"}, {45, 45, "tcp"}, {89, 89, "tcp"}, {99, 99, "tcp"}})
-
-	err = inst2.ClosePorts("2", []network.PortRange{{45, 45, "tcp"}, {99, 99, "tcp"}, {20, 30, "tcp"}})
+	c.Assert(
+		rules, jc.DeepEquals,
+		[]network.IngressRule{
+			network.MustNewIngressRule("tcp", 20, 30, "0.0.0.0/0"),
+			network.MustNewIngressRule("tcp", 45, 45, "0.0.0.0/0"),
+			network.MustNewIngressRule("tcp", 89, 89, "0.0.0.0/0"),
+			network.MustNewIngressRule("tcp", 99, 99, "0.0.0.0/0"),
+		},
+	)
+	err = inst2.ClosePorts(
+		"2", []network.IngressRule{
+			network.MustNewIngressRule("tcp", 45, 45),
+			network.MustNewIngressRule("tcp", 99, 99),
+			network.MustNewIngressRule("tcp", 20, 30),
+		})
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Check that we can close ports and that there's no crosstalk.
-	ports, err = inst2.Ports("2")
+	rules, err = inst2.IngressRules("2")
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ports, gc.DeepEquals, []network.PortRange{{89, 89, "tcp"}})
-	ports, err = inst1.Ports("1")
+	c.Assert(
+		rules, jc.DeepEquals,
+		[]network.IngressRule{
+			network.MustNewIngressRule("tcp", 89, 89, "0.0.0.0/0"),
+		},
+	)
+	rules, err = inst1.IngressRules("1")
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ports, gc.DeepEquals, []network.PortRange{{45, 45, "tcp"}, {80, 100, "tcp"}, {67, 67, "udp"}})
+	c.Assert(
+		rules, jc.DeepEquals,
+		[]network.IngressRule{
+			network.MustNewIngressRule("tcp", 45, 45, "0.0.0.0/0"),
+			network.MustNewIngressRule("tcp", 80, 100, "0.0.0.0/0"),
+			network.MustNewIngressRule("udp", 67, 67, "0.0.0.0/0"),
+		},
+	)
 
 	// Check that we can close multiple ports.
-	err = inst1.ClosePorts("1", []network.PortRange{{45, 45, "tcp"}, {67, 67, "udp"}, {80, 100, "tcp"}})
+	err = inst1.ClosePorts(
+		"1", []network.IngressRule{
+			network.MustNewIngressRule("tcp", 45, 45),
+			network.MustNewIngressRule("udp", 67, 67),
+			network.MustNewIngressRule("tcp", 80, 100),
+		})
 	c.Assert(err, jc.ErrorIsNil)
-	ports, err = inst1.Ports("1")
-	c.Assert(ports, gc.HasLen, 0)
+	rules, err = inst1.IngressRules("1")
+	c.Assert(rules, gc.HasLen, 0)
 
 	// Check that we can close ports that aren't there.
-	err = inst2.ClosePorts("2", []network.PortRange{{111, 111, "tcp"}, {222, 222, "udp"}, {600, 700, "tcp"}})
+	err = inst2.ClosePorts(
+		"2", []network.IngressRule{
+			network.MustNewIngressRule("tcp", 111, 111),
+			network.MustNewIngressRule("udp", 222, 222),
+			network.MustNewIngressRule("tcp", 600, 700),
+		})
 	c.Assert(err, jc.ErrorIsNil)
-	ports, err = inst2.Ports("2")
-	c.Assert(ports, gc.DeepEquals, []network.PortRange{{89, 89, "tcp"}})
+	rules, err = inst2.IngressRules("2")
+	c.Assert(
+		rules, jc.DeepEquals,
+		[]network.IngressRule{
+			network.MustNewIngressRule("tcp", 89, 89, "0.0.0.0/0"),
+		},
+	)
 
 	// Check errors when acting on environment.
-	err = t.Env.OpenPorts([]network.PortRange{{80, 80, "tcp"}})
-	c.Assert(err, gc.ErrorMatches, `invalid firewall mode "instance" for opening ports on environment`)
+	err = t.Env.OpenPorts([]network.IngressRule{network.MustNewIngressRule("tcp", 80, 80)})
+	c.Assert(err, gc.ErrorMatches, `invalid firewall mode "instance" for opening ports on model`)
 
-	err = t.Env.ClosePorts([]network.PortRange{{80, 80, "tcp"}})
-	c.Assert(err, gc.ErrorMatches, `invalid firewall mode "instance" for closing ports on environment`)
+	err = t.Env.ClosePorts([]network.IngressRule{network.MustNewIngressRule("tcp", 80, 80)})
+	c.Assert(err, gc.ErrorMatches, `invalid firewall mode "instance" for closing ports on model`)
 
-	_, err = t.Env.Ports()
-	c.Assert(err, gc.ErrorMatches, `invalid firewall mode "instance" for retrieving ports from environment`)
+	_, err = t.Env.IngressRules()
+	c.Assert(err, gc.ErrorMatches, `invalid firewall mode "instance" for retrieving ingress rules from model`)
 }
 
 func (t *LiveTests) TestGlobalPorts(c *gc.C) {
@@ -337,50 +489,88 @@ func (t *LiveTests) TestGlobalPorts(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Create instances and check open ports on both instances.
-	inst1, _ := jujutesting.AssertStartInstance(c, t.Env, "1")
+	inst1, _ := jujutesting.AssertStartInstance(c, t.Env, t.ControllerUUID, "1")
 	defer t.Env.StopInstances(inst1.Id())
-	ports, err := t.Env.Ports()
+	rules, err := t.Env.IngressRules()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ports, gc.HasLen, 0)
+	c.Assert(rules, gc.HasLen, 0)
 
-	inst2, _ := jujutesting.AssertStartInstance(c, t.Env, "2")
-	ports, err = t.Env.Ports()
+	inst2, _ := jujutesting.AssertStartInstance(c, t.Env, t.ControllerUUID, "2")
+	rules, err = t.Env.IngressRules()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ports, gc.HasLen, 0)
+	c.Assert(rules, gc.HasLen, 0)
 	defer t.Env.StopInstances(inst2.Id())
 
-	err = t.Env.OpenPorts([]network.PortRange{{67, 67, "udp"}, {45, 45, "tcp"}, {89, 89, "tcp"}, {99, 99, "tcp"}, {100, 110, "tcp"}})
+	err = t.Env.OpenPorts([]network.IngressRule{
+		network.MustNewIngressRule("udp", 67, 67),
+		network.MustNewIngressRule("tcp", 45, 45),
+		network.MustNewIngressRule("tcp", 89, 89),
+		network.MustNewIngressRule("tcp", 99, 99),
+		network.MustNewIngressRule("tcp", 100, 110),
+	})
 	c.Assert(err, jc.ErrorIsNil)
 
-	ports, err = t.Env.Ports()
+	rules, err = t.Env.IngressRules()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ports, gc.DeepEquals, []network.PortRange{{45, 45, "tcp"}, {89, 89, "tcp"}, {99, 99, "tcp"}, {100, 110, "tcp"}, {67, 67, "udp"}})
+	c.Assert(
+		rules, jc.DeepEquals,
+		[]network.IngressRule{
+			network.MustNewIngressRule("tcp", 45, 45, "0.0.0.0/0"),
+			network.MustNewIngressRule("tcp", 89, 89, "0.0.0.0/0"),
+			network.MustNewIngressRule("tcp", 99, 99, "0.0.0.0/0"),
+			network.MustNewIngressRule("tcp", 100, 110, "0.0.0.0/0"),
+			network.MustNewIngressRule("udp", 67, 67, "0.0.0.0/0"),
+		},
+	)
 
 	// Check closing some ports.
-	err = t.Env.ClosePorts([]network.PortRange{{99, 99, "tcp"}, {67, 67, "udp"}})
+	err = t.Env.ClosePorts([]network.IngressRule{
+		network.MustNewIngressRule("tcp", 99, 99),
+		network.MustNewIngressRule("udp", 67, 67),
+	})
 	c.Assert(err, jc.ErrorIsNil)
 
-	ports, err = t.Env.Ports()
+	rules, err = t.Env.IngressRules()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ports, gc.DeepEquals, []network.PortRange{{45, 45, "tcp"}, {89, 89, "tcp"}, {100, 110, "tcp"}})
+	c.Assert(
+		rules, jc.DeepEquals,
+		[]network.IngressRule{
+			network.MustNewIngressRule("tcp", 45, 45, "0.0.0.0/0"),
+			network.MustNewIngressRule("tcp", 89, 89, "0.0.0.0/0"),
+			network.MustNewIngressRule("tcp", 100, 110, "0.0.0.0/0"),
+		},
+	)
 
 	// Check that we can close ports that aren't there.
-	err = t.Env.ClosePorts([]network.PortRange{{111, 111, "tcp"}, {222, 222, "udp"}, {2000, 2500, "tcp"}})
+	err = t.Env.ClosePorts([]network.IngressRule{
+		network.MustNewIngressRule("tcp", 111, 111),
+		network.MustNewIngressRule("udp", 222, 222),
+		network.MustNewIngressRule("tcp", 2000, 2500),
+	})
 	c.Assert(err, jc.ErrorIsNil)
 
-	ports, err = t.Env.Ports()
+	rules, err = t.Env.IngressRules()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ports, gc.DeepEquals, []network.PortRange{{45, 45, "tcp"}, {89, 89, "tcp"}, {100, 110, "tcp"}})
+	c.Assert(
+		rules, jc.DeepEquals,
+		[]network.IngressRule{
+			network.MustNewIngressRule("tcp", 45, 45, "0.0.0.0/0"),
+			network.MustNewIngressRule("tcp", 89, 89, "0.0.0.0/0"),
+			network.MustNewIngressRule("tcp", 100, 110, "0.0.0.0/0"),
+		},
+	)
 
 	// Check errors when acting on instances.
-	err = inst1.OpenPorts("1", []network.PortRange{{80, 80, "tcp"}})
+	err = inst1.OpenPorts(
+		"1", []network.IngressRule{network.MustNewIngressRule("tcp", 80, 80)})
 	c.Assert(err, gc.ErrorMatches, `invalid firewall mode "global" for opening ports on instance`)
 
-	err = inst1.ClosePorts("1", []network.PortRange{{80, 80, "tcp"}})
+	err = inst1.ClosePorts(
+		"1", []network.IngressRule{network.MustNewIngressRule("tcp", 80, 80)})
 	c.Assert(err, gc.ErrorMatches, `invalid firewall mode "global" for closing ports on instance`)
 
-	_, err = inst1.Ports("1")
-	c.Assert(err, gc.ErrorMatches, `invalid firewall mode "global" for retrieving ports from instance`)
+	_, err = inst1.IngressRules("1")
+	c.Assert(err, gc.ErrorMatches, `invalid firewall mode "global" for retrieving ingress rules from instance`)
 }
 
 func (t *LiveTests) TestBootstrapMultiple(c *gc.C) {
@@ -388,13 +578,10 @@ func (t *LiveTests) TestBootstrapMultiple(c *gc.C) {
 	// already up, this has been moved into the bootstrap command.
 	t.BootstrapOnce(c)
 
-	err := bootstrap.EnsureNotBootstrapped(t.Env)
-	c.Assert(err, gc.ErrorMatches, "environment is already bootstrapped")
-
 	c.Logf("destroy env")
 	env := t.Env
 	t.Destroy(c)
-	err = env.Destroy() // Again, should work fine and do nothing.
+	err := env.Destroy() // Again, should work fine and do nothing.
 	c.Assert(err, jc.ErrorIsNil)
 
 	// check that we can bootstrap after destroy
@@ -412,25 +599,32 @@ func (t *LiveTests) TestBootstrapAndDeploy(c *gc.C) {
 	c.Logf("opening state")
 	st := t.Env.(jujutesting.GetStater).GetStateInAPIServer()
 
-	env, err := st.Environment()
+	model, err := st.Model()
 	c.Assert(err, jc.ErrorIsNil)
-	owner := env.Owner()
+	owner := model.Owner()
 
 	c.Logf("opening API connection")
-	apiState, err := juju.NewAPIState(owner, t.Env, api.DefaultDialOpts())
+	controllerCfg, err := st.ControllerConfig()
+	c.Assert(err, jc.ErrorIsNil)
+	caCert, _ := controllerCfg.CACert()
+	apiInfo, err := environs.APIInfo(model.Tag().Id(), model.Tag().Id(), caCert, controllerCfg.APIPort(), t.Env)
+	c.Assert(err, jc.ErrorIsNil)
+	apiInfo.Tag = owner
+	apiInfo.Password = AdminSecret
+	apiState, err := api.Open(apiInfo, api.DefaultDialOpts())
 	c.Assert(err, jc.ErrorIsNil)
 	defer apiState.Close()
 
 	// Check that the agent version has made it through the
 	// bootstrap process (it's optional in the config.Config)
-	cfg, err := st.EnvironConfig()
+	cfg, err := st.ModelConfig()
 	c.Assert(err, jc.ErrorIsNil)
 	agentVersion, ok := cfg.AgentVersion()
 	c.Check(ok, jc.IsTrue)
-	c.Check(agentVersion, gc.Equals, version.Current.Number)
+	c.Check(agentVersion, gc.Equals, jujuversion.Current)
 
 	// Check that the constraints have been set in the environment.
-	cons, err := st.EnvironConstraints()
+	cons, err := st.ModelConstraints()
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(cons.String(), gc.Equals, "mem=2048M")
 
@@ -451,8 +645,11 @@ func (t *LiveTests) TestBootstrapAndDeploy(c *gc.C) {
 	defer mw0.Stop()
 
 	// If the series has not been specified, we expect the most recent Ubuntu LTS release to be used.
-	expectedVersion := version.Current
-	expectedVersion.Series = config.LatestLtsSeries()
+	expectedVersion := version.Binary{
+		Number: jujuversion.Current,
+		Arch:   arch.HostArch(),
+		Series: series.LatestLts(),
+	}
 
 	mtools0 := waitAgentTools(c, mw0, expectedVersion)
 
@@ -462,9 +659,9 @@ func (t *LiveTests) TestBootstrapAndDeploy(c *gc.C) {
 	url := testcharms.Repo.ClonedURL(repoDir, mtools0.Version.Series, "dummy")
 	sch, err := jujutesting.PutCharm(st, url, &charmrepo.LocalRepository{Path: repoDir}, false)
 	c.Assert(err, jc.ErrorIsNil)
-	svc, err := st.AddService("dummy", owner.String(), sch, nil, nil, nil)
+	svc, err := st.AddApplication(state.AddApplicationArgs{Name: "dummy", Charm: sch})
 	c.Assert(err, jc.ErrorIsNil)
-	units, err := juju.AddUnits(st, svc, 1, "")
+	units, err := juju.AddUnits(st, svc, "dummy", 1, nil)
 	c.Assert(err, jc.ErrorIsNil)
 	unit := units[0]
 
@@ -658,28 +855,12 @@ func (t *LiveTests) checkUpgrade(c *gc.C, st *state.State, newVersion version.Bi
 	}
 }
 
+// TODO(katco): 2016-08-09: lp:1611427
 var waitAgent = utils.AttemptStrategy{
 	Total: 30 * time.Second,
 	Delay: 1 * time.Second,
 }
 
-func (t *LiveTests) assertStartInstance(c *gc.C, m *state.Machine) {
-	// Wait for machine to get an instance id.
-	for a := waitAgent.Start(); a.Next(); {
-		err := m.Refresh()
-		c.Assert(err, jc.ErrorIsNil)
-		instId, err := m.InstanceId()
-		if err != nil {
-			c.Assert(err, jc.Satisfies, errors.IsNotProvisioned)
-			continue
-		}
-		_, err = t.Env.Instances([]instance.Id{instId})
-		c.Assert(err, jc.ErrorIsNil)
-		return
-	}
-	c.Fatalf("provisioner failed to start machine after %v", waitAgent.Total)
-}
-
 func (t *LiveTests) assertStopInstance(c *gc.C, env environs.Environ, instId instance.Id) {
 	var err error
 	for a := waitAgent.Start(); a.Next(); {
@@ -695,47 +876,35 @@ func (t *LiveTests) assertStopInstance(c *gc.C, env environs.Environ, instId ins
 	c.Fatalf("provisioner failed to stop machine after %v", waitAgent.Total)
 }
 
-// assertInstanceId asserts that the machine has an instance id
-// that matches that of the given instance. If the instance is nil,
-// It asserts that the instance id is unset.
-func assertInstanceId(c *gc.C, m *state.Machine, inst instance.Instance) {
-	var wantId, gotId instance.Id
-	var err error
-	if inst != nil {
-		wantId = inst.Id()
-	}
-	for a := waitAgent.Start(); a.Next(); {
-		err := m.Refresh()
-		c.Assert(err, jc.ErrorIsNil)
-		gotId, err = m.InstanceId()
-		if err != nil {
-			c.Assert(err, jc.Satisfies, errors.IsNotProvisioned)
-			if inst == nil {
-				return
-			}
-			continue
-		}
-		break
-	}
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(gotId, gc.Equals, wantId)
-}
-
 // Check that we get a consistent error when asking for an instance without
 // a valid machine config.
 func (t *LiveTests) TestStartInstanceWithEmptyNonceFails(c *gc.C) {
 	machineId := "4"
-	stateInfo := jujutesting.FakeStateInfo(machineId)
 	apiInfo := jujutesting.FakeAPIInfo(machineId)
-	instanceConfig, err := instancecfg.NewInstanceConfig(machineId, "", "released", "quantal", true, nil, stateInfo, apiInfo)
+	instanceConfig, err := instancecfg.NewInstanceConfig(coretesting.ControllerTag, machineId, "", "released", "quantal", apiInfo)
 	c.Assert(err, jc.ErrorIsNil)
 
 	t.PrepareOnce(c)
-	possibleTools := envtesting.AssertUploadFakeToolsVersions(c, t.toolsStorage, "released", "released", version.MustParseBinary("5.4.5-trusty-amd64"))
-	result, err := t.Env.StartInstance(environs.StartInstanceParams{
+	possibleTools := coretools.List(envtesting.AssertUploadFakeToolsVersions(
+		c, t.toolsStorage, "released", "released", version.MustParseBinary("5.4.5-trusty-amd64"),
+	))
+	fakeCallback := func(_ status.Status, _ string, _ map[string]interface{}) error {
+		return nil
+	}
+	params := environs.StartInstanceParams{
+		ControllerUUID: coretesting.ControllerTag.Id(),
 		Tools:          possibleTools,
 		InstanceConfig: instanceConfig,
-	})
+		StatusCallback: fakeCallback,
+	}
+	err = jujutesting.SetImageMetadata(
+		t.Env,
+		possibleTools.AllSeries(),
+		possibleTools.Arches(),
+		&params.ImageMetadata,
+	)
+	c.Check(err, jc.ErrorIsNil)
+	result, err := t.Env.StartInstance(params)
 	if result != nil && result.Instance != nil {
 		err := t.Env.StopInstances(result.Instance.Id())
 		c.Check(err, jc.ErrorIsNil)
@@ -749,31 +918,44 @@ func (t *LiveTests) TestBootstrapWithDefaultSeries(c *gc.C) {
 		c.Skip("HasProvisioner is false; cannot test deployment")
 	}
 
-	current := version.Current
+	current := version.Binary{
+		Number: jujuversion.Current,
+		Arch:   arch.HostArch(),
+		Series: series.MustHostSeries(),
+	}
 	other := current
 	other.Series = "quantal"
 	if current == other {
 		other.Series = "precise"
 	}
 
-	dummyCfg, err := config.New(config.NoDefaults, dummy.SampleConfig().Merge(coretesting.Attrs{
-		"state-server": false,
-		"name":         "dummy storage",
-	}))
-	dummyenv, err := environs.Prepare(dummyCfg, envtesting.BootstrapContext(c), configstore.NewMem())
+	dummyCfg := dummy.SampleConfig().Merge(coretesting.Attrs{
+		"controller": false,
+		"name":       "dummy storage",
+	})
+	args := t.prepareForBootstrapParams(c)
+	args.ModelConfig = dummyCfg
+	dummyenv, err := bootstrap.Prepare(envtesting.BootstrapContext(c),
+		jujuclient.NewMemStore(),
+		args,
+	)
 	c.Assert(err, jc.ErrorIsNil)
 	defer dummyenv.Destroy()
 
 	t.Destroy(c)
 
-	attrs := t.TestConfig.Merge(coretesting.Attrs{"default-series": other.Series})
-	cfg, err := config.New(config.NoDefaults, attrs)
+	attrs := t.TestConfig.Merge(coretesting.Attrs{
+		"name":           "livetests",
+		"default-series": other.Series,
+	})
+	args.ModelConfig = attrs
+	env, err := bootstrap.Prepare(envtesting.BootstrapContext(c),
+		t.ControllerStore,
+		args)
 	c.Assert(err, jc.ErrorIsNil)
-	env, err := environs.Prepare(cfg, envtesting.BootstrapContext(c), t.ConfigStore)
-	c.Assert(err, jc.ErrorIsNil)
-	defer environs.Destroy(env, t.ConfigStore)
+	defer environs.Destroy("livetests", env, t.ControllerStore)
 
-	err = bootstrap.Bootstrap(envtesting.BootstrapContext(c), env, bootstrap.BootstrapParams{})
+	err = bootstrap.Bootstrap(envtesting.BootstrapContext(c), env, t.bootstrapParams())
 	c.Assert(err, jc.ErrorIsNil)
 
 	st := t.Env.(jujutesting.GetStater).GetStateInAPIServer()
