@@ -28,17 +28,24 @@ const LegacyUpstart = "legacy-upstart"
 // (space list|create, subnet list|add).
 const PostNetCLIMVP = "post-net-cli-mvp"
 
-// VSphereProvider enables the generic vmware provider.
-const VSphereProvider = "vsphere-provider"
-
 // ImageMetadata allows custom image metadata to be recorded in state.
 const ImageMetadata = "image-metadata"
 
-// Migration enables the 'juju migrate' command.
-const Migration = "migration"
-
 // DeveloperMode allows access to developer specific commands and behaviour.
 const DeveloperMode = "developer-mode"
 
 // CrossModelRelations allows cross model relations functionality.
 const CrossModelRelations = "cross-model"
+
+// LXDStorage enables the LXD storage provider.
+const LXDStorage = "lxd-storage"
+
+// PersistentStorage enables the persistent storage feature.
+const PersistentStorage = "persistent-storage"
+
+// StrictMigration will cause migration to error if there are unexported
+// values for annotations, status, status history, or settings.
+const StrictMigration = "strict-migration"
+
+// CAAS enables creating models on CAAS infrastructure (k8s, etc)
+const CAAS = "caas"
