@@ -21,10 +21,29 @@ import (
 	"github.com/juju/juju/mongo"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/state/multiwatcher"
+	"github.com/juju/juju/status"
 	"github.com/juju/juju/testing"
 	coretools "github.com/juju/juju/tools"
 )
 
+func fakeCallback(_ status.Status, _ string, _ map[string]interface{}) error {
+	return nil
+}
+
+// FakeStateInfo holds information about no state - it will always
+// give an error when connected to.  The machine id gives the machine id
+// of the machine to be started.
+func FakeStateInfo(machineId string) *mongo.MongoInfo {
+	return &mongo.MongoInfo{
+		Info: mongo.Info{
+			Addrs:  []string{"0.1.2.3:1234"},
+			CACert: testing.CACert,
+		},
+		Tag:      names.NewMachineTag(machineId),
+		Password: "unimportant",
+	}
+}
+
 // FakeAPIInfo holds information about no state - it will always
 // give an error when connected to.  The machine id gives the machine id
 // of the machine to be started.
@@ -115,7 +134,7 @@ func StartInstanceWithConstraints(
 ) (
 	instance.Instance, *instance.HardwareCharacteristics, []network.InterfaceInfo, error,
 ) {
-	params := environs.StartInstanceParams{ControllerUUID: controllerUUID, Constraints: cons}
+	params := environs.StartInstanceParams{ControllerUUID: controllerUUID, Constraints: cons, StatusCallback: fakeCallback}
 	result, err := StartInstanceWithParams(env, machineId, params)
 	if err != nil {
 		return nil, nil, nil, errors.Trace(err)
@@ -190,7 +209,7 @@ func fillinStartInstanceParams(env environs.Environ, machineId string, isControl
 			Config: testing.FakeControllerConfig(),
 			MongoInfo: &mongo.MongoInfo{
 				Info: mongo.Info{
-					Addrs:  []string{"127.0.0.1:1234"},
+					Addrs:  []string{"localhost:1234"},
 					CACert: "CA CERT\n" + testing.CACert,
 				},
 				Password: "mongosecret",
@@ -203,6 +222,9 @@ func fillinStartInstanceParams(env environs.Environ, machineId string, isControl
 	instanceConfig.Tags = instancecfg.InstanceTags(env.Config().UUID(), params.ControllerUUID, cfg, nil)
 	params.Tools = possibleTools
 	params.InstanceConfig = instanceConfig
+	if params.StatusCallback == nil {
+		params.StatusCallback = fakeCallback
+	}
 	return nil
 }
 
