@@ -11,8 +11,11 @@ import (
 	"gopkg.in/juju/charm.v6-unstable"
 	"gopkg.in/juju/names.v2"
 
+	"github.com/juju/juju/cloud"
 	coremigration "github.com/juju/juju/core/migration"
 	"github.com/juju/juju/migration"
+	"github.com/juju/juju/resource"
+	"github.com/juju/juju/resource/resourcetesting"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/status"
 	"github.com/juju/juju/testing"
@@ -68,6 +71,21 @@ func (*SourcePrecheckSuite) TestCharmUpgrades(c *gc.C) {
 	c.Assert(err, gc.ErrorMatches, "unit spanner/1 is upgrading")
 }
 
+func (*SourcePrecheckSuite) TestPendingResources(c *gc.C) {
+	backend := newHappyBackend()
+	backend.pendingResources = []resource.Resource{
+		resourcetesting.NewResource(c, nil, "blob", "foo", "body").Resource,
+	}
+	err := migration.SourcePrecheck(backend)
+	// Pending resources shouldn't prevent a migration. If they exist
+	// alongside an application, they're remains of a previous failed
+	// deploy that haven't been cleaned up (see lp:1705730). If they
+	// exist without an application that indicates an impending
+	// application deployment - the migration exporter won't migrate
+	// pending resources.
+	c.Assert(err, jc.ErrorIsNil)
+}
+
 func (*SourcePrecheckSuite) TestImportingModel(c *gc.C) {
 	backend := newFakeBackend()
 	backend.model.migrationMode = state.MigrationModeImporting
@@ -199,6 +217,21 @@ func (s *SourcePrecheckSuite) TestDeadUnit(c *gc.C) {
 	c.Assert(err.Error(), gc.Equals, "unit foo/0 is dead")
 }
 
+func (s *SourcePrecheckSuite) TestUnitExecuting(c *gc.C) {
+	backend := &fakeBackend{
+		apps: []migration.PrecheckApplication{
+			&fakeApp{
+				name: "foo",
+				units: []migration.PrecheckUnit{
+					&fakeUnit{name: "foo/0", agentStatus: status.Executing},
+				},
+			},
+		},
+	}
+	err := migration.SourcePrecheck(backend)
+	c.Assert(err, jc.ErrorIsNil)
+}
+
 func (s *SourcePrecheckSuite) TestUnitNotIdle(c *gc.C) {
 	backend := &fakeBackend{
 		apps: []migration.PrecheckApplication{
@@ -211,7 +244,7 @@ func (s *SourcePrecheckSuite) TestUnitNotIdle(c *gc.C) {
 		},
 	}
 	err := migration.SourcePrecheck(backend)
-	c.Assert(err.Error(), gc.Equals, "unit foo/0 not idle (failed)")
+	c.Assert(err.Error(), gc.Equals, "unit foo/0 not idle or executing (failed)")
 }
 
 func (s *SourcePrecheckSuite) TestUnitLost(c *gc.C) {
@@ -226,7 +259,7 @@ func (s *SourcePrecheckSuite) TestUnitLost(c *gc.C) {
 		},
 	}
 	err := migration.SourcePrecheck(backend)
-	c.Assert(err.Error(), gc.Equals, "unit foo/0 not idle (lost)")
+	c.Assert(err.Error(), gc.Equals, "unit foo/0 not idle or executing (lost)")
 }
 
 func (*SourcePrecheckSuite) TestDyingControllerModel(c *gc.C) {
@@ -307,14 +340,75 @@ func (s *TargetPrecheckSuite) TestSuccess(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 }
 
-func (s *TargetPrecheckSuite) TestVersionLessThanSource(c *gc.C) {
+func (s *TargetPrecheckSuite) TestModelVersionAheadOfTarget(c *gc.C) {
 	backend := newFakeBackend()
-	s.modelInfo.AgentVersion = version.MustParse("1.2.4")
+
+	sourceVersion := backendVersion
+	sourceVersion.Patch++
+	s.modelInfo.AgentVersion = sourceVersion
+
 	err := migration.TargetPrecheck(backend, s.modelInfo)
 	c.Assert(err.Error(), gc.Equals,
 		`model has higher version than target controller (1.2.4 > 1.2.3)`)
 }
 
+func (s *TargetPrecheckSuite) TestSourceControllerMajorAhead(c *gc.C) {
+	backend := newFakeBackend()
+
+	sourceVersion := backendVersion
+	sourceVersion.Major++
+	sourceVersion.Minor = 0
+	sourceVersion.Patch = 0
+	s.modelInfo.ControllerAgentVersion = sourceVersion
+
+	err := migration.TargetPrecheck(backend, s.modelInfo)
+	c.Assert(err.Error(), gc.Equals,
+		`source controller has higher version than target controller (2.0.0 > 1.2.3)`)
+}
+
+func (s *TargetPrecheckSuite) TestSourceControllerMinorAhead(c *gc.C) {
+	backend := newFakeBackend()
+
+	sourceVersion := backendVersion
+	sourceVersion.Minor++
+	sourceVersion.Patch = 0
+	s.modelInfo.ControllerAgentVersion = sourceVersion
+
+	err := migration.TargetPrecheck(backend, s.modelInfo)
+	c.Assert(err.Error(), gc.Equals,
+		`source controller has higher version than target controller (1.3.0 > 1.2.3)`)
+}
+
+func (s *TargetPrecheckSuite) TestSourceControllerPatchAhead(c *gc.C) {
+	backend := newFakeBackend()
+
+	sourceVersion := backendVersion
+	sourceVersion.Patch++
+	s.modelInfo.ControllerAgentVersion = sourceVersion
+
+	c.Assert(migration.TargetPrecheck(backend, s.modelInfo), jc.ErrorIsNil)
+}
+
+func (s *TargetPrecheckSuite) TestSourceControllerBuildAhead(c *gc.C) {
+	backend := newFakeBackend()
+
+	sourceVersion := backendVersion
+	sourceVersion.Build++
+	s.modelInfo.ControllerAgentVersion = sourceVersion
+
+	c.Assert(migration.TargetPrecheck(backend, s.modelInfo), jc.ErrorIsNil)
+}
+
+func (s *TargetPrecheckSuite) TestSourceControllerTagMismatch(c *gc.C) {
+	backend := newFakeBackend()
+
+	sourceVersion := backendVersion
+	sourceVersion.Tag = "alpha"
+	s.modelInfo.ControllerAgentVersion = sourceVersion
+
+	c.Assert(migration.TargetPrecheck(backend, s.modelInfo), jc.ErrorIsNil)
+}
+
 func (s *TargetPrecheckSuite) TestDying(c *gc.C) {
 	backend := newFakeBackend()
 	backend.model.life = state.Dying
@@ -554,9 +648,19 @@ type fakeBackend struct {
 	apps       []migration.PrecheckApplication
 	allAppsErr error
 
+	credentials    cloud.Credential
+	credentialsErr error
+
+	pendingResources    []resource.Resource
+	pendingResourcesErr error
+
 	controllerBackend *fakeBackend
 }
 
+func (b *fakeBackend) Close() error {
+	return nil
+}
+
 func (b *fakeBackend) Model() (migration.PrecheckModel, error) {
 	return &b.model, nil
 }
@@ -581,6 +685,10 @@ func (b *fakeBackend) IsMigrationActive(string) (bool, error) {
 	return b.migrationActive, b.migrationActiveErr
 }
 
+func (b *fakeBackend) CloudCredential(tag names.CloudCredentialTag) (cloud.Credential, error) {
+	return b.credentials, b.credentialsErr
+}
+
 func (b *fakeBackend) AllMachines() ([]migration.PrecheckMachine, error) {
 	return b.machines, b.allMachinesErr
 }
@@ -590,7 +698,11 @@ func (b *fakeBackend) AllApplications() ([]migration.PrecheckApplication, error)
 
 }
 
-func (b *fakeBackend) ControllerBackend() (migration.PrecheckBackend, error) {
+func (b *fakeBackend) ListPendingResources(app string) ([]resource.Resource, error) {
+	return b.pendingResources, b.pendingResourcesErr
+}
+
+func (b *fakeBackend) ControllerBackend() (migration.PrecheckBackendCloser, error) {
 	if b.controllerBackend == nil {
 		return b, nil
 	}
@@ -603,6 +715,7 @@ type fakeModel struct {
 	owner         names.UserTag
 	life          state.Life
 	migrationMode state.MigrationMode
+	credential    string
 }
 
 func (m *fakeModel) UUID() string {
@@ -625,6 +738,13 @@ func (m *fakeModel) MigrationMode() state.MigrationMode {
 	return m.migrationMode
 }
 
+func (m *fakeModel) CloudCredential() (names.CloudCredentialTag, bool) {
+	if names.IsValidCloudCredential(m.credential) {
+		return names.NewCloudCredentialTag(m.credential), true
+	}
+	return names.CloudCredentialTag{}, false
+}
+
 type fakeMachine struct {
 	id             string
 	version        version.Binary
