@@ -280,8 +280,14 @@ func SelectPeerAddress(addrs []network.Address) string {
 // SelectPeerHostPort returns the HostPort to use as the mongo replica set peer
 // by selecting it from the given hostPorts.
 func SelectPeerHostPort(hostPorts []network.HostPort) string {
+	// TODO(macgreagoir) IPv6. We were always choosing a cloud-local or
+	// falling back to machine-local here (with true arg) but this doesn't
+	// make sense in IPv6, where the IPv6 [public] address is ignored in
+	// favour of ip6-localhost. Only pass true if we find an IPv4 address
+	// in the slice.
 	logger.Debugf("selecting mongo peer hostPort by scope from %+v", hostPorts)
-	return network.SelectMongoHostPortsByScope(hostPorts, true)[0]
+	allowMachineLocal := network.HostPortsHasIPv4Address(hostPorts)
+	return network.SelectMongoHostPortsByScope(hostPorts, allowMachineLocal)[0]
 }
 
 // SelectPeerHostPortBySpace returns the HostPort to use as the mongo replica set peer
@@ -345,6 +351,58 @@ func mongoPath(version Version, stat func(string) (os.FileInfo, error), lookPath
 
 }
 
+/*
+Values set as per bug:
+https://bugs.launchpad.net/juju/+bug/1656430
+net.ipv4.tcp_max_syn_backlog = 4096
+net.core.somaxconn = 16384
+net.core.netdev_max_backlog = 1000
+net.ipv4.tcp_fin_timeout = 30
+
+Values set as per mongod recommendation (see syslog on default mongod run)
+/sys/kernel/mm/transparent_hugepage/enabled 'always' > 'never'
+/sys/kernel/mm/transparent_hugepage/defrag 'always' > 'never'
+*/
+var mongoKernelTweaks = map[string]string{
+	"/sys/kernel/mm/transparent_hugepage/enabled": "never",
+	"/sys/kernel/mm/transparent_hugepage/defrag":  "never",
+	"/proc/sys/net/ipv4/tcp_max_syn_backlog":      "4096",
+	"/proc/sys/net/core/somaxconn":                "16384",
+	"/proc/sys/net/core/netdev_max_backlog":       "1000",
+	"/proc/sys/net/ipv4/tcp_fin_timeout":          "30",
+}
+
+// NewMemoryProfile returns a Memory Profile from the passed value.
+func NewMemoryProfile(m string) (MemoryProfile, error) {
+	mp := MemoryProfile(m)
+	if err := mp.Validate(); err != nil {
+		return MemoryProfile(""), err
+	}
+	return mp, nil
+}
+
+// MemoryProfile represents a type of meory configuration for Mongo.
+type MemoryProfile string
+
+// String returns a string representation of this profile value.
+func (m MemoryProfile) String() string {
+	return string(m)
+}
+
+func (m MemoryProfile) Validate() error {
+	if m != MemoryProfileLow && m != MemoryProfileDefault {
+		return errors.NotValidf("memory profile %q", m)
+	}
+	return nil
+}
+
+const (
+	// MemoryProfileLow will use as little memory as possible in mongo.
+	MemoryProfileLow MemoryProfile = "low"
+	// MemoryProfileDefault will use mongo config ootb.
+	MemoryProfileDefault = "default"
+)
+
 // EnsureServerParams is a parameter struct for EnsureServer.
 type EnsureServerParams struct {
 	// APIPort is the port to connect to the api server.
@@ -384,6 +442,10 @@ type EnsureServerParams struct {
 	// SetNUMAControlPolicy preference - whether the user
 	// wants to set the numa control policy when starting mongo.
 	SetNUMAControlPolicy bool
+
+	// MemoryProfile determines which value is going to be used by
+	// the cache and future memory tweaks.
+	MemoryProfile MemoryProfile
 }
 
 // EnsureServer ensures that the MongoDB server is installed,
@@ -392,6 +454,11 @@ type EnsureServerParams struct {
 // This method will remove old versions of the mongo init service as necessary
 // before installing the new version.
 func EnsureServer(args EnsureServerParams) error {
+	return ensureServer(args, mongoKernelTweaks)
+}
+
+func ensureServer(args EnsureServerParams, mongoKernelTweaks map[string]string) error {
+	tweakSysctlForMongo(mongoKernelTweaks)
 	logger.Infof(
 		"Ensuring mongo server is running; data directory %s; port %d",
 		args.DataDir, args.StatePort,
@@ -410,8 +477,7 @@ func EnsureServer(args EnsureServerParams) error {
 		}
 	}
 
-	operatingsystem := series.HostSeries()
-	if err := installMongod(operatingsystem, args.SetNUMAControlPolicy); err != nil {
+	if err := installMongod(series.MustHostSeries(), args.SetNUMAControlPolicy); err != nil {
 		// This isn't treated as fatal because the Juju MongoDB
 		// package is likely to be already installed anyway. There
 		// could just be a temporary issue with apt-get/yum/whatever
@@ -450,15 +516,16 @@ func EnsureServer(args EnsureServerParams) error {
 	}
 
 	svcConf := newConf(ConfigArgs{
-		DataDir:     args.DataDir,
-		DBDir:       dbDir,
-		MongoPath:   mongoPath,
-		Port:        args.StatePort,
-		OplogSizeMB: oplogSizeMB,
-		WantNUMACtl: args.SetNUMAControlPolicy,
-		Version:     mgoVersion,
-		Auth:        true,
-		IPv6:        network.SupportsIPv6(),
+		DataDir:       args.DataDir,
+		DBDir:         dbDir,
+		MongoPath:     mongoPath,
+		Port:          args.StatePort,
+		OplogSizeMB:   oplogSizeMB,
+		WantNUMACtl:   args.SetNUMAControlPolicy,
+		Version:       mgoVersion,
+		Auth:          true,
+		IPv6:          network.SupportsIPv6(),
+		MemoryProfile: args.MemoryProfile,
 	})
 	svc, err := newService(ServiceName, svcConf)
 	if err != nil {
@@ -495,12 +562,35 @@ func EnsureServer(args EnsureServerParams) error {
 	if err := preallocOplog(dbDir, oplogSizeMB); err != nil {
 		return fmt.Errorf("error creating oplog files: %v", err)
 	}
+
 	if err := service.InstallAndStart(svc); err != nil {
 		return errors.Trace(err)
 	}
 	return nil
 }
 
+func truncateAndWriteIfExists(procFile, value string) error {
+	if _, err := os.Stat(procFile); os.IsNotExist(err) {
+		logger.Debugf("%q does not exist, will not set %q", procFile, value)
+		return errors.Errorf("%q does not exist, will not set %q", procFile, value)
+	}
+	f, err := os.OpenFile(procFile, os.O_WRONLY|os.O_TRUNC, 0600)
+	if err != nil {
+		return errors.Trace(err)
+	}
+	defer f.Close()
+	_, err = f.WriteString(value)
+	return errors.Trace(err)
+}
+
+func tweakSysctlForMongo(editables map[string]string) {
+	for editableFile, value := range editables {
+		if err := truncateAndWriteIfExists(editableFile, value); err != nil {
+			logger.Errorf("could not set the value of %q to %q because of: %v\n", editableFile, value, err)
+		}
+	}
+}
+
 // UpdateSSLKey writes a new SSL key used by mongo to validate connections from Juju controller(s)
 func UpdateSSLKey(dataDir, cert, privateKey string) error {
 	certKey := cert + "\n" + privateKey
