@@ -146,7 +146,7 @@ func (s *MongoSuite) SetUpTest(c *gc.C) {
 		}
 		return 0, fmt.Errorf("not a directory")
 	})
-	s.PatchValue(mongo.MinOplogSizeMB, 1)
+	s.PatchValue(mongo.SmallOplogSizeMB, 1)
 
 	testPath := c.MkDir()
 	s.mongodConfigPath = filepath.Join(testPath, "mongodConfig")
@@ -157,7 +157,7 @@ func (s *MongoSuite) SetUpTest(c *gc.C) {
 }
 
 func (s *MongoSuite) patchSeries(ser string) {
-	s.PatchValue(&series.HostSeries, func() string { return ser })
+	s.PatchValue(&series.MustHostSeries, func() string { return ser })
 }
 
 func (s *MongoSuite) TestJujuMongodPath(c *gc.C) {
@@ -259,6 +259,34 @@ func (s *MongoSuite) TestEnsureServerServerExistsAndRunning(c *gc.C) {
 	s.data.CheckCallNames(c, "Installed", "Exists", "Running")
 }
 
+func (s *MongoSuite) TestEnsureServerSetsSysctlValues(c *gc.C) {
+	dataDir := c.MkDir()
+	dataFilePath := filepath.Join(dataDir, "mongoKernelTweaks")
+	dataFile, err := os.Create(dataFilePath)
+	c.Assert(err, jc.ErrorIsNil)
+	_, err = dataFile.WriteString("original value")
+	c.Assert(err, jc.ErrorIsNil)
+	dataFile.Close()
+
+	pm, err := coretesting.GetPackageManager()
+	c.Assert(err, jc.ErrorIsNil)
+	testing.PatchExecutableAsEchoArgs(c, s, pm.PackageManager)
+
+	s.data.SetStatus(mongo.ServiceName, "running")
+
+	contents, err := ioutil.ReadFile(dataFilePath)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(string(contents), gc.Equals, "original value")
+
+	err = mongo.SysctlEditableEnsureServer(makeEnsureServerParams(dataDir),
+		map[string]string{dataFilePath: "new value"})
+	c.Assert(err, jc.ErrorIsNil)
+
+	contents, err = ioutil.ReadFile(dataFilePath)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(string(contents), gc.Equals, "new value")
+}
+
 func (s *MongoSuite) TestEnsureServerServerExistsNotRunningIsStarted(c *gc.C) {
 	dataDir := c.MkDir()
 
