@@ -7,10 +7,10 @@ import (
 	"net"
 	"sort"
 	"strconv"
-	"time"
 
 	"github.com/juju/errors"
 	"github.com/juju/utils/set"
+	"strings"
 )
 
 // HostPort associates an address with a port.
@@ -35,6 +35,20 @@ func (hp HostPort) GoString() string {
 	return hp.String()
 }
 
+// Less reports whether hp1 is ordered before hp2
+// according to the criteria used by SortHostPorts.
+func (hp1 HostPort) Less(hp2 HostPort) bool {
+	order1 := hp1.sortOrder()
+	order2 := hp2.sortOrder()
+	if order1 == order2 {
+		if hp1.Address.Value == hp2.Address.Value {
+			return hp1.Port < hp2.Port
+		}
+		return hp1.Address.Value < hp2.Address.Value
+	}
+	return order1 < order2
+}
+
 // AddressesWithPort returns the given addresses all
 // associated with the given port.
 func AddressesWithPort(addrs []Address, port int) []HostPort {
@@ -108,17 +122,7 @@ type hostPortsPreferringIPv4Slice []HostPort
 func (hp hostPortsPreferringIPv4Slice) Len() int      { return len(hp) }
 func (hp hostPortsPreferringIPv4Slice) Swap(i, j int) { hp[i], hp[j] = hp[j], hp[i] }
 func (hp hostPortsPreferringIPv4Slice) Less(i, j int) bool {
-	hp1 := hp[i]
-	hp2 := hp[j]
-	order1 := hp1.sortOrder()
-	order2 := hp2.sortOrder()
-	if order1 == order2 {
-		if hp1.Address.Value == hp2.Address.Value {
-			return hp1.Port < hp2.Port
-		}
-		return hp1.Address.Value < hp2.Address.Value
-	}
-	return order1 < order2
+	return hp[i].Less(hp[j])
 }
 
 // SortHostPorts sorts the given HostPort slice according to the sortOrder of
@@ -195,6 +199,22 @@ func HostPortsToStrings(hps []HostPort) []string {
 	return result
 }
 
+// APIHostPortsToNoProxyString converts list of lists of NetAddrs() to
+// a NoProxy-like comma separated string, ignoring local addresses
+func APIHostPortsToNoProxyString(ahp [][]HostPort) string {
+	noProxySet := set.NewStrings()
+	for _, host := range ahp {
+		for _, hp := range host {
+			if hp.Address.Scope == ScopeMachineLocal ||
+				hp.Address.Scope == ScopeLinkLocal {
+				continue
+			}
+			noProxySet.Add(hp.Address.Value)
+		}
+	}
+	return strings.Join(noProxySet.SortedValues(), ",")
+}
+
 // CollapseHostPorts returns a flattened list of HostPorts keeping the
 // same order they appear in serversHostPorts.
 func CollapseHostPorts(serversHostPorts [][]HostPort) []HostPort {
@@ -241,40 +261,3 @@ func UniqueHostPorts(hostPorts []HostPort) []HostPort {
 
 	return results
 }
-
-// Dialer defines a Dial() method matching the signature of net.Dial().
-type Dialer interface {
-	Dial(network, address string) (net.Conn, error)
-}
-
-// ReachableHostPort dials the entries in the given hostPorts, in parallel,
-// using the given dialer, closing successfully established connections
-// immediately. Individual connection errors are discarded, and an error is
-// returned only if none of the hostPorts can be reached when the given timeout
-// expires.
-//
-// Usually, a net.Dialer initialized with a non-empty Timeout field is passed
-// for dialer.
-func ReachableHostPort(hostPorts []HostPort, dialer Dialer, timeout time.Duration) (HostPort, error) {
-	uniqueHPs := UniqueHostPorts(hostPorts)
-	successful := make(chan HostPort, 1)
-
-	for _, hostPort := range uniqueHPs {
-		go func(hp HostPort) {
-			conn, err := dialer.Dial("tcp", hp.NetAddr())
-			if err == nil {
-				conn.Close()
-				successful <- hp
-			}
-		}(hostPort)
-	}
-
-	select {
-	case result := <-successful:
-		logger.Infof("dialed %q successfully", result)
-		return result, nil
-
-	case <-time.After(timeout):
-		return HostPort{}, errors.Errorf("cannot connect to any address: %v", hostPorts)
-	}
-}
