@@ -33,6 +33,11 @@ func (s *InterfaceInfoSuite) SetUpTest(c *gc.C) {
 		{DNSServers: network.NewAddresses("1.1.1.1", "2.2.2.2")},
 		{GatewayAddress: network.NewAddress("4.3.2.1")},
 		{AvailabilityZones: []string{"foo", "bar"}},
+		{Routes: []network.Route{{
+			DestinationCIDR: "0.1.2.3/24",
+			GatewayIP:       "0.1.2.1",
+			Metric:          0,
+		}}},
 	}
 }
 
@@ -61,6 +66,11 @@ func (s *InterfaceInfoSuite) TestAdditionalFields(c *gc.C) {
 	c.Check(s.info[5].DNSServers, jc.DeepEquals, network.NewAddresses("1.1.1.1", "2.2.2.2"))
 	c.Check(s.info[6].GatewayAddress, jc.DeepEquals, network.NewAddress("4.3.2.1"))
 	c.Check(s.info[7].AvailabilityZones, jc.DeepEquals, []string{"foo", "bar"})
+	c.Check(s.info[8].Routes, jc.DeepEquals, []network.Route{{
+		DestinationCIDR: "0.1.2.3/24",
+		GatewayIP:       "0.1.2.1",
+		Metric:          0,
+	}})
 }
 
 func (s *InterfaceInfoSuite) TestSortInterfaceInfo(c *gc.C) {
@@ -78,6 +88,88 @@ func (s *InterfaceInfoSuite) TestSortInterfaceInfo(c *gc.C) {
 	c.Assert(info, jc.DeepEquals, expectedInfo)
 }
 
+type RouteSuite struct{}
+
+var _ = gc.Suite(&RouteSuite{})
+
+func checkRouteIsValid(c *gc.C, r network.Route) {
+	c.Check(r.Validate(), jc.ErrorIsNil)
+}
+
+func checkRouteErrEquals(c *gc.C, r network.Route, errString string) {
+	err := r.Validate()
+	c.Assert(err, gc.NotNil)
+	c.Check(err.Error(), gc.Equals, errString)
+}
+
+func (s *RouteSuite) TestValidIPv4(c *gc.C) {
+	checkRouteIsValid(c, network.Route{
+		DestinationCIDR: "0.1.2.3/24",
+		GatewayIP:       "0.1.2.1",
+		Metric:          20,
+	})
+}
+
+func (s *RouteSuite) TestValidIPv6(c *gc.C) {
+	checkRouteIsValid(c, network.Route{
+		DestinationCIDR: "2001:db8:a0b:12f0::1/64",
+		GatewayIP:       "2001:db8:a0b:12f0::1",
+		Metric:          10,
+	})
+}
+
+func (s *RouteSuite) TestInvalidMixedIP(c *gc.C) {
+	checkRouteErrEquals(c, network.Route{
+		DestinationCIDR: "0.1.2.3/24",
+		GatewayIP:       "2001:db8::1",
+		Metric:          10,
+	}, "DestinationCIDR is IPv4 (0.1.2.3/24) but GatewayIP is IPv6 (2001:db8::1)")
+	checkRouteErrEquals(c, network.Route{
+		DestinationCIDR: "2001:db8::1/64",
+		GatewayIP:       "0.1.2.1",
+		Metric:          10,
+	}, "DestinationCIDR is IPv6 (2001:db8::1/64) but GatewayIP is IPv4 (0.1.2.1)")
+}
+
+func (s *RouteSuite) TestInvalidNotCIDR(c *gc.C) {
+	checkRouteErrEquals(c, network.Route{
+		DestinationCIDR: "0.1.2.3",
+		GatewayIP:       "0.1.2.1",
+		Metric:          10,
+	}, "DestinationCIDR not valid: invalid CIDR address: 0.1.2.3")
+	checkRouteErrEquals(c, network.Route{
+		DestinationCIDR: "2001:db8::2",
+		GatewayIP:       "2001:db8::1",
+		Metric:          10,
+	}, "DestinationCIDR not valid: invalid CIDR address: 2001:db8::2")
+}
+
+func (s *RouteSuite) TestInvalidNotIP(c *gc.C) {
+	checkRouteErrEquals(c, network.Route{
+		DestinationCIDR: "0.1.2.3/24",
+		GatewayIP:       "0.1.2.1/16",
+		Metric:          10,
+	}, `GatewayIP is not a valid IP address: "0.1.2.1/16"`)
+	checkRouteErrEquals(c, network.Route{
+		DestinationCIDR: "0.1.2.3/24",
+		GatewayIP:       "",
+		Metric:          10,
+	}, `GatewayIP is not a valid IP address: ""`)
+	checkRouteErrEquals(c, network.Route{
+		DestinationCIDR: "2001:db8::2/64",
+		GatewayIP:       "",
+		Metric:          10,
+	}, `GatewayIP is not a valid IP address: ""`)
+}
+
+func (s *RouteSuite) TestInvalidMetric(c *gc.C) {
+	checkRouteErrEquals(c, network.Route{
+		DestinationCIDR: "0.1.2.3/24",
+		GatewayIP:       "0.1.2.1",
+		Metric:          -1,
+	}, `Metric is negative: -1`)
+}
+
 type NetworkSuite struct {
 	testing.BaseSuite
 }
@@ -118,6 +210,7 @@ func (s *NetworkSuite) TestFilterBridgeAddresses(c *gc.C) {
 	// 10.0.3.4 and 10.0.3.5/24 on that bridge.
 	// We also put 10.0.4.1 and 10.0.5.1/24 onto whatever bridge LXD is
 	// configured to use.
+	// And 192.168.122.1 on virbr0
 	netConf := []byte(`
   # comments ignored
 LXC_BR= ignored
@@ -141,6 +234,10 @@ LXC_BRIDGE="ignored"`[1:])
 				// Try a CIDR 10.0.5.1/24 as well.
 				&net.IPNet{IP: net.IPv4(10, 0, 5, 1), Mask: net.IPv4Mask(255, 255, 255, 0)},
 			}, nil
+		} else if name == network.DefaultKVMBridge {
+			return []net.Addr{
+				&net.IPAddr{IP: net.IPv4(192, 168, 122, 1)},
+			}, nil
 		}
 		c.Fatalf("unknown bridge name: %q", name)
 		return nil, nil
@@ -151,14 +248,16 @@ LXC_BRIDGE="ignored"`[1:])
 		"127.0.0.1",
 		"2001:db8::1",
 		"10.0.0.1",
-		"10.0.3.1",  // filtered (directly as IP)
-		"10.0.3.3",  // filtered (by the 10.0.3.5/24 CIDR)
-		"10.0.3.5",  // filtered (directly)
-		"10.0.3.4",  // filtered (directly)
-		"10.0.4.1",  // filtered (directly from LXD bridge)
-		"10.0.5.10", // filtered (from LXD bridge, 10.0.5.1/24)
-		"10.0.6.10", // unfiltered
+		"10.0.3.1",      // filtered (directly as IP)
+		"10.0.3.3",      // filtered (by the 10.0.3.5/24 CIDR)
+		"10.0.3.5",      // filtered (directly)
+		"10.0.3.4",      // filtered (directly)
+		"10.0.4.1",      // filtered (directly from LXD bridge)
+		"10.0.5.10",     // filtered (from LXD bridge, 10.0.5.1/24)
+		"10.0.6.10",     // unfiltered
+		"192.168.122.1", // filtered (from virbr0 bridge, 192.168.122.1)
 		"192.168.123.42",
+		"localhost", // unfiltered because it isn't an IP address
 	)
 	filteredAddresses := network.NewAddresses(
 		"127.0.0.1",
@@ -166,6 +265,7 @@ LXC_BRIDGE="ignored"`[1:])
 		"10.0.0.1",
 		"10.0.6.10",
 		"192.168.123.42",
+		"localhost",
 	)
 	c.Assert(network.FilterBridgeAddresses(inputAddresses), jc.DeepEquals, filteredAddresses)
 }
@@ -176,3 +276,22 @@ func (s *NetworkSuite) TestNoAddressError(c *gc.C) {
 	c.Assert(network.IsNoAddressError(err), jc.IsTrue)
 	c.Assert(network.IsNoAddressError(errors.New("address found")), jc.IsFalse)
 }
+
+func checkQuoteSpaceSet(c *gc.C, expected string, spaces ...string) {
+	spaceSet := set.NewStrings(spaces...)
+	c.Check(network.QuoteSpaceSet(spaceSet), gc.Equals, expected)
+}
+
+func (s *NetworkSuite) TestQuoteSpaceSet(c *gc.C) {
+	// Only the 'empty string' space
+	checkQuoteSpaceSet(c, `""`, "")
+	// No spaces
+	checkQuoteSpaceSet(c, `<none>`)
+	// One space
+	checkQuoteSpaceSet(c, `"a"`, "a")
+	// Two spaces are sorted
+	checkQuoteSpaceSet(c, `"a", "b"`, "a", "b")
+	checkQuoteSpaceSet(c, `"a", "b"`, "b", "a")
+	// Mixed
+	checkQuoteSpaceSet(c, `"", "b"`, "b", "")
+}
