@@ -10,6 +10,7 @@ import (
 	"sort"
 	"strings"
 	"sync"
+	"time"
 
 	"github.com/Azure/azure-sdk-for-go/arm/compute"
 	"github.com/Azure/azure-sdk-for-go/arm/network"
@@ -20,9 +21,11 @@ import (
 	"github.com/Azure/go-autorest/autorest/to"
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
+	"github.com/juju/retry"
 	"github.com/juju/utils/arch"
 	"github.com/juju/utils/os"
 	jujuseries "github.com/juju/utils/series"
+	"github.com/juju/version"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/cloudconfig/instancecfg"
@@ -39,6 +42,7 @@ import (
 	internalazurestorage "github.com/juju/juju/provider/azure/internal/azurestorage"
 	"github.com/juju/juju/provider/azure/internal/errorutils"
 	"github.com/juju/juju/provider/azure/internal/tracing"
+	"github.com/juju/juju/provider/azure/internal/useragent"
 	"github.com/juju/juju/provider/common"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/tools"
@@ -47,9 +51,10 @@ import (
 const (
 	jujuMachineNameTag = tags.JujuTagPrefix + "machine-name"
 
-	// defaultRootDiskSize is the default root disk size to give
-	// to a VM, if none is specified.
-	defaultRootDiskSize = 30 * 1024 // 30 GiB
+	// minRootDiskSize is the minimum root disk size Azure
+	// accepts for a VM's OS disk.
+	// It will be used if none is specified by the user.
+	minRootDiskSize = 30 * 1024 // 30 GiB
 
 	// serviceErrorCodeDeploymentCannotBeCancelled is the error code for
 	// service errors in response to an attempt to cancel a deployment
@@ -97,11 +102,12 @@ type azureEnviron struct {
 	storageClient      azurestorage.Client
 	storageAccountName string
 
-	mu                sync.Mutex
-	config            *azureModelConfig
-	instanceTypes     map[string]instances.InstanceType
-	storageAccount    *storage.Account
-	storageAccountKey *storage.AccountKey
+	mu                     sync.Mutex
+	config                 *azureModelConfig
+	instanceTypes          map[string]instances.InstanceType
+	storageAccount         *storage.Account
+	storageAccountKey      *storage.AccountKey
+	commonResourcesCreated bool
 }
 
 var _ environs.Environ = (*azureEnviron)(nil)
@@ -172,6 +178,7 @@ func (env *azureEnviron) initEnviron() error {
 		"azure.network":   &env.network.Client,
 	}
 	for id, client := range clients {
+		useragent.UpdateClient(client)
 		client.Authorizer = env.authorizer
 		logger := loggo.GetLogger(id)
 		if env.provider.config.Sender != nil {
@@ -207,7 +214,7 @@ func (env *azureEnviron) Create(args environs.CreateParams) error {
 	if err := verifyCredentials(env); err != nil {
 		return errors.Trace(err)
 	}
-	return errors.Trace(env.initResourceGroup(args.ControllerUUID))
+	return errors.Trace(env.initResourceGroup(args.ControllerUUID, false))
 }
 
 // Bootstrap is part of the Environ interface.
@@ -215,7 +222,7 @@ func (env *azureEnviron) Bootstrap(
 	ctx environs.BootstrapContext,
 	args environs.BootstrapParams,
 ) (*environs.BootstrapResult, error) {
-	if err := env.initResourceGroup(args.ControllerConfig.ControllerUUID()); err != nil {
+	if err := env.initResourceGroup(args.ControllerConfig.ControllerUUID(), true); err != nil {
 		return nil, errors.Annotate(err, "creating controller resource group")
 	}
 	result, err := common.Bootstrap(ctx, env, args)
@@ -229,14 +236,8 @@ func (env *azureEnviron) Bootstrap(
 	return result, nil
 }
 
-// BootstrapMessage is part of the Environ interface.
-func (env *azureEnviron) BootstrapMessage() string {
-	return ""
-}
-
 // initResourceGroup creates a resource group for this environment.
-func (env *azureEnviron) initResourceGroup(controllerUUID string) error {
-	location := env.location
+func (env *azureEnviron) initResourceGroup(controllerUUID string, controller bool) error {
 	resourceGroupsClient := resources.GroupsClient{env.resources}
 
 	env.mu.Lock()
@@ -245,17 +246,72 @@ func (env *azureEnviron) initResourceGroup(controllerUUID string) error {
 		names.NewControllerTag(controllerUUID),
 		env.config,
 	)
+	storageAccountType := env.config.storageAccountType
 	env.mu.Unlock()
 
 	logger.Debugf("creating resource group %q", env.resourceGroup)
 	err := env.callAPI(func() (autorest.Response, error) {
 		group, err := resourceGroupsClient.CreateOrUpdate(env.resourceGroup, resources.ResourceGroup{
-			Location: to.StringPtr(location),
+			Location: to.StringPtr(env.location),
 			Tags:     to.StringMapPtr(tags),
 		})
 		return group.Response, err
 	})
-	return errors.Annotate(err, "creating resource group")
+	if err != nil {
+		return errors.Annotate(err, "creating resource group")
+	}
+
+	if !controller {
+		// When we create a resource group for a non-controller model,
+		// we must create the common resources up-front. This is so
+		// that parallel deployments do not affect dynamic changes,
+		// e.g. those made by the firewaller. For the controller model,
+		// we fold the creation of these resources into the bootstrap
+		// machine's deployment.
+		if err := env.createCommonResourceDeployment(
+			tags, storageAccountType, nil,
+		); err != nil {
+			return errors.Trace(err)
+		}
+	}
+	return nil
+}
+
+func (env *azureEnviron) createCommonResourceDeployment(
+	tags map[string]string,
+	storageAccountType string,
+	rules []network.SecurityRule,
+) error {
+	const apiPort = -1
+	commonResources := networkTemplateResources(
+		env.location, tags, apiPort, rules,
+	)
+	commonResources = append(commonResources, storageAccountTemplateResource(
+		env.location, tags,
+		env.storageAccountName,
+		storageAccountType,
+	))
+
+	// We perform this deployment asynchronously, to avoid blocking
+	// the "juju add-model" command; Create is called synchronously.
+	// Eventually we should have Create called asynchronously, but
+	// until then we do this, and ensure that the deployment has
+	// completed before we schedule additional deployments.
+	deploymentsClient := resources.DeploymentsClient{env.resources}
+	deploymentsClient.ResponseInspector = asyncCreationRespondDecorator(
+		deploymentsClient.ResponseInspector,
+	)
+	template := armtemplates.Template{Resources: commonResources}
+	if err := createDeployment(
+		env.callAPI,
+		deploymentsClient,
+		env.resourceGroup,
+		"common", // deployment name
+		template,
+	); err != nil {
+		return errors.Trace(err)
+	}
+	return nil
 }
 
 // ControllerInstances is specified in the Environ interface.
@@ -389,10 +445,12 @@ func (env *azureEnviron) StartInstance(args environs.StartInstanceParams) (*envi
 	// If the user has not specified a root-disk size, then
 	// set a sensible default.
 	var rootDisk uint64
-	if args.Constraints.RootDisk != nil {
+	// Azure complains if we try and specify a root disk size less than the minimum.
+	// See http://pad.lv/1645408
+	if args.Constraints.RootDisk != nil && *args.Constraints.RootDisk > minRootDiskSize {
 		rootDisk = *args.Constraints.RootDisk
 	} else {
-		rootDisk = defaultRootDiskSize
+		rootDisk = minRootDiskSize
 		args.Constraints.RootDisk = &rootDisk
 	}
 
@@ -439,7 +497,7 @@ func (env *azureEnviron) StartInstance(args environs.StartInstanceParams) (*envi
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
-	logger.Infof("picked tools %q", selectedTools[0].Version)
+	logger.Infof("picked agent binaries %q", selectedTools[0].Version)
 
 	// Finalize the instance config, which we'll render to CustomData below.
 	if err := args.InstanceConfig.SetTools(selectedTools); err != nil {
@@ -516,15 +574,40 @@ func (env *azureEnviron) createVirtualMachine(
 		}
 		apiPort = apiPorts[0]
 	}
-	resources := networkTemplateResources(env.location, envTags, apiPort)
-	resources = append(resources, storageAccountTemplateResource(
-		env.location, envTags,
-		env.storageAccountName, storageAccountType,
-	))
+
+	var nicDependsOn, vmDependsOn []string
+	var resources []armtemplates.Resource
+	createCommonResources := instanceConfig.Bootstrap != nil
+	if createCommonResources {
+		// We're starting the bootstrap machine, so we will create the
+		// common resources in the same deployment.
+		commonResources := networkTemplateResources(env.location, envTags, apiPort, nil)
+		commonResources = append(commonResources, storageAccountTemplateResource(
+			env.location, envTags,
+			env.storageAccountName, storageAccountType,
+		))
+		resources = append(resources, commonResources...)
+		nicDependsOn = append(nicDependsOn, fmt.Sprintf(
+			`[resourceId('Microsoft.Network/virtualNetworks', '%s')]`,
+			internalNetworkName,
+		))
+		vmDependsOn = append(vmDependsOn, fmt.Sprintf(
+			`[resourceId('Microsoft.Storage/storageAccounts', '%s')]`,
+			env.storageAccountName,
+		))
+	} else {
+		// Wait for the common resource deployment to complete.
+		if err := env.waitCommonResourcesCreated(); err != nil {
+			return errors.Annotate(
+				err, "waiting for common resources to be created",
+			)
+		}
+	}
 
 	osProfile, seriesOS, err := newOSProfile(
 		vmName, instanceConfig,
 		env.provider.config.RandomWindowsAdminPassword,
+		env.provider.config.GenerateSSHKey,
 	)
 	if err != nil {
 		return errors.Annotate(err, "creating OS profile")
@@ -534,7 +617,6 @@ func (env *azureEnviron) createVirtualMachine(
 		return errors.Annotate(err, "creating storage profile")
 	}
 
-	var vmDependsOn []string
 	var availabilitySetSubResource *compute.SubResource
 	availabilitySetName, err := availabilitySetName(
 		vmName, vmTags, instanceConfig.Controller != nil,
@@ -593,6 +675,7 @@ func (env *azureEnviron) createVirtualMachine(
 	}
 	nicName := vmName + "-primary"
 	nicId := fmt.Sprintf(`[resourceId('Microsoft.Network/networkInterfaces', '%s')]`, nicName)
+	nicDependsOn = append(nicDependsOn, publicIPAddressId)
 	ipConfigurations := []network.InterfaceIPConfiguration{{
 		Name: to.StringPtr("primary"),
 		Properties: &network.InterfaceIPConfigurationPropertiesFormat{
@@ -614,13 +697,7 @@ func (env *azureEnviron) createVirtualMachine(
 		Properties: &network.InterfacePropertiesFormat{
 			IPConfigurations: &ipConfigurations,
 		},
-		DependsOn: []string{
-			publicIPAddressId,
-			fmt.Sprintf(
-				`[resourceId('Microsoft.Network/virtualNetworks', '%s')]`,
-				internalNetworkName,
-			),
-		},
+		DependsOn: nicDependsOn,
 	})
 
 	nics := []compute.NetworkInterfaceReference{{
@@ -630,10 +707,6 @@ func (env *azureEnviron) createVirtualMachine(
 		},
 	}}
 	vmDependsOn = append(vmDependsOn, nicId)
-	vmDependsOn = append(vmDependsOn, fmt.Sprintf(
-		`[resourceId('Microsoft.Storage/storageAccounts', '%s')]`,
-		env.storageAccountName,
-	))
 	resources = append(resources, armtemplates.Resource{
 		APIVersion: compute.APIVersion,
 		Type:       "Microsoft.Compute/virtualMachines",
@@ -698,6 +771,83 @@ func (env *azureEnviron) createVirtualMachine(
 	return nil
 }
 
+// waitCommonResourcesCreated waits for the "common" deployment to complete.
+func (env *azureEnviron) waitCommonResourcesCreated() error {
+	env.mu.Lock()
+	defer env.mu.Unlock()
+	if env.commonResourcesCreated {
+		return nil
+	}
+	if err := env.waitCommonResourcesCreatedLocked(); err != nil {
+		return errors.Trace(err)
+	}
+	env.commonResourcesCreated = true
+	return nil
+}
+
+type deploymentIncompleteError struct {
+	error
+}
+
+func (env *azureEnviron) waitCommonResourcesCreatedLocked() error {
+	deploymentsClient := resources.DeploymentsClient{env.resources}
+
+	// Release the lock while we're waiting, to avoid blocking others.
+	env.mu.Unlock()
+	defer env.mu.Lock()
+
+	// Wait for up to 5 minutes, with a 5 second polling interval,
+	// for the "common" deployment to be in one of the terminal
+	// states. The deployment typically takes only around 30 seconds,
+	// but we allow for a longer duration to be defensive.
+	waitDeployment := func() error {
+		var result resources.DeploymentExtended
+		if err := env.callAPI(func() (autorest.Response, error) {
+			var err error
+			result, err = deploymentsClient.Get(env.resourceGroup, "common")
+			return result.Response, err
+		}); err != nil {
+			if result.StatusCode == http.StatusNotFound {
+				// The controller model does not have a "common"
+				// deployment, as its common resources are created
+				// in the machine-0 deployment to keep bootstrap times
+				// optimal. Treat lack of a common deployment as an
+				// indication that the model is the controller model.
+				return nil
+			}
+			return errors.Annotate(err, "querying common deployment")
+		}
+		if result.Properties == nil {
+			return deploymentIncompleteError{errors.New("deployment incomplete")}
+		}
+
+		state := to.String(result.Properties.ProvisioningState)
+		if state == "Succeeded" {
+			// The deployment has succeeded, so the resources are
+			// ready for use.
+			return nil
+		}
+		err := errors.Errorf("common resource deployment status is %q", state)
+		switch state {
+		case "Canceled", "Failed", "Deleted":
+		default:
+			err = deploymentIncompleteError{err}
+		}
+		return err
+	}
+	return retry.Call(retry.CallArgs{
+		Func: waitDeployment,
+		IsFatalError: func(err error) bool {
+			_, ok := err.(deploymentIncompleteError)
+			return !ok
+		},
+		Attempts:    -1,
+		Delay:       5 * time.Second,
+		MaxDuration: 5 * time.Minute,
+		Clock:       env.provider.config.RetryClock,
+	})
+}
+
 // createAvailabilitySet creates the availability set for a machine to use
 // if it doesn't already exist, and returns the availability set's ID. The
 // algorithm used for choosing the availability set is:
@@ -791,6 +941,7 @@ func newOSProfile(
 	vmName string,
 	instanceConfig *instancecfg.InstanceConfig,
 	randomAdminPassword func() string,
+	generateSSHKey func(string) (string, string, error),
 ) (*compute.OSProfile, os.OSType, error) {
 	logger.Debugf("creating OS profile for %q", vmName)
 
@@ -813,9 +964,24 @@ func newOSProfile(
 		// SSH keys are handled by custom data, but must also be
 		// specified in order to forego providing a password, and
 		// disable password authentication.
+		authorizedKeys := instanceConfig.AuthorizedKeys
+		if len(authorizedKeys) == 0 {
+			// Azure requires that machines be provisioned with
+			// either a password or at least one SSH key. We
+			// generate a key-pair to make Azure happy, but throw
+			// away the private key so that nobody will be able
+			// to log into the machine directly unless the keys
+			// are updated with one that Juju tracks.
+			_, public, err := generateSSHKey("")
+			if err != nil {
+				return nil, os.Unknown, errors.Trace(err)
+			}
+			authorizedKeys = public
+		}
+
 		publicKeys := []compute.SSHPublicKey{{
 			Path:    to.StringPtr("/home/ubuntu/.ssh/authorized_keys"),
-			KeyData: to.StringPtr(instanceConfig.AuthorizedKeys),
+			KeyData: to.StringPtr(authorizedKeys),
 		}}
 		osProfile.AdminUsername = to.StringPtr("ubuntu")
 		osProfile.LinuxConfiguration = &compute.LinuxConfiguration{
@@ -1071,6 +1237,158 @@ func (env *azureEnviron) instances(
 	return matching, nil
 }
 
+// AdoptResources is part of the Environ interface.
+func (env *azureEnviron) AdoptResources(controllerUUID string, fromVersion version.Number) error {
+	groupClient := resources.GroupsClient{env.resources}
+
+	err := env.updateGroupControllerTag(&groupClient, env.resourceGroup, controllerUUID)
+	if err != nil {
+		// If we can't update the group there's no point updating the
+		// contained resources - the group will be killed if the
+		// controller is destroyed, taking the other things with it.
+		return errors.Trace(err)
+	}
+
+	resourceClient := resources.Client{env.resources}
+	var failed []string
+
+	apiVersions, err := collectAPIVersions(env.callAPI, env.resources)
+	if err != nil {
+		return errors.Trace(err)
+	}
+
+	var res resources.ResourceListResult
+	err = env.callAPI(func() (autorest.Response, error) {
+		var err error
+		res, err = groupClient.ListResources(env.resourceGroup, "", nil)
+		return res.Response, err
+	})
+	if err != nil {
+		return errors.Annotate(err, "listing resources")
+	}
+	for res.Value != nil {
+		for _, resource := range *res.Value {
+			// We need to set the API version to a value that's
+			// correct for the specific resource type. If we leave it
+			// as the version for the Microsoft.Resources provider we
+			// get NoRegisteredProviderFound errors.
+			resourceClient.APIVersion = apiVersions[to.String(resource.Type)]
+			err := env.updateResourceControllerTag(&resourceClient, resource, controllerUUID)
+			if err != nil {
+				name := to.String(resource.Name)
+				logger.Errorf("error updating resource tags for %q: %v", name, err)
+				failed = append(failed, name)
+			}
+		}
+		err = env.callAPI(func() (autorest.Response, error) {
+			var err error
+			res, err = groupClient.ListResourcesNextResults(res)
+			return res.Response, err
+		})
+		if err != nil {
+			return errors.Annotate(err, "getting next page of resources")
+		}
+	}
+
+	if len(failed) > 0 {
+		return errors.Errorf("failed to update controller for some resources: %v", failed)
+	}
+	return nil
+}
+
+func (env *azureEnviron) updateGroupControllerTag(client *resources.GroupsClient, groupName, controllerUUID string) error {
+	var group resources.ResourceGroup
+	err := env.callAPI(func() (autorest.Response, error) {
+		var err error
+		group, err = client.Get(groupName)
+		return group.Response, err
+	})
+	if err != nil {
+		return errors.Trace(err)
+	}
+
+	logger.Debugf("updating resource group %s juju controller uuid to %s",
+		to.String(group.Name), controllerUUID)
+	groupTags := toTags(group.Tags)
+	groupTags[tags.JujuController] = controllerUUID
+	group.Tags = to.StringMapPtr(groupTags)
+
+	// The Azure API forbids specifying ProvisioningState on the update.
+	if group.Properties != nil {
+		(*group.Properties).ProvisioningState = nil
+	}
+
+	err = env.callAPI(func() (autorest.Response, error) {
+		res, err := client.CreateOrUpdate(groupName, group)
+		return res.Response, err
+	})
+	return errors.Annotatef(err, "updating controller for resource group %q", groupName)
+}
+
+func (env *azureEnviron) updateResourceControllerTag(client *resources.Client, stubResource resources.GenericResource, controllerUUID string) error {
+	stubTags := toTags(stubResource.Tags)
+	if stubTags[tags.JujuController] == controllerUUID {
+		// No update needed.
+		return nil
+	}
+
+	namespace, parentPath, subtype, err := splitResourceType(to.String(stubResource.Type))
+	if err != nil {
+		return errors.Annotatef(err, "splitting resource type")
+	}
+
+	// Need to get the resource individually to ensure that the
+	// properties are populated.
+	var resource resources.GenericResource
+	err = env.callAPI(func() (autorest.Response, error) {
+		var err error
+		resource, err = client.Get(
+			env.resourceGroup,
+			namespace,
+			parentPath,
+			subtype,
+			to.String(stubResource.Name),
+		)
+		return resource.Response, err
+	})
+	if err != nil {
+		return errors.Annotatef(err, "getting full resource %q", to.String(stubResource.Name))
+	}
+
+	logger.Debugf("updating %s (%s) juju controller uuid to %s",
+		to.String(resource.Name), to.String(resource.Type), controllerUUID)
+	resourceTags := toTags(resource.Tags)
+	resourceTags[tags.JujuController] = controllerUUID
+	resource.Tags = to.StringMapPtr(resourceTags)
+
+	err = env.callAPI(func() (autorest.Response, error) {
+		res, err := client.CreateOrUpdate(
+			env.resourceGroup,
+			namespace,
+			parentPath,
+			subtype,
+			to.String(resource.Name),
+			resource,
+		)
+		return res.Response, err
+	})
+	return errors.Annotatef(err, "updating controller for %q", to.String(resource.Name))
+}
+
+// splitResourceType breaks the resource type into provider namespace,
+// parent path and subtype so we can pass the components to the
+// resource CreateOrUpdate method.
+func splitResourceType(resourceType string) (string, string, string, error) {
+	parts := strings.Split(resourceType, "/")
+	if len(parts) < 2 {
+		return "", "", "", errors.Errorf("expected at least 2 parts in resource type %q", resourceType)
+	}
+	namespace := parts[0]
+	subtype := parts[len(parts)-1]
+	parentPath := strings.Join(parts[1:len(parts)-1], "/")
+	return namespace, parentPath, subtype, nil
+}
+
 // AllInstances is specified in the InstanceBroker interface.
 func (env *azureEnviron) AllInstances() ([]instance.Instance, error) {
 	return env.allInstances(env.resourceGroup, true /* refresh addresses */, false /* all instances */)
@@ -1104,6 +1422,13 @@ func (env *azureEnviron) allInstances(
 	azureInstances := make([]*azureInstance, 0, len(*deploymentsResult.Value))
 	for _, deployment := range *deploymentsResult.Value {
 		name := to.String(deployment.Name)
+		if _, err := names.ParseMachineTag(name); err != nil {
+			// Deployments we create for Juju machines are named
+			// with the machine tag. We also create a "common"
+			// deployment, so this will exclude that VM and any
+			// other stray deployment resources.
+			continue
+		}
 		if deployment.Properties == nil || deployment.Properties.Dependencies == nil {
 			continue
 		}
@@ -1257,18 +1582,18 @@ var errNoFwGlobal = errors.New("global firewall mode is not supported")
 
 // OpenPorts is specified in the Environ interface. However, Azure does not
 // support the global firewall mode.
-func (env *azureEnviron) OpenPorts(ports []jujunetwork.PortRange) error {
+func (env *azureEnviron) OpenPorts(ports []jujunetwork.IngressRule) error {
 	return errNoFwGlobal
 }
 
 // ClosePorts is specified in the Environ interface. However, Azure does not
 // support the global firewall mode.
-func (env *azureEnviron) ClosePorts(ports []jujunetwork.PortRange) error {
+func (env *azureEnviron) ClosePorts(ports []jujunetwork.IngressRule) error {
 	return errNoFwGlobal
 }
 
 // Ports is specified in the Environ interface.
-func (env *azureEnviron) Ports() ([]jujunetwork.PortRange, error) {
+func (env *azureEnviron) IngressRules() ([]jujunetwork.IngressRule, error) {
 	return nil, errNoFwGlobal
 }
 
