@@ -25,6 +25,7 @@ import (
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
 	"github.com/juju/utils/arch"
+	"github.com/juju/version"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/names.v2"
 
@@ -43,11 +44,9 @@ import (
 	jujutesting "github.com/juju/juju/juju/testing"
 	"github.com/juju/juju/provider/azure"
 	"github.com/juju/juju/provider/azure/internal/armtemplates"
-	"github.com/juju/juju/provider/azure/internal/azureauth"
 	"github.com/juju/juju/provider/azure/internal/azuretesting"
 	"github.com/juju/juju/testing"
 	"github.com/juju/juju/tools"
-	"github.com/juju/version"
 )
 
 const storageAccountName = "juju400d80004b1d0d06f00d"
@@ -68,25 +67,10 @@ var (
 	centos7ImageReference = compute.ImageReference{
 		Publisher: to.StringPtr("OpenLogic"),
 		Offer:     to.StringPtr("CentOS"),
-		Sku:       to.StringPtr("7.1"),
+		Sku:       to.StringPtr("7.3"),
 		Version:   to.StringPtr("latest"),
 	}
 
-	sshPublicKeys = []compute.SSHPublicKey{{
-		Path:    to.StringPtr("/home/ubuntu/.ssh/authorized_keys"),
-		KeyData: to.StringPtr(testing.FakeAuthKeys),
-	}}
-	linuxOsProfile = compute.OSProfile{
-		ComputerName:  to.StringPtr("machine-0"),
-		CustomData:    to.StringPtr("<juju-goes-here>"),
-		AdminUsername: to.StringPtr("ubuntu"),
-		LinuxConfiguration: &compute.LinuxConfiguration{
-			DisablePasswordAuthentication: to.BoolPtr(true),
-			SSH: &compute.SSHConfiguration{
-				PublicKeys: &sshPublicKeys,
-			},
-		},
-	}
 	windowsOsProfile = compute.OSProfile{
 		ComputerName:  to.StringPtr("machine-0"),
 		CustomData:    to.StringPtr("<juju-goes-here>"),
@@ -117,7 +101,10 @@ type environSuite struct {
 	storageAccount     *storage.Account
 	storageAccountKeys *storage.AccountListKeysResult
 	ubuntuServerSKUs   []compute.VirtualMachineImageResource
+	commonDeployment   *resources.DeploymentExtended
 	deployment         *resources.Deployment
+	sshPublicKeys      []compute.SSHPublicKey
+	linuxOsProfile     compute.OSProfile
 }
 
 var _ = gc.Suite(&environSuite{})
@@ -136,8 +123,7 @@ func (s *environSuite) SetUpTest(c *gc.C) {
 		RetryClock: &gitjujutesting.AutoAdvancingClock{
 			&s.retryClock, s.retryClock.Advance,
 		},
-		RandomWindowsAdminPassword:        func() string { return "sorandom" },
-		InteractiveCreateServicePrincipal: azureauth.InteractiveCreateServicePrincipal,
+		RandomWindowsAdminPassword: func() string { return "sorandom" },
 	})
 
 	s.controllerUUID = testing.ControllerTag.Id()
@@ -213,7 +199,29 @@ func (s *environSuite) SetUpTest(c *gc.C) {
 		{Name: to.StringPtr("16.04-LTS")},
 	}
 
+	s.commonDeployment = &resources.DeploymentExtended{
+		Properties: &resources.DeploymentPropertiesExtended{
+			ProvisioningState: to.StringPtr("Succeeded"),
+		},
+	}
+
 	s.deployment = nil
+
+	s.sshPublicKeys = []compute.SSHPublicKey{{
+		Path:    to.StringPtr("/home/ubuntu/.ssh/authorized_keys"),
+		KeyData: to.StringPtr(testing.FakeAuthKeys),
+	}}
+	s.linuxOsProfile = compute.OSProfile{
+		ComputerName:  to.StringPtr("machine-0"),
+		CustomData:    to.StringPtr("<juju-goes-here>"),
+		AdminUsername: to.StringPtr("ubuntu"),
+		LinuxConfiguration: &compute.LinuxConfiguration{
+			DisablePasswordAuthentication: to.BoolPtr(true),
+			SSH: &compute.SSHConfiguration{
+				PublicKeys: &s.sshPublicKeys,
+			},
+		},
+	}
 }
 
 func (s *environSuite) openEnviron(c *gc.C, attrs ...testing.Attrs) environs.Environ {
@@ -321,11 +329,16 @@ func (s *environSuite) initResourceGroupSenders() azuretesting.Senders {
 	return senders
 }
 
-func (s *environSuite) startInstanceSenders(controller bool) azuretesting.Senders {
+func (s *environSuite) startInstanceSenders(bootstrap bool) azuretesting.Senders {
 	senders := azuretesting.Senders{s.vmSizesSender()}
 	if s.ubuntuServerSKUs != nil {
 		senders = append(senders, s.makeSender(".*/Canonical/.*/UbuntuServer/skus", s.ubuntuServerSKUs))
 	}
+	if !bootstrap {
+		// When starting an instance, we must wait for the common
+		// deployment to complete.
+		senders = append(senders, s.makeSender("/deployments/common", s.commonDeployment))
+	}
 	senders = append(senders, s.makeSender("/deployments/machine-0", s.deployment))
 	return senders
 }
@@ -453,10 +466,35 @@ func (s *environSuite) TestCloudEndpointManagementURI(c *gc.C) {
 }
 
 func (s *environSuite) TestStartInstance(c *gc.C) {
+	s.assertStartInstance(c, nil)
+}
+
+func (s *environSuite) TestStartInstanceRootDiskSmallerThanMin(c *gc.C) {
+	wantedRootDisk := 22
+	s.assertStartInstance(c, &wantedRootDisk)
+}
+
+func (s *environSuite) TestStartInstanceRootDiskLargerThanMin(c *gc.C) {
+	wantedRootDisk := 40
+	s.assertStartInstance(c, &wantedRootDisk)
+}
+
+func (s *environSuite) assertStartInstance(c *gc.C, wantedRootDisk *int) {
 	env := s.openEnviron(c)
 	s.sender = s.startInstanceSenders(false)
 	s.requests = nil
-	result, err := env.StartInstance(makeStartInstanceParams(c, s.controllerUUID, "quantal"))
+	args := makeStartInstanceParams(c, s.controllerUUID, "quantal")
+	expectedRootDisk := uint64(30 * 1024) // 30 GiB
+	expectedDiskSize := 32
+	if wantedRootDisk != nil {
+		cons := constraints.MustParse(fmt.Sprintf("root-disk=%dG", *wantedRootDisk))
+		args.Constraints = cons
+		if *wantedRootDisk > 30 {
+			expectedRootDisk = uint64(*wantedRootDisk * 1024)
+			expectedDiskSize = *wantedRootDisk + 2
+		}
+	}
+	result, err := env.StartInstance(args)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(result, gc.NotNil)
 	c.Assert(result.Instance, gc.NotNil)
@@ -466,18 +504,41 @@ func (s *environSuite) TestStartInstance(c *gc.C) {
 
 	arch := "amd64"
 	mem := uint64(1792)
-	rootDisk := uint64(30 * 1024) // 30 GiB
 	cpuCores := uint64(1)
 	c.Assert(result.Hardware, jc.DeepEquals, &instance.HardwareCharacteristics{
 		Arch:     &arch,
 		Mem:      &mem,
-		RootDisk: &rootDisk,
+		RootDisk: &expectedRootDisk,
 		CpuCores: &cpuCores,
 	})
+	s.assertStartInstanceRequests(c, s.requests, assertStartInstanceRequestsParams{
+		imageReference: &quantalImageReference,
+		diskSizeGB:     expectedDiskSize,
+		osProfile:      &s.linuxOsProfile,
+		instanceType:   "Standard_A1",
+	})
+}
+
+func (s *environSuite) TestStartInstanceNoAuthorizedKeys(c *gc.C) {
+	env := s.openEnviron(c)
+	cfg, err := env.Config().Remove([]string{"authorized-keys"})
+	c.Assert(err, jc.ErrorIsNil)
+	err = env.SetConfig(cfg)
+	c.Assert(err, jc.ErrorIsNil)
+
+	s.sender = s.startInstanceSenders(false)
+	s.requests = nil
+	_, err = env.StartInstance(makeStartInstanceParams(c, s.controllerUUID, "quantal"))
+	c.Assert(err, jc.ErrorIsNil)
+
+	s.PatchValue(&s.sshPublicKeys, []compute.SSHPublicKey{{
+		Path:    to.StringPtr("/home/ubuntu/.ssh/authorized_keys"),
+		KeyData: to.StringPtr("public"),
+	}})
 	s.assertStartInstanceRequests(c, s.requests, assertStartInstanceRequestsParams{
 		imageReference: &quantalImageReference,
 		diskSizeGB:     32,
-		osProfile:      &linuxOsProfile,
+		osProfile:      &s.linuxOsProfile,
 		instanceType:   "Standard_A1",
 	})
 }
@@ -556,7 +617,7 @@ func (s *environSuite) TestStartInstanceCentOS(c *gc.C) {
 			AutoUpgradeMinorVersion: to.BoolPtr(true),
 			Settings:                &vmExtensionSettings,
 		},
-		osProfile:    &linuxOsProfile,
+		osProfile:    &s.linuxOsProfile,
 		instanceType: "Standard_A1",
 	})
 }
@@ -592,7 +653,7 @@ func (s *environSuite) TestStartInstanceTooManyRequests(c *gc.C) {
 	s.assertStartInstanceRequests(c, s.requests[:numExpectedStartInstanceRequests], assertStartInstanceRequestsParams{
 		imageReference: &quantalImageReference,
 		diskSizeGB:     32,
-		osProfile:      &linuxOsProfile,
+		osProfile:      &s.linuxOsProfile,
 		instanceType:   "Standard_A1",
 	})
 
@@ -652,6 +713,57 @@ func (s *environSuite) TestStartInstanceTooManyRequestsTimeout(c *gc.C) {
 	})
 }
 
+func (s *environSuite) TestStartInstanceCommonDeployment(c *gc.C) {
+	// StartInstance waits for the "common" deployment to complete
+	// successfully before creating the VM deployment. If the deployment
+	// is seen to be in a terminal state, the process will stop
+	// immediately.
+	s.commonDeployment.Properties.ProvisioningState = to.StringPtr("Failed")
+
+	env := s.openEnviron(c)
+	senders := s.startInstanceSenders(false)
+	s.sender = senders
+	s.requests = nil
+
+	_, err := env.StartInstance(makeStartInstanceParams(c, s.controllerUUID, "quantal"))
+	c.Assert(err, gc.ErrorMatches,
+		`creating virtual machine "machine-0": `+
+			`waiting for common resources to be created: `+
+			`common resource deployment status is "Failed"`)
+}
+
+func (s *environSuite) TestStartInstanceCommonDeploymentRetryTimeout(c *gc.C) {
+	// StartInstance waits for the "common" deployment to complete
+	// successfully before creating the VM deployment.
+	s.commonDeployment.Properties.ProvisioningState = to.StringPtr("Running")
+
+	env := s.openEnviron(c)
+	senders := s.startInstanceSenders(false)
+
+	const failures = 60 // 5 minutes / 5 seconds
+	head, tail := senders[:2], senders[2:]
+	for i := 0; i < failures; i++ {
+		head = append(head, s.makeSender("/deployments/common", s.commonDeployment))
+	}
+	senders = append(head, tail...)
+	s.sender = senders
+	s.requests = nil
+
+	_, err := env.StartInstance(makeStartInstanceParams(c, s.controllerUUID, "quantal"))
+	c.Assert(err, gc.ErrorMatches,
+		`creating virtual machine "machine-0": `+
+			`waiting for common resources to be created: `+
+			`max duration exceeded: deployment incomplete`)
+
+	var expectedCalls []gitjujutesting.StubCall
+	for i := 0; i < failures; i++ {
+		expectedCalls = append(expectedCalls, gitjujutesting.StubCall{
+			"After", []interface{}{5 * time.Second},
+		})
+	}
+	s.retryClock.CheckCalls(c, expectedCalls)
+}
+
 func (s *environSuite) TestStartInstanceDistributionGroup(c *gc.C) {
 	c.Skip("TODO: test StartInstance's DistributionGroup behaviour")
 }
@@ -671,12 +783,15 @@ func (s *environSuite) TestStartInstanceServiceAvailabilitySet(c *gc.C) {
 		availabilitySetName: "mysql",
 		imageReference:      &quantalImageReference,
 		diskSizeGB:          32,
-		osProfile:           &linuxOsProfile,
+		osProfile:           &s.linuxOsProfile,
 		instanceType:        "Standard_A1",
 	})
 }
 
-const numExpectedStartInstanceRequests = 3
+// numExpectedStartInstanceRequests is the number of expected requests base
+// by StartInstance method calls. The number is one less for Bootstrap, which
+// does not require a query on the common deployment.
+const numExpectedStartInstanceRequests = 4
 
 type assertStartInstanceRequestsParams struct {
 	availabilitySetName string
@@ -739,11 +854,13 @@ func (s *environSuite) assertStartInstanceRequests(
 		},
 	}}
 
+	createCommonResources := false
 	subnetName := "juju-internal-subnet"
 	privateIPAddress := "192.168.0.4"
 	if args.availabilitySetName == "juju-controller" {
 		subnetName = "juju-controller-subnet"
 		privateIPAddress = "192.168.16.4"
+		createCommonResources = true
 	}
 	subnetId := fmt.Sprintf(
 		`[concat(resourceId('Microsoft.Network/virtualNetworks', 'juju-internal-network'), '/subnets/%s')]`,
@@ -772,42 +889,48 @@ func (s *environSuite) assertStartInstanceRequests(
 			Primary: to.BoolPtr(true),
 		},
 	}}
-	vmDependsOn := []string{
-		nicId,
-		`[resourceId('Microsoft.Storage/storageAccounts', '` + storageAccountName + `')]`,
-	}
 
-	addressPrefixes := []string{"192.168.0.0/20", "192.168.16.0/20"}
-	templateResources := []armtemplates.Resource{{
-		APIVersion: network.APIVersion,
-		Type:       "Microsoft.Network/networkSecurityGroups",
-		Name:       "juju-internal-nsg",
-		Location:   "westus",
-		Tags:       to.StringMap(s.envTags),
-		Properties: &network.SecurityGroupPropertiesFormat{
-			SecurityRules: &securityRules,
-		},
-	}, {
-		APIVersion: network.APIVersion,
-		Type:       "Microsoft.Network/virtualNetworks",
-		Name:       "juju-internal-network",
-		Location:   "westus",
-		Tags:       to.StringMap(s.envTags),
-		Properties: &network.VirtualNetworkPropertiesFormat{
-			AddressSpace: &network.AddressSpace{&addressPrefixes},
-			Subnets:      &subnets,
-		},
-		DependsOn: []string{nsgId},
-	}, {
-		APIVersion: storage.APIVersion,
-		Type:       "Microsoft.Storage/storageAccounts",
-		Name:       storageAccountName,
-		Location:   "westus",
-		Tags:       to.StringMap(s.envTags),
-		StorageSku: &storage.Sku{
-			Name: storage.SkuName("Standard_LRS"),
-		},
-	}}
+	var nicDependsOn, vmDependsOn []string
+	var templateResources []armtemplates.Resource
+	if createCommonResources {
+		addressPrefixes := []string{"192.168.0.0/20", "192.168.16.0/20"}
+		templateResources = append(templateResources, []armtemplates.Resource{{
+			APIVersion: network.APIVersion,
+			Type:       "Microsoft.Network/networkSecurityGroups",
+			Name:       "juju-internal-nsg",
+			Location:   "westus",
+			Tags:       to.StringMap(s.envTags),
+			Properties: &network.SecurityGroupPropertiesFormat{
+				SecurityRules: &securityRules,
+			},
+		}, {
+			APIVersion: network.APIVersion,
+			Type:       "Microsoft.Network/virtualNetworks",
+			Name:       "juju-internal-network",
+			Location:   "westus",
+			Tags:       to.StringMap(s.envTags),
+			Properties: &network.VirtualNetworkPropertiesFormat{
+				AddressSpace: &network.AddressSpace{&addressPrefixes},
+				Subnets:      &subnets,
+			},
+			DependsOn: []string{nsgId},
+		}, {
+			APIVersion: storage.APIVersion,
+			Type:       "Microsoft.Storage/storageAccounts",
+			Name:       storageAccountName,
+			Location:   "westus",
+			Tags:       to.StringMap(s.envTags),
+			StorageSku: &storage.Sku{
+				Name: storage.SkuName("Standard_LRS"),
+			},
+		}}...)
+		vmDependsOn = append(vmDependsOn,
+			`[resourceId('Microsoft.Storage/storageAccounts', '`+storageAccountName+`')]`,
+		)
+		nicDependsOn = append(nicDependsOn,
+			`[resourceId('Microsoft.Network/virtualNetworks', 'juju-internal-network')]`,
+		)
+	}
 
 	var availabilitySetSubResource *compute.SubResource
 	if args.availabilitySetName != "" {
@@ -825,7 +948,7 @@ func (s *environSuite) assertStartInstanceRequests(
 		availabilitySetSubResource = &compute.SubResource{
 			ID: to.StringPtr(availabilitySetId),
 		}
-		vmDependsOn = append([]string{availabilitySetId}, vmDependsOn...)
+		vmDependsOn = append(vmDependsOn, availabilitySetId)
 	}
 
 	templateResources = append(templateResources, []armtemplates.Resource{{
@@ -846,10 +969,7 @@ func (s *environSuite) assertStartInstanceRequests(
 		Properties: &network.InterfacePropertiesFormat{
 			IPConfigurations: &ipConfigurations,
 		},
-		DependsOn: []string{
-			publicIPAddressId,
-			`[resourceId('Microsoft.Network/virtualNetworks', 'juju-internal-network')]`,
-		},
+		DependsOn: append(nicDependsOn, publicIPAddressId),
 	}, {
 		APIVersion: compute.APIVersion,
 		Type:       "Microsoft.Compute/virtualMachines",
@@ -879,7 +999,7 @@ func (s *environSuite) assertStartInstanceRequests(
 			NetworkProfile:  &compute.NetworkProfile{&nics},
 			AvailabilitySet: availabilitySetSubResource,
 		},
-		DependsOn: vmDependsOn,
+		DependsOn: append(vmDependsOn, nicId),
 	}}...)
 	if args.vmExtension != nil {
 		templateResources = append(templateResources, armtemplates.Resource{
@@ -904,34 +1024,44 @@ func (s *environSuite) assertStartInstanceRequests(
 		},
 	}
 
+	var i int
+	nexti := func() int {
+		i++
+		return i - 1
+	}
+
 	// Validate HTTP request bodies.
 	var startInstanceRequests startInstanceRequests
 	if args.vmExtension != nil {
 		// It must be Windows or CentOS, so
 		// there should be no image query.
 		c.Assert(requests, gc.HasLen, numExpectedStartInstanceRequests-1)
-		c.Assert(requests[0].Method, gc.Equals, "GET") // vmSizes
-		c.Assert(requests[1].Method, gc.Equals, "PUT") // create deployment
+		c.Assert(requests[nexti()].Method, gc.Equals, "GET") // vmSizes
 		startInstanceRequests.vmSizes = requests[0]
-		startInstanceRequests.deployment = requests[1]
 	} else {
-		c.Assert(requests, gc.HasLen, numExpectedStartInstanceRequests)
+		if createCommonResources {
+			c.Assert(requests, gc.HasLen, numExpectedStartInstanceRequests-1)
+		} else {
+			c.Assert(requests, gc.HasLen, numExpectedStartInstanceRequests)
+		}
 		if args.needsProviderInit {
-			c.Assert(requests[0].Method, gc.Equals, "PUT") // resource groups
-			c.Assert(requests[1].Method, gc.Equals, "GET") // skus
-			c.Assert(requests[2].Method, gc.Equals, "PUT") // create deployment
+			c.Assert(requests[nexti()].Method, gc.Equals, "PUT") // resource groups
+			c.Assert(requests[nexti()].Method, gc.Equals, "GET") // skus
 			startInstanceRequests.resourceGroups = requests[0]
 			startInstanceRequests.skus = requests[1]
-			startInstanceRequests.deployment = requests[2]
 		} else {
-			c.Assert(requests[0].Method, gc.Equals, "GET") // vmSizes
-			c.Assert(requests[1].Method, gc.Equals, "GET") // skus
-			c.Assert(requests[2].Method, gc.Equals, "PUT") // create deployment
+			c.Assert(requests[nexti()].Method, gc.Equals, "GET") // vmSizes
+			c.Assert(requests[nexti()].Method, gc.Equals, "GET") // skus
 			startInstanceRequests.vmSizes = requests[0]
 			startInstanceRequests.skus = requests[1]
-			startInstanceRequests.deployment = requests[2]
 		}
 	}
+	if !createCommonResources {
+		c.Assert(requests[nexti()].Method, gc.Equals, "GET") // wait for common deployment
+	}
+	ideployment := nexti()
+	c.Assert(requests[ideployment].Method, gc.Equals, "PUT") // create deployment
+	startInstanceRequests.deployment = requests[ideployment]
 
 	// Marshal/unmarshal the deployment we expect, so it's in map form.
 	var expected resources.Deployment
@@ -991,13 +1121,13 @@ func (s *environSuite) TestBootstrap(c *gc.C) {
 	c.Assert(result.Arch, gc.Equals, "amd64")
 	c.Assert(result.Series, gc.Equals, "quantal")
 
-	c.Assert(len(s.requests), gc.Equals, numExpectedStartInstanceRequests+1)
+	c.Assert(len(s.requests), gc.Equals, numExpectedStartInstanceRequests)
 	s.vmTags[tags.JujuIsController] = to.StringPtr("true")
 	s.assertStartInstanceRequests(c, s.requests[1:], assertStartInstanceRequestsParams{
 		availabilitySetName: "juju-controller",
 		imageReference:      &quantalImageReference,
 		diskSizeGB:          32,
-		osProfile:           &linuxOsProfile,
+		osProfile:           &s.linuxOsProfile,
 		instanceType:        "Standard_D1",
 	})
 }
@@ -1040,13 +1170,13 @@ func (s *environSuite) TestBootstrapInstanceConstraints(c *gc.C) {
 	// amd64 should pass the rest of the test.
 	c.Assert(err, jc.ErrorIsNil)
 
-	c.Assert(len(s.requests), gc.Equals, numExpectedStartInstanceRequests+1)
+	c.Assert(len(s.requests), gc.Equals, numExpectedStartInstanceRequests)
 	s.vmTags[tags.JujuIsController] = to.StringPtr("true")
 	s.assertStartInstanceRequests(c, s.requests[1:], assertStartInstanceRequestsParams{
 		availabilitySetName: "juju-controller",
 		imageReference:      &quantalImageReference,
 		diskSizeGB:          32,
-		osProfile:           &linuxOsProfile,
+		osProfile:           &s.linuxOsProfile,
 		needsProviderInit:   true,
 		instanceType:        "Standard_D1",
 	})
@@ -1063,6 +1193,30 @@ func (s *environSuite) TestAllInstancesResourceGroupNotFound(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 }
 
+func (s *environSuite) TestAllInstancesIgnoresCommonDeployment(c *gc.C) {
+	env := s.openEnviron(c)
+
+	dependencies := []resources.Dependency{{
+		ID: to.StringPtr("whatever"),
+	}}
+	deployments := []resources.DeploymentExtended{{
+		// common deployment should be ignored
+		Name: to.StringPtr("common"),
+		Properties: &resources.DeploymentPropertiesExtended{
+			ProvisioningState: to.StringPtr("Succeeded"),
+			Dependencies:      &dependencies,
+		},
+	}}
+	result := resources.DeploymentListResult{Value: &deployments}
+	s.sender = azuretesting.Senders{
+		s.makeSender("/deployments", result),
+	}
+
+	instances, err := env.AllInstances()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(instances, gc.HasLen, 0)
+}
+
 func (s *environSuite) TestStopInstancesNotFound(c *gc.C) {
 	env := s.openEnviron(c)
 	sender0 := mocks.NewSender()
@@ -1346,3 +1500,299 @@ func (s *environSuite) TestDestroyControllerErrors(c *gc.C) {
 	c.Check(destroyErr, gc.ErrorMatches, ".*foo.*")
 	c.Check(destroyErr, gc.ErrorMatches, ".*bar.*")
 }
+
+func (s *environSuite) TestInstanceInformation(c *gc.C) {
+	env := s.openEnviron(c)
+	s.sender = s.startInstanceSenders(false)
+	types, err := env.InstanceTypes(constraints.Value{})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(types.InstanceTypes, gc.HasLen, 6)
+
+	cons := constraints.MustParse("mem=4G")
+	types, err = env.InstanceTypes(cons)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(types.InstanceTypes, gc.HasLen, 2)
+}
+
+func (s *environSuite) TestAdoptResources(c *gc.C) {
+	providersResult := makeProvidersResult()
+	resourcesResult := makeResourcesResult()
+
+	res1 := (*resourcesResult.Value)[0]
+	res1.Properties = &map[string]interface{}{"has-properties": true}
+
+	res2 := (*resourcesResult.Value)[1]
+	res2.Properties = &map[string]interface{}{"has-properties": true}
+
+	env := s.openEnviron(c)
+
+	s.sender = azuretesting.Senders{
+		s.makeSender(".*/resourcegroups/juju-testenv-.*", makeResourceGroupResult()),
+		s.makeSender(".*/resourcegroups/juju-testenv-.*", nil),
+
+		s.makeSender(".*/providers", providersResult),
+		s.makeSender(".*/resourceGroups/juju-testenv-.*/resources", resourcesResult),
+
+		// First request is the get, second is the update. (The
+		// lowercase resourcegroups here is a quirk of the
+		// autogenerated Go SDK.)
+		s.makeSender(".*/resourcegroups/.*/providers/Beck.Replica/liars/scissor/boxing-day-blues", res1),
+		s.makeSender(".*/resourcegroups/.*/providers/Beck.Replica/liars/scissor/boxing-day-blues", res1),
+
+		s.makeSender(".*/resourcegroups/.*/providers/Tuneyards.Bizness//micachu/drop-dead", res2),
+		s.makeSender(".*/resourcegroups/.*/providers/Tuneyards.Bizness//micachu/drop-dead", res2),
+	}
+
+	err := env.AdoptResources("new-controller", version.MustParse("1.2.4"))
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Check that properties and tags are preserved and the correct
+	// API version is sent.
+	checkAPIVersion := func(ix uint, expectedMethod, expectedVersion string) {
+		req := s.requests[ix]
+		c.Check(req.Method, gc.Equals, expectedMethod)
+		c.Check(req.URL.Query().Get("api-version"), gc.Equals, expectedVersion)
+	}
+	// Resource group get and update.
+	checkAPIVersion(0, "GET", "2016-02-01")
+	checkAPIVersion(1, "PUT", "2016-02-01")
+	// Resources.
+	checkAPIVersion(4, "GET", "2016-12-17")
+	checkAPIVersion(5, "PUT", "2016-12-17")
+	checkAPIVersion(6, "GET", "2016-12-13")
+	checkAPIVersion(7, "PUT", "2016-12-13")
+
+	checkTagsAndProperties := func(ix uint) {
+		req := s.requests[ix]
+		data := make([]byte, req.ContentLength)
+		_, err := req.Body.Read(data)
+		c.Assert(err, jc.ErrorIsNil)
+
+		var resource resources.GenericResource
+		err = json.Unmarshal(data, &resource)
+		c.Assert(err, jc.ErrorIsNil)
+
+		rTags := to.StringMap(*resource.Tags)
+		c.Check(rTags["something else"], gc.Equals, "good")
+		c.Check(rTags[tags.JujuController], gc.Equals, "new-controller")
+		c.Check(*resource.Properties, gc.DeepEquals, map[string]interface{}{"has-properties": true})
+	}
+	checkTagsAndProperties(5)
+	checkTagsAndProperties(7)
+
+	// Also check the tags are right for the resource group.
+	req := s.requests[1] // the resource group update.
+	data := make([]byte, req.ContentLength)
+	_, err = req.Body.Read(data)
+	c.Assert(err, jc.ErrorIsNil)
+	var group resources.ResourceGroup
+	err = json.Unmarshal(data, &group)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Check that the provisioning state wasn't sent back.
+	c.Check((*group.Properties).ProvisioningState, gc.IsNil)
+
+	gTags := to.StringMap(*group.Tags)
+	c.Check(gTags["something else"], gc.Equals, "good")
+	c.Check(gTags[tags.JujuController], gc.Equals, "new-controller")
+}
+
+func makeProvidersResult() resources.ProviderListResult {
+	providers := []resources.Provider{{
+		Namespace: to.StringPtr("Beck.Replica"),
+		ResourceTypes: &[]resources.ProviderResourceType{{
+			ResourceType: to.StringPtr("battles/ladida"),
+			APIVersions:  &[]string{"2016-12-15", "2014-02-02"},
+		}, {
+			ResourceType: to.StringPtr("liars/scissor"),
+			APIVersions:  &[]string{"2016-12-17", "2015-03-02"},
+		}},
+	}, {
+		Namespace: to.StringPtr("Tuneyards.Bizness"),
+		ResourceTypes: &[]resources.ProviderResourceType{{
+			ResourceType: to.StringPtr("slaves/debbie"),
+			APIVersions:  &[]string{"2016-12-14", "2014-04-02"},
+		}, {
+			ResourceType: to.StringPtr("micachu"),
+			APIVersions:  &[]string{"2016-12-13", "2015-05-02"},
+		}},
+	}}
+	return resources.ProviderListResult{Value: &providers}
+}
+
+func makeResourcesResult() resources.ResourceListResult {
+	theResources := []resources.GenericResource{{
+		Name: to.StringPtr("boxing-day-blues"),
+		Type: to.StringPtr("Beck.Replica/liars/scissor"),
+		Tags: to.StringMapPtr(map[string]string{
+			tags.JujuController: "old-controller",
+			"something else":    "good",
+		}),
+	}, {
+		Name: to.StringPtr("drop-dead"),
+		Type: to.StringPtr("Tuneyards.Bizness/micachu"),
+		Tags: to.StringMapPtr(map[string]string{
+			tags.JujuController: "old-controller",
+			"something else":    "good",
+		}),
+	}}
+	return resources.ResourceListResult{Value: &theResources}
+}
+
+func makeResourceGroupResult() resources.ResourceGroup {
+	return resources.ResourceGroup{
+		Name: to.StringPtr("charles"),
+		Properties: &resources.ResourceGroupProperties{
+			ProvisioningState: to.StringPtr("very yes"),
+		},
+		Tags: to.StringMapPtr(map[string]string{
+			tags.JujuController: "old-controller",
+			"something else":    "good",
+		}),
+	}
+}
+
+func (s *environSuite) TestAdoptResourcesErrorGettingGroup(c *gc.C) {
+	env := s.openEnviron(c)
+	sender := s.makeSender(".*/resourcegroups/juju-testenv-.*", nil)
+	sender.SetError(errors.New("uhoh"))
+	s.sender = azuretesting.Senders{sender}
+
+	err := env.AdoptResources("new-controller", version.MustParse("1.0.0"))
+	c.Assert(err, gc.ErrorMatches, ".*uhoh$")
+	c.Assert(s.requests, gc.HasLen, 1)
+}
+
+func (s *environSuite) TestAdoptResourcesErrorUpdatingGroup(c *gc.C) {
+	env := s.openEnviron(c)
+	errorSender := s.makeSender(".*/resourcegroups/juju-testenv-.*", nil)
+	errorSender.SetError(errors.New("uhoh"))
+	s.sender = azuretesting.Senders{
+		s.makeSender(".*/resourcegroups/juju-testenv-.*", makeResourceGroupResult()),
+		errorSender,
+	}
+
+	err := env.AdoptResources("new-controller", version.MustParse("1.0.0"))
+	c.Assert(err, gc.ErrorMatches, ".*uhoh$")
+	c.Assert(s.requests, gc.HasLen, 2)
+}
+
+func (s *environSuite) TestAdoptResourcesErrorGettingVersions(c *gc.C) {
+	env := s.openEnviron(c)
+	errorSender := s.makeSender(".*/providers", nil)
+	errorSender.SetError(errors.New("uhoh"))
+	s.sender = azuretesting.Senders{
+		s.makeSender(".*/resourcegroups/juju-testenv-.*", makeResourceGroupResult()),
+		s.makeSender(".*/resourcegroups/juju-testenv-.*", nil),
+		errorSender,
+	}
+
+	err := env.AdoptResources("new-controller", version.MustParse("1.0.0"))
+	c.Assert(err, gc.ErrorMatches, ".*uhoh$")
+	c.Assert(s.requests, gc.HasLen, 3)
+}
+
+func (s *environSuite) TestAdoptResourcesErrorListingResources(c *gc.C) {
+	env := s.openEnviron(c)
+	errorSender := s.makeSender(".*/resourceGroups/juju-testenv-.*/resources", nil)
+	errorSender.SetError(errors.New("ouch!"))
+	s.sender = azuretesting.Senders{
+		s.makeSender(".*/resourcegroups/juju-testenv-.*", makeResourceGroupResult()),
+		s.makeSender(".*/resourcegroups/juju-testenv-.*", nil),
+		s.makeSender(".*/providers", resources.ProviderListResult{}),
+		errorSender,
+	}
+
+	err := env.AdoptResources("new-controller", version.MustParse("1.0.0"))
+	c.Assert(err, gc.ErrorMatches, ".*ouch!$")
+	c.Assert(s.requests, gc.HasLen, 4)
+}
+
+func (s *environSuite) TestAdoptResourcesNoUpdateNeeded(c *gc.C) {
+	providersResult := makeProvidersResult()
+	resourcesResult := makeResourcesResult()
+
+	// Give the first resource the right controller tag so it doesn't need updating.
+	res1 := (*resourcesResult.Value)[0]
+	(*res1.Tags)[tags.JujuController] = to.StringPtr("new-controller")
+	res2 := (*resourcesResult.Value)[1]
+
+	env := s.openEnviron(c)
+
+	s.sender = azuretesting.Senders{
+		s.makeSender(".*/resourcegroups/juju-testenv-.*", makeResourceGroupResult()),
+		s.makeSender(".*/resourcegroups/juju-testenv-.*", nil),
+		s.makeSender(".*/providers", providersResult),
+		s.makeSender(".*/resourceGroups/juju-testenv-.*/resources", resourcesResult),
+
+		// Doesn't bother updating res1, continues to do res2.
+		s.makeSender(".*/resourcegroups/.*/providers/Tuneyards.Bizness//micachu/drop-dead", res2),
+		s.makeSender(".*/resourcegroups/.*/providers/Tuneyards.Bizness//micachu/drop-dead", res2),
+	}
+
+	err := env.AdoptResources("new-controller", version.MustParse("1.2.4"))
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(s.requests, gc.HasLen, 6)
+}
+
+func (s *environSuite) TestAdoptResourcesErrorGettingFullResource(c *gc.C) {
+	providersResult := makeProvidersResult()
+	resourcesResult := makeResourcesResult()
+
+	res2 := (*resourcesResult.Value)[1]
+
+	env := s.openEnviron(c)
+
+	errorSender := s.makeSender(
+		".*/resourcegroups/.*/providers/Beck.Replica/liars/scissor/boxing-day-blues", nil)
+	errorSender.SetError(errors.New("flagrant error! virus=very yes"))
+
+	s.sender = azuretesting.Senders{
+		s.makeSender(".*/resourcegroups/juju-testenv-.*", makeResourceGroupResult()),
+		s.makeSender(".*/resourcegroups/juju-testenv-.*", nil),
+		s.makeSender(".*/providers", providersResult),
+		s.makeSender(".*/resourceGroups/juju-testenv-.*/resources", resourcesResult),
+
+		// The first resource yields an error but the update continues.
+		errorSender,
+
+		s.makeSender(".*/resourcegroups/.*/providers/Tuneyards.Bizness//micachu/drop-dead", res2),
+		s.makeSender(".*/resourcegroups/.*/providers/Tuneyards.Bizness//micachu/drop-dead", res2),
+	}
+
+	err := env.AdoptResources("new-controller", version.MustParse("1.2.4"))
+	c.Check(err, gc.ErrorMatches, `failed to update controller for some resources: \[boxing-day-blues\]`)
+	c.Check(s.requests, gc.HasLen, 7)
+}
+
+func (s *environSuite) TestAdoptResourcesErrorUpdating(c *gc.C) {
+	providersResult := makeProvidersResult()
+	resourcesResult := makeResourcesResult()
+
+	res1 := (*resourcesResult.Value)[0]
+	res2 := (*resourcesResult.Value)[1]
+
+	env := s.openEnviron(c)
+
+	errorSender := s.makeSender(
+		".*/resourcegroups/.*/providers/Beck.Replica/liars/scissor/boxing-day-blues", nil)
+	errorSender.SetError(errors.New("oopsie"))
+
+	s.sender = azuretesting.Senders{
+		s.makeSender(".*/resourcegroups/juju-testenv-.*", makeResourceGroupResult()),
+		s.makeSender(".*/resourcegroups/juju-testenv-.*", nil),
+		s.makeSender(".*/providers", providersResult),
+		s.makeSender(".*/resourceGroups/juju-testenv-.*/resources", resourcesResult),
+
+		// Updating the first resource yields an error but the update continues.
+		s.makeSender(".*/resourcegroups/.*/providers/Beck.Replica/liars/scissor/boxing-day-blues", res1),
+		errorSender,
+
+		s.makeSender(".*/resourcegroups/.*/providers/Tuneyards.Bizness//micachu/drop-dead", res2),
+		s.makeSender(".*/resourcegroups/.*/providers/Tuneyards.Bizness//micachu/drop-dead", res2),
+	}
+
+	err := env.AdoptResources("new-controller", version.MustParse("1.2.4"))
+	c.Check(err, gc.ErrorMatches, `failed to update controller for some resources: \[boxing-day-blues\]`)
+	c.Check(s.requests, gc.HasLen, 8)
+}
