@@ -11,12 +11,12 @@ import (
 	"github.com/Azure/azure-sdk-for-go/arm/network"
 	"github.com/Azure/go-autorest/autorest"
 	"github.com/Azure/go-autorest/autorest/to"
-
 	"github.com/juju/errors"
+	"gopkg.in/juju/names.v2"
+
 	"github.com/juju/juju/instance"
 	jujunetwork "github.com/juju/juju/network"
 	"github.com/juju/juju/status"
-	"gopkg.in/juju/names.v2"
 )
 
 type azureInstance struct {
@@ -210,7 +210,7 @@ func (inst *azureInstance) primaryNetworkAddress() (jujunetwork.Address, error)
 }
 
 // OpenPorts is specified in the Instance interface.
-func (inst *azureInstance) OpenPorts(machineId string, ports []jujunetwork.PortRange) error {
+func (inst *azureInstance) OpenPorts(machineId string, rules []jujunetwork.IngressRule) error {
 	nsgClient := network.SecurityGroupsClient{inst.env.network}
 	securityRuleClient := network.SecurityRulesClient{inst.env.network}
 	primaryNetworkAddress, err := inst.primaryNetworkAddress()
@@ -240,8 +240,10 @@ func (inst *azureInstance) OpenPorts(machineId string, ports []jujunetwork.PortR
 	// NSG in memory, so we can easily tell which priorities are available.
 	vmName := resourceName(names.NewMachineTag(machineId))
 	prefix := instanceNetworkSecurityRulePrefix(instance.Id(vmName))
-	for _, ports := range ports {
-		ruleName := securityRuleName(prefix, ports)
+
+	singleSourceIngressRules := explodeIngressRules(rules)
+	for _, rule := range singleSourceIngressRules {
+		ruleName := securityRuleName(prefix, rule)
 
 		// Check if the rule already exists; OpenPorts must be idempotent.
 		var found bool
@@ -259,33 +261,35 @@ func (inst *azureInstance) OpenPorts(machineId string, ports []jujunetwork.PortR
 
 		priority, err := nextSecurityRulePriority(nsg, securityRuleInternalMax+1, securityRuleMax)
 		if err != nil {
-			return errors.Annotatef(err, "getting security rule priority for %s", ports)
+			return errors.Annotatef(err, "getting security rule priority for %s", rule)
 		}
 
 		var protocol network.SecurityRuleProtocol
-		switch ports.Protocol {
+		switch rule.Protocol {
 		case "tcp":
 			protocol = network.TCP
 		case "udp":
 			protocol = network.UDP
 		default:
-			return errors.Errorf("invalid protocol %q", ports.Protocol)
+			return errors.Errorf("invalid protocol %q", rule.Protocol)
 		}
 
 		var portRange string
-		if ports.FromPort != ports.ToPort {
-			portRange = fmt.Sprintf("%d-%d", ports.FromPort, ports.ToPort)
+		if rule.FromPort != rule.ToPort {
+			portRange = fmt.Sprintf("%d-%d", rule.FromPort, rule.ToPort)
 		} else {
-			portRange = fmt.Sprint(ports.FromPort)
+			portRange = fmt.Sprint(rule.FromPort)
 		}
 
-		rule := network.SecurityRule{
+		// rule has a single source CIDR
+		from := rule.SourceCIDRs[0]
+		securityRule := network.SecurityRule{
 			Properties: &network.SecurityRulePropertiesFormat{
-				Description:              to.StringPtr(ports.String()),
+				Description:              to.StringPtr(rule.String()),
 				Protocol:                 protocol,
 				SourcePortRange:          to.StringPtr("*"),
 				DestinationPortRange:     to.StringPtr(portRange),
-				SourceAddressPrefix:      to.StringPtr("*"),
+				SourceAddressPrefix:      to.StringPtr(from),
 				DestinationAddressPrefix: to.StringPtr(primaryNetworkAddress.Value),
 				Access:    network.Allow,
 				Priority:  to.Int32Ptr(priority),
@@ -294,19 +298,19 @@ func (inst *azureInstance) OpenPorts(machineId string, ports []jujunetwork.PortR
 		}
 		if err := inst.env.callAPI(func() (autorest.Response, error) {
 			return securityRuleClient.CreateOrUpdate(
-				inst.env.resourceGroup, securityGroupName, ruleName, rule,
+				inst.env.resourceGroup, securityGroupName, ruleName, securityRule,
 				nil, // abort channel
 			)
 		}); err != nil {
-			return errors.Annotatef(err, "creating security rule for %s", ports)
+			return errors.Annotatef(err, "creating security rule for %s", securityRule)
 		}
-		securityRules = append(securityRules, rule)
+		securityRules = append(securityRules, securityRule)
 	}
 	return nil
 }
 
 // ClosePorts is specified in the Instance interface.
-func (inst *azureInstance) ClosePorts(machineId string, ports []jujunetwork.PortRange) error {
+func (inst *azureInstance) ClosePorts(machineId string, rules []jujunetwork.IngressRule) error {
 	securityRuleClient := network.SecurityRulesClient{inst.env.network}
 	securityGroupName := internalSecurityGroupName
 
@@ -314,8 +318,10 @@ func (inst *azureInstance) ClosePorts(machineId string, ports []jujunetwork.Port
 	// on changes made by the provisioner.
 	vmName := resourceName(names.NewMachineTag(machineId))
 	prefix := instanceNetworkSecurityRulePrefix(instance.Id(vmName))
-	for _, ports := range ports {
-		ruleName := securityRuleName(prefix, ports)
+
+	singleSourceIngressRules := explodeIngressRules(rules)
+	for _, rule := range singleSourceIngressRules {
+		ruleName := securityRuleName(prefix, rule)
 		logger.Debugf("deleting security rule %q", ruleName)
 		var result autorest.Response
 		if err := inst.env.callAPI(func() (autorest.Response, error) {
@@ -334,8 +340,8 @@ func (inst *azureInstance) ClosePorts(machineId string, ports []jujunetwork.Port
 	return nil
 }
 
-// Ports is specified in the Instance interface.
-func (inst *azureInstance) Ports(machineId string) (ports []jujunetwork.PortRange, err error) {
+// IngressRules is specified in the Instance interface.
+func (inst *azureInstance) IngressRules(machineId string) (rules []jujunetwork.IngressRule, err error) {
 	nsgClient := network.SecurityGroupsClient{inst.env.network}
 	securityGroupName := internalSecurityGroupName
 	var nsg network.SecurityGroup
@@ -352,6 +358,9 @@ func (inst *azureInstance) Ports(machineId string) (ports []jujunetwork.PortRang
 
 	vmName := resourceName(names.NewMachineTag(machineId))
 	prefix := instanceNetworkSecurityRulePrefix(instance.Id(vmName))
+
+	// Keep track of all the SourceAddressPrefixes for each port range.
+	portSourceCIDRs := make(map[jujunetwork.PortRange]*[]string)
 	for _, rule := range *nsg.Properties.SecurityRules {
 		if rule.Properties.Direction != network.Inbound {
 			continue
@@ -391,12 +400,36 @@ func (inst *azureInstance) Ports(machineId string) (ports []jujunetwork.PortRang
 		default:
 			protocols = []string{"tcp", "udp"}
 		}
+
+		// Record the SourceAddressPrefix for the port range.
+		remotePrefix := to.String(rule.Properties.SourceAddressPrefix)
+		if remotePrefix == "" || remotePrefix == "*" {
+			remotePrefix = "0.0.0.0/0"
+		}
 		for _, protocol := range protocols {
 			portRange.Protocol = protocol
-			ports = append(ports, portRange)
+			sourceCIDRs, ok := portSourceCIDRs[portRange]
+			if !ok {
+				sourceCIDRs = &[]string{}
+				portSourceCIDRs[portRange] = sourceCIDRs
+			}
+			*sourceCIDRs = append(*sourceCIDRs, remotePrefix)
 		}
 	}
-	return ports, nil
+	// Combine all the port ranges and remote prefixes.
+	for portRange, sourceCIDRs := range portSourceCIDRs {
+		rule, err := jujunetwork.NewIngressRule(
+			portRange.Protocol,
+			portRange.FromPort,
+			portRange.ToPort,
+			*sourceCIDRs...)
+		if err != nil {
+			return nil, errors.Trace(err)
+		}
+		rules = append(rules, rule)
+	}
+	jujunetwork.SortIngressRules(rules)
+	return rules, nil
 }
 
 // deleteInstanceNetworkSecurityRules deletes network security rules in the
@@ -454,12 +487,42 @@ func instanceNetworkSecurityRulePrefix(id instance.Id) string {
 	return string(id) + "-"
 }
 
-// securityRuleName returns the security rule name for the given port range,
+// securityRuleName returns the security rule name for the given ingress rule,
 // and prefix returned by instanceNetworkSecurityRulePrefix.
-func securityRuleName(prefix string, ports jujunetwork.PortRange) string {
-	ruleName := fmt.Sprintf("%s%s-%d", prefix, ports.Protocol, ports.FromPort)
-	if ports.FromPort != ports.ToPort {
-		ruleName += fmt.Sprintf("-%d", ports.ToPort)
+func securityRuleName(prefix string, rule jujunetwork.IngressRule) string {
+	ruleName := fmt.Sprintf("%s%s-%d", prefix, rule.Protocol, rule.FromPort)
+	if rule.FromPort != rule.ToPort {
+		ruleName += fmt.Sprintf("-%d", rule.ToPort)
+	}
+	// The rule parameter must have a single source cidr.
+	// Ensure the rule name can be a valid URL path component.
+	cidr := rule.SourceCIDRs[0]
+	if cidr != "0.0.0.0/0" && cidr != "*" {
+		cidr = strings.Replace(cidr, ".", "-", -1)
+		cidr = strings.Replace(cidr, "::", "-", -1)
+		cidr = strings.Replace(cidr, "/", "-", -1)
+		ruleName = fmt.Sprintf("%s-cidr-%s", ruleName, cidr)
 	}
 	return ruleName
 }
+
+// explodeIngressRules creates a slice of ingress rules, each rule in the
+// result having a single source CIDR. The results contain a copy of each
+// specified rule with each copy having one of the source CIDR values,
+func explodeIngressRules(inRules jujunetwork.IngressRuleSlice) jujunetwork.IngressRuleSlice {
+	// If any rule has an empty source CIDR slice, a default
+	// source value of "*" is used.
+	var singleSourceIngressRules jujunetwork.IngressRuleSlice
+	for _, rule := range inRules {
+		sourceCIDRs := rule.SourceCIDRs
+		if len(sourceCIDRs) == 0 {
+			sourceCIDRs = []string{"*"}
+		}
+		for _, sr := range sourceCIDRs {
+			r := rule
+			r.SourceCIDRs = []string{sr}
+			singleSourceIngressRules = append(singleSourceIngressRules, r)
+		}
+	}
+	return singleSourceIngressRules
+}
