@@ -11,20 +11,20 @@ import "github.com/juju/juju/network"
 
 // OpenPorts opens the given ports for the whole environment.
 // Must only be used if the environment was setup with the FwGlobal firewall mode.
-func (env *environ) OpenPorts(ports []network.PortRange) error {
+func (env *environ) OpenPorts(ports []network.IngressRule) error {
 	logger.Debugf("pretending to open ports %v for all instances", ports)
 	return nil
 }
 
 // ClosePorts closes the given ports for the whole environment.
 // Must only be used if the environment was setup with the FwGlobal firewall mode.
-func (env *environ) ClosePorts(ports []network.PortRange) error {
+func (env *environ) ClosePorts(ports []network.IngressRule) error {
 	logger.Debugf("pretending to close ports %v for all instances", ports)
 	return nil
 }
 
-// Ports returns the ports opened for the whole environment.
+// IngressRules returns the ports opened for the whole environment.
 // Must only be used if the environment was setup with the FwGlobal firewall mode.
-func (env *environ) Ports() ([]network.PortRange, error) {
+func (env *environ) IngressRules() ([]network.IngressRule, error) {
 	return nil, nil
 }
