@@ -4,8 +4,10 @@
 package common
 
 import (
+	"bufio"
 	"fmt"
 	"io"
+	"io/ioutil"
 	"os"
 	"path"
 	"strings"
@@ -19,6 +21,7 @@ import (
 	"github.com/juju/utils/series"
 	"github.com/juju/utils/shell"
 	"github.com/juju/utils/ssh"
+	cryptossh "golang.org/x/crypto/ssh"
 
 	"github.com/juju/juju/agent"
 	"github.com/juju/juju/cloudconfig"
@@ -116,6 +119,7 @@ func BootstrapInstance(ctx environs.BootstrapContext, env environs.Environ, args
 	}
 	instanceConfig.EnableOSRefreshUpdate = env.Config().EnableOSRefreshUpdate()
 	instanceConfig.EnableOSUpgrade = env.Config().EnableOSUpgrade()
+	instanceConfig.NetBondReconfigureDelay = env.Config().NetBondReconfigureDelay()
 
 	instanceConfig.Tags = instancecfg.InstanceTags(envCfg.UUID(), args.ControllerConfig.ControllerUUID(), envCfg, instanceConfig.Jobs)
 	maybeSetBridge := func(icfg *instancecfg.InstanceConfig) {
@@ -132,10 +136,13 @@ func BootstrapInstance(ctx environs.BootstrapContext, env environs.Environ, args
 	}
 	maybeSetBridge(instanceConfig)
 
-	bootstrapMsg := env.BootstrapMessage()
-	if bootstrapMsg != "" {
-		ctx.Infof(bootstrapMsg)
+	// We're creating a new instance; inject host keys so that we can then
+	// make an SSH connection with known keys.
+	initialSSHHostKeys, err := generateSSHHostKeys()
+	if err != nil {
+		return nil, "", nil, errors.Annotate(err, "generating SSH host keys")
 	}
+	instanceConfig.Bootstrap.InitialSSHHostKeys = initialSSHHostKeys
 
 	cloudRegion := args.CloudName
 	if args.CloudRegion != "" {
@@ -189,6 +196,7 @@ func BootstrapInstance(ctx environs.BootstrapContext, env environs.Environ, args
 	finalize := func(ctx environs.BootstrapContext, icfg *instancecfg.InstanceConfig, opts environs.BootstrapDialOpts) error {
 		icfg.Bootstrap.BootstrapMachineInstanceId = result.Instance.Id()
 		icfg.Bootstrap.BootstrapMachineHardwareCharacteristics = result.Hardware
+		icfg.Bootstrap.InitialSSHHostKeys = initialSSHHostKeys
 		envConfig := env.Config()
 		if result.Config != nil {
 			updated, err := envConfig.Apply(result.Config.UnknownAttrs())
@@ -246,6 +254,8 @@ var FinishBootstrap = func(
 	interrupted := make(chan os.Signal, 1)
 	ctx.InterruptNotify(interrupted)
 	defer ctx.StopInterruptNotify(interrupted)
+
+	hostSSHOptions := bootstrapSSHOptionsFunc(instanceConfig)
 	addr, err := WaitSSH(
 		ctx.GetStderr(),
 		interrupted,
@@ -253,11 +263,19 @@ var FinishBootstrap = func(
 		GetCheckNonceCommand(instanceConfig),
 		&RefreshableInstance{inst, env},
 		opts,
+		hostSSHOptions,
 	)
 	if err != nil {
 		return err
 	}
-	return ConfigureMachine(ctx, client, addr, instanceConfig)
+
+	sshOptions, cleanup, err := hostSSHOptions(addr)
+	if err != nil {
+		return err
+	}
+	defer cleanup()
+
+	return ConfigureMachine(ctx, client, addr, instanceConfig, sshOptions)
 }
 
 func GetCheckNonceCommand(instanceConfig *instancecfg.InstanceConfig) string {
@@ -282,7 +300,13 @@ func GetCheckNonceCommand(instanceConfig *instancecfg.InstanceConfig) string {
 	return checkNonceCommand
 }
 
-func ConfigureMachine(ctx environs.BootstrapContext, client ssh.Client, host string, instanceConfig *instancecfg.InstanceConfig) error {
+func ConfigureMachine(
+	ctx environs.BootstrapContext,
+	client ssh.Client,
+	host string,
+	instanceConfig *instancecfg.InstanceConfig,
+	sshOptions *ssh.Options,
+) error {
 	// Bootstrap is synchronous, and will spawn a subprocess
 	// to complete the procedure. If the user hits Ctrl-C,
 	// SIGINT is sent to the foreground process attached to
@@ -313,12 +337,81 @@ func ConfigureMachine(ctx environs.BootstrapContext, client ssh.Client, host str
 	return sshinit.RunConfigureScript(script, sshinit.ConfigureParams{
 		Host:           "ubuntu@" + host,
 		Client:         client,
+		SSHOptions:     sshOptions,
 		Config:         cloudcfg,
 		ProgressWriter: ctx.GetStderr(),
 		Series:         instanceConfig.Series,
 	})
 }
 
+// HostSSHOptionsFunc is a function that, given a hostname, returns
+// an ssh.Options and a cleanup function, or an error.
+type HostSSHOptionsFunc func(host string) (*ssh.Options, func(), error)
+
+// DefaultHostSSHOptions returns a a nil *ssh.Options, which means
+// to use the defaults; and a no-op cleanup function.
+func DefaultHostSSHOptions(host string) (*ssh.Options, func(), error) {
+	return nil, func() {}, nil
+}
+
+// bootstrapSSHOptionsFunc that takes a bootstrap machine's InstanceConfig
+// and returns a HostSSHOptionsFunc.
+func bootstrapSSHOptionsFunc(instanceConfig *instancecfg.InstanceConfig) HostSSHOptionsFunc {
+	return func(host string) (*ssh.Options, func(), error) {
+		return hostBootstrapSSHOptions(host, instanceConfig)
+	}
+}
+
+func hostBootstrapSSHOptions(
+	host string,
+	instanceConfig *instancecfg.InstanceConfig,
+) (_ *ssh.Options, cleanup func(), err error) {
+	cleanup = func() {}
+	defer func() {
+		if err != nil {
+			cleanup()
+		}
+	}()
+
+	options := &ssh.Options{}
+	options.SetStrictHostKeyChecking(ssh.StrictHostChecksYes)
+
+	// If any host keys are being injected, we'll set up a
+	// known_hosts file with their contents, and accept only
+	// them.
+	hostKeys := instanceConfig.Bootstrap.InitialSSHHostKeys
+	var algos []string
+	var pubKeys []string
+	if hostKeys.RSA != nil {
+		algos = append(algos, cryptossh.KeyAlgoRSA)
+		pubKeys = append(pubKeys, hostKeys.RSA.Public)
+	}
+	if len(pubKeys) == 0 {
+		return options, cleanup, nil
+	}
+
+	// Create a temporary known_hosts file.
+	f, err := ioutil.TempFile("", "juju-known-hosts")
+	if err != nil {
+		return nil, cleanup, errors.Trace(err)
+	}
+	cleanup = func() {
+		f.Close()
+		os.RemoveAll(f.Name())
+	}
+	w := bufio.NewWriter(f)
+	for _, pubKey := range pubKeys {
+		fmt.Fprintln(w, host, strings.TrimSpace(pubKey))
+	}
+	if err := w.Flush(); err != nil {
+		return nil, cleanup, errors.Annotate(err, "writing known_hosts")
+	}
+
+	options.SetHostKeyAlgorithms(algos...)
+	options.SetKnownHostsFile(f.Name())
+	return options, cleanup, nil
+}
+
 // InstanceRefresher is the subet of the Instance interface required
 // for waiting for SSH access to become availble.
 type InstanceRefresher interface {
@@ -351,9 +444,10 @@ func (i *RefreshableInstance) Refresh() error {
 }
 
 type hostChecker struct {
-	addr   network.Address
-	client ssh.Client
-	wg     *sync.WaitGroup
+	addr           network.Address
+	client         ssh.Client
+	hostSSHOptions HostSSHOptionsFunc
+	wg             *sync.WaitGroup
 
 	// checkDelay is the amount of time to wait between retries.
 	checkDelay time.Duration
@@ -376,15 +470,22 @@ func (*hostChecker) Close() error {
 
 func (hc *hostChecker) loop(dying <-chan struct{}) (io.Closer, error) {
 	defer hc.wg.Done()
+
+	address := hc.addr.Value
+	sshOptions, cleanup, err := hc.hostSSHOptions(address)
+	if err != nil {
+		return nil, err
+	}
+	defer cleanup()
+
 	// The value of connectSSH is taken outside the goroutine that may outlive
 	// hostChecker.loop, or we evoke the wrath of the race detector.
 	connectSSH := connectSSH
-	done := make(chan error, 1)
 	var lastErr error
+	done := make(chan error, 1)
 	for {
-		address := hc.addr.Value
 		go func() {
-			done <- connectSSH(hc.client, address, hc.checkHostScript)
+			done <- connectSSH(hc.client, address, hc.checkHostScript, sshOptions)
 		}()
 		select {
 		case <-dying:
@@ -406,9 +507,10 @@ func (hc *hostChecker) loop(dying <-chan struct{}) (io.Closer, error) {
 
 type parallelHostChecker struct {
 	*parallel.Try
-	client ssh.Client
-	stderr io.Writer
-	wg     sync.WaitGroup
+	client         ssh.Client
+	hostSSHOptions HostSSHOptionsFunc
+	stderr         io.Writer
+	wg             sync.WaitGroup
 
 	// active is a map of adresses to channels for addresses actively
 	// being tested. The goroutine testing the address will continue
@@ -434,6 +536,7 @@ func (p *parallelHostChecker) UpdateAddresses(addrs []network.Address) {
 		hc := &hostChecker{
 			addr:            addr,
 			client:          p.client,
+			hostSSHOptions:  p.hostSSHOptions,
 			checkDelay:      p.checkDelay,
 			checkHostScript: p.checkHostScript,
 			closed:          closed,
@@ -461,8 +564,8 @@ func (p *parallelHostChecker) Close() error {
 
 // connectSSH is called to connect to the specified host and
 // execute the "checkHostScript" bash script on it.
-var connectSSH = func(client ssh.Client, host, checkHostScript string) error {
-	cmd := client.Command("ubuntu@"+host, []string{"/bin/bash"}, nil)
+var connectSSH = func(client ssh.Client, host, checkHostScript string, options *ssh.Options) error {
+	cmd := client.Command("ubuntu@"+host, []string{"/bin/bash"}, options)
 	cmd.Stdin = strings.NewReader(checkHostScript)
 	output, err := cmd.CombinedOutput()
 	if err != nil && len(output) > 0 {
@@ -487,6 +590,7 @@ func WaitSSH(
 	checkHostScript string,
 	inst InstanceRefresher,
 	opts environs.BootstrapDialOpts,
+	hostSSHOptions HostSSHOptionsFunc,
 ) (addr string, err error) {
 	globalTimeout := time.After(opts.Timeout)
 	pollAddresses := time.NewTimer(0)
@@ -501,6 +605,7 @@ func WaitSSH(
 		active:          make(map[network.Address]chan struct{}),
 		checkDelay:      opts.RetryDelay,
 		checkHostScript: checkHostScript,
+		hostSSHOptions:  hostSSHOptions,
 	}
 	defer checker.wg.Wait()
 	defer checker.Kill()
@@ -551,3 +656,18 @@ func WaitSSH(
 		}
 	}
 }
+
+func generateSSHHostKeys() (instancecfg.SSHHostKeys, error) {
+	// Generate a single ssh-rsa key. We'll configure the SSH client
+	// such that that is the only host key type we'll accept.
+	var keys instancecfg.SSHHostKeys
+	private, public, err := ssh.GenerateKey("juju-bootstrap")
+	if err != nil {
+		return keys, errors.Annotate(err, "generating SSH key")
+	}
+	keys.RSA = &instancecfg.SSHKeyPair{
+		Private: private,
+		Public:  public,
+	}
+	return keys, nil
+}
