@@ -4,18 +4,24 @@
 package common_test
 
 import (
-	"bytes"
+	"crypto/rsa"
 	"fmt"
+	"io/ioutil"
 	"os"
+	"regexp"
 	"strings"
+	"sync"
 	"time"
 
+	"github.com/juju/cmd/cmdtesting"
 	"github.com/juju/errors"
+	"github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils/arch"
 	"github.com/juju/utils/series"
 	"github.com/juju/utils/ssh"
 	"github.com/juju/version"
+	cryptossh "golang.org/x/crypto/ssh"
 	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/cloudconfig/instancecfg"
@@ -48,7 +54,7 @@ type cleaner interface {
 func (s *BootstrapSuite) SetUpTest(c *gc.C) {
 	s.FakeJujuXDGDataHomeSuite.SetUpTest(c)
 	s.ToolsFixture.SetUpTest(c)
-	s.PatchValue(common.ConnectSSH, func(_ ssh.Client, host, checkHostScript string) error {
+	s.PatchValue(common.ConnectSSH, func(_ ssh.Client, host, checkHostScript string, opts *ssh.Options) error {
 		return fmt.Errorf("mock connection failure to %s", host)
 	})
 }
@@ -74,7 +80,7 @@ func minimalConfig(c *gc.C) *config.Config {
 		"ca-cert":         coretesting.CACert,
 		"ca-private-key":  coretesting.CAKey,
 		"authorized-keys": coretesting.FakeAuthKeys,
-		"default-series":  series.HostSeries(),
+		"default-series":  series.MustHostSeries(),
 	}
 	cfg, err := config.New(config.UseDefaults, attrs)
 	c.Assert(err, jc.ErrorIsNil)
@@ -116,6 +122,8 @@ func (s *BootstrapSuite) TestCannotStartInstance(c *gc.C) {
 			"juju-controller-uuid": coretesting.ControllerTag.Id(),
 			"juju-is-controller":   "true",
 		}
+		expectedMcfg.NetBondReconfigureDelay = env.Config().NetBondReconfigureDelay()
+		icfg.Bootstrap.InitialSSHHostKeys.RSA = nil
 
 		c.Assert(icfg, jc.DeepEquals, expectedMcfg)
 		return nil, nil, nil, errors.Errorf("meh, not started")
@@ -134,7 +142,7 @@ func (s *BootstrapSuite) TestCannotStartInstance(c *gc.C) {
 				Version: version.Binary{
 					Number: jujuversion.Current,
 					Arch:   arch.HostArch(),
-					Series: series.HostSeries(),
+					Series: series.MustHostSeries(),
 				},
 			},
 		}})
@@ -143,7 +151,7 @@ func (s *BootstrapSuite) TestCannotStartInstance(c *gc.C) {
 
 func (s *BootstrapSuite) TestBootstrapSeries(c *gc.C) {
 	s.PatchValue(&jujuversion.Current, coretesting.FakeVersionNumber)
-	s.PatchValue(&series.HostSeries, func() string { return "precise" })
+	s.PatchValue(&series.MustHostSeries, func() string { return "precise" })
 	stor := newStorage(s, c)
 	checkInstanceId := "i-success"
 	checkHardware := instance.MustParseHardware("arch=ppc64el mem=2T")
@@ -194,12 +202,25 @@ func (s *BootstrapSuite) TestSuccess(c *gc.C) {
 	checkInstanceId := "i-success"
 	checkHardware := instance.MustParseHardware("arch=ppc64el mem=2T")
 
+	var innerInstanceConfig *instancecfg.InstanceConfig
+	inst := &mockInstance{
+		id:        checkInstanceId,
+		addresses: network.NewAddresses("testing.invalid"),
+	}
 	startInstance := func(
-		_ string, _ constraints.Value, _ []string, _ tools.List, icfg *instancecfg.InstanceConfig,
+		_ string, _ constraints.Value, _ []string, tools tools.List, icfg *instancecfg.InstanceConfig,
 	) (
 		instance.Instance, *instance.HardwareCharacteristics, []network.InterfaceInfo, error,
 	) {
-		return &mockInstance{id: checkInstanceId}, &checkHardware, nil, nil
+		innerInstanceConfig = icfg
+		c.Assert(icfg.Bootstrap.InitialSSHHostKeys.RSA, gc.NotNil)
+		privKey, err := cryptossh.ParseRawPrivateKey([]byte(icfg.Bootstrap.InitialSSHHostKeys.RSA.Private))
+		c.Assert(err, jc.ErrorIsNil)
+		c.Assert(privKey, gc.FitsTypeOf, &rsa.PrivateKey{})
+		pubKey, _, _, _, err := cryptossh.ParseAuthorizedKey([]byte(icfg.Bootstrap.InitialSSHHostKeys.RSA.Public))
+		c.Assert(err, jc.ErrorIsNil)
+		c.Assert(pubKey.Type(), gc.Equals, cryptossh.KeyAlgoRSA)
+		return inst, &checkHardware, nil, nil
 	}
 	var mocksConfig = minimalConfig(c)
 	var getConfigCalled int
@@ -212,13 +233,19 @@ func (s *BootstrapSuite) TestSuccess(c *gc.C) {
 		return nil
 	}
 
+	var instancesMu sync.Mutex
 	env := &mockEnviron{
 		storage:       stor,
 		startInstance: startInstance,
 		config:        getConfig,
 		setConfig:     setConfig,
+		instances: func(ids []instance.Id) ([]instance.Instance, error) {
+			instancesMu.Lock()
+			defer instancesMu.Unlock()
+			return []instance.Instance{inst}, nil
+		},
 	}
-	inner := coretesting.Context(c)
+	inner := cmdtesting.Context(c)
 	ctx := modelcmd.BootstrapContext(inner)
 	result, err := common.Bootstrap(ctx, env, environs.BootstrapParams{
 		ControllerConfig: coretesting.FakeControllerConfig(),
@@ -227,17 +254,58 @@ func (s *BootstrapSuite) TestSuccess(c *gc.C) {
 				Version: version.Binary{
 					Number: jujuversion.Current,
 					Arch:   arch.HostArch(),
-					Series: series.HostSeries(),
+					Series: series.MustHostSeries(),
 				},
 			},
 		}})
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(result.Arch, gc.Equals, "ppc64el") // based on hardware characteristics
 	c.Assert(result.Series, gc.Equals, config.PreferredSeries(mocksConfig))
-	output := inner.Stderr.(*bytes.Buffer)
-	lines := strings.Split(output.String(), "\n")
-	c.Assert(len(lines), jc.GreaterThan, 1)
-	c.Assert(lines[0], gc.Equals, "Some message")
+	c.Assert(result.Finalize, gc.NotNil)
+
+	// Check that we make the SSH connection with desired options.
+	var knownHosts string
+	re := regexp.MustCompile(
+		"ssh '-o' 'StrictHostKeyChecking yes' " +
+			"'-o' 'PasswordAuthentication no' " +
+			"'-o' 'ServerAliveInterval 30' " +
+			"'-o' 'UserKnownHostsFile (.*)' " +
+			"'-o' 'HostKeyAlgorithms ssh-rsa' " +
+			"'ubuntu@testing.invalid' '/bin/bash'")
+	testing.PatchExecutableAsEchoArgs(c, s, "ssh")
+	testing.PatchExecutableAsEchoArgs(c, s, "scp")
+	s.PatchValue(common.ConnectSSH, func(_ ssh.Client, host, checkHostScript string, opts *ssh.Options) error {
+		// Stop WaitSSH from continuing.
+		client, err := ssh.NewOpenSSHClient()
+		if err != nil {
+			return err
+		}
+		cmd := client.Command("ubuntu@"+host, []string{"/bin/bash"}, opts)
+		if err := cmd.Run(); err != nil {
+			return err
+		}
+		sshArgs := testing.ReadEchoArgs(c, "ssh")
+		submatch := re.FindStringSubmatch(sshArgs)
+		if c.Check(submatch, gc.NotNil, gc.Commentf("%s", sshArgs)) {
+			knownHostsFile := submatch[1]
+			knownHostsFile = strings.Replace(knownHostsFile, `\"`, ``, -1)
+			knownHostsBytes, err := ioutil.ReadFile(knownHostsFile)
+			if err != nil {
+				return err
+			}
+			knownHosts = string(knownHostsBytes)
+		}
+		return nil
+	})
+	err = result.Finalize(ctx, innerInstanceConfig, environs.BootstrapDialOpts{
+		Timeout: coretesting.LongWait,
+	})
+	c.Assert(err, gc.ErrorMatches, "invalid machine configuration: .*") // icfg hasn't been finalized
+	c.Assert(
+		string(knownHosts),
+		gc.Equals,
+		"testing.invalid "+innerInstanceConfig.Bootstrap.InitialSSHHostKeys.RSA.Public,
+	)
 }
 
 type neverRefreshes struct {
@@ -278,26 +346,38 @@ var testSSHTimeout = environs.BootstrapDialOpts{
 }
 
 func (s *BootstrapSuite) TestWaitSSHTimesOutWaitingForAddresses(c *gc.C) {
-	ctx := coretesting.Context(c)
-	_, err := common.WaitSSH(ctx.Stderr, nil, ssh.DefaultClient, "/bin/true", neverAddresses{}, testSSHTimeout)
+	ctx := cmdtesting.Context(c)
+	_, err := common.WaitSSH(
+		ctx.Stderr, nil, ssh.DefaultClient, "/bin/true", neverAddresses{}, testSSHTimeout,
+		common.DefaultHostSSHOptions,
+	)
 	c.Check(err, gc.ErrorMatches, `waited for `+testSSHTimeout.Timeout.String()+` without getting any addresses`)
-	c.Check(coretesting.Stderr(ctx), gc.Matches, "Waiting for address\n")
+	c.Check(cmdtesting.Stderr(ctx), gc.Matches, "Waiting for address\n")
 }
 
 func (s *BootstrapSuite) TestWaitSSHKilledWaitingForAddresses(c *gc.C) {
-	ctx := coretesting.Context(c)
+	ctx := cmdtesting.Context(c)
 	interrupted := make(chan os.Signal, 1)
 	interrupted <- os.Interrupt
-	_, err := common.WaitSSH(ctx.Stderr, interrupted, ssh.DefaultClient, "/bin/true", neverAddresses{}, testSSHTimeout)
+	_, err := common.WaitSSH(
+		ctx.Stderr, interrupted, ssh.DefaultClient, "/bin/true", neverAddresses{}, testSSHTimeout,
+		common.DefaultHostSSHOptions,
+	)
 	c.Check(err, gc.ErrorMatches, "interrupted")
-	c.Check(coretesting.Stderr(ctx), gc.Matches, "Waiting for address\n")
+	c.Check(cmdtesting.Stderr(ctx), gc.Matches, "Waiting for address\n")
 }
 
 func (s *BootstrapSuite) TestWaitSSHNoticesProvisioningFailures(c *gc.C) {
-	ctx := coretesting.Context(c)
-	_, err := common.WaitSSH(ctx.Stderr, nil, ssh.DefaultClient, "/bin/true", failsProvisioning{}, testSSHTimeout)
+	ctx := cmdtesting.Context(c)
+	_, err := common.WaitSSH(
+		ctx.Stderr, nil, ssh.DefaultClient, "/bin/true", failsProvisioning{}, testSSHTimeout,
+		common.DefaultHostSSHOptions,
+	)
 	c.Check(err, gc.ErrorMatches, `instance provisioning failed`)
-	_, err = common.WaitSSH(ctx.Stderr, nil, ssh.DefaultClient, "/bin/true", failsProvisioning{message: "blargh"}, testSSHTimeout)
+	_, err = common.WaitSSH(
+		ctx.Stderr, nil, ssh.DefaultClient, "/bin/true", failsProvisioning{message: "blargh"}, testSSHTimeout,
+		common.DefaultHostSSHOptions,
+	)
 	c.Check(err, gc.ErrorMatches, `instance provisioning failed \(blargh\)`)
 }
 
@@ -310,10 +390,13 @@ func (brokenAddresses) Addresses() ([]network.Address, error) {
 }
 
 func (s *BootstrapSuite) TestWaitSSHStopsOnBadError(c *gc.C) {
-	ctx := coretesting.Context(c)
-	_, err := common.WaitSSH(ctx.Stderr, nil, ssh.DefaultClient, "/bin/true", brokenAddresses{}, testSSHTimeout)
+	ctx := cmdtesting.Context(c)
+	_, err := common.WaitSSH(
+		ctx.Stderr, nil, ssh.DefaultClient, "/bin/true", brokenAddresses{}, testSSHTimeout,
+		common.DefaultHostSSHOptions,
+	)
 	c.Check(err, gc.ErrorMatches, "getting addresses: Addresses will never work")
-	c.Check(coretesting.Stderr(ctx), gc.Equals, "Waiting for address\n")
+	c.Check(cmdtesting.Stderr(ctx), gc.Equals, "Waiting for address\n")
 }
 
 type neverOpensPort struct {
@@ -326,12 +409,15 @@ func (n *neverOpensPort) Addresses() ([]network.Address, error) {
 }
 
 func (s *BootstrapSuite) TestWaitSSHTimesOutWaitingForDial(c *gc.C) {
-	ctx := coretesting.Context(c)
+	ctx := cmdtesting.Context(c)
 	// 0.x.y.z addresses are always invalid
-	_, err := common.WaitSSH(ctx.Stderr, nil, ssh.DefaultClient, "/bin/true", &neverOpensPort{addr: "0.1.2.3"}, testSSHTimeout)
+	_, err := common.WaitSSH(
+		ctx.Stderr, nil, ssh.DefaultClient, "/bin/true", &neverOpensPort{addr: "0.1.2.3"}, testSSHTimeout,
+		common.DefaultHostSSHOptions,
+	)
 	c.Check(err, gc.ErrorMatches,
 		`waited for `+testSSHTimeout.Timeout.String()+` without being able to connect: mock connection failure to 0.1.2.3`)
-	c.Check(coretesting.Stderr(ctx), gc.Matches,
+	c.Check(cmdtesting.Stderr(ctx), gc.Matches,
 		"Waiting for address\n"+
 			"(Attempting to connect to 0.1.2.3:22\n)+")
 }
@@ -354,14 +440,17 @@ func (i *interruptOnDial) Addresses() ([]network.Address, error) {
 }
 
 func (s *BootstrapSuite) TestWaitSSHKilledWaitingForDial(c *gc.C) {
-	ctx := coretesting.Context(c)
+	ctx := cmdtesting.Context(c)
 	timeout := testSSHTimeout
 	timeout.Timeout = 1 * time.Minute
 	interrupted := make(chan os.Signal, 1)
-	_, err := common.WaitSSH(ctx.Stderr, interrupted, ssh.DefaultClient, "", &interruptOnDial{name: "0.1.2.3", interrupted: interrupted}, timeout)
+	_, err := common.WaitSSH(
+		ctx.Stderr, interrupted, ssh.DefaultClient, "", &interruptOnDial{name: "0.1.2.3", interrupted: interrupted}, timeout,
+		common.DefaultHostSSHOptions,
+	)
 	c.Check(err, gc.ErrorMatches, "interrupted")
 	// Exact timing is imprecise but it should have tried a few times before being killed
-	c.Check(coretesting.Stderr(ctx), gc.Matches,
+	c.Check(cmdtesting.Stderr(ctx), gc.Matches,
 		"Waiting for address\n"+
 			"(Attempting to connect to 0.1.2.3:22\n)+")
 }
@@ -386,7 +475,8 @@ func (ac *addressesChange) Addresses() ([]network.Address, error) {
 }
 
 func (s *BootstrapSuite) TestWaitSSHRefreshAddresses(c *gc.C) {
-	ctx := coretesting.Context(c)
+	coretesting.SkipIfWindowsBug(c, "lp:1604961")
+	ctx := cmdtesting.Context(c)
 	_, err := common.WaitSSH(ctx.Stderr, nil, ssh.DefaultClient, "", &addressesChange{addrs: [][]string{
 		nil,
 		nil,
@@ -394,11 +484,11 @@ func (s *BootstrapSuite) TestWaitSSHRefreshAddresses(c *gc.C) {
 		{"0.1.2.3"},
 		nil,
 		{"0.1.2.4"},
-	}}, testSSHTimeout)
+	}}, testSSHTimeout, common.DefaultHostSSHOptions)
 	// Not necessarily the last one in the list, due to scheduling.
 	c.Check(err, gc.ErrorMatches,
 		`waited for `+testSSHTimeout.Timeout.String()+` without being able to connect: mock connection failure to 0.1.2.[34]`)
-	stderr := coretesting.Stderr(ctx)
+	stderr := cmdtesting.Stderr(ctx)
 	c.Check(stderr, gc.Matches,
 		"Waiting for address\n"+
 			"(.|\n)*(Attempting to connect to 0.1.2.3:22\n)+(.|\n)*")
