@@ -20,7 +20,9 @@ package dummy
 
 import (
 	"fmt"
+	"io"
 	"net"
+	"net/http"
 	"os"
 	"runtime"
 	"strconv"
@@ -29,6 +31,7 @@ import (
 	"time"
 
 	"github.com/juju/errors"
+	"github.com/juju/jsonschema"
 	"github.com/juju/loggo"
 	"github.com/juju/retry"
 	"github.com/juju/schema"
@@ -37,6 +40,7 @@ import (
 	"github.com/juju/utils/arch"
 	"github.com/juju/utils/clock"
 	"github.com/juju/utils/series"
+	"github.com/juju/version"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/environschema.v1"
 	"gopkg.in/juju/names.v2"
@@ -56,6 +60,7 @@ import (
 	"github.com/juju/juju/mongo/mongotest"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/provider/common"
+	"github.com/juju/juju/pubsub/centralhub"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/state/multiwatcher"
 	"github.com/juju/juju/state/stateenvirons"
@@ -126,8 +131,9 @@ func stateInfo() *mongo.MongoInfo {
 	addrs := []string{net.JoinHostPort("localhost", mongoPort)}
 	return &mongo.MongoInfo{
 		Info: mongo.Info{
-			Addrs:  addrs,
-			CACert: testing.CACert,
+			Addrs:      addrs,
+			CACert:     testing.CACert,
+			DisableTLS: !gitjujutesting.MgoServer.SSLEnabled(),
 		},
 	}
 }
@@ -193,14 +199,14 @@ type OpOpenPorts struct {
 	Env        string
 	MachineId  string
 	InstanceId instance.Id
-	Ports      []network.PortRange
+	Rules      []network.IngressRule
 }
 
 type OpClosePorts struct {
 	Env        string
 	MachineId  string
 	InstanceId instance.Id
-	Ports      []network.PortRange
+	Rules      []network.IngressRule
 }
 
 type OpPutFile struct {
@@ -237,7 +243,7 @@ type environState struct {
 	maxId          int // maximum instance id allocated so far.
 	maxAddr        int // maximum allocated address last byte
 	insts          map[instance.Id]*dummyInstance
-	globalPorts    map[network.PortRange]bool
+	globalRules    network.IngressRuleSlice
 	bootstrapped   bool
 	apiListener    net.Listener
 	apiServer      *apiserver.Server
@@ -414,7 +420,6 @@ func newState(name string, ops chan<- Operation, newStatePolicy state.NewPolicyF
 		ops:            ops,
 		newStatePolicy: newStatePolicy,
 		insts:          make(map[instance.Id]*dummyInstance),
-		globalPorts:    make(map[network.PortRange]bool),
 		creator:        string(buf),
 	}
 	return s
@@ -532,17 +537,17 @@ var _ config.ConfigSchemaSource = (*environProvider)(nil)
 
 // ConfigSchema returns extra config attributes specific
 // to this provider only.
-func (p environProvider) ConfigSchema() schema.Fields {
+func (p *environProvider) ConfigSchema() schema.Fields {
 	return configFields
 }
 
 // ConfigDefaults returns the default values for the
 // provider specific config attributes.
-func (p environProvider) ConfigDefaults() schema.Defaults {
+func (p *environProvider) ConfigDefaults() schema.Defaults {
 	return configDefaults
 }
 
-func (environProvider) CredentialSchemas() map[cloud.AuthType]cloud.CredentialSchema {
+func (*environProvider) CredentialSchemas() map[cloud.AuthType]cloud.CredentialSchema {
 	return map[cloud.AuthType]cloud.CredentialSchema{
 		cloud.EmptyAuthType: {},
 		cloud.UserPassAuthType: {
@@ -593,6 +598,11 @@ func (e *environ) state() (*environState, error) {
 	return state, nil
 }
 
+// Version is part of the EnvironProvider interface.
+func (*environProvider) Version() int {
+	return 0
+}
+
 func (p *environProvider) Open(args environs.OpenParams) (environs.Environ, error) {
 	p.mu.Lock()
 	defer p.mu.Unlock()
@@ -613,6 +623,17 @@ func (p *environProvider) Open(args environs.OpenParams) (environs.Environ, erro
 	return env, nil
 }
 
+// CloudSchema returns the schema used to validate input for add-cloud.  Since
+// this provider does not support custom clouds, this always returns nil.
+func (p *environProvider) CloudSchema() *jsonschema.Schema {
+	return nil
+}
+
+// Ping tests the connection to the cloud, to verify the endpoint is valid.
+func (p *environProvider) Ping(endpoint string) error {
+	return errors.NotImplementedf("Ping")
+}
+
 // PrepareConfig is specified in the EnvironProvider interface.
 func (p *environProvider) PrepareConfig(args environs.PrepareConfigParams) (*config.Config, error) {
 	if _, err := dummy.newConfig(args.Config); err != nil {
@@ -699,7 +720,7 @@ func (e *environ) Bootstrap(ctx environs.BootstrapContext, args environs.Bootstr
 		return nil, errors.New("no CA certificate in controller configuration")
 	}
 
-	logger.Infof("would pick tools from %s", availableTools)
+	logger.Infof("would pick agent binaries from %s", availableTools)
 
 	estate, err := e.state()
 	if err != nil {
@@ -716,7 +737,6 @@ func (e *environ) Bootstrap(ctx environs.BootstrapContext, args environs.Bootstr
 	i := &dummyInstance{
 		id:           BootstrapInstanceId,
 		addresses:    network.NewAddresses("localhost"),
-		ports:        make(map[network.PortRange]bool),
 		machineId:    agent.BootstrapMachineId,
 		series:       series,
 		firewallMode: e.Config().FirewallMode(),
@@ -739,7 +759,7 @@ func (e *environ) Bootstrap(ctx environs.BootstrapContext, args environs.Bootstr
 			if icfg.Bootstrap.ControllerCloudCredentialName != "" {
 				cloudCredentialTag = names.NewCloudCredentialTag(fmt.Sprintf(
 					"%s/%s/%s",
-					icfg.Bootstrap.ControllerCloudName,
+					icfg.Bootstrap.ControllerCloud.Name,
 					adminUser.Id(),
 					icfg.Bootstrap.ControllerCloudCredentialName,
 				))
@@ -762,13 +782,12 @@ func (e *environ) Bootstrap(ctx environs.BootstrapContext, args environs.Bootstr
 					Owner:                   adminUser,
 					Config:                  icfg.Bootstrap.ControllerModelConfig,
 					Constraints:             icfg.Bootstrap.BootstrapMachineConstraints,
-					CloudName:               icfg.Bootstrap.ControllerCloudName,
+					CloudName:               icfg.Bootstrap.ControllerCloud.Name,
 					CloudRegion:             icfg.Bootstrap.ControllerCloudRegion,
 					CloudCredential:         cloudCredentialTag,
 					StorageProviderRegistry: e,
 				},
 				Cloud:            icfg.Bootstrap.ControllerCloud,
-				CloudName:        icfg.Bootstrap.ControllerCloudName,
 				CloudCredentials: cloudCredentials,
 				MongoInfo:        info,
 				MongoDialOpts:    mongotest.DialOpts(),
@@ -778,20 +797,25 @@ func (e *environ) Bootstrap(ctx environs.BootstrapContext, args environs.Bootstr
 				return err
 			}
 			if err := st.SetModelConstraints(args.ModelConstraints); err != nil {
+				st.Close()
 				return err
 			}
 			if err := st.SetAdminMongoPassword(icfg.Controller.MongoInfo.Password); err != nil {
+				st.Close()
 				return err
 			}
 			if err := st.MongoSession().DB("admin").Login("admin", icfg.Controller.MongoInfo.Password); err != nil {
+				st.Close()
 				return err
 			}
 			env, err := st.Model()
 			if err != nil {
+				st.Close()
 				return err
 			}
 			owner, err := st.User(env.Owner())
 			if err != nil {
+				st.Close()
 				return err
 			}
 			// We log this out for test purposes only. No one in real life can use
@@ -800,24 +824,33 @@ func (e *environ) Bootstrap(ctx environs.BootstrapContext, args environs.Bootstr
 			logger.Debugf("setting password for %q to %q", owner.Name(), icfg.Controller.MongoInfo.Password)
 			owner.SetPassword(icfg.Controller.MongoInfo.Password)
 
-			estate.apiStatePool = state.NewStatePool(st)
-
-			estate.apiServer, err = apiserver.NewServer(st, estate.apiListener, apiserver.ServerConfig{
+			statePool := state.NewStatePool(st)
+			machineTag := names.NewMachineTag("0")
+			estate.apiServer, err = apiserver.NewServer(statePool, estate.apiListener, apiserver.ServerConfig{
 				Clock:       clock.WallClock,
 				Cert:        testing.ServerCert,
 				Key:         testing.ServerKey,
-				Tag:         names.NewMachineTag("0"),
+				Tag:         machineTag,
 				DataDir:     DataDir,
 				LogDir:      LogDir,
-				StatePool:   estate.apiStatePool,
+				Hub:         centralhub.New(machineTag),
 				NewObserver: func() observer.Observer { return &fakeobserver.Instance{} },
 				// Should never be used but prevent external access just in case.
 				AutocertURL: "https://0.1.2.3/no-autocert-here",
+				RegisterIntrospectionHandlers: func(f func(path string, h http.Handler)) {
+					f("navel", http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+						io.WriteString(w, "gazing")
+					}))
+				},
+				RateLimitConfig: apiserver.DefaultRateLimitConfig(),
 			})
 			if err != nil {
+				statePool.Close()
+				st.Close()
 				panic(err)
 			}
 			estate.apiState = st
+			estate.apiStatePool = statePool
 		}
 		estate.ops <- OpFinalizeBootstrap{Context: ctx, Env: e.name, InstanceConfig: icfg}
 		return nil
@@ -831,11 +864,6 @@ func (e *environ) Bootstrap(ctx environs.BootstrapContext, args environs.Bootstr
 	return bsResult, nil
 }
 
-// BootstrapMessage is part of the Environ interface.
-func (e *environ) BootstrapMessage() string {
-	return ""
-}
-
 func (e *environ) ControllerInstances(controllerUUID string) ([]instance.Id, error) {
 	estate, err := e.state()
 	if err != nil {
@@ -876,6 +904,12 @@ func (e *environ) SetConfig(cfg *config.Config) error {
 	return nil
 }
 
+// AdoptResources is part of the Environ interface.
+func (e *environ) AdoptResources(controllerUUID string, fromVersion version.Number) error {
+	// This provider doesn't track instance -> controller.
+	return nil
+}
+
 func (e *environ) Destroy() (res error) {
 	defer delay()
 	estate, err := e.state()
@@ -970,16 +1004,17 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (*environs.St
 	if args.InstanceConfig.APIInfo.Tag != names.NewMachineTag(machineId) {
 		return nil, errors.New("entity tag must match started machine")
 	}
-	logger.Infof("would pick tools from %s", args.Tools)
+	logger.Infof("would pick agent binaries from %s", args.Tools)
 	series := args.Tools.OneSeries()
 
 	idString := fmt.Sprintf("%s-%d", e.name, estate.maxId)
-	addrs := network.NewAddresses(idString+".dns", "127.0.0.1")
+	// Add the addresses we want to see in the machine doc. This means both
+	// IPv4 and IPv6 loopback, as well as the DNS name.
+	addrs := network.NewAddresses(idString+".dns", "127.0.0.1", "::1")
 	logger.Debugf("StartInstance addresses: %v", addrs)
 	i := &dummyInstance{
 		id:           instance.Id(idString),
 		addresses:    addrs,
-		ports:        make(map[network.PortRange]bool),
 		machineId:    machineId,
 		series:       series,
 		firewallMode: e.Config().FirewallMode(),
@@ -1146,6 +1181,11 @@ func (env *environ) SupportsSpaceDiscovery() (bool, error) {
 	return true, nil
 }
 
+// SupportsContainerAddresses is specified on environs.Networking.
+func (env *environ) SupportsContainerAddresses() (bool, error) {
+	return false, errors.NotSupportedf("container addresses")
+}
+
 // Spaces is specified on environs.Networking.
 func (env *environ) Spaces() ([]network.SpaceInfo, error) {
 	if err := env.checkBroken("Spaces"); err != nil {
@@ -1370,7 +1410,7 @@ func (e *environ) AllInstances() ([]instance.Instance, error) {
 	return insts, nil
 }
 
-func (e *environ) OpenPorts(ports []network.PortRange) error {
+func (e *environ) OpenPorts(rules []network.IngressRule) error {
 	if mode := e.ecfg().FirewallMode(); mode != config.FwGlobal {
 		return fmt.Errorf("invalid firewall mode %q for opening ports on model", mode)
 	}
@@ -1380,13 +1420,25 @@ func (e *environ) OpenPorts(ports []network.PortRange) error {
 	}
 	estate.mu.Lock()
 	defer estate.mu.Unlock()
-	for _, p := range ports {
-		estate.globalPorts[p] = true
+	for _, r := range rules {
+		if len(r.SourceCIDRs) == 0 {
+			r.SourceCIDRs = []string{"0.0.0.0/0"}
+		}
+		found := false
+		for _, rule := range estate.globalRules {
+			if r.String() == rule.String() {
+				found = true
+			}
+		}
+		if !found {
+			estate.globalRules = append(estate.globalRules, r)
+		}
 	}
+
 	return nil
 }
 
-func (e *environ) ClosePorts(ports []network.PortRange) error {
+func (e *environ) ClosePorts(rules []network.IngressRule) error {
 	if mode := e.ecfg().FirewallMode(); mode != config.FwGlobal {
 		return fmt.Errorf("invalid firewall mode %q for closing ports on model", mode)
 	}
@@ -1396,15 +1448,19 @@ func (e *environ) ClosePorts(ports []network.PortRange) error {
 	}
 	estate.mu.Lock()
 	defer estate.mu.Unlock()
-	for _, p := range ports {
-		delete(estate.globalPorts, p)
+	for _, r := range rules {
+		for i, rule := range estate.globalRules {
+			if r.String() == rule.String() {
+				estate.globalRules = estate.globalRules[:i+copy(estate.globalRules[i:], estate.globalRules[i+1:])]
+			}
+		}
 	}
 	return nil
 }
 
-func (e *environ) Ports() (ports []network.PortRange, err error) {
+func (e *environ) IngressRules() (rules []network.IngressRule, err error) {
 	if mode := e.ecfg().FirewallMode(); mode != config.FwGlobal {
-		return nil, fmt.Errorf("invalid firewall mode %q for retrieving ports from model", mode)
+		return nil, fmt.Errorf("invalid firewall mode %q for retrieving ingress rules from model", mode)
 	}
 	estate, err := e.state()
 	if err != nil {
@@ -1412,10 +1468,10 @@ func (e *environ) Ports() (ports []network.PortRange, err error) {
 	}
 	estate.mu.Lock()
 	defer estate.mu.Unlock()
-	for p := range estate.globalPorts {
-		ports = append(ports, p)
+	for _, r := range estate.globalRules {
+		rules = append(rules, r)
 	}
-	network.SortPortRanges(ports)
+	network.SortIngressRules(rules)
 	return
 }
 
@@ -1425,7 +1481,7 @@ func (*environ) Provider() environs.EnvironProvider {
 
 type dummyInstance struct {
 	state        *environState
-	ports        map[network.PortRange]bool
+	rules        network.IngressRuleSlice
 	id           instance.Id
 	status       string
 	machineId    string
@@ -1507,9 +1563,9 @@ func (inst *dummyInstance) Addresses() ([]network.Address, error) {
 	return append([]network.Address{}, inst.addresses...), nil
 }
 
-func (inst *dummyInstance) OpenPorts(machineId string, ports []network.PortRange) error {
+func (inst *dummyInstance) OpenPorts(machineId string, rules []network.IngressRule) error {
 	defer delay()
-	logger.Infof("openPorts %s, %#v", machineId, ports)
+	logger.Infof("openPorts %s, %#v", machineId, rules)
 	if inst.firewallMode != config.FwInstance {
 		return fmt.Errorf("invalid firewall mode %q for opening ports on instance",
 			inst.firewallMode)
@@ -1526,15 +1582,33 @@ func (inst *dummyInstance) OpenPorts(machineId string, ports []network.PortRange
 		Env:        inst.state.name,
 		MachineId:  machineId,
 		InstanceId: inst.Id(),
-		Ports:      ports,
+		Rules:      rules,
 	}
-	for _, p := range ports {
-		inst.ports[p] = true
+	for _, r := range rules {
+		if len(r.SourceCIDRs) == 0 {
+			r.SourceCIDRs = []string{"0.0.0.0/0"}
+		}
+		found := false
+		for i, rule := range inst.rules {
+			if r.PortRange == rule.PortRange {
+				ruleCopy := r
+				inst.rules[i] = ruleCopy
+				found = true
+				break
+			}
+			if r.String() == rule.String() {
+				found = true
+				break
+			}
+		}
+		if !found {
+			inst.rules = append(inst.rules, r)
+		}
 	}
 	return nil
 }
 
-func (inst *dummyInstance) ClosePorts(machineId string, ports []network.PortRange) error {
+func (inst *dummyInstance) ClosePorts(machineId string, rules []network.IngressRule) error {
 	defer delay()
 	if inst.firewallMode != config.FwInstance {
 		return fmt.Errorf("invalid firewall mode %q for closing ports on instance",
@@ -1552,32 +1626,36 @@ func (inst *dummyInstance) ClosePorts(machineId string, ports []network.PortRang
 		Env:        inst.state.name,
 		MachineId:  machineId,
 		InstanceId: inst.Id(),
-		Ports:      ports,
+		Rules:      rules,
 	}
-	for _, p := range ports {
-		delete(inst.ports, p)
+	for _, r := range rules {
+		for i, rule := range inst.rules {
+			if r.String() == rule.String() {
+				inst.rules = inst.rules[:i+copy(inst.rules[i:], inst.rules[i+1:])]
+			}
+		}
 	}
 	return nil
 }
 
-func (inst *dummyInstance) Ports(machineId string) (ports []network.PortRange, err error) {
+func (inst *dummyInstance) IngressRules(machineId string) (rules []network.IngressRule, err error) {
 	defer delay()
 	if inst.firewallMode != config.FwInstance {
-		return nil, fmt.Errorf("invalid firewall mode %q for retrieving ports from instance",
+		return nil, fmt.Errorf("invalid firewall mode %q for retrieving ingress rules from instance",
 			inst.firewallMode)
 	}
 	if inst.machineId != machineId {
-		panic(fmt.Errorf("Ports with mismatched machine id, expected %q got %q", inst.machineId, machineId))
+		panic(fmt.Errorf("Rules with mismatched machine id, expected %q got %q", inst.machineId, machineId))
 	}
 	inst.state.mu.Lock()
 	defer inst.state.mu.Unlock()
-	if err := inst.checkBroken("Ports"); err != nil {
+	if err := inst.checkBroken("IngressRules"); err != nil {
 		return nil, err
 	}
-	for p := range inst.ports {
-		ports = append(ports, p)
+	for _, r := range inst.rules {
+		rules = append(rules, r)
 	}
-	network.SortPortRanges(ports)
+	network.SortIngressRules(rules)
 	return
 }
 
@@ -1601,3 +1679,25 @@ func (e *environ) AllocateContainerAddresses(hostInstanceID instance.Id, contain
 func (e *environ) ReleaseContainerAddresses(interfaces []network.ProviderInterfaceInfo) error {
 	return errors.NotSupportedf("container address allocation")
 }
+
+// ProviderSpaceInfo implements NetworkingEnviron.
+func (*environ) ProviderSpaceInfo(space *network.SpaceInfo) (*environs.ProviderSpaceInfo, error) {
+	return nil, errors.NotSupportedf("provider space info")
+}
+
+// AreSpacesRoutable implements NetworkingEnviron.
+func (*environ) AreSpacesRoutable(space1, space2 *environs.ProviderSpaceInfo) (bool, error) {
+	return false, nil
+}
+
+// SSHAddresses implements environs.SSHAddresses.
+// For testing we cut "100.100.100.100" out of this list.
+func (*environ) SSHAddresses(addresses []network.Address) ([]network.Address, error) {
+	var rv []network.Address
+	for _, addr := range addresses {
+		if addr.Value != "100.100.100.100" {
+			rv = append(rv, addr)
+		}
+	}
+	return rv, nil
+}
