@@ -8,6 +8,7 @@ import (
 	"sync"
 
 	"github.com/juju/errors"
+	"google.golang.org/api/compute/v1"
 
 	jujucloud "github.com/juju/juju/cloud"
 	"github.com/juju/juju/environs"
@@ -28,12 +29,19 @@ type gceConnection interface {
 	Instances(prefix string, statuses ...string) ([]google.Instance, error)
 	AddInstance(spec google.InstanceSpec, zones ...string) (*google.Instance, error)
 	RemoveInstances(prefix string, ids ...string) error
+	UpdateMetadata(key, value string, ids ...string) error
 
-	Ports(fwname string) ([]network.PortRange, error)
-	OpenPorts(fwname string, ports ...network.PortRange) error
-	ClosePorts(fwname string, ports ...network.PortRange) error
+	IngressRules(fwname string) ([]network.IngressRule, error)
+	OpenPorts(fwname string, rules ...network.IngressRule) error
+	ClosePorts(fwname string, rules ...network.IngressRule) error
 
 	AvailabilityZones(region string) ([]google.AvailabilityZone, error)
+	// Subnetworks returns the subnetworks that machines can be
+	// assigned to in the given region.
+	Subnetworks(region string) ([]*compute.Subnetwork, error)
+	// Networks returns the available networks that exist across
+	// regions.
+	Networks() ([]*compute.Network, error)
 
 	// Storage related methods.
 
@@ -55,6 +63,8 @@ type gceConnection interface {
 	DetachDisk(zone, instanceId, volumeName string) error
 	// InstanceDisks returns a list of the disks attached to the passed instance.
 	InstanceDisks(zone, instanceId string) ([]*google.AttachedDisk, error)
+	// ListMachineTypes returns a list of machines available in the project and zone provided.
+	ListMachineTypes(zone string) ([]google.MachineType, error)
 }
 
 type environ struct {
@@ -191,26 +201,21 @@ func (env *environ) Bootstrap(ctx environs.BootstrapContext, params environs.Boo
 	// Ensure the API server port is open (globally for all instances
 	// on the network, not just for the specific node of the state
 	// server). See LP bug #1436191 for details.
-	ports := network.PortRange{
-		FromPort: params.ControllerConfig.APIPort(),
-		ToPort:   params.ControllerConfig.APIPort(),
-		Protocol: "tcp",
-	}
-	if err := env.gce.OpenPorts(env.globalFirewallName(), ports); err != nil {
+	rule := network.NewOpenIngressRule(
+		"tcp",
+		params.ControllerConfig.APIPort(),
+		params.ControllerConfig.APIPort(),
+	)
+	if err := env.gce.OpenPorts(env.globalFirewallName(), rule); err != nil {
 		return nil, errors.Trace(err)
 	}
 	return bootstrap(ctx, env, params)
 }
 
-// BootstrapMessage is part of the Environ interface.
-func (env *environ) BootstrapMessage() string {
-	return ""
-}
-
 // Destroy shuts down all known machines and destroys the rest of the
 // known environment.
 func (env *environ) Destroy() error {
-	ports, err := env.Ports()
+	ports, err := env.IngressRules()
 	if err != nil {
 		return errors.Trace(err)
 	}
