@@ -12,6 +12,7 @@ import (
 
 	"github.com/juju/errors"
 	"github.com/juju/utils"
+	"golang.org/x/net/context"
 	"google.golang.org/api/compute/v1"
 	"google.golang.org/api/googleapi"
 )
@@ -115,18 +116,23 @@ func (rc *rawConn) RemoveInstance(projectID, zone, id string) error {
 	return errors.Trace(err)
 }
 
-func (rc *rawConn) GetFirewall(projectID, name string) (*compute.Firewall, error) {
+func (rc *rawConn) GetFirewalls(projectID, namePrefix string) ([]*compute.Firewall, error) {
 	call := rc.Firewalls.List(projectID)
-	call = call.Filter("name eq " + name)
 	firewallList, err := call.Do()
 	if err != nil {
 		return nil, errors.Annotate(err, "while getting firewall from GCE")
 	}
 
 	if len(firewallList.Items) == 0 {
-		return nil, errors.NotFoundf("firewall %q", name)
+		return nil, errors.NotFoundf("firewall %q", namePrefix)
 	}
-	return firewallList.Items[0], nil
+	var result []*compute.Firewall
+	for _, fw := range firewallList.Items {
+		if strings.HasPrefix(fw.Name, namePrefix) {
+			result = append(result, fw)
+		}
+	}
+	return result, nil
 }
 
 func (rc *rawConn) AddFirewall(projectID string, firewall *compute.Firewall) error {
@@ -293,7 +299,7 @@ func isWaitError(err error) bool {
 }
 
 type opDoer interface {
-	Do() (*compute.Operation, error)
+	Do(...googleapi.CallOption) (*compute.Operation, error)
 }
 
 // checkOperation requests a new copy of the given operation from the
@@ -357,3 +363,51 @@ func (rc *rawConn) waitOperation(projectID string, op *compute.Operation, attemp
 	logger.Infof("GCE operation %q finished", op.Name)
 	return nil
 }
+
+// ListMachineTypes returns a list of machines available in the project and zone provided.
+func (rc *rawConn) ListMachineTypes(projectID, zone string) (*compute.MachineTypeList, error) {
+	op := rc.MachineTypes.List(projectID, zone)
+	machines, err := op.Do()
+	if err != nil {
+		return nil, errors.Annotatef(err, "listing machine types for project %q and zone %q", projectID, zone)
+	}
+	return machines, nil
+}
+
+func (rc *rawConn) SetMetadata(projectID, zone, instanceID string, metadata *compute.Metadata) error {
+	call := rc.Instances.SetMetadata(projectID, zone, instanceID, metadata)
+	op, err := call.Do()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	err = rc.waitOperation(projectID, op, attemptsLong)
+	return errors.Trace(err)
+}
+
+func (rc *rawConn) ListSubnetworks(projectID, region string) ([]*compute.Subnetwork, error) {
+	ctx := context.Background()
+	call := rc.Subnetworks.List(projectID, region)
+	var results []*compute.Subnetwork
+	err := call.Pages(ctx, func(page *compute.SubnetworkList) error {
+		results = append(results, page.Items...)
+		return nil
+	})
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	return results, nil
+}
+
+func (rc *rawConn) ListNetworks(projectID string) ([]*compute.Network, error) {
+	ctx := context.Background()
+	call := rc.Networks.List(projectID)
+	var results []*compute.Network
+	err := call.Pages(ctx, func(page *compute.NetworkList) error {
+		results = append(results, page.Items...)
+		return nil
+	})
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	return results, nil
+}
