@@ -97,10 +97,13 @@ func (s *BaseSuite) SetUpTest(c *gc.C) {
 		Type:  network.IPv4Address,
 		Scope: network.ScopeCloudLocal,
 	}}
-	s.RawMetadata = compute.Metadata{Items: []*compute.MetadataItems{{
-		Key:   "eggs",
-		Value: "steak",
-	}}}
+	s.RawMetadata = compute.Metadata{
+		Items: []*compute.MetadataItems{{
+			Key:   "eggs",
+			Value: StringPtr("steak"),
+		}},
+		Fingerprint: "heymumwatchthis",
+	}
 	s.Metadata = map[string]string{
 		"eggs": "steak",
 	}
@@ -127,11 +130,12 @@ func (s *BaseSuite) SetUpTest(c *gc.C) {
 	}
 	s.Instance = Instance{
 		InstanceSummary: InstanceSummary{
-			ID:        "spam",
-			ZoneName:  "a-zone",
-			Status:    StatusRunning,
-			Metadata:  s.Metadata,
-			Addresses: s.Addresses,
+			ID:                "spam",
+			ZoneName:          "a-zone",
+			Status:            StatusRunning,
+			Metadata:          s.Metadata,
+			Addresses:         s.Addresses,
+			NetworkInterfaces: s.RawInstance.NetworkInterfaces,
 		},
 		spec: &s.InstanceSpec,
 	}
@@ -141,6 +145,10 @@ func (s *BaseSuite) NewWaitError(op *compute.Operation, cause error) error {
 	return waitError{op, cause}
 }
 
+func StringPtr(val string) *string {
+	return &val
+}
+
 type fakeCall struct {
 	FuncName string
 
@@ -158,6 +166,7 @@ type fakeCall struct {
 	AttachedDisk *compute.AttachedDisk
 	DeviceName   string
 	ComputeDisk  *compute.Disk
+	Metadata     *compute.Metadata
 }
 
 type fakeConn struct {
@@ -166,13 +175,15 @@ type fakeConn struct {
 	Project       *compute.Project
 	Instance      *compute.Instance
 	Instances     []*compute.Instance
-	Firewall      *compute.Firewall
+	Firewalls     []*compute.Firewall
 	Zones         []*compute.Zone
 	Err           error
 	FailOnCall    int
 	Disks         []*compute.Disk
 	Disk          *compute.Disk
 	AttachedDisks []*compute.AttachedDisk
+	Networks      []*compute.Network
+	Subnetworks   []*compute.Subnetwork
 }
 
 func (rc *fakeConn) GetProject(projectID string) (*compute.Project, error) {
@@ -254,9 +265,9 @@ func (rc *fakeConn) RemoveInstance(projectID, zone, id string) error {
 	return err
 }
 
-func (rc *fakeConn) GetFirewall(projectID, name string) (*compute.Firewall, error) {
+func (rc *fakeConn) GetFirewalls(projectID, name string) ([]*compute.Firewall, error) {
 	call := fakeCall{
-		FuncName:  "GetFirewall",
+		FuncName:  "GetFirewalls",
 		ProjectID: projectID,
 		Name:      name,
 	}
@@ -266,7 +277,7 @@ func (rc *fakeConn) GetFirewall(projectID, name string) (*compute.Firewall, erro
 	if len(rc.Calls) != rc.FailOnCall+1 {
 		err = nil
 	}
-	return rc.Firewall, err
+	return rc.Firewalls, err
 }
 
 func (rc *fakeConn) AddFirewall(projectID string, firewall *compute.Firewall) error {
@@ -442,3 +453,72 @@ func (rc *fakeConn) InstanceDisks(project, zone, instanceId string) ([]*compute.
 	}
 	return rc.AttachedDisks, err
 }
+
+func (rc *fakeConn) ListMachineTypes(projectID, zone string) (*compute.MachineTypeList, error) {
+	call := fakeCall{
+		FuncName:  "ListMachineTypes",
+		ProjectID: projectID,
+		ZoneName:  zone,
+	}
+	rc.Calls = append(rc.Calls, call)
+	machineType := compute.MachineTypeList{
+		Items: []*compute.MachineType{
+			{Name: "type-1", MemoryMb: 1024},
+			{Name: "type-2", MemoryMb: 2048},
+		},
+	}
+	return &machineType, nil
+
+}
+
+func (rc *fakeConn) SetMetadata(projectID, zone, instanceID string, metadata *compute.Metadata) error {
+	call := fakeCall{
+		FuncName:   "SetMetadata",
+		ProjectID:  projectID,
+		ZoneName:   zone,
+		InstanceId: instanceID,
+		Metadata:   metadata,
+	}
+	rc.Calls = append(rc.Calls, call)
+
+	err := rc.Err
+	if len(rc.Calls) != rc.FailOnCall+1 {
+		err = nil
+	}
+	return err
+}
+
+func (rc *fakeConn) ListNetworks(projectID string) ([]*compute.Network, error) {
+	call := fakeCall{
+		FuncName:  "ListNetworks",
+		ProjectID: projectID,
+	}
+	rc.Calls = append(rc.Calls, call)
+
+	err := rc.Err
+	if len(rc.Calls) != rc.FailOnCall+1 {
+		err = nil
+	}
+	if err != nil {
+		return nil, err
+	}
+	return rc.Networks, nil
+}
+
+func (rc *fakeConn) ListSubnetworks(projectID, region string) ([]*compute.Subnetwork, error) {
+	call := fakeCall{
+		FuncName:  "ListSubnetworks",
+		ProjectID: projectID,
+		Region:    region,
+	}
+	rc.Calls = append(rc.Calls, call)
+
+	err := rc.Err
+	if len(rc.Calls) != rc.FailOnCall+1 {
+		err = nil
+	}
+	if err != nil {
+		return nil, err
+	}
+	return rc.Subnetworks, nil
+}
