@@ -11,6 +11,7 @@ import (
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils/arch"
 	"github.com/juju/version"
+	"google.golang.org/api/compute/v1"
 	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/cloud"
@@ -119,7 +120,7 @@ type BaseSuiteUnpatched struct {
 	StartInstArgs   environs.StartInstanceParams
 	InstanceType    instances.InstanceType
 
-	Ports []network.PortRange
+	Rules []network.IngressRule
 }
 
 var _ environs.Environ = (*environ)(nil)
@@ -214,11 +215,7 @@ func (s *BaseSuiteUnpatched) initInst(c *gc.C) {
 }
 
 func (s *BaseSuiteUnpatched) initNet(c *gc.C) {
-	s.Ports = []network.PortRange{{
-		FromPort: 80,
-		ToPort:   80,
-		Protocol: "tcp",
-	}}
+	s.Rules = []network.IngressRule{network.MustNewIngressRule("tcp", 80, 80)}
 }
 
 func (s *BaseSuiteUnpatched) setConfig(c *gc.C, cfg *config.Config) {
@@ -275,6 +272,17 @@ func (s *BaseSuiteUnpatched) NewBaseInstance(c *gc.C, id string) *google.Instanc
 		Status:    google.StatusRunning,
 		Metadata:  s.UbuntuMetadata,
 		Addresses: s.Addresses,
+		NetworkInterfaces: []*compute.NetworkInterface{{
+			Name:       "somenetif",
+			NetworkIP:  "10.0.10.3",
+			Network:    "https://www.googleapis.com/compute/v1/projects/sonic-youth/global/networks/go-team",
+			Subnetwork: "https://www.googleapis.com/compute/v1/projects/sonic-youth/regions/asia-east1/subnetworks/go-team",
+			AccessConfigs: []*compute.AccessConfig{{
+				Type:  "ONE_TO_ONE_NAT",
+				Name:  "ExternalNAT",
+				NatIP: "25.185.142.226",
+			}},
+		}},
 	}
 	return google.NewInstance(summary, &instanceSpec)
 }
@@ -284,6 +292,10 @@ func (s *BaseSuiteUnpatched) NewInstance(c *gc.C, id string) *environInstance {
 	return newInstance(base, s.Env)
 }
 
+func (s *BaseSuiteUnpatched) NewInstanceFromBase(base *google.Instance) *environInstance {
+	return newInstance(base, s.Env)
+}
+
 type BaseSuite struct {
 	BaseSuiteUnpatched
 
@@ -459,21 +471,25 @@ type fakeConnCall struct {
 	Statuses     []string
 	InstanceSpec google.InstanceSpec
 	FirewallName string
-	PortRanges   []network.PortRange
+	Rules        []network.IngressRule
 	Region       string
 	Disks        []google.DiskSpec
 	VolumeName   string
 	InstanceId   string
 	Mode         string
+	Key          string
+	Value        string
 }
 
 type fakeConn struct {
 	Calls []fakeConnCall
 
-	Inst       *google.Instance
-	Insts      []google.Instance
-	PortRanges []network.PortRange
-	Zones      []google.AvailabilityZone
+	Inst      *google.Instance
+	Insts     []google.Instance
+	Rules     []network.IngressRule
+	Zones     []google.AvailabilityZone
+	Subnets   []*compute.Subnetwork
+	Networks_ []*compute.Network
 
 	GoogleDisks   []*google.Disk
 	GoogleDisk    *google.Disk
@@ -534,28 +550,38 @@ func (fc *fakeConn) RemoveInstances(prefix string, ids ...string) error {
 	return fc.err()
 }
 
-func (fc *fakeConn) Ports(fwname string) ([]network.PortRange, error) {
+func (fc *fakeConn) UpdateMetadata(key, value string, ids ...string) error {
 	fc.Calls = append(fc.Calls, fakeConnCall{
-		FuncName:     "Ports",
-		FirewallName: fwname,
-	})
-	return fc.PortRanges, fc.err()
-}
-
-func (fc *fakeConn) OpenPorts(fwname string, ports ...network.PortRange) error {
-	fc.Calls = append(fc.Calls, fakeConnCall{
-		FuncName:     "OpenPorts",
-		FirewallName: fwname,
-		PortRanges:   ports,
+		FuncName: "UpdateMetadata",
+		Key:      key,
+		Value:    value,
+		IDs:      ids,
 	})
 	return fc.err()
 }
 
-func (fc *fakeConn) ClosePorts(fwname string, ports ...network.PortRange) error {
+func (fc *fakeConn) IngressRules(fwname string) ([]network.IngressRule, error) {
+	fc.Calls = append(fc.Calls, fakeConnCall{
+		FuncName:     "Ports",
+		FirewallName: fwname,
+	})
+	return fc.Rules, fc.err()
+}
+
+func (fc *fakeConn) OpenPorts(fwname string, rules ...network.IngressRule) error {
+	fc.Calls = append(fc.Calls, fakeConnCall{
+		FuncName:     "OpenPorts",
+		FirewallName: fwname,
+		Rules:        rules,
+	})
+	return fc.err()
+}
+
+func (fc *fakeConn) ClosePorts(fwname string, rules ...network.IngressRule) error {
 	fc.Calls = append(fc.Calls, fakeConnCall{
 		FuncName:     "ClosePorts",
 		FirewallName: fwname,
-		PortRanges:   ports,
+		Rules:        rules,
 	})
 	return fc.err()
 }
@@ -568,6 +594,21 @@ func (fc *fakeConn) AvailabilityZones(region string) ([]google.AvailabilityZone,
 	return fc.Zones, fc.err()
 }
 
+func (fc *fakeConn) Subnetworks(region string) ([]*compute.Subnetwork, error) {
+	fc.Calls = append(fc.Calls, fakeConnCall{
+		FuncName: "Subnetworks",
+		Region:   region,
+	})
+	return fc.Subnets, fc.err()
+}
+
+func (fc *fakeConn) Networks() ([]*compute.Network, error) {
+	fc.Calls = append(fc.Calls, fakeConnCall{
+		FuncName: "Networks",
+	})
+	return fc.Networks_, fc.err()
+}
+
 func (fc *fakeConn) CreateDisks(zone string, disks []google.DiskSpec) ([]*google.Disk, error) {
 	fc.Calls = append(fc.Calls, fakeConnCall{
 		FuncName: "CreateDisks",
@@ -644,3 +685,16 @@ func (fc *fakeConn) WasCalled(funcName string) (bool, []fakeConnCall) {
 	}
 	return called, calls
 }
+
+func (fc *fakeConn) ListMachineTypes(zone string) ([]google.MachineType, error) {
+	call := fakeConnCall{
+		FuncName: "ListMachineTypes",
+		ZoneName: zone,
+	}
+	fc.Calls = append(fc.Calls, call)
+
+	return []google.MachineType{
+		{Name: "type-1", MemoryMb: 1024},
+		{Name: "type-2", MemoryMb: 2048},
+	}, nil
+}
