@@ -20,6 +20,7 @@ import (
 	"github.com/juju/utils/os"
 	"github.com/juju/utils/series"
 	"github.com/juju/utils/set"
+	"github.com/juju/version"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/cloudconfig/cloudinit"
@@ -59,7 +60,6 @@ const statusPollInterval = 5 * time.Second
 var (
 	ReleaseNodes         = releaseNodes
 	DeploymentStatusCall = deploymentStatusCall
-	GetCapabilities      = getCapabilities
 	GetMAAS2Controller   = getMAAS2Controller
 )
 
@@ -101,15 +101,27 @@ type maasEnviron struct {
 
 	// apiVersion tells us if we are using the MAAS 1.0 or 2.0 api.
 	apiVersion string
+
+	// GetCapabilities is a function that connects to MAAS to return its set of
+	// capabilities.
+	GetCapabilities MaasCapabilities
 }
 
 var _ environs.Environ = (*maasEnviron)(nil)
 
-func NewEnviron(cloud environs.CloudSpec, cfg *config.Config) (*maasEnviron, error) {
+// MaasCapabilities represents a function that gets the capabilities of a MAAS
+// installation.
+type MaasCapabilities func(client *gomaasapi.MAASObject, serverURL string) (set.Strings, error)
+
+func NewEnviron(cloud environs.CloudSpec, cfg *config.Config, getCaps MaasCapabilities) (*maasEnviron, error) {
+	if getCaps == nil {
+		getCaps = getCapabilities
+	}
 	env := &maasEnviron{
-		name:  cfg.Name(),
-		uuid:  cfg.UUID(),
-		cloud: cloud,
+		name:            cfg.Name(),
+		uuid:            cfg.UUID(),
+		cloud:           cloud,
+		GetCapabilities: getCaps,
 	}
 	err := env.SetConfig(cfg)
 	if err != nil {
@@ -182,11 +194,6 @@ func (env *maasEnviron) Bootstrap(ctx environs.BootstrapContext, args environs.B
 	return bsResult, nil
 }
 
-// BootstrapMessage is part of the Environ interface.
-func (env *maasEnviron) BootstrapMessage() string {
-	return ""
-}
-
 // ControllerInstances is specified in the Environ interface.
 func (env *maasEnviron) ControllerInstances(controllerUUID string) ([]instance.Id, error) {
 	if !env.usingMAAS2() {
@@ -273,12 +280,17 @@ func (env *maasEnviron) SetConfig(cfg *config.Config) error {
 	switch {
 	case gomaasapi.IsUnsupportedVersionError(err):
 		apiVersion = apiVersion1
-		authClient, err := gomaasapi.NewAuthenticatedClient(maasServer, maasOAuth, apiVersion1)
+		_, _, includesVersion := gomaasapi.SplitVersionedURL(maasServer)
+		versionURL := maasServer
+		if !includesVersion {
+			versionURL = gomaasapi.AddAPIVersionToURL(maasServer, apiVersion1)
+		}
+		authClient, err := gomaasapi.NewAuthenticatedClient(versionURL, maasOAuth)
 		if err != nil {
 			return errors.Trace(err)
 		}
 		env.maasClientUnlocked = gomaasapi.NewMAAS(*authClient)
-		caps, err := GetCapabilities(env.maasClientUnlocked, maasServer)
+		caps, err := env.GetCapabilities(env.maasClientUnlocked, maasServer)
 		if err != nil {
 			return errors.Trace(err)
 		}
@@ -314,6 +326,11 @@ func (env *maasEnviron) SupportsSpaceDiscovery() (bool, error) {
 	return true, nil
 }
 
+// SupportsContainerAddresses is specified on environs.Networking.
+func (env *maasEnviron) SupportsContainerAddresses() (bool, error) {
+	return true, nil
+}
+
 // allArchitectures2 uses the MAAS2 controller to get architectures from boot
 // resources.
 func (env *maasEnviron) allArchitectures2() ([]string, error) {
@@ -612,25 +629,29 @@ func getCapabilities(client *gomaasapi.MAASObject, serverURL string) (set.String
 	for a := shortAttempt.Start(); a.Next(); {
 		version := client.GetSubObject("version/")
 		result, err = version.CallGet("", nil)
-		if err != nil {
-			if err, ok := errors.Cause(err).(gomaasapi.ServerError); ok && err.StatusCode == 404 {
-				message := "could not connect to MAAS controller - check the endpoint is correct"
-				trimmedURL := strings.TrimRight(serverURL, "/")
-				if !strings.HasSuffix(trimmedURL, "/MAAS") {
-					message += " (it normally ends with /MAAS)"
-				}
-				return caps, errors.NewNotSupported(nil, message)
-			}
-		} else {
+		if err == nil {
 			break
 		}
+		if err, ok := errors.Cause(err).(gomaasapi.ServerError); ok && err.StatusCode == 404 {
+			logger.Debugf("Failed attempting to get capabilities from maas endpoint %q: %v", serverURL, err)
+
+			message := "could not connect to MAAS controller - check the endpoint is correct"
+			trimmedURL := strings.TrimRight(serverURL, "/")
+			if !strings.HasSuffix(trimmedURL, "/MAAS") {
+				message += " (it normally ends with /MAAS)"
+			}
+			return caps, errors.NewNotSupported(nil, message)
+		}
 	}
 	if err != nil {
+		logger.Debugf("Can't connect to maas server at endpoint %q: %v", serverURL, err)
 		return caps, err
 	}
 	info, err := result.GetMap()
 	if err != nil {
-		return caps, err
+		logger.Debugf("Invalid data returned from maas endpoint %q: %v", serverURL, err)
+		// invalid data of some sort, probably not a MAAS server.
+		return caps, errors.New("failed to get expected data from server")
 	}
 	capsObj, ok := info["capabilities"]
 	if !ok {
@@ -638,12 +659,14 @@ func getCapabilities(client *gomaasapi.MAASObject, serverURL string) (set.String
 	}
 	items, err := capsObj.GetArray()
 	if err != nil {
-		return caps, err
+		logger.Debugf("Invalid data returned from maas endpoint %q: %v", serverURL, err)
+		return caps, errors.New("failed to get expected data from server")
 	}
 	for _, item := range items {
 		val, err := item.GetString()
 		if err != nil {
-			return set.NewStrings(), err
+			logger.Debugf("Invalid data returned from maas endpoint %q: %v", serverURL, err)
+			return set.NewStrings(), errors.New("failed to get expected data from server")
 		}
 		caps.Add(val)
 	}
@@ -983,15 +1006,7 @@ func (environ *maasEnviron) StartInstance(args environs.StartInstanceParams) (
 		return nil, errors.Trace(err)
 	}
 
-	// We need to extract the names of all interfaces of the selected node,
-	// which are both linked to subnets, and have an IP address, in order to
-	// pass those to the bridge script.
-	interfaceNamesToBridge, err := instanceConfiguredInterfaceNames(environ.usingMAAS2(), inst, subnetsMap)
-	if err != nil {
-		return nil, errors.Trace(err)
-	}
-
-	cloudcfg, err := environ.newCloudinitConfig(hostname, series, interfaceNamesToBridge)
+	cloudcfg, err := environ.newCloudinitConfig(hostname, series)
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
@@ -1318,68 +1333,9 @@ func (environ *maasEnviron) selectNode2(args selectNodeArgs) (maasInstance, erro
 	return inst, nil
 }
 
-// setupJujuNetworking returns a string representing the script to run
-// in order to prepare the Juju-specific networking config on a node.
-func setupJujuNetworking(interfacesToBridge []string) string {
-	// For ubuntu series < xenial we prefer python2 over python3
-	// as we don't want to invalidate lots of testing against
-	// known cloud-image contents. A summary of Ubuntu releases
-	// and python inclusion in the default install of Ubuntu
-	// Server is as follows:
-	//
-	// 12.04 precise:  python 2 (2.7.3)
-	// 14.04 trusty:   python 2 (2.7.5) and python3 (3.4.0)
-	// 14.10 utopic:   python 2 (2.7.8) and python3 (3.4.2)
-	// 15.04 vivid:    python 2 (2.7.9) and python3 (3.4.3)
-	// 15.10 wily:     python 2 (2.7.9) and python3 (3.4.3)
-	// 16.04 xenial:   python 3 only (3.5.1)
-	//
-	// going forward:  python 3 only
-
-	return fmt.Sprintf(`
-trap 'rm -f %[1]q' EXIT
-
-if [ -x /usr/bin/python2 ]; then
-    juju_networking_preferred_python_binary=/usr/bin/python2
-elif [ -x /usr/bin/python3 ]; then
-    juju_networking_preferred_python_binary=/usr/bin/python3
-elif [ -x /usr/bin/python ]; then
-    juju_networking_preferred_python_binary=/usr/bin/python
-fi
-
-if [ ! -z "${juju_networking_preferred_python_binary:-}" ]; then
-    if [ -f %[1]q ]; then
-# We are sharing this code between 2.0 and 1.25.
-# For the moment we want 2.0 to bridge all interfaces linked
-# to a subnet, while for 1.25 we only bridge the default route interface.
-# This setting allows us to easily switch the behaviour when merging
-# the code between those various branches.
-        juju_bridge_linked_interfaces=1
-        if [ $juju_bridge_linked_interfaces -eq 1 ]; then
-            $juju_networking_preferred_python_binary %[1]q --bridge-prefix=%[2]q --interfaces-to-bridge=%[5]q --one-time-backup --activate %[4]q
-        else
-            juju_ipv4_interface_to_bridge=$(ip -4 route list exact default | head -n1 | cut -d' ' -f5)
-            $juju_networking_preferred_python_binary %[1]q --bridge-name=%[3]q --interfaces-to-bridge="${juju_ipv4_interface_to_bridge:-unknown}" --one-time-backup --activate %[4]q
-        fi
-    fi
-else
-    echo "error: no Python installation found; cannot run Juju's bridge script"
-fi`,
-		bridgeScriptPath,
-		instancecfg.DefaultBridgePrefix,
-		instancecfg.DefaultBridgeName,
-		"/etc/network/interfaces",
-		strings.Join(interfacesToBridge, " "),
-	)
-}
-
-func renderEtcNetworkInterfacesScript(interfacesToBridge ...string) string {
-	return setupJujuNetworking(interfacesToBridge)
-}
-
 // newCloudinitConfig creates a cloudinit.Config structure suitable as a base
 // for initialising a MAAS node.
-func (environ *maasEnviron) newCloudinitConfig(hostname, forSeries string, interfacesToBridge []string) (cloudinit.CloudConfig, error) {
+func (environ *maasEnviron) newCloudinitConfig(hostname, forSeries string) (cloudinit.CloudConfig, error) {
 	cloudcfg, err := cloudinit.New(forSeries)
 	if err != nil {
 		return nil, err
@@ -1410,8 +1366,6 @@ func (environ *maasEnviron) newCloudinitConfig(hostname, forSeries string, inter
 			break
 		}
 		cloudcfg.AddPackage("bridge-utils")
-		cloudcfg.AddBootTextFile(bridgeScriptPath, bridgeScriptPython, 0755)
-		cloudcfg.AddScripts(setupJujuNetworking(interfacesToBridge))
 	}
 	return cloudcfg, nil
 }
@@ -1421,7 +1375,7 @@ func (environ *maasEnviron) releaseNodes1(nodes gomaasapi.MAASObject, ids url.Va
 	if err == nil {
 		return nil
 	}
-	maasErr, ok := errors.Cause(err).(gomaasapi.ServerError)
+	maasErr, ok := gomaasapi.GetServerError(err)
 	if !ok {
 		return errors.Annotate(err, "cannot release nodes")
 	}
@@ -2073,18 +2027,18 @@ func (environ *maasEnviron) DestroyController(controllerUUID string) error {
 }
 
 // MAAS does not do firewalling so these port methods do nothing.
-func (*maasEnviron) OpenPorts([]network.PortRange) error {
+func (*maasEnviron) OpenPorts([]network.IngressRule) error {
 	logger.Debugf("unimplemented OpenPorts() called")
 	return nil
 }
 
-func (*maasEnviron) ClosePorts([]network.PortRange) error {
+func (*maasEnviron) ClosePorts([]network.IngressRule) error {
 	logger.Debugf("unimplemented ClosePorts() called")
 	return nil
 }
 
-func (*maasEnviron) Ports() ([]network.PortRange, error) {
-	logger.Debugf("unimplemented Ports() called")
+func (*maasEnviron) IngressRules() ([]network.IngressRule, error) {
+	logger.Debugf("unimplemented Rules() called")
 	return nil, nil
 }
 
@@ -2224,36 +2178,6 @@ func (env *maasEnviron) allocateContainerAddresses1(hostInstanceID instance.Id,
 }
 
 func (env *maasEnviron) allocateContainerAddresses2(hostInstanceID instance.Id, containerTag names.MachineTag, preparedInfo []network.InterfaceInfo) ([]network.InterfaceInfo, error) {
-	subnetCIDRToSubnet := make(map[string]gomaasapi.Subnet)
-	spaces, err := env.maasController.Spaces()
-	if err != nil {
-		return nil, errors.Trace(err)
-	}
-	for _, space := range spaces {
-		for _, subnet := range space.Subnets() {
-			subnetCIDRToSubnet[subnet.CIDR()] = subnet
-		}
-	}
-
-	var primaryNICInfo network.InterfaceInfo
-	primaryNICName := "eth0"
-	for _, nic := range preparedInfo {
-		if nic.InterfaceName == primaryNICName {
-			primaryNICInfo = nic
-			break
-		}
-	}
-	if primaryNICInfo.InterfaceName == "" {
-		return nil, errors.Errorf("cannot find primary interface for container")
-	}
-	logger.Debugf("primary device NIC prepared info: %+v", primaryNICInfo)
-
-	primaryNICSubnetCIDR := primaryNICInfo.CIDR
-	subnet, hasSubnet := subnetCIDRToSubnet[primaryNICSubnetCIDR]
-	if !hasSubnet {
-		logger.Debugf("primary device NIC %q has no linked subnet - leaving unconfigured", primaryNICInfo.InterfaceName)
-	}
-	primaryMACAddress := primaryNICInfo.MACAddress
 	args := gomaasapi.MachinesArgs{
 		AgentName: env.uuid,
 		SystemIDs: []string{string(hostInstanceID)},
@@ -2270,73 +2194,35 @@ func (env *maasEnviron) allocateContainerAddresses2(hostInstanceID instance.Id,
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
-	createDeviceArgs := gomaasapi.CreateMachineDeviceArgs{
-		Hostname:      deviceName,
-		MACAddress:    primaryMACAddress,
-		Subnet:        subnet, // can be nil
-		InterfaceName: primaryNICName,
-	}
-	device, err := machine.CreateDevice(createDeviceArgs)
+	params, err := env.prepareDeviceDetails(deviceName, machine, preparedInfo)
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
-	interface_set := device.InterfaceSet()
-	if len(interface_set) != 1 {
-		// Shouldn't be possible as machine.CreateDevice always returns us
-		// one interface.
-		return nil, errors.Errorf("unexpected number of interfaces in response from creating device: %v", interface_set)
+
+	// Check to see if we've already tried to allocate information for this device:
+	device, err := env.checkForExistingDevice(params)
+	if err != nil {
+		return nil, errors.Trace(err)
 	}
-	primaryNICVLAN := interface_set[0].VLAN()
-
-	nameToParentName := make(map[string]string)
-	for _, nic := range preparedInfo {
-		nameToParentName[nic.InterfaceName] = nic.ParentInterfaceName
-		if nic.InterfaceName != primaryNICName {
-			createArgs := gomaasapi.CreateInterfaceArgs{
-				Name:       nic.InterfaceName,
-				MTU:        nic.MTU,
-				MACAddress: nic.MACAddress,
-			}
-
-			subnet, knownSubnet := subnetCIDRToSubnet[nic.CIDR]
-			if !knownSubnet {
-				logger.Warningf("NIC %v has no subnet - setting to manual and using untagged VLAN", nic.InterfaceName)
-				createArgs.VLAN = primaryNICVLAN
-			} else {
-				createArgs.VLAN = subnet.VLAN()
-				logger.Infof("linking NIC %v to subnet %v - using static IP", nic.InterfaceName, subnet.CIDR())
-			}
-
-			createdNIC, err := device.CreateInterface(createArgs)
-			if err != nil {
-				return nil, errors.Annotate(err, "creating device interface")
-			}
-			logger.Debugf("created device interface: %+v", createdNIC)
-
-			if !knownSubnet {
-				continue
-			}
-
-			linkArgs := gomaasapi.LinkSubnetArgs{
-				Mode:   gomaasapi.LinkModeStatic,
-				Subnet: subnet,
-			}
-
-			if err := createdNIC.LinkSubnet(linkArgs); err != nil {
-				logger.Warningf("linking NIC %v to subnet %v failed: %v", nic.InterfaceName, subnet.CIDR(), err)
-			} else {
-				logger.Debugf("linked device interface to subnet: %+v", createdNIC)
-			}
+	if device == nil {
+		device, err = env.createAndPopulateDevice(params)
+		if err != nil {
+			return nil, errors.Annotatef(err,
+				"failed to create MAAS device for %q",
+				params.Name)
 		}
 	}
 
-	finalInterfaces, err := env.deviceInterfaceInfo2(device.SystemID(), nameToParentName)
+	// TODO(jam): the old code used to reload the device from its SystemID()
+	nameToParentName := make(map[string]string)
+	for _, nic := range preparedInfo {
+		nameToParentName[nic.InterfaceName] = nic.ParentInterfaceName
+	}
+	interfaces, err := env.deviceInterfaceInfo2(device, nameToParentName, params.CIDRToStaticRoutes)
 	if err != nil {
 		return nil, errors.Annotate(err, "cannot get device interfaces")
 	}
-	logger.Debugf("allocated device interfaces: %+v", finalInterfaces)
-
-	return finalInterfaces, nil
+	return interfaces, nil
 }
 
 func (env *maasEnviron) ReleaseContainerAddresses(interfaces []network.ProviderInterfaceInfo) error {
@@ -2414,3 +2300,54 @@ func (env *maasEnviron) releaseContainerAddresses2(macAddresses []string) error
 	}
 	return nil
 }
+
+// AdoptResources updates all the instances to indicate they
+// are now associated with the specified controller.
+func (env *maasEnviron) AdoptResources(controllerUUID string, fromVersion version.Number) error {
+	if !env.usingMAAS2() {
+		// We don't track instance -> controller for MAAS1.
+		return nil
+	}
+
+	instances, err := env.AllInstances()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	var failed []instance.Id
+	for _, instance := range instances {
+		maas2Instance, ok := instance.(*maas2Instance)
+		if !ok {
+			// This should never happen.
+			return errors.Errorf("instance %q wasn't a maas2Instance", instance.Id())
+		}
+		// From the MAAS docs: "[SetOwnerData] will not remove any
+		// previous keys unless explicitly passed with an empty
+		// string." So not passing all of the keys here is fine.
+		// https://maas.ubuntu.com/docs2.0/api.html#machine
+		err := maas2Instance.machine.SetOwnerData(map[string]string{tags.JujuController: controllerUUID})
+		if err != nil {
+			logger.Errorf("error setting controller uuid tag for %q: %v", instance.Id(), err)
+			failed = append(failed, instance.Id())
+		}
+	}
+
+	if failed != nil {
+		return errors.Errorf("failed to update controller for some instances: %v", failed)
+	}
+	return nil
+}
+
+// ProviderSpaceInfo implements environs.NetworkingEnviron.
+func (*maasEnviron) ProviderSpaceInfo(space *network.SpaceInfo) (*environs.ProviderSpaceInfo, error) {
+	return nil, errors.NotSupportedf("provider space info")
+}
+
+// AreSpacesRoutable implements environs.NetworkingEnviron.
+func (*maasEnviron) AreSpacesRoutable(space1, space2 *environs.ProviderSpaceInfo) (bool, error) {
+	return false, nil
+}
+
+// SSHAddresses implements environs.SSHAddresses.
+func (*maasEnviron) SSHAddresses(addresses []network.Address) ([]network.Address, error) {
+	return addresses, nil
+}
