@@ -17,6 +17,7 @@ import (
 	"github.com/juju/utils"
 	"github.com/juju/utils/arch"
 	"github.com/juju/utils/series"
+	"github.com/juju/utils/set"
 	"github.com/juju/version"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/names.v2"
@@ -37,6 +38,13 @@ import (
 	coretesting "github.com/juju/juju/testing"
 )
 
+// ensure we conform to the right interfaces
+var (
+	_ environs.NetworkingEnviron = (*maasEnviron)(nil)
+	// Should maasEnviron implement this? It needs ConfigDefaults
+	// _ config.ConfigSchemaSource  = (*maasEnviron)(nil)
+)
+
 type environSuite struct {
 	providerSuite
 }
@@ -455,6 +463,7 @@ func (suite *environSuite) TestSupportsSpaces(c *gc.C) {
 	supported, err := env.SupportsSpaces()
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(supported, jc.IsTrue)
+	c.Check(env, jc.Satisfies, environs.SupportsSpaces)
 }
 
 func (suite *environSuite) TestSupportsSpaceDiscovery(c *gc.C) {
@@ -464,6 +473,14 @@ func (suite *environSuite) TestSupportsSpaceDiscovery(c *gc.C) {
 	c.Assert(supported, jc.IsTrue)
 }
 
+func (suite *environSuite) TestSupportsContainerAddresses(c *gc.C) {
+	env := suite.makeEnviron()
+	supported, err := env.SupportsContainerAddresses()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(supported, jc.IsTrue)
+	c.Check(env, jc.Satisfies, environs.SupportsContainerAddresses)
+}
+
 func (suite *environSuite) TestSubnetsWithInstanceIdAndSubnetIds(c *gc.C) {
 	server := suite.testMAASObject.TestServer
 	var subnetIDs []network.Id
@@ -1093,3 +1110,53 @@ func (s *environSuite) TestReleaseContainerAddresses_HandlesDupes(c *gc.C) {
 	}
 	c.Assert(systemIds, gc.DeepEquals, []string{"device3"})
 }
+
+func (s *environSuite) TestAdoptResources(c *gc.C) {
+	s.addNode(allocatedNode)
+	env := s.makeEnviron()
+	// Shouldn't do anything in MAAS1.
+	err := env.AdoptResources("other-controller", version.MustParse("3.2.1"))
+	c.Assert(err, jc.ErrorIsNil)
+}
+
+func (s *environSuite) TestUsingNonVersionURLForAPI(c *gc.C) {
+	var gotURL *url.URL
+	configuredURL := s.testMAASObject.TestServer.URL
+	_, err := s.makeEnvironWithURL(
+		configuredURL,
+		func(client *gomaasapi.MAASObject, serverURL string) (set.Strings, error) {
+			gotURL = client.URL()
+			return set.NewStrings("network-deployment-ubuntu"), nil
+		},
+	)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(gotURL.String(), gc.Equals, configuredURL+"/api/1.0/")
+}
+
+func (s *environSuite) TestUsingVersionURLForAPI(c *gc.C) {
+	var gotURL *url.URL
+	configuredURL := s.testMAASObject.TestServer.URL + "/api/1.0/"
+	_, err := s.makeEnvironWithURL(
+		configuredURL,
+		func(client *gomaasapi.MAASObject, serverURL string) (set.Strings, error) {
+			gotURL = client.URL()
+			return set.NewStrings("network-deployment-ubuntu"), nil
+		},
+	)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(gotURL.String(), gc.Equals, configuredURL)
+}
+
+func (s *environSuite) TestUsingUnknownVersionURLForAPI(c *gc.C) {
+	var gotURL *url.URL
+	configuredURL := s.testMAASObject.TestServer.URL + "/api/3.0/"
+	_, err := s.makeEnvironWithURL(
+		configuredURL,
+		func(client *gomaasapi.MAASObject, serverURL string) (set.Strings, error) {
+			gotURL = client.URL()
+			return set.NewStrings("network-deployment-ubuntu"), nil
+		},
+	)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(gotURL.String(), gc.Equals, configuredURL)
+}
