@@ -60,7 +60,7 @@ func (suite *maas2EnvironSuite) getEnvWithServer(c *gc.C) (*maasEnviron, error)
 	attrs := coretesting.FakeConfig().Merge(maasEnvAttrs)
 	cfg, err := config.New(config.NoDefaults, attrs)
 	c.Assert(err, jc.ErrorIsNil)
-	return NewEnviron(cloud, cfg)
+	return NewEnviron(cloud, cfg, nil)
 }
 
 func (suite *maas2EnvironSuite) TestNewEnvironWithController(c *gc.C) {
@@ -529,7 +529,7 @@ func (suite *maas2EnvironSuite) TestAcquireNodeInterfaces(c *gc.C) {
 	cons := constraints.Value{
 		Spaces: stringslicep("foo", "^bar"),
 	}
-	// In the tests below "space:5" means foo, "space:6" means bar.
+	// In the tests below Space 2 means foo, Space 3 means bar.
 	for i, test := range []struct {
 		interfaces        []interfaceBinding
 		expectedPositives []gomaasapi.InterfaceSpec
@@ -553,8 +553,9 @@ func (suite *maas2EnvironSuite) TestAcquireNodeInterfaces(c *gc.C) {
 		expectedPositives: []gomaasapi.InterfaceSpec{{"name-1", "7"}, {"name-2", "8"}, {"name-3", "9"}, {"0", "2"}},
 		expectedNegatives: []string{"3"},
 	}, {
-		interfaces:    []interfaceBinding{{"", "anything"}},
-		expectedError: "interface bindings cannot have empty names",
+		interfaces:        []interfaceBinding{{"", "anything"}},
+		expectedPositives: []gomaasapi.InterfaceSpec{{"0", "anything"}, {"1", "2"}},
+		expectedNegatives: []string{"3"},
 	}, {
 		interfaces:    []interfaceBinding{{"shared-db", "3"}},
 		expectedError: `negative space "bar" from constraints clashes with interface bindings`,
@@ -567,7 +568,7 @@ func (suite *maas2EnvironSuite) TestAcquireNodeInterfaces(c *gc.C) {
 		expectedNegatives: []string{"3"},
 	}, {
 		interfaces:    []interfaceBinding{{"", ""}},
-		expectedError: "interface bindings cannot have empty names",
+		expectedError: `invalid interface binding "": space provider ID is required`,
 	}, {
 		interfaces: []interfaceBinding{
 			{"valid", "ok"},
@@ -575,7 +576,7 @@ func (suite *maas2EnvironSuite) TestAcquireNodeInterfaces(c *gc.C) {
 			{"valid-name-empty-space", ""},
 			{"", ""},
 		},
-		expectedError: "interface bindings cannot have empty names",
+		expectedError: `invalid interface binding "valid-name-empty-space": space provider ID is required`,
 	}, {
 		interfaces:    []interfaceBinding{{"foo", ""}},
 		expectedError: `invalid interface binding "foo": space provider ID is required`,
@@ -979,6 +980,13 @@ func (suite *maas2EnvironSuite) TestAllocateContainerAddressesSingleNic(c *gc.C)
 		cidr:       "192.168.1.0/24",
 		dnsServers: []string{"10.20.19.2", "10.20.19.3"},
 	}
+	staticRoute2to1 := fakeStaticRoute{
+		id:          1,
+		source:      subnet2,
+		destination: subnet1,
+		gatewayIP:   "192.168.1.1",
+		metric:      100,
+	}
 
 	interfaces := []gomaasapi.Interface{
 		&fakeInterface{
@@ -1041,7 +1049,8 @@ func (suite *maas2EnvironSuite) TestAllocateContainerAddressesSingleNic(c *gc.C)
 				subnets: []gomaasapi.Subnet{subnet1, subnet2},
 			},
 		},
-		devices: []gomaasapi.Device{device},
+		devices:      []gomaasapi.Device{device},
+		staticRoutes: []gomaasapi.StaticRoute{staticRoute2to1},
 	}
 	suite.injectController(controller)
 	suite.setupFakeTools(c)
@@ -1071,10 +1080,216 @@ func (suite *maas2EnvironSuite) TestAllocateContainerAddressesSingleNic(c *gc.C)
 		DNSServers:        network.NewAddressesOnSpace("freckles", "10.20.19.2", "10.20.19.3"),
 		MTU:               1500,
 		GatewayAddress:    network.NewAddressOnSpace("freckles", "192.168.1.1"),
+		Routes: []network.Route{{
+			DestinationCIDR: subnet1.CIDR(),
+			GatewayIP:       "192.168.1.1",
+			Metric:          100,
+		}},
 	}}
 	c.Assert(result, jc.DeepEquals, expected)
 }
 
+func (suite *maas2EnvironSuite) TestAllocateContainerAddressesNoStaticRoutesAPI(c *gc.C) {
+	// MAAS 2.0 doesn't have support for static routes, and generates an Error
+	vlan1 := fakeVLAN{
+		id:  5001,
+		mtu: 1500,
+	}
+	subnet1 := fakeSubnet{
+		id:         3,
+		space:      "freckles",
+		vlan:       vlan1,
+		gateway:    "10.20.19.2",
+		cidr:       "10.20.19.0/24",
+		dnsServers: []string{"10.20.19.2", "10.20.19.3"},
+	}
+
+	interfaces := []gomaasapi.Interface{
+		&fakeInterface{
+			id:         91,
+			name:       "eth0",
+			type_:      "physical",
+			enabled:    true,
+			macAddress: "52:54:00:70:9b:fe",
+			vlan:       vlan1,
+			links: []gomaasapi.Link{
+				&fakeLink{
+					id:        436,
+					subnet:    &subnet1,
+					ipAddress: "10.20.19.103",
+					mode:      "static",
+				},
+			},
+			parents:  []string{},
+			children: []string{"eth0.100", "eth0.250", "eth0.50"},
+		},
+	}
+	// This will be returned by the fakeController when we call CreateDevice
+	deviceInterfaces := []gomaasapi.Interface{
+		&fakeInterface{
+			id:         93,
+			name:       "eth0",
+			type_:      "physical",
+			enabled:    true,
+			macAddress: "53:54:00:70:9b:ff",
+			vlan:       vlan1,
+			links: []gomaasapi.Link{
+				&fakeLink{
+					id:        480,
+					subnet:    &subnet1,
+					ipAddress: "10.20.19.104",
+					mode:      "static",
+				},
+			},
+			parents:  []string{},
+			children: []string{},
+		},
+	}
+	stub := &testing.Stub{}
+	device := &fakeDevice{
+		Stub:         stub,
+		interfaceSet: deviceInterfaces,
+		systemID:     "foo",
+	}
+	// MAAS 2.0 gives us this kind of error back, I'm not sure of the conten of
+	// the Headers or BodyMessage, but it is a 404 with a particular error
+	// string that we've seen.
+	body := "Unknown API endpoint: /MAAS/api/2.0/static-routes/."
+	notFound := gomaasapi.ServerError{
+		StatusCode:  http.StatusNotFound,
+		BodyMessage: body,
+	}
+	wrap1 := errors.Annotatef(notFound, "ServerError: 404 NOT FOUND (%s)", body)
+	staticRoutesErr := gomaasapi.NewUnexpectedError(wrap1)
+	var env *maasEnviron
+	controller := &fakeController{
+		Stub: stub,
+		machines: []gomaasapi.Machine{&fakeMachine{
+			Stub:         stub,
+			systemID:     "1",
+			architecture: arch.HostArch(),
+			interfaceSet: interfaces,
+			createDevice: device,
+		}},
+		spaces: []gomaasapi.Space{
+			fakeSpace{
+				name:    "freckles",
+				id:      4567,
+				subnets: []gomaasapi.Subnet{subnet1},
+			},
+		},
+		devices:           []gomaasapi.Device{device},
+		staticRoutesError: staticRoutesErr,
+	}
+	suite.injectController(controller)
+	suite.setupFakeTools(c)
+	env = suite.makeEnviron(c, nil)
+
+	prepared := []network.InterfaceInfo{{
+		MACAddress:    "52:54:00:70:9b:fe",
+		CIDR:          "10.20.19.0/24",
+		InterfaceName: "eth0",
+	}}
+	ignored := names.NewMachineTag("1/lxd/0")
+	result, err := env.AllocateContainerAddresses(instance.Id("1"), ignored, prepared)
+	c.Assert(err, jc.ErrorIsNil)
+	expected := []network.InterfaceInfo{{
+		DeviceIndex:       0,
+		MACAddress:        "53:54:00:70:9b:ff",
+		CIDR:              "10.20.19.0/24",
+		ProviderId:        "93",
+		ProviderSubnetId:  "3",
+		VLANTag:           0,
+		ProviderVLANId:    "5001",
+		ProviderAddressId: "480",
+		InterfaceName:     "eth0",
+		InterfaceType:     "ethernet",
+		ConfigType:        "static",
+		Address:           network.NewAddressOnSpace("freckles", "10.20.19.104"),
+		DNSServers:        network.NewAddressesOnSpace("freckles", "10.20.19.2", "10.20.19.3"),
+		MTU:               1500,
+		GatewayAddress:    network.NewAddressOnSpace("freckles", "10.20.19.2"),
+		Routes:            []network.Route{},
+	}}
+	c.Assert(result, jc.DeepEquals, expected)
+}
+
+func (suite *maas2EnvironSuite) TestAllocateContainerAddressesStaticRoutesDenied(c *gc.C) {
+	// I don't have a specific error that we've triggered, but we want to make
+	// sure that we don't suppress all error responses from MAAS just because
+	// we know we want to skip 404
+	vlan1 := fakeVLAN{
+		id:  5001,
+		mtu: 1500,
+	}
+	subnet1 := fakeSubnet{
+		id:         3,
+		space:      "freckles",
+		vlan:       vlan1,
+		gateway:    "10.20.19.2",
+		cidr:       "10.20.19.0/24",
+		dnsServers: []string{"10.20.19.2", "10.20.19.3"},
+	}
+
+	interfaces := []gomaasapi.Interface{
+		&fakeInterface{
+			id:         91,
+			name:       "eth0",
+			type_:      "physical",
+			enabled:    true,
+			macAddress: "52:54:00:70:9b:fe",
+			vlan:       vlan1,
+			links: []gomaasapi.Link{
+				&fakeLink{
+					id:        436,
+					subnet:    &subnet1,
+					ipAddress: "10.20.19.103",
+					mode:      "static",
+				},
+			},
+			parents:  []string{},
+			children: []string{"eth0.100", "eth0.250", "eth0.50"},
+		},
+	}
+	body := "I have failed you"
+	internalError := gomaasapi.ServerError{
+		StatusCode:  http.StatusInternalServerError,
+		BodyMessage: body,
+	}
+	staticRoutesErr := errors.Annotatef(internalError, "ServerError: %v (%s)", http.StatusInternalServerError, body)
+	var env *maasEnviron
+	controller := &fakeController{
+		Stub: &testing.Stub{},
+		machines: []gomaasapi.Machine{&fakeMachine{
+			Stub:         &testing.Stub{},
+			systemID:     "1",
+			architecture: arch.HostArch(),
+			interfaceSet: interfaces,
+		}},
+		spaces: []gomaasapi.Space{
+			fakeSpace{
+				name:    "freckles",
+				id:      4567,
+				subnets: []gomaasapi.Subnet{subnet1},
+			},
+		},
+		staticRoutesError: staticRoutesErr,
+	}
+	suite.injectController(controller)
+	suite.setupFakeTools(c)
+	env = suite.makeEnviron(c, nil)
+
+	prepared := []network.InterfaceInfo{{
+		MACAddress:    "52:54:00:70:9b:fe",
+		CIDR:          "10.20.19.0/24",
+		InterfaceName: "eth0",
+	}}
+	ignored := names.NewMachineTag("1/lxd/0")
+	_, err := env.AllocateContainerAddresses(instance.Id("1"), ignored, prepared)
+	c.Assert(err, gc.NotNil)
+	c.Assert(err, gc.ErrorMatches, ".*ServerError: 500 \\(I have failed you\\).*")
+}
+
 func (suite *maas2EnvironSuite) TestAllocateContainerAddressesDualNic(c *gc.C) {
 	vlan1 := fakeVLAN{
 		id:  5001,
@@ -1100,6 +1315,13 @@ func (suite *maas2EnvironSuite) TestAllocateContainerAddressesDualNic(c *gc.C) {
 		cidr:       "192.168.1.0/24",
 		dnsServers: []string{"10.20.19.2", "10.20.19.3"},
 	}
+	staticRoute2to1 := fakeStaticRoute{
+		id:          1,
+		source:      subnet2,
+		destination: subnet1,
+		gatewayIP:   "192.168.1.1",
+		metric:      100,
+	}
 
 	interfaces := []gomaasapi.Interface{
 		&fakeInterface{
@@ -1197,7 +1419,8 @@ func (suite *maas2EnvironSuite) TestAllocateContainerAddressesDualNic(c *gc.C) {
 				subnets: []gomaasapi.Subnet{subnet1, subnet2},
 			},
 		},
-		devices: []gomaasapi.Device{device},
+		devices:      []gomaasapi.Device{device},
+		staticRoutes: []gomaasapi.StaticRoute{staticRoute2to1},
 	}
 	suite.injectController(controller)
 	env := suite.makeEnviron(c, nil)
@@ -1227,7 +1450,7 @@ func (suite *maas2EnvironSuite) TestAllocateContainerAddressesDualNic(c *gc.C) {
 		MTU:               1500,
 		GatewayAddress:    network.NewAddressOnSpace("freckles", "10.20.19.2"),
 	}, {
-		DeviceIndex:       0,
+		DeviceIndex:       1,
 		MACAddress:        "52:54:00:70:9b:f4",
 		CIDR:              "192.168.1.0/24",
 		ProviderId:        "94",
@@ -1241,6 +1464,11 @@ func (suite *maas2EnvironSuite) TestAllocateContainerAddressesDualNic(c *gc.C) {
 		DNSServers:        network.NewAddressesOnSpace("freckles", "10.20.19.2", "10.20.19.3"),
 		MTU:               1500,
 		GatewayAddress:    network.NewAddressOnSpace("freckles", "192.168.1.1"),
+		Routes: []network.Route{{
+			DestinationCIDR: "10.20.19.0/24",
+			GatewayIP:       "192.168.1.1",
+			Metric:          100,
+		}},
 	}}
 	ignored := names.NewMachineTag("1/lxd/0")
 	result, err := env.AllocateContainerAddresses(instance.Id("1"), ignored, prepared)
@@ -1260,12 +1488,25 @@ func (suite *maas2EnvironSuite) assertAllocateContainerAddressesFails(c *gc.C, c
 }
 
 func (suite *maas2EnvironSuite) TestAllocateContainerAddressesSpacesError(c *gc.C) {
-	controller := &fakeController{spacesError: errors.New("boom")}
+	machine := &fakeMachine{
+		Stub:     &testing.Stub{},
+		systemID: "1",
+	}
+	controller := &fakeController{
+		machines:    []gomaasapi.Machine{machine},
+		spacesError: errors.New("boom"),
+	}
 	suite.assertAllocateContainerAddressesFails(c, controller, nil, "boom")
 }
 
 func (suite *maas2EnvironSuite) TestAllocateContainerAddressesPrimaryInterfaceMissing(c *gc.C) {
-	controller := &fakeController{}
+	machine := &fakeMachine{
+		Stub:     &testing.Stub{},
+		systemID: "1",
+	}
+	controller := &fakeController{
+		machines: []gomaasapi.Machine{machine},
+	}
 	suite.assertAllocateContainerAddressesFails(c, controller, nil, "cannot find primary interface for container")
 }
 
@@ -1315,14 +1556,14 @@ func getArgs(c *gc.C, calls []testing.StubCall) interface{} {
 	return args[0]
 }
 
-func (suite *maas2EnvironSuite) TestAllocateContainerAddressesCreateDevicerror(c *gc.C) {
+func (suite *maas2EnvironSuite) TestAllocateContainerAddressesCreateDeviceError(c *gc.C) {
 	subnet := makeFakeSubnet(3)
 	var env *maasEnviron
 	machine := &fakeMachine{
 		Stub:     &testing.Stub{},
 		systemID: "1",
 	}
-	machine.SetErrors(errors.New("boom"))
+	machine.SetErrors(nil, errors.New("bad device call"))
 	controller := &fakeController{
 		machines: []gomaasapi.Machine{machine},
 		spaces: []gomaasapi.Space{
@@ -1340,17 +1581,16 @@ func (suite *maas2EnvironSuite) TestAllocateContainerAddressesCreateDevicerror(c
 	}
 	ignored := names.NewMachineTag("1/lxd/0")
 	_, err := env.AllocateContainerAddresses(instance.Id("1"), ignored, prepared)
-	c.Assert(err, gc.ErrorMatches, "boom")
-	args := getArgs(c, machine.Calls())
-	maasArgs, ok := args.(gomaasapi.CreateMachineDeviceArgs)
-	c.Assert(ok, jc.IsTrue)
-	expected := gomaasapi.CreateMachineDeviceArgs{
+	c.Assert(err, gc.ErrorMatches, `failed to create MAAS device for "juju-06f00d-1-lxd-0": bad device call`)
+	machine.CheckCall(c, 0, "Devices", gomaasapi.DevicesArgs{
+		Hostname: []string{"juju-06f00d-1-lxd-0"},
+	})
+	machine.CheckCall(c, 1, "CreateDevice", gomaasapi.CreateMachineDeviceArgs{
 		Hostname:      "juju-06f00d-1-lxd-0",
 		Subnet:        subnet,
 		MACAddress:    "DEADBEEF",
 		InterfaceName: "eth0",
-	}
-	c.Assert(maasArgs, jc.DeepEquals, expected)
+	})
 }
 
 func (suite *maas2EnvironSuite) TestAllocateContainerAddressesSubnetMissing(c *gc.C) {
@@ -1423,6 +1663,7 @@ func (suite *maas2EnvironSuite) TestAllocateContainerAddressesSubnetMissing(c *g
 	allocated, err := env.AllocateContainerAddresses(instance.Id("1"), ignored, prepared)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(allocated, jc.DeepEquals, []network.InterfaceInfo{{
+		DeviceIndex:    0,
 		MACAddress:     "53:54:00:70:9b:ff",
 		ProviderId:     "93",
 		ProviderVLANId: "0",
@@ -1434,6 +1675,7 @@ func (suite *maas2EnvironSuite) TestAllocateContainerAddressesSubnetMissing(c *g
 		ConfigType:     "manual",
 		MTU:            1500,
 	}, {
+		DeviceIndex:    1,
 		MACAddress:     "53:54:00:70:9b:f1",
 		ProviderId:     "94",
 		ProviderVLANId: "0",
@@ -1481,7 +1723,7 @@ func (suite *maas2EnvironSuite) TestAllocateContainerAddressesCreateInterfaceErr
 	}
 	ignored := names.NewMachineTag("1/lxd/0")
 	_, err := env.AllocateContainerAddresses(instance.Id("1"), ignored, prepared)
-	c.Assert(err, gc.ErrorMatches, "creating device interface: boom")
+	c.Assert(err, gc.ErrorMatches, `failed to create MAAS device for "juju-06f00d-1-lxd-0": creating device interface: boom`)
 	args := getArgs(c, device.Calls())
 	maasArgs, ok := args.(gomaasapi.CreateInterfaceArgs)
 	c.Assert(ok, jc.IsTrue)
@@ -1533,6 +1775,7 @@ func (suite *maas2EnvironSuite) TestAllocateContainerAddressesLinkSubnetError(c
 	allocated, err := env.AllocateContainerAddresses(instance.Id("1"), ignored, prepared)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(allocated, jc.DeepEquals, []network.InterfaceInfo{{
+		DeviceIndex:      0,
 		CIDR:             "",
 		ProviderId:       "0",
 		ProviderSubnetId: "",
@@ -1545,6 +1788,7 @@ func (suite *maas2EnvironSuite) TestAllocateContainerAddressesLinkSubnetError(c
 		Disabled:         true,
 		NoAutoStart:      true,
 	}, {
+		DeviceIndex:      1,
 		CIDR:             "",
 		ProviderId:       "0",
 		ProviderSubnetId: "",
@@ -1567,6 +1811,7 @@ func (suite *maas2EnvironSuite) TestAllocateContainerAddressesLinkSubnetError(c
 	}
 	c.Assert(maasArgs, jc.DeepEquals, expected)
 }
+
 func (suite *maas2EnvironSuite) TestStorageReturnsStorage(c *gc.C) {
 	controller := newFakeController()
 	env := suite.makeEnviron(c, controller)
@@ -1581,6 +1826,334 @@ func (suite *maas2EnvironSuite) TestStorageReturnsStorage(c *gc.C) {
 	c.Check(specificStorage.maasController, gc.Equals, controller)
 }
 
+func (suite *maas2EnvironSuite) TestAllocateContainerReuseExistingDevice(c *gc.C) {
+	stub := &testing.Stub{}
+	vlan1 := fakeVLAN{
+		id:  5001,
+		mtu: 1500,
+	}
+	subnet1 := fakeSubnet{
+		id:         3,
+		space:      "space-1",
+		vlan:       vlan1,
+		gateway:    "10.20.19.2",
+		cidr:       "10.20.19.0/24",
+		dnsServers: []string{"10.20.19.2", "10.20.19.3"},
+	}
+	interfaces := []gomaasapi.Interface{
+		&fakeInterface{
+			id:         91,
+			name:       "eth0",
+			type_:      "physical",
+			enabled:    true,
+			macAddress: "52:54:00:70:9b:fe",
+			vlan:       vlan1,
+			links: []gomaasapi.Link{
+				&fakeLink{
+					id:        436,
+					subnet:    &subnet1,
+					ipAddress: "10.20.19.103",
+					mode:      "static",
+				},
+			},
+			parents: []string{},
+		},
+	}
+	deviceInterfaces := []gomaasapi.Interface{
+		&fakeInterface{
+			id:         93,
+			name:       "eth0",
+			type_:      "physical",
+			enabled:    true,
+			macAddress: "53:54:00:70:9b:ff",
+			vlan:       vlan1,
+			links: []gomaasapi.Link{
+				&fakeLink{
+					id:        480,
+					subnet:    &subnet1,
+					ipAddress: "10.20.19.105",
+					mode:      "static",
+				},
+			},
+			parents: []string{},
+		},
+	}
+	var env *maasEnviron
+	device := &fakeDevice{
+		Stub:         stub,
+		interfaceSet: deviceInterfaces,
+		systemID:     "foo",
+	}
+	controller := &fakeController{
+		Stub: stub,
+		machines: []gomaasapi.Machine{&fakeMachine{
+			Stub:         stub,
+			systemID:     "1",
+			architecture: arch.HostArch(),
+			interfaceSet: interfaces,
+			// Instead of having createDevice return it, Devices()
+			// returns it from the beginning
+			createDevice: nil,
+			devices:      []gomaasapi.Device{device},
+		}},
+		spaces: []gomaasapi.Space{
+			fakeSpace{
+				name:    "space-1",
+				id:      4567,
+				subnets: []gomaasapi.Subnet{subnet1},
+			},
+		},
+		devices: []gomaasapi.Device{device},
+	}
+	suite.injectController(controller)
+	suite.setupFakeTools(c)
+	env = suite.makeEnviron(c, nil)
+
+	prepared := []network.InterfaceInfo{{
+		MACAddress:    "53:54:00:70:9b:ff",
+		CIDR:          "10.20.19.0/24",
+		InterfaceName: "eth0",
+	}}
+	containerTag := names.NewMachineTag("1/lxd/0")
+	result, err := env.AllocateContainerAddresses(instance.Id("1"), containerTag, prepared)
+	c.Assert(err, jc.ErrorIsNil)
+	expected := []network.InterfaceInfo{{
+		DeviceIndex:       0,
+		MACAddress:        "53:54:00:70:9b:ff",
+		CIDR:              "10.20.19.0/24",
+		ProviderId:        "93",
+		ProviderSubnetId:  "3",
+		VLANTag:           0,
+		ProviderVLANId:    "5001",
+		ProviderAddressId: "480",
+		InterfaceName:     "eth0",
+		InterfaceType:     "ethernet",
+		ConfigType:        "static",
+		Address:           network.NewAddressOnSpace("space-1", "10.20.19.105"),
+		DNSServers:        network.NewAddressesOnSpace("space-1", "10.20.19.2", "10.20.19.3"),
+		MTU:               1500,
+		GatewayAddress:    network.NewAddressOnSpace("space-1", "10.20.19.2"),
+		Routes:            []network.Route{},
+	}}
+	c.Assert(result, jc.DeepEquals, expected)
+}
+
+func (suite *maas2EnvironSuite) TestAllocateContainerRefusesReuseInvalidNIC(c *gc.C) {
+	vlan1 := fakeVLAN{
+		id:  5001,
+		mtu: 1500,
+	}
+	vlan2 := fakeVLAN{
+		id:  5002,
+		mtu: 1500,
+	}
+	subnet1 := fakeSubnet{
+		id:         3,
+		space:      "freckles",
+		vlan:       vlan1,
+		gateway:    "10.20.19.2",
+		cidr:       "10.20.19.0/24",
+		dnsServers: []string{"10.20.19.2", "10.20.19.3"},
+	}
+	subnet2 := fakeSubnet{
+		id:         4,
+		space:      "freckles",
+		vlan:       vlan2,
+		gateway:    "192.168.1.1",
+		cidr:       "192.168.1.0/24",
+		dnsServers: []string{"192.168.1.2"},
+	}
+	subnet3 := fakeSubnet{
+		id:         5,
+		space:      "freckles",
+		vlan:       vlan2,
+		gateway:    "192.168.1.1",
+		cidr:       "192.168.2.0/24",
+		dnsServers: []string{"192.168.1.2"},
+	}
+	interfaces := []gomaasapi.Interface{
+		&fakeInterface{
+			id:         91,
+			name:       "eth0",
+			type_:      "physical",
+			enabled:    true,
+			macAddress: "52:54:00:70:9b:fe",
+			vlan:       vlan1,
+			links: []gomaasapi.Link{
+				&fakeLink{
+					id:        436,
+					subnet:    &subnet1,
+					ipAddress: "10.20.19.103",
+					mode:      "static",
+				},
+			},
+			parents: []string{},
+		},
+		&fakeInterface{
+			id:         92,
+			name:       "eth1",
+			type_:      "physical",
+			enabled:    true,
+			macAddress: "52:54:00:70:9b:ff",
+			vlan:       vlan2,
+			links: []gomaasapi.Link{
+				&fakeLink{
+					id:        437,
+					subnet:    &subnet2,
+					ipAddress: "192.168.1.100",
+					mode:      "static",
+				},
+			},
+			parents: []string{},
+		},
+	}
+	badDeviceInterfaces := []gomaasapi.Interface{
+		&fakeInterface{
+			id:         93,
+			name:       "eth0",
+			type_:      "physical",
+			enabled:    true,
+			macAddress: "53:54:00:70:88:aa",
+			vlan:       vlan1,
+			links: []gomaasapi.Link{
+				&fakeLink{
+					id:        480,
+					subnet:    &subnet1,
+					ipAddress: "10.20.19.105",
+					mode:      "static",
+				},
+			},
+			parents: []string{},
+		},
+		// This interface is linked to the wrong subnet
+		&fakeInterface{
+			id:         94,
+			name:       "eth1",
+			type_:      "physical",
+			enabled:    true,
+			macAddress: "53:54:00:70:88:bb",
+			vlan:       vlan1,
+			links: []gomaasapi.Link{
+				&fakeLink{
+					id:        481,
+					subnet:    &subnet3,
+					ipAddress: "192.168.2.100",
+					mode:      "static",
+				},
+			},
+			parents: []string{},
+		},
+	}
+	goodSecondInterface := &fakeInterface{
+		id:         94,
+		name:       "eth1",
+		type_:      "physical",
+		enabled:    true,
+		macAddress: "53:54:00:70:88:bb",
+		vlan:       vlan2,
+		links: []gomaasapi.Link{
+			&fakeLink{
+				id:        481,
+				subnet:    &subnet2,
+				ipAddress: "192.168.1.101",
+				mode:      "static",
+			},
+		},
+		parents: []string{},
+	}
+	goodDeviceInterfaces := []gomaasapi.Interface{
+		badDeviceInterfaces[0],
+	}
+	var env *maasEnviron
+	stub := &testing.Stub{}
+	badDevice := &fakeDevice{
+		Stub:         stub,
+		interfaceSet: badDeviceInterfaces,
+		systemID:     "foo",
+	}
+	goodDevice := &fakeDevice{
+		Stub:         stub,
+		interfaceSet: goodDeviceInterfaces,
+		systemID:     "foo",
+		interface_:   goodSecondInterface,
+	}
+	machine := &fakeMachine{
+		Stub:         stub,
+		systemID:     "1",
+		architecture: arch.HostArch(),
+		interfaceSet: interfaces,
+		createDevice: goodDevice,
+		// Devices will first list the bad device, and then
+		// createDevice will create the right one
+		devices: []gomaasapi.Device{badDevice},
+	}
+	badDevice.deleteCB = func() { machine.devices = machine.devices[:0] }
+	controller := &fakeController{
+		Stub:     stub,
+		machines: []gomaasapi.Machine{machine},
+		spaces: []gomaasapi.Space{
+			fakeSpace{
+				name:    "space-1",
+				id:      4567,
+				subnets: []gomaasapi.Subnet{subnet1},
+			},
+		},
+		devices: []gomaasapi.Device{goodDevice},
+	}
+	suite.injectController(controller)
+	suite.setupFakeTools(c)
+	env = suite.makeEnviron(c, nil)
+
+	prepared := []network.InterfaceInfo{{
+		MACAddress:    "53:54:00:70:88:aa",
+		CIDR:          "10.20.19.0/24",
+		InterfaceName: "eth0",
+	}, {
+		MACAddress:    "53:54:00:70:88:bb",
+		CIDR:          "192.168.1.0/24",
+		InterfaceName: "eth1",
+	}}
+	containerTag := names.NewMachineTag("1/lxd/0")
+	result, err := env.AllocateContainerAddresses(instance.Id("1"), containerTag, prepared)
+	c.Assert(err, jc.ErrorIsNil)
+	expected := []network.InterfaceInfo{{
+		DeviceIndex:       0,
+		MACAddress:        "53:54:00:70:88:aa",
+		CIDR:              "10.20.19.0/24",
+		ProviderId:        "93",
+		ProviderSubnetId:  "3",
+		VLANTag:           0,
+		ProviderVLANId:    "5001",
+		ProviderAddressId: "480",
+		InterfaceName:     "eth0",
+		InterfaceType:     "ethernet",
+		ConfigType:        "static",
+		Address:           network.NewAddressOnSpace("freckles", "10.20.19.105"),
+		DNSServers:        network.NewAddressesOnSpace("freckles", "10.20.19.2", "10.20.19.3"),
+		MTU:               1500,
+		GatewayAddress:    network.NewAddressOnSpace("freckles", "10.20.19.2"),
+		Routes:            []network.Route{},
+	}, {
+		DeviceIndex:       1,
+		MACAddress:        "53:54:00:70:88:bb",
+		CIDR:              "192.168.1.0/24",
+		ProviderId:        "94",
+		ProviderSubnetId:  "4",
+		VLANTag:           0,
+		ProviderVLANId:    "5002",
+		ProviderAddressId: "481",
+		InterfaceName:     "eth1",
+		InterfaceType:     "ethernet",
+		ConfigType:        "static",
+		Address:           network.NewAddressOnSpace("freckles", "192.168.1.101"),
+		DNSServers:        network.NewAddressesOnSpace("freckles", "192.168.1.2"),
+		MTU:               1500,
+		GatewayAddress:    network.NewAddressOnSpace("freckles", "192.168.1.1"),
+		Routes:            []network.Route{},
+	}}
+	c.Assert(result, jc.DeepEquals, expected)
+}
+
 func (suite *maas2EnvironSuite) TestStartInstanceEndToEnd(c *gc.C) {
 	suite.setupFakeTools(c)
 	machine := newFakeMachine("gus", arch.HostArch(), "Deployed")
@@ -1837,6 +2410,50 @@ func (suite *maas2EnvironSuite) TestReleaseContainerAddressesErrorDeletingDevice
 	dev1.CheckCallNames(c, "Delete")
 }
 
+func (suite *maas2EnvironSuite) TestAdoptResources(c *gc.C) {
+	machine1 := newFakeMachine("big-fig-wasp", "gaudi", "good")
+	machine2 := newFakeMachine("robot-stop", "hundertwasser", "fine")
+	machine3 := newFakeMachine("gamma-knife", "von-neumann", "acceptable")
+	controller := newFakeController()
+	controller.machines = append(controller.machines, machine1, machine3)
+	env := suite.makeEnviron(c, controller)
+
+	err := env.AdoptResources("some-other-controller", version.MustParse("1.2.3"))
+	c.Assert(err, jc.ErrorIsNil)
+
+	machine1.CheckCallNames(c, "SetOwnerData")
+	c.Assert(machine1.Calls()[0].Args[0], gc.DeepEquals, map[string]string{
+		tags.JujuController: "some-other-controller",
+	})
+	machine2.CheckCallNames(c)
+	machine3.CheckCallNames(c, "SetOwnerData")
+	c.Assert(machine3.Calls()[0].Args[0], gc.DeepEquals, map[string]string{
+		tags.JujuController: "some-other-controller",
+	})
+}
+
+func (suite *maas2EnvironSuite) TestAdoptResourcesError(c *gc.C) {
+	machine1 := newFakeMachine("evil-death-roll", "frank-lloyd-wright", "ok")
+	machine2 := newFakeMachine("people-vultures", "gehry", "adequate")
+	controller := newFakeController()
+	controller.machines = append(controller.machines, machine1, machine2)
+	env := suite.makeEnviron(c, controller)
+
+	machine1.SetErrors(errors.New("blorp"))
+
+	err := env.AdoptResources("some-other-controller", version.MustParse("3.2.1"))
+	c.Assert(err, gc.ErrorMatches, `failed to update controller for some instances: \[evil-death-roll\]`)
+
+	machine1.CheckCallNames(c, "SetOwnerData")
+	c.Assert(machine1.Calls()[0].Args[0], gc.DeepEquals, map[string]string{
+		tags.JujuController: "some-other-controller",
+	})
+	machine2.CheckCallNames(c, "SetOwnerData")
+	c.Assert(machine2.Calls()[0].Args[0], gc.DeepEquals, map[string]string{
+		tags.JujuController: "some-other-controller",
+	})
+}
+
 func newFakeDevice(systemID, macAddress string) *fakeDevice {
 	return &fakeDevice{
 		Stub:     &testing.Stub{},
