@@ -10,6 +10,7 @@ import (
 	"path/filepath"
 	"strconv"
 
+	"github.com/juju/errors"
 	"github.com/juju/gomaasapi"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
@@ -62,7 +63,7 @@ func (s *baseProviderSuite) SetUpTest(c *gc.C) {
 	s.ToolsFixture.SetUpTest(c)
 	s.PatchValue(&jujuversion.Current, coretesting.FakeVersionNumber)
 	s.PatchValue(&arch.HostArch, func() string { return arch.AMD64 })
-	s.PatchValue(&series.HostSeries, func() string { return series.LatestLts() })
+	s.PatchValue(&series.MustHostSeries, func() string { return series.LatestLts() })
 }
 
 func (s *baseProviderSuite) TearDownTest(c *gc.C) {
@@ -95,13 +96,9 @@ func (s *providerSuite) SetUpSuite(c *gc.C) {
 
 func (s *providerSuite) SetUpTest(c *gc.C) {
 	s.baseProviderSuite.SetUpTest(c)
-	mockCapabilities := func(*gomaasapi.MAASObject, string) (set.Strings, error) {
-		return set.NewStrings("network-deployment-ubuntu"), nil
-	}
 	mockGetController := func(string, string) (gomaasapi.Controller, error) {
 		return nil, gomaasapi.NewUnsupportedVersionError("oops")
 	}
-	s.PatchValue(&GetCapabilities, mockCapabilities)
 	s.PatchValue(&GetMAAS2Controller, mockGetController)
 	// Creating a space ensures that the spaces endpoint won't 404.
 	s.testMAASObject.TestServer.NewSpace(spaceJSON(gomaasapi.CreateSpace{Name: "space-0"}))
@@ -125,24 +122,37 @@ var maasEnvAttrs = coretesting.Attrs{
 	},
 }
 
-// makeEnviron creates a functional maasEnviron for a test.
-func (suite *providerSuite) makeEnviron() *maasEnviron {
+func (suite *providerSuite) makeEnvironWithURL(url string, getCapabilities MaasCapabilities) (*maasEnviron, error) {
 	cred := cloud.NewCredential(cloud.OAuth1AuthType, map[string]string{
 		"maas-oauth": "a:b:c",
 	})
 	cloud := environs.CloudSpec{
 		Type:       "maas",
 		Name:       "maas",
-		Endpoint:   suite.testMAASObject.TestServer.URL,
+		Endpoint:   url,
 		Credential: &cred,
 	}
 	attrs := coretesting.FakeConfig().Merge(maasEnvAttrs)
 	suite.controllerUUID = coretesting.FakeControllerConfig().ControllerUUID()
 	cfg, err := config.New(config.NoDefaults, attrs)
 	if err != nil {
-		panic(err)
+		return nil, errors.Trace(err)
 	}
-	env, err := NewEnviron(cloud, cfg)
+	env, err := NewEnviron(cloud, cfg, getCapabilities)
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	return env, nil
+}
+
+// makeEnviron creates a functional maasEnviron for a test.
+func (suite *providerSuite) makeEnviron() *maasEnviron {
+	env, err := suite.makeEnvironWithURL(
+		suite.testMAASObject.TestServer.URL,
+		func(client *gomaasapi.MAASObject, serverURL string) (set.Strings, error) {
+			return set.NewStrings("network-deployment-ubuntu"), nil
+		},
+	)
 	if err != nil {
 		panic(err)
 	}
