@@ -18,6 +18,7 @@ import (
 	"github.com/juju/utils/arch"
 	"github.com/juju/utils/featureflag"
 	"github.com/juju/utils/ssh"
+	"github.com/juju/version"
 
 	"github.com/juju/juju/agent"
 	"github.com/juju/juju/cloudconfig/instancecfg"
@@ -25,13 +26,13 @@ import (
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/config"
 	"github.com/juju/juju/environs/manual"
+	"github.com/juju/juju/environs/manual/sshprovisioner"
 	"github.com/juju/juju/feature"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju/names"
 	"github.com/juju/juju/mongo"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/provider/common"
-	"github.com/juju/juju/worker/terminationworker"
 )
 
 const (
@@ -41,8 +42,7 @@ const (
 )
 
 var (
-	logger                 = loggo.GetLogger("juju.provider.manual")
-	manualCheckProvisioned = manual.CheckProvisioned
+	logger = loggo.GetLogger("juju.provider.manual")
 )
 
 type manualEnviron struct {
@@ -102,7 +102,7 @@ func (e *manualEnviron) Create(environs.CreateParams) error {
 
 // Bootstrap is part of the Environ interface.
 func (e *manualEnviron) Bootstrap(ctx environs.BootstrapContext, args environs.BootstrapParams) (*environs.BootstrapResult, error) {
-	provisioned, err := manualCheckProvisioned(e.host)
+	provisioned, err := sshprovisioner.CheckProvisioned(e.host)
 	if err != nil {
 		return nil, errors.Annotate(err, "failed to check provisioned status")
 	}
@@ -119,7 +119,7 @@ func (e *manualEnviron) Bootstrap(ctx environs.BootstrapContext, args environs.B
 		if err := instancecfg.FinishInstanceConfig(icfg, e.Config()); err != nil {
 			return err
 		}
-		return common.ConfigureMachine(ctx, ssh.DefaultClient, e.host, icfg)
+		return common.ConfigureMachine(ctx, ssh.DefaultClient, e.host, icfg, nil)
 	}
 
 	result := &environs.BootstrapResult{
@@ -130,11 +130,6 @@ func (e *manualEnviron) Bootstrap(ctx environs.BootstrapContext, args environs.B
 	return result, nil
 }
 
-// BootstrapMessage is part of the Environ interface.
-func (e *manualEnviron) BootstrapMessage() string {
-	return ""
-}
-
 // ControllerInstances is specified in the Environ interface.
 func (e *manualEnviron) ControllerInstances(controllerUUID string) ([]instance.Id, error) {
 	if !isRunningController() {
@@ -178,10 +173,16 @@ func (e *manualEnviron) verifyBootstrapHost() error {
 	return nil
 }
 
+// AdoptResources is part of the Environ interface.
+func (e *manualEnviron) AdoptResources(controllerUUID string, fromVersion version.Number) error {
+	// This provider doesn't track instance -> controller.
+	return nil
+}
+
 func (e *manualEnviron) SetConfig(cfg *config.Config) error {
 	e.mu.Lock()
 	defer e.mu.Unlock()
-	_, err := manualProvider{}.validate(cfg, e.cfg.Config)
+	_, err := ManualProvider{}.validate(cfg, e.cfg.Config)
 	if err != nil {
 		return err
 	}
@@ -237,29 +238,47 @@ func (e *manualEnviron) Destroy() error {
 // DestroyController implements the Environ interface.
 func (e *manualEnviron) DestroyController(controllerUUID string) error {
 	script := `
+# Signal the jujud process to stop, then check it has done so before cleaning-up
+# after it.
 set -x
-touch %s
-# If jujud is running, we then wait for a while for it to stop.
+touch %[1]s
+
 stopped=0
-if pkill -%d jujud; then
-    for i in ` + "`seq 1 30`" + `; do
+function wait_for_jujud {
+    for i in {1..30}; do
         if pgrep jujud > /dev/null ; then
             sleep 1
         else
-            echo "jujud stopped"
+            echo jujud stopped
             stopped=1
+            logger --id jujud stopped on attempt $i
             break
         fi
     done
-fi
-if [ $stopped -ne 1 ]; then
+}
+
+# There might be no jujud at all (for example, after a failed deployment) so
+# don't require pkill to succeed before looking for a jujud process.
+# SIGABRT not SIGTERM, as abort lets the worker know it should uninstall itself,
+# rather than terminate normally.
+pkill -SIGABRT jujud
+wait_for_jujud
+
+[[ $stopped -ne 1 ]] && {
     # If jujud didn't stop nicely, we kill it hard here.
-    %spkill -9 jujud
-    service %s stop
-fi
+    %[2]spkill -SIGKILL jujud && wait_for_jujud
+}
+[[ $stopped -ne 1 ]] && {
+    echo jujud removal failed
+    logger --id $(ps -o pid,cmd,state -p $(pgrep jujud) | awk 'NR != 1 {printf("Process %%d (%%s) has state %%s\n", $1, $2, $3)}')
+    exit 1
+}
+service %[3]s stop && logger --id stopped %[3]s
+apt-get -y purge juju-mongo*
+apt-get -y autoremove
 rm -f /etc/init/juju*
-rm -f /etc/systemd/system/juju*
-rm -fr %s %s
+rm -f /etc/systemd/system{,/multi-user.target.wants}/juju*
+rm -fr %[4]s %[5]s
 exit 0
 `
 	var diagnostics string
@@ -280,7 +299,6 @@ exit 0
 			agent.DefaultPaths.DataDir,
 			agent.UninstallFile,
 		)),
-		terminationworker.TerminationSignal,
 		diagnostics,
 		mongo.ServiceName,
 		utils.ShQuote(agent.DefaultPaths.DataDir),
@@ -331,7 +349,7 @@ func (e *manualEnviron) seriesAndHardwareCharacteristics() (_ *instance.Hardware
 	if e.hw != nil {
 		return e.hw, e.series, nil
 	}
-	hw, series, err := manual.DetectSeriesAndHardwareCharacteristics(e.host)
+	hw, series, err := sshprovisioner.DetectSeriesAndHardwareCharacteristics(e.host)
 	if err != nil {
 		return nil, "", errors.Trace(err)
 	}
@@ -339,20 +357,20 @@ func (e *manualEnviron) seriesAndHardwareCharacteristics() (_ *instance.Hardware
 	return e.hw, e.series, nil
 }
 
-func (e *manualEnviron) OpenPorts(ports []network.PortRange) error {
+func (e *manualEnviron) OpenPorts(rules []network.IngressRule) error {
 	return nil
 }
 
-func (e *manualEnviron) ClosePorts(ports []network.PortRange) error {
+func (e *manualEnviron) ClosePorts(rules []network.IngressRule) error {
 	return nil
 }
 
-func (e *manualEnviron) Ports() ([]network.PortRange, error) {
+func (e *manualEnviron) IngressRules() ([]network.IngressRule, error) {
 	return nil, nil
 }
 
 func (*manualEnviron) Provider() environs.EnvironProvider {
-	return manualProvider{}
+	return ManualProvider{}
 }
 
 func isRunningController() bool {
