@@ -12,9 +12,9 @@ import (
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/goose.v1/cinder"
-	"gopkg.in/goose.v1/identity"
-	"gopkg.in/goose.v1/nova"
+	"gopkg.in/goose.v2/cinder"
+	"gopkg.in/goose.v2/identity"
+	"gopkg.in/goose.v2/nova"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/environs/tags"
@@ -473,6 +473,7 @@ type mockAdapter struct {
 	volumeStatusNotifier  func(string, string, int, time.Duration) <-chan error
 	detachVolume          func(string, string) error
 	listVolumeAttachments func(string) ([]nova.VolumeAttachment, error)
+	setVolumeMetadata     func(string, map[string]string) (map[string]string, error)
 }
 
 func (ma *mockAdapter) GetVolume(volumeId string) (*cinder.Volume, error) {
@@ -534,6 +535,14 @@ func (ma *mockAdapter) ListVolumeAttachments(serverId string) ([]nova.VolumeAtta
 	return nil, nil
 }
 
+func (ma *mockAdapter) SetVolumeMetadata(volumeId string, metadata map[string]string) (map[string]string, error) {
+	ma.MethodCall(ma, "SetVolumeMetadata", volumeId, metadata)
+	if ma.setVolumeMetadata != nil {
+		return ma.setVolumeMetadata(volumeId, metadata)
+	}
+	return nil, nil
+}
+
 type testEndpointResolver struct {
 	authenticated   bool
 	regionEndpoints map[string]identity.ServiceURLs
