@@ -7,21 +7,26 @@ package openstack
 
 import (
 	"fmt"
-	"log"
 	"net/url"
 	"strings"
 	"sync"
 	"time"
 
 	"github.com/juju/errors"
+	"github.com/juju/jsonschema"
 	"github.com/juju/loggo"
+	"github.com/juju/retry"
 	"github.com/juju/utils"
+	"github.com/juju/utils/clock"
 	"github.com/juju/version"
-	"gopkg.in/goose.v1/cinder"
-	"gopkg.in/goose.v1/client"
-	gooseerrors "gopkg.in/goose.v1/errors"
-	"gopkg.in/goose.v1/identity"
-	"gopkg.in/goose.v1/nova"
+	"gopkg.in/goose.v2/cinder"
+	"gopkg.in/goose.v2/client"
+	gooseerrors "gopkg.in/goose.v2/errors"
+	"gopkg.in/goose.v2/identity"
+	gooselogging "gopkg.in/goose.v2/logging"
+	"gopkg.in/goose.v2/neutron"
+	"gopkg.in/goose.v2/nova"
+	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/cloud"
 	"github.com/juju/juju/cloudconfig/instancecfg"
@@ -38,7 +43,6 @@ import (
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/status"
 	"github.com/juju/juju/tools"
-	"github.com/juju/utils/clock"
 )
 
 var logger = loggo.GetLogger("juju.provider.openstack")
@@ -47,6 +51,15 @@ type EnvironProvider struct {
 	environs.ProviderCredentials
 	Configurator      ProviderConfigurator
 	FirewallerFactory FirewallerFactory
+	FlavorFilter      FlavorFilter
+
+	// NetworkingDecorator, if non-nil, will be used to
+	// decorate the default networking implementation.
+	// This can be used to override behaviour.
+	NetworkingDecorator NetworkingDecorator
+
+	// ClientFromEndpoint returns an Openstack client for the given endpoint.
+	ClientFromEndpoint func(endpoint string) client.AuthenticatingClient
 }
 
 var (
@@ -55,9 +68,59 @@ var (
 )
 
 var providerInstance *EnvironProvider = &EnvironProvider{
-	OpenstackCredentials{},
-	&defaultConfigurator{},
-	&firewallerFactory{},
+	ProviderCredentials: OpenstackCredentials{},
+	Configurator:        &defaultConfigurator{},
+	FirewallerFactory:   &firewallerFactory{},
+	FlavorFilter:        FlavorFilterFunc(AcceptAllFlavors),
+	NetworkingDecorator: nil,
+	ClientFromEndpoint:  newGooseClient,
+}
+
+var cloudSchema = &jsonschema.Schema{
+	Type:     []jsonschema.Type{jsonschema.ObjectType},
+	Required: []string{cloud.EndpointKey, cloud.AuthTypesKey, cloud.RegionsKey},
+	Order:    []string{cloud.EndpointKey, cloud.AuthTypesKey, cloud.RegionsKey},
+	Properties: map[string]*jsonschema.Schema{
+		cloud.EndpointKey: {
+			Singular: "the API endpoint url for the cloud",
+			Type:     []jsonschema.Type{jsonschema.StringType},
+			Format:   jsonschema.FormatURI,
+		},
+		cloud.AuthTypesKey: {
+			Singular:    "auth type",
+			Plural:      "auth types",
+			Type:        []jsonschema.Type{jsonschema.ArrayType},
+			UniqueItems: jsonschema.Bool(true),
+			Items: &jsonschema.ItemSpec{
+				Schemas: []*jsonschema.Schema{{
+					Type: []jsonschema.Type{jsonschema.StringType},
+					Enum: []interface{}{
+						string(cloud.AccessKeyAuthType),
+						string(cloud.UserPassAuthType),
+					},
+				}},
+			},
+		},
+		cloud.RegionsKey: {
+			Type:     []jsonschema.Type{jsonschema.ObjectType},
+			Singular: "region",
+			Plural:   "regions",
+			AdditionalProperties: &jsonschema.Schema{
+				Type:          []jsonschema.Type{jsonschema.ObjectType},
+				Required:      []string{cloud.EndpointKey},
+				MaxProperties: jsonschema.Int(1),
+				Properties: map[string]*jsonschema.Schema{
+					cloud.EndpointKey: &jsonschema.Schema{
+						Singular:      "the API endpoint url for the region",
+						Type:          []jsonschema.Type{jsonschema.StringType},
+						Format:        jsonschema.FormatURI,
+						Default:       "",
+						PromptDefault: "use cloud api url",
+					},
+				},
+			},
+		},
+	},
 }
 
 var makeServiceURL = client.AuthenticatingClient.MakeServiceURL
@@ -72,6 +135,11 @@ var shortAttempt = utils.AttemptStrategy{
 	Delay: 200 * time.Millisecond,
 }
 
+// Version is part of the EnvironProvider interface.
+func (EnvironProvider) Version() int {
+	return 0
+}
+
 func (p EnvironProvider) Open(args environs.OpenParams) (environs.Environ, error) {
 	logger.Infof("opening model %q", args.Config.Name())
 	if err := validateCloudSpec(args.Cloud); err != nil {
@@ -84,17 +152,40 @@ func (p EnvironProvider) Open(args environs.OpenParams) (environs.Environ, error
 	}
 
 	e := &Environ{
-		name:      args.Config.Name(),
-		uuid:      uuid,
-		cloud:     args.Cloud,
-		namespace: namespace,
-		clock:     clock.WallClock,
+		name:         args.Config.Name(),
+		uuid:         uuid,
+		cloud:        args.Cloud,
+		namespace:    namespace,
+		clock:        clock.WallClock,
+		configurator: p.Configurator,
+		flavorFilter: p.FlavorFilter,
 	}
 	e.firewaller = p.FirewallerFactory.GetFirewaller(e)
-	e.configurator = p.Configurator
+
+	var networking Networking = &switchingNetworking{env: e}
+	if p.NetworkingDecorator != nil {
+		var err error
+		networking, err = p.NetworkingDecorator.DecorateNetworking(networking)
+		if err != nil {
+			return nil, errors.Trace(err)
+		}
+	}
+	e.networking = networking
+
 	if err := e.SetConfig(args.Config); err != nil {
 		return nil, err
 	}
+
+	e.ecfgMutex.Lock()
+	defer e.ecfgMutex.Unlock()
+	client, err := authClient(e.cloud, e.ecfgUnlocked)
+	if err != nil {
+		return nil, errors.Annotate(err, "cannot set config")
+	}
+	e.clientUnlocked = client
+	e.novaUnlocked = nova.New(e.clientUnlocked)
+	e.neutronUnlocked = neutron.New(e.clientUnlocked)
+
 	return e, nil
 }
 
@@ -115,6 +206,25 @@ func (EnvironProvider) DetectRegions() ([]cloud.Region, error) {
 	}}, nil
 }
 
+// CloudSchema returns the schema for adding new clouds of this type.
+func (p EnvironProvider) CloudSchema() *jsonschema.Schema {
+	return cloudSchema
+}
+
+// Ping tests the connection to the cloud, to verify the endpoint is valid.
+func (p EnvironProvider) Ping(endpoint string) error {
+	c := p.ClientFromEndpoint(endpoint)
+	if _, err := c.IdentityAuthOptions(); err != nil {
+		return errors.Wrap(err, errors.Errorf("No Openstack server running at %s", endpoint))
+	}
+	return nil
+}
+
+// newGooseClient is the default function in EnvironProvider.ClientFromEndpoint.
+func newGooseClient(endpoint string) client.AuthenticatingClient {
+	return client.NewClient(&identity.Credentials{URL: endpoint}, 0, nil)
+}
+
 // PrepareConfig is specified in the EnvironProvider interface.
 func (p EnvironProvider) PrepareConfig(args environs.PrepareConfigParams) (*config.Config, error) {
 	if err := validateCloudSpec(args.Cloud); err != nil {
@@ -159,11 +269,12 @@ type Environ struct {
 	cloud     environs.CloudSpec
 	namespace instance.Namespace
 
-	ecfgMutex    sync.Mutex
-	ecfgUnlocked *environConfig
-	client       client.AuthenticatingClient
-	novaUnlocked *nova.Client
-	volumeURL    *url.URL
+	ecfgMutex       sync.Mutex
+	ecfgUnlocked    *environConfig
+	clientUnlocked  client.AuthenticatingClient
+	novaUnlocked    *nova.Client
+	neutronUnlocked *neutron.Client
+	volumeURL       *url.URL
 
 	// keystoneImageDataSource caches the result of getKeystoneImageSource.
 	keystoneImageDataSourceMutex sync.Mutex
@@ -176,7 +287,9 @@ type Environ struct {
 	availabilityZonesMutex sync.Mutex
 	availabilityZones      []common.AvailabilityZone
 	firewaller             Firewaller
+	networking             Networking
 	configurator           ProviderConfigurator
+	flavorFilter           FlavorFilter
 
 	// Clock is defined so it can be replaced for testing
 	clock clock.Clock
@@ -196,7 +309,7 @@ type openstackInstance struct {
 	mu           sync.Mutex
 	serverDetail *nova.ServerDetail
 	// floatingIP is non-nil iff use-floating-ip is true.
-	floatingIP *nova.FloatingIP
+	floatingIP *string
 }
 
 func (inst *openstackInstance) String() string {
@@ -296,8 +409,8 @@ func (inst *openstackInstance) Addresses() ([]network.Address, error) {
 		return nil, err
 	}
 	var floatingIP string
-	if inst.floatingIP != nil && inst.floatingIP.IP != "" {
-		floatingIP = inst.floatingIP.IP
+	if inst.floatingIP != nil {
+		floatingIP = *inst.floatingIP
 		logger.Debugf("instance %v has floating IP address: %v", inst.Id(), floatingIP)
 	}
 	return convertNovaAddresses(floatingIP, addresses), nil
@@ -338,16 +451,16 @@ func convertNovaAddresses(publicIP string, addresses map[string][]nova.IPAddress
 	return machineAddresses
 }
 
-func (inst *openstackInstance) OpenPorts(machineId string, ports []network.PortRange) error {
-	return inst.e.firewaller.OpenInstancePorts(inst, machineId, ports)
+func (inst *openstackInstance) OpenPorts(machineId string, rules []network.IngressRule) error {
+	return inst.e.firewaller.OpenInstancePorts(inst, machineId, rules)
 }
 
-func (inst *openstackInstance) ClosePorts(machineId string, ports []network.PortRange) error {
-	return inst.e.firewaller.CloseInstancePorts(inst, machineId, ports)
+func (inst *openstackInstance) ClosePorts(machineId string, rules []network.IngressRule) error {
+	return inst.e.firewaller.CloseInstancePorts(inst, machineId, rules)
 }
 
-func (inst *openstackInstance) Ports(machineId string) ([]network.PortRange, error) {
-	return inst.e.firewaller.InstancePorts(inst, machineId)
+func (inst *openstackInstance) IngressRules(machineId string) ([]network.IngressRule, error) {
+	return inst.e.firewaller.InstanceIngressRules(inst, machineId)
 }
 
 func (e *Environ) ecfg() *environConfig {
@@ -357,6 +470,13 @@ func (e *Environ) ecfg() *environConfig {
 	return ecfg
 }
 
+func (e *Environ) client() client.AuthenticatingClient {
+	e.ecfgMutex.Lock()
+	client := e.clientUnlocked
+	e.ecfgMutex.Unlock()
+	return client
+}
+
 func (e *Environ) nova() *nova.Client {
 	e.ecfgMutex.Lock()
 	nova := e.novaUnlocked
@@ -364,6 +484,13 @@ func (e *Environ) nova() *nova.Client {
 	return nova
 }
 
+func (e *Environ) neutron() *neutron.Client {
+	e.ecfgMutex.Lock()
+	neutron := e.neutronUnlocked
+	e.ecfgMutex.Unlock()
+	return neutron
+}
+
 var unsupportedConstraints = []string{
 	constraints.Tags,
 	constraints.CpuPower,
@@ -496,16 +623,29 @@ func (e *Environ) PrecheckInstance(series string, cons constraints.Value, placem
 // PrepareForBootstrap is part of the Environ interface.
 func (e *Environ) PrepareForBootstrap(ctx environs.BootstrapContext) error {
 	// Verify credentials.
-	if err := authenticateClient(e.client); err != nil {
+	if err := authenticateClient(e.client()); err != nil {
 		return err
 	}
+	if !e.supportsNeutron() {
+		logger.Warningf(`Using deprecated OpenStack APIs.
+
+  Neutron networking is not supported by this OpenStack cloud.
+  Falling back to deprecated Nova networking.
+
+  Support for deprecated Nova networking APIs will be removed
+  in a future Juju release. Please upgrade to OpenStack Icehouse
+  or newer to maintain compatibility.
+
+`,
+		)
+	}
 	return nil
 }
 
 // Create is part of the Environ interface.
 func (e *Environ) Create(environs.CreateParams) error {
 	// Verify credentials.
-	if err := authenticateClient(e.client); err != nil {
+	if err := authenticateClient(e.client()); err != nil {
 		return err
 	}
 	// TODO(axw) 2016-08-04 #1609643
@@ -517,15 +657,17 @@ func (e *Environ) Bootstrap(ctx environs.BootstrapContext, args environs.Bootstr
 	// The client's authentication may have been reset when finding tools if the agent-version
 	// attribute was updated so we need to re-authenticate. This will be a no-op if already authenticated.
 	// An authenticated client is needed for the URL() call below.
-	if err := authenticateClient(e.client); err != nil {
+	if err := authenticateClient(e.client()); err != nil {
 		return nil, err
 	}
 	return common.Bootstrap(ctx, e, args)
 }
 
-// BootstrapMessage is part of the Environ interface.
-func (e *Environ) BootstrapMessage() string {
-	return ""
+func (e *Environ) supportsNeutron() bool {
+	client := e.client()
+	endpointMap := client.EndpointsForRegion(e.cloud.Region)
+	_, ok := endpointMap["network"]
+	return ok
 }
 
 func (e *Environ) ControllerInstances(controllerUUID string) ([]instance.Id, error) {
@@ -567,9 +709,11 @@ func newCredentials(spec environs.CloudSpec) (identity.Credentials, identity.Aut
 		// TODO(axw) we need a way of saying to use legacy auth.
 		cred.User = credAttrs[CredAttrUserName]
 		cred.Secrets = credAttrs[CredAttrPassword]
-		cred.DomainName = credAttrs[CredAttrDomainName]
+		cred.ProjectDomain = credAttrs[CredAttrProjectDomainName]
+		cred.UserDomain = credAttrs[CredAttrUserDomainName]
+		cred.Domain = credAttrs[CredAttrDomainName]
 		authMode = identity.AuthUserPass
-		if cred.DomainName != "" {
+		if cred.Domain != "" || cred.UserDomain != "" || cred.ProjectDomain != "" {
 			authMode = identity.AuthUserPassV3
 		}
 	case cloud.AccessKeyAuthType:
@@ -584,14 +728,15 @@ func determineBestClient(
 	options identity.AuthOptions,
 	client client.AuthenticatingClient,
 	cred identity.Credentials,
-	newClient func(*identity.Credentials, identity.AuthMode, *log.Logger) client.AuthenticatingClient,
+	newClient func(*identity.Credentials, identity.AuthMode, gooselogging.CompatLogger) client.AuthenticatingClient,
+	logger gooselogging.CompatLogger,
 ) client.AuthenticatingClient {
 	for _, option := range options {
 		if option.Mode != identity.AuthUserPassV3 {
 			continue
 		}
 		cred.URL = option.Endpoint
-		v3client := newClient(&cred, identity.AuthUserPassV3, nil)
+		v3client := newClient(&cred, identity.AuthUserPassV3, logger)
 		// V3 being advertised is not necessaritly a guarantee that it will
 		// work.
 		err := v3client.Authenticate()
@@ -603,18 +748,18 @@ func determineBestClient(
 }
 
 func authClient(spec environs.CloudSpec, ecfg *environConfig) (client.AuthenticatingClient, error) {
-
 	identityClientVersion, err := identityClientVersion(spec.Endpoint)
 	if err != nil {
 		return nil, errors.Annotate(err, "cannot create a client")
 	}
 	cred, authMode := newCredentials(spec)
 
+	gooseLogger := gooselogging.LoggoLogger{loggo.GetLogger("goose")}
 	newClient := client.NewClient
 	if ecfg.SSLHostnameVerification() == false {
 		newClient = client.NewNonValidatingClient
 	}
-	client := newClient(&cred, authMode, nil)
+	client := newClient(&cred, authMode, gooseLogger)
 
 	// before returning, lets make sure that we want to have AuthMode
 	// AuthUserPass instead of its V3 counterpart.
@@ -623,20 +768,20 @@ func authClient(spec environs.CloudSpec, ecfg *environConfig) (client.Authentica
 		if err != nil {
 			logger.Errorf("cannot determine available auth versions %v", err)
 		} else {
-			client = determineBestClient(options, client, cred, newClient)
+			client = determineBestClient(
+				options,
+				client,
+				cred,
+				newClient,
+				gooseLogger,
+			)
 		}
 	}
 
-	// By default, the client requires "compute" and
-	// "object-store". Juju only requires "compute".
+	// Juju requires "compute" at a minimum. We'll use "network" if it's
+	// available in preference to the Nova network APIs; and "volume" or
+	// "volume2" for storage if either one is available.
 	client.SetRequiredServiceTypes([]string{"compute"})
-
-	// By default, the client performs API version discovery.
-	// Because we're using some deprecated endpoints, we have
-	// to be careful not to use too new a version. Disable
-	// discovery until we've moved off those deprecated APIs.
-	client.SetVersionDiscoveryEnabled(false)
-
 	return client, nil
 }
 
@@ -671,12 +816,6 @@ func (e *Environ) SetConfig(cfg *config.Config) error {
 	defer e.ecfgMutex.Unlock()
 	e.ecfgUnlocked = ecfg
 
-	client, err := authClient(e.cloud, ecfg)
-	if err != nil {
-		return errors.Annotate(err, "cannot set config")
-	}
-	e.client = client
-	e.novaUnlocked = nova.New(e.client)
 	return nil
 }
 
@@ -742,13 +881,15 @@ func (e *Environ) getKeystoneDataSource(mu *sync.Mutex, datasource *simplestream
 	if *datasource != nil {
 		return *datasource, nil
 	}
-	if !e.client.IsAuthenticated() {
-		if err := authenticateClient(e.client); err != nil {
+
+	client := e.client()
+	if !client.IsAuthenticated() {
+		if err := authenticateClient(client); err != nil {
 			return nil, err
 		}
 	}
 
-	url, err := makeServiceURL(e.client, keystoneName, "", nil)
+	url, err := makeServiceURL(client, keystoneName, "", nil)
 	if err != nil {
 		return nil, errors.NewNotSupported(err, fmt.Sprintf("cannot make service URL: %v", err))
 	}
@@ -760,77 +901,16 @@ func (e *Environ) getKeystoneDataSource(mu *sync.Mutex, datasource *simplestream
 	return *datasource, nil
 }
 
-// resolveNetwork takes either a network id or label and returns a network id
-func (e *Environ) resolveNetwork(networkName string) (string, error) {
-	if utils.IsValidUUIDString(networkName) {
-		// Network id supplied, assume valid as boot will fail if not
-		return networkName, nil
-	}
-	// Network label supplied, resolve to a network id
-	networks, err := e.nova().ListNetworks()
-	if err != nil {
-		return "", err
-	}
-	var networkIds = []string{}
-	for _, network := range networks {
-		if network.Label == networkName {
-			networkIds = append(networkIds, network.Id)
-		}
-	}
-	switch len(networkIds) {
-	case 1:
-		return networkIds[0], nil
-	case 0:
-		return "", errors.Errorf("No networks exist with label %q", networkName)
-	}
-	return "", errors.Errorf("Multiple networks with label %q: %v", networkName, networkIds)
-}
-
-// allocatePublicIP tries to find an available floating IP address, or
-// allocates a new one, returning it, or an error
-func (e *Environ) allocatePublicIP() (*nova.FloatingIP, error) {
-	fips, err := e.nova().ListFloatingIPs()
-	if err != nil {
-		return nil, err
-	}
-	var newfip *nova.FloatingIP
-	for _, fip := range fips {
-		newfip = &fip
-		if fip.InstanceId != nil && *fip.InstanceId != "" {
-			// unavailable, skip
-			newfip = nil
-			continue
-		} else {
-			logger.Debugf("found unassigned public ip: %v", newfip.IP)
-			// unassigned, we can use it
-			return newfip, nil
-		}
-	}
-	if newfip == nil {
-		// allocate a new IP and use it
-		newfip, err = e.nova().AllocateFloatingIP()
-		if err != nil {
-			return nil, err
-		}
-		logger.Debugf("allocated new public IP: %v", newfip.IP)
-	}
-	return newfip, nil
-}
-
 // assignPublicIP tries to assign the given floating IP address to the
 // specified server, or returns an error.
-func (e *Environ) assignPublicIP(fip *nova.FloatingIP, serverId string) (err error) {
-	if fip == nil {
+func (e *Environ) assignPublicIP(fip *string, serverId string) (err error) {
+	if *fip == "" {
 		return errors.Errorf("cannot assign a nil public IP to %q", serverId)
 	}
-	if fip.InstanceId != nil && *fip.InstanceId == serverId {
-		// IP already assigned, nothing to do
-		return nil
-	}
 	// At startup nw_info is not yet cached so this may fail
 	// temporarily while the server is being built
 	for a := common.LongAttempt.Start(); a.Next(); {
-		err = e.nova().AddServerFloatingIP(serverId, fip.IP)
+		err = e.nova().AddServerFloatingIP(serverId, *fip)
 		if err == nil {
 			return nil
 		}
@@ -929,50 +1009,100 @@ func (e *Environ) StartInstance(args environs.StartInstanceParams) (*environs.St
 	}
 	logger.Debugf("openstack user data; %d bytes", len(userData))
 
-	var networks = e.firewaller.InitialNetworks()
+	networks, err := e.networking.DefaultNetworks()
+	if err != nil {
+		return nil, errors.Annotate(err, "getting initial networks")
+	}
 	usingNetwork := e.ecfg().network()
 	if usingNetwork != "" {
-		networkId, err := e.resolveNetwork(usingNetwork)
+		networkId, err := e.networking.ResolveNetwork(usingNetwork, false)
 		if err != nil {
 			return nil, err
 		}
 		logger.Debugf("using network id %q", networkId)
 		networks = append(networks, nova.ServerNetworks{NetworkId: networkId})
 	}
-	withPublicIP := e.ecfg().useFloatingIP()
-	var publicIP *nova.FloatingIP
-	if withPublicIP {
-		logger.Debugf("allocating public IP address for openstack node")
-		if fip, err := e.allocatePublicIP(); err != nil {
-			return nil, errors.Annotate(err, "cannot allocate a public IP as needed")
-		} else {
-			publicIP = fip
-			logger.Infof("allocated public IP %s", publicIP.IP)
+
+	// For BUG 1680787: openstack: add support for neutron networks where port
+	// security is disabled.
+	// If any network specified for instance boot has PortSecurityEnabled equals
+	// false, don't create security groups, instance boot will fail.
+	createSecurityGroups := true
+	if len(networks) > 0 && e.supportsNeutron() {
+		client := e.neutron()
+		for _, n := range networks {
+			net, err := client.GetNetworkV2(n.NetworkId)
+			if err != nil {
+				return nil, err
+			}
+			if net.PortSecurityEnabled != nil &&
+				*net.PortSecurityEnabled == false {
+				createSecurityGroups = *net.PortSecurityEnabled
+				logger.Infof("network %q has port_security_enabled set to false. Not using security groups.", net.Id)
+				break
+			}
 		}
 	}
 
-	var apiPort int
-	if args.InstanceConfig.Controller != nil {
-		apiPort = args.InstanceConfig.Controller.Config.APIPort()
-	} else {
-		// All ports are the same so pick the first.
-		apiPort = args.InstanceConfig.APIInfo.Ports()[0]
-	}
-	var groupNames = make([]nova.SecurityGroupName, 0)
-	groups, err := e.firewaller.SetUpGroups(args.ControllerUUID, args.InstanceConfig.MachineId, apiPort)
-	if err != nil {
-		return nil, errors.Annotate(err, "cannot set up groups")
+	var novaGroupNames = []nova.SecurityGroupName{}
+	if createSecurityGroups {
+		var apiPort int
+		if args.InstanceConfig.Controller != nil {
+			apiPort = args.InstanceConfig.Controller.Config.APIPort()
+		} else {
+			// All ports are the same so pick the first.
+			apiPort = args.InstanceConfig.APIInfo.Ports()[0]
+		}
+		groupNames, err := e.firewaller.SetUpGroups(args.ControllerUUID, args.InstanceConfig.MachineId, apiPort)
+		if err != nil {
+			return nil, errors.Annotate(err, "cannot set up groups")
+		}
+		novaGroupNames = make([]nova.SecurityGroupName, len(groupNames))
+		for i, name := range groupNames {
+			novaGroupNames[i].Name = name
+		}
 	}
 
-	for _, g := range groups {
-		groupNames = append(groupNames, nova.SecurityGroupName{g.Name})
-	}
 	machineName := resourceName(
 		e.namespace,
 		e.name,
 		args.InstanceConfig.MachineId,
 	)
 
+	waitForActiveServerDetails := func(
+		client *nova.Client,
+		id string,
+		timeout time.Duration,
+	) (server *nova.ServerDetail, err error) {
+
+		var errStillBuilding = errors.Errorf("instance %q has status BUILD", id)
+		err = retry.Call(retry.CallArgs{
+			Clock:       e.clock,
+			Delay:       10 * time.Second,
+			MaxDuration: timeout,
+			Func: func() error {
+				server, err = client.GetServer(id)
+				if err != nil {
+					return err
+				}
+				if server.Status == nova.StatusBuild {
+					return errStillBuilding
+				}
+				return nil
+			},
+			NotifyFunc: func(lastError error, attempt int) {
+				args.StatusCallback(status.Provisioning, fmt.Sprintf("%s, wait 10 seconds before retry, attempt %d", lastError, attempt), nil)
+			},
+			IsFatalError: func(err error) bool {
+				return err != errStillBuilding
+			},
+		})
+		if err != nil {
+			return nil, err
+		}
+		return server, nil
+	}
+
 	tryStartNovaInstance := func(
 		attempts utils.AttemptStrategy,
 		client *nova.Client,
@@ -980,7 +1110,26 @@ func (e *Environ) StartInstance(args environs.StartInstanceParams) (*environs.St
 	) (server *nova.Entity, err error) {
 		for a := attempts.Start(); a.Next(); {
 			server, err = client.RunServer(instanceOpts)
-			if err == nil || gooseerrors.IsNotFound(err) == false {
+			if err != nil {
+				break
+			}
+			var serverDetail *nova.ServerDetail
+			serverDetail, err = waitForActiveServerDetails(client, server.Id, 5*time.Minute)
+			if err != nil {
+				server = nil
+				break
+			} else if serverDetail.Status == nova.StatusActive {
+				break
+			} else if serverDetail.Status == nova.StatusError {
+				// Perhaps there is an error case where a retry in the same AZ
+				// is a good idea.
+				logger.Infof("Instance %q in ERROR state with fault %q", server.Id, serverDetail.Fault.Message)
+				logger.Infof("Deleting instance %q in ERROR state", server.Id)
+				if err = e.terminateInstances([]instance.Id{instance.Id(server.Id)}); err != nil {
+					logger.Debugf("Failed to delete instance in ERROR state, %q", err)
+				}
+				server = nil
+				err = errors.New(serverDetail.Fault.Message)
 				break
 			}
 		}
@@ -994,20 +1143,21 @@ func (e *Environ) StartInstance(args environs.StartInstanceParams) (*environs.St
 		availabilityZones []string,
 	) (server *nova.Entity, err error) {
 		for _, zone := range availabilityZones {
+			logger.Infof("trying to build instance in availability zone %q", zone)
 			instanceOpts.AvailabilityZone = zone
 			e.configurator.ModifyRunServerOptions(&instanceOpts)
 			server, err = tryStartNovaInstance(attempts, client, instanceOpts)
+			// 'No valid host available' is typically a resource error,
+			// therefore it a good idea to try another AZ if available.
 			if err == nil || isNoValidHostsError(err) == false {
 				break
 			}
+			logger.Infof("failed to build instance in availability zone %q", zone)
 
-			logger.Infof("no valid hosts available in zone %q, trying another availability zone", zone)
 		}
-
 		if err != nil {
 			err = errors.Annotate(err, "cannot run instance")
 		}
-
 		return server, err
 	}
 
@@ -1016,7 +1166,7 @@ func (e *Environ) StartInstance(args environs.StartInstanceParams) (*environs.St
 		FlavorId:           spec.InstanceType.Id,
 		ImageId:            spec.Image.Id,
 		UserData:           userData,
-		SecurityGroupNames: groupNames,
+		SecurityGroupNames: novaGroupNames,
 		Networks:           networks,
 		Metadata:           args.InstanceConfig.Tags,
 	}
@@ -1037,16 +1187,24 @@ func (e *Environ) StartInstance(args environs.StartInstanceParams) (*environs.St
 		instType:     &spec.InstanceType,
 	}
 	logger.Infof("started instance %q", inst.Id())
+	withPublicIP := e.ecfg().useFloatingIP()
 	if withPublicIP {
+		var publicIP *string
+		logger.Debugf("allocating public IP address for openstack node")
+		if fip, err := e.networking.AllocatePublicIP(inst.Id()); err != nil {
+			return nil, errors.Annotate(err, "cannot allocate a public IP as needed")
+		} else {
+			publicIP = fip
+			logger.Infof("allocated public IP %s", *publicIP)
+		}
 		if err := e.assignPublicIP(publicIP, string(inst.Id())); err != nil {
 			if err := e.terminateInstances([]instance.Id{inst.Id()}); err != nil {
 				// ignore the failure at this stage, just log it
 				logger.Debugf("failed to terminate instance %q: %v", inst.Id(), err)
 			}
-			return nil, errors.Annotatef(err, "cannot assign public address %s to instance %q", publicIP.IP, inst.Id())
+			return nil, errors.Annotatef(err, "cannot assign public address %s to instance %q", publicIP, inst.Id())
 		}
 		inst.floatingIP = publicIP
-		logger.Infof("assigned public IP %s to %q", publicIP.IP, inst.Id())
 	}
 	return &environs.StartInstanceResult{
 		Instance: inst,
@@ -1055,10 +1213,8 @@ func (e *Environ) StartInstance(args environs.StartInstanceParams) (*environs.St
 }
 
 func isNoValidHostsError(err error) bool {
-	if gooseErr, ok := err.(gooseerrors.Error); ok {
-		if cause := gooseErr.Cause(); cause != nil {
-			return strings.Contains(cause.Error(), "No valid host was found")
-		}
+	if cause := errors.Cause(err); cause != nil {
+		return strings.Contains(cause.Error(), "No valid host was found")
 	}
 	return false
 }
@@ -1077,7 +1233,7 @@ func (e *Environ) StopInstances(ids ...instance.Id) error {
 		return err
 	}
 	if securityGroupNames != nil {
-		return e.deleteSecurityGroups(securityGroupNames)
+		return e.firewaller.DeleteGroups(securityGroupNames...)
 	}
 	return nil
 }
@@ -1131,16 +1287,21 @@ func (e *Environ) listServers(ids []instance.Id) ([]nova.ServerDetail, error) {
 // updateFloatingIPAddresses updates the instances with any floating IP address
 // that have been assigned to those instances.
 func (e *Environ) updateFloatingIPAddresses(instances map[string]instance.Instance) error {
-	fips, err := e.nova().ListFloatingIPs()
+	servers, err := e.nova().ListServersDetail(jujuMachineFilter())
 	if err != nil {
 		return err
 	}
-	for _, fip := range fips {
-		if fip.InstanceId != nil && *fip.InstanceId != "" {
-			instId := *fip.InstanceId
-			if inst, ok := instances[instId]; ok {
-				instFip := fip
-				inst.(*openstackInstance).floatingIP = &instFip
+	for _, server := range servers {
+		// server.Addresses is a map with entries containing []nova.IPAddress
+		for _, net := range server.Addresses {
+			for _, addr := range net {
+				if addr.Type == "floating" {
+					instId := server.Id
+					if inst, ok := instances[instId]; ok {
+						instFip := &addr.Address
+						inst.(*openstackInstance).floatingIP = instFip
+					}
+				}
 			}
 		}
 	}
@@ -1202,6 +1363,69 @@ func (e *Environ) Instances(ids []instance.Id) ([]instance.Instance, error) {
 	return insts, err
 }
 
+// AdoptResources is part of the Environ interface.
+func (e *Environ) AdoptResources(controllerUUID string, fromVersion version.Number) error {
+	var failed []string
+	controllerTag := map[string]string{tags.JujuController: controllerUUID}
+
+	instances, err := e.AllInstances()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	for _, instance := range instances {
+		err := e.TagInstance(instance.Id(), controllerTag)
+		if err != nil {
+			logger.Errorf("error updating controller tag for instance %s: %v", instance.Id(), err)
+			failed = append(failed, string(instance.Id()))
+		}
+	}
+
+	failedVolumes, err := e.adoptVolumes(controllerTag)
+	if err != nil {
+		return errors.Trace(err)
+	}
+	failed = append(failed, failedVolumes...)
+
+	err = e.firewaller.UpdateGroupController(controllerUUID)
+	if err != nil {
+		return errors.Trace(err)
+	}
+	if len(failed) != 0 {
+		return errors.Errorf("error updating controller tag for some resources: %v", failed)
+	}
+	return nil
+}
+
+func (e *Environ) adoptVolumes(controllerTag map[string]string) ([]string, error) {
+	cinder, err := e.cinderProvider()
+	if errors.IsNotSupported(err) {
+		logger.Debugf("volumes not supported: not transferring ownership for volumes")
+		return nil, nil
+	}
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	// TODO(axw): fix the storage API.
+	volumeSource, err := cinder.VolumeSource(nil)
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	volumeIds, err := volumeSource.ListVolumes()
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+
+	var failed []string
+	for _, volumeId := range volumeIds {
+		_, err := cinder.storageAdapter.SetVolumeMetadata(volumeId, controllerTag)
+		if err != nil {
+			logger.Errorf("error updating controller tag for volume %s: %v", volumeId, err)
+			failed = append(failed, volumeId)
+		}
+	}
+	return failed, nil
+}
+
 // AllInstances returns all instances in this environment.
 func (e *Environ) AllInstances() ([]instance.Instance, error) {
 	tagFilter := tagValue{tags.JujuModel, e.ecfg().UUID()}
@@ -1338,57 +1562,45 @@ func jujuMachineFilter() *nova.Filter {
 	return filter
 }
 
-// portsToRuleInfo maps port ranges to nova rules
-func portsToRuleInfo(groupId string, ports []network.PortRange) []nova.RuleInfo {
-	rules := make([]nova.RuleInfo, len(ports))
-	for i, portRange := range ports {
-		rules[i] = nova.RuleInfo{
+// rulesToRuleInfo maps ingress rules to nova rules
+func rulesToRuleInfo(groupId string, rules []network.IngressRule) []neutron.RuleInfoV2 {
+	var result []neutron.RuleInfoV2
+	for _, r := range rules {
+		ruleInfo := neutron.RuleInfoV2{
+			Direction:     "ingress",
 			ParentGroupId: groupId,
-			FromPort:      portRange.FromPort,
-			ToPort:        portRange.ToPort,
-			IPProtocol:    portRange.Protocol,
-			Cidr:          "0.0.0.0/0",
+			PortRangeMin:  r.FromPort,
+			PortRangeMax:  r.ToPort,
+			IPProtocol:    r.Protocol,
+		}
+		sourceCIDRs := r.SourceCIDRs
+		if len(sourceCIDRs) == 0 {
+			sourceCIDRs = []string{"0.0.0.0/0"}
+		}
+		for _, sr := range sourceCIDRs {
+			ruleInfo.RemoteIPPrefix = sr
+			result = append(result, ruleInfo)
 		}
 	}
-	return rules
+	return result
 }
 
-func (e *Environ) OpenPorts(ports []network.PortRange) error {
-	return e.firewaller.OpenPorts(ports)
+func (e *Environ) OpenPorts(rules []network.IngressRule) error {
+	return e.firewaller.OpenPorts(rules)
 }
 
-func (e *Environ) ClosePorts(ports []network.PortRange) error {
-	return e.firewaller.ClosePorts(ports)
+func (e *Environ) ClosePorts(rules []network.IngressRule) error {
+	return e.firewaller.ClosePorts(rules)
 }
 
-func (e *Environ) Ports() ([]network.PortRange, error) {
-	return e.firewaller.Ports()
+func (e *Environ) IngressRules() ([]network.IngressRule, error) {
+	return e.firewaller.IngressRules()
 }
 
 func (e *Environ) Provider() environs.EnvironProvider {
 	return providerInstance
 }
 
-// deleteSecurityGroups deletes the given security groups. If a security
-// group is also used by another environment (see bug #1300755), an attempt
-// to delete this group fails. A warning is logged in this case.
-func (e *Environ) deleteSecurityGroups(securityGroupNames []string) error {
-	novaclient := e.nova()
-	allSecurityGroups, err := novaclient.ListSecurityGroups()
-	if err != nil {
-		return err
-	}
-	for _, securityGroup := range allSecurityGroups {
-		for _, name := range securityGroupNames {
-			if securityGroup.Name == name {
-				deleteSecurityGroup(novaclient, name, securityGroup.Id, e.clock)
-				break
-			}
-		}
-	}
-	return nil
-}
-
 func (e *Environ) terminateInstances(ids []instance.Id) error {
 	if len(ids) == 0 {
 		return nil
@@ -1474,3 +1686,58 @@ func validateAuthURL(authURL string) error {
 	}
 	return nil
 }
+
+// Subnets is specified on environs.Networking.
+func (e *Environ) Subnets(instId instance.Id, subnetIds []network.Id) ([]network.SubnetInfo, error) {
+	return e.networking.Subnets(instId, subnetIds)
+}
+
+// NetworkInterfaces is specified on environs.Networking.
+func (e *Environ) NetworkInterfaces(instId instance.Id) ([]network.InterfaceInfo, error) {
+	return e.networking.NetworkInterfaces(instId)
+}
+
+// SupportsSpaces is specified on environs.Networking.
+func (e *Environ) SupportsSpaces() (bool, error) {
+	return false, nil
+}
+
+// SupportsSpaceDiscovery is specified on environs.Networking.
+func (e *Environ) SupportsSpaceDiscovery() (bool, error) {
+	return false, nil
+}
+
+// Spaces is specified on environs.Networking.
+func (e *Environ) Spaces() ([]network.SpaceInfo, error) {
+	return nil, errors.NotSupportedf("spaces")
+}
+
+// SupportsContainerAddresses is specified on environs.Networking.
+func (e *Environ) SupportsContainerAddresses() (bool, error) {
+	return false, errors.NotSupportedf("container address")
+}
+
+// AllocateContainerAddresses is specified on environs.Networking.
+func (e *Environ) AllocateContainerAddresses(hostInstanceID instance.Id, containerTag names.MachineTag, preparedInfo []network.InterfaceInfo) ([]network.InterfaceInfo, error) {
+	return nil, errors.NotSupportedf("allocate container address")
+}
+
+// ReleaseContainerAddresses is specified on environs.Networking.
+func (e *Environ) ReleaseContainerAddresses(interfaces []network.ProviderInterfaceInfo) error {
+	return errors.NotSupportedf("release container address")
+}
+
+// ProviderSpaceInfo is specified on environs.NetworkingEnviron.
+func (*Environ) ProviderSpaceInfo(space *network.SpaceInfo) (*environs.ProviderSpaceInfo, error) {
+	return nil, errors.NotSupportedf("provider space info")
+}
+
+// AreSpacesRoutable is specified on environs.NetworkingEnviron.
+func (*Environ) AreSpacesRoutable(space1, space2 *environs.ProviderSpaceInfo) (bool, error) {
+	return false, nil
+}
+
+// SSHAddresses is specified on environs.SSHAddresses.
+func (*Environ) SSHAddresses(addresses []network.Address) ([]network.Address, error) {
+	return addresses, nil
+}
