@@ -56,6 +56,11 @@ func allCollections() collectionSchema {
 			global:         true,
 			rawAccess:      true,
 			explicitCreate: &mgo.CollectionInfo{},
+			indexes: []mgo.Index{{
+				// The "s" field is used in queries
+				// by mgo/txn.Runner.ResumeAll.
+				Key: []string{"s"},
+			}},
 		},
 		txnLogC: {
 			// This collection is used by mgo/txn to record the set of documents
@@ -241,8 +246,12 @@ func allCollections() collectionSchema {
 		// -----
 
 		// These collections hold information associated with applications.
-		charmsC:       {},
-		applicationsC: {},
+		charmsC: {},
+		applicationsC: {
+			indexes: []mgo.Index{{
+				Key: []string{"model-uuid", "name"},
+			}},
+		},
 		unitsC: {
 			indexes: []mgo.Index{{
 				Key: []string{"model-uuid", "application"},
@@ -269,16 +278,24 @@ func allCollections() collectionSchema {
 				Key: []string{"model-uuid", "endpoints.applicationname"},
 			}},
 		},
-		relationScopesC: {},
+		relationScopesC: {
+			indexes: []mgo.Index{{
+				Key: []string{"model-uuid", "key", "departing"},
+			}},
+		},
 
 		// -----
 
 		// These collections hold information associated with machines.
 		containerRefsC: {},
 		instanceDataC:  {},
-		machinesC:      {},
-		rebootC:        {},
-		sshHostKeysC:   {},
+		machinesC: {
+			indexes: []mgo.Index{{
+				Key: []string{"model-uuid", "machineid"},
+			}},
+		},
+		rebootC:      {},
+		sshHostKeysC: {},
 
 		// This collection contains information from removed machines
 		// that needs to be cleaned up in the provider.
@@ -295,6 +312,8 @@ func allCollections() collectionSchema {
 		filesystemsC: {
 			indexes: []mgo.Index{{
 				Key: []string{"model-uuid", "storageid"},
+			}, {
+				Key: []string{"model-uuid", "machineid"},
 			}},
 		},
 		filesystemAttachmentsC: {},
@@ -313,6 +332,8 @@ func allCollections() collectionSchema {
 		volumesC: {
 			indexes: []mgo.Index{{
 				Key: []string{"model-uuid", "storageid"},
+			}, {
+				Key: []string{"model-uuid", "machineid"},
 			}},
 		},
 		volumeAttachmentsC: {},
@@ -324,9 +345,13 @@ func allCollections() collectionSchema {
 		subnetsC:              {},
 		linkLayerDevicesC:     {},
 		linkLayerDevicesRefsC: {},
-		ipAddressesC:          {},
-		endpointBindingsC:     {},
-		openedPortsC:          {},
+		ipAddressesC: {
+			indexes: []mgo.Index{{
+				Key: []string{"model-uuid", "machine-id", "device-name"},
+			}},
+		},
+		endpointBindingsC: {},
+		openedPortsC:      {},
 
 		// -----
 
@@ -373,11 +398,21 @@ func allCollections() collectionSchema {
 
 		constraintsC:        {},
 		storageConstraintsC: {},
-		statusesC:           {},
+		statusesC: {
+			indexes: []mgo.Index{{
+				Key: []string{"model-uuid", "_id"},
+			}},
+		},
 		statusesHistoryC: {
 			rawAccess: true,
 			indexes: []mgo.Index{{
 				Key: []string{"model-uuid", "globalkey", "updated"},
+			}, {
+				// used for migration and model-specific pruning
+				Key: []string{"model-uuid", "-updated", "-_id"},
+			}, {
+				// used for global pruning (after size check)
+				Key: []string{"-updated"},
 			}},
 		},
 
@@ -403,16 +438,18 @@ func allCollections() collectionSchema {
 			applicationOffersC: {
 				indexes: []mgo.Index{{Key: []string{"model-uuid", "url"}}},
 			},
-			// This collection holds information about applications that have been
-			// offered (exported) for use in other models managed by the same
-			// host controller.
-			localApplicationDirectoryC: {
-				global: true,
+			remoteApplicationsC: {},
+			// remoteEntitiesC holds information about entities involved in
+			// cross-model relations.
+			remoteEntitiesC: {
 				indexes: []mgo.Index{{
-					Key: []string{"url"},
+					Key: []string{"model-uuid", "source-model-uuid", "token"},
+				}, {
+					Key: []string{"model-uuid", "source-model-uuid"},
 				}},
 			},
-			remoteApplicationsC: {},
+			// tokensC holds unique tokens for the model.
+			tokensC: {},
 		} {
 			result[name] = details
 		}
@@ -502,7 +539,8 @@ const (
 	// "resources" (see resource/persistence/mongo.go)
 
 	// Cross model relations
-	localApplicationDirectoryC = "localapplicationdirectory"
-	applicationOffersC         = "applicationOffers"
-	remoteApplicationsC        = "remoteApplications"
+	applicationOffersC  = "applicationOffers"
+	remoteApplicationsC = "remoteApplications"
+	remoteEntitiesC     = "remoteEntities"
+	tokensC             = "tokens"
 )
