@@ -11,12 +11,13 @@ import (
 	"github.com/juju/utils/featureflag"
 	"gopkg.in/juju/names.v2"
 	"gopkg.in/mgo.v2"
+	"gopkg.in/mgo.v2/bson"
 
+	"github.com/juju/juju/environs/config"
 	"github.com/juju/juju/feature"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/state/multiwatcher"
 	"github.com/juju/juju/state/watcher"
-	"github.com/juju/juju/state/workers"
 	"github.com/juju/juju/status"
 )
 
@@ -24,7 +25,7 @@ import (
 // a single model from the State.
 type allWatcherStateBacking struct {
 	st               *State
-	watcher          workers.TxnLogWatcher
+	watcher          *watcher.Watcher
 	collectionByName map[string]allWatcherStateCollection
 }
 
@@ -32,7 +33,7 @@ type allWatcherStateBacking struct {
 // for all models from the State.
 type allModelWatcherStateBacking struct {
 	st               *State
-	watcher          workers.TxnLogWatcher
+	watcher          *watcher.Watcher
 	stPool           *StatePool
 	collectionByName map[string]allWatcherStateCollection
 }
@@ -93,6 +94,8 @@ func makeAllWatcherCollectionInfo(collNames ...string) map[string]allWatcherStat
 			collection.subsidiary = true
 		case remoteApplicationsC:
 			collection.docType = reflect.TypeOf(backingRemoteApplication{})
+		case applicationOffersC:
+			collection.docType = reflect.TypeOf(backingApplicationOffer{})
 		default:
 			panic(errors.Errorf("unknown collection %q", collName))
 		}
@@ -114,14 +117,53 @@ func makeAllWatcherCollectionInfo(collNames ...string) map[string]allWatcherStat
 
 type backingModel modelDoc
 
+func (e *backingModel) isNotFoundAndModelDead(err error) bool {
+	// Return true if the error is not found and the model is dead.
+	// This will be the case if the model has been marked dead, pending cleanup.
+	return errors.IsNotFound(err) && e.Life == Dead
+}
+
 func (e *backingModel) updated(st *State, store *multiwatcherStore, id string) error {
-	store.Update(&multiwatcher.ModelInfo{
+	cfg, err := st.ModelConfig()
+	if e.isNotFoundAndModelDead(err) {
+		// Treat it as if the model is removed.
+		return e.removed(store, e.UUID, e.UUID, st)
+	}
+	if err != nil {
+		return errors.Trace(err)
+	}
+	info := &multiwatcher.ModelInfo{
 		ModelUUID:      e.UUID,
 		Name:           e.Name,
 		Life:           multiwatcher.Life(e.Life.String()),
 		Owner:          e.Owner,
 		ControllerUUID: e.ControllerUUID,
-	})
+		Config:         cfg.AllAttrs(),
+	}
+	c, err := readConstraints(st, modelGlobalKey)
+	// Treat it as if the model is removed.
+	if e.isNotFoundAndModelDead(err) {
+		return e.removed(store, e.UUID, e.UUID, st)
+	}
+	if err != nil {
+		return errors.Trace(err)
+	}
+	info.Constraints = c
+	modelStatus, err := getStatus(st.db(), modelGlobalKey, "model")
+	if e.isNotFoundAndModelDead(err) {
+		// Treat it as if the model is removed.
+		return e.removed(store, e.UUID, e.UUID, st)
+	}
+	if err != nil {
+		return errors.Trace(err)
+	}
+	info.Status = multiwatcher.StatusInfo{
+		Current: modelStatus.Status,
+		Message: modelStatus.Message,
+		Data:    normaliseStatusData(modelStatus.Data),
+		Since:   modelStatus.Since,
+	}
+	store.Update(info)
 	return nil
 }
 
@@ -378,24 +420,24 @@ func (u *backingUnit) mongoId() string {
 
 type backingApplication applicationDoc
 
-func (svc *backingApplication) updated(st *State, store *multiwatcherStore, id string) error {
-	if svc.CharmURL == nil {
+func (app *backingApplication) updated(st *State, store *multiwatcherStore, id string) error {
+	if app.CharmURL == nil {
 		return errors.Errorf("charm url is nil")
 	}
 	info := &multiwatcher.ApplicationInfo{
 		ModelUUID:   st.ModelUUID(),
-		Name:        svc.Name,
-		Exposed:     svc.Exposed,
-		CharmURL:    svc.CharmURL.String(),
-		Life:        multiwatcher.Life(svc.Life.String()),
-		MinUnits:    svc.MinUnits,
-		Subordinate: svc.Subordinate,
+		Name:        app.Name,
+		Exposed:     app.Exposed,
+		CharmURL:    app.CharmURL.String(),
+		Life:        multiwatcher.Life(app.Life.String()),
+		MinUnits:    app.MinUnits,
+		Subordinate: app.Subordinate,
 	}
 	oldInfo := store.Get(info.EntityId())
 	needConfig := false
 	if oldInfo == nil {
-		logger.Debugf("new application %q added to backing state", svc.Name)
-		key := applicationGlobalKey(svc.Name)
+		logger.Debugf("new application %q added to backing state", app.Name)
+		key := applicationGlobalKey(app.Name)
 		// We're adding the entry for the first time,
 		// so fetch the associated child documents.
 		c, err := readConstraints(st, key)
@@ -404,7 +446,7 @@ func (svc *backingApplication) updated(st *State, store *multiwatcherStore, id s
 		}
 		info.Constraints = c
 		needConfig = true
-		applicationStatus, err := getStatus(st, key, "application")
+		applicationStatus, err := getStatus(st.db(), key, "application")
 		if err != nil {
 			return errors.Annotatef(err, "reading application status for key %s", key)
 		}
@@ -443,7 +485,7 @@ func (svc *backingApplication) updated(st *State, store *multiwatcherStore, id s
 		}
 	}
 	if needConfig {
-		doc, err := readSettingsDoc(st, settingsC, applicationSettingsKey(svc.Name, svc.CharmURL))
+		doc, err := readSettingsDoc(st.db(), settingsC, applicationSettingsKey(app.Name, app.CharmURL))
 		if err != nil {
 			return errors.Trace(err)
 		}
@@ -453,7 +495,7 @@ func (svc *backingApplication) updated(st *State, store *multiwatcherStore, id s
 	return nil
 }
 
-func (svc *backingApplication) removed(store *multiwatcherStore, modelUUID, id string, _ *State) error {
+func (app *backingApplication) removed(store *multiwatcherStore, modelUUID, id string, _ *State) error {
 	store.Remove(multiwatcher.EntityId{
 		Kind:      "application",
 		ModelUUID: modelUUID,
@@ -462,28 +504,33 @@ func (svc *backingApplication) removed(store *multiwatcherStore, modelUUID, id s
 	return nil
 }
 
-func (svc *backingApplication) mongoId() string {
-	return svc.DocID
+func (app *backingApplication) mongoId() string {
+	return app.DocID
 }
 
 type backingRemoteApplication remoteApplicationDoc
 
-func (svc *backingRemoteApplication) updated(st *State, store *multiwatcherStore, id string) error {
-	if svc.Name == "" {
+func (app *backingRemoteApplication) updated(st *State, store *multiwatcherStore, id string) error {
+	if app.Name == "" {
 		return errors.Errorf("remote application name is not set")
 	}
+	if app.IsConsumerProxy {
+		// Since this is a consumer proxy, we update the offer
+		// info in this (the offering) model.
+		return app.updateOfferInfo(st, store)
+	}
 	info := &multiwatcher.RemoteApplicationInfo{
 		ModelUUID:      st.ModelUUID(),
-		Name:           svc.Name,
-		ApplicationURL: svc.URL,
-		Life:           multiwatcher.Life(svc.Life.String()),
+		Name:           app.Name,
+		ApplicationURL: app.URL,
+		Life:           multiwatcher.Life(app.Life.String()),
 	}
 	oldInfo := store.Get(info.EntityId())
 	if oldInfo == nil {
-		logger.Debugf("new remote application %q added to backing state", svc.Name)
+		logger.Debugf("new remote application %q added to backing state", app.Name)
 		// Fetch the status.
-		key := remoteApplicationGlobalKey(svc.Name)
-		serviceStatus, err := getStatus(st, key, "remote application")
+		key := remoteApplicationGlobalKey(app.Name)
+		serviceStatus, err := getStatus(st.db(), key, "remote application")
 		if err != nil {
 			return errors.Annotatef(err, "reading remote application status for key %s", key)
 		}
@@ -493,26 +540,104 @@ func (svc *backingRemoteApplication) updated(st *State, store *multiwatcherStore
 			Data:    normaliseStatusData(serviceStatus.Data),
 			Since:   serviceStatus.Since,
 		}
-		logger.Debugf("service status %#v", info.Status)
+		logger.Debugf("remote application status %#v", info.Status)
 	}
 	if store.Get(info.EntityId()) == nil {
-		logger.Debugf("new remote application %q added to backing state", svc.Name)
+		logger.Debugf("new remote application %q added to backing state", app.Name)
 	}
 	store.Update(info)
 	return nil
 }
 
-func (svc *backingRemoteApplication) removed(store *multiwatcherStore, modelUUID, id string, _ *State) error {
+func (app *backingRemoteApplication) updateOfferInfo(st *State, store *multiwatcherStore) error {
+	offerId := multiwatcher.EntityId{
+		Kind:      "applicationOffer",
+		ModelUUID: st.ModelUUID(),
+		Id:        app.OfferName,
+	}
+	// If we have an existing remote application,
+	// adjust any offer info also.
+	info := store.Get(offerId)
+	if info == nil {
+		return nil
+	}
+	offerInfo := info.(*multiwatcher.ApplicationOfferInfo)
+	remoteConnection, err := st.RemoteConnectionStatus(offerInfo.OfferName)
+	if err != nil {
+		return errors.Trace(err)
+	}
+	offerInfo.ConnectedCount = remoteConnection.ConnectionCount()
+	store.Update(offerInfo)
+	return nil
+}
+
+func (app *backingRemoteApplication) removed(store *multiwatcherStore, modelUUID, id string, st *State) (err error) {
+	err = app.updateOfferInfo(st, store)
+	if err != nil {
+		// We log the error but don't prevent the remote app removal.
+		logger.Errorf("updating application offer info: %v", err)
+	}
 	store.Remove(multiwatcher.EntityId{
 		Kind:      "remoteApplication",
 		ModelUUID: modelUUID,
 		Id:        id,
 	})
+	return err
+}
+
+func (app *backingRemoteApplication) mongoId() string {
+	return app.DocID
+}
+
+type backingApplicationOffer applicationOfferDoc
+
+func (offer *backingApplicationOffer) updated(st *State, store *multiwatcherStore, id string) error {
+	info := &multiwatcher.ApplicationOfferInfo{
+		ModelUUID:       st.ModelUUID(),
+		OfferName:       offer.OfferName,
+		ApplicationName: offer.ApplicationName,
+	}
+	err := updateOfferInfo(st, info)
+	if err != nil {
+		return errors.Annotatef(err, "reading application offer details for %s", offer.OfferName)
+	}
+	store.Update(info)
+	return nil
+}
+
+func updateOfferInfo(st *State, offerInfo *multiwatcher.ApplicationOfferInfo) error {
+	offers := NewApplicationOffers(st)
+	offer, err := offers.ApplicationOffer(offerInfo.OfferName)
+	if err != nil {
+		return errors.Trace(err)
+	}
+	localApp, err := st.Application(offer.ApplicationName)
+	if err != nil {
+		return errors.Trace(err)
+	}
+	curl, _ := localApp.CharmURL()
+	offerInfo.ApplicationName = offer.ApplicationName
+	offerInfo.CharmName = curl.Name
+
+	remoteConnection, err := st.RemoteConnectionStatus(offerInfo.OfferName)
+	if err != nil {
+		return errors.Trace(err)
+	}
+	offerInfo.ConnectedCount = remoteConnection.ConnectionCount()
+	return nil
+}
+
+func (offer *backingApplicationOffer) removed(store *multiwatcherStore, modelUUID, id string, _ *State) error {
+	store.Remove(multiwatcher.EntityId{
+		Kind:      "applicationOffer",
+		ModelUUID: modelUUID,
+		Id:        id,
+	})
 	return nil
 }
 
-func (svc *backingRemoteApplication) mongoId() string {
-	return svc.DocID
+func (offer *backingApplicationOffer) mongoId() string {
+	return offer.DocID
 }
 
 type backingAction actionDoc
@@ -662,7 +787,7 @@ func (s *backingStatus) updated(st *State, store *multiwatcherStore, id string)
 		newInfo := *info
 		// Get the unit's current recorded status from state.
 		// It's needed to reset the unit status when a unit comes off error.
-		statusInfo, err := getStatus(st, unitGlobalKey(newInfo.Name), "unit")
+		statusInfo, err := getStatus(st.db(), unitGlobalKey(newInfo.Name), "unit")
 		if err != nil {
 			return err
 		}
@@ -670,6 +795,10 @@ func (s *backingStatus) updated(st *State, store *multiwatcherStore, id string)
 			return err
 		}
 		info0 = &newInfo
+	case *multiwatcher.ModelInfo:
+		newInfo := *info
+		newInfo.Status = s.toStatusInfo()
+		info0 = &newInfo
 	case *multiwatcher.ApplicationInfo:
 		newInfo := *info
 		newInfo.Status = s.toStatusInfo()
@@ -760,12 +889,16 @@ func (c *backingConstraints) updated(st *State, store *multiwatcherStore, id str
 	case *multiwatcher.UnitInfo, *multiwatcher.MachineInfo:
 		// We don't (yet) publish unit or machine constraints.
 		return nil
+	case *multiwatcher.ModelInfo:
+		newInfo := *info
+		newInfo.Constraints = constraintsDoc(*c).value()
+		info0 = &newInfo
 	case *multiwatcher.ApplicationInfo:
 		newInfo := *info
 		newInfo.Constraints = constraintsDoc(*c).value()
 		info0 = &newInfo
 	default:
-		return errors.Errorf("status for unexpected entity with id %q; type %T", id, info)
+		return errors.Errorf("constraints for unexpected entity with id %q; type %T", id, info)
 	}
 	store.Update(info0)
 	return nil
@@ -791,6 +924,16 @@ func (s *backingSettings) updated(st *State, store *multiwatcherStore, id string
 	case nil:
 		// The parent info doesn't exist. Ignore the status until it does.
 		return nil
+	case *multiwatcher.ModelInfo:
+		// We need to construct a model config so that coercion
+		// of raw settings values occurs.
+		cfg, err := config.New(config.NoDefaults, s.Settings)
+		if err != nil {
+			return errors.Trace(err)
+		}
+		newInfo := *info
+		newInfo.Config = cfg.AllAttrs()
+		info0 = &newInfo
 	case *multiwatcher.ApplicationInfo:
 		// If we're seeing settings for the application with a different
 		// charm URL, we ignore them - we will fetch
@@ -985,12 +1128,17 @@ func backingEntityIdForOpenedPortsKey(modelUUID, key string) (multiwatcher.Entit
 // backingEntityIdForGlobalKey returns the entity id for the given global key.
 // It returns false if the key is not recognized.
 func backingEntityIdForGlobalKey(modelUUID, key string) (multiwatcher.EntityId, bool) {
-	if len(key) < 3 || key[1] != '#' {
+	if key == modelGlobalKey {
+		return (&multiwatcher.ModelInfo{
+			ModelUUID: modelUUID,
+		}).EntityId(), true
+	} else if len(key) < 3 || key[1] != '#' {
 		return multiwatcher.EntityId{}, false
 	}
 	id := key[2:]
 	switch key[0] {
 	case 'm':
+		id = strings.TrimSuffix(id, "#instance")
 		return (&multiwatcher.MachineInfo{
 			ModelUUID: modelUUID,
 			Id:        id,
@@ -1037,12 +1185,11 @@ type backingEntityDoc interface {
 	mongoId() string
 }
 
-func newAllWatcherStateBacking(st *State) Backing {
+func newAllWatcherStateBacking(st *State, params WatchParams) Backing {
 	collections := makeAllWatcherCollectionInfo(
 		machinesC,
 		unitsC,
 		applicationsC,
-		remoteApplicationsC,
 		relationsC,
 		annotationsC,
 		statusesC,
@@ -1053,19 +1200,23 @@ func newAllWatcherStateBacking(st *State) Backing {
 		blocksC,
 	)
 	if featureflag.Enabled(feature.CrossModelRelations) {
-		crossModelCollections := makeAllWatcherCollectionInfo(remoteApplicationsC)
+		cmrCollections := []string{remoteApplicationsC}
+		if params.IncludeOffers {
+			cmrCollections = append(cmrCollections, applicationOffersC)
+		}
+		crossModelCollections := makeAllWatcherCollectionInfo(cmrCollections...)
 		for name, w := range crossModelCollections {
 			collections[name] = w
 		}
 	}
 	return &allWatcherStateBacking{
 		st:               st,
-		watcher:          st.workers.TxnLogWatcher(),
+		watcher:          st.workers.txnLogWatcher(),
 		collectionByName: collections,
 	}
 }
 
-func (b *allWatcherStateBacking) filterEnv(docID interface{}) bool {
+func (b *allWatcherStateBacking) filterModel(docID interface{}) bool {
 	_, err := b.st.strictLocalID(docID.(string))
 	return err == nil
 }
@@ -1073,7 +1224,7 @@ func (b *allWatcherStateBacking) filterEnv(docID interface{}) bool {
 // Watch watches all the collections.
 func (b *allWatcherStateBacking) Watch(in chan<- watcher.Change) {
 	for _, c := range b.collectionByName {
-		b.watcher.WatchCollectionWithFilter(c.name, in, b.filterEnv)
+		b.watcher.WatchCollectionWithFilter(c.name, in, b.filterModel)
 	}
 }
 
@@ -1097,7 +1248,7 @@ func (b *allWatcherStateBacking) Changed(all *multiwatcherStore, change watcher.
 	if !ok {
 		return errors.Errorf("unknown collection %q in fetch request", change.C)
 	}
-	col, closer := b.st.getCollection(c.name)
+	col, closer := b.st.db().GetCollection(c.name)
 	defer closer()
 	doc := reflect.New(c.docType).Interface().(backingEntityDoc)
 
@@ -1124,7 +1275,7 @@ func (b *allWatcherStateBacking) Release() error {
 	return nil
 }
 
-func NewAllModelWatcherStateBacking(st *State) Backing {
+func NewAllModelWatcherStateBacking(st *State, pool *StatePool) Backing {
 	collections := makeAllWatcherCollectionInfo(
 		modelsC,
 		machinesC,
@@ -1145,8 +1296,8 @@ func NewAllModelWatcherStateBacking(st *State) Backing {
 	}
 	return &allModelWatcherStateBacking{
 		st:               st,
-		watcher:          st.workers.TxnLogWatcher(),
-		stPool:           NewStatePool(st),
+		watcher:          st.workers.txnLogWatcher(),
+		stPool:           pool,
 		collectionByName: collections,
 	}
 }
@@ -1180,11 +1331,11 @@ func (b *allModelWatcherStateBacking) GetAll(all *multiwatcherStore) error {
 }
 
 func (b *allModelWatcherStateBacking) loadAllWatcherEntitiesForModel(m *Model, all *multiwatcherStore) error {
-	st, err := b.st.ForModel(m.ModelTag())
+	st, releaser, err := b.stPool.Get(m.UUID())
 	if err != nil {
 		return errors.Trace(err)
 	}
-	defer st.Close()
+	defer releaser()
 
 	err = loadAllWatcherEntities(st, b.collectionByName, all)
 	if err != nil {
@@ -1208,10 +1359,10 @@ func (b *allModelWatcherStateBacking) Changed(all *multiwatcherStore, change wat
 
 	doc := reflect.New(c.docType).Interface().(backingEntityDoc)
 
-	st, err := b.getState(change.C, modelUUID)
+	st, releaser, err := b.getState(modelUUID)
 	if err != nil {
-		_, envErr := b.st.GetModel(names.NewModelTag(modelUUID))
-		if errors.IsNotFound(envErr) {
+		_, modelErr := b.st.GetModel(names.NewModelTag(modelUUID))
+		if errors.IsNotFound(modelErr) {
 			// The entity's model is gone so remove the entity
 			// from the store.
 			doc.removed(all, modelUUID, id, nil)
@@ -1219,8 +1370,9 @@ func (b *allModelWatcherStateBacking) Changed(all *multiwatcherStore, change wat
 		}
 		return errors.Trace(err)
 	}
+	defer releaser()
 
-	col, closer := st.getCollection(c.name)
+	col, closer := st.db().GetCollection(c.name)
 	defer closer()
 
 	// TODO - see TODOs in allWatcherStateBacking.Changed()
@@ -1248,22 +1400,18 @@ func (b *allModelWatcherStateBacking) idForChange(change watcher.Change) (string
 	return modelUUID, id, nil
 }
 
-func (b *allModelWatcherStateBacking) getState(collName, modelUUID string) (*State, error) {
-	if collName == modelsC {
-		return b.st, nil
-	}
-
-	st, err := b.stPool.Get(modelUUID)
+func (b *allModelWatcherStateBacking) getState(modelUUID string) (*State, StatePoolReleaser, error) {
+	st, releaser, err := b.stPool.Get(modelUUID)
 	if err != nil {
-		return nil, errors.Trace(err)
+		return nil, nil, errors.Trace(err)
 	}
-	return st, nil
+	return st, releaser, nil
 }
 
 // Release implements the Backing interface.
 func (b *allModelWatcherStateBacking) Release() error {
-	err := b.stPool.Close()
-	return errors.Trace(err)
+	// Nothing to release.
+	return nil
 }
 
 func loadAllWatcherEntities(st *State, collectionByName map[string]allWatcherStateCollection, all *multiwatcherStore) error {
@@ -1279,7 +1427,14 @@ func loadAllWatcherEntities(st *State, collectionByName map[string]allWatcherSta
 		col, closer := db.GetCollection(c.name)
 		defer closer()
 		infoSlicePtr := reflect.New(reflect.SliceOf(c.docType))
-		if err := col.Find(nil).All(infoSlicePtr.Interface()); err != nil {
+
+		// models is a global collection so need to filter on UUID.
+		var filter bson.M
+		if c.name == modelsC {
+			filter = bson.M{"_id": st.ModelUUID()}
+		}
+
+		if err := col.Find(filter).All(infoSlicePtr.Interface()); err != nil {
 			return errors.Errorf("cannot get all %s: %v", c.name, err)
 		}
 		infos := infoSlicePtr.Elem()
