@@ -6,6 +6,7 @@ package state_test
 import (
 	"fmt"
 	"sort"
+	"strings"
 
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
@@ -18,11 +19,13 @@ import (
 	"gopkg.in/mgo.v2/txn"
 
 	"github.com/juju/juju/constraints"
+	"github.com/juju/juju/resource/resourcetesting"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/state/testing"
 	"github.com/juju/juju/status"
 	coretesting "github.com/juju/juju/testing"
 	"github.com/juju/juju/testing/factory"
+	"gopkg.in/juju/names.v2"
 )
 
 type ApplicationSuite struct {
@@ -45,6 +48,18 @@ func (s *ApplicationSuite) SetUpTest(c *gc.C) {
 	s.mysql = s.AddTestingService(c, "mysql", s.charm)
 }
 
+func (s *ApplicationSuite) assertNeedsCleanup(c *gc.C) {
+	dirty, err := s.State.NeedsCleanup()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(dirty, jc.IsTrue)
+}
+
+func (s *ApplicationSuite) assertNoCleanup(c *gc.C) {
+	dirty, err := s.State.NeedsCleanup()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(dirty, jc.IsFalse)
+}
+
 func (s *ApplicationSuite) TestSetCharm(c *gc.C) {
 	ch, force, err := s.mysql.Charm()
 	c.Assert(err, jc.ErrorIsNil)
@@ -185,6 +200,7 @@ func (s *ApplicationSuite) TestSetCharmUpdatesBindings(c *gc.C) {
 		Name:  "yoursql",
 		Charm: oldCharm,
 		EndpointBindings: map[string]string{
+			"":       "db",
 			"server": "db",
 			"client": "client",
 		}})
@@ -198,90 +214,17 @@ func (s *ApplicationSuite) TestSetCharmUpdatesBindings(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(updatedBindings, jc.DeepEquals, map[string]string{
 		// Existing bindings are preserved.
+		"":        "db",
 		"server":  "db",
 		"client":  "client",
-		"cluster": "", // inherited from defaults in AddService.
-		// New endpoints use empty defaults.
-		"foo":  "",
-		"baz":  "",
-		"just": "",
+		"cluster": "db", // inherited from defaults in AddService.
+		// New endpoints use defaults.
+		"foo":  "db",
+		"baz":  "db",
+		"just": "db",
 	})
 }
 
-func (s *ApplicationSuite) TestSetCharmWithWeirdlyNamedEndpoints(c *gc.C) {
-	// This test ensures if special characters appear in endpoint names of the
-	// charm metadata, they are properly escaped before saving to mongo, and
-	// unescaped when read back.
-	_, err := s.State.AddSpace("client", "", nil, true)
-	c.Assert(err, jc.ErrorIsNil)
-	_, err = s.State.AddSpace("db", "", nil, false)
-	c.Assert(err, jc.ErrorIsNil)
-
-	initialBindings := map[string]string{
-		"$pull":     "db",
-		"$set.foo":  "",
-		"cli ent .": "client",
-		".":         "db",
-	}
-	weirdOldCharm := s.AddMetaCharm(c, "mysql", `
-name: mysql
-summary: "Fake MySQL Database engine"
-description: "Complete with nonsense relations"
-provides:
-  $pull: mysql
-  $set.foo: bar
-requires:
-  foo: something
-  "cli ent .": mysql
-peers:
-  ".": bad
-  "$": mysql
-`, 42)
-	weirdNewCharm := s.AddMetaCharm(c, "mysql", `
-name: mysql
-summary: "Fake MySQL Database engine"
-description: "Complete with nonsense relations"
-provides:
-  ser$ver2: mysql
-  $pull: mysql
-  $set.foo: bar
-requires:
-  "cli ent 2": mysql
-peers:
-  "$": mysql
-  ".": bad
-`, 43)
-
-	weirdService := s.AddTestingServiceWithBindings(c, "weird", weirdOldCharm, initialBindings)
-	readBindings, err := weirdService.EndpointBindings()
-	c.Assert(err, jc.ErrorIsNil)
-	expectedBindings := map[string]string{
-		"cli ent .": "client",
-		"foo":       "",
-		"$":         "",
-		".":         "db",
-		"$set.foo":  "",
-		"$pull":     "db",
-	}
-	c.Check(readBindings, jc.DeepEquals, expectedBindings)
-
-	cfg := state.SetCharmConfig{Charm: weirdNewCharm}
-	err = weirdService.SetCharm(cfg)
-	c.Assert(err, jc.ErrorIsNil)
-	readBindings, err = weirdService.EndpointBindings()
-	c.Assert(err, jc.ErrorIsNil)
-
-	expectedBindings = map[string]string{
-		"ser$ver2":  "",
-		"cli ent 2": "",
-		"$":         "",
-		".":         "db",
-		"$set.foo":  "",
-		"$pull":     "db",
-	}
-	c.Check(readBindings, jc.DeepEquals, expectedBindings)
-}
-
 var metaBase = `
 name: mysql
 summary: "Fake MySQL Database engine"
@@ -1227,10 +1170,21 @@ func (s *ApplicationSuite) TestMysqlEndpoints(c *gc.C) {
 			Scope:     charm.ScopeGlobal,
 		},
 	})
+	serverAdminEP, err := s.mysql.Endpoint("server-admin")
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(serverAdminEP, gc.DeepEquals, state.Endpoint{
+		ApplicationName: "mysql",
+		Relation: charm.Relation{
+			Interface: "mysql-root",
+			Name:      "server-admin",
+			Role:      charm.RoleProvider,
+			Scope:     charm.ScopeGlobal,
+		},
+	})
 
 	eps, err := s.mysql.Endpoints()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(eps, gc.DeepEquals, []state.Endpoint{jiEP, serverEP})
+	c.Assert(eps, jc.SameContents, []state.Endpoint{jiEP, serverEP, serverAdminEP})
 }
 
 func (s *ApplicationSuite) TestRiakEndpoints(c *gc.C) {
@@ -1727,22 +1681,16 @@ func (s *ApplicationSuite) TestDestroyQueuesUnitCleanup(c *gc.C) {
 		}
 	}
 
-	// Check state is clean.
-	dirty, err := s.State.NeedsCleanup()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(dirty, jc.IsFalse)
+	s.assertNoCleanup(c)
 
 	// Destroy mysql, and check units are not touched.
-	err = s.mysql.Destroy()
+	err := s.mysql.Destroy()
 	c.Assert(err, jc.ErrorIsNil)
 	for _, unit := range units {
 		assertLife(c, unit, state.Alive)
 	}
 
-	// Check a cleanup doc was added.
-	dirty, err = s.State.NeedsCleanup()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(dirty, jc.IsTrue)
+	s.assertNeedsCleanup(c)
 
 	// Run the cleanup and check the units.
 	err = s.State.Cleanup()
@@ -1756,16 +1704,12 @@ func (s *ApplicationSuite) TestDestroyQueuesUnitCleanup(c *gc.C) {
 	}
 
 	// Check for queued unit cleanups, and run them.
-	dirty, err = s.State.NeedsCleanup()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(dirty, jc.IsTrue)
+	s.assertNeedsCleanup(c)
 	err = s.State.Cleanup()
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Check we're now clean.
-	dirty, err = s.State.NeedsCleanup()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(dirty, jc.IsFalse)
+	s.assertNoCleanup(c)
 }
 
 func (s *ApplicationSuite) TestRemoveServiceMachine(c *gc.C) {
@@ -1785,18 +1729,44 @@ func (s *ApplicationSuite) TestRemoveServiceMachine(c *gc.C) {
 	assertLife(c, machine, state.Dying)
 }
 
-func (s *ApplicationSuite) TestRemoveQueuesLocalCharmCleanup(c *gc.C) {
-	// Check state is clean.
-	dirty, err := s.State.NeedsCleanup()
+func (s *ApplicationSuite) TestApplicationCleanupRemovesStorageConstraints(c *gc.C) {
+	ch := s.AddTestingCharm(c, "storage-block")
+	storage := map[string]state.StorageConstraints{
+		"data": makeStorageCons("loop", 1024, 1),
+	}
+	app := s.AddTestingServiceWithStorage(c, "storage-block", ch, storage)
+	u, err := app.AddUnit()
+	c.Assert(err, jc.ErrorIsNil)
+	err = u.SetCharmURL(ch.URL())
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(dirty, jc.IsFalse)
 
-	err = s.mysql.Destroy()
+	c.Assert(app.Destroy(), gc.IsNil)
+	assertLife(c, app, state.Dying)
+	assertCleanupCount(c, s.State, 2)
+
+	// These next API calls are normally done by the uniter.
+	err = s.State.RemoveStorageAttachment(names.NewStorageTag("data/0"), u.UnitTag())
+	c.Assert(err, jc.ErrorIsNil)
+	err = u.EnsureDead()
+	c.Assert(err, jc.ErrorIsNil)
+	err = u.Remove()
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Ensure storage constraints and settings are now gone.
+	_, err = state.AppStorageConstraints(app)
+	c.Assert(err, jc.Satisfies, errors.IsNotFound)
+	settings := state.GetApplicationSettings(s.State, app)
+	err = settings.Read()
+	c.Assert(err, jc.Satisfies, errors.IsNotFound)
+}
+
+func (s *ApplicationSuite) TestRemoveQueuesLocalCharmCleanup(c *gc.C) {
+	s.assertNoCleanup(c)
+
+	err := s.mysql.Destroy()
 
 	// Check a cleanup doc was added.
-	dirty, err = s.State.NeedsCleanup()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(dirty, jc.IsTrue)
+	s.assertNeedsCleanup(c)
 
 	// Run the cleanup
 	err = s.State.Cleanup()
@@ -1807,9 +1777,56 @@ func (s *ApplicationSuite) TestRemoveQueuesLocalCharmCleanup(c *gc.C) {
 	c.Check(err, jc.Satisfies, errors.IsNotFound)
 
 	// Check we're now clean.
-	dirty, err = s.State.NeedsCleanup()
+	s.assertNoCleanup(c)
+}
+
+func (s *ApplicationSuite) TestDestroyQueuesResourcesCleanup(c *gc.C) {
+	s.assertNoCleanup(c)
+
+	// Add a resource to the application, ensuring it is stored.
+	rSt, err := s.State.Resources()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(dirty, jc.IsFalse)
+	const content = "abc"
+	res := resourcetesting.NewCharmResource(c, "blob", content)
+	outRes, err := rSt.SetResource(s.mysql.Name(), "user", res, strings.NewReader(content))
+	c.Assert(err, jc.ErrorIsNil)
+	storagePath := state.ResourceStoragePath(c, s.State, outRes.ID)
+	c.Assert(state.IsBlobStored(c, s.State, storagePath), jc.IsTrue)
+
+	// Detroy the application.
+	err = s.mysql.Destroy()
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Cleanup should be registered but not yet run.
+	s.assertNeedsCleanup(c)
+	c.Assert(state.IsBlobStored(c, s.State, storagePath), jc.IsTrue)
+
+	// Run the cleanup.
+	err = s.State.Cleanup()
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Check we're now clean.
+	s.assertNoCleanup(c)
+	c.Assert(state.IsBlobStored(c, s.State, storagePath), jc.IsFalse)
+}
+
+func (s *ApplicationSuite) TestDestroyWithPlaceholderResources(c *gc.C) {
+	s.assertNoCleanup(c)
+
+	// Add a placeholder resource to the application.
+	rSt, err := s.State.Resources()
+	c.Assert(err, jc.ErrorIsNil)
+	res := resourcetesting.NewPlaceholderResource(c, "blob", s.mysql.Name())
+	outRes, err := rSt.SetResource(s.mysql.Name(), "user", res.Resource, nil)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(outRes.IsPlaceholder(), jc.IsTrue)
+
+	// Detroy the application.
+	err = s.mysql.Destroy()
+	c.Assert(err, jc.ErrorIsNil)
+
+	// No cleanup required for placeholder resources.
+	state.AssertNoCleanupsWithKind(c, s.State, "resourceBlob")
 }
 
 func (s *ApplicationSuite) TestReadUnitWithChangingState(c *gc.C) {
@@ -2642,3 +2659,50 @@ func (s *ApplicationSuite) TestSetCharmHandlesMissingBindingsAsDefaults(c *gc.C)
 
 	s.assertApplicationRemovedWithItsBindings(c, service)
 }
+
+func (s *ApplicationSuite) setupAppicationWithUnitsForUpgradeCharmScenario(c *gc.C, numOfUnits int) (deployedV int, err error) {
+	originalCharmMeta := mysqlBaseMeta + `
+peers:
+  replication:
+    interface: pgreplication
+`
+	originalCharm := s.AddMetaCharm(c, "mysql", originalCharmMeta, 2)
+	cfg := state.SetCharmConfig{Charm: originalCharm}
+	err = s.mysql.SetCharm(cfg)
+	c.Assert(err, jc.ErrorIsNil)
+	s.assertApplicationRelations(c, s.mysql, "mysql:replication")
+	deployedV = s.mysql.CharmModifiedVersion()
+
+	for i := 0; i < numOfUnits; i++ {
+		_, err = s.mysql.AddUnit()
+		c.Assert(err, jc.ErrorIsNil)
+	}
+
+	// New mysql charm renames peer relation.
+	updatedCharmMeta := mysqlBaseMeta + `
+peers:
+  replication:
+    interface: pgpeer
+`
+	updatedCharm := s.AddMetaCharm(c, "mysql", updatedCharmMeta, 3)
+
+	cfg = state.SetCharmConfig{Charm: updatedCharm}
+	err = s.mysql.SetCharm(cfg)
+	return
+}
+
+func (s *ApplicationSuite) TestRenamePeerRelationOnUpgradeWithOneUnit(c *gc.C) {
+	obtainedV, err := s.setupAppicationWithUnitsForUpgradeCharmScenario(c, 1)
+
+	// ensure upgrade happened
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(s.mysql.CharmModifiedVersion() == obtainedV+1, jc.IsTrue)
+}
+
+func (s *ApplicationSuite) TestRenamePeerRelationOnUpgradeWithMoreThanOneUnit(c *gc.C) {
+	obtainedV, err := s.setupAppicationWithUnitsForUpgradeCharmScenario(c, 2)
+
+	// ensure upgrade did not happen
+	c.Assert(err, gc.ErrorMatches, `*would break relation "mysql:replication"*`)
+	c.Assert(s.mysql.CharmModifiedVersion() == obtainedV, jc.IsTrue)
+}
