@@ -13,6 +13,7 @@ import (
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/instance"
+	"github.com/juju/juju/resource/resourcetesting"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/state/storage"
 	"github.com/juju/juju/testing/factory"
@@ -29,8 +30,8 @@ func (s *CleanupSuite) SetUpTest(c *gc.C) {
 	s.assertDoesNotNeedCleanup(c)
 }
 
-func (s *CleanupSuite) TestCleanupDyingServiceUnits(c *gc.C) {
-	// Create a service with some units.
+func (s *CleanupSuite) TestCleanupDyingApplicationUnits(c *gc.C) {
+	// Create a application with some units.
 	mysql := s.AddTestingService(c, "mysql", s.AddTestingCharm(c, "mysql"))
 	units := make([]*state.Unit, 3)
 	for i := range units {
@@ -41,7 +42,7 @@ func (s *CleanupSuite) TestCleanupDyingServiceUnits(c *gc.C) {
 	preventUnitDestroyRemove(c, units[0])
 	s.assertDoesNotNeedCleanup(c)
 
-	// Destroy the service and check the units are unaffected, but a cleanup
+	// Destroy the application and check the units are unaffected, but a cleanup
 	// has been scheduled.
 	err := mysql.Destroy()
 	c.Assert(err, jc.ErrorIsNil)
@@ -66,8 +67,8 @@ func (s *CleanupSuite) TestCleanupDyingServiceUnits(c *gc.C) {
 	s.assertCleanupCount(c, 1)
 }
 
-func (s *CleanupSuite) TestCleanupDyingServiceCharm(c *gc.C) {
-	// Create a service and  a charm.
+func (s *CleanupSuite) TestCleanupDyingApplicationCharm(c *gc.C) {
+	// Create a application and a charm.
 	ch := s.AddTestingCharm(c, "mysql")
 	mysql := s.AddTestingService(c, "mysql", ch)
 
@@ -77,7 +78,7 @@ func (s *CleanupSuite) TestCleanupDyingServiceCharm(c *gc.C) {
 	err := stor.Put(storagePath, bytes.NewReader([]byte("data")), 4)
 	c.Assert(err, jc.ErrorIsNil)
 
-	// Destroy the service and check that a cleanup has been scheduled.
+	// Destroy the application and check that a cleanup has been scheduled.
 	err = mysql.Destroy()
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertNeedsCleanup(c)
@@ -88,6 +89,55 @@ func (s *CleanupSuite) TestCleanupDyingServiceCharm(c *gc.C) {
 	c.Assert(err, jc.Satisfies, errors.IsNotFound)
 }
 
+func (s *CleanupSuite) TestCleanupRemoteApplication(c *gc.C) {
+	app, err := s.State.AddRemoteApplication(state.AddRemoteApplicationParams{
+		Name:        "remote-app",
+		SourceModel: names.NewModelTag("test"),
+		Token:       "token",
+	})
+	c.Assert(err, jc.ErrorIsNil)
+
+	err = app.Destroy()
+	c.Assert(err, jc.ErrorIsNil)
+	// Removed immediately since there are no relations yet.
+	s.assertDoesNotNeedCleanup(c)
+	_, err = s.State.RemoteApplication("remote-app")
+	c.Assert(err, jc.Satisfies, errors.IsNotFound)
+}
+
+func (s *CleanupSuite) TestCleanupRemoteApplicationWithRelations(c *gc.C) {
+	mysqlEps := []charm.Relation{
+		{
+			Interface: "mysql",
+			Name:      "db",
+			Role:      charm.RoleProvider,
+			Scope:     charm.ScopeGlobal,
+		},
+	}
+	remoteApp, err := s.State.AddRemoteApplication(state.AddRemoteApplicationParams{
+		Name:        "mysql",
+		SourceModel: s.State.ModelTag(),
+		Token:       "t0",
+		Endpoints:   mysqlEps,
+	})
+	c.Assert(err, jc.ErrorIsNil)
+
+	s.AddTestingService(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
+	eps, err := s.State.InferEndpoints("wordpress", "mysql")
+	c.Assert(err, jc.ErrorIsNil)
+	_, err = s.State.AddRelation(eps[0], eps[1])
+	c.Assert(err, jc.ErrorIsNil)
+
+	err = remoteApp.Destroy()
+	c.Assert(err, jc.ErrorIsNil)
+	s.assertNeedsCleanup(c)
+
+	// Run the cleanup, and check that the remote app is removed.
+	s.assertCleanupRuns(c)
+	_, err = s.State.RemoteApplication("mysql")
+	c.Assert(err, jc.Satisfies, errors.IsNotFound)
+}
+
 func (s *CleanupSuite) TestCleanupControllerModels(c *gc.C) {
 	s.assertDoesNotNeedCleanup(c)
 
@@ -101,7 +151,7 @@ func (s *CleanupSuite) TestCleanupControllerModels(c *gc.C) {
 	s.assertDoesNotNeedCleanup(c)
 
 	// Destroy the controller and check the model is unaffected, but a
-	// cleanup for the model and services has been scheduled.
+	// cleanup for the model and applications has been scheduled.
 	controllerEnv, err := s.State.Model()
 	c.Assert(err, jc.ErrorIsNil)
 	err = controllerEnv.DestroyIncludingHosted()
@@ -109,7 +159,7 @@ func (s *CleanupSuite) TestCleanupControllerModels(c *gc.C) {
 
 	// Two cleanups should be scheduled. One to destroy the hosted
 	// models, the other to destroy the controller model's
-	// services.
+	// applications.
 	s.assertCleanupCount(c, 1)
 	err = otherEnv.Refresh()
 	c.Assert(err, jc.ErrorIsNil)
@@ -134,9 +184,11 @@ func (s *CleanupSuite) TestCleanupModelMachines(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Create a relation with a unit in scope and assigned to the hosted machine.
-	pr := NewPeerRelation(c, s.State)
+	pr := newPeerRelation(c, s.State)
 	err = pr.u0.AssignToMachine(machine)
 	c.Assert(err, jc.ErrorIsNil)
+	preventPeerUnitsDestroyRemove(c, pr)
+
 	err = pr.ru0.EnterScope(nil)
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertDoesNotNeedCleanup(c)
@@ -162,10 +214,10 @@ func (s *CleanupSuite) TestCleanupModelMachines(c *gc.C) {
 	assertLife(c, stateMachine, state.Alive)
 }
 
-func (s *CleanupSuite) TestCleanupModelServices(c *gc.C) {
+func (s *CleanupSuite) TestCleanupModelApplications(c *gc.C) {
 	s.assertDoesNotNeedCleanup(c)
 
-	// Create a service with some units.
+	// Create a application with some units.
 	mysql := s.AddTestingService(c, "mysql", s.AddTestingCharm(c, "mysql"))
 	units := make([]*state.Unit, 3)
 	for i := range units {
@@ -175,11 +227,11 @@ func (s *CleanupSuite) TestCleanupModelServices(c *gc.C) {
 	}
 	s.assertDoesNotNeedCleanup(c)
 
-	// Destroy the model and check the service and units are
+	// Destroy the model and check the application and units are
 	// unaffected, but a cleanup for the application has been scheduled.
-	env, err := s.State.Model()
+	model, err := s.State.Model()
 	c.Assert(err, jc.ErrorIsNil)
-	err = env.Destroy()
+	err = model.Destroy()
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertNeedsCleanup(c)
 	s.assertCleanupRuns(c)
@@ -192,7 +244,7 @@ func (s *CleanupSuite) TestCleanupModelServices(c *gc.C) {
 		c.Assert(unit.Life(), gc.Equals, state.Alive)
 	}
 
-	// The first cleanup Destroys the service, which
+	// The first cleanup Destroys the application, which
 	// schedules another cleanup to destroy the units,
 	// then we need another pass for the actions cleanup
 	// which is queued on the next pass
@@ -208,14 +260,15 @@ func (s *CleanupSuite) TestCleanupModelServices(c *gc.C) {
 
 func (s *CleanupSuite) TestCleanupRelationSettings(c *gc.C) {
 	// Create a relation with a unit in scope.
-	pr := NewPeerRelation(c, s.State)
+	pr := newPeerRelation(c, s.State)
+	preventPeerUnitsDestroyRemove(c, pr)
 	rel := pr.ru0.Relation()
 	err := pr.ru0.EnterScope(map[string]interface{}{"some": "settings"})
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertDoesNotNeedCleanup(c)
 
-	// Destroy the service, check the relation's still around.
-	err = pr.svc.Destroy()
+	// Destroy the application, check the relation's still around.
+	err = pr.app.Destroy()
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertCleanupCount(c, 2)
 	err = rel.Refresh()
@@ -254,9 +307,10 @@ func (s *CleanupSuite) TestCleanupForceDestroyedMachineUnit(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Create a relation with a unit in scope and assigned to the machine.
-	pr := NewPeerRelation(c, s.State)
+	pr := newPeerRelation(c, s.State)
 	err = pr.u0.AssignToMachine(machine)
 	c.Assert(err, jc.ErrorIsNil)
+	preventPeerUnitsDestroyRemove(c, pr)
 	err = pr.ru0.EnterScope(nil)
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertDoesNotNeedCleanup(c)
@@ -287,8 +341,8 @@ func (s *CleanupSuite) TestCleanupForceDestroyMachineCleansStorageAttachments(c
 	storage := map[string]state.StorageConstraints{
 		"data": makeStorageCons("loop", 1024, 1),
 	}
-	service := s.AddTestingServiceWithStorage(c, "storage-block", ch, storage)
-	u, err := service.AddUnit()
+	application := s.AddTestingServiceWithStorage(c, "storage-block", ch, storage)
+	u, err := application.AddUnit()
 	c.Assert(err, jc.ErrorIsNil)
 	err = u.AssignToMachine(machine)
 	c.Assert(err, jc.ErrorIsNil)
@@ -333,7 +387,7 @@ func (s *CleanupSuite) TestCleanupForceDestroyedMachineWithContainer(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Create active units (in relation scope, with subordinates).
-	prr := NewProReqRelation(c, &s.ConnSuite, charm.ScopeContainer)
+	prr := newProReqRelation(c, &s.ConnSuite, charm.ScopeContainer)
 	err = prr.pru0.EnterScope(nil)
 	c.Assert(err, jc.ErrorIsNil)
 	err = prr.pru1.EnterScope(nil)
@@ -348,6 +402,7 @@ func (s *CleanupSuite) TestCleanupForceDestroyedMachineWithContainer(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	err = prr.pu1.AssignToMachine(container)
 	c.Assert(err, jc.ErrorIsNil)
+	preventProReqUnitsDestroyRemove(c, prr)
 	s.assertDoesNotNeedCleanup(c)
 
 	// Force removal of the top-level machine.
@@ -385,7 +440,8 @@ func (s *CleanupSuite) TestCleanupForceDestroyedMachineWithContainer(c *gc.C) {
 
 func (s *CleanupSuite) TestCleanupDyingUnit(c *gc.C) {
 	// Create active unit, in a relation.
-	prr := NewProReqRelation(c, &s.ConnSuite, charm.ScopeGlobal)
+	prr := newProReqRelation(c, &s.ConnSuite, charm.ScopeGlobal)
+	preventProReqUnitsDestroyRemove(c, prr)
 	err := prr.pru0.EnterScope(nil)
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -419,7 +475,8 @@ func (s *CleanupSuite) TestCleanupDyingUnit(c *gc.C) {
 
 func (s *CleanupSuite) TestCleanupDyingUnitAlreadyRemoved(c *gc.C) {
 	// Create active unit, in a relation.
-	prr := NewProReqRelation(c, &s.ConnSuite, charm.ScopeGlobal)
+	prr := newProReqRelation(c, &s.ConnSuite, charm.ScopeGlobal)
+	preventProReqUnitsDestroyRemove(c, prr)
 	err := prr.pru0.EnterScope(nil)
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -445,7 +502,7 @@ func (s *CleanupSuite) TestCleanupDyingUnitAlreadyRemoved(c *gc.C) {
 }
 
 func (s *CleanupSuite) TestCleanupActions(c *gc.C) {
-	// Create a service with a unit.
+	// Create a application with a unit.
 	dummy := s.AddTestingService(c, "dummy", s.AddTestingCharm(c, "dummy"))
 	unit, err := dummy.AddUnit()
 	c.Assert(err, jc.ErrorIsNil)
@@ -486,6 +543,34 @@ func (s *CleanupSuite) TestCleanupActions(c *gc.C) {
 	s.assertDoesNotNeedCleanup(c)
 }
 
+func (s *CleanupSuite) TestCleanupWithCompletedActions(c *gc.C) {
+	// Create a application with a unit.
+	dummy := s.AddTestingService(c, "dummy", s.AddTestingCharm(c, "dummy"))
+	unit, err := dummy.AddUnit()
+	c.Assert(err, jc.ErrorIsNil)
+	s.assertDoesNotNeedCleanup(c)
+
+	// Add a completed action to the unit.
+	action, err := unit.AddAction("snapshot", nil)
+	c.Assert(err, jc.ErrorIsNil)
+	action, err = action.Finish(state.ActionResults{
+		Status:  state.ActionCompleted,
+		Message: "done",
+	})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(action.Status(), gc.Equals, state.ActionCompleted)
+
+	// Destroy application and run cleanups.
+	err = dummy.Destroy()
+	c.Assert(err, jc.ErrorIsNil)
+	// First cleanup marks all units of the application as dying.
+	s.assertCleanupRuns(c)
+	// Second cleanup clear pending actions.
+	s.assertCleanupRuns(c)
+	// Check no cleanups.
+	s.assertDoesNotNeedCleanup(c)
+}
+
 func (s *CleanupSuite) TestCleanupStorageAttachments(c *gc.C) {
 	s.assertDoesNotNeedCleanup(c)
 
@@ -493,8 +578,8 @@ func (s *CleanupSuite) TestCleanupStorageAttachments(c *gc.C) {
 	storage := map[string]state.StorageConstraints{
 		"data": makeStorageCons("loop", 1024, 1),
 	}
-	service := s.AddTestingServiceWithStorage(c, "storage-block", ch, storage)
-	u, err := service.AddUnit()
+	application := s.AddTestingServiceWithStorage(c, "storage-block", ch, storage)
+	u, err := application.AddUnit()
 	c.Assert(err, jc.ErrorIsNil)
 
 	// check no cleanups
@@ -524,17 +609,17 @@ func (s *CleanupSuite) TestCleanupStorageAttachments(c *gc.C) {
 func (s *CleanupSuite) TestCleanupStorageInstances(c *gc.C) {
 	ch := s.AddTestingCharm(c, "storage-block")
 	storage := map[string]state.StorageConstraints{
-		"data": makeStorageCons("loop", 1024, 1),
+		"allecto": makeStorageCons("modelscoped-block", 1024, 1),
 	}
-	service := s.AddTestingServiceWithStorage(c, "storage-block", ch, storage)
-	u, err := service.AddUnit()
+	application := s.AddTestingServiceWithStorage(c, "storage-block", ch, storage)
+	u, err := application.AddUnit()
 	c.Assert(err, jc.ErrorIsNil)
 
 	// check no cleanups
 	s.assertDoesNotNeedCleanup(c)
 
 	// this tag matches the storage instance created for the unit above.
-	storageTag := names.NewStorageTag("data/0")
+	storageTag := names.NewStorageTag("allecto/0")
 
 	si, err := s.State.StorageInstance(storageTag)
 	c.Assert(err, jc.ErrorIsNil)
@@ -546,29 +631,27 @@ func (s *CleanupSuite) TestCleanupStorageInstances(c *gc.C) {
 	si, err = s.State.StorageInstance(storageTag)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(si.Life(), gc.Equals, state.Dying)
-	sa, err := s.State.UnitStorageAttachments(u.UnitTag())
+	sa, err := s.State.StorageAttachment(storageTag, u.UnitTag())
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(sa, gc.HasLen, 1)
-	c.Assert(sa[0].Life(), gc.Equals, state.Alive)
+	c.Assert(sa.Life(), gc.Equals, state.Alive)
 	s.assertCleanupRuns(c)
 
 	// After running the cleanup, the attachment should be dying.
-	sa, err = s.State.UnitStorageAttachments(u.UnitTag())
+	sa, err = s.State.StorageAttachment(storageTag, u.UnitTag())
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(sa, gc.HasLen, 1)
-	c.Assert(sa[0].Life(), gc.Equals, state.Dying)
+	c.Assert(sa.Life(), gc.Equals, state.Dying)
 
 	// check no cleanups
 	s.assertDoesNotNeedCleanup(c)
 }
 
 func (s *CleanupSuite) TestCleanupMachineStorage(c *gc.C) {
-	ch := s.AddTestingCharm(c, "storage-filesystem")
+	ch := s.AddTestingCharm(c, "storage-block")
 	storage := map[string]state.StorageConstraints{
-		"data": makeStorageCons("loop", 1024, 1),
+		"data": makeStorageCons("modelscoped", 1024, 1),
 	}
-	service := s.AddTestingServiceWithStorage(c, "storage-filesystem", ch, storage)
-	unit, err := service.AddUnit()
+	application := s.AddTestingServiceWithStorage(c, "storage-block", ch, storage)
+	unit, err := application.AddUnit()
 	c.Assert(err, jc.ErrorIsNil)
 	err = s.State.AssignUnit(unit, state.AssignCleanEmpty)
 	c.Assert(err, jc.ErrorIsNil)
@@ -577,31 +660,26 @@ func (s *CleanupSuite) TestCleanupMachineStorage(c *gc.C) {
 	machine, err := s.State.Machine(machineId)
 	c.Assert(err, jc.ErrorIsNil)
 
-	// Destroy the service, so we can destroy the machine.
+	// Destroy the application, so we can destroy the machine.
 	err = unit.Destroy()
 	s.assertCleanupRuns(c)
-	err = service.Destroy()
+	err = application.Destroy()
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertCleanupRuns(c)
 
 	// check no cleanups
 	s.assertDoesNotNeedCleanup(c)
 
-	// destroy machine and run cleanups; the filesystem attachment
-	// should be marked dying, but the volume attachment should not
-	// since it contains the filesystem.
+	// destroy machine and run cleanups; the volume attachment
+	// should be marked dying.
 	err = machine.Destroy()
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertCleanupRuns(c)
 
-	fas, err := s.State.MachineFilesystemAttachments(machine.MachineTag())
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(fas, gc.HasLen, 1)
-	c.Assert(fas[0].Life(), gc.Equals, state.Dying)
 	vas, err := s.State.MachineVolumeAttachments(machine.MachineTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(vas, gc.HasLen, 1)
-	c.Assert(vas[0].Life(), gc.Equals, state.Alive)
+	c.Assert(vas[0].Life(), gc.Equals, state.Dying)
 
 	// check no cleanups
 	s.assertDoesNotNeedCleanup(c)
@@ -647,6 +725,53 @@ func (s *CleanupSuite) TestCleanupFilesystemAttachments(c *gc.C) {
 	c.Assert(attachment.Life(), gc.Equals, state.Dying)
 }
 
+func (s *CleanupSuite) TestCleanupResourceBlob(c *gc.C) {
+	app := s.AddTestingService(c, "wp", s.AddTestingCharm(c, "wordpress"))
+	data := "ancient-debris"
+	res := resourcetesting.NewResource(c, nil, "mug", "wp", data).Resource
+	resources, err := s.State.Resources()
+	c.Assert(err, jc.ErrorIsNil)
+	_, err = resources.SetResource("wp", res.Username, res.Resource, bytes.NewBufferString(data))
+	c.Assert(err, jc.ErrorIsNil)
+
+	err = app.Destroy()
+	c.Assert(err, jc.ErrorIsNil)
+
+	path := "application-wp/resources/mug"
+	stateStorage := storage.NewStorage(s.State.ModelUUID(), s.State.MongoSession())
+	closer, _, err := stateStorage.Get(path)
+	c.Assert(err, jc.ErrorIsNil)
+	err = closer.Close()
+	c.Assert(err, jc.ErrorIsNil)
+
+	s.assertCleanupRuns(c)
+
+	_, _, err = stateStorage.Get(path)
+	c.Assert(err, jc.Satisfies, errors.IsNotFound)
+}
+
+func (s *CleanupSuite) TestCleanupResourceBlobHandlesMissing(c *gc.C) {
+	app := s.AddTestingService(c, "wp", s.AddTestingCharm(c, "wordpress"))
+	data := "ancient-debris"
+	res := resourcetesting.NewResource(c, nil, "mug", "wp", data).Resource
+	resources, err := s.State.Resources()
+	c.Assert(err, jc.ErrorIsNil)
+	_, err = resources.SetResource("wp", res.Username, res.Resource, bytes.NewBufferString(data))
+	c.Assert(err, jc.ErrorIsNil)
+
+	err = app.Destroy()
+	c.Assert(err, jc.ErrorIsNil)
+
+	path := "application-wp/resources/mug"
+	stateStorage := storage.NewStorage(s.State.ModelUUID(), s.State.MongoSession())
+	err = stateStorage.Remove(path)
+	c.Assert(err, jc.ErrorIsNil)
+
+	s.assertCleanupRuns(c)
+	// Make sure the cleanup completed successfully.
+	s.assertDoesNotNeedCleanup(c)
+}
+
 func (s *CleanupSuite) TestNothingToCleanup(c *gc.C) {
 	s.assertDoesNotNeedCleanup(c)
 	s.assertCleanupRuns(c)
@@ -665,9 +790,7 @@ func (s *CleanupSuite) assertNeedsCleanup(c *gc.C) {
 }
 
 func (s *CleanupSuite) assertDoesNotNeedCleanup(c *gc.C) {
-	actual, err := s.State.NeedsCleanup()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(actual, jc.IsFalse)
+	state.AssertNoCleanups(c, s.State)
 }
 
 // assertCleanupCount is useful because certain cleanups cause other cleanups
