@@ -15,9 +15,11 @@ import (
 	jc "github.com/juju/testing/checkers"
 	jujutxn "github.com/juju/txn"
 	txntesting "github.com/juju/txn/testing"
+	jutils "github.com/juju/utils"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/charm.v6-unstable"
 	"gopkg.in/juju/names.v2"
+	worker "gopkg.in/juju/worker.v1"
 	"gopkg.in/mgo.v2"
 	"gopkg.in/mgo.v2/bson"
 	"gopkg.in/mgo.v2/txn"
@@ -28,10 +30,10 @@ import (
 	"github.com/juju/juju/mongo/utils"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/permission"
+	"github.com/juju/juju/state/storage"
 	"github.com/juju/juju/status"
 	"github.com/juju/juju/testcharms"
 	"github.com/juju/juju/version"
-	"github.com/juju/juju/worker"
 )
 
 const (
@@ -57,7 +59,6 @@ var (
 	AddVolumeOps                         = (*State).addVolumeOps
 	CombineMeterStatus                   = combineMeterStatus
 	ApplicationGlobalKey                 = applicationGlobalKey
-	ReadSettings                         = readSettings
 	ControllerInheritedSettingsGlobalKey = controllerInheritedSettingsGlobalKey
 	ModelGlobalKey                       = modelGlobalKey
 	MergeBindings                        = mergeBindings
@@ -73,19 +74,38 @@ type (
 	BlockDevicesDoc blockDevicesDoc
 )
 
+// EnsureWorkersStarted ensures that all the automatically
+// started state workers are running, so that tests which
+// insert transaction hooks are less likely to have the hooks
+// run by some other worker, and any side effects of starting
+// the workers (for example, creating collections) will have
+// taken effect.
+func EnsureWorkersStarted(st *State) {
+	// Note: we don't start the all-watcher workers, as
+	// they're started on demand anyway.
+	st.workers.txnLogWatcher()
+	st.workers.presenceWatcher()
+	st.workers.leadershipManager()
+	st.workers.singularManager()
+}
+
 func SetTestHooks(c *gc.C, st *State, hooks ...jujutxn.TestHook) txntesting.TransactionChecker {
+	EnsureWorkersStarted(st)
 	return txntesting.SetTestHooks(c, newRunnerForHooks(st), hooks...)
 }
 
 func SetBeforeHooks(c *gc.C, st *State, fs ...func()) txntesting.TransactionChecker {
+	EnsureWorkersStarted(st)
 	return txntesting.SetBeforeHooks(c, newRunnerForHooks(st), fs...)
 }
 
 func SetAfterHooks(c *gc.C, st *State, fs ...func()) txntesting.TransactionChecker {
+	EnsureWorkersStarted(st)
 	return txntesting.SetAfterHooks(c, newRunnerForHooks(st), fs...)
 }
 
 func SetRetryHooks(c *gc.C, st *State, block, check func()) txntesting.TransactionChecker {
+	EnsureWorkersStarted(st)
 	return txntesting.SetRetryHooks(c, newRunnerForHooks(st), block, check)
 }
 
@@ -96,12 +116,12 @@ func newRunnerForHooks(st *State) jujutxn.Runner {
 	return runner
 }
 
-func OfferAtURL(sd crossmodel.ApplicationDirectory, url string) (*applicationOfferDoc, error) {
-	return sd.(*applicationDirectory).offerAtURL(url)
+func OfferForName(sd crossmodel.ApplicationOffers, name string) (*applicationOfferDoc, error) {
+	return sd.(*applicationOffers).offerForName(name)
 }
 
-func MakeApplicationOfferDoc(sd crossmodel.ApplicationDirectory, url string, offer crossmodel.ApplicationOffer) applicationOfferDoc {
-	return sd.(*applicationDirectory).makeApplicationOfferDoc(offer)
+func MakeApplicationOffer(sd crossmodel.ApplicationOffers, offer *applicationOfferDoc) (*crossmodel.ApplicationOffer, error) {
+	return sd.(*applicationOffers).makeApplicationOffer(*offer)
 }
 
 // SetPolicy updates the State's policy field to the
@@ -118,7 +138,7 @@ func (doc *MachineDoc) String() string {
 }
 
 func ServiceSettingsRefCount(st *State, appName string, curl *charm.URL) (int, error) {
-	refcounts, closer := st.getCollection(refcountsC)
+	refcounts, closer := st.db().GetCollection(refcountsC)
 	defer closer()
 
 	key := applicationSettingsKey(appName, curl)
@@ -166,7 +186,7 @@ func AddTestingServiceWithBindings(c *gc.C, st *State, name string, ch *Charm, b
 
 func addTestingService(c *gc.C, st *State, series, name string, ch *Charm, bindings map[string]string, storage map[string]StorageConstraints) *Application {
 	c.Assert(ch, gc.NotNil)
-	service, err := st.AddApplication(AddApplicationArgs{
+	app, err := st.AddApplication(AddApplicationArgs{
 		Name:             name,
 		Series:           series,
 		Charm:            ch,
@@ -174,7 +194,7 @@ func addTestingService(c *gc.C, st *State, series, name string, ch *Charm, bindi
 		Storage:          storage,
 	})
 	c.Assert(err, jc.ErrorIsNil)
-	return service
+	return app
 }
 
 func AddCustomCharm(c *gc.C, st *State, name, filename, content, series string, revision int) *Charm {
@@ -251,7 +271,7 @@ func TxnRevno(st *State, collName string, id interface{}) (int64, error) {
 	var doc struct {
 		TxnRevno int64 `bson:"txn-revno"`
 	}
-	coll, closer := st.getCollection(collName)
+	coll, closer := st.db().GetCollection(collName)
 	defer closer()
 	err := coll.FindId(id).One(&doc)
 	if err != nil {
@@ -263,7 +283,7 @@ func TxnRevno(st *State, collName string, id interface{}) (int64, error) {
 // MinUnitsRevno returns the Revno of the minUnits document
 // associated with the given application name.
 func MinUnitsRevno(st *State, applicationname string) (int, error) {
-	minUnitsCollection, closer := st.getCollection(minUnitsC)
+	minUnitsCollection, closer := st.db().GetCollection(minUnitsC)
 	defer closer()
 	var doc minUnitsDoc
 	if err := minUnitsCollection.FindId(applicationname).One(&doc); err != nil {
@@ -289,7 +309,7 @@ func CheckUserExists(st *State, name string) (bool, error) {
 	return st.checkUserExists(name)
 }
 
-func WatcherMergeIds(st *State, changeset *[]string, updates map[interface{}]bool, idconv func(string) string) error {
+func WatcherMergeIds(st *State, changeset *[]string, updates map[interface{}]bool, idconv func(string) (string, error)) error {
 	return mergeIds(st, changeset, updates, idconv)
 }
 
@@ -306,7 +326,7 @@ func NewActionStatusWatcher(st *State, receivers []ActionReceiver, statuses ...A
 }
 
 func GetAllUpgradeInfos(st *State) ([]*UpgradeInfo, error) {
-	upgradeInfos, closer := st.getCollection(upgradeInfoC)
+	upgradeInfos, closer := st.db().GetCollection(upgradeInfoC)
 	defer closer()
 
 	var out []*UpgradeInfo
@@ -326,6 +346,18 @@ func UserModelNameIndex(username, modelName string) string {
 	return userModelNameIndex(username, modelName)
 }
 
+func (m *Model) UniqueIndexExists() bool {
+	db, dbCloser := m.modelDatabase()
+	defer dbCloser()
+	coll, closer := db.GetCollection(usermodelnameC)
+	defer closer()
+
+	var doc bson.M
+	err := coll.FindId(m.uniqueIndexID()).One(&doc)
+
+	return err == nil
+}
+
 func DocID(st *State, id string) string {
 	return st.docID(id)
 }
@@ -343,7 +375,7 @@ func GetUnitModelUUID(unit *Unit) string {
 }
 
 func GetCollection(st *State, name string) (mongo.Collection, func()) {
-	return st.getCollection(name)
+	return st.db().GetCollection(name)
 }
 
 func GetRawCollection(st *State, name string) (*mgo.Collection, func()) {
@@ -368,13 +400,28 @@ func Sequence(st *State, name string) (int, error) {
 	return st.sequence(name)
 }
 
-func SetModelLifeDead(st *State, modelUUID string) error {
+func SequenceWithMin(st *State, name string, minVal int) (int, error) {
+	return st.sequenceWithMin(name, minVal)
+}
+
+func SequenceEnsure(st *State, name string, nextVal int) error {
+	sequences, closer := st.getRawCollection(sequenceC)
+	defer closer()
+	updater := newDbSeqUpdater(sequences, st.ModelUUID(), name)
+	return updater.ensure(nextVal)
+}
+
+func (m *Model) SetDead() error {
 	ops := []txn.Op{{
 		C:      modelsC,
-		Id:     modelUUID,
+		Id:     m.doc.UUID,
 		Update: bson.D{{"$set", bson.D{{"life", Dead}}}},
+	}, {
+		C:      usermodelnameC,
+		Id:     m.uniqueIndexID(),
+		Remove: true,
 	}}
-	return st.runTransaction(ops)
+	return m.globalState.db().RunTransaction(ops)
 }
 
 func HostedModelCount(c *gc.C, st *State) int {
@@ -433,7 +480,6 @@ func AssertHostPortConversion(c *gc.C, netHostPort network.HostPort) {
 
 // MakeLogDoc creates a database document for a single log message.
 func MakeLogDoc(
-	modelUUID string,
 	entity names.Tag,
 	t time.Time,
 	module string,
@@ -442,15 +488,14 @@ func MakeLogDoc(
 	msg string,
 ) *logDoc {
 	return &logDoc{
-		Id:        bson.NewObjectId(),
-		Time:      t.UnixNano(),
-		ModelUUID: modelUUID,
-		Entity:    entity.String(),
-		Version:   version.Current.String(),
-		Module:    module,
-		Location:  location,
-		Level:     int(level),
-		Message:   msg,
+		Id:       bson.NewObjectId(),
+		Time:     t.UnixNano(),
+		Entity:   entity.String(),
+		Version:  version.Current.String(),
+		Module:   module,
+		Location: location,
+		Level:    int(level),
+		Message:  msg,
 	}
 }
 
@@ -466,28 +511,36 @@ func IsManagerMachineError(err error) bool {
 	return errors.Cause(err) == managerMachineError
 }
 
-var ActionNotificationIdToActionId = actionNotificationIdToActionId
+func MakeActionIdConverter(st *State) func(string) (string, error) {
+	return func(id string) (string, error) {
+		id, err := st.strictLocalID(id)
+		if err != nil {
+			return "", errors.Trace(err)
+		}
+		return actionNotificationIdToActionId(id), err
+	}
+}
 
 func UpdateModelUserLastConnection(st *State, e permission.UserAccess, when time.Time) error {
 	return st.updateLastModelConnection(e.UserTag, when)
 }
 
-func RemoveEndpointBindingsForService(c *gc.C, service *Application) {
-	globalKey := service.globalKey()
+func RemoveEndpointBindingsForService(c *gc.C, app *Application) {
+	globalKey := app.globalKey()
 	removeOp := removeEndpointBindingsOp(globalKey)
 
-	txnError := service.st.runTransaction([]txn.Op{removeOp})
+	txnError := app.st.runTransaction([]txn.Op{removeOp})
 	err := onAbort(txnError, nil) // ignore ErrAborted as it asserts DocExists
 	c.Assert(err, jc.ErrorIsNil)
 }
 
-func RelationCount(service *Application) int {
-	return service.doc.RelationCount
+func RelationCount(app *Application) int {
+	return app.doc.RelationCount
 }
 
-func AssertEndpointBindingsNotFoundForService(c *gc.C, service *Application) {
-	globalKey := service.globalKey()
-	storedBindings, _, err := readEndpointBindings(service.st, globalKey)
+func AssertEndpointBindingsNotFoundForService(c *gc.C, app *Application) {
+	globalKey := app.globalKey()
+	storedBindings, _, err := readEndpointBindings(app.st, globalKey)
 	c.Assert(storedBindings, gc.IsNil)
 	c.Assert(err, gc.ErrorMatches, fmt.Sprintf("endpoint bindings for %q not found", globalKey))
 	c.Assert(err, jc.Satisfies, errors.IsNotFound)
@@ -532,7 +585,7 @@ func PrimeUnitStatusHistory(
 ) {
 	globalKey := unit.globalKey()
 
-	history, closer := unit.st.getCollection(statusesHistoryC)
+	history, closer := unit.st.db().GetCollection(statusesHistoryC)
 	defer closer()
 	historyW := history.Writeable()
 
@@ -562,13 +615,120 @@ func PrimeUnitStatusHistory(
 		StatusData: data,
 		Updated:    clock.Now().UnixNano(),
 	}
-	buildTxn := updateStatusSource(unit.st, globalKey, doc)
+
+	var buildTxn jujutxn.TransactionSource = func(int) ([]txn.Op, error) {
+		return statusSetOps(unit.st.db(), doc, globalKey)
+	}
+
 	err := unit.st.run(buildTxn)
 	c.Assert(err, jc.ErrorIsNil)
 }
 
+// PrimeActions generates actions to be pruned. The method pads each
+// entry with a 1MB string. This should allow us to infer the
+// approximate size of the entry and limit the number of entries that
+// must be generated for size related tests.
+func PrimeActions(c *gc.C, age time.Time, unit *Unit, count int) {
+	actionCollection, closer := unit.st.db().GetCollection(actionsC)
+	defer closer()
+
+	actionCollectionWriter := actionCollection.Writeable()
+
+	const numBytes = 1 * 1000 * 1000
+	var padding [numBytes]byte
+	var actionDocs []interface{}
+	for i := 0; i < count; i++ {
+		id, err := jutils.NewUUID()
+		c.Assert(err, jc.ErrorIsNil)
+		actionDocs = append(actionDocs, actionDoc{
+			DocId:     id.String(),
+			ModelUUID: unit.st.ModelUUID(),
+			Receiver:  unit.Name(),
+			Completed: age,
+			Status:    ActionCompleted,
+			Message:   string(padding[:numBytes]),
+		})
+	}
+
+	err := actionCollectionWriter.Insert(actionDocs...)
+	c.Assert(err, jc.ErrorIsNil)
+}
+
 // GetInternalWorkers returns the internal workers managed by a State
 // to allow inspection in tests.
 func GetInternalWorkers(st *State) worker.Worker {
 	return st.workers
 }
+
+// ResourceStoragePath returns the path used to store resource content
+// in the managed blob store, given the resource ID.
+func ResourceStoragePath(c *gc.C, st *State, id string) string {
+	p := NewResourcePersistence(st.newPersistence())
+	_, storagePath, err := p.GetResource(id)
+	c.Assert(err, jc.ErrorIsNil)
+	return storagePath
+}
+
+// IsBlobStored returns true if a given storage path is in used in the
+// managed blob store.
+func IsBlobStored(c *gc.C, st *State, storagePath string) bool {
+	stor := storage.NewStorage(st.ModelUUID(), st.MongoSession())
+	r, _, err := stor.Get(storagePath)
+	if err != nil {
+		if errors.IsNotFound(err) {
+			return false
+		}
+		c.Fatalf("Get failed: %v", err)
+		return false
+	}
+	r.Close()
+	return true
+}
+
+// AssertNoCleanupsWithKind checks that there are no cleanups
+// of a given kind scheduled.
+func AssertNoCleanupsWithKind(c *gc.C, st *State, kind cleanupKind) {
+	var docs []cleanupDoc
+	cleanups, closer := st.db().GetCollection(cleanupsC)
+	defer closer()
+	err := cleanups.Find(nil).All(&docs)
+	c.Assert(err, jc.ErrorIsNil)
+	for _, doc := range docs {
+		if doc.Kind == kind {
+			c.Fatalf("found cleanup of kind %q", kind)
+		}
+	}
+}
+
+// AssertNoCleanups checks that there are no cleanups scheduled.
+func AssertNoCleanups(c *gc.C, st *State) {
+	var docs []cleanupDoc
+	cleanups, closer := st.db().GetCollection(cleanupsC)
+	defer closer()
+	err := cleanups.Find(nil).All(&docs)
+	c.Assert(err, jc.ErrorIsNil)
+	if len(docs) > 0 {
+		c.Fatalf("unexpected cleanups: %+v", docs)
+	}
+}
+
+// GetApplicationSettings allows access to settings collection for a
+// given application.
+func GetApplicationSettings(st *State, app *Application) *Settings {
+	return newSettings(st.db(), settingsC, app.settingsKey())
+}
+
+// GetControllerSettings allows access to settings collection for
+// the controller.
+func GetControllerSettings(st *State) *Settings {
+	return newSettings(st.db(), controllersC, controllerSettingsGlobalKey)
+}
+
+// NewSLALevel returns a new SLA level.
+func NewSLALevel(level string) (slaLevel, error) {
+	return newSLALevel(level)
+}
+
+func AppStorageConstraints(app *Application) (map[string]StorageConstraints, error) {
+	return readStorageConstraints(app.st, app.storageConstraintsKey())
+}
