@@ -31,7 +31,7 @@ var ErrNoBackingVolume = errors.New("filesystem has no backing volume")
 // entities managed by a filesystem provider.
 type Filesystem interface {
 	GlobalEntity
-	LifeBinder
+	Lifer
 	status.StatusGetter
 	status.StatusSetter
 
@@ -98,19 +98,21 @@ type filesystemAttachment struct {
 
 // filesystemDoc records information about a filesystem in the model.
 type filesystemDoc struct {
-	DocID        string `bson:"_id"`
-	FilesystemId string `bson:"filesystemid"`
-	ModelUUID    string `bson:"model-uuid"`
-	Life         Life   `bson:"life"`
-	StorageId    string `bson:"storageid,omitempty"`
-	VolumeId     string `bson:"volumeid,omitempty"`
-	// TODO(axw) 2015-06-22 #1467379
-	// upgrade step to set "attachmentcount" and "binding"
-	// for 1.24 models.
+	DocID           string            `bson:"_id"`
+	FilesystemId    string            `bson:"filesystemid"`
+	ModelUUID       string            `bson:"model-uuid"`
+	Life            Life              `bson:"life"`
+	StorageId       string            `bson:"storageid,omitempty"`
+	VolumeId        string            `bson:"volumeid,omitempty"`
 	AttachmentCount int               `bson:"attachmentcount"`
-	Binding         string            `bson:"binding,omitempty"`
 	Info            *FilesystemInfo   `bson:"info,omitempty"`
 	Params          *FilesystemParams `bson:"params,omitempty"`
+
+	// MachineId is the ID of the machine that a non-detachable
+	// volume is initially attached to. We use this to identify
+	// the volume as being non-detachable, and to determine
+	// which volumes must be removed along with said machine.
+	MachineId string `bson:"machineid,omitempty"`
 }
 
 // filesystemAttachmentDoc records information about a filesystem attachment.
@@ -131,10 +133,6 @@ type FilesystemParams struct {
 	// that the filesystem is to be assigned to.
 	storage names.StorageTag
 
-	// binding, if non-nil, is the tag of the entity to which
-	// the filesystem's lifecycle will be bound.
-	binding names.Tag
-
 	Pool string `bson:"pool"`
 	Size uint64 `bson:"size"`
 }
@@ -173,21 +171,6 @@ type FilesystemAttachmentParams struct {
 
 // validate validates the contents of the filesystem document.
 func (f *filesystem) validate() error {
-	if f.doc.Binding != "" {
-		tag, err := names.ParseTag(f.doc.Binding)
-		if err != nil {
-			return errors.Annotate(err, "parsing binding")
-		}
-		switch tag.(type) {
-		case names.ModelTag:
-			// TODO(axw) support binding to model
-			return errors.NotSupportedf("binding to model")
-		case names.MachineTag:
-		case names.StorageTag:
-		default:
-			return errors.Errorf("invalid binding: %v", f.doc.Binding)
-		}
-	}
 	return nil
 }
 
@@ -211,31 +194,6 @@ func (f *filesystem) Life() Life {
 	return f.doc.Life
 }
 
-// LifeBinding is required to implement LifeBinder.
-//
-// Below is the set of possible entity types that a volume may be bound
-// to, and a description of the effects of doing so:
-//
-//   Machine:     If the filesystem is bound to a machine, then the
-//                filesystem will be destroyed when it is detached from
-//                the machine. It is not permitted for a filesystem to
-//                be attached to multiple machines while it is bound to
-//                a machine.
-//   Storage:     If the filesystem is bound to a storage instance,
-//                then the filesystem will be destroyed when the
-//                storage insance is removed from state.
-//   Model: If the filesystem is bound to the model, then
-//                the filesystem must be destroyed prior to the
-//                model being destroyed.
-func (f *filesystem) LifeBinding() names.Tag {
-	if f.doc.Binding == "" {
-		return nil
-	}
-	// Tag is validated in filesystem.validate.
-	tag, _ := names.ParseTag(f.doc.Binding)
-	return tag
-}
-
 // Storage is required to implement Filesystem.
 func (f *filesystem) Storage() (names.StorageTag, error) {
 	if f.doc.StorageId == "" {
@@ -348,7 +306,7 @@ func (st *State) VolumeFilesystem(tag names.VolumeTag) (Filesystem, error) {
 }
 
 func (st *State) filesystems(query interface{}) ([]*filesystem, error) {
-	coll, cleanup := st.getCollection(filesystemsC)
+	coll, cleanup := st.db().GetCollection(filesystemsC)
 	defer cleanup()
 
 	var fDocs []filesystemDoc
@@ -368,7 +326,7 @@ func (st *State) filesystems(query interface{}) ([]*filesystem, error) {
 }
 
 func (st *State) filesystem(query bson.D, description string) (*filesystem, error) {
-	coll, cleanup := st.getCollection(filesystemsC)
+	coll, cleanup := st.db().GetCollection(filesystemsC)
 	defer cleanup()
 
 	f := filesystem{st: st}
@@ -387,7 +345,7 @@ func (st *State) filesystem(query bson.D, description string) (*filesystem, erro
 // FilesystemAttachment returns the FilesystemAttachment corresponding to
 // the specified filesystem and machine.
 func (st *State) FilesystemAttachment(machine names.MachineTag, filesystem names.FilesystemTag) (FilesystemAttachment, error) {
-	coll, cleanup := st.getCollection(filesystemAttachmentsC)
+	coll, cleanup := st.db().GetCollection(filesystemAttachmentsC)
 	defer cleanup()
 
 	var att filesystemAttachment
@@ -421,7 +379,7 @@ func (st *State) MachineFilesystemAttachments(machine names.MachineTag) ([]Files
 }
 
 func (st *State) filesystemAttachments(query bson.D) ([]FilesystemAttachment, error) {
-	coll, cleanup := st.getCollection(filesystemAttachmentsC)
+	coll, cleanup := st.db().GetCollection(filesystemAttachmentsC)
 	defer cleanup()
 
 	var docs []filesystemAttachmentDoc
@@ -441,54 +399,108 @@ func (st *State) filesystemAttachments(query bson.D) ([]FilesystemAttachment, er
 // removeMachineFilesystemsOps returns txn.Ops to remove non-persistent filesystems
 // attached to the specified machine. This is used when the given machine is
 // being removed from state.
-func (st *State) removeMachineFilesystemsOps(machine names.MachineTag) ([]txn.Op, error) {
-	attachments, err := st.MachineFilesystemAttachments(machine)
+func (st *State) removeMachineFilesystemsOps(m *Machine) ([]txn.Op, error) {
+	// A machine cannot transition to Dead if it has any detachable storage
+	// attached, so any attachments are for machine-bound storage.
+	//
+	// Even if a filesystem is "non-detachable", there still exist filesystem
+	// attachments, and they may be removed independently of the filesystem.
+	// For example, the user may request that the filesystem be destroyed.
+	// This will cause the filesystem to become Dying, and the attachment
+	// to be Dying, then Dead, and finally removed. Only once the attachment
+	// is removed will the filesystem transition to Dead and then be removed.
+	// Therefore, there may be filesystems that are bound, but not attached,
+	// to the machine.
+	machineFilesystems, err := st.filesystems(bson.D{{"machineid", m.Id()}})
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
-	ops := make([]txn.Op, 0, len(attachments))
-	for _, a := range attachments {
-		filesystemTag := a.Filesystem()
-		// When removing the machine, there should only remain
-		// non-persistent storage. This will be implicitly
-		// removed when the machine is removed, so we do not
-		// use removeFilesystemAttachmentOps or removeFilesystemOps,
-		// which track and update related documents.
+	ops := make([]txn.Op, 0, 2*len(machineFilesystems)+len(m.doc.Filesystems))
+	for _, filesystemId := range m.doc.Filesystems {
 		ops = append(ops, txn.Op{
 			C:      filesystemAttachmentsC,
-			Id:     filesystemAttachmentId(machine.Id(), filesystemTag.Id()),
+			Id:     filesystemAttachmentId(m.Id(), filesystemId),
 			Assert: txn.DocExists,
 			Remove: true,
 		})
-		canRemove, err := isFilesystemInherentlyMachineBound(st, filesystemTag)
-		if err != nil {
-			return nil, errors.Trace(err)
+	}
+	for _, f := range machineFilesystems {
+		filesystemId := f.Tag().Id()
+		if f.doc.StorageId != "" {
+			// The volume is assigned to a storage instance;
+			// make sure we also remove the storage instance.
+			// There should be no storage attachments remaining,
+			// as the units must have been removed before the
+			// machine can be; and the storage attachments must
+			// have been removed before the unit can be.
+			ops = append(ops,
+				txn.Op{
+					C:      storageInstancesC,
+					Id:     f.doc.StorageId,
+					Assert: txn.DocExists,
+					Remove: true,
+				},
+			)
 		}
-		if !canRemove {
-			return nil, errors.Errorf("machine has non-machine bound filesystem %v", filesystemTag.Id())
-		}
-		ops = append(ops, txn.Op{
-			C:      filesystemsC,
-			Id:     filesystemTag.Id(),
-			Assert: txn.DocExists,
-			Remove: true,
-		})
+		ops = append(ops,
+			txn.Op{
+				C:      filesystemsC,
+				Id:     filesystemId,
+				Assert: txn.DocExists,
+				Remove: true,
+			},
+			removeModelFilesystemRefOp(st, filesystemId),
+		)
 	}
 	return ops, nil
 }
 
-// isFilesystemInherentlyMachineBound reports whether or not the filesystem
-// with the specified tag is inherently bound to the lifetime of the machine,
-// and will be removed along with it, leaving no resources dangling.
-func isFilesystemInherentlyMachineBound(st *State, tag names.FilesystemTag) (bool, error) {
-	// TODO(axw) when we have support for persistent filesystems,
-	// e.g. NFS shares, then we need to check the filesystem info
-	// to decide whether or not to remove.
+// isDetachableFilesystemTag reports whether or not the filesystem with the
+// specified tag is detachable.
+func isDetachableFilesystemTag(st *State, tag names.FilesystemTag) (bool, error) {
+	f, err := st.filesystemByTag(tag)
+	if err != nil {
+		return false, errors.Trace(err)
+	}
+	return f.detachable(), nil
+}
+
+// detachable reports whether or not the filesystem is detachable.
+func (f *filesystem) detachable() bool {
+	return f.doc.MachineId == ""
+}
+
+// isDetachableFilesystemPool reports whether or not the given
+// storage pool will create a filesystem that is not inherently
+// bound to a machine, and therefore can be detached.
+func isDetachableFilesystemPool(st *State, pool string) (bool, error) {
+	_, provider, err := poolStorageProvider(st, pool)
+	if err != nil {
+		return false, errors.Trace(err)
+	}
+	if provider.Scope() == storage.ScopeMachine {
+		// Any storage created by a machine cannot be detached from
+		// the machine, and must be destroyed along with it.
+		return false, nil
+	}
+	if !provider.Dynamic() {
+		// The storage provider only accomodates provisioning storage
+		// statically along with the machine. Such storage is bound
+		// to the machine.
+		return false, nil
+	}
+	if !provider.Supports(storage.StorageKindFilesystem) {
+		// TODO(axw) remove this when volume-backed filesystems
+		// inherit the scope of the volume. For now, volume-backed
+		// filesystems are always machine-scoped.
+		return false, nil
+	}
 	return true, nil
 }
 
 // DetachFilesystem marks the filesystem attachment identified by the specified machine
-// and filesystem tags as Dying, if it is Alive.
+// and filesystem tags as Dying, if it is Alive. DetachFilesystem will fail for
+// inherently machine-bound filesystems.
 func (st *State) DetachFilesystem(machine names.MachineTag, filesystem names.FilesystemTag) (err error) {
 	defer errors.DeferredAnnotatef(&err, "detaching filesystem %s from machine %s", filesystem.Id(), machine.Id())
 	buildTxn := func(attempt int) ([]txn.Op, error) {
@@ -499,6 +511,13 @@ func (st *State) DetachFilesystem(machine names.MachineTag, filesystem names.Fil
 		if fsa.Life() != Alive {
 			return nil, jujutxn.ErrNoOperations
 		}
+		detachable, err := isDetachableFilesystemTag(st, filesystem)
+		if err != nil {
+			return nil, errors.Trace(err)
+		}
+		if !detachable {
+			return nil, errors.New("filesystem is not detachable")
+		}
 		ops := detachFilesystemOps(machine, filesystem)
 		return ops, nil
 	}
@@ -550,16 +569,26 @@ func (st *State) RemoveFilesystemAttachment(machine names.MachineTag, filesystem
 			return nil, errors.Trace(err)
 		}
 		ops := removeFilesystemAttachmentOps(machine, f)
-		// If the filesystem is backed by a volume, the volume
-		// attachment can and should be destroyed once the
-		// filesystem attachment is removed.
 		volumeAttachment, err := st.filesystemVolumeAttachment(machine, filesystem)
 		if err != nil {
 			if errors.Cause(err) != ErrNoBackingVolume && !errors.IsNotFound(err) {
 				return nil, errors.Trace(err)
 			}
-		} else if volumeAttachment.Life() == Alive {
-			ops = append(ops, detachVolumeOps(machine, volumeAttachment.Volume())...)
+		} else {
+			// The filesystem is backed by a volume. Since the
+			// filesystem has been detached, we should now
+			// detach the volume as well if it is detachable.
+			// If the volume is not detachable, we'll just
+			// destroy it along with the filesystem.
+			volume := volumeAttachment.Volume()
+			detachableVolume, err := isDetachableVolumeTag(st, volume)
+			if err != nil {
+				return nil, errors.Trace(err)
+			}
+			if detachableVolume {
+				volOps := detachVolumeOps(machine, volume)
+				ops = append(ops, volOps...)
+			}
 		}
 		return ops, nil
 	}
@@ -569,8 +598,7 @@ func (st *State) RemoveFilesystemAttachment(machine names.MachineTag, filesystem
 func removeFilesystemAttachmentOps(m names.MachineTag, f *filesystem) []txn.Op {
 	decrefFilesystemOp := machineStorageDecrefOp(
 		filesystemsC, f.doc.FilesystemId,
-		f.doc.AttachmentCount, f.doc.Life,
-		m, f.doc.Binding,
+		f.doc.AttachmentCount, f.doc.Life, m,
 	)
 	return []txn.Op{{
 		C:      filesystemAttachmentsC,
@@ -588,9 +616,11 @@ func removeFilesystemAttachmentOps(m names.MachineTag, f *filesystem) []txn.Op {
 // DestroyFilesystem ensures that the filesystem and any attachments to it will
 // be destroyed and removed from state at some point in the future.
 func (st *State) DestroyFilesystem(tag names.FilesystemTag) (err error) {
+	defer errors.DeferredAnnotatef(&err, "destroying filesystem %s", tag.Id())
 	buildTxn := func(attempt int) ([]txn.Op, error) {
 		filesystem, err := st.filesystemByTag(tag)
-		if errors.IsNotFound(err) {
+		if errors.IsNotFound(err) && attempt > 0 {
+			// On the first attempt, we expect it to exist.
 			return nil, jujutxn.ErrNoOperations
 		} else if err != nil {
 			return nil, errors.Trace(err)
@@ -598,29 +628,66 @@ func (st *State) DestroyFilesystem(tag names.FilesystemTag) (err error) {
 		if filesystem.doc.Life != Alive {
 			return nil, jujutxn.ErrNoOperations
 		}
-		return destroyFilesystemOps(st, filesystem), nil
+		if filesystem.doc.StorageId != "" {
+			return nil, errors.Errorf(
+				"filesystem is assigned to %s",
+				names.ReadableString(names.NewStorageTag(filesystem.doc.StorageId)),
+			)
+		}
+		hasNoStorageAssignment := bson.D{{"$or", []bson.D{
+			{{"storageid", ""}},
+			{{"storageid", bson.D{{"$exists", false}}}},
+		}}}
+		return destroyFilesystemOps(st, filesystem, hasNoStorageAssignment)
 	}
 	return st.run(buildTxn)
 }
 
-func destroyFilesystemOps(st *State, f *filesystem) []txn.Op {
+func destroyFilesystemOps(st *State, f *filesystem, extraAssert bson.D) ([]txn.Op, error) {
+	baseAssert := append(isAliveDoc, extraAssert...)
 	if f.doc.AttachmentCount == 0 {
 		hasNoAttachments := bson.D{{"attachmentcount", 0}}
 		return []txn.Op{{
 			C:      filesystemsC,
 			Id:     f.doc.FilesystemId,
-			Assert: append(hasNoAttachments, isAliveDoc...),
+			Assert: append(hasNoAttachments, baseAssert...),
 			Update: bson.D{{"$set", bson.D{{"life", Dead}}}},
-		}}
+		}}, nil
 	}
 	hasAttachments := bson.D{{"attachmentcount", bson.D{{"$gt", 0}}}}
-	cleanupOp := newCleanupOp(cleanupAttachmentsForDyingFilesystem, f.doc.FilesystemId)
-	return []txn.Op{{
+	ops := []txn.Op{{
 		C:      filesystemsC,
 		Id:     f.doc.FilesystemId,
-		Assert: append(hasAttachments, isAliveDoc...),
+		Assert: append(hasAttachments, baseAssert...),
 		Update: bson.D{{"$set", bson.D{{"life", Dying}}}},
-	}, cleanupOp}
+	}}
+	if !f.detachable() {
+		// This filesystem cannot be directly detached, so we do
+		// not issue a cleanup. Since there can (should!) be only
+		// one attachment for the lifetime of the filesystem, we
+		// can look it up and destroy it along with the filesystem.
+		attachments, err := st.FilesystemAttachments(f.FilesystemTag())
+		if err != nil {
+			return nil, errors.Trace(err)
+		}
+		if len(attachments) != 1 {
+			return nil, errors.Errorf(
+				"expected 1 attachment, found %d",
+				len(attachments),
+			)
+		}
+		detachOps := detachFilesystemOps(
+			attachments[0].Machine(),
+			f.FilesystemTag(),
+		)
+		ops = append(ops, detachOps...)
+	} else {
+		ops = append(ops, newCleanupOp(
+			cleanupAttachmentsForDyingFilesystem,
+			f.doc.FilesystemId,
+		))
+	}
+	return ops, nil
 }
 
 // RemoveFilesystem removes the filesystem from state. RemoveFilesystem will
@@ -652,20 +719,23 @@ func removeFilesystemOps(st *State, filesystem Filesystem) ([]txn.Op, error) {
 			Assert: txn.DocExists,
 			Remove: true,
 		},
+		removeModelFilesystemRefOp(st, filesystem.Tag().Id()),
 		removeStatusOp(st, filesystem.globalKey()),
 	}
 	// If the filesystem is backed by a volume, the volume should
-	// be destroyed once the filesystem is removed if it is bound
-	// to the filesystem.
+	// be destroyed once the filesystem is removed. The volume must
+	// not be destroyed before the filesystem is removed.
 	volumeTag, err := filesystem.Volume()
 	if err == nil {
 		volume, err := st.volumeByTag(volumeTag)
 		if err != nil {
 			return nil, errors.Trace(err)
 		}
-		if volume.LifeBinding() == filesystem.Tag() {
-			ops = append(ops, destroyVolumeOps(st, volume)...)
+		volOps, err := destroyVolumeOps(st, volume, nil)
+		if err != nil {
+			return nil, errors.Trace(err)
 		}
+		ops = append(ops, volOps...)
 	} else if err != ErrNoBackingVolume {
 		return nil, errors.Trace(err)
 	}
@@ -711,13 +781,21 @@ func newFilesystemId(st *State, machineId string) (string, error) {
 // directly, a volume will be created and Juju will manage a filesystem
 // on it.
 func (st *State) addFilesystemOps(params FilesystemParams, machineId string) ([]txn.Op, names.FilesystemTag, names.VolumeTag, error) {
-	if params.binding == nil {
-		params.binding = names.NewMachineTag(machineId)
-	}
-	params, err := st.filesystemParamsWithDefaults(params)
+	// TODO(axw) the scope of a volume-backed filesystem should be the
+	// same as the volume. Machine storage provisioners would be
+	// responsible for managing filesystems backed by volumes attached
+	// to that machine. Making this change will enable persistent
+	// filesystems; until then, destroying a volume-backed filesystem
+	// always destroys the volume.
+	params, err := st.filesystemParamsWithDefaults(params, machineId)
 	if err != nil {
 		return nil, names.FilesystemTag{}, names.VolumeTag{}, errors.Trace(err)
 	}
+	detachable, err := isDetachableFilesystemPool(st, params.Pool)
+	if err != nil {
+		return nil, names.FilesystemTag{}, names.VolumeTag{}, errors.Trace(err)
+	}
+	origMachineId := machineId
 	machineId, err = st.validateFilesystemParams(params, machineId)
 	if err != nil {
 		return nil, names.FilesystemTag{}, names.VolumeTag{}, errors.Annotate(err, "validating filesystem params")
@@ -741,7 +819,6 @@ func (st *State) addFilesystemOps(params FilesystemParams, machineId string) ([]
 		var volumeOps []txn.Op
 		volumeParams := VolumeParams{
 			params.storage,
-			filesystemTag, // volume is bound to filesystem
 			params.Pool,
 			params.Size,
 		}
@@ -761,11 +838,13 @@ func (st *State) addFilesystemOps(params FilesystemParams, machineId string) ([]
 		FilesystemId: filesystemId,
 		VolumeId:     volumeId,
 		StorageId:    params.storage.Id(),
-		Binding:      params.binding.String(),
 		Params:       &params,
 		// Every filesystem is created with one attachment.
 		AttachmentCount: 1,
 	}
+	if !detachable {
+		doc.MachineId = origMachineId
+	}
 	ops = append(ops, st.newFilesystemOps(doc, status)...)
 	return ops, filesystemTag, volumeTag, nil
 }
@@ -782,24 +861,23 @@ func (st *State) newFilesystemOps(doc filesystemDoc, status statusDoc) []txn.Op
 	}
 }
 
-func (st *State) filesystemParamsWithDefaults(params FilesystemParams) (FilesystemParams, error) {
-	if params.Pool != "" {
-		return params, nil
+func (st *State) filesystemParamsWithDefaults(params FilesystemParams, machineId string) (FilesystemParams, error) {
+	if params.Pool == "" {
+		modelConfig, err := st.ModelConfig()
+		if err != nil {
+			return FilesystemParams{}, errors.Trace(err)
+		}
+		cons := StorageConstraints{
+			Pool:  params.Pool,
+			Size:  params.Size,
+			Count: 1,
+		}
+		poolName, err := defaultStoragePool(modelConfig, storage.StorageKindFilesystem, cons)
+		if err != nil {
+			return FilesystemParams{}, errors.Annotate(err, "getting default filesystem storage pool")
+		}
+		params.Pool = poolName
 	}
-	envConfig, err := st.ModelConfig()
-	if err != nil {
-		return FilesystemParams{}, errors.Trace(err)
-	}
-	cons := StorageConstraints{
-		Pool:  params.Pool,
-		Size:  params.Size,
-		Count: 1,
-	}
-	poolName, err := defaultStoragePool(envConfig, storage.StorageKindFilesystem, cons)
-	if err != nil {
-		return FilesystemParams{}, errors.Annotate(err, "getting default filesystem storage pool")
-	}
-	params.Pool = poolName
 	return params, nil
 }
 
@@ -1140,7 +1218,7 @@ func filesystemGlobalKey(name string) string {
 
 // FilesystemStatus returns the status of the specified filesystem.
 func (st *State) FilesystemStatus(tag names.FilesystemTag) (status.StatusInfo, error) {
-	return getStatus(st, filesystemGlobalKey(tag.Id()), "filesystem")
+	return getStatus(st.db(), filesystemGlobalKey(tag.Id()), "filesystem")
 }
 
 // SetFilesystemStatus sets the status of the specified filesystem.
@@ -1166,12 +1244,12 @@ func (st *State) SetFilesystemStatus(tag names.FilesystemTag, fsStatus status.St
 	default:
 		return errors.Errorf("cannot set invalid status %q", fsStatus)
 	}
-	return setStatus(st, setStatusParams{
+	return setStatus(st.db(), setStatusParams{
 		badge:     "filesystem",
 		globalKey: filesystemGlobalKey(tag.Id()),
 		status:    fsStatus,
 		message:   info,
 		rawData:   data,
-		updated:   updated,
+		updated:   timeOrNow(updated, st.clock),
 	})
 }
