@@ -5,7 +5,6 @@ package state_test
 
 import (
 	"fmt"
-	"strconv"
 
 	"github.com/juju/errors"
 	jc "github.com/juju/testing/checkers"
@@ -15,6 +14,7 @@ import (
 	"github.com/juju/juju/container"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/network"
+	"github.com/juju/juju/network/containerizer"
 	"github.com/juju/juju/state"
 )
 
@@ -28,6 +28,8 @@ type linkLayerDevicesStateSuite struct {
 
 	otherState        *state.State
 	otherStateMachine *state.Machine
+
+	bridgePolicy *containerizer.BridgePolicy
 }
 
 var _ = gc.Suite(&linkLayerDevicesStateSuite{})
@@ -42,6 +44,7 @@ func (s *linkLayerDevicesStateSuite) SetUpTest(c *gc.C) {
 	s.otherState = s.NewStateForModelNamed(c, "other-model")
 	s.otherStateMachine, err = s.otherState.AddMachine("quantal", state.JobHostUnits)
 	c.Assert(err, jc.ErrorIsNil)
+	s.bridgePolicy = &containerizer.BridgePolicy{}
 }
 
 func (s *linkLayerDevicesStateSuite) TestSetLinkLayerDevicesNoArgs(c *gc.C) {
@@ -143,30 +146,12 @@ func (s *linkLayerDevicesStateSuite) TestSetLinkLayerDevicesWhenMachineNotAliveO
 		Name: "eth0",
 		Type: state.EthernetDevice,
 	}
-	s.assertSetLinkLayerDevicesFailsForArgs(c, args, "machine not found or not alive")
+	s.assertSetLinkLayerDevicesFailsForArgs(c, args, `machine "0" not alive`)
 
 	err = s.machine.Remove()
 	c.Assert(err, jc.ErrorIsNil)
 
-	s.assertSetLinkLayerDevicesFailsForArgs(c, args, "machine not found or not alive")
-}
-
-func (s *linkLayerDevicesStateSuite) TestSetLinkLayerDevicesWhenModelNotAlive(c *gc.C) {
-	otherModel, err := s.otherState.Model()
-	c.Assert(err, jc.ErrorIsNil)
-	err = otherModel.Destroy()
-	c.Assert(err, jc.ErrorIsNil)
-
-	args := state.LinkLayerDeviceArgs{
-		Name: "eth0",
-		Type: state.EthernetDevice,
-	}
-	err = s.otherStateMachine.SetLinkLayerDevices(args)
-	expectedError := fmt.Sprintf(
-		"cannot set link-layer devices to machine %q: model %q is no longer alive",
-		s.otherStateMachine.Id(), otherModel.Name(),
-	)
-	c.Assert(err, gc.ErrorMatches, expectedError)
+	s.assertSetLinkLayerDevicesFailsForArgs(c, args, `machine "0" not alive`)
 }
 
 func (s *linkLayerDevicesStateSuite) TestSetLinkLayerDevicesWithMissingParentSameMachine(c *gc.C) {
@@ -605,6 +590,17 @@ func (s *linkLayerDevicesStateSuite) TestLinkLayerDeviceRemoveRemovesProviderID(
 	c.Assert(err, jc.ErrorIsNil)
 }
 
+func (s *linkLayerDevicesStateSuite) TestSetLinkLayerDevicesNoop(c *gc.C) {
+	args := state.LinkLayerDeviceArgs{
+		Name: "foo",
+		Type: state.EthernetDevice,
+	}
+	err := s.machine.SetLinkLayerDevices(args)
+	c.Assert(err, jc.ErrorIsNil)
+	err = s.machine.SetLinkLayerDevices(args)
+	c.Assert(err, jc.ErrorIsNil)
+}
+
 func (s *linkLayerDevicesStateSuite) removeDeviceAndAssertSuccess(c *gc.C, givenDevice *state.LinkLayerDevice) {
 	err := givenDevice.Remove()
 	c.Assert(err, jc.ErrorIsNil)
@@ -634,6 +630,397 @@ func (s *linkLayerDevicesStateSuite) TestMachineRemoveAllLinkLayerDevicesNoError
 	c.Assert(err, jc.ErrorIsNil)
 }
 
+func (s *linkLayerDevicesStateSuite) createSpaceAndSubnet(c *gc.C, spaceName, CIDR string) {
+	_, err := s.State.AddSpace(spaceName, network.Id(spaceName), nil, true)
+	c.Assert(err, jc.ErrorIsNil)
+	_, err = s.State.AddSubnet(state.SubnetInfo{
+		CIDR:      CIDR,
+		SpaceName: spaceName,
+	})
+	c.Assert(err, jc.ErrorIsNil)
+}
+
+// setupTwoSpaces creates a 'default' and a 'dmz' space, each with a single
+// registered subnet. 10.0.0.0/24 for 'default', and '10.10.0.0/24' for 'dmz'
+func (s *linkLayerDevicesStateSuite) setupTwoSpaces(c *gc.C) {
+	s.createSpaceAndSubnet(c, "default", "10.0.0.0/24")
+	s.createSpaceAndSubnet(c, "dmz", "10.10.0.0/24")
+}
+
+func (s *linkLayerDevicesStateSuite) setupMachineWithOneNIC(c *gc.C) {
+	s.setupTwoSpaces(c)
+	// In the default space
+	s.createNICWithIP(c, s.machine, "eth0", "10.0.0.20/24")
+}
+
+func (s *linkLayerDevicesStateSuite) createNICWithIP(c *gc.C, machine *state.Machine, deviceName, cidrAddress string) {
+	err := machine.SetLinkLayerDevices(
+		state.LinkLayerDeviceArgs{
+			Name:       deviceName,
+			Type:       state.EthernetDevice,
+			ParentName: "",
+			IsUp:       true,
+		},
+	)
+	c.Assert(err, jc.ErrorIsNil)
+	err = machine.SetDevicesAddresses(
+		state.LinkLayerDeviceAddress{
+			DeviceName:   deviceName,
+			CIDRAddress:  cidrAddress,
+			ConfigMethod: state.StaticAddress,
+		},
+	)
+	c.Assert(err, jc.ErrorIsNil)
+}
+
+func (s *linkLayerDevicesStateSuite) createLoopbackNIC(c *gc.C, machine *state.Machine) {
+	err := machine.SetLinkLayerDevices(
+		state.LinkLayerDeviceArgs{
+			Name:       "lo",
+			Type:       state.LoopbackDevice,
+			ParentName: "",
+			IsUp:       true,
+		},
+	)
+	c.Assert(err, jc.ErrorIsNil)
+	err = machine.SetDevicesAddresses(
+		state.LinkLayerDeviceAddress{
+			DeviceName:   "lo",
+			CIDRAddress:  "127.0.0.1/24",
+			ConfigMethod: state.StaticAddress,
+		},
+	)
+	c.Assert(err, jc.ErrorIsNil)
+}
+
+func (s *linkLayerDevicesStateSuite) createBridgeWithIP(c *gc.C, machine *state.Machine, bridgeName, cidrAddress string) {
+	err := machine.SetLinkLayerDevices(
+		state.LinkLayerDeviceArgs{
+			Name:       bridgeName,
+			Type:       state.BridgeDevice,
+			ParentName: "",
+			IsUp:       true,
+		},
+	)
+	c.Assert(err, jc.ErrorIsNil)
+	err = machine.SetDevicesAddresses(
+		state.LinkLayerDeviceAddress{
+			DeviceName:   bridgeName,
+			CIDRAddress:  cidrAddress,
+			ConfigMethod: state.StaticAddress,
+		},
+	)
+	c.Assert(err, jc.ErrorIsNil)
+}
+
+// createAllDefaultDevices creates the loopback, lxcbr0, lxdbr0, and virbr0 devices
+func (s *linkLayerDevicesStateSuite) createAllDefaultDevices(c *gc.C, machine *state.Machine) {
+	// loopback
+	s.createLoopbackNIC(c, s.machine)
+	// container.DefaultLxcBridge
+	s.createBridgeWithIP(c, s.machine, "lxcbr0", "10.0.3.1/24")
+	// container.DefaultLxdBridge
+	s.createBridgeWithIP(c, s.machine, "lxdbr0", "10.0.4.1/24")
+	// container.DefaultKvmBridge
+	s.createBridgeWithIP(c, s.machine, "virbr0", "192.168.124.1/24")
+}
+
+// createNICAndBridgeWithIP creates a network interface and a bridge on the
+// machine, and assigns the requested CIDRAddress to the bridge.
+func (s *linkLayerDevicesStateSuite) createNICAndBridgeWithIP(c *gc.C, machine *state.Machine, deviceName, bridgeName, cidrAddress string) {
+	s.createBridgeWithIP(c, machine, bridgeName, cidrAddress)
+	err := machine.SetLinkLayerDevices(
+		state.LinkLayerDeviceArgs{
+			Name:       deviceName,
+			Type:       state.EthernetDevice,
+			ParentName: bridgeName,
+			IsUp:       true,
+		},
+	)
+	c.Assert(err, jc.ErrorIsNil)
+}
+
+func (s *linkLayerDevicesStateSuite) setupMachineInTwoSpaces(c *gc.C) {
+	s.setupTwoSpaces(c)
+	s.createNICAndBridgeWithIP(c, s.machine, "ens33", "br-ens33", "10.0.0.20/24")
+	s.createNICAndBridgeWithIP(c, s.machine, "ens0p10", "br-ens0p10", "10.10.0.20/24")
+}
+
+func (s *linkLayerDevicesStateSuite) TestLinkLayerDevicesForSpaces(c *gc.C) {
+	s.setupTwoSpaces(c)
+	// Is put into the 'default' space
+	s.createNICAndBridgeWithIP(c, s.machine, "eth0", "br-eth0", "10.0.0.20/24")
+	res, err := s.machine.LinkLayerDevicesForSpaces([]string{"default"})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(res, gc.HasLen, 1)
+	devices, ok := res["default"]
+	c.Check(ok, jc.IsTrue)
+	// TODO(jam): 2016-12-13 Eventually we should probably notice that 'eth0'
+	// *is* part of the right space, possibly just because its parent device
+	// is in the space
+	c.Check(devices, gc.HasLen, 1)
+	c.Check(devices[0].Name(), gc.Equals, "br-eth0")
+	c.Check(devices[0].Type(), gc.Equals, state.BridgeDevice)
+}
+
+func (s *linkLayerDevicesStateSuite) TestLinkLayerDevicesForSpacesNoSuchSpace(c *gc.C) {
+	s.setupTwoSpaces(c)
+	// Is put into the 'default' space
+	s.createNICAndBridgeWithIP(c, s.machine, "eth0", "br-eth0", "10.0.0.20/24")
+	res, err := s.machine.LinkLayerDevicesForSpaces([]string{"dmz"})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(res, gc.HasLen, 0)
+}
+
+func (s *linkLayerDevicesStateSuite) TestLinkLayerDevicesForSpacesNoBridge(c *gc.C) {
+	s.setupMachineWithOneNIC(c)
+	res, err := s.machine.LinkLayerDevicesForSpaces([]string{"default"})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(res, gc.HasLen, 1)
+	devices, ok := res["default"]
+	c.Check(ok, jc.IsTrue)
+	c.Check(devices, gc.HasLen, 1)
+	c.Check(devices[0].Name(), gc.Equals, "eth0")
+	c.Check(devices[0].Type(), gc.Equals, state.EthernetDevice)
+}
+
+func (s *linkLayerDevicesStateSuite) TestLinkLayerDevicesForSpacesMultipleSpaces(c *gc.C) {
+	s.setupTwoSpaces(c)
+	// Is put into the 'default' space
+	s.createNICAndBridgeWithIP(c, s.machine, "eth0", "br-eth0", "10.0.0.20/24")
+	// Now add a NIC in the dmz space, but without a bridge
+	s.createNICWithIP(c, s.machine, "eth1", "10.10.0.20/24")
+	res, err := s.machine.LinkLayerDevicesForSpaces([]string{"default", "dmz"})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(res, gc.HasLen, 2)
+	defaultDevices, ok := res["default"]
+	c.Check(ok, jc.IsTrue)
+	c.Check(defaultDevices, gc.HasLen, 1)
+	c.Check(defaultDevices[0].Name(), gc.Equals, "br-eth0")
+	dmzDevices, ok := res["dmz"]
+	c.Check(ok, jc.IsTrue)
+	c.Check(dmzDevices, gc.HasLen, 1)
+	c.Check(dmzDevices[0].Name(), gc.Equals, "eth1")
+}
+
+func (s *linkLayerDevicesStateSuite) TestLinkLayerDevicesForSpacesWithExtraAddresses(c *gc.C) {
+	s.setupTwoSpaces(c)
+	// Is put into the 'default' space
+	s.createNICAndBridgeWithIP(c, s.machine, "eth0", "br-eth0", "10.0.0.20/24")
+	// When we poll the machine, we include any IP addresses that we
+	// find. One of them is always the loopback, but we could find any
+	// other addresses that someone created on the machine that we
+	// don't know what they are.
+	s.createNICWithIP(c, s.machine, "lo", "127.0.0.1/24")
+	s.createNICWithIP(c, s.machine, "ens5", "172.99.0.24/24")
+	res, err := s.machine.LinkLayerDevicesForSpaces([]string{"default"})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(res, gc.HasLen, 1)
+	defaultDevices, ok := res["default"]
+	c.Check(ok, jc.IsTrue)
+	c.Check(defaultDevices, gc.HasLen, 1)
+	c.Check(defaultDevices[0].Name(), gc.Equals, "br-eth0")
+}
+
+func (s *linkLayerDevicesStateSuite) TestLinkLayerDevicesForSpacesSortOrder(c *gc.C) {
+	s.setupTwoSpaces(c)
+	// Is put into the 'default' space
+	s.createNICAndBridgeWithIP(c, s.machine, "eth0", "br-eth0", "10.0.0.20/24")
+	// Add more devices to the "default" space, to make sure the result comes
+	// back in NaturallySorted order
+	devices := []state.LinkLayerDeviceArgs{{
+		Name:       "eth1",
+		Type:       state.EthernetDevice,
+		MACAddress: "11:23:45:67:89:ab:cd:ef",
+		ParentName: "br-eth1",
+		IsUp:       true,
+	}, {
+		Name:       "br-eth1",
+		Type:       state.BridgeDevice,
+		MACAddress: "11:23:45:67:89:ab:cd:ef",
+		IsUp:       true,
+	}, {
+		Name:       "eth1.1",
+		Type:       state.EthernetDevice,
+		MACAddress: "22:23:45:67:89:ab:cd:ef",
+		ParentName: "br-eth1.1",
+		IsUp:       true,
+	}, {
+		Name:       "br-eth1.1",
+		Type:       state.BridgeDevice,
+		MACAddress: "22:23:45:67:89:ab:cd:ef",
+		IsUp:       true,
+	}, {
+		Name:       "eth1:1",
+		Type:       state.EthernetDevice,
+		MACAddress: "32:23:45:67:89:ab:cd:ef",
+		ParentName: "br-eth1:1",
+		IsUp:       true,
+	}, {
+		Name:       "br-eth1:1",
+		Type:       state.BridgeDevice,
+		MACAddress: "32:23:45:67:89:ab:cd:ef",
+		IsUp:       true,
+	}, {
+		Name:       "eth10",
+		Type:       state.EthernetDevice,
+		MACAddress: "42:23:45:67:89:ab:cd:ef",
+		ParentName: "br-eth10",
+		IsUp:       true,
+	}, {
+		Name:       "br-eth10",
+		Type:       state.BridgeDevice,
+		MACAddress: "42:23:45:67:89:ab:cd:ef",
+		IsUp:       true,
+	}, {
+		Name:       "eth10.2",
+		Type:       state.EthernetDevice,
+		MACAddress: "52:23:45:67:89:ab:cd:ef",
+		ParentName: "br-eth10.2",
+		IsUp:       true,
+	}, {
+		Name:       "br-eth10.2",
+		Type:       state.BridgeDevice,
+		MACAddress: "52:23:45:67:89:ab:cd:ef",
+		IsUp:       true,
+	}, {
+		Name:       "eth2",
+		Type:       state.EthernetDevice,
+		MACAddress: "61:23:45:67:89:ab:cd:ef",
+		IsUp:       true,
+	}, {
+		Name:       "eth20",
+		Type:       state.EthernetDevice,
+		MACAddress: "61:23:45:67:89:ab:cd:ef",
+		IsUp:       true,
+	}, {
+		Name:       "eth3",
+		Type:       state.EthernetDevice,
+		MACAddress: "61:23:45:67:89:ab:cd:ef",
+		IsUp:       true,
+	}}
+	err := s.machine.SetParentLinkLayerDevicesBeforeTheirChildren(devices)
+	c.Assert(err, jc.ErrorIsNil)
+	addresses := make([]state.LinkLayerDeviceAddress, 0, len(devices))
+	for i, device := range devices {
+		if device.ParentName != "" {
+			// Devices *on* the bridge do not end up with IP addresses, only
+			// the Bridge device has an address.
+			continue
+		}
+		addresses = append(addresses, state.LinkLayerDeviceAddress{
+			DeviceName:   device.Name,
+			CIDRAddress:  fmt.Sprintf("10.0.0.1%02d/24", i),
+			ConfigMethod: state.StaticAddress,
+		})
+	}
+	err = s.machine.SetDevicesAddresses(addresses...)
+	c.Assert(err, jc.ErrorIsNil)
+	res, err := s.machine.LinkLayerDevicesForSpaces([]string{"default"})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(res, gc.HasLen, 1)
+	defaultDevices, ok := res["default"]
+	c.Check(ok, jc.IsTrue)
+	names := make([]string, 0, len(defaultDevices))
+	for _, dev := range defaultDevices {
+		names = append(names, dev.Name())
+	}
+	c.Check(names, gc.DeepEquals, []string{
+		"br-eth0", "br-eth1", "br-eth1.1", "br-eth1:1", "br-eth10", "br-eth10.2",
+		"eth2", "eth3", "eth20",
+	})
+}
+
+func (s *linkLayerDevicesStateSuite) TestLinkLayerDevicesForSpacesInUnknownSpace(c *gc.C) {
+	// We explicitly ask for the devices that are not in a known space.
+	s.setupTwoSpaces(c)
+	s.createNICWithIP(c, s.machine, "ens4", "172.99.0.24/24")
+	s.createNICWithIP(c, s.machine, "ens5", "192.168.10.12/24")
+	res, err := s.machine.LinkLayerDevicesForSpaces([]string{""})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(res, gc.HasLen, 1)
+	devices, ok := res[""]
+	c.Assert(ok, jc.IsTrue)
+	c.Assert(devices, gc.HasLen, 2)
+	c.Check(devices[0].Name(), gc.Equals, "ens4")
+	c.Check(devices[0].Type(), gc.Equals, state.EthernetDevice)
+	c.Check(devices[1].Name(), gc.Equals, "ens5")
+	c.Check(devices[1].Type(), gc.Equals, state.EthernetDevice)
+}
+
+func (s *linkLayerDevicesStateSuite) TestLinkLayerDevicesForSpacesWithUnknown(c *gc.C) {
+	// We ask for devices for which we don't know their space, as well as
+	// ones that we do.
+	s.setupTwoSpaces(c)
+	// default space, bridged
+	s.createNICAndBridgeWithIP(c, s.machine, "ens4", "br-ens4", "10.0.0.21/24")
+	// unknown space
+	s.createNICWithIP(c, s.machine, "ens5", "192.168.10.12/24")
+	// loopback device
+	s.createLoopbackNIC(c, s.machine)
+	res, err := s.machine.LinkLayerDevicesForSpaces([]string{"", "default"})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(res, gc.HasLen, 2)
+	devices, ok := res[""]
+	c.Assert(ok, jc.IsTrue)
+	c.Assert(devices, gc.HasLen, 1)
+	c.Check(devices[0].Name(), gc.Equals, "ens5")
+	c.Check(devices[0].Type(), gc.Equals, state.EthernetDevice)
+	devices, ok = res["default"]
+	c.Assert(ok, jc.IsTrue)
+	c.Assert(devices, gc.HasLen, 1)
+	c.Check(devices[0].Name(), gc.Equals, "br-ens4")
+	c.Check(devices[0].Type(), gc.Equals, state.BridgeDevice)
+}
+
+func (s *linkLayerDevicesStateSuite) TestLinkLayerDevicesForSpacesWithNoAddress(c *gc.C) {
+	// We create a record for the 'lxdbr0' bridge, but it doesn't have an
+	// address yet (which is the case when we first show up on a machine.)
+	err := s.machine.SetLinkLayerDevices(
+		state.LinkLayerDeviceArgs{
+			Name:       "lxdbr0",
+			Type:       state.BridgeDevice,
+			ParentName: "",
+			IsUp:       true,
+		},
+	)
+	c.Assert(err, jc.ErrorIsNil)
+	// unknown space
+	s.createNICWithIP(c, s.machine, "ens5", "192.168.10.12/24")
+	res, err := s.machine.LinkLayerDevicesForSpaces([]string{""})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(res, gc.HasLen, 1)
+	devices, ok := res[""]
+	c.Assert(ok, jc.IsTrue)
+	c.Assert(devices, gc.HasLen, 2)
+	names := make([]string, len(devices))
+	for i, dev := range devices {
+		names[i] = dev.Name()
+	}
+	c.Check(names, gc.DeepEquals, []string{"ens5", "lxdbr0"})
+}
+
+func (s *linkLayerDevicesStateSuite) TestLinkLayerDevicesForSpacesUnknownIgnoresLoopAndIncludesKnownBridges(c *gc.C) {
+	// TODO(jam): 2016-12-28 arguably we should also be aware of Docker
+	// devices, possibly the better plan is to look at whether there are
+	// routes from the given bridge out into the rest of the world.
+	s.setupTwoSpaces(c)
+	// Unknown device, bridge, loopback, lxcbr0, lxdbr0 and virbr0
+	s.createNICWithIP(c, s.machine, "ens3", "10.99.0.10/24")
+	s.createNICAndBridgeWithIP(c, s.machine, "ens4", "br-ens4", "10.100.0.21/24")
+	s.createAllDefaultDevices(c, s.machine)
+	res, err := s.machine.LinkLayerDevicesForSpaces([]string{""})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(res, gc.HasLen, 1)
+	devices, ok := res[""]
+	c.Assert(ok, jc.IsTrue)
+	names := make([]string, len(devices))
+	for i, dev := range devices {
+		names[i] = dev.Name()
+	}
+	c.Check(names, gc.DeepEquals, []string{"br-ens4", "ens3", "lxcbr0", "lxdbr0", "virbr0"})
+}
+
 func (s *linkLayerDevicesStateSuite) TestSetLinkLayerDevicesWithLightStateChurn(c *gc.C) {
 	childArgs, churnHook := s.prepareSetLinkLayerDevicesWithStateChurn(c)
 	defer state.SetTestHooks(c, s.State, churnHook).Check()
@@ -780,10 +1167,11 @@ func (s *linkLayerDevicesStateSuite) addContainerMachine(c *gc.C) {
 }
 
 func (s *linkLayerDevicesStateSuite) TestSetLinkLayerDevicesAllowsParentBridgeDeviceForContainerDevice(c *gc.C) {
-	// Add default bridges per container type to ensure they will not be skipped
+	// Add default bridges per container type to ensure they will be skipped
 	// when deciding which host bridges to use for the container NICs.
 	s.addParentBridgeDeviceWithContainerDevicesAsChildren(c, container.DefaultLxdBridge, "vethX", 1)
 	s.addParentBridgeDeviceWithContainerDevicesAsChildren(c, container.DefaultKvmBridge, "vethY", 1)
+	s.addParentBridgeDeviceWithContainerDevicesAsChildren(c, container.DefaultLxcBridge, "vethZ", 1)
 	parentDevice, _ := s.addParentBridgeDeviceWithContainerDevicesAsChildren(c, "br-eth1.250", "eth", 1)
 	childDevice, err := s.containerMachine.LinkLayerDevice("eth0")
 	c.Assert(err, jc.ErrorIsNil)
@@ -927,7 +1315,7 @@ func (s *linkLayerDevicesStateSuite) TestSetLinkLayerDevicesToContainerWhenConta
 		c.Assert(err, jc.ErrorIsNil)
 	}
 
-	s.assertSetLinkLayerDevicesToContainerFailsWithBeforeHook(c, beforeHook, `.*machine not found or not alive`)
+	s.assertSetLinkLayerDevicesToContainerFailsWithBeforeHook(c, beforeHook, `.*machine "0/lxd/0" not alive`)
 }
 
 func (s *linkLayerDevicesStateSuite) assertSetLinkLayerDevicesToContainerFailsWithBeforeHook(c *gc.C, beforeHook func(), expectedError string) {
@@ -1035,99 +1423,3 @@ func (s *linkLayerDevicesStateSuite) testMachineSetParentLinkLayerDevicesBeforeT
 		}
 	}
 }
-
-func (s *linkLayerDevicesStateSuite) TestSetContainerLinkLayerDevices(c *gc.C) {
-	err := s.machine.SetParentLinkLayerDevicesBeforeTheirChildren(nestedDevicesArgs)
-	c.Assert(err, jc.ErrorIsNil)
-	s.addContainerMachine(c)
-	s.assertNoDevicesOnMachine(c, s.containerMachine)
-
-	err = s.machine.SetContainerLinkLayerDevices(s.containerMachine)
-	c.Assert(err, jc.ErrorIsNil)
-
-	containerDevices, err := s.containerMachine.AllLinkLayerDevices()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(containerDevices, gc.HasLen, 3)
-
-	for i, containerDevice := range containerDevices {
-		c.Check(containerDevice.Name(), gc.Matches, "eth"+strconv.Itoa(i))
-		c.Check(containerDevice.Type(), gc.Equals, state.EthernetDevice)
-		c.Check(containerDevice.MTU(), gc.Equals, uint(0)) // inherited from the parent device.
-		c.Check(containerDevice.MACAddress(), gc.Matches, "00:16:3e(:[0-9a-f]{2}){3}")
-		c.Check(containerDevice.IsUp(), jc.IsTrue)
-		c.Check(containerDevice.IsAutoStart(), jc.IsTrue)
-		c.Check(containerDevice.ParentName(), gc.Matches, `m#0#d#br-bond0(|\.12|\.34)`)
-	}
-}
-
-func (s *linkLayerDevicesStateSuite) TestSetContainerLinkLayerDevicesCorrectlyPaired(c *gc.C) {
-	// The device names chosen and the order are very explicit. We
-	// need to ensure that we have a list that does not sort well
-	// alphabetically. This is because SetParentLinkLayerDevices()
-	// uses a natural sort ordering and we want to verify the
-	// pairing between the container's NIC name and its parent in
-	// the host machine during this unit test.
-
-	devicesArgs := []state.LinkLayerDeviceArgs{
-		state.LinkLayerDeviceArgs{
-			Name: "br-eth10",
-			Type: state.BridgeDevice,
-		},
-		state.LinkLayerDeviceArgs{
-			Name: "br-eth1",
-			Type: state.BridgeDevice,
-		},
-		state.LinkLayerDeviceArgs{
-			Name: "br-eth10.100",
-			Type: state.BridgeDevice,
-		},
-		state.LinkLayerDeviceArgs{
-			Name: "br-eth2",
-			Type: state.BridgeDevice,
-		},
-		state.LinkLayerDeviceArgs{
-			Name: "br-eth0",
-			Type: state.BridgeDevice,
-		},
-		state.LinkLayerDeviceArgs{
-			Name: "br-eth4",
-			Type: state.BridgeDevice,
-		},
-		state.LinkLayerDeviceArgs{
-			Name: "br-eth3",
-			Type: state.BridgeDevice,
-		},
-	}
-
-	expectedParents := []string{
-		"br-eth0",
-		"br-eth1",
-		"br-eth2",
-		"br-eth3",
-		"br-eth4",
-		"br-eth10",
-		"br-eth10.100",
-	}
-
-	err := s.machine.SetParentLinkLayerDevicesBeforeTheirChildren(devicesArgs[:])
-	c.Assert(err, jc.ErrorIsNil)
-	s.addContainerMachine(c)
-	s.assertNoDevicesOnMachine(c, s.containerMachine)
-
-	err = s.machine.SetContainerLinkLayerDevices(s.containerMachine)
-	c.Assert(err, jc.ErrorIsNil)
-
-	containerDevices, err := s.containerMachine.AllLinkLayerDevices()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(containerDevices, gc.HasLen, len(devicesArgs))
-
-	for i, containerDevice := range containerDevices {
-		c.Check(containerDevice.Name(), gc.Matches, "eth"+strconv.Itoa(i))
-		c.Check(containerDevice.Type(), gc.Equals, state.EthernetDevice)
-		c.Check(containerDevice.MTU(), gc.Equals, uint(0)) // inherited from the parent device.
-		c.Check(containerDevice.MACAddress(), gc.Matches, "00:16:3e(:[0-9a-f]{2}){3}")
-		c.Check(containerDevice.IsUp(), jc.IsTrue)
-		c.Check(containerDevice.IsAutoStart(), jc.IsTrue)
-		c.Check(containerDevice.ParentName(), gc.Equals, fmt.Sprintf("m#0#d#%s", expectedParents[i]))
-	}
-}
