@@ -208,7 +208,6 @@ type instanceData struct {
 	MachineId  string      `bson:"machineid"`
 	InstanceId instance.Id `bson:"instanceid"`
 	ModelUUID  string      `bson:"model-uuid"`
-	Status     string      `bson:"status,omitempty"`
 	Arch       *string     `bson:"arch,omitempty"`
 	Mem        *uint64     `bson:"mem,omitempty"`
 	RootDisk   *uint64     `bson:"rootdisk,omitempty"`
@@ -216,6 +215,10 @@ type instanceData struct {
 	CpuPower   *uint64     `bson:"cpupower,omitempty"`
 	Tags       *[]string   `bson:"tags,omitempty"`
 	AvailZone  *string     `bson:"availzone,omitempty"`
+
+	// KeepInstance is set to true if, on machine removal from Juju,
+	// the cloud instance should be retained.
+	KeepInstance bool `bson:"keep-instance,omitempty"`
 }
 
 func hardwareCharacteristics(instData instanceData) *instance.HardwareCharacteristics {
@@ -240,7 +243,7 @@ func (m *Machine) HardwareCharacteristics() (*instance.HardwareCharacteristics,
 }
 
 func getInstanceData(st *State, id string) (instanceData, error) {
-	instanceDataCollection, closer := st.getCollection(instanceDataC)
+	instanceDataCollection, closer := st.db().GetCollection(instanceDataC)
 	defer closer()
 
 	var instData instanceData
@@ -278,6 +281,34 @@ func (m *Machine) Jobs() []MachineJob {
 	return m.doc.Jobs
 }
 
+// SetKeepInstance sets whether the cloud machine instance
+// will be retained when the machine is removed from Juju.
+// This is only relevant if an instance exists.
+func (m *Machine) SetKeepInstance(keepInstance bool) error {
+	ops := []txn.Op{{
+		C:      instanceDataC,
+		Id:     m.doc.DocID,
+		Assert: txn.DocExists,
+		Update: bson.D{{"$set", bson.D{{"keep-instance", keepInstance}}}},
+	}}
+	if err := m.st.runTransaction(ops); err != nil {
+		// If instance doc doesn't exist, that's ok; there's nothing to keep,
+		// but that's not an error we care about.
+		return errors.Annotatef(onAbort(err, nil), "cannot set KeepInstance on machine %v", m)
+	}
+	return nil
+}
+
+// KeepInstance reports whether a machine, when removed from
+// Juju, will cause the corresponding cloud instance to be stopped.
+func (m *Machine) KeepInstance() (bool, error) {
+	instData, err := getInstanceData(m.st, m.Id())
+	if err != nil {
+		return false, err
+	}
+	return instData.KeepInstance, nil
+}
+
 // WantsVote reports whether the machine is a controller
 // that wants to take part in peer voting.
 func (m *Machine) WantsVote() bool {
@@ -515,7 +546,7 @@ func IsHasAssignedUnitsError(err error) bool {
 // Containers returns the container ids belonging to a parent machine.
 // TODO(wallyworld): move this method to a service
 func (m *Machine) Containers() ([]string, error) {
-	containerRefs, closer := m.st.getCollection(containerRefsC)
+	containerRefs, closer := m.st.db().GetCollection(containerRefsC)
 	defer closer()
 
 	var mc machineContainers
@@ -729,7 +760,7 @@ func (original *Machine) advanceLifecycle(life Life) (err error) {
 
 		if life == Dead {
 			// A machine may not become Dead until it has no more
-			// attachments to inherently machine-bound storage.
+			// attachments to detachable storage.
 			storageAsserts, err := m.assertNoPersistentStorage()
 			if err != nil {
 				return nil, errors.Trace(err)
@@ -754,21 +785,21 @@ func (m *Machine) assertNoPersistentStorage() (bson.D, error) {
 	attachments := make(set.Tags)
 	for _, v := range m.doc.Volumes {
 		tag := names.NewVolumeTag(v)
-		machineBound, err := isVolumeInherentlyMachineBound(m.st, tag)
+		detachable, err := isDetachableVolumeTag(m.st, tag)
 		if err != nil {
 			return nil, errors.Trace(err)
 		}
-		if !machineBound {
+		if detachable {
 			attachments.Add(tag)
 		}
 	}
 	for _, f := range m.doc.Filesystems {
 		tag := names.NewFilesystemTag(f)
-		machineBound, err := isFilesystemInherentlyMachineBound(m.st, tag)
+		detachable, err := isDetachableFilesystemTag(m.st, tag)
 		if err != nil {
 			return nil, errors.Trace(err)
 		}
-		if !machineBound {
+		if detachable {
 			attachments.Add(tag)
 		}
 	}
@@ -861,11 +892,11 @@ func (m *Machine) removeOps() ([]txn.Op, error) {
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
-	filesystemOps, err := m.st.removeMachineFilesystemsOps(m.MachineTag())
+	filesystemOps, err := m.st.removeMachineFilesystemsOps(m)
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
-	volumeOps, err := m.st.removeMachineVolumesOps(m.MachineTag())
+	volumeOps, err := m.st.removeMachineVolumesOps(m)
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
@@ -923,17 +954,22 @@ func (m *Machine) Refresh() error {
 
 // AgentPresence returns whether the respective remote agent is alive.
 func (m *Machine) AgentPresence() (bool, error) {
-	pwatcher := m.st.workers.PresenceWatcher()
+	pwatcher := m.st.workers.presenceWatcher()
 	return pwatcher.Alive(m.globalKey())
 }
 
 // WaitAgentPresence blocks until the respective agent is alive.
+// These should really only be used in the test suite.
 func (m *Machine) WaitAgentPresence(timeout time.Duration) (err error) {
 	defer errors.DeferredAnnotatef(&err, "waiting for agent of machine %v", m)
 	ch := make(chan presence.Change)
-	pwatcher := m.st.workers.PresenceWatcher()
+	pwatcher := m.st.workers.presenceWatcher()
 	pwatcher.Watch(m.globalKey(), ch)
 	defer pwatcher.Unwatch(m.globalKey(), ch)
+	pingBatcher := m.st.getPingBatcher()
+	if err := pingBatcher.Sync(); err != nil {
+		return err
+	}
 	for i := 0; i < 2; i++ {
 		select {
 		case change := <-ch:
@@ -954,11 +990,15 @@ func (m *Machine) WaitAgentPresence(timeout time.Duration) (err error) {
 // It returns the started pinger.
 func (m *Machine) SetAgentPresence() (*presence.Pinger, error) {
 	presenceCollection := m.st.getPresenceCollection()
-	p := presence.NewPinger(presenceCollection, m.st.modelTag, m.globalKey())
+	recorder := m.st.getPingBatcher()
+	p := presence.NewPinger(presenceCollection, m.st.modelTag, m.globalKey(),
+		func() presence.PingRecorder { return m.st.getPingBatcher() })
 	err := p.Start()
 	if err != nil {
 		return nil, err
 	}
+	// Make sure this Agent status is written to the database before returning.
+	recorder.Sync()
 	// We preform a manual sync here so that the
 	// presence pinger has the most up-to-date information when it
 	// starts. This ensures that commands run immediately after bootstrap
@@ -967,7 +1007,7 @@ func (m *Machine) SetAgentPresence() (*presence.Pinger, error) {
 	//
 	// TODO: Does not work for multiple controllers. Trigger a sync across all controllers.
 	if m.IsManager() {
-		m.st.workers.PresenceWatcher().Sync()
+		m.st.workers.presenceWatcher().Sync()
 	}
 	return p, nil
 }
@@ -988,7 +1028,7 @@ func (m *Machine) InstanceId() (instance.Id, error) {
 // InstanceStatus returns the provider specific instance status for this machine,
 // or a NotProvisionedError if instance is not yet provisioned.
 func (m *Machine) InstanceStatus() (status.StatusInfo, error) {
-	machineStatus, err := getStatus(m.st, m.globalInstanceKey(), "instance")
+	machineStatus, err := getStatus(m.st.db(), m.globalInstanceKey(), "instance")
 	if err != nil {
 		logger.Warningf("error when retrieving instance status for machine: %s, %v", m.Id(), err)
 		return status.StatusInfo{}, err
@@ -998,13 +1038,13 @@ func (m *Machine) InstanceStatus() (status.StatusInfo, error) {
 
 // SetInstanceStatus sets the provider specific instance status for a machine.
 func (m *Machine) SetInstanceStatus(sInfo status.StatusInfo) (err error) {
-	return setStatus(m.st, setStatusParams{
+	return setStatus(m.st.db(), setStatusParams{
 		badge:     "instance",
 		globalKey: m.globalInstanceKey(),
 		status:    sInfo.Status,
 		message:   sInfo.Message,
 		rawData:   sInfo.Data,
-		updated:   sInfo.Since,
+		updated:   timeOrNow(sInfo.Since, m.st.clock),
 	})
 
 }
@@ -1016,7 +1056,7 @@ func (m *Machine) SetInstanceStatus(sInfo status.StatusInfo) (err error) {
 // this juju machine is deployed.
 func (m *Machine) InstanceStatusHistory(filter status.StatusHistoryFilter) ([]status.StatusInfo, error) {
 	args := &statusHistoryArgs{
-		st:        m.st,
+		db:        m.st.db(),
 		globalKey: m.globalInstanceKey(),
 		filter:    filter,
 	}
@@ -1043,7 +1083,7 @@ func (m *Machine) AvailabilityZone() (string, error) {
 // Units returns all the units that have been assigned to the machine.
 func (m *Machine) Units() (units []*Unit, err error) {
 	defer errors.DeferredAnnotatef(&err, "cannot get units assigned to machine %v", m)
-	unitsCollection, closer := m.st.getCollection(unitsC)
+	unitsCollection, closer := m.st.db().GetCollection(unitsC)
 	defer closer()
 
 	pudocs := []unitDoc{}
@@ -1065,6 +1105,75 @@ func (m *Machine) Units() (units []*Unit, err error) {
 	return units, nil
 }
 
+// XXX(jam): 2016-12-09 These are just copied from
+// provider/maas/constraints.go, but they should be tied to machine
+// constraints, *not* tied to provider/maas constraints.
+// convertSpacesFromConstraints extracts spaces from constraints and converts
+// them to two lists of positive and negative spaces.
+func convertSpacesFromConstraints(spaces *[]string) ([]string, []string) {
+	if spaces == nil || len(*spaces) == 0 {
+		return nil, nil
+	}
+	return parseDelimitedValues(*spaces)
+}
+
+// parseDelimitedValues parses a slice of raw values coming from constraints
+// (Tags or Spaces). The result is split into two slices - positives and
+// negatives (prefixed with "^"). Empty values are ignored.
+func parseDelimitedValues(rawValues []string) (positives, negatives []string) {
+	for _, value := range rawValues {
+		if value == "" || value == "^" {
+			// Neither of these cases should happen in practise, as constraints
+			// are validated before setting them and empty names for spaces or
+			// tags are not allowed.
+			continue
+		}
+		if strings.HasPrefix(value, "^") {
+			negatives = append(negatives, strings.TrimPrefix(value, "^"))
+		} else {
+			positives = append(positives, value)
+		}
+	}
+	return positives, negatives
+}
+
+// DesiredSpaces returns the name of all spaces that this machine needs
+// access to.  This is the combined value of all of the direct constraints
+// for the machine, as well as the spaces listed for all bindings of units
+// being deployed to that machine.
+func (m *Machine) DesiredSpaces() (set.Strings, error) {
+	spaces := set.NewStrings()
+	units, err := m.Units()
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	constraints, err := m.Constraints()
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	// We ignore negative spaces as it doesn't change what spaces we do want.
+	positiveSpaces, _ := convertSpacesFromConstraints(constraints.Spaces)
+	for _, space := range positiveSpaces {
+		spaces.Add(space)
+	}
+	bindings := set.NewStrings()
+	for _, unit := range units {
+		app, err := unit.Application()
+		if err != nil {
+			return nil, errors.Trace(err)
+		}
+		endpointBindings, err := app.EndpointBindings()
+		for _, space := range endpointBindings {
+			if space != "" {
+				bindings.Add(space)
+			}
+		}
+	}
+	logger.Tracef("machine %q found constraints %s and bindings %s",
+		m.Id(), network.QuoteSpaceSet(spaces), network.QuoteSpaceSet(bindings))
+	return spaces.Union(bindings), nil
+}
+
 // SetProvisioned sets the provider specific machine id, nonce and also metadata for
 // this machine. Once set, the instance id cannot be changed.
 //
@@ -1080,6 +1189,16 @@ func (m *Machine) SetProvisioned(id instance.Id, nonce string, characteristics *
 		return fmt.Errorf("instance id and nonce cannot be empty")
 	}
 
+	coll, closer := m.st.getRawCollection(instanceDataC)
+	defer closer()
+	count, err := coll.Find(bson.D{{"instanceid", id}}).Count()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	if count > 0 {
+		logger.Warningf("duplicate instance id %q already saved", id)
+	}
+
 	if characteristics == nil {
 		characteristics = &instance.HardwareCharacteristics{}
 	}
@@ -1141,12 +1260,30 @@ func (m *Machine) SetInstanceInfo(
 	if err := m.SetDevicesAddressesIdempotently(devicesAddrs); err != nil {
 		return errors.Trace(err)
 	}
+
+	// Record volumes and volume attachments, and set the initial
+	// status: attached or attaching.
 	if err := setProvisionedVolumeInfo(m.st, volumes); err != nil {
 		return errors.Trace(err)
 	}
 	if err := setMachineVolumeAttachmentInfo(m.st, m.Id(), volumeAttachments); err != nil {
 		return errors.Trace(err)
 	}
+	volumeStatus := make(map[names.VolumeTag]status.Status)
+	for tag := range volumes {
+		volumeStatus[tag] = status.Attaching
+	}
+	for tag := range volumeAttachments {
+		volumeStatus[tag] = status.Attached
+	}
+	for tag, status := range volumeStatus {
+		if err := m.st.SetVolumeStatus(tag, status, "", nil, nil); err != nil {
+			return errors.Annotatef(
+				err, "setting status of %s", names.ReadableString(tag),
+			)
+		}
+	}
+
 	return m.SetProvisioned(id, nonce, characteristics)
 }
 
@@ -1519,7 +1656,7 @@ func (m *Machine) SetConstraints(cons constraints.Value) (err error) {
 
 // Status returns the status of the machine.
 func (m *Machine) Status() (status.StatusInfo, error) {
-	mStatus, err := getStatus(m.st, m.globalKey(), "machine")
+	mStatus, err := getStatus(m.st.db(), m.globalKey(), "machine")
 	if err != nil {
 		return mStatus, err
 	}
@@ -1548,13 +1685,13 @@ func (m *Machine) SetStatus(statusInfo status.StatusInfo) error {
 	default:
 		return errors.Errorf("cannot set invalid status %q", statusInfo.Status)
 	}
-	return setStatus(m.st, setStatusParams{
+	return setStatus(m.st.db(), setStatusParams{
 		badge:     "machine",
 		globalKey: m.globalKey(),
 		status:    statusInfo.Status,
 		message:   statusInfo.Message,
 		rawData:   statusInfo.Data,
-		updated:   statusInfo.Since,
+		updated:   timeOrNow(statusInfo.Since, m.st.clock),
 	})
 }
 
@@ -1563,7 +1700,7 @@ func (m *Machine) SetStatus(statusInfo status.StatusInfo) error {
 // representing past statuses for this machine.
 func (m *Machine) StatusHistory(filter status.StatusHistoryFilter) ([]status.StatusInfo, error) {
 	args := &statusHistoryArgs{
-		st:        m.st,
+		db:        m.st.db(),
 		globalKey: m.globalKey(),
 		filter:    filter,
 	}
