@@ -9,20 +9,25 @@ import (
 	"net"
 
 	"github.com/juju/errors"
+	jujutxn "github.com/juju/txn"
 	"github.com/juju/utils/set"
 	"gopkg.in/mgo.v2"
 	"gopkg.in/mgo.v2/bson"
 	"gopkg.in/mgo.v2/txn"
 
-	"github.com/juju/juju/container"
 	"github.com/juju/juju/network"
 )
 
+// defaultSpaceName is the name of the default space to assign containers.
+// Currently hard-coded to 'default', we may consider making this a model
+// config
+const defaultSpaceName = "default"
+
 // LinkLayerDevice returns the link-layer device matching the given name. An
 // error satisfying errors.IsNotFound() is returned when no such device exists
 // on the machine.
 func (m *Machine) LinkLayerDevice(name string) (*LinkLayerDevice, error) {
-	linkLayerDevices, closer := m.st.getCollection(linkLayerDevicesC)
+	linkLayerDevices, closer := m.st.db().GetCollection(linkLayerDevicesC)
 	defer closer()
 
 	linkLayerDeviceDocID := m.linkLayerDeviceDocIDFromName(name)
@@ -64,7 +69,7 @@ func (m *Machine) AllLinkLayerDevices() ([]*LinkLayerDevice, error) {
 }
 
 func (m *Machine) forEachLinkLayerDeviceDoc(docFieldsToSelect bson.D, callbackFunc func(resultDoc *linkLayerDeviceDoc)) error {
-	linkLayerDevices, closer := m.st.getCollection(linkLayerDevicesC)
+	linkLayerDevices, closer := m.st.db().GetCollection(linkLayerDevicesC)
 	defer closer()
 
 	query := linkLayerDevices.Find(bson.D{{"machine-id", m.doc.Id}})
@@ -178,7 +183,7 @@ func (m *Machine) SetLinkLayerDevices(devicesArgs ...LinkLayerDeviceArgs) (err e
 	defer errors.DeferredAnnotatef(&err, "cannot set link-layer devices to machine %q", m.doc.Id)
 
 	if len(devicesArgs) == 0 {
-		logger.Warningf("no device addresses to set")
+		logger.Debugf("no device addresses to set")
 		return nil
 	}
 
@@ -188,10 +193,10 @@ func (m *Machine) SetLinkLayerDevices(devicesArgs ...LinkLayerDeviceArgs) (err e
 			return nil, errors.Trace(err)
 		}
 
+		if m.doc.Life != Alive {
+			return nil, errors.Errorf("machine %q not alive", m.doc.Id)
+		}
 		if attempt > 0 {
-			if err := checkModelActive(m.st); err != nil {
-				return nil, errors.Trace(err)
-			}
 			if err := m.isStillAlive(); err != nil {
 				return nil, errors.Trace(err)
 			}
@@ -207,8 +212,9 @@ func (m *Machine) SetLinkLayerDevices(devicesArgs ...LinkLayerDeviceArgs) (err e
 			}
 		}
 
+		// We've checked the model is alive directly, and we assert the machine is alive, we don't need to also
+		// assert the model is alive, because then the machine would be dying as well.
 		ops := []txn.Op{
-			assertModelActiveOp(m.st.ModelUUID()),
 			m.assertAliveOp(),
 		}
 
@@ -216,6 +222,11 @@ func (m *Machine) SetLinkLayerDevices(devicesArgs ...LinkLayerDeviceArgs) (err e
 		if err != nil {
 			return nil, errors.Trace(err)
 		}
+		if len(setDevicesOps) == 0 {
+			// No need to assert only that the machine is alive
+			logger.Debugf("no changes to LinkLayerDevices for machine %q", m.Id())
+			return nil, jujutxn.ErrNoOperations
+		}
 		return append(ops, setDevicesOps...), nil
 	}
 	if err := m.st.run(buildTxn); err != nil {
@@ -233,7 +244,7 @@ func (st *State) allProviderIDsForAddresses() (set.Strings, error) {
 }
 
 func (st *State) allProviderIDsForEntity(entityName string) (set.Strings, error) {
-	idCollection, closer := st.getCollection(providerIDsC)
+	idCollection, closer := st.db().GetCollection(providerIDsC)
 	defer closer()
 
 	allProviderIDs := set.NewStrings()
@@ -438,7 +449,7 @@ func (m *Machine) assertAliveOp() txn.Op {
 }
 
 func (m *Machine) setDevicesFromDocsOps(newDocs []linkLayerDeviceDoc) ([]txn.Op, error) {
-	devices, closer := m.st.getCollection(linkLayerDevicesC)
+	devices, closer := m.st.db().GetCollection(linkLayerDevicesC)
 	defer closer()
 
 	var ops []txn.Op
@@ -503,6 +514,7 @@ func (m *Machine) parentDocIDFromDeviceDoc(doc *linkLayerDeviceDoc) (string, err
 }
 
 func (m *Machine) updateLinkLayerDeviceOps(existingDoc, newDoc *linkLayerDeviceDoc) (ops []txn.Op, err error) {
+	// none of the ops in this function are assert-only, so callers can know if there are any changes by just checking len(ops)
 	var newParentDocID string
 	if newDoc.ParentName != "" {
 		newParentDocID, err = m.parentDocIDFromDeviceDoc(newDoc)
@@ -532,7 +544,11 @@ func (m *Machine) updateLinkLayerDeviceOps(existingDoc, newDoc *linkLayerDeviceD
 		ops = append(ops, assertLinkLayerDeviceExistsOp(existingParentDocID))
 		ops = append(ops, decrementDeviceNumChildrenOp(existingParentDocID))
 	}
-	ops = append(ops, updateLinkLayerDeviceDocOp(existingDoc, newDoc))
+	updateDeviceOp, deviceHasChanges := updateLinkLayerDeviceDocOp(existingDoc, newDoc)
+	if deviceHasChanges {
+		// we only include the op if it will actually change something
+		ops = append(ops, updateDeviceOp)
+	}
 
 	if newDoc.ProviderID != "" {
 		if existingDoc.ProviderID != "" && existingDoc.ProviderID != newDoc.ProviderID {
@@ -591,7 +607,7 @@ func (m *Machine) SetDevicesAddresses(devicesAddresses ...LinkLayerDeviceAddress
 	defer errors.DeferredAnnotatef(&err, "cannot set link-layer device addresses of machine %q", m.doc.Id)
 
 	if len(devicesAddresses) == 0 {
-		logger.Warningf("no device addresses to set")
+		logger.Debugf("no device addresses to set")
 		return nil
 	}
 
@@ -601,13 +617,10 @@ func (m *Machine) SetDevicesAddresses(devicesAddresses ...LinkLayerDeviceAddress
 			return nil, errors.Trace(err)
 		}
 
+		if err := m.isStillAlive(); err != nil {
+			return nil, errors.Trace(err)
+		}
 		if attempt > 0 {
-			if err := checkModelActive(m.st); err != nil {
-				return nil, errors.Trace(err)
-			}
-			if err := m.isStillAlive(); err != nil {
-				return nil, errors.Trace(err)
-			}
 			allIds, err := m.st.allProviderIDsForAddresses()
 			if err != nil {
 				return nil, errors.Trace(err)
@@ -620,8 +633,9 @@ func (m *Machine) SetDevicesAddresses(devicesAddresses ...LinkLayerDeviceAddress
 			}
 		}
 
+		// we checked the model is active, but we only assert the machine is alive, because it will be dying if
+		// the model is dying.
 		ops := []txn.Op{
-			assertModelActiveOp(m.st.ModelUUID()),
 			m.assertAliveOp(),
 		}
 
@@ -629,6 +643,12 @@ func (m *Machine) SetDevicesAddresses(devicesAddresses ...LinkLayerDeviceAddress
 		if err != nil {
 			return nil, errors.Trace(err)
 		}
+		if len(setAddressesOps) == 0 {
+			// no actual address changes to be queued, so no need to create an op that just asserts
+			// the machine is alive
+			logger.Debugf("no changes to DevicesAddresses for machine %q", m.Id())
+			return nil, jujutxn.ErrNoOperations
+		}
 		return append(ops, setAddressesOps...), nil
 	}
 	if err := m.st.run(buildTxn); err != nil {
@@ -712,7 +732,7 @@ func (m *Machine) newIPAddressDocFromArgs(args *LinkLayerDeviceAddress) (*ipAddr
 	subnetCIDR := ipNet.String()
 	subnet, err := m.st.Subnet(subnetCIDR)
 	if errors.IsNotFound(err) {
-		logger.Infof(
+		logger.Debugf(
 			"address %q on machine %q uses unknown or machine-local subnet %q",
 			addressValue, m.Id(), subnetCIDR,
 		)
@@ -752,27 +772,32 @@ func (m *Machine) verifySubnetAlive(subnet *Subnet) error {
 }
 
 func (m *Machine) setDevicesAddressesFromDocsOps(newDocs []ipAddressDoc) ([]txn.Op, error) {
-	addresses, closer := m.st.getCollection(ipAddressesC)
+	addresses, closer := m.st.db().GetCollection(ipAddressesC)
 	defer closer()
 
 	var ops []txn.Op
 
 	for _, newDoc := range newDocs {
+		var thisDeviceOps []txn.Op
+		hasChanges := false
 		deviceDocID := m.linkLayerDeviceDocIDFromName(newDoc.DeviceName)
-		ops = append(ops, assertLinkLayerDeviceExistsOp(deviceDocID))
+		thisDeviceOps = append(thisDeviceOps, assertLinkLayerDeviceExistsOp(deviceDocID))
 
 		var existingDoc ipAddressDoc
 		err := addresses.FindId(newDoc.DocID).One(&existingDoc)
 		if err == mgo.ErrNotFound {
 			// Address does not exist yet - insert it.
-			ops = append(ops, insertIPAddressDocOp(&newDoc))
+			hasChanges = true
+			thisDeviceOps = append(thisDeviceOps, insertIPAddressDocOp(&newDoc))
 			if newDoc.ProviderID != "" {
 				id := network.Id(newDoc.ProviderID)
-				ops = append(ops, m.st.networkEntityGlobalKeyOp("address", id))
+				thisDeviceOps = append(thisDeviceOps, m.st.networkEntityGlobalKeyOp("address", id))
 			}
 		} else if err == nil {
 			// Address already exists - update what's possible.
-			ops = append(ops, updateIPAddressDocOp(&existingDoc, &newDoc))
+			var ipOp txn.Op
+			ipOp, hasChanges = updateIPAddressDocOp(&existingDoc, &newDoc)
+			thisDeviceOps = append(thisDeviceOps, ipOp)
 			if newDoc.ProviderID != "" {
 				if existingDoc.ProviderID != "" && existingDoc.ProviderID != newDoc.ProviderID {
 					return nil, errors.Errorf("cannot change ProviderID of link address %q", existingDoc.Value)
@@ -780,17 +805,21 @@ func (m *Machine) setDevicesAddressesFromDocsOps(newDocs []ipAddressDoc) ([]txn.
 				if existingDoc.ProviderID != newDoc.ProviderID {
 					// Need to insert the new provider id in providerIDsC
 					id := network.Id(newDoc.ProviderID)
-					ops = append(ops, m.st.networkEntityGlobalKeyOp("address", id))
+					thisDeviceOps = append(thisDeviceOps, m.st.networkEntityGlobalKeyOp("address", id))
+					hasChanges = true
 				}
 			}
 		} else {
 			return nil, errors.Trace(err)
 		}
 
-		ops, err = m.maybeAssertSubnetAliveOps(&newDoc, ops)
+		thisDeviceOps, err = m.maybeAssertSubnetAliveOps(&newDoc, thisDeviceOps)
 		if err != nil {
 			return nil, errors.Trace(err)
 		}
+		if hasChanges {
+			ops = append(ops, thisDeviceOps...)
+		}
 	}
 	return ops, nil
 }
@@ -847,6 +876,37 @@ func (m *Machine) AllAddresses() ([]*Address, error) {
 	return allAddresses, nil
 }
 
+// AllSpaces returns the set of spaces that this machine is actively
+// connected to.
+func (m *Machine) AllSpaces() (set.Strings, error) {
+	// TODO(jam): 2016-12-18 This should evolve to look at the
+	// LinkLayerDevices directly, instead of using the Addresses the devices
+	// are in to link back to spaces.
+	spaces := set.NewStrings()
+	addresses, err := m.AllAddresses()
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	for _, address := range addresses {
+		subnet, err := address.Subnet()
+		if err != nil {
+			if errors.IsNotFound(err) {
+				// We don't know what this subnet is, so it can't be a space. It
+				// might just be the loopback device.
+				continue
+			}
+			return nil, errors.Trace(err)
+		}
+		spaceName := subnet.SpaceName()
+		if spaceName != "" {
+			spaces.Add(spaceName)
+		}
+	}
+	logger.Tracef("machine %q found AllSpaces() = %s",
+		m.Id(), network.QuoteSpaceSet(spaces))
+	return spaces, nil
+}
+
 // AllNetworkAddresses returns the result of AllAddresses(), but transformed to
 // []network.Address.
 func (m *Machine) AllNetworkAddresses() ([]network.Address, error) {
@@ -859,9 +919,112 @@ func (m *Machine) AllNetworkAddresses() ([]network.Address, error) {
 	for i := range stateAddresses {
 		networkAddresses[i] = stateAddresses[i].NetworkAddress()
 	}
+	// TODO(jam): 20161130 NetworkAddress object has a SpaceName attribute.
+	// However, we are not filling in that information here.
 	return networkAddresses, nil
 }
 
+// deviceMapToSortedList takes a map from device name to LinkLayerDevice
+// object, and returns the list of LinkLayerDevice object using
+// NaturallySortDeviceNames
+func deviceMapToSortedList(deviceMap map[string]*LinkLayerDevice) []*LinkLayerDevice {
+	names := make([]string, 0, len(deviceMap))
+	for name, _ := range deviceMap {
+		// name must == device.Name()
+		names = append(names, name)
+	}
+	sortedNames := network.NaturallySortDeviceNames(names...)
+	result := make([]*LinkLayerDevice, len(sortedNames))
+	for i, name := range sortedNames {
+		result[i] = deviceMap[name]
+	}
+	return result
+}
+
+// LinkLayerDevicesForSpaces takes a list of spaces, and returns the devices on
+// this machine that are in that space that we feel would be useful for
+// containers to know about.  (eg, if there is a host device that has been
+// bridged, we return the bridge, rather than the underlying device, but if we
+// have only the host device, we return that.)
+// Note that devices like 'lxdbr0' that are bridges that might might not be
+// externally accessible may be returned if "" is listed as one of the desired
+// spaces.
+func (m *Machine) LinkLayerDevicesForSpaces(spaces []string) (map[string][]*LinkLayerDevice, error) {
+	addresses, err := m.AllAddresses()
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	devices, err := m.AllLinkLayerDevices()
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	deviceByName := make(map[string]*LinkLayerDevice, len(devices))
+	for _, dev := range devices {
+		deviceByName[dev.Name()] = dev
+	}
+	requestedSpaces := set.NewStrings(spaces...)
+	spaceToDevices := make(map[string]map[string]*LinkLayerDevice, 0)
+	processedDeviceNames := set.NewStrings()
+	includeDevice := func(spaceName string, device *LinkLayerDevice) {
+		spaceInfo, ok := spaceToDevices[spaceName]
+		if !ok {
+			spaceInfo = make(map[string]*LinkLayerDevice)
+			spaceToDevices[spaceName] = spaceInfo
+		}
+		spaceInfo[device.Name()] = device
+	}
+	// First pass, iterate the addresses, lookup the associated spaces, and
+	// gather the devices.
+	for _, addr := range addresses {
+		subnet, err := addr.Subnet()
+		spaceName := ""
+		if err != nil {
+			if errors.IsNotFound(err) {
+				// unknown subnets are considered part of the "unknown" space
+				spaceName = ""
+			} else {
+				// We don't understand the error, so error out for now
+				return nil, errors.Trace(err)
+			}
+		} else {
+			spaceName = subnet.SpaceName()
+		}
+		device, ok := deviceByName[addr.DeviceName()]
+		if !ok {
+			return nil, errors.Errorf("address %v for machine %q refers to a missing device %q",
+				addr, m.Id(), addr.DeviceName())
+		}
+		processedDeviceNames.Add(device.Name())
+		if device.Type() == LoopbackDevice {
+			// We skip loopback devices here
+			continue
+		}
+		includeDevice(spaceName, device)
+	}
+	// Now grab any devices we may have missed. For now, any device without an
+	// address must be in the "unknown" space.
+	for devName, device := range deviceByName {
+		if processedDeviceNames.Contains(devName) {
+			continue
+		}
+		// Loopback devices aren't considered part of the empty space
+		// Also, devices that are attached to another device also aren't
+		// considered to be in the unknown space.
+		if device.Type() == LoopbackDevice || device.ParentName() != "" {
+			continue
+		}
+		includeDevice("", device)
+	}
+	result := make(map[string][]*LinkLayerDevice, len(spaceToDevices))
+	for spaceName, deviceMap := range spaceToDevices {
+		if !requestedSpaces.Contains(spaceName) {
+			continue
+		}
+		result[spaceName] = deviceMapToSortedList(deviceMap)
+	}
+	return result, nil
+}
+
 // SetParentLinkLayerDevicesBeforeTheirChildren splits the given devicesArgs
 // into multiple sets of args and calls SetLinkLayerDevices() for each set, such
 // that child devices are set only after their parents.
@@ -925,59 +1088,19 @@ func (m *Machine) SetDevicesAddressesIdempotently(devicesAddresses []LinkLayerDe
 	return nil
 }
 
-// SetContainerLinkLayerDevices sets the link-layer devices of the given
-// containerMachine, setting each device linked to the corresponding
-// BridgeDevice of the host machine m.
-func (m *Machine) SetContainerLinkLayerDevices(containerMachine *Machine) error {
-	allDevices, err := m.AllLinkLayerDevices()
-	if err != nil {
-		return errors.Annotate(err, "cannot get host machine devices")
+func DefineEthernetDeviceOnBridge(name string, hostBridge *LinkLayerDevice) (LinkLayerDeviceArgs, error) {
+	if hostBridge.Type() != BridgeDevice {
+		return LinkLayerDeviceArgs{}, errors.Errorf("hostBridge must be a Bridge Device not %q", hostBridge.Type())
 	}
-
-	bridgeDevicesByName := make(map[string]*LinkLayerDevice)
-	bridgeDeviceNames := make([]string, 0, len(allDevices))
-
-	for _, hostDevice := range allDevices {
-		deviceType, name := hostDevice.Type(), hostDevice.Name()
-		// Since the default bridges (for each container type) are
-		// machine-local, and there's neither a way (at least not yet) nor any
-		// point in allocating addresses from the (machine-local) subnets
-		// configured on those bridges, we need to ignore them below.
-		if deviceType == BridgeDevice {
-			switch name {
-			case container.DefaultLxdBridge, container.DefaultKvmBridge:
-				logger.Debugf("skipping host bridge %q", name)
-				continue
-			}
-			bridgeDevicesByName[name] = hostDevice
-			bridgeDeviceNames = append(bridgeDeviceNames, name)
-		}
-	}
-
-	sortedBridgeDeviceNames := network.NaturallySortDeviceNames(bridgeDeviceNames...)
-	logger.Debugf("using host machine %q bridge devices: %v", m.Id(), sortedBridgeDeviceNames)
-	containerDevicesArgs := make([]LinkLayerDeviceArgs, len(bridgeDeviceNames))
-
-	for i, hostBridgeName := range sortedBridgeDeviceNames {
-		hostBridge := bridgeDevicesByName[hostBridgeName]
-		containerDevicesArgs[i] = LinkLayerDeviceArgs{
-			Name:        fmt.Sprintf("eth%d", i),
-			Type:        EthernetDevice,
-			MACAddress:  generateMACAddress(),
-			MTU:         hostBridge.MTU(),
-			IsUp:        true,
-			IsAutoStart: true,
-			ParentName:  hostBridge.globalKey(),
-		}
-	}
-	logger.Debugf("prepared container %q network config: %+v", containerMachine.Id(), containerDevicesArgs)
-
-	if err := containerMachine.SetLinkLayerDevices(containerDevicesArgs...); err != nil {
-		return errors.Trace(err)
-	}
-
-	logger.Debugf("container %q network config set", containerMachine.Id())
-	return nil
+	return LinkLayerDeviceArgs{
+		Name:        name,
+		Type:        EthernetDevice,
+		MACAddress:  generateMACAddress(),
+		MTU:         hostBridge.MTU(),
+		IsUp:        true,
+		IsAutoStart: true,
+		ParentName:  hostBridge.globalKey(),
+	}, nil
 }
 
 // MACAddressTemplate is used to generate a unique MAC address for a
@@ -997,3 +1120,125 @@ func generateMACAddress() string {
 	}
 	return fmt.Sprintf(macAddressTemplate, digits...)
 }
+
+// MachineNetworkInfoResult contains an error or a list of NetworkInfo structures for a specific space.
+type MachineNetworkInfoResult struct {
+	NetworkInfos []network.NetworkInfo
+	Error        *error
+}
+
+// Add address to a device in list or create a new device with this address.
+func addAddressToResult(networkInfos []network.NetworkInfo, address *Address) ([]network.NetworkInfo, error) {
+	ifaceAddress := network.InterfaceAddress{
+		Address: address.Value(),
+		CIDR:    address.SubnetCIDR(),
+	}
+	for i := range networkInfos {
+		networkInfo := &networkInfos[i]
+		if networkInfo.InterfaceName == address.DeviceName() {
+			networkInfo.Addresses = append(networkInfo.Addresses, ifaceAddress)
+			return networkInfos, nil
+		}
+	}
+
+	MAC := ""
+	device, err := address.Device()
+	if err == nil {
+		MAC = device.MACAddress()
+	} else if !errors.IsNotFound(err) {
+		return nil, err
+	}
+	networkInfo := network.NetworkInfo{
+		InterfaceName: address.DeviceName(),
+		MACAddress:    MAC,
+		Addresses:     []network.InterfaceAddress{ifaceAddress},
+	}
+	return append(networkInfos, networkInfo), nil
+}
+
+// GetNetworkInfoForSpaces returns MachineNetworkInfoResult with a list of devices for each space in spaces
+// TODO(wpk): 2017-05-04 This does not work for L2-only devices as it iterates over addresses, needs to be fixed.
+// When changing the method we have to keep the ordering.
+func (m *Machine) GetNetworkInfoForSpaces(spaces set.Strings) map[string](MachineNetworkInfoResult) {
+	results := make(map[string](MachineNetworkInfoResult))
+
+	var privateAddress network.Address
+
+	if spaces.Contains("") {
+		var err error
+		privateAddress, err = m.PrivateAddress()
+		if err != nil {
+			error := errors.Annotatef(err, "getting machine %q preferred private address", m.MachineTag())
+			results[""] = MachineNetworkInfoResult{Error: &error}
+			spaces.Remove("")
+		}
+	}
+
+	addresses, err := m.AllAddresses()
+	logger.Debugf("Looking for something from spaces %v in %v", spaces, addresses)
+	if err != nil {
+		newErr := errors.Annotate(err, "cannot get devices addresses")
+		result := MachineNetworkInfoResult{Error: &newErr}
+		for space := range spaces {
+			if _, ok := results[space]; !ok {
+				results[space] = result
+			}
+		}
+		return results
+	}
+	actualSpaces := set.NewStrings()
+	for _, addr := range addresses {
+		subnet, err := addr.Subnet()
+		switch {
+		case errors.IsNotFound(err):
+			logger.Debugf("skipping %s: not linked to a known subnet (%v)", addr, err)
+		case err != nil:
+			logger.Errorf("cannot get subnet for address %q - %q", addr, err)
+		default:
+			space := subnet.SpaceName()
+			actualSpaces.Add(space)
+			if spaces.Contains(space) {
+				r := results[space]
+				r.NetworkInfos, err = addAddressToResult(r.NetworkInfos, addr)
+				if err != nil {
+					r.Error = &err
+				} else {
+					results[space] = r
+				}
+			}
+			if spaces.Contains("") && privateAddress.Value == addr.Value() {
+				r := results[""]
+				r.NetworkInfos, err = addAddressToResult(r.NetworkInfos, addr)
+				if err != nil {
+					r.Error = &err
+				} else {
+					results[""] = r
+				}
+			}
+		}
+	}
+
+	// For a spaceless environment we won't find a subnet that's linked to privateAddress,
+	// we have to work around that and at least return minimal information for --primary-address.
+	if r, filledPrivateAddress := results[""]; !filledPrivateAddress && spaces.Contains("") {
+		r.NetworkInfos = []network.NetworkInfo{{
+			Addresses: []network.InterfaceAddress{{
+				Address: privateAddress.Value,
+			}},
+		}}
+		results[""] = r
+	}
+	actualSpacesStr := network.QuoteSpaceSet(actualSpaces)
+
+	for space := range spaces {
+		if _, ok := results[space]; !ok {
+			newErr := errors.Errorf("machine %q has no devices in space %q, only spaces %s", m.doc.Id, space, actualSpacesStr)
+			results[space] = MachineNetworkInfoResult{
+				Error: &newErr,
+			}
+		}
+		return results
+	}
+
+	return results
+}
