@@ -4,22 +4,51 @@
 package state
 
 import (
+	"fmt"
 	"strings"
 	"time"
 
+	"github.com/juju/description"
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
+	"github.com/juju/utils/featureflag"
 	"github.com/juju/utils/set"
 	"gopkg.in/juju/names.v2"
 	"gopkg.in/mgo.v2/bson"
 
-	"github.com/juju/juju/core/description"
+	"github.com/juju/juju/feature"
 	"github.com/juju/juju/payload"
+	"github.com/juju/juju/resource"
 	"github.com/juju/juju/storage/poolmanager"
 )
 
+// ExportConfig allows certain aspects of the model to be skipped
+// during the export. The intent of this is to be able to get a partial
+// export to support other API calls, like status.
+type ExportConfig struct {
+	SkipActions            bool
+	SkipAnnotations        bool
+	SkipCloudImageMetadata bool
+	SkipCredentials        bool
+	SkipIPAddresses        bool
+	SkipSettings           bool
+	SkipSSHHostKeys        bool
+	SkipStatusHistory      bool
+	SkipLinkLayerDevices   bool
+}
+
+// ExportPartial the current model for the State optionally skipping
+// aspects as defined by the ExportConfig.
+func (st *State) ExportPartial(cfg ExportConfig) (description.Model, error) {
+	return st.exportImpl(cfg)
+}
+
 // Export the current model for the State.
 func (st *State) Export() (description.Model, error) {
+	return st.exportImpl(ExportConfig{})
+}
+
+func (st *State) exportImpl(cfg ExportConfig) (description.Model, error) {
 	dbModel, err := st.Model()
 	if err != nil {
 		return nil, errors.Trace(err)
@@ -27,6 +56,7 @@ func (st *State) Export() (description.Model, error) {
 
 	export := exporter{
 		st:      st,
+		cfg:     cfg,
 		dbModel: dbModel,
 		logger:  loggo.GetLogger("juju.state.export-model"),
 	}
@@ -50,9 +80,10 @@ func (st *State) Export() (description.Model, error) {
 	}
 
 	modelConfig, found := export.modelSettings[modelGlobalKey]
-	if !found {
+	if !found && !cfg.SkipSettings {
 		return nil, errors.New("missing model config")
 	}
+	delete(export.modelSettings, modelGlobalKey)
 
 	blocks, err := export.readBlocks()
 	if err != nil {
@@ -65,12 +96,23 @@ func (st *State) Export() (description.Model, error) {
 		Owner:              dbModel.Owner(),
 		Config:             modelConfig.Settings,
 		LatestToolsVersion: dbModel.LatestToolsVersion(),
+		EnvironVersion:     dbModel.EnvironVersion(),
 		Blocks:             blocks,
 	}
-	if creds, credsSet := dbModel.CloudCredential(); credsSet {
-		args.CloudCredential = creds.Id()
-	}
 	export.model = description.NewModel(args)
+	if credsTag, credsSet := dbModel.CloudCredential(); credsSet && !cfg.SkipCredentials {
+		creds, err := st.CloudCredential(credsTag)
+		if err != nil {
+			return nil, errors.Trace(err)
+		}
+		export.model.SetCloudCredential(description.CloudCredentialArgs{
+			Owner:      credsTag.Owner(),
+			Cloud:      credsTag.Cloud(),
+			Name:       credsTag.Name(),
+			AuthType:   string(creds.AuthType()),
+			Attributes: creds.Attributes(),
+		})
+	}
 	modelKey := dbModel.globalKey()
 	export.model.SetAnnotations(export.getAnnotations(modelKey))
 	if err := export.sequences(); err != nil {
@@ -81,7 +123,9 @@ func (st *State) Export() (description.Model, error) {
 		return nil, errors.Trace(err)
 	}
 	export.model.SetConstraints(constraintsArgs)
-
+	if err := export.modelStatus(); err != nil {
+		return nil, errors.Trace(err)
+	}
 	if err := export.modelUsers(); err != nil {
 		return nil, errors.Trace(err)
 	}
@@ -124,16 +168,29 @@ func (st *State) Export() (description.Model, error) {
 		return nil, errors.Trace(err)
 	}
 
-	if err := export.model.Validate(); err != nil {
-		return nil, errors.Trace(err)
+	// If we are doing a partial export, it doesn't really make sense
+	// to validate the model.
+	fullExport := ExportConfig{}
+	if cfg == fullExport {
+		if err := export.model.Validate(); err != nil {
+			return nil, errors.Trace(err)
+		}
 	}
 
-	export.logExtras()
+	export.model.SetSLA(dbModel.SLALevel(), dbModel.SLAOwner(), string(dbModel.SLACredential()))
+	export.model.SetMeterStatus(dbModel.MeterStatus().Code.String(), dbModel.MeterStatus().Info)
+
+	if featureflag.Enabled(feature.StrictMigration) {
+		if err := export.checkUnexportedValues(); err != nil {
+			return nil, errors.Trace(err)
+		}
+	}
 
 	return export.model, nil
 }
 
 type exporter struct {
+	cfg     ExportConfig
 	st      *State
 	dbModel *Model
 	model   description.Model
@@ -151,7 +208,7 @@ type exporter struct {
 }
 
 func (e *exporter) sequences() error {
-	sequences, closer := e.st.getCollection(sequenceC)
+	sequences, closer := e.st.db().GetCollection(sequenceC)
 	defer closer()
 
 	var docs []sequenceDoc
@@ -166,7 +223,7 @@ func (e *exporter) sequences() error {
 }
 
 func (e *exporter) readBlocks() (map[string]string, error) {
-	blocks, closer := e.st.getCollection(blocksC)
+	blocks, closer := e.st.db().GetCollection(blocksC)
 	defer closer()
 
 	var docs []blockDoc
@@ -184,6 +241,17 @@ func (e *exporter) readBlocks() (map[string]string, error) {
 	return result, nil
 }
 
+func (e *exporter) modelStatus() error {
+	statusArgs, err := e.statusArgs(modelGlobalKey)
+	if err != nil {
+		return errors.Annotatef(err, "status for model")
+	}
+
+	e.model.SetStatus(statusArgs)
+	e.model.SetStatusHistory(e.statusHistoryArgs(modelGlobalKey))
+	return nil
+}
+
 func (e *exporter) modelUsers() error {
 	users, err := e.dbModel.Users()
 	if err != nil {
@@ -225,7 +293,7 @@ func (e *exporter) machines() error {
 	}
 
 	// Read all the open ports documents.
-	openedPorts, closer := e.st.getCollection(openedPortsC)
+	openedPorts, closer := e.st.db().GetCollection(openedPortsC)
 	defer closer()
 	var portsData []portsDoc
 	if err := openedPorts.Find(nil).All(&portsData); err != nil {
@@ -262,7 +330,7 @@ func (e *exporter) machines() error {
 }
 
 func (e *exporter) loadMachineInstanceData() (map[string]instanceData, error) {
-	instanceDataCollection, closer := e.st.getCollection(instanceDataC)
+	instanceDataCollection, closer := e.st.db().GetCollection(instanceDataC)
 	defer closer()
 
 	var instData []instanceData
@@ -278,7 +346,7 @@ func (e *exporter) loadMachineInstanceData() (map[string]instanceData, error) {
 }
 
 func (e *exporter) loadMachineBlockDevices() (map[string][]BlockDeviceInfo, error) {
-	coll, closer := e.st.getCollection(blockDevicesC)
+	coll, closer := e.st.db().GetCollection(blockDevicesC)
 	defer closer()
 
 	var deviceData []blockDevicesDoc
@@ -337,6 +405,14 @@ func (e *exporter) newMachine(exParent description.Machine, machine *Machine, in
 		return nil, errors.NotValidf("missing instance data for machine %s", machine.Id())
 	}
 	exMachine.SetInstance(e.newCloudInstanceArgs(instData))
+	instance := exMachine.Instance()
+	instanceKey := machine.globalInstanceKey()
+	statusArgs, err := e.statusArgs(instanceKey)
+	if err != nil {
+		return nil, errors.Annotatef(err, "status for machine instance %s", machine.Id())
+	}
+	instance.SetStatus(statusArgs)
+	instance.SetStatusHistory(e.statusHistoryArgs(instanceKey))
 
 	// We don't rely on devices being there. If they aren't, we get an empty slice,
 	// which is fine to iterate over with range.
@@ -347,6 +423,7 @@ func (e *exporter) newMachine(exParent description.Machine, machine *Machine, in
 			Label:          device.Label,
 			UUID:           device.UUID,
 			HardwareID:     device.HardwareId,
+			WWN:            device.WWN,
 			BusAddress:     device.BusAddress,
 			Size:           device.Size,
 			FilesystemType: device.FilesystemType,
@@ -357,7 +434,7 @@ func (e *exporter) newMachine(exParent description.Machine, machine *Machine, in
 
 	// Find the current machine status.
 	globalKey := machine.globalKey()
-	statusArgs, err := e.statusArgs(globalKey)
+	statusArgs, err = e.statusArgs(globalKey)
 	if err != nil {
 		return nil, errors.Annotatef(err, "status for machine %s", machine.Id())
 	}
@@ -432,7 +509,6 @@ func (e *exporter) newAddressArgs(a address) description.AddressArgs {
 func (e *exporter) newCloudInstanceArgs(data instanceData) description.CloudInstanceArgs {
 	inst := description.CloudInstanceArgs{
 		InstanceId: string(data.InstanceId),
-		Status:     data.Status,
 	}
 	if data.Arch != nil {
 		inst.Architecture = *data.Arch
@@ -490,15 +566,25 @@ func (e *exporter) applications() error {
 		return errors.Trace(err)
 	}
 
+	resourcesSt, err := e.st.Resources()
+	if err != nil {
+		return errors.Trace(err)
+	}
+
 	for _, application := range applications {
 		applicationUnits := e.units[application.Name()]
 		leader := leaders[application.Name()]
+		resources, err := resourcesSt.ListResources(application.Name())
+		if err != nil {
+			return errors.Trace(err)
+		}
 		if err := e.addApplication(addApplicationContext{
 			application:      application,
 			units:            applicationUnits,
 			meterStatus:      meterStatus,
 			leader:           leader,
 			payloads:         payloads,
+			resources:        resources,
 			endpoingBindings: bindings,
 		}); err != nil {
 			return errors.Trace(err)
@@ -508,7 +594,7 @@ func (e *exporter) applications() error {
 }
 
 func (e *exporter) readAllStorageConstraints() error {
-	coll, closer := e.st.getCollection(storageConstraintsC)
+	coll, closer := e.st.db().GetCollection(storageConstraintsC)
 	defer closer()
 
 	storageConstraints := make(map[string]storageConstraintsDoc)
@@ -556,6 +642,7 @@ type addApplicationContext struct {
 	meterStatus      map[string]*meterStatusDoc
 	leader           string
 	payloads         map[string][]payload.FullPayloadInfo
+	resources        resource.ServiceResources
 	endpoingBindings map[string]bindingsMap
 }
 
@@ -568,13 +655,15 @@ func (e *exporter) addApplication(ctx addApplicationContext) error {
 	storageConstraintsKey := application.storageConstraintsKey()
 
 	applicationSettingsDoc, found := e.modelSettings[settingsKey]
-	if !found {
+	if !found && !e.cfg.SkipSettings {
 		return errors.Errorf("missing settings for application %q", appName)
 	}
+	delete(e.modelSettings, settingsKey)
 	leadershipSettingsDoc, found := e.modelSettings[leadershipKey]
-	if !found {
+	if !found && !e.cfg.SkipSettings {
 		return errors.Errorf("missing leadership settings for application %q", appName)
 	}
+	delete(e.modelSettings, leadershipKey)
 
 	args := description.ApplicationArgs{
 		Tag:                  application.ApplicationTag(),
@@ -611,6 +700,10 @@ func (e *exporter) addApplication(ctx addApplicationContext) error {
 	}
 	exApplication.SetConstraints(constraintsArgs)
 
+	if err := e.setResources(exApplication, ctx.resources); err != nil {
+		return errors.Trace(err)
+	}
+
 	for _, unit := range ctx.units {
 		agentKey := unit.globalAgentKey()
 		unitMeterStatus, found := ctx.meterStatus[agentKey]
@@ -618,7 +711,7 @@ func (e *exporter) addApplication(ctx addApplicationContext) error {
 			return errors.Errorf("missing meter status for unit %s", unit.Name())
 		}
 
-		workloadVersion, err := unit.WorkloadVersion()
+		workloadVersion, err := e.unitWorkloadVersion(unit)
 		if err != nil {
 			return errors.Trace(err)
 		}
@@ -640,6 +733,8 @@ func (e *exporter) addApplication(ctx addApplicationContext) error {
 		}
 		exUnit := exApplication.AddUnit(args)
 
+		e.setUnitResources(exUnit, ctx.resources.UnitResources)
+
 		if err := e.setUnitPayloads(exUnit, ctx.payloads[unit.UnitTag().Id()]); err != nil {
 			return errors.Trace(err)
 		}
@@ -687,6 +782,80 @@ func (e *exporter) addApplication(ctx addApplicationContext) error {
 	return nil
 }
 
+func (e *exporter) unitWorkloadVersion(unit *Unit) (string, error) {
+	// Rather than call unit.WorkloadVersion(), which does a database
+	// query, we go directly to the status value that is stored.
+	key := unit.globalWorkloadVersionKey()
+	info, err := e.statusArgs(key)
+	if err != nil {
+		return "", errors.Trace(err)
+	}
+	return info.Message, nil
+}
+
+func (e *exporter) setResources(exApp description.Application, resources resource.ServiceResources) error {
+	if len(resources.Resources) != len(resources.CharmStoreResources) {
+		return errors.New("number of resources don't match charm store resources")
+	}
+
+	for i, resource := range resources.Resources {
+		exResource := exApp.AddResource(description.ResourceArgs{
+			Name: resource.Name,
+		})
+		exResource.SetApplicationRevision(description.ResourceRevisionArgs{
+			Revision:       resource.Revision,
+			Type:           resource.Type.String(),
+			Path:           resource.Path,
+			Description:    resource.Description,
+			Origin:         resource.Origin.String(),
+			FingerprintHex: resource.Fingerprint.Hex(),
+			Size:           resource.Size,
+			Timestamp:      resource.Timestamp,
+			Username:       resource.Username,
+		})
+		csResource := resources.CharmStoreResources[i]
+		exResource.SetCharmStoreRevision(description.ResourceRevisionArgs{
+			Revision:       csResource.Revision,
+			Type:           csResource.Type.String(),
+			Path:           csResource.Path,
+			Description:    csResource.Description,
+			Origin:         csResource.Origin.String(),
+			Size:           csResource.Size,
+			FingerprintHex: csResource.Fingerprint.Hex(),
+		})
+	}
+
+	return nil
+}
+
+func (e *exporter) setUnitResources(exUnit description.Unit, allResources []resource.UnitResources) {
+	for _, resource := range findUnitResources(exUnit.Name(), allResources) {
+		exUnit.AddResource(description.UnitResourceArgs{
+			Name: resource.Name,
+			RevisionArgs: description.ResourceRevisionArgs{
+				Revision:       resource.Revision,
+				Type:           resource.Type.String(),
+				Path:           resource.Path,
+				Description:    resource.Description,
+				Origin:         resource.Origin.String(),
+				FingerprintHex: resource.Fingerprint.Hex(),
+				Size:           resource.Size,
+				Timestamp:      resource.Timestamp,
+				Username:       resource.Username,
+			},
+		})
+	}
+}
+
+func findUnitResources(unitName string, allResources []resource.UnitResources) []resource.Resource {
+	for _, unitResources := range allResources {
+		if unitResources.Tag.Id() == unitName {
+			return unitResources.Resources
+		}
+	}
+	return nil
+}
+
 func (e *exporter) setUnitPayloads(exUnit description.Unit, payloads []payload.FullPayloadInfo) error {
 	unitID := exUnit.Tag().Id()
 	machineID := exUnit.Machine().Id()
@@ -741,14 +910,26 @@ func (e *exporter) relations() error {
 				if err != nil {
 					return errors.Trace(err)
 				}
+				valid, err := ru.Valid()
+				if err != nil {
+					return errors.Trace(err)
+				}
+				if !valid {
+					// It doesn't make sense for this application to have a
+					// relations scope for this endpoint. For example the
+					// situation where we have a subordinate charm related to
+					// two different principals.
+					continue
+				}
 				key := ru.key()
 				if !relationScopes.Contains(key) {
 					return errors.Errorf("missing relation scope for %s and %s", relation, unit.Name())
 				}
 				settingsDoc, found := e.modelSettings[key]
-				if !found {
+				if !found && !e.cfg.SkipSettings {
 					return errors.Errorf("missing relation settings for %s and %s", relation, unit.Name())
 				}
+				delete(e.modelSettings, key)
 				exEndPoint.SetUnitSettings(unit.Name(), settingsDoc.Settings)
 			}
 		}
@@ -774,6 +955,9 @@ func (e *exporter) spaces() error {
 }
 
 func (e *exporter) linklayerdevices() error {
+	if e.cfg.SkipLinkLayerDevices {
+		return nil
+	}
 	linklayerdevices, err := e.st.AllLinkLayerDevices()
 	if err != nil {
 		return errors.Trace(err)
@@ -803,18 +987,28 @@ func (e *exporter) subnets() error {
 	e.logger.Debugf("read %d subnets", len(subnets))
 
 	for _, subnet := range subnets {
-		e.model.AddSubnet(description.SubnetArgs{
-			CIDR:             subnet.CIDR(),
-			ProviderId:       string(subnet.ProviderId()),
-			VLANTag:          subnet.VLANTag(),
-			AvailabilityZone: subnet.AvailabilityZone(),
-			SpaceName:        subnet.SpaceName(),
-		})
+		args := description.SubnetArgs{
+			CIDR:              subnet.CIDR(),
+			ProviderId:        string(subnet.ProviderId()),
+			ProviderNetworkId: string(subnet.ProviderNetworkId()),
+			VLANTag:           subnet.VLANTag(),
+			SpaceName:         subnet.SpaceName(),
+		}
+		// TODO(babbageclunk): at the moment state.Subnet only stores
+		// one AZ.
+		az := subnet.AvailabilityZone()
+		if az != "" {
+			args.AvailabilityZones = []string{az}
+		}
+		e.model.AddSubnet(args)
 	}
 	return nil
 }
 
 func (e *exporter) ipaddresses() error {
+	if e.cfg.SkipIPAddresses {
+		return nil
+	}
 	ipaddresses, err := e.st.AllIPAddresses()
 	if err != nil {
 		return errors.Trace(err)
@@ -837,6 +1031,9 @@ func (e *exporter) ipaddresses() error {
 }
 
 func (e *exporter) sshHostKeys() error {
+	if e.cfg.SkipSSHHostKeys {
+		return nil
+	}
 	machines, err := e.st.AllMachines()
 	if err != nil {
 		return errors.Trace(err)
@@ -860,6 +1057,9 @@ func (e *exporter) sshHostKeys() error {
 }
 
 func (e *exporter) cloudimagemetadata() error {
+	if e.cfg.SkipCloudImageMetadata {
+		return nil
+	}
 	cloudimagemetadata, err := e.st.CloudImageMetadataStorage.AllCloudImageMetadata()
 	if err != nil {
 		return errors.Trace(err)
@@ -885,6 +1085,9 @@ func (e *exporter) cloudimagemetadata() error {
 }
 
 func (e *exporter) actions() error {
+	if e.cfg.SkipActions {
+		return nil
+	}
 	actions, err := e.st.AllActions()
 	if err != nil {
 		return errors.Trace(err)
@@ -909,7 +1112,7 @@ func (e *exporter) actions() error {
 }
 
 func (e *exporter) readAllRelationScopes() (set.Strings, error) {
-	relationScopes, closer := e.st.getCollection(relationScopesC)
+	relationScopes, closer := e.st.db().GetCollection(relationScopesC)
 	defer closer()
 
 	docs := []relationScopeDoc{}
@@ -927,11 +1130,11 @@ func (e *exporter) readAllRelationScopes() (set.Strings, error) {
 }
 
 func (e *exporter) readAllUnits() (map[string][]*Unit, error) {
-	unitsCollection, closer := e.st.getCollection(unitsC)
+	unitsCollection, closer := e.st.db().GetCollection(unitsC)
 	defer closer()
 
 	docs := []unitDoc{}
-	err := unitsCollection.Find(nil).All(&docs)
+	err := unitsCollection.Find(nil).Sort("name").All(&docs)
 	if err != nil {
 		return nil, errors.Annotate(err, "cannot get all units")
 	}
@@ -945,7 +1148,7 @@ func (e *exporter) readAllUnits() (map[string][]*Unit, error) {
 }
 
 func (e *exporter) readAllEndpointBindings() (map[string]bindingsMap, error) {
-	bindings, closer := e.st.getCollection(endpointBindingsC)
+	bindings, closer := e.st.db().GetCollection(endpointBindingsC)
 	defer closer()
 
 	docs := []endpointBindingsDoc{}
@@ -962,7 +1165,7 @@ func (e *exporter) readAllEndpointBindings() (map[string]bindingsMap, error) {
 }
 
 func (e *exporter) readAllMeterStatus() (map[string]*meterStatusDoc, error) {
-	meterStatuses, closer := e.st.getCollection(meterStatusC)
+	meterStatuses, closer := e.st.db().GetCollection(meterStatusC)
 	defer closer()
 
 	docs := []meterStatusDoc{}
@@ -979,7 +1182,7 @@ func (e *exporter) readAllMeterStatus() (map[string]*meterStatusDoc, error) {
 }
 
 func (e *exporter) readLastConnectionTimes() (map[string]time.Time, error) {
-	lastConnections, closer := e.st.getCollection(modelUserLastConnectionC)
+	lastConnections, closer := e.st.db().GetCollection(modelUserLastConnectionC)
 	defer closer()
 
 	var docs []modelUserLastConnectionDoc
@@ -995,7 +1198,12 @@ func (e *exporter) readLastConnectionTimes() (map[string]time.Time, error) {
 }
 
 func (e *exporter) readAllAnnotations() error {
-	annotations, closer := e.st.getCollection(annotationsC)
+	e.annotations = make(map[string]annotatorDoc)
+	if e.cfg.SkipAnnotations {
+		return nil
+	}
+
+	annotations, closer := e.st.db().GetCollection(annotationsC)
 	defer closer()
 
 	var docs []annotatorDoc
@@ -1004,7 +1212,6 @@ func (e *exporter) readAllAnnotations() error {
 	}
 	e.logger.Debugf("read %d annotations docs", len(docs))
 
-	e.annotations = make(map[string]annotatorDoc)
 	for _, doc := range docs {
 		e.annotations[doc.GlobalKey] = doc
 	}
@@ -1012,7 +1219,7 @@ func (e *exporter) readAllAnnotations() error {
 }
 
 func (e *exporter) readAllConstraints() error {
-	constraintsCollection, closer := e.st.getCollection(constraintsC)
+	constraintsCollection, closer := e.st.db().GetCollection(constraintsC)
 	defer closer()
 
 	// Since the constraintsDoc doesn't include any global key or _id
@@ -1050,7 +1257,12 @@ func (e *exporter) getAnnotations(key string) map[string]string {
 }
 
 func (e *exporter) readAllSettings() error {
-	settings, closer := e.st.getCollection(settingsC)
+	e.modelSettings = make(map[string]settingsDoc)
+	if e.cfg.SkipSettings {
+		return nil
+	}
+
+	settings, closer := e.st.db().GetCollection(settingsC)
 	defer closer()
 
 	var docs []settingsDoc
@@ -1058,7 +1270,6 @@ func (e *exporter) readAllSettings() error {
 		return errors.Trace(err)
 	}
 
-	e.modelSettings = make(map[string]settingsDoc)
 	for _, doc := range docs {
 		key := e.st.localID(doc.DocID)
 		e.modelSettings[key] = doc
@@ -1067,7 +1278,7 @@ func (e *exporter) readAllSettings() error {
 }
 
 func (e *exporter) readAllStatuses() error {
-	statuses, closer := e.st.getCollection(statusesC)
+	statuses, closer := e.st.db().GetCollection(statusesC)
 	defer closer()
 
 	var docs []bson.M
@@ -1091,11 +1302,14 @@ func (e *exporter) readAllStatuses() error {
 }
 
 func (e *exporter) readAllStatusHistory() error {
-	statuses, closer := e.st.getCollection(statusesHistoryC)
+	statuses, closer := e.st.db().GetCollection(statusesHistoryC)
 	defer closer()
 
 	count := 0
 	e.statusHistory = make(map[string][]historicalStatusDoc)
+	if e.cfg.SkipStatusHistory {
+		return nil
+	}
 	var doc historicalStatusDoc
 	// In tests, sorting by time can leave the results
 	// underconstrained - include document id for deterministic
@@ -1123,6 +1337,7 @@ func (e *exporter) statusArgs(globalKey string) (description.StatusArgs, error)
 	if !found {
 		return result, errors.NotFoundf("status data for %s", globalKey)
 	}
+	delete(e.status, globalKey)
 
 	status, ok := statusDoc["status"].(string)
 	if !ok {
@@ -1154,7 +1369,7 @@ func (e *exporter) statusArgs(globalKey string) (description.StatusArgs, error)
 func (e *exporter) statusHistoryArgs(globalKey string) []description.StatusArgs {
 	history := e.statusHistory[globalKey]
 	result := make([]description.StatusArgs, len(history))
-	e.logger.Debugf("found %d status history docs for %s", len(history), globalKey)
+	e.logger.Tracef("found %d status history docs for %s", len(history), globalKey)
 	for i, doc := range history {
 		result[i] = description.StatusArgs{
 			Value:   string(doc.Status),
@@ -1163,7 +1378,7 @@ func (e *exporter) statusHistoryArgs(globalKey string) []description.StatusArgs
 			Updated: time.Unix(0, doc.Updated),
 		}
 	}
-
+	delete(e.statusHistory, globalKey)
 	return result
 }
 
@@ -1171,7 +1386,7 @@ func (e *exporter) constraintsArgs(globalKey string) (description.ConstraintsArg
 	doc, found := e.constraints[globalKey]
 	if !found {
 		// No constraints for this key.
-		e.logger.Debugf("no constraints found for key %q", globalKey)
+		e.logger.Tracef("no constraints found for key %q", globalKey)
 		return description.ConstraintsArgs{}, nil
 	}
 	// We capture any type error using a closure to avoid having to return
@@ -1184,7 +1399,7 @@ func (e *exporter) constraintsArgs(globalKey string) (description.ConstraintsArg
 		case string:
 			return value
 		default:
-			optionalErr = errors.Errorf("expected uint64 for %s, got %T", name, value)
+			optionalErr = errors.Errorf("expected string for %s, got %T", name, value)
 		}
 		return ""
 	}
@@ -1205,8 +1420,19 @@ func (e *exporter) constraintsArgs(globalKey string) (description.ConstraintsArg
 		case nil:
 		case []string:
 			return value
+		case []interface{}:
+			var result []string
+			for _, val := range value {
+				sval, ok := val.(string)
+				if !ok {
+					optionalErr = errors.Errorf("expected string slice for %s, got %T value", name, val)
+					return nil
+				}
+				result = append(result, sval)
+			}
+			return result
 		default:
-			optionalErr = errors.Errorf("expected []string] for %s, got %T", name, value)
+			optionalErr = errors.Errorf("expected []string for %s, got %T", name, value)
 		}
 		return nil
 	}
@@ -1228,14 +1454,51 @@ func (e *exporter) constraintsArgs(globalKey string) (description.ConstraintsArg
 	return result, nil
 }
 
-func (e *exporter) logExtras() {
+func (e *exporter) checkUnexportedValues() error {
+	var missing []string
+
 	// As annotations are saved into the model, they are removed from the
 	// exporter's map. If there are any left at the end, we are missing
-	// things. Not an error just now, just a warning that we have missed
-	// something. Could potentially be an error at a later date when
-	// migrations are complete (but probably not).
+	// things.
 	for key, doc := range e.annotations {
-		e.logger.Warningf("unexported annotation for %s, %s", doc.Tag, key)
+		missing = append(missing, fmt.Sprintf("unexported annotations for %s, %s", doc.Tag, key))
+	}
+
+	for key := range e.modelSettings {
+		missing = append(missing, fmt.Sprintf("unexported settings for %s", key))
+	}
+
+	for key := range e.status {
+		missing = append(missing, fmt.Sprintf("unexported status for %s", key))
+	}
+
+	for key := range e.statusHistory {
+		missing = append(missing, fmt.Sprintf("unexported status history for %s", key))
+	}
+
+	if len(missing) > 0 {
+		content := strings.Join(missing, "\n  ")
+		return errors.Errorf("migration missed some docs:\n  %s", content)
+	}
+	return nil
+}
+
+func (e *exporter) addRemoteSpace(descApp description.RemoteApplication, space RemoteSpace) {
+	descSpace := descApp.AddSpace(description.RemoteSpaceArgs{
+		CloudType:          space.CloudType,
+		Name:               space.Name,
+		ProviderId:         space.ProviderId,
+		ProviderAttributes: space.ProviderAttributes,
+	})
+	for _, subnet := range space.Subnets {
+		descSpace.AddSubnet(description.SubnetArgs{
+			CIDR:              subnet.CIDR,
+			ProviderId:        subnet.ProviderId,
+			VLANTag:           subnet.VLANTag,
+			AvailabilityZones: subnet.AvailabilityZones,
+			ProviderSpaceId:   subnet.ProviderSpaceId,
+			ProviderNetworkId: subnet.ProviderNetworkId,
+		})
 	}
 }
 
@@ -1256,7 +1519,7 @@ func (e *exporter) storage() error {
 }
 
 func (e *exporter) volumes() error {
-	coll, closer := e.st.getCollection(volumesC)
+	coll, closer := e.st.db().GetCollection(volumesC)
 	defer closer()
 
 	attachments, err := e.readVolumeAttachments()
@@ -1281,8 +1544,7 @@ func (e *exporter) volumes() error {
 
 func (e *exporter) addVolume(vol *volume, volAttachments []volumeAttachmentDoc) error {
 	args := description.VolumeArgs{
-		Tag:     vol.VolumeTag(),
-		Binding: vol.LifeBinding(),
+		Tag: vol.VolumeTag(),
 	}
 	if tag, err := vol.StorageInstance(); err == nil {
 		// only returns an error when no storage tag.
@@ -1300,6 +1562,7 @@ func (e *exporter) addVolume(vol *volume, volAttachments []volumeAttachmentDoc)
 		args.Size = info.Size
 		args.Pool = info.Pool
 		args.HardwareID = info.HardwareId
+		args.WWN = info.WWN
 		args.VolumeID = info.VolumeId
 		args.Persistent = info.Persistent
 	} else {
@@ -1346,7 +1609,7 @@ func (e *exporter) addVolume(vol *volume, volAttachments []volumeAttachmentDoc)
 }
 
 func (e *exporter) readVolumeAttachments() (map[string][]volumeAttachmentDoc, error) {
-	coll, closer := e.st.getCollection(volumeAttachmentsC)
+	coll, closer := e.st.db().GetCollection(volumeAttachmentsC)
 	defer closer()
 
 	result := make(map[string][]volumeAttachmentDoc)
@@ -1366,7 +1629,7 @@ func (e *exporter) readVolumeAttachments() (map[string][]volumeAttachmentDoc, er
 }
 
 func (e *exporter) filesystems() error {
-	coll, closer := e.st.getCollection(filesystemsC)
+	coll, closer := e.st.db().GetCollection(filesystemsC)
 	defer closer()
 
 	attachments, err := e.readFilesystemAttachments()
@@ -1398,7 +1661,6 @@ func (e *exporter) addFilesystem(fs *filesystem, fsAttachments []filesystemAttac
 		Tag:     fs.FilesystemTag(),
 		Storage: storage,
 		Volume:  volume,
-		Binding: fs.LifeBinding(),
 	}
 	logger.Debugf("addFilesystem: %#v", fs.doc)
 	if info, err := fs.Info(); err == nil {
@@ -1450,7 +1712,7 @@ func (e *exporter) addFilesystem(fs *filesystem, fsAttachments []filesystemAttac
 }
 
 func (e *exporter) readFilesystemAttachments() (map[string][]filesystemAttachmentDoc, error) {
-	coll, closer := e.st.getCollection(filesystemAttachmentsC)
+	coll, closer := e.st.db().GetCollection(filesystemAttachmentsC)
 	defer closer()
 
 	result := make(map[string][]filesystemAttachmentDoc)
@@ -1470,7 +1732,7 @@ func (e *exporter) readFilesystemAttachments() (map[string][]filesystemAttachmen
 }
 
 func (e *exporter) storageInstances() error {
-	coll, closer := e.st.getCollection(storageInstancesC)
+	coll, closer := e.st.db().GetCollection(storageInstancesC)
 	defer closer()
 
 	attachments, err := e.readStorageAttachments()
@@ -1494,19 +1756,25 @@ func (e *exporter) storageInstances() error {
 }
 
 func (e *exporter) addStorage(instance *storageInstance, attachments []names.UnitTag) error {
+	owner, ok := instance.Owner()
+	if !ok {
+		owner = nil
+	}
+	cons := description.StorageInstanceConstraints(instance.doc.Constraints)
 	args := description.StorageArgs{
 		Tag:         instance.StorageTag(),
 		Kind:        instance.Kind().String(),
-		Owner:       instance.Owner(),
+		Owner:       owner,
 		Name:        instance.StorageName(),
 		Attachments: attachments,
+		Constraints: &cons,
 	}
 	e.model.AddStorage(args)
 	return nil
 }
 
 func (e *exporter) readStorageAttachments() (map[string][]names.UnitTag, error) {
-	coll, closer := e.st.getCollection(storageAttachmentsC)
+	coll, closer := e.st.db().GetCollection(storageAttachmentsC)
 	defer closer()
 
 	result := make(map[string][]names.UnitTag)
@@ -1556,6 +1824,7 @@ func (m storagePoolSettingsManager) ListSettings(keyPrefix string) (map[string]m
 	for key, doc := range m.e.modelSettings {
 		if strings.HasPrefix(key, keyPrefix) {
 			result[key] = doc.Settings
+			delete(m.e.modelSettings, key)
 		}
 	}
 	return result, nil
