@@ -4,27 +4,35 @@
 package state_test
 
 import (
+	"bytes"
+	"io/ioutil"
 	"math/rand"
 	"time"
 
+	"github.com/juju/description"
 	jc "github.com/juju/testing/checkers"
+	"github.com/juju/utils/arch"
 	"github.com/juju/version"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/charm.v6-unstable"
+	charmresource "gopkg.in/juju/charm.v6-unstable/resource"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/constraints"
-	"github.com/juju/juju/core/description"
+	"github.com/juju/juju/feature"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/payload"
 	"github.com/juju/juju/permission"
 	"github.com/juju/juju/provider/dummy"
+	"github.com/juju/juju/resource"
+	"github.com/juju/juju/resource/resourcetesting"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/state/cloudimagemetadata"
 	"github.com/juju/juju/status"
 	"github.com/juju/juju/storage/poolmanager"
 	"github.com/juju/juju/storage/provider"
 	"github.com/juju/juju/testing/factory"
+	jujuversion "github.com/juju/juju/version"
 )
 
 // Constraints stores megabytes by default for memory and root disk.
@@ -67,7 +75,7 @@ func (s *MigrationBaseSuite) setRandSequenceValue(c *gc.C, name string) int {
 func (s *MigrationBaseSuite) primeStatusHistory(c *gc.C, entity statusSetter, statusVal status.Status, count int) {
 	primeStatusHistory(c, entity, statusVal, count, func(i int) map[string]interface{} {
 		return map[string]interface{}{"index": count - i}
-	}, 0)
+	}, 0, "")
 }
 
 func (s *MigrationBaseSuite) makeApplicationWithLeader(c *gc.C, applicationname string, count int, leader int) {
@@ -126,6 +134,11 @@ type MigrationExportSuite struct {
 
 var _ = gc.Suite(&MigrationExportSuite{})
 
+func (s *MigrationExportSuite) SetUpTest(c *gc.C) {
+	s.MigrationBaseSuite.SetUpTest(c)
+	s.SetFeatureFlags(feature.StrictMigration)
+}
+
 func (s *MigrationExportSuite) checkStatusHistory(c *gc.C, history []description.Status, statusVal status.Status) {
 	for i, st := range history {
 		c.Logf("status history #%d: %s", i, st.Updated())
@@ -147,6 +160,9 @@ func (s *MigrationExportSuite) TestModelInfo(c *gc.C) {
 	machineSeq := s.setRandSequenceValue(c, "machine")
 	fooSeq := s.setRandSequenceValue(c, "application-foo")
 	s.State.SwitchBlockOn(state.ChangeBlock, "locked down")
+	environVersion := 123
+	err = stModel.SetEnvironVersion(environVersion)
+	c.Assert(err, jc.ErrorIsNil)
 
 	model, err := s.State.Export()
 	c.Assert(err, jc.ErrorIsNil)
@@ -163,6 +179,7 @@ func (s *MigrationExportSuite) TestModelInfo(c *gc.C) {
 	modelCfg["resource-tags"] = map[string]string{}
 	c.Assert(modelCfg, jc.DeepEquals, modelAttrs)
 	c.Assert(model.LatestToolsVersion(), gc.Equals, latestTools)
+	c.Assert(model.EnvironVersion(), gc.Equals, environVersion)
 	c.Assert(model.Annotations(), jc.DeepEquals, testAnnotations)
 	constraints := model.Constraints()
 	c.Assert(constraints, gc.NotNil)
@@ -223,8 +240,34 @@ func (s *MigrationExportSuite) TestModelUsers(c *gc.C) {
 	c.Assert(exportedBob.Access(), gc.Equals, "read")
 }
 
+func (s *MigrationExportSuite) TestSLAs(c *gc.C) {
+	err := s.State.SetSLA("essential", "bob", []byte("creds"))
+	c.Assert(err, jc.ErrorIsNil)
+
+	model, err := s.State.Export()
+	c.Assert(err, jc.ErrorIsNil)
+
+	sla := model.SLA()
+
+	c.Assert(sla.Level(), gc.Equals, "essential")
+	c.Assert(sla.Credentials(), gc.DeepEquals, "creds")
+}
+
+func (s *MigrationExportSuite) TestMeterStatus(c *gc.C) {
+	err := s.State.SetModelMeterStatus("RED", "red info message")
+	c.Assert(err, jc.ErrorIsNil)
+
+	model, err := s.State.Export()
+	c.Assert(err, jc.ErrorIsNil)
+
+	sla := model.MeterStatus()
+
+	c.Assert(sla.Code(), gc.Equals, "RED")
+	c.Assert(sla.Info(), gc.Equals, "red info message")
+}
+
 func (s *MigrationExportSuite) TestMachines(c *gc.C) {
-	s.assertMachinesMigrated(c, constraints.MustParse("arch=amd64 mem=8G"))
+	s.assertMachinesMigrated(c, constraints.MustParse("arch=amd64 mem=8G tags=foo,bar spaces=dmz"))
 }
 
 func (s *MigrationExportSuite) TestMachinesWithVirtConstraint(c *gc.C) {
@@ -285,6 +328,7 @@ func (s *MigrationExportSuite) TestMachineDevices(c *gc.C) {
 		Label:          "sda-label",
 		UUID:           "some-uuid",
 		HardwareId:     "magic",
+		WWN:            "drbr",
 		BusAddress:     "bus stop",
 		Size:           16 * 1024 * 1024 * 1024,
 		FilesystemType: "ext4",
@@ -310,6 +354,7 @@ func (s *MigrationExportSuite) TestMachineDevices(c *gc.C) {
 	c.Check(ex1.Label(), gc.Equals, "sda-label")
 	c.Check(ex1.UUID(), gc.Equals, "some-uuid")
 	c.Check(ex1.HardwareID(), gc.Equals, "magic")
+	c.Check(ex1.WWN(), gc.Equals, "drbr")
 	c.Check(ex1.BusAddress(), gc.Equals, "bus stop")
 	c.Check(ex1.Size(), gc.Equals, uint64(16*1024*1024*1024))
 	c.Check(ex1.FilesystemType(), gc.Equals, "ext4")
@@ -568,6 +613,66 @@ func (s *MigrationExportSuite) TestRelations(c *gc.C) {
 	checkEndpoint(exEps[1], wordpress_0.Name(), wpEp, wordpressSettings)
 }
 
+func (s *MigrationExportSuite) TestSubordinateRelations(c *gc.C) {
+	wordpress := state.AddTestingService(c, s.State, "wordpress", state.AddTestingCharm(c, s.State, "wordpress"))
+	mysql := state.AddTestingService(c, s.State, "mysql", state.AddTestingCharm(c, s.State, "mysql"))
+	wordpress_0 := s.Factory.MakeUnit(c, &factory.UnitParams{Application: wordpress})
+	mysql_0 := s.Factory.MakeUnit(c, &factory.UnitParams{Application: mysql})
+
+	logging := s.AddTestingService(c, "logging", s.AddTestingCharm(c, "logging"))
+
+	addSubordinate := func(app *state.Application, unit *state.Unit) {
+		eps, err := s.State.InferEndpoints(app.Name(), logging.Name())
+		c.Assert(err, jc.ErrorIsNil)
+		rel, err := s.State.AddRelation(eps...)
+		c.Assert(err, jc.ErrorIsNil)
+		pru, err := rel.Unit(unit)
+		c.Assert(err, jc.ErrorIsNil)
+		err = pru.EnterScope(nil)
+		c.Assert(err, jc.ErrorIsNil)
+		// Need to reload the doc to get the subordinates.
+		err = unit.Refresh()
+		c.Assert(err, jc.ErrorIsNil)
+		subordinates := unit.SubordinateNames()
+		c.Assert(subordinates, gc.HasLen, 1)
+		loggingUnit, err := s.State.Unit(subordinates[0])
+		c.Assert(err, jc.ErrorIsNil)
+		sub, err := rel.Unit(loggingUnit)
+		c.Assert(err, jc.ErrorIsNil)
+		err = sub.EnterScope(nil)
+		c.Assert(err, jc.ErrorIsNil)
+	}
+
+	addSubordinate(mysql, mysql_0)
+	addSubordinate(wordpress, wordpress_0)
+
+	setTools := func(unit *state.Unit) {
+		app, err := unit.Application()
+		c.Assert(err, jc.ErrorIsNil)
+		agentTools := version.Binary{
+			Number: jujuversion.Current,
+			Arch:   arch.HostArch(),
+			Series: app.Series(),
+		}
+		err = unit.SetAgentVersion(agentTools)
+		c.Assert(err, jc.ErrorIsNil)
+	}
+
+	units, err := logging.AllUnits()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(units, gc.HasLen, 2)
+
+	for _, unit := range units {
+		setTools(unit)
+	}
+
+	model, err := s.State.Export()
+	c.Assert(err, jc.ErrorIsNil)
+
+	rels := model.Relations()
+	c.Assert(rels, gc.HasLen, 2)
+}
+
 func (s *MigrationExportSuite) TestSpaces(c *gc.C) {
 	s.Factory.MakeSpace(c, &factory.SpaceParams{
 		Name: "one", ProviderID: network.Id("provider"), IsPublic: true})
@@ -614,13 +719,34 @@ func (s *MigrationExportSuite) TestLinkLayerDevices(c *gc.C) {
 	c.Assert(device.Type(), gc.Equals, string(state.EthernetDevice))
 }
 
+func (s *MigrationExportSuite) TestLinkLayerDevicesSkipped(c *gc.C) {
+	machine := s.Factory.MakeMachine(c, &factory.MachineParams{
+		Constraints: constraints.MustParse("arch=amd64 mem=8G"),
+	})
+	deviceArgs := state.LinkLayerDeviceArgs{
+		Name: "foo",
+		Type: state.EthernetDevice,
+	}
+	err := machine.SetLinkLayerDevices(deviceArgs)
+	c.Assert(err, jc.ErrorIsNil)
+
+	model, err := s.State.ExportPartial(state.ExportConfig{
+		SkipLinkLayerDevices: true,
+	})
+	c.Assert(err, jc.ErrorIsNil)
+
+	devices := model.LinkLayerDevices()
+	c.Assert(devices, gc.HasLen, 0)
+}
+
 func (s *MigrationExportSuite) TestSubnets(c *gc.C) {
 	_, err := s.State.AddSubnet(state.SubnetInfo{
-		CIDR:             "10.0.0.0/24",
-		ProviderId:       network.Id("foo"),
-		VLANTag:          64,
-		AvailabilityZone: "bar",
-		SpaceName:        "bam",
+		CIDR:              "10.0.0.0/24",
+		ProviderId:        network.Id("foo"),
+		ProviderNetworkId: network.Id("rust"),
+		VLANTag:           64,
+		AvailabilityZone:  "bar",
+		SpaceName:         "bam",
 	})
 	c.Assert(err, jc.ErrorIsNil)
 	_, err = s.State.AddSpace("bam", "", nil, true)
@@ -634,8 +760,9 @@ func (s *MigrationExportSuite) TestSubnets(c *gc.C) {
 	subnet := subnets[0]
 	c.Assert(subnet.CIDR(), gc.Equals, "10.0.0.0/24")
 	c.Assert(subnet.ProviderId(), gc.Equals, "foo")
+	c.Assert(subnet.ProviderNetworkId(), gc.Equals, "rust")
 	c.Assert(subnet.VLANTag(), gc.Equals, 64)
-	c.Assert(subnet.AvailabilityZone(), gc.Equals, "bar")
+	c.Assert(subnet.AvailabilityZones(), gc.DeepEquals, []string{"bar"})
 	c.Assert(subnet.SpaceName(), gc.Equals, "bam")
 }
 
@@ -680,6 +807,39 @@ func (s *MigrationExportSuite) TestIPAddresses(c *gc.C) {
 	c.Assert(addr.GatewayAddress(), gc.Equals, "0.1.2.1")
 }
 
+func (s *MigrationExportSuite) TestIPAddressesSkipped(c *gc.C) {
+	machine := s.Factory.MakeMachine(c, &factory.MachineParams{
+		Constraints: constraints.MustParse("arch=amd64 mem=8G"),
+	})
+	_, err := s.State.AddSubnet(state.SubnetInfo{CIDR: "0.1.2.0/24"})
+	c.Assert(err, jc.ErrorIsNil)
+	deviceArgs := state.LinkLayerDeviceArgs{
+		Name: "foo",
+		Type: state.EthernetDevice,
+	}
+	err = machine.SetLinkLayerDevices(deviceArgs)
+	c.Assert(err, jc.ErrorIsNil)
+	args := state.LinkLayerDeviceAddress{
+		DeviceName:       "foo",
+		ConfigMethod:     state.StaticAddress,
+		CIDRAddress:      "0.1.2.3/24",
+		ProviderID:       "bar",
+		DNSServers:       []string{"bam", "mam"},
+		DNSSearchDomains: []string{"weeee"},
+		GatewayAddress:   "0.1.2.1",
+	}
+	err = machine.SetDevicesAddresses(args)
+	c.Assert(err, jc.ErrorIsNil)
+
+	model, err := s.State.ExportPartial(state.ExportConfig{
+		SkipIPAddresses: true,
+	})
+	c.Assert(err, jc.ErrorIsNil)
+
+	addresses := model.IPAddresses()
+	c.Assert(addresses, gc.HasLen, 0)
+}
+
 func (s *MigrationExportSuite) TestSSHHostKeys(c *gc.C) {
 	machine := s.Factory.MakeMachine(c, &factory.MachineParams{
 		Constraints: constraints.MustParse("arch=amd64 mem=8G"),
@@ -697,7 +857,23 @@ func (s *MigrationExportSuite) TestSSHHostKeys(c *gc.C) {
 	c.Assert(key.Keys(), jc.DeepEquals, []string{"bam", "mam"})
 }
 
-func (s *MigrationExportSuite) TestCloudImageMetadatas(c *gc.C) {
+func (s *MigrationExportSuite) TestSSHHostKeysSkipped(c *gc.C) {
+	machine := s.Factory.MakeMachine(c, &factory.MachineParams{
+		Constraints: constraints.MustParse("arch=amd64 mem=8G"),
+	})
+	err := s.State.SetSSHHostKeys(machine.MachineTag(), []string{"bam", "mam"})
+	c.Assert(err, jc.ErrorIsNil)
+
+	model, err := s.State.ExportPartial(state.ExportConfig{
+		SkipSSHHostKeys: true,
+	})
+	c.Assert(err, jc.ErrorIsNil)
+
+	keys := model.SSHHostKeys()
+	c.Assert(keys, gc.HasLen, 0)
+}
+
+func (s *MigrationExportSuite) TestCloudImageMetadata(c *gc.C) {
 	storageSize := uint64(3)
 	attrs := cloudimagemetadata.MetadataAttributes{
 		Stream:          "stream",
@@ -736,6 +912,33 @@ func (s *MigrationExportSuite) TestCloudImageMetadatas(c *gc.C) {
 	c.Check(image.DateCreated(), gc.Equals, int64(2))
 }
 
+func (s *MigrationExportSuite) TestCloudImageMetadataSkipped(c *gc.C) {
+	storageSize := uint64(3)
+	attrs := cloudimagemetadata.MetadataAttributes{
+		Stream:          "stream",
+		Region:          "region-test",
+		Version:         "14.04",
+		Series:          "trusty",
+		Arch:            "arch",
+		VirtType:        "virtType-test",
+		RootStorageType: "rootStorageType-test",
+		RootStorageSize: &storageSize,
+		Source:          "test",
+	}
+	metadata := []cloudimagemetadata.Metadata{{attrs, 2, "1", 2}}
+
+	err := s.State.CloudImageMetadataStorage.SaveMetadata(metadata)
+	c.Assert(err, jc.ErrorIsNil)
+
+	model, err := s.State.ExportPartial(state.ExportConfig{
+		SkipCloudImageMetadata: true,
+	})
+	c.Assert(err, jc.ErrorIsNil)
+
+	images := model.CloudImageMetadata()
+	c.Assert(images, gc.HasLen, 0)
+}
+
 func (s *MigrationExportSuite) TestActions(c *gc.C) {
 	machine := s.Factory.MakeMachine(c, &factory.MachineParams{
 		Constraints: constraints.MustParse("arch=amd64 mem=8G"),
@@ -755,6 +958,22 @@ func (s *MigrationExportSuite) TestActions(c *gc.C) {
 	c.Check(action.Message(), gc.Equals, "")
 }
 
+func (s *MigrationExportSuite) TestActionsSkipped(c *gc.C) {
+	machine := s.Factory.MakeMachine(c, &factory.MachineParams{
+		Constraints: constraints.MustParse("arch=amd64 mem=8G"),
+	})
+	_, err := s.State.EnqueueAction(machine.MachineTag(), "foo", nil)
+	c.Assert(err, jc.ErrorIsNil)
+
+	model, err := s.State.ExportPartial(state.ExportConfig{
+		SkipActions: true,
+	})
+	c.Assert(err, jc.ErrorIsNil)
+
+	actions := model.Actions()
+	c.Assert(actions, gc.HasLen, 0)
+}
+
 type goodToken struct{}
 
 // Check implements leadership.Token
@@ -778,6 +997,7 @@ func (s *MigrationExportSuite) TestVolumes(c *gc.C) {
 	volTag := names.NewVolumeTag("0/0")
 	err := s.State.SetVolumeInfo(volTag, state.VolumeInfo{
 		HardwareId: "magic",
+		WWN:        "drbr",
 		Size:       1500,
 		VolumeId:   "volume id",
 		Persistent: true,
@@ -799,13 +1019,11 @@ func (s *MigrationExportSuite) TestVolumes(c *gc.C) {
 	provisioned, notProvisioned := volumes[0], volumes[1]
 
 	c.Check(provisioned.Tag(), gc.Equals, volTag)
-	binding, err := provisioned.Binding()
-	c.Check(err, jc.ErrorIsNil)
-	c.Check(binding, gc.Equals, machineTag)
 	c.Check(provisioned.Provisioned(), jc.IsTrue)
 	c.Check(provisioned.Size(), gc.Equals, uint64(1500))
 	c.Check(provisioned.Pool(), gc.Equals, "loop")
 	c.Check(provisioned.HardwareID(), gc.Equals, "magic")
+	c.Check(provisioned.WWN(), gc.Equals, "drbr")
 	c.Check(provisioned.VolumeID(), gc.Equals, "volume id")
 	c.Check(provisioned.Persistent(), jc.IsTrue)
 	attachments := provisioned.Attachments()
@@ -819,9 +1037,6 @@ func (s *MigrationExportSuite) TestVolumes(c *gc.C) {
 	c.Check(attachment.BusAddress(), gc.Equals, "bus address")
 
 	c.Check(notProvisioned.Tag(), gc.Equals, names.NewVolumeTag("0/1"))
-	binding, err = notProvisioned.Binding()
-	c.Check(err, jc.ErrorIsNil)
-	c.Check(binding, gc.Equals, machineTag)
 	c.Check(notProvisioned.Provisioned(), jc.IsFalse)
 	c.Check(notProvisioned.Size(), gc.Equals, uint64(4000))
 	c.Check(notProvisioned.Pool(), gc.Equals, "loop")
@@ -880,9 +1095,6 @@ func (s *MigrationExportSuite) TestFilesystems(c *gc.C) {
 	c.Check(provisioned.Tag(), gc.Equals, fsTag)
 	c.Check(provisioned.Volume(), gc.Equals, names.VolumeTag{})
 	c.Check(provisioned.Storage(), gc.Equals, names.StorageTag{})
-	binding, err := provisioned.Binding()
-	c.Check(err, jc.ErrorIsNil)
-	c.Check(binding, gc.Equals, machineTag)
 	c.Check(provisioned.Provisioned(), jc.IsTrue)
 	c.Check(provisioned.Size(), gc.Equals, uint64(1500))
 	c.Check(provisioned.Pool(), gc.Equals, "rootfs")
@@ -898,9 +1110,6 @@ func (s *MigrationExportSuite) TestFilesystems(c *gc.C) {
 	c.Check(notProvisioned.Tag(), gc.Equals, names.NewFilesystemTag("0/1"))
 	c.Check(notProvisioned.Volume(), gc.Equals, names.VolumeTag{})
 	c.Check(notProvisioned.Storage(), gc.Equals, names.StorageTag{})
-	binding, err = notProvisioned.Binding()
-	c.Check(err, jc.ErrorIsNil)
-	c.Check(binding, gc.Equals, machineTag)
 	c.Check(notProvisioned.Provisioned(), jc.IsFalse)
 	c.Check(notProvisioned.Size(), gc.Equals, uint64(4000))
 	c.Check(notProvisioned.Pool(), gc.Equals, "rootfs")
@@ -1010,3 +1219,116 @@ func (s *MigrationExportSuite) TestPayloads(c *gc.C) {
 	c.Check(payload.State(), gc.Equals, original.Status)
 	c.Check(payload.Labels(), jc.DeepEquals, original.Labels)
 }
+
+func (s *MigrationExportSuite) TestResources(c *gc.C) {
+	app := s.Factory.MakeApplication(c, nil)
+	unit1 := s.Factory.MakeUnit(c, &factory.UnitParams{
+		Application: app,
+	})
+	unit2 := s.Factory.MakeUnit(c, &factory.UnitParams{
+		Application: app,
+	})
+
+	st, err := s.State.Resources()
+	c.Assert(err, jc.ErrorIsNil)
+
+	setUnitResource := func(u *state.Unit) {
+		_, reader, err := st.OpenResourceForUniter(u, "spam")
+		c.Assert(err, jc.ErrorIsNil)
+		defer reader.Close()
+		_, err = ioutil.ReadAll(reader) // Need to read the content to set the resource for the unit.
+		c.Assert(err, jc.ErrorIsNil)
+	}
+
+	const body = "ham"
+	const bodySize = int64(len(body))
+
+	// Initially set revision 1 for the application.
+	res1 := s.newResource(c, app.Name(), "spam", 1, body)
+	res1, err = st.SetResource(app.Name(), res1.Username, res1.Resource, bytes.NewBufferString(body))
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Unit 1 gets revision 1.
+	setUnitResource(unit1)
+
+	// Now set revision 2 for the application.
+	res2 := s.newResource(c, app.Name(), "spam", 2, body)
+	res2, err = st.SetResource(app.Name(), res2.Username, res2.Resource, bytes.NewBufferString(body))
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Unit 2 gets revision 2.
+	setUnitResource(unit2)
+
+	// Revision 3 is in the charmstore.
+	res3 := resourcetesting.NewCharmResource(c, "spam", body)
+	res3.Revision = 3
+	err = st.SetCharmStoreResources(app.Name(), []charmresource.Resource{res3}, time.Now())
+	c.Assert(err, jc.ErrorIsNil)
+
+	model, err := s.State.Export()
+	c.Assert(err, jc.ErrorIsNil)
+
+	applications := model.Applications()
+	c.Assert(applications, gc.HasLen, 1)
+	exApp := applications[0]
+
+	exResources := exApp.Resources()
+	c.Assert(exResources, gc.HasLen, 1)
+
+	exResource := exResources[0]
+	c.Check(exResource.Name(), gc.Equals, "spam")
+
+	checkExRevBase := func(exRev description.ResourceRevision, res charmresource.Resource) {
+		c.Check(exRev.Revision(), gc.Equals, res.Revision)
+		c.Check(exRev.Type(), gc.Equals, res.Type.String())
+		c.Check(exRev.Path(), gc.Equals, res.Path)
+		c.Check(exRev.Description(), gc.Equals, res.Description)
+		c.Check(exRev.Origin(), gc.Equals, res.Origin.String())
+		c.Check(exRev.FingerprintHex(), gc.Equals, res.Fingerprint.Hex())
+		c.Check(exRev.Size(), gc.Equals, bodySize)
+	}
+
+	checkExRev := func(exRev description.ResourceRevision, res resource.Resource) {
+		checkExRevBase(exRev, res.Resource)
+		c.Check(exRev.Timestamp().UTC(), gc.Equals, truncateDBTime(res.Timestamp))
+		c.Check(exRev.Username(), gc.Equals, res.Username)
+	}
+
+	checkExRev(exResource.ApplicationRevision(), res2)
+
+	csRev := exResource.CharmStoreRevision()
+	checkExRevBase(csRev, res3)
+	// These shouldn't be set for charmstore only revisions.
+	c.Check(csRev.Timestamp(), gc.Equals, time.Time{})
+	c.Check(csRev.Username(), gc.Equals, "")
+
+	// Units
+	units := exApp.Units()
+	c.Assert(units, gc.HasLen, 2)
+
+	checkUnitRes := func(exUnit description.Unit, unit *state.Unit, res resource.Resource) {
+		c.Assert(exUnit.Name(), gc.Equals, unit.Name())
+		exResources := exUnit.Resources()
+		c.Assert(exResources, gc.HasLen, 1)
+		exRes := exResources[0]
+		c.Check(exRes.Name(), gc.Equals, "spam")
+		checkExRev(exRes.Revision(), res)
+	}
+	checkUnitRes(units[0], unit1, res1)
+	checkUnitRes(units[1], unit2, res2)
+}
+
+func (s *MigrationExportSuite) newResource(c *gc.C, appName, name string, revision int, body string) resource.Resource {
+	opened := resourcetesting.NewResource(c, nil, name, appName, body)
+	res := opened.Resource
+	res.Revision = revision
+	return res
+}
+
+func (s *MigrationExportSuite) TestModelStatus(c *gc.C) {
+	model, err := s.State.Export()
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Check(model.Status().Value(), gc.Equals, "available")
+	c.Check(model.StatusHistory(), gc.HasLen, 1)
+}
