@@ -7,16 +7,18 @@ import (
 	"fmt"
 	"time" // only uses time.Time values
 
+	"github.com/juju/description"
 	"github.com/juju/errors"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
+	"github.com/juju/utils/arch"
 	"github.com/juju/version"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/charm.v6-unstable"
 	"gopkg.in/juju/names.v2"
+	"gopkg.in/yaml.v2"
 
 	"github.com/juju/juju/constraints"
-	"github.com/juju/juju/core/description"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/payload"
 	"github.com/juju/juju/permission"
@@ -27,6 +29,7 @@ import (
 	"github.com/juju/juju/storage/provider"
 	coretesting "github.com/juju/juju/testing"
 	"github.com/juju/juju/testing/factory"
+	jujuversion "github.com/juju/juju/version"
 )
 
 type MigrationImportSuite struct {
@@ -56,10 +59,27 @@ func (s *MigrationImportSuite) TestExisting(c *gc.C) {
 	c.Assert(err, jc.Satisfies, errors.IsAlreadyExists)
 }
 
-func (s *MigrationImportSuite) importModel(c *gc.C) (*state.Model, *state.State) {
+func (s *MigrationImportSuite) importModel(c *gc.C, transform ...func(map[string]interface{})) (*state.Model, *state.State) {
 	out, err := s.State.Export()
 	c.Assert(err, jc.ErrorIsNil)
 
+	if len(transform) > 0 {
+		var outM map[string]interface{}
+		outYaml, err := description.Serialize(out)
+		c.Assert(err, jc.ErrorIsNil)
+		err = yaml.Unmarshal(outYaml, &outM)
+		c.Assert(err, jc.ErrorIsNil)
+
+		for _, transform := range transform {
+			transform(outM)
+		}
+
+		outYaml, err = yaml.Marshal(outM)
+		c.Assert(err, jc.ErrorIsNil)
+		out, err = description.Deserialize(outYaml)
+		c.Assert(err, jc.ErrorIsNil)
+	}
+
 	uuid := utils.MustNewUUID().String()
 	in := newModel(out, uuid, "new")
 
@@ -90,6 +110,10 @@ func (s *MigrationImportSuite) TestNewModel(c *gc.C) {
 	original, err := s.State.Model()
 	c.Assert(err, jc.ErrorIsNil)
 
+	environVersion := 123
+	err = original.SetEnvironVersion(environVersion)
+	c.Assert(err, jc.ErrorIsNil)
+
 	err = s.State.SetAnnotations(original, testAnnotations)
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -106,8 +130,20 @@ func (s *MigrationImportSuite) TestNewModel(c *gc.C) {
 	c.Assert(newModel.Owner(), gc.Equals, original.Owner())
 	c.Assert(newModel.LatestToolsVersion(), gc.Equals, latestTools)
 	c.Assert(newModel.MigrationMode(), gc.Equals, state.MigrationModeImporting)
+	c.Assert(newModel.EnvironVersion(), gc.Equals, environVersion)
 	s.assertAnnotations(c, newSt, newModel)
 
+	statusInfo, err := newModel.Status()
+	c.Check(err, jc.ErrorIsNil)
+	c.Check(statusInfo.Status, gc.Equals, status.Busy)
+	c.Check(statusInfo.Message, gc.Equals, "importing")
+	// One for original "available", one for "busy (importing)"
+	history, err := newModel.StatusHistory(status.StatusHistoryFilter{Size: 5})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(history, gc.HasLen, 2)
+	c.Check(history[0].Status, gc.Equals, status.Busy)
+	c.Check(history[1].Status, gc.Equals, status.Available)
+
 	originalConfig, err := original.Config()
 	c.Assert(err, jc.ErrorIsNil)
 	originalAttrs := originalConfig.AllAttrs()
@@ -210,6 +246,50 @@ func (s *MigrationImportSuite) TestModelUsers(c *gc.C) {
 	c.Assert(allUsers, gc.HasLen, 3)
 }
 
+func (s *MigrationImportSuite) TestSLA(c *gc.C) {
+	err := s.State.SetSLA("essential", "bob", []byte("creds"))
+	c.Assert(err, jc.ErrorIsNil)
+	newModel, newSt := s.importModel(c)
+
+	c.Assert(newModel.SLALevel(), gc.Equals, "essential")
+	c.Assert(newModel.SLACredential(), jc.DeepEquals, []byte("creds"))
+	level, err := newSt.SLALevel()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(level, gc.Equals, "essential")
+	creds, err := newSt.SLACredential()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(creds, jc.DeepEquals, []byte("creds"))
+}
+
+func (s *MigrationImportSuite) TestMeterStatus(c *gc.C) {
+	err := s.State.SetModelMeterStatus("RED", "info message")
+	c.Assert(err, jc.ErrorIsNil)
+	newModel, newSt := s.importModel(c)
+
+	ms := newModel.MeterStatus()
+	c.Assert(ms.Code.String(), gc.Equals, "RED")
+	c.Assert(ms.Info, gc.Equals, "info message")
+	ms, err = newSt.ModelMeterStatus()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(ms.Code.String(), gc.Equals, "RED")
+	c.Assert(ms.Info, gc.Equals, "info message")
+}
+
+func (s *MigrationImportSuite) TestMeterStatusNotAvailable(c *gc.C) {
+	newModel, newSt := s.importModel(c, func(desc map[string]interface{}) {
+		c.Log(desc["meter-status"])
+		desc["meter-status"].(map[interface{}]interface{})["code"] = ""
+	})
+
+	ms := newModel.MeterStatus()
+	c.Assert(ms.Code.String(), gc.Equals, "NOT AVAILABLE")
+	c.Assert(ms.Info, gc.Equals, "")
+	ms, err := newSt.ModelMeterStatus()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(ms.Code.String(), gc.Equals, "NOT AVAILABLE")
+	c.Assert(ms.Info, gc.Equals, "")
+}
+
 func (s *MigrationImportSuite) AssertMachineEqual(c *gc.C, newMachine, oldMachine *state.Machine) {
 	c.Assert(newMachine.Id(), gc.Equals, oldMachine.Id())
 	c.Assert(newMachine.Principals(), jc.DeepEquals, oldMachine.Principals())
@@ -227,6 +307,24 @@ func (s *MigrationImportSuite) AssertMachineEqual(c *gc.C, newMachine, oldMachin
 	oldTools, err := oldMachine.AgentTools()
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(newTools, jc.DeepEquals, oldTools)
+
+	oldStatus, err := oldMachine.Status()
+	c.Assert(err, jc.ErrorIsNil)
+	newStatus, err := newMachine.Status()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(newStatus, jc.DeepEquals, oldStatus)
+
+	oldInstID, err := oldMachine.InstanceId()
+	c.Assert(err, jc.ErrorIsNil)
+	newInstID, err := newMachine.InstanceId()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(newInstID, gc.Equals, oldInstID)
+
+	oldStatus, err = oldMachine.InstanceStatus()
+	c.Assert(err, jc.ErrorIsNil)
+	newStatus, err = newMachine.InstanceStatus()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(newStatus, jc.DeepEquals, oldStatus)
 }
 
 func (s *MigrationImportSuite) TestMachines(c *gc.C) {
@@ -290,6 +388,7 @@ func (s *MigrationImportSuite) TestMachineDevices(c *gc.C) {
 		Label:          "sda-label",
 		UUID:           "some-uuid",
 		HardwareId:     "magic",
+		WWN:            "drbr",
 		BusAddress:     "bus stop",
 		Size:           16 * 1024 * 1024 * 1024,
 		FilesystemType: "ext4",
@@ -314,7 +413,12 @@ func (s *MigrationImportSuite) TestMachineDevices(c *gc.C) {
 func (s *MigrationImportSuite) TestApplications(c *gc.C) {
 	// Add a application with both settings and leadership settings.
 	cons := constraints.MustParse("arch=amd64 mem=8G")
+	charm := s.Factory.MakeCharm(c, &factory.CharmParams{
+		Name: "starsay", // it has resources
+	})
+	c.Assert(charm.Meta().Resources, gc.HasLen, 3)
 	application := s.Factory.MakeApplication(c, &factory.ApplicationParams{
+		Charm: charm,
 		Settings: map[string]interface{}{
 			"foo": "bar",
 		},
@@ -369,6 +473,12 @@ func (s *MigrationImportSuite) TestApplications(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	// Can't test the constraints directly, so go through the string repr.
 	c.Assert(newCons.String(), gc.Equals, cons.String())
+
+	rSt, err := newSt.Resources()
+	c.Assert(err, jc.ErrorIsNil)
+	resources, err := rSt.ListResources(imported.Name())
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(resources.Resources, gc.HasLen, 3)
 }
 
 func (s *MigrationImportSuite) TestApplicationLeaders(c *gc.C) {
@@ -389,6 +499,104 @@ func (s *MigrationImportSuite) TestApplicationLeaders(c *gc.C) {
 	})
 }
 
+func (s *MigrationImportSuite) TestCharmRevSequencesNotImported(c *gc.C) {
+	s.Factory.MakeApplication(c, &factory.ApplicationParams{
+		Charm: s.Factory.MakeCharm(c, &factory.CharmParams{
+			Name: "mysql",
+			URL:  "local:trusty/mysql-2",
+		}),
+	})
+	// Sequence should be set in the source model.
+	const charmSeqName = "charmrev-local:trusty/mysql"
+	nextVal, err := state.Sequence(s.State, charmSeqName)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(nextVal, gc.Equals, 3)
+
+	out, err := s.State.Export()
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Assert(len(out.Applications()), gc.Equals, 1)
+
+	uuid := utils.MustNewUUID().String()
+	in := newModel(out, uuid, "new")
+
+	_, newSt, err := s.State.Import(in)
+	c.Assert(err, jc.ErrorIsNil)
+	defer newSt.Close()
+
+	// Charm revision sequence shouldn't have been imported. The
+	// import of the charm binaries (done separately later) will
+	// handle this.
+	nextVal, err = state.Sequence(newSt, charmSeqName)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(nextVal, gc.Equals, 0)
+}
+
+func (s *MigrationImportSuite) TestApplicationsSubordinatesAfter(c *gc.C) {
+	// Test for https://bugs.launchpad.net/juju/+bug/1650249
+	subordinate := s.Factory.MakeApplication(c, &factory.ApplicationParams{
+		Charm: s.Factory.MakeCharm(c, &factory.CharmParams{Name: "logging"}),
+	})
+
+	principal := s.Factory.MakeApplication(c, &factory.ApplicationParams{
+		Charm: s.Factory.MakeCharm(c, &factory.CharmParams{Name: "mysql"}),
+	})
+	unit := s.Factory.MakeUnit(c, &factory.UnitParams{Application: principal})
+
+	sEndpoint, err := subordinate.Endpoint("info")
+	c.Assert(err, jc.ErrorIsNil)
+	pEndpoint, err := principal.Endpoint("juju-info")
+	c.Assert(err, jc.ErrorIsNil)
+	relation := s.Factory.MakeRelation(c, &factory.RelationParams{
+		Endpoints: []state.Endpoint{sEndpoint, pEndpoint},
+	})
+
+	ru, err := relation.Unit(unit)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Ensure the subordinate unit is created.
+	err = ru.EnterScope(nil)
+	c.Assert(err, jc.ErrorIsNil)
+
+	tools, err := unit.AgentTools()
+	c.Assert(err, jc.ErrorIsNil)
+
+	sUnits, err := subordinate.AllUnits()
+	for _, u := range sUnits {
+		// For some reason the EnterScope call doesn't set up the
+		// version or enter the scope for the subordinate unit on the
+		// other side of the relation.
+		err := u.SetAgentVersion(tools.Version)
+		c.Assert(err, jc.ErrorIsNil)
+		ru, err := relation.Unit(u)
+		c.Assert(err, jc.ErrorIsNil)
+		err = ru.EnterScope(nil)
+		c.Assert(err, jc.ErrorIsNil)
+	}
+
+	out, err := s.State.Export()
+	c.Assert(err, jc.ErrorIsNil)
+
+	apps := out.Applications()
+	c.Assert(len(apps), gc.Equals, 2)
+
+	// This test is only valid if the subordinate logging application
+	// comes first in the model output.
+	if apps[0].Name() != "logging" {
+		out = &swapModel{out, c}
+	}
+
+	uuid := utils.MustNewUUID().String()
+	in := newModel(out, uuid, "new")
+
+	_, newSt, err := s.State.Import(in)
+	c.Assert(err, jc.ErrorIsNil)
+	// add the cleanup here to close the model.
+	s.AddCleanup(func(c *gc.C) {
+		c.Check(newSt.Close(), jc.ErrorIsNil)
+	})
+}
+
 func (s *MigrationImportSuite) TestUnits(c *gc.C) {
 	s.assertUnitsMigrated(c, constraints.MustParse("arch=amd64 mem=8G"))
 }
@@ -648,11 +856,12 @@ func (s *MigrationImportSuite) TestLinkLayerDeviceMigratesReferences(c *gc.C) {
 
 func (s *MigrationImportSuite) TestSubnets(c *gc.C) {
 	original, err := s.State.AddSubnet(state.SubnetInfo{
-		CIDR:             "10.0.0.0/24",
-		ProviderId:       network.Id("foo"),
-		VLANTag:          64,
-		AvailabilityZone: "bar",
-		SpaceName:        "bam",
+		CIDR:              "10.0.0.0/24",
+		ProviderId:        network.Id("foo"),
+		ProviderNetworkId: network.Id("elm"),
+		VLANTag:           64,
+		AvailabilityZone:  "bar",
+		SpaceName:         "bam",
 	})
 	c.Assert(err, jc.ErrorIsNil)
 	_, err = s.State.AddSpace("bam", "", nil, true)
@@ -665,6 +874,7 @@ func (s *MigrationImportSuite) TestSubnets(c *gc.C) {
 
 	c.Assert(subnet.CIDR(), gc.Equals, "10.0.0.0/24")
 	c.Assert(subnet.ProviderId(), gc.Equals, network.Id("foo"))
+	c.Assert(subnet.ProviderNetworkId(), gc.Equals, network.Id("elm"))
 	c.Assert(subnet.VLANTag(), gc.Equals, 64)
 	c.Assert(subnet.AvailabilityZone(), gc.Equals, "bar")
 	c.Assert(subnet.SpaceName(), gc.Equals, "bam")
@@ -803,6 +1013,7 @@ func (s *MigrationImportSuite) TestVolumes(c *gc.C) {
 	volTag := names.NewVolumeTag("0/0")
 	volInfo := state.VolumeInfo{
 		HardwareId: "magic",
+		WWN:        "drbr",
 		Size:       1500,
 		Pool:       "loop",
 		VolumeId:   "volume id",
@@ -947,14 +1158,91 @@ func (s *MigrationImportSuite) TestStorage(c *gc.C) {
 	c.Check(instance.Kind(), gc.Equals, original.Kind())
 	c.Check(instance.Life(), gc.Equals, original.Life())
 	c.Check(instance.StorageName(), gc.Equals, original.StorageName())
+	c.Check(instance.Pool(), gc.Equals, original.Pool())
 	c.Check(state.StorageAttachmentCount(instance), gc.Equals, originalCount)
 
 	attachments, err := newSt.StorageAttachments(storageTag)
-
 	c.Assert(attachments, gc.HasLen, 1)
 	c.Assert(attachments[0].Unit(), gc.Equals, u.UnitTag())
 }
 
+func (s *MigrationImportSuite) TestStorageInstanceConstraints(c *gc.C) {
+	_, _, storageTag := s.makeUnitWithStorage(c)
+	_, newSt := s.importModel(c, func(desc map[string]interface{}) {
+		storages := desc["storages"].(map[interface{}]interface{})
+		for _, item := range storages["storages"].([]interface{}) {
+			storage := item.(map[interface{}]interface{})
+			cons := storage["constraints"].(map[interface{}]interface{})
+			cons["pool"] = "static"
+		}
+	})
+	instance, err := newSt.StorageInstance(storageTag)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(instance.Pool(), gc.Equals, "static")
+}
+
+func (s *MigrationImportSuite) TestStorageInstanceConstraintsFallback(c *gc.C) {
+	_, u, storageTag0 := s.makeUnitWithStorage(c)
+
+	err := s.State.AddStorageForUnit(u.UnitTag(), "allecto", state.StorageConstraints{
+		Count: 3,
+		Size:  1234,
+		Pool:  "modelscoped",
+	})
+	c.Assert(err, jc.ErrorIsNil)
+	storageTag1 := names.NewStorageTag("allecto/1")
+	storageTag2 := names.NewStorageTag("allecto/2")
+
+	// We delete the storage instance constraints for each storage
+	// instance. For data/0 and allecto/1 we also delete the volume,
+	// and we delete the application storage constraints for "data".
+	//
+	// We expect:
+	//  - for data/0, to get the defaults (loop, 1G)
+	//  - for allecto/1, to get the application storage constraints
+	//  - for allecto/2, to get the volume pool/size
+
+	_, newSt := s.importModel(c, func(desc map[string]interface{}) {
+		applications := desc["applications"].(map[interface{}]interface{})
+		volumes := desc["volumes"].(map[interface{}]interface{})
+		storages := desc["storages"].(map[interface{}]interface{})
+		storages["version"] = 2
+
+		app := applications["applications"].([]interface{})[0].(map[interface{}]interface{})
+		sc := app["storage-constraints"].(map[interface{}]interface{})
+		delete(sc, "data")
+		sc["allecto"].(map[interface{}]interface{})["pool"] = "modelscoped-block"
+
+		var keepVolumes []interface{}
+		for _, item := range volumes["volumes"].([]interface{}) {
+			volume := item.(map[interface{}]interface{})
+			switch volume["storage-id"] {
+			case storageTag0.Id(), storageTag1.Id():
+			default:
+				keepVolumes = append(keepVolumes, volume)
+			}
+		}
+		volumes["volumes"] = keepVolumes
+
+		for _, item := range storages["storages"].([]interface{}) {
+			storage := item.(map[interface{}]interface{})
+			delete(storage, "constraints")
+		}
+	})
+
+	instance0, err := newSt.StorageInstance(storageTag0)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(instance0.Pool(), gc.Equals, "loop")
+
+	instance1, err := newSt.StorageInstance(storageTag1)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(instance1.Pool(), gc.Equals, "modelscoped-block")
+
+	instance2, err := newSt.StorageInstance(storageTag2)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(instance2.Pool(), gc.Equals, "modelscoped")
+}
+
 func (s *MigrationImportSuite) TestStoragePools(c *gc.C) {
 	pm := poolmanager.New(state.NewStateSettings(s.State), provider.CommonStorageProviders())
 	_, err := pm.Create("test-pool", provider.LoopProviderType, map[string]interface{}{
@@ -1020,6 +1308,185 @@ func (s *MigrationImportSuite) TestPayloads(c *gc.C) {
 	c.Check(payload.Machine, gc.Equals, machineID)
 }
 
+func (s *MigrationImportSuite) TestRemoteApplications(c *gc.C) {
+	// For now we want to prevent importing models that have remote
+	// applications - cross-model relations don't support relations
+	// with the models in different controllers.
+	_, err := s.State.AddRemoteApplication(state.AddRemoteApplicationParams{
+		Name:        "gravy-rainbow",
+		URL:         "me/model.rainbow",
+		SourceModel: s.State.ModelTag(),
+		Token:       "charisma",
+		Endpoints: []charm.Relation{{
+			Interface: "mysql",
+			Name:      "db",
+			Role:      charm.RoleProvider,
+			Scope:     charm.ScopeGlobal,
+		}, {
+			Interface: "mysql-root",
+			Name:      "db-admin",
+			Limit:     5,
+			Role:      charm.RoleProvider,
+			Scope:     charm.ScopeGlobal,
+		}, {
+			Interface: "logging",
+			Name:      "logging",
+			Role:      charm.RoleProvider,
+			Scope:     charm.ScopeGlobal,
+		}},
+	})
+	c.Assert(err, jc.ErrorIsNil)
+
+	out, err := s.State.Export()
+	c.Assert(err, jc.ErrorIsNil)
+
+	uuid := utils.MustNewUUID().String()
+	in := newModel(out, uuid, "new")
+	// Models for this version of Juju don't export remote
+	// applications but we still want to guard against accidentally
+	// importing any that may exist from earlier versions.
+	in.AddRemoteApplication(description.RemoteApplicationArgs{
+		SourceModel: coretesting.ModelTag,
+		OfferUUID:   utils.MustNewUUID().String(),
+		Tag:         names.NewApplicationTag("remote"),
+	})
+
+	_, newSt, err := s.State.Import(in)
+	if err == nil {
+		defer newSt.Close()
+	}
+	c.Assert(err, gc.ErrorMatches, "can't import models with remote applications")
+}
+
+func (s *MigrationImportSuite) TestApplicationsWithNilConfigValues(c *gc.C) {
+	application := s.Factory.MakeApplication(c, &factory.ApplicationParams{
+		Settings: map[string]interface{}{
+			"foo": "bar",
+		},
+	})
+	s.primeStatusHistory(c, application, status.Active, 5)
+	// Since above factory method calls newly updated state.AddApplication(...)
+	// which removes config settings with nil value before writing
+	// application into database,
+	// strip config setting values to nil directly to simulate
+	// what could happen to some applications in 2.0 and 2.1.
+	// For more context, see https://bugs.launchpad.net/juju/+bug/1667199
+	settings := state.GetApplicationSettings(s.State, application)
+	settings.Set("foo", nil)
+	_, err := settings.Write()
+	c.Assert(err, jc.ErrorIsNil)
+
+	_, newSt := s.importModel(c)
+
+	importedApplications, err := newSt.AllApplications()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(importedApplications, gc.HasLen, 1)
+	importedApplication := importedApplications[0]
+
+	// Ensure that during import application settings with nil config values
+	// were stripped and not written into database.
+	importedSettings := state.GetApplicationSettings(newSt, importedApplication)
+	_, importedFound := importedSettings.Get("foo")
+	c.Assert(importedFound, jc.IsFalse)
+}
+
+func (s *MigrationImportSuite) TestOneSubordinateTwoGuvnors(c *gc.C) {
+	// Check that invalid relationscopes aren't created when importing
+	// a subordinate related to 2 principals.
+	wordpress := state.AddTestingService(c, s.State, "wordpress", state.AddTestingCharm(c, s.State, "wordpress"))
+	mysql := state.AddTestingService(c, s.State, "mysql", state.AddTestingCharm(c, s.State, "mysql"))
+	wordpress0 := s.Factory.MakeUnit(c, &factory.UnitParams{Application: wordpress})
+	mysql0 := s.Factory.MakeUnit(c, &factory.UnitParams{Application: mysql})
+
+	logging := s.AddTestingService(c, "logging", s.AddTestingCharm(c, "logging"))
+
+	addSubordinate := func(app *state.Application, unit *state.Unit) string {
+		eps, err := s.State.InferEndpoints(app.Name(), logging.Name())
+		c.Assert(err, jc.ErrorIsNil)
+		rel, err := s.State.AddRelation(eps...)
+		c.Assert(err, jc.ErrorIsNil)
+		pru, err := rel.Unit(unit)
+		c.Assert(err, jc.ErrorIsNil)
+		err = pru.EnterScope(nil)
+		c.Assert(err, jc.ErrorIsNil)
+		// Need to reload the doc to get the subordinates.
+		err = unit.Refresh()
+		c.Assert(err, jc.ErrorIsNil)
+		subordinates := unit.SubordinateNames()
+		c.Assert(subordinates, gc.HasLen, 1)
+		loggingUnit, err := s.State.Unit(subordinates[0])
+		c.Assert(err, jc.ErrorIsNil)
+		sub, err := rel.Unit(loggingUnit)
+		c.Assert(err, jc.ErrorIsNil)
+		err = sub.EnterScope(nil)
+		c.Assert(err, jc.ErrorIsNil)
+		return rel.String()
+	}
+
+	logMysqlKey := addSubordinate(mysql, mysql0)
+	logWpKey := addSubordinate(wordpress, wordpress0)
+
+	units, err := logging.AllUnits()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(units, gc.HasLen, 2)
+
+	for _, unit := range units {
+		app, err := unit.Application()
+		c.Assert(err, jc.ErrorIsNil)
+		agentTools := version.Binary{
+			Number: jujuversion.Current,
+			Arch:   arch.HostArch(),
+			Series: app.Series(),
+		}
+		err = unit.SetAgentVersion(agentTools)
+		c.Assert(err, jc.ErrorIsNil)
+	}
+
+	_, newSt := s.importModel(c)
+
+	logMysqlRel, err := newSt.KeyRelation(logMysqlKey)
+	c.Assert(err, jc.ErrorIsNil)
+	logWpRel, err := newSt.KeyRelation(logWpKey)
+	c.Assert(err, jc.ErrorIsNil)
+
+	mysqlLogUnit, err := newSt.Unit("logging/0")
+	c.Assert(err, jc.ErrorIsNil)
+	wpLogUnit, err := newSt.Unit("logging/1")
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Sanity checks
+	name, ok := mysqlLogUnit.PrincipalName()
+	c.Assert(ok, jc.IsTrue)
+	c.Assert(name, gc.Equals, "mysql/0")
+
+	name, ok = wpLogUnit.PrincipalName()
+	c.Assert(ok, jc.IsTrue)
+	c.Assert(name, gc.Equals, "wordpress/0")
+
+	checkScope := func(unit *state.Unit, rel *state.Relation, expected bool) {
+		ru, err := rel.Unit(unit)
+		c.Assert(err, jc.ErrorIsNil)
+		// Sanity check
+		valid, err := ru.Valid()
+		c.Assert(err, jc.ErrorIsNil)
+		c.Check(valid, gc.Equals, expected)
+
+		inscope, err := ru.InScope()
+		c.Assert(err, jc.ErrorIsNil)
+		c.Check(inscope, gc.Equals, expected)
+	}
+	// The WP logging unit shouldn't be in scope for the mysql-logging
+	// relation.
+	checkScope(wpLogUnit, logMysqlRel, false)
+	// Similarly, the mysql logging unit shouldn't be in scope for the
+	// wp-logging relation.
+	checkScope(mysqlLogUnit, logWpRel, false)
+
+	// But obviously the units should be in their relations.
+	checkScope(mysqlLogUnit, logMysqlRel, true)
+	checkScope(wpLogUnit, logWpRel, true)
+}
+
 // newModel replaces the uuid and name of the config attributes so we
 // can use all the other data to validate imports. An owner and name of the
 // model are unique together in a controller.
@@ -1043,3 +1510,17 @@ func (m *mockModel) Config() map[string]interface{} {
 	c["name"] = m.name
 	return c
 }
+
+// swapModel will swap the order of the applications appearing in the
+// model.
+type swapModel struct {
+	description.Model
+	c *gc.C
+}
+
+func (m swapModel) Applications() []description.Application {
+	values := m.Model.Applications()
+	m.c.Assert(len(values), gc.Equals, 2)
+	values[0], values[1] = values[1], values[0]
+	return values
+}
