@@ -42,6 +42,7 @@ func (s *MigrationSuite) TestKnownCollections(c *gc.C) {
 		unitsC,
 		meterStatusC, // red / green status for metrics of units
 		payloadsC,
+		"resources",
 
 		// relation
 		relationsC,
@@ -157,27 +158,37 @@ func (s *MigrationSuite) TestKnownCollections(c *gc.C) {
 		// Recreated whilst migrating actions.
 		actionNotificationsC,
 
-		//Cross Model Relations - TODO
-		localApplicationDirectoryC,
-		remoteApplicationsC,
-		applicationOffersC,
+		// Global settings store controller specific configuration settings
+		// and are not to be migrated.
+		globalSettingsC,
+
+		// The auditing collection stores a large amount of historical data
+		// and will be streamed across after migration in a similar way to
+		// logging.
+		auditingC,
+
+		// There is a precheck to ensure that there are no pending reboots
+		// for the model being migrated, and as such, there is no need to
+		// migrate that information.
+		rebootC,
+
+		// Charms are added into the migrated model during the binary transfer
+		// phase after the initial model migration.
+		charmsC,
+
+		// Metrics manager maintains controller specific state relating to
+		// the store and forward of charm metrics. Nothing to migrate here.
+		metricsManagerC,
 	)
 
 	// THIS SET WILL BE REMOVED WHEN MIGRATIONS ARE COMPLETE
 	todoCollections := set.NewStrings(
-		// model configuration
-		globalSettingsC,
-
-		// machine
-		rebootC,
-
-		// service / unit
-		charmsC,
-		"resources",
-
 		// uncategorised
-		metricsManagerC, // should really be copied across
-		auditingC,
+		//Cross Model Relations - TODO
+		remoteApplicationsC,
+		applicationOffersC,
+		tokensC,
+		remoteEntitiesC,
 	)
 
 	envCollections := set.NewStrings()
@@ -215,6 +226,9 @@ func (s *MigrationSuite) TestModelDocFields(c *gc.C) {
 		"CloudRegion",
 		"CloudCredential",
 		"LatestAvailableTools",
+		"SLA",
+		"MeterStatus",
+		"EnvironVersion",
 	)
 	s.AssertExportedFields(c, modelDoc{}, fields)
 }
@@ -301,7 +315,12 @@ func (s *MigrationSuite) TestMachineDocFields(c *gc.C) {
 }
 
 func (s *MigrationSuite) TestInstanceDataFields(c *gc.C) {
-	fields := set.NewStrings(
+	ignored := set.NewStrings(
+		// KeepInstance is only set when a machine is
+		// dying/dead (to be removed).
+		"KeepInstance",
+	)
+	migrated := set.NewStrings(
 		// DocID is the env + machine id
 		"DocID",
 		"MachineId",
@@ -310,7 +329,6 @@ func (s *MigrationSuite) TestInstanceDataFields(c *gc.C) {
 		"ModelUUID",
 
 		"InstanceId",
-		"Status",
 		"Arch",
 		"Mem",
 		"RootDisk",
@@ -319,7 +337,7 @@ func (s *MigrationSuite) TestInstanceDataFields(c *gc.C) {
 		"Tags",
 		"AvailZone",
 	)
-	s.AssertExportedFields(c, instanceData{}, fields)
+	s.AssertExportedFields(c, instanceData{}, migrated.Union(ignored))
 }
 
 func (s *MigrationSuite) TestApplicationDocFields(c *gc.C) {
@@ -563,6 +581,7 @@ func (s *MigrationSuite) TestBlockDeviceFields(c *gc.C) {
 		"Label",
 		"UUID",
 		"HardwareId",
+		"WWN",
 		"BusAddress",
 		"Size",
 		"FilesystemType",
@@ -591,6 +610,7 @@ func (s *MigrationSuite) TestSubnetDocFields(c *gc.C) {
 		"SpaceName",
 		"ProviderId",
 		"AvailabilityZone",
+		"ProviderNetworkId",
 	)
 	s.AssertExportedFields(c, subnetDoc{}, migrated.Union(ignored))
 }
@@ -665,19 +685,19 @@ func (s *MigrationSuite) TestVolumeDocFields(c *gc.C) {
 		"ModelUUID",
 		"DocID",
 		"Life",
+		"MachineId", // recreated from pool properties
 	)
 	migrated := set.NewStrings(
 		"Name",
 		"StorageId",
 		"AttachmentCount", // through count of attachment instances
-		"Binding",
 		"Info",
 		"Params",
 	)
 	s.AssertExportedFields(c, volumeDoc{}, migrated.Union(ignored))
 	// The info and params fields ar structs.
 	s.AssertExportedFields(c, VolumeInfo{}, set.NewStrings(
-		"HardwareId", "Size", "Pool", "VolumeId", "Persistent"))
+		"HardwareId", "WWN", "Size", "Pool", "VolumeId", "Persistent"))
 	s.AssertExportedFields(c, VolumeParams{}, set.NewStrings(
 		"Size", "Pool"))
 }
@@ -707,13 +727,13 @@ func (s *MigrationSuite) TestFilesystemDocFields(c *gc.C) {
 		"ModelUUID",
 		"DocID",
 		"Life",
+		"MachineId", // recreated from pool properties
 	)
 	migrated := set.NewStrings(
 		"FilesystemId",
 		"StorageId",
 		"VolumeId",
 		"AttachmentCount", // through count of attachment instances
-		"Binding",
 		"Info",
 		"Params",
 	)
@@ -757,6 +777,7 @@ func (s *MigrationSuite) TestStorageInstanceDocFields(c *gc.C) {
 		"Owner",
 		"StorageName",
 		"AttachmentCount", // through count of attachment instances
+		"Constraints",
 	)
 	s.AssertExportedFields(c, storageInstanceDoc{}, migrated.Union(ignored))
 }
