@@ -32,7 +32,7 @@ var _ = gc.Suite(new(MigrationSuite))
 
 func (s *MigrationSuite) SetUpTest(c *gc.C) {
 	s.ConnSuite.SetUpTest(c)
-	s.clock = jujutesting.NewClock(time.Now().Truncate(time.Second))
+	s.clock = jujutesting.NewClock(truncateDBTime(time.Now()))
 	err := s.State.SetClockForTesting(s.clock)
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -75,7 +75,6 @@ func (s *MigrationSuite) TestCreate(c *gc.C) {
 	c.Check(mig.EndTime().IsZero(), jc.IsTrue)
 	c.Check(mig.StatusMessage(), gc.Equals, "starting")
 	c.Check(mig.InitiatedBy(), gc.Equals, "admin")
-	c.Check(mig.ExternalControl(), jc.IsFalse)
 
 	info, err := mig.TargetInfo()
 	c.Assert(err, jc.ErrorIsNil)
@@ -90,15 +89,6 @@ func (s *MigrationSuite) TestCreate(c *gc.C) {
 	c.Check(model.MigrationMode(), gc.Equals, state.MigrationModeExporting)
 }
 
-func (s *MigrationSuite) TestCreateExternalControl(c *gc.C) {
-	spec := s.stdSpec
-	spec.ExternalControl = true
-	mig, err := s.State2.CreateMigration(spec)
-	c.Assert(err, jc.ErrorIsNil)
-	c.Check(mig.ModelUUID(), gc.Equals, s.State2.ModelUUID())
-	c.Check(mig.ExternalControl(), jc.IsTrue)
-}
-
 func (s *MigrationSuite) TestIsMigrationActive(c *gc.C) {
 	check := func(expected bool) {
 		isActive, err := s.State2.IsMigrationActive()
@@ -179,9 +169,9 @@ func (s *MigrationSuite) TestSpecValidation(c *gc.C) {
 	}, {
 		"TargetInfo is validated",
 		func(spec *state.MigrationSpec) {
-			spec.TargetInfo.CACert = ""
+			spec.TargetInfo.Addrs = nil
 		},
-		"empty CACert not valid",
+		"empty Addrs not valid",
 	}}
 	for _, test := range tests {
 		c.Logf("---- %s -----------", test.label)
@@ -284,10 +274,10 @@ func (s *MigrationSuite) TestGetsLatestAttempt(c *gc.C) {
 	}
 }
 
-func (s *MigrationSuite) TestLatestMigrationPreviousMigration(c *gc.C) {
+func (s *MigrationSuite) TestLatestMigrationWithPrevious(c *gc.C) {
 	// Check the scenario of a model having been migrated away and
-	// then migrated back. The previous migration shouldn't be
-	// reported by LatestMigration.
+	// then migrated back several times. The previous migrations
+	// shouldn't be reported by LatestMigration.
 
 	// Make it appear as if the model has been successfully
 	// migrated. Don't actually remove model documents to simulate it
@@ -300,24 +290,29 @@ func (s *MigrationSuite) TestLatestMigrationPreviousMigration(c *gc.C) {
 		migration.REAP,
 		migration.DONE,
 	}
-	mig, err := s.State2.CreateMigration(s.stdSpec)
-	c.Assert(err, jc.ErrorIsNil)
-	for _, phase := range phases {
-		c.Assert(mig.SetPhase(phase), jc.ErrorIsNil)
+	for i := 0; i < 10; i++ {
+		mig, err := s.State2.CreateMigration(s.stdSpec)
+		c.Assert(err, jc.ErrorIsNil)
+		for _, phase := range phases {
+			c.Assert(mig.SetPhase(phase), jc.ErrorIsNil)
+		}
+		state.ResetMigrationMode(c, s.State2)
 	}
-	state.ResetMigrationMode(c, s.State2)
 
 	// Previous migration shouldn't be reported.
-	_, err = s.State2.LatestMigration()
+	_, err := s.State2.LatestMigration()
 	c.Check(errors.IsNotFound(err), jc.IsTrue)
 
 	// Start a new migration attempt, which should be reported.
-	mig2, err := s.State2.CreateMigration(s.stdSpec)
+	migNext, err := s.State2.CreateMigration(s.stdSpec)
 	c.Assert(err, jc.ErrorIsNil)
 
-	mig2b, err := s.State2.LatestMigration()
+	migNextb, err := s.State2.LatestMigration()
 	c.Check(err, jc.ErrorIsNil)
-	c.Check(mig2b.Id(), gc.Equals, mig2.Id())
+	c.Check(migNextb.Id(), gc.Equals, migNext.Id())
+	phase, err := migNextb.Phase()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(phase, gc.Equals, migration.QUIESCE)
 }
 
 func (s *MigrationSuite) TestMigration(c *gc.C) {
@@ -816,7 +811,5 @@ func isMigrationActive(c *gc.C, st *state.State) bool {
 
 func checkIdAndAttempt(c *gc.C, mig state.ModelMigration, expected int) {
 	c.Check(mig.Id(), gc.Equals, fmt.Sprintf("%s:%d", mig.ModelUUID(), expected))
-	attempt, err := mig.Attempt()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Check(attempt, gc.Equals, expected)
+	c.Check(mig.Attempt(), gc.Equals, expected)
 }
