@@ -129,6 +129,7 @@ type MachineInfo struct {
 	AgentStatus              StatusInfo                        `json:"agent-status"`
 	InstanceStatus           StatusInfo                        `json:"instance-status"`
 	Life                     Life                              `json:"life"`
+	Config                   map[string]interface{}            `json:"config,omitempty"`
 	Series                   string                            `json:"series"`
 	SupportedContainers      []instance.ContainerType          `json:"supported-containers"`
 	SupportedContainersKnown bool                              `json:"supported-containers-known"`
@@ -201,15 +202,14 @@ func (i *ApplicationInfo) EntityId() EntityId {
 // RemoteApplicationInfo holds the information about a remote application that is
 // tracked by multiwatcherStore.
 type RemoteApplicationInfo struct {
-	ModelUUID      string
-	Name           string
-	ApplicationURL string
-	Life           Life
-	Status         StatusInfo
+	ModelUUID      string     `json:"model-uuid"`
+	Name           string     `json:"name"`
+	ApplicationURL string     `json:"application-url"`
+	Life           Life       `json:"life"`
+	Status         StatusInfo `json:"status"`
 }
 
-// EntityId returns a unique identifier for a remote application across
-// environments.
+// EntityId returns a unique identifier for a remote application across models.
 func (i *RemoteApplicationInfo) EntityId() EntityId {
 	return EntityId{
 		Kind:      "remoteApplication",
@@ -218,6 +218,25 @@ func (i *RemoteApplicationInfo) EntityId() EntityId {
 	}
 }
 
+// ApplicationOfferInfo holds the information about an application offer that is
+// tracked by multiwatcherStore.
+type ApplicationOfferInfo struct {
+	ModelUUID       string `json:"model-uuid"`
+	OfferName       string `json:"offer-name"`
+	ApplicationName string `json:"application-name"`
+	CharmName       string `json:"charm-name"`
+	ConnectedCount  int    `json:"connected-count"`
+}
+
+// EntityId returns a unique identifier for an application offer across models.
+func (i *ApplicationOfferInfo) EntityId() EntityId {
+	return EntityId{
+		Kind:      "applicationOffer",
+		ModelUUID: i.ModelUUID,
+		Id:        i.OfferName,
+	}
+}
+
 // Port identifies a network port number for a particular protocol.
 type Port struct {
 	Protocol string `json:"protocol"`
@@ -416,11 +435,14 @@ const (
 // ModelInfo holds the information about an model that is
 // tracked by multiwatcherStore.
 type ModelInfo struct {
-	ModelUUID      string `json:"model-uuid"`
-	Name           string `json:"name"`
-	Life           Life   `json:"life"`
-	Owner          string `json:"owner"`
-	ControllerUUID string `json:"controller-uuid"`
+	ModelUUID      string                 `json:"model-uuid"`
+	Name           string                 `json:"name"`
+	Life           Life                   `json:"life"`
+	Owner          string                 `json:"owner"`
+	ControllerUUID string                 `json:"controller-uuid"`
+	Config         map[string]interface{} `json:"config,omitempty"`
+	Status         StatusInfo             `json:"status"`
+	Constraints    constraints.Value      `json:"constraints"`
 }
 
 // EntityId returns a unique identifier for an model.
