@@ -5,12 +5,14 @@ package state
 
 import (
 	"fmt"
+	"runtime/pprof"
 	"strings"
 
 	"github.com/juju/errors"
 	"github.com/juju/utils"
 	"github.com/juju/utils/clock"
 	"gopkg.in/juju/names.v2"
+	worker "gopkg.in/juju/worker.v1"
 	"gopkg.in/mgo.v2"
 	"gopkg.in/mgo.v2/txn"
 
@@ -23,38 +25,93 @@ import (
 	"github.com/juju/juju/status"
 	"github.com/juju/juju/storage"
 	"github.com/juju/juju/storage/poolmanager"
-	"github.com/juju/juju/worker"
 )
 
-// Open connects to the server described by the given
-// info, waits for it to be initialized, and returns a new State
-// representing the model connected to.
-//
-// A policy may be provided, which will be used to validate and
-// modify behaviour of certain operations in state. A nil policy
-// may be provided.
+// Register the state tracker as a new profile.
+var profileTracker = pprof.NewProfile("juju/state/tracker")
+
+// OpenParams contains the parameters for opening the state database.
+type OpenParams struct {
+	// Clock is the clock used for time-related operations.
+	Clock clock.Clock
+
+	// ControllerTag is the tag of the controller.
+	ControllerTag names.ControllerTag
+
+	// ControllerModelTag is the tag of the controller model.
+	ControllerModelTag names.ModelTag
+
+	// MongoInfo is the mongo.MongoInfo used for dialling the
+	// Mongo connection.
+	MongoInfo *mongo.MongoInfo
+
+	// MongoDialOpts is the mongo.DialOpts used to control how
+	// Mongo connections are made.
+	MongoDialOpts mongo.DialOpts
+
+	// NewPolicy, if non-nil, returns a policy which will be used to
+	// validate and modify behaviour of certain operations in state.
+	NewPolicy NewPolicyFunc
+
+	// RunTransactionObserver, if non-nil, is a function that will
+	// be called after mgo/txn transactions are run, successfully
+	// or not.
+	RunTransactionObserver RunTransactionObserverFunc
+
+	// InitDatabaseFunc, if non-nil, is a function that will be called
+	// just after the state database is opened.
+	InitDatabaseFunc InitDatabaseFunc
+}
+
+// Validate validates the OpenParams.
+func (p OpenParams) Validate() error {
+	if p.Clock == nil {
+		return errors.NotValidf("nil Clock")
+	}
+	if p.ControllerTag == (names.ControllerTag{}) {
+		return errors.NotValidf("empty ControllerTag")
+	}
+	if p.ControllerModelTag == (names.ModelTag{}) {
+		return errors.NotValidf("empty ControllerModelTag")
+	}
+	if p.MongoInfo == nil {
+		return errors.NotValidf("nil MongoInfo")
+	}
+	return nil
+}
+
+// Open connects to the server with the given parameters, waits for it
+// to be initialized, and returns a new State representing the model
+// connected to.
 //
 // Open returns unauthorizedError if access is unauthorized.
-func Open(
-	controllerModelTag names.ModelTag,
-	controllerTag names.ControllerTag,
-	info *mongo.MongoInfo, opts mongo.DialOpts,
-	newPolicy NewPolicyFunc,
-) (*State, error) {
-	st, err := open(controllerModelTag, info, opts, newPolicy, clock.WallClock)
+func Open(args OpenParams) (*State, error) {
+	if err := args.Validate(); err != nil {
+		return nil, errors.Annotate(err, "validating args")
+	}
+	st, err := open(
+		args.ControllerModelTag,
+		args.MongoInfo,
+		args.MongoDialOpts,
+		args.InitDatabaseFunc,
+		nil,
+		args.NewPolicy,
+		args.Clock,
+		args.RunTransactionObserver,
+	)
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
 	if _, err := st.Model(); err != nil {
 		if err := st.Close(); err != nil {
-			logger.Errorf("closing State for %s: %v", controllerModelTag, err)
+			logger.Errorf("closing State for %s: %v", args.ControllerModelTag, err)
 		}
-		return nil, errors.Annotatef(err, "cannot read model %s", controllerModelTag.Id())
+		return nil, maybeUnauthorized(err, fmt.Sprintf("cannot read model %s", args.ControllerModelTag.Id()))
 	}
 
 	// State should only be Opened on behalf of a controller environ; all
 	// other *States should be created via ForModel.
-	if err := st.start(controllerTag); err != nil {
+	if err := st.start(args.ControllerTag); err != nil {
 		return nil, errors.Trace(err)
 	}
 	return st, nil
@@ -63,8 +120,11 @@ func Open(
 func open(
 	controllerModelTag names.ModelTag,
 	info *mongo.MongoInfo, opts mongo.DialOpts,
+	initDatabase InitDatabaseFunc,
+	controllerConfig *controller.Config,
 	newPolicy NewPolicyFunc,
 	clock clock.Clock,
+	runTransactionObserver RunTransactionObserverFunc,
 ) (*State, error) {
 	logger.Infof("opening state, mongo addresses: %q; entity %v", info.Addrs, info.Tag)
 	logger.Debugf("dialing mongo")
@@ -81,10 +141,18 @@ func open(
 	}
 	logger.Debugf("mongodb login successful")
 
-	st, err := newState(controllerModelTag, controllerModelTag, session, info, newPolicy, clock)
+	st, err := newState(controllerModelTag, controllerModelTag, session, info, newPolicy, clock, runTransactionObserver)
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
+	if initDatabase != nil {
+		if err := initDatabase(session, controllerModelTag.Id(), controllerConfig); err != nil {
+			session.Close()
+			return nil, errors.Trace(err)
+		}
+		logger.Debugf("mongodb initialised")
+	}
+
 	return st, nil
 }
 
@@ -113,10 +181,6 @@ type InitializeParams struct {
 	// the controller model.
 	ControllerModelArgs ModelArgs
 
-	// CloudName is the name of the cloud that the controller
-	// runs in.
-	CloudName string
-
 	// Cloud contains the properties of the cloud that the
 	// controller runs in.
 	Cloud cloud.Cloud
@@ -169,19 +233,13 @@ func (p InitializeParams) Validate() error {
 	if p.MongoInfo == nil {
 		return errors.NotValidf("nil MongoInfo")
 	}
-	if p.CloudName == "" {
-		return errors.NotValidf("empty CloudName")
-	}
-	if p.Cloud.Type == "" {
-		return errors.NotValidf("empty Cloud")
-	}
 	if err := validateCloud(p.Cloud); err != nil {
 		return errors.Annotate(err, "validating cloud")
 	}
-	if _, err := validateCloudRegion(p.Cloud, p.CloudName, p.ControllerModelArgs.CloudRegion); err != nil {
+	if _, err := validateCloudRegion(p.Cloud, p.ControllerModelArgs.CloudRegion); err != nil {
 		return errors.Annotate(err, "validating controller model cloud region")
 	}
-	if _, err := validateCloudCredentials(p.Cloud, p.CloudName, p.CloudCredentials); err != nil {
+	if _, err := validateCloudCredentials(p.Cloud, p.CloudCredentials); err != nil {
 		return errors.Annotate(err, "validating cloud credentials")
 	}
 	creds := make(map[string]cloud.Credential, len(p.CloudCredentials))
@@ -190,7 +248,6 @@ func (p InitializeParams) Validate() error {
 	}
 	if _, err := validateCloudCredential(
 		p.Cloud,
-		p.CloudName,
 		creds,
 		p.ControllerModelArgs.CloudCredential,
 	); err != nil {
@@ -199,6 +256,26 @@ func (p InitializeParams) Validate() error {
 	return nil
 }
 
+// InitDatabaseFunc defines a function used to
+// create the collections and indices in a Juju database.
+type InitDatabaseFunc func(*mgo.Session, string, *controller.Config) error
+
+// InitDatabase creates all the collections and indices in a Juju database.
+func InitDatabase(session *mgo.Session, modelUUID string, settings *controller.Config) error {
+	schema := allCollections()
+	err := schema.Create(
+		session.DB(jujuDB),
+		settings,
+	)
+	if err != nil {
+		return errors.Trace(err)
+	}
+	if err := InitDbLogs(session, modelUUID); err != nil {
+		return errors.Trace(err)
+	}
+	return nil
+}
+
 // Initialize sets up an initial empty state and returns it.
 // This needs to be performed only once for the initial controller model.
 // It returns unauthorizedError if access is unauthorized.
@@ -207,10 +284,14 @@ func Initialize(args InitializeParams) (_ *State, err error) {
 		return nil, errors.Annotate(err, "validating initialization args")
 	}
 
-	// When creating the controller model, the new model
-	// UUID is also used as the controller UUID.
 	modelTag := names.NewModelTag(args.ControllerModelArgs.Config.UUID())
-	st, err := open(modelTag, args.MongoInfo, args.MongoDialOpts, args.NewPolicy, args.Clock)
+	if !names.IsValidModel(modelTag.Id()) {
+		return nil, errors.New("invalid model UUID")
+	}
+
+	st, err := open(
+		modelTag, args.MongoInfo, args.MongoDialOpts,
+		InitDatabase, &args.ControllerConfig, args.NewPolicy, args.Clock, nil)
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
@@ -234,7 +315,7 @@ func Initialize(args InitializeParams) (_ *State, err error) {
 
 	logger.Infof("initializing controller model %s", modelTag.Id())
 
-	modelOps, err := st.modelSetupOps(
+	modelOps, modelStatusDoc, err := st.modelSetupOps(
 		args.ControllerConfig.ControllerUUID(),
 		args.ControllerModelArgs,
 		&lineage{
@@ -264,11 +345,11 @@ func Initialize(args InitializeParams) (_ *State, err error) {
 			Id:     modelGlobalKey,
 			Assert: txn.DocMissing,
 			Insert: &controllersDoc{
-				CloudName: args.CloudName,
+				CloudName: args.Cloud.Name,
 				ModelUUID: st.ModelUUID(),
 			},
 		},
-		createCloudOp(args.Cloud, args.CloudName),
+		createCloudOp(args.Cloud),
 		txn.Op{
 			C:      controllersC,
 			Id:     apiHostPortsKey,
@@ -294,7 +375,7 @@ func Initialize(args InitializeParams) (_ *State, err error) {
 		// Create an entry keyed on cloudname#<key>, value for each region in
 		// region-config. The values here are themselves
 		// map[string]interface{}.
-		ops = append(ops, createSettingsOp(globalSettingsC, regionSettingsGlobalKey(args.CloudName, k), v))
+		ops = append(ops, createSettingsOp(globalSettingsC, regionSettingsGlobalKey(args.Cloud.Name, k), v))
 	}
 
 	for tag, cred := range args.CloudCredentials {
@@ -309,6 +390,7 @@ func Initialize(args InitializeParams) (_ *State, err error) {
 	if err := st.start(controllerTag); err != nil {
 		return nil, errors.Trace(err)
 	}
+	probablyUpdateStatusHistory(st.db(), modelGlobalKey, modelStatusDoc)
 	return st, nil
 }
 
@@ -320,19 +402,20 @@ type lineage struct {
 }
 
 // modelSetupOps returns the transactions necessary to set up a model.
-func (st *State) modelSetupOps(controllerUUID string, args ModelArgs, inherited *lineage) ([]txn.Op, error) {
+func (st *State) modelSetupOps(controllerUUID string, args ModelArgs, inherited *lineage) ([]txn.Op, statusDoc, error) {
+	var modelStatusDoc statusDoc
 	if inherited != nil {
 		if err := checkControllerInheritedConfig(inherited.ControllerConfig); err != nil {
-			return nil, errors.Trace(err)
+			return nil, modelStatusDoc, errors.Trace(err)
 		}
 	}
 	if err := checkModelConfig(args.Config); err != nil {
-		return nil, errors.Trace(err)
+		return nil, modelStatusDoc, errors.Trace(err)
 	}
 
 	controllerModelUUID := st.controllerModelTag.Id()
 	modelUUID := args.Config.UUID()
-	modelStatusDoc := statusDoc{
+	modelStatusDoc = statusDoc{
 		ModelUUID: modelUUID,
 		Updated:   st.clock.Now().UnixNano(),
 		Status:    status.Available,
@@ -353,7 +436,7 @@ func (st *State) modelSetupOps(controllerUUID string, args ModelArgs, inherited
 	// Create the default storage pools for the model.
 	defaultStoragePoolsOps, err := st.createDefaultStoragePoolsOps(args.StorageProviderRegistry)
 	if err != nil {
-		return nil, errors.Trace(err)
+		return nil, modelStatusDoc, errors.Trace(err)
 	}
 	ops = append(ops, defaultStoragePoolsOps...)
 
@@ -387,7 +470,7 @@ func (st *State) modelSetupOps(controllerUUID string, args ModelArgs, inherited
 	}
 	modelCfg, err := composeModelConfigAttributes(args.Config.AllAttrs(), configSources...)
 	if err != nil {
-		return nil, errors.Trace(err)
+		return nil, modelStatusDoc, errors.Trace(err)
 	}
 	// Some values require marshalling before storage.
 	modelCfg = config.CoerceForStorage(modelCfg)
@@ -400,11 +483,12 @@ func (st *State) modelSetupOps(controllerUUID string, args ModelArgs, inherited
 			modelUUID, controllerUUID,
 			args.CloudName, args.CloudRegion, args.CloudCredential,
 			args.MigrationMode,
+			args.EnvironVersion,
 		),
 		createUniqueOwnerModelNameOp(args.Owner, args.Config.Name()),
 	)
 	ops = append(ops, modelUserOps...)
-	return ops, nil
+	return ops, modelStatusDoc, nil
 }
 
 func (st *State) createDefaultStoragePoolsOps(registry storage.ProviderRegistry) ([]txn.Op, error) {
@@ -457,6 +541,7 @@ func isUnauthorized(err error) bool {
 	}
 	if err, ok := err.(*mgo.QueryError); ok {
 		return err.Code == 10057 ||
+			err.Code == 13 ||
 			err.Message == "need to login" ||
 			err.Message == "unauthorized"
 	}
@@ -466,6 +551,7 @@ func isUnauthorized(err error) bool {
 // newState creates an incomplete *State, with no running workers or
 // controllerTag. You must start() the returned *State before it will
 // function correctly.
+// modelTag is used to filter all queries and transactions.
 //
 // newState takes responsibility for the supplied *mgo.Session, and will
 // close it if it cannot be returned under the aegis of a *State.
@@ -474,6 +560,7 @@ func newState(
 	session *mgo.Session, mongoInfo *mongo.MongoInfo,
 	newPolicy NewPolicyFunc,
 	clock clock.Clock,
+	runTransactionObserver RunTransactionObserverFunc,
 ) (_ *State, err error) {
 
 	defer func() {
@@ -482,29 +569,29 @@ func newState(
 		}
 	}()
 
-	// Set up database.
-	rawDB := session.DB(jujuDB)
-	database, err := allCollections().Load(rawDB, modelTag.Id())
-	if err != nil {
-		return nil, errors.Trace(err)
-	}
-	if err := InitDbLogs(session); err != nil {
-		return nil, errors.Trace(err)
+	db := &database{
+		raw:                    session.DB(jujuDB),
+		schema:                 allCollections(),
+		modelUUID:              modelTag.Id(),
+		runTransactionObserver: runTransactionObserver,
 	}
 
 	// Create State.
 	st := &State{
-		clock:              clock,
-		modelTag:           modelTag,
-		controllerModelTag: controllerModelTag,
-		mongoInfo:          mongoInfo,
-		session:            session,
-		database:           database,
-		newPolicy:          newPolicy,
+		clock:                  clock,
+		modelTag:               modelTag,
+		controllerModelTag:     controllerModelTag,
+		mongoInfo:              mongoInfo,
+		session:                session,
+		database:               db,
+		newPolicy:              newPolicy,
+		runTransactionObserver: runTransactionObserver,
 	}
 	if newPolicy != nil {
 		st.policy = newPolicy(st)
 	}
+	// Record this State instance with the global tracker.
+	profileTracker.Add(st, 1)
 	return st, nil
 }
 
@@ -522,35 +609,14 @@ func (st *State) CACert() string {
 func (st *State) Close() (err error) {
 	defer errors.DeferredAnnotatef(&err, "closing state failed")
 
-	var errs []error
-	handle := func(name string, err error) {
-		if err != nil {
-			errs = append(errs, errors.Annotatef(err, "error stopping %s", name))
-		}
-	}
 	if st.workers != nil {
-		handle("standard workers", worker.Stop(st.workers))
-	}
-
-	st.mu.Lock()
-	if st.allManager != nil {
-		handle("allwatcher manager", st.allManager.Stop())
-	}
-	if st.allModelManager != nil {
-		handle("allModelWatcher manager", st.allModelManager.Stop())
-	}
-	if st.allModelWatcherBacking != nil {
-		handle("allModelWatcher backing", st.allModelWatcherBacking.Release())
+		if err := worker.Stop(st.workers); err != nil {
+			return errors.Annotatef(err, "failed to stop workers")
+		}
 	}
 	st.session.Close()
-	st.mu.Unlock()
-
-	if len(errs) > 0 {
-		for _, err := range errs[1:] {
-			logger.Errorf("while closing state: %v", err)
-		}
-		return errs[0]
-	}
 	logger.Debugf("closed state without error")
+	// Remove the reference.
+	profileTracker.Remove(st)
 	return nil
 }
