@@ -47,11 +47,11 @@ func (s *PrecheckerSuite) SetUpTest(c *gc.C) {
 
 func (s *PrecheckerSuite) TestPrecheckInstance(c *gc.C) {
 	// PrecheckInstance should be called with the specified
-	// series and placement, and the specified constraints
+	// series and no placement, and the specified constraints
 	// merged with the model constraints, when attempting
 	// to create an instance.
 	envCons := constraints.MustParse("mem=4G")
-	placement := "abc123"
+	placement := ""
 	template, err := s.addOneMachine(c, envCons, placement)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(s.prechecker.precheckInstanceSeries, gc.Equals, template.Series)
@@ -62,6 +62,21 @@ func (s *PrecheckerSuite) TestPrecheckInstance(c *gc.C) {
 	c.Assert(s.prechecker.precheckInstanceConstraints, gc.DeepEquals, cons)
 }
 
+func (s *PrecheckerSuite) TestPrecheckInstanceWithPlacement(c *gc.C) {
+	// PrecheckInstance should be called with the specified
+	// series and placement. If placement is provided all
+	// model constraints should be ignored, otherwise they
+	// should be merged with provided constraints, when
+	// attempting to create an instance
+	envCons := constraints.MustParse("mem=4G")
+	placement := "abc123"
+	template, err := s.addOneMachine(c, envCons, placement)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(s.prechecker.precheckInstanceSeries, gc.Equals, template.Series)
+	c.Assert(s.prechecker.precheckInstancePlacement, gc.Equals, placement)
+	c.Assert(s.prechecker.precheckInstanceConstraints, gc.DeepEquals, template.Constraints)
+}
+
 func (s *PrecheckerSuite) TestPrecheckErrors(c *gc.C) {
 	// Ensure that AddOneMachine fails when PrecheckInstance returns an error.
 	s.prechecker.precheckInstanceError = fmt.Errorf("no instance for you")
