@@ -79,7 +79,7 @@ func removeRebootDocOp(st *State, machineId string) txn.Op {
 }
 
 func (m *Machine) clearFlag() error {
-	reboot, closer := m.st.getCollection(rebootC)
+	reboot, closer := m.st.db().GetCollection(rebootC)
 	defer closer()
 
 	docID := m.doc.DocID
@@ -107,7 +107,7 @@ func (m *Machine) SetRebootFlag(flag bool) error {
 
 // GetRebootFlag returns the reboot flag for this machine.
 func (m *Machine) GetRebootFlag() (bool, error) {
-	rebootCol, closer := m.st.getCollection(rebootC)
+	rebootCol, closer := m.st.db().GetCollection(rebootC)
 	defer closer()
 
 	count, err := rebootCol.FindId(m.doc.DocID).Count()
@@ -133,7 +133,7 @@ func (m *Machine) machinesToCareAboutRebootsFor() []string {
 // If we are a container, and our parent needs to reboot, this should return:
 // ShouldShutdown
 func (m *Machine) ShouldRebootOrShutdown() (RebootAction, error) {
-	rebootCol, closer := m.st.getCollection(rebootC)
+	rebootCol, closer := m.st.db().GetCollection(rebootC)
 	defer closer()
 
 	machines := m.machinesToCareAboutRebootsFor()
