@@ -10,11 +10,14 @@ import (
 
 	"github.com/juju/errors"
 	jujutxn "github.com/juju/txn"
+	"github.com/juju/utils/set"
 	"gopkg.in/juju/charm.v6-unstable"
 	"gopkg.in/juju/names.v2"
 	"gopkg.in/mgo.v2"
 	"gopkg.in/mgo.v2/bson"
 	"gopkg.in/mgo.v2/txn"
+
+	"github.com/juju/juju/network"
 )
 
 // RelationUnit holds information about a single unit in a relation, and
@@ -93,13 +96,13 @@ func (ru *RelationUnit) EnterScope(settings map[string]interface{}) error {
 			Id:     ru.unitName,
 			Assert: isAliveDoc,
 		})
+		ops = append(ops, txn.Op{
+			C:      relationsC,
+			Id:     relationDocID,
+			Assert: isAliveDoc,
+			Update: bson.D{{"$inc", bson.D{{"unitcount", 1}}}},
+		})
 	}
-	ops = append(ops, txn.Op{
-		C:      relationsC,
-		Id:     relationDocID,
-		Assert: isAliveDoc,
-		Update: bson.D{{"$inc", bson.D{{"unitcount", 1}}}},
-	})
 
 	// * Create the unit settings in this relation, if they do not already
 	//   exist; or completely overwrite them if they do. This must happen
@@ -114,7 +117,7 @@ func (ru *RelationUnit) EnterScope(settings map[string]interface{}) error {
 		ops = append(ops, createSettingsOp(settingsC, ruKey, settings))
 	} else {
 		var rop txn.Op
-		rop, settingsChanged, err = replaceSettingsOp(ru.st, settingsC, ruKey, settings)
+		rop, settingsChanged, err = replaceSettingsOp(ru.st.db(), settingsC, ruKey, settings)
 		if err != nil {
 			return err
 		}
@@ -204,7 +207,7 @@ func (ru *RelationUnit) EnterScope(settings map[string]interface{}) error {
 // exists and is Alive, its name will be returned as well; if one exists
 // but is not Alive, ErrCannotEnterScopeYet is returned.
 func (ru *RelationUnit) subordinateOps() ([]txn.Op, string, error) {
-	units, closer := ru.st.getCollection(unitsC)
+	units, closer := ru.st.db().GetCollection(unitsC)
 	defer closer()
 
 	if !ru.isPrincipal || ru.endpoint.Scope != charm.ScopeContainer {
@@ -243,7 +246,7 @@ func (ru *RelationUnit) subordinateOps() ([]txn.Op, string, error) {
 // but does not *actually* leave the scope, to avoid triggering relation
 // cleanup.
 func (ru *RelationUnit) PrepareLeaveScope() error {
-	relationScopes, closer := ru.st.getCollection(relationScopesC)
+	relationScopes, closer := ru.st.db().GetCollection(relationScopesC)
 	defer closer()
 
 	key := ru.key()
@@ -266,7 +269,7 @@ func (ru *RelationUnit) PrepareLeaveScope() error {
 // leaves, it is removed immediately. It is not an error to leave a scope
 // that the unit is not, or never was, a member of.
 func (ru *RelationUnit) LeaveScope() error {
-	relationScopes, closer := ru.st.getCollection(relationScopesC)
+	relationScopes, closer := ru.st.db().GetCollection(relationScopesC)
 	defer closer()
 
 	key := ru.key()
@@ -342,6 +345,62 @@ func (ru *RelationUnit) LeaveScope() error {
 	return nil
 }
 
+// Valid returns whether this RelationUnit is one that can actually
+// exist in the relation. For container-scoped relations, RUs can be
+// created for subordinate units whose principal unit isn't a member
+// of the relation. There are too many places that rely on being able
+// to construct a nonsensical RU to query InScope or Joined, so we
+// allow them to be constructed but they will always return false for
+// Valid.
+// TODO(babbageclunk): unpick the reliance on creating invalid RUs.
+func (ru *RelationUnit) Valid() (bool, error) {
+	if ru.endpoint.Scope != charm.ScopeContainer || ru.isPrincipal {
+		return true, nil
+	}
+	// A subordinate container-scoped relation unit is valid if:
+	// the other end of the relation is also a subordinate charm
+	// or its principal unit is also a member of the relation.
+	appName, err := names.UnitApplication(ru.unitName)
+	if err != nil {
+		return false, errors.Trace(err)
+	}
+	var otherAppName string
+	for _, ep := range ru.relation.Endpoints() {
+		if ep.ApplicationName != appName {
+			otherAppName = ep.ApplicationName
+		}
+	}
+	if otherAppName == "" {
+		return false, errors.Errorf("couldn't find other endpoint")
+	}
+	otherApp, err := ru.st.Application(otherAppName)
+	if err != nil {
+		return false, errors.Trace(err)
+	}
+	if !otherApp.IsPrincipal() {
+		return true, nil
+	}
+
+	unit, err := ru.st.Unit(ru.unitName)
+	if err != nil {
+		return false, errors.Trace(err)
+	}
+	// No need to check the flag here - we know we're subordinate.
+	pName, _ := unit.PrincipalName()
+	principalAppName, err := names.UnitApplication(pName)
+	if err != nil {
+		return false, errors.Trace(err)
+	}
+	// If the other application is a principal, only allow it if it's in the relation.
+	_, err = ru.relation.Endpoint(principalAppName)
+	if errors.IsNotFound(err) {
+		return false, nil
+	} else if err != nil {
+		return false, errors.Trace(err)
+	}
+	return true, nil
+}
+
 // InScope returns whether the relation unit has entered scope and not left it.
 func (ru *RelationUnit) InScope() (bool, error) {
 	return ru.inScope(nil)
@@ -356,7 +415,7 @@ func (ru *RelationUnit) Joined() (bool, error) {
 // inScope returns whether a scope document exists satisfying the supplied
 // selector.
 func (ru *RelationUnit) inScope(sel bson.D) (bool, error) {
-	relationScopes, closer := ru.st.getCollection(relationScopesC)
+	relationScopes, closer := ru.st.db().GetCollection(relationScopesC)
 	defer closer()
 
 	sel = append(sel, bson.D{{"_id", ru.key()}}...)
@@ -384,7 +443,7 @@ func watchRelationScope(
 // Settings returns a Settings which allows access to the unit's settings
 // within the relation.
 func (ru *RelationUnit) Settings() (*Settings, error) {
-	return readSettings(ru.st, settingsC, ru.key())
+	return readSettings(ru.st.db(), settingsC, ru.key())
 }
 
 // ReadSettings returns a map holding the settings of the unit with the
@@ -403,13 +462,66 @@ func (ru *RelationUnit) ReadSettings(uname string) (m map[string]interface{}, er
 	if err != nil {
 		return nil, err
 	}
-	node, err := readSettings(ru.st, settingsC, key)
+	node, err := readSettings(ru.st.db(), settingsC, key)
 	if err != nil {
 		return nil, err
 	}
 	return node.Map(), nil
 }
 
+// SettingsAddress returns the address that should be set as
+// `private-address` in the settings for the this unit in the context
+// of this relation. Generally this will be the address of the unit in
+// the space for the default binding, but if this is a cross-model
+// relation then it will be the public address. If this is cross-model
+// and there's no public address for the unit, return an error.
+func (ru *RelationUnit) SettingsAddress() (network.Address, error) {
+	unit, err := ru.st.Unit(ru.unitName)
+	if err != nil {
+		return network.Address{}, errors.Trace(err)
+	}
+	if crossmodel, err := ru.relation.IsCrossModel(); err != nil {
+		return network.Address{}, errors.Trace(err)
+	} else if !crossmodel {
+		space, err := unit.GetSpaceForBinding("")
+		if err != nil {
+			return network.Address{}, errors.Trace(err)
+		}
+		m, err := unit.machine()
+		if err != nil {
+			return network.Address{}, errors.Trace(err)
+		}
+		networkInfos := m.GetNetworkInfoForSpaces(set.NewStrings(space))
+		info, ok := networkInfos[space]
+		if !ok || len(info.NetworkInfos) == 0 || len(info.NetworkInfos[0].Addresses) == 0 {
+			privateAddress, err := unit.PrivateAddress()
+			if err == nil {
+				logger.Warningf("Can't find an address for default binding space %q, falling back to units' private address %q", space, privateAddress.String())
+			}
+			return privateAddress, err
+		}
+		boundAddress := network.NewAddress(info.NetworkInfos[0].Addresses[0].Address)
+		logger.Debugf("Found an address for a default binding for an app - %v", boundAddress.String())
+		return boundAddress, nil
+	}
+
+	address, err := unit.PublicAddress()
+	if err != nil {
+		// TODO(wallyworld) - it's ok to return a private address sometimes
+		// TODO return an error when it's not possible to use the private address
+		logger.Warningf("no public address available for unit %q in cross model relation %q , using private address", unit.Name(), ru.relation)
+		return unit.PrivateAddress()
+	}
+	if address.Scope != network.ScopePublic {
+		logger.Debugf(
+			"no public address for unit %q in cross-model relation %q",
+			unit.Name(),
+			ru.relation,
+		)
+	}
+	return address, nil
+}
+
 // unitKey returns a string, based on the relation and the supplied unit name,
 // which is used as a key for that unit within this relation in the settings,
 // presence, and relationScopes collections.
@@ -441,7 +553,7 @@ type relationScopeDoc struct {
 	DocID     string `bson:"_id"`
 	Key       string `bson:"key"`
 	ModelUUID string `bson:"model-uuid"`
-	Departing bool
+	Departing bool   `bson:"departing"`
 }
 
 func (d *relationScopeDoc) unitName() string {
