@@ -5,6 +5,7 @@ package state_test
 
 import (
 	"fmt"
+	"math/rand"
 	"sort"
 	"strconv"
 	"time"
@@ -76,7 +77,7 @@ func (s *RelationUnitSuite) TestReadSettingsErrors(c *gc.C) {
 }
 
 func (s *RelationUnitSuite) TestPeerSettings(c *gc.C) {
-	pr := NewPeerRelation(c, s.State)
+	pr := newPeerRelation(c, s.State)
 	rus := RUs{pr.ru0, pr.ru1}
 
 	// Check missing settings cannot be read by any RU.
@@ -150,20 +151,26 @@ func (s *RelationUnitSuite) TestPeerSettings(c *gc.C) {
 }
 
 func (s *RelationUnitSuite) TestRemoteUnitErrors(c *gc.C) {
-	_, err := s.State.AddRemoteApplication("mysql", "local:/u/me/mysql", []charm.Relation{{
-		Interface: "mysql",
-		Name:      "server",
-		Role:      charm.RoleProvider,
-		Scope:     charm.ScopeGlobal,
-	}})
+	_, err := s.State.AddRemoteApplication(state.AddRemoteApplicationParams{
+		Name:        "mysql",
+		SourceModel: coretesting.ModelTag,
+		Endpoints: []charm.Relation{{
+			Interface: "mysql",
+			Name:      "server",
+			Role:      charm.RoleProvider,
+			Scope:     charm.ScopeGlobal,
+		}}})
 	c.Assert(err, jc.ErrorIsNil)
 
-	_, err = s.State.AddRemoteApplication("mysql1", "local:/u/me/mysql", []charm.Relation{{
-		Interface: "mysql",
-		Name:      "server",
-		Role:      charm.RoleProvider,
-		Scope:     charm.ScopeGlobal,
-	}})
+	_, err = s.State.AddRemoteApplication(state.AddRemoteApplicationParams{
+		Name:        "mysql1",
+		SourceModel: coretesting.ModelTag,
+		Endpoints: []charm.Relation{{
+			Interface: "mysql",
+			Name:      "server",
+			Role:      charm.RoleProvider,
+			Scope:     charm.ScopeGlobal,
+		}}})
 	c.Assert(err, jc.ErrorIsNil)
 	s.AddTestingService(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
 
@@ -183,12 +190,12 @@ func (s *RelationUnitSuite) TestRemoteUnitErrors(c *gc.C) {
 }
 
 func (s *RelationUnitSuite) TestProReqSettings(c *gc.C) {
-	prr := NewProReqRelation(c, &s.ConnSuite, charm.ScopeGlobal)
+	prr := newProReqRelation(c, &s.ConnSuite, charm.ScopeGlobal)
 	s.testProReqSettings(c, prr.pru0, prr.pru1, prr.rru0, prr.rru1)
 }
 
 func (s *RelationUnitSuite) TestRemoteProReqSettings(c *gc.C) {
-	prr := NewRemoteProReqRelation(c, &s.ConnSuite)
+	prr := newRemoteProReqRelation(c, &s.ConnSuite)
 	s.testProReqSettings(c, prr.pru0, prr.pru1, prr.rru0, prr.rru1)
 }
 
@@ -222,7 +229,7 @@ func (s *RelationUnitSuite) testProReqSettings(c *gc.C, pru0, pru1, rru0, rru1 *
 }
 
 func (s *RelationUnitSuite) TestContainerSettings(c *gc.C) {
-	prr := NewProReqRelation(c, &s.ConnSuite, charm.ScopeContainer)
+	prr := newProReqRelation(c, &s.ConnSuite, charm.ScopeContainer)
 	rus := RUs{prr.pru0, prr.pru1, prr.rru0, prr.rru1}
 
 	// Check missing settings cannot be read by any RU.
@@ -335,7 +342,8 @@ func (s *RelationUnitSuite) TestContainerCreateSubordinate(c *gc.C) {
 }
 
 func (s *RelationUnitSuite) TestDestroyRelationWithUnitsInScope(c *gc.C) {
-	pr := NewPeerRelation(c, s.State)
+	pr := newPeerRelation(c, s.State)
+	preventPeerUnitsDestroyRemove(c, pr)
 	rel := pr.ru0.Relation()
 
 	// Enter two units, and check that Destroying the service sets the
@@ -348,7 +356,7 @@ func (s *RelationUnitSuite) TestDestroyRelationWithUnitsInScope(c *gc.C) {
 	err = pr.ru1.EnterScope(nil)
 	c.Assert(err, jc.ErrorIsNil)
 	assertJoined(c, pr.ru1)
-	err = pr.svc.Destroy()
+	err = pr.app.Destroy()
 	c.Assert(err, jc.ErrorIsNil)
 	err = rel.Refresh()
 	c.Assert(err, jc.ErrorIsNil)
@@ -369,7 +377,7 @@ func (s *RelationUnitSuite) TestDestroyRelationWithUnitsInScope(c *gc.C) {
 	err = pr.ru0.LeaveScope()
 	c.Assert(err, jc.ErrorIsNil)
 	assertNotInScope(c, pr.ru0)
-	err = pr.svc.Destroy()
+	err = pr.app.Destroy()
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Check that unit settings for the original unit still exist, and have
@@ -402,7 +410,7 @@ func (s *RelationUnitSuite) TestDestroyRelationWithUnitsInScope(c *gc.C) {
 }
 
 func (s *RelationUnitSuite) TestAliveRelationScope(c *gc.C) {
-	pr := NewPeerRelation(c, s.State)
+	pr := newPeerRelation(c, s.State)
 	rel := pr.ru0.Relation()
 
 	// Two units enter...
@@ -453,7 +461,7 @@ func (s *RelationUnitSuite) TestAliveRelationScope(c *gc.C) {
 
 func (s *StateSuite) TestWatchWatchScopeDiesOnStateClose(c *gc.C) {
 	testWatcherDiesWhenStateCloses(c, s.modelTag, s.State.ControllerTag(), func(c *gc.C, st *state.State) waiter {
-		pr := NewPeerRelation(c, st)
+		pr := newPeerRelation(c, st)
 		w := pr.ru0.WatchScope()
 		<-w.Changes()
 		return w
@@ -461,7 +469,7 @@ func (s *StateSuite) TestWatchWatchScopeDiesOnStateClose(c *gc.C) {
 }
 
 func (s *RelationUnitSuite) TestPeerWatchScope(c *gc.C) {
-	pr := NewPeerRelation(c, s.State)
+	pr := newPeerRelation(c, s.State)
 
 	// Test empty initial event.
 	w0 := pr.ru0.WatchScope()
@@ -522,12 +530,12 @@ func (s *RelationUnitSuite) TestPeerWatchScope(c *gc.C) {
 }
 
 func (s *RelationUnitSuite) TestProReqWatchScope(c *gc.C) {
-	prr := NewProReqRelation(c, &s.ConnSuite, charm.ScopeGlobal)
+	prr := newProReqRelation(c, &s.ConnSuite, charm.ScopeGlobal)
 	s.testProReqWatchScope(c, prr.pru0, prr.pru1, prr.rru0, prr.rru1, prr.watches)
 }
 
 func (s *RelationUnitSuite) TestRemoteProReqWatchScope(c *gc.C) {
-	prr := NewRemoteProReqRelation(c, &s.ConnSuite)
+	prr := newRemoteProReqRelation(c, &s.ConnSuite)
 	s.testProReqWatchScope(c, prr.pru0, prr.pru1, prr.rru0, prr.rru1, prr.watches)
 }
 
@@ -619,7 +627,7 @@ func (s *RelationUnitSuite) testProReqWatchScope(
 }
 
 func (s *RelationUnitSuite) TestContainerWatchScope(c *gc.C) {
-	prr := NewProReqRelation(c, &s.ConnSuite, charm.ScopeContainer)
+	prr := newProReqRelation(c, &s.ConnSuite, charm.ScopeContainer)
 
 	// Test empty initial events for all RUs.
 	ws := prr.watches()
@@ -689,7 +697,7 @@ func (s *RelationUnitSuite) TestContainerWatchScope(c *gc.C) {
 }
 
 func (s *RelationUnitSuite) TestCoalesceWatchScope(c *gc.C) {
-	pr := NewPeerRelation(c, s.State)
+	pr := newPeerRelation(c, s.State)
 
 	// Test empty initial event.
 	w0 := pr.ru0.WatchScope()
@@ -723,12 +731,12 @@ func (s *RelationUnitSuite) TestCoalesceWatchScope(c *gc.C) {
 }
 
 func (s *RelationUnitSuite) TestPrepareLeaveScope(c *gc.C) {
-	prr := NewProReqRelation(c, &s.ConnSuite, charm.ScopeGlobal)
+	prr := newProReqRelation(c, &s.ConnSuite, charm.ScopeGlobal)
 	s.testPrepareLeaveScope(c, prr.rel, prr.pru0, prr.pru1, prr.rru0, prr.rru1)
 }
 
 func (s *RelationUnitSuite) TestPrepareLeaveScopeRemote(c *gc.C) {
-	prr := NewRemoteProReqRelation(c, &s.ConnSuite)
+	prr := newRemoteProReqRelation(c, &s.ConnSuite)
 	s.testPrepareLeaveScope(c, prr.rel, prr.pru0, prr.pru1, prr.rru0, prr.rru1)
 }
 
@@ -804,20 +812,218 @@ func (s *RelationUnitSuite) assertNoScopeChange(c *gc.C, ws ...*state.RelationSc
 	}
 }
 
+func (s *RelationUnitSuite) TestSettingsAddress(c *gc.C) {
+	prr := newProReqRelation(c, &s.ConnSuite, charm.ScopeGlobal)
+	err := prr.pu0.AssignToNewMachine()
+	c.Assert(err, jc.ErrorIsNil)
+	id, err := prr.pu0.AssignedMachineId()
+	c.Assert(err, jc.ErrorIsNil)
+	machine, err := s.State.Machine(id)
+	c.Assert(err, jc.ErrorIsNil)
+
+	err = machine.SetProviderAddresses(
+		network.NewScopedAddress("1.2.3.4", network.ScopeCloudLocal),
+		network.NewScopedAddress("4.3.2.1", network.ScopePublic),
+	)
+
+	address, err := prr.pru0.SettingsAddress()
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Assert(address, gc.DeepEquals, network.NewAddress("1.2.3.4"))
+}
+
+func (s *RelationUnitSuite) addDevicesWithAddresses(c *gc.C, machine *state.Machine, addresses ...string) {
+	for _, address := range addresses {
+		name := fmt.Sprintf("e%x", rand.Int31())
+		deviceArgs := state.LinkLayerDeviceArgs{
+			Name: name,
+			Type: state.EthernetDevice,
+		}
+		err := machine.SetLinkLayerDevices(deviceArgs)
+		c.Assert(err, jc.ErrorIsNil)
+		device, err := machine.LinkLayerDevice(name)
+		c.Assert(err, jc.ErrorIsNil)
+
+		addressesArg := state.LinkLayerDeviceAddress{
+			DeviceName:   name,
+			ConfigMethod: state.StaticAddress,
+			CIDRAddress:  address,
+		}
+		err = machine.SetDevicesAddresses(addressesArg)
+		c.Assert(err, jc.ErrorIsNil)
+		deviceAddresses, err := device.Addresses()
+		c.Assert(err, jc.ErrorIsNil)
+		c.Assert(deviceAddresses, gc.HasLen, 1)
+	}
+}
+
+func (s *RelationUnitSuite) TestSettingsAddressWithSpaces(c *gc.C) {
+	s.State.AddSubnet(state.SubnetInfo{CIDR: "1.2.0.0/16"})
+	s.State.AddSpace("space-1", "pid-1", []string{"1.2.0.0/16"}, false)
+	s.State.AddSubnet(state.SubnetInfo{CIDR: "2.2.0.0/16"})
+	s.State.AddSpace("space-2", "pid-2", []string{"2.2.0.0/16"}, false)
+	s.State.AddSubnet(state.SubnetInfo{CIDR: "3.2.0.0/16"})
+	s.State.AddSpace("space-3", "pid-3", []string{"2.2.0.0/16"}, false)
+	s.State.AddSubnet(state.SubnetInfo{CIDR: "4.3.0.0/16"})
+	s.State.AddSpace("public-4", "pid-4", []string{"4.3.0.0/16"}, true)
+
+	// We want to have all bindings set so that no actual binding is
+	// really set to the default.
+	bindings := map[string]string{
+		"":             "space-3",
+		"server-admin": "space-1",
+		"server":       "space-2",
+	}
+
+	prr := newProReqRelationWithBindings(c, &s.ConnSuite, charm.ScopeGlobal, bindings, nil)
+	err := prr.pu0.AssignToNewMachine()
+	c.Assert(err, jc.ErrorIsNil)
+	id, err := prr.pu0.AssignedMachineId()
+	c.Assert(err, jc.ErrorIsNil)
+	machine, err := s.State.Machine(id)
+	c.Assert(err, jc.ErrorIsNil)
+
+	addresses := []network.Address{
+		network.NewScopedAddress("1.2.3.4", network.ScopeCloudLocal),
+		network.NewScopedAddress("2.2.3.4", network.ScopeCloudLocal),
+		network.NewScopedAddress("3.2.3.4", network.ScopeCloudLocal),
+		network.NewScopedAddress("4.3.2.1", network.ScopePublic),
+	}
+	err = machine.SetProviderAddresses(addresses...)
+	c.Assert(err, jc.ErrorIsNil)
+
+	s.addDevicesWithAddresses(c, machine, "1.2.3.4/16", "2.2.3.4/16", "3.2.3.4/16", "4.3.2.1/16")
+
+	address, err := prr.pru0.SettingsAddress()
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Assert(address, gc.DeepEquals, network.NewAddress("2.2.3.4"))
+}
+
+func (s *RelationUnitSuite) TestSettingsAddressRemoteRelation(c *gc.C) {
+	prr := newRemoteProReqRelation(c, &s.ConnSuite)
+	err := prr.ru0.AssignToNewMachine()
+	c.Assert(err, jc.ErrorIsNil)
+	id, err := prr.ru0.AssignedMachineId()
+	c.Assert(err, jc.ErrorIsNil)
+	machine, err := s.State.Machine(id)
+	c.Assert(err, jc.ErrorIsNil)
+
+	err = machine.SetProviderAddresses(
+		network.NewScopedAddress("1.2.3.4", network.ScopeCloudLocal),
+		network.NewScopedAddress("4.3.2.1", network.ScopePublic),
+	)
+
+	address, err := prr.rru0.SettingsAddress()
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Assert(address, gc.DeepEquals, network.NewScopedAddress("4.3.2.1", network.ScopePublic))
+}
+
+func (s *RelationUnitSuite) TestSettingsAddressRemoteRelationNoPublicAddr(c *gc.C) {
+	prr := newRemoteProReqRelation(c, &s.ConnSuite)
+	err := prr.ru0.AssignToNewMachine()
+	c.Assert(err, jc.ErrorIsNil)
+	id, err := prr.ru0.AssignedMachineId()
+	c.Assert(err, jc.ErrorIsNil)
+	machine, err := s.State.Machine(id)
+	c.Assert(err, jc.ErrorIsNil)
+
+	err = machine.SetProviderAddresses(
+		network.NewScopedAddress("1.2.3.4", network.ScopeCloudLocal),
+	)
+
+	address, err := prr.rru0.SettingsAddress()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(address, gc.DeepEquals, network.NewScopedAddress("1.2.3.4", network.ScopeCloudLocal))
+}
+
+func (s *RelationUnitSuite) TestValidYes(c *gc.C) {
+	prr := newProReqRelation(c, &s.ConnSuite, charm.ScopeContainer)
+	rus := []*state.RelationUnit{prr.pru0, prr.pru1, prr.rru0, prr.rru1}
+	for _, ru := range rus {
+		result, err := ru.Valid()
+		c.Assert(err, jc.ErrorIsNil)
+		c.Assert(result, jc.IsTrue)
+	}
+}
+
+func (s *RelationUnitSuite) TestValidNo(c *gc.C) {
+	mysqlLogging := newProReqRelation(c, &s.ConnSuite, charm.ScopeContainer)
+	wpApp := s.AddTestingService(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
+	wpLogging := newProReqRelationForApps(c, &s.ConnSuite, wpApp, mysqlLogging.rsvc)
+
+	// We have logging related to mysql and to wordpress. We can
+	// create an invalid RU by taking a logging unit from
+	// mysql-logging and getting the wp-logging RU for it.
+	ru, err := wpLogging.rel.Unit(mysqlLogging.ru0)
+	c.Assert(err, jc.ErrorIsNil)
+
+	result, err := ru.Valid()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(result, jc.IsFalse)
+}
+
+func (s *RelationUnitSuite) TestValidSubordinateToSubordinate(c *gc.C) {
+	// Relate mysql and logging...
+	mysqlLogging := newProReqRelation(c, &s.ConnSuite, charm.ScopeContainer)
+	monApp := s.AddTestingService(c, "monitoring", s.AddTestingCharm(c, "monitoring"))
+	// Relate mysql and monitoring - relation to a non-subordinate
+	// needed to trigger creation of monitoring unit.
+	mysqlMonitoring := newProReqRelationForApps(c, &s.ConnSuite, mysqlLogging.psvc, monApp)
+	// Monitor the logging app (newProReqRelationForApps assumes that
+	// the providing app is a principal, so we need to do it by hand).
+	loggingApp := mysqlLogging.rsvc
+
+	// Can't infer endpoints because they're ambiguous.
+	ep1, err := loggingApp.Endpoint("juju-info")
+	c.Assert(err, jc.ErrorIsNil)
+	ep2, err := monApp.Endpoint("info")
+	c.Assert(err, jc.ErrorIsNil)
+
+	rel, err := s.State.AddRelation(ep1, ep2)
+	c.Assert(err, jc.ErrorIsNil)
+	prr := &ProReqRelation{rel: rel, psvc: loggingApp, rsvc: monApp}
+
+	// The units already exist, create the relation units.
+	prr.pu0 = mysqlLogging.ru0
+	prr.pru0, err = rel.Unit(prr.pu0)
+	c.Assert(err, jc.ErrorIsNil)
+
+	prr.ru0 = mysqlMonitoring.ru0
+	prr.rru0, err = rel.Unit(prr.ru0)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Logging monitoring relation units should be valid.
+	res, err := prr.rru0.Valid()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(res, jc.IsTrue)
+	res, err = prr.pru0.Valid()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(res, jc.IsTrue)
+}
+
 type PeerRelation struct {
 	rel                *state.Relation
-	svc                *state.Application
+	app                *state.Application
 	u0, u1, u2, u3     *state.Unit
 	ru0, ru1, ru2, ru3 *state.RelationUnit
 }
 
-func NewPeerRelation(c *gc.C, st *state.State) *PeerRelation {
+func preventPeerUnitsDestroyRemove(c *gc.C, pr *PeerRelation) {
+	preventUnitDestroyRemove(c, pr.u0)
+	preventUnitDestroyRemove(c, pr.u1)
+	preventUnitDestroyRemove(c, pr.u2)
+	preventUnitDestroyRemove(c, pr.u3)
+}
+
+func newPeerRelation(c *gc.C, st *state.State) *PeerRelation {
 	svc := state.AddTestingService(c, st, "riak", state.AddTestingCharm(c, st, "riak"))
 	ep, err := svc.Endpoint("ring")
 	c.Assert(err, jc.ErrorIsNil)
 	rel, err := st.EndpointsRelation(ep)
 	c.Assert(err, jc.ErrorIsNil)
-	pr := &PeerRelation{rel: rel, svc: svc}
+	pr := &PeerRelation{rel: rel, app: svc}
 	pr.u0, pr.ru0 = addRU(c, svc, rel, nil)
 	pr.u1, pr.ru1 = addRU(c, svc, rel, nil)
 	pr.u2, pr.ru2 = addRU(c, svc, rel, nil)
@@ -832,27 +1038,49 @@ type ProReqRelation struct {
 	pru0, pru1, rru0, rru1 *state.RelationUnit
 }
 
-func NewProReqRelation(c *gc.C, s *ConnSuite, scope charm.RelationScope) *ProReqRelation {
-	psvc := s.AddTestingService(c, "mysql", s.AddTestingCharm(c, "mysql"))
-	var rsvc *state.Application
+func preventProReqUnitsDestroyRemove(c *gc.C, prr *ProReqRelation) {
+	preventUnitDestroyRemove(c, prr.pu0)
+	preventUnitDestroyRemove(c, prr.pu1)
+	preventUnitDestroyRemove(c, prr.ru0)
+	preventUnitDestroyRemove(c, prr.ru1)
+}
+
+func newProReqRelation(c *gc.C, s *ConnSuite, scope charm.RelationScope) *ProReqRelation {
+	papp := s.AddTestingService(c, "mysql", s.AddTestingCharm(c, "mysql"))
+	var rapp *state.Application
 	if scope == charm.ScopeGlobal {
-		rsvc = s.AddTestingService(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
+		rapp = s.AddTestingService(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
 	} else {
-		rsvc = s.AddTestingService(c, "logging", s.AddTestingCharm(c, "logging"))
+		rapp = s.AddTestingService(c, "logging", s.AddTestingCharm(c, "logging"))
 	}
-	eps, err := s.State.InferEndpoints("mysql", rsvc.Name())
+	return newProReqRelationForApps(c, s, papp, rapp)
+}
+
+func newProReqRelationWithBindings(c *gc.C, s *ConnSuite, scope charm.RelationScope, pbindings, rbindings map[string]string) *ProReqRelation {
+	papp := s.AddTestingServiceWithBindings(c, "mysql", s.AddTestingCharm(c, "mysql"), pbindings)
+	var rapp *state.Application
+	if scope == charm.ScopeGlobal {
+		rapp = s.AddTestingServiceWithBindings(c, "wordpress", s.AddTestingCharm(c, "wordpress"), rbindings)
+	} else {
+		rapp = s.AddTestingServiceWithBindings(c, "logging", s.AddTestingCharm(c, "logging"), rbindings)
+	}
+	return newProReqRelationForApps(c, s, papp, rapp)
+}
+
+func newProReqRelationForApps(c *gc.C, s *ConnSuite, proApp, reqApp *state.Application) *ProReqRelation {
+	eps, err := s.State.InferEndpoints(proApp.Name(), reqApp.Name())
 	c.Assert(err, jc.ErrorIsNil)
 	rel, err := s.State.AddRelation(eps...)
 	c.Assert(err, jc.ErrorIsNil)
-	prr := &ProReqRelation{rel: rel, psvc: psvc, rsvc: rsvc}
-	prr.pu0, prr.pru0 = addRU(c, psvc, rel, nil)
-	prr.pu1, prr.pru1 = addRU(c, psvc, rel, nil)
-	if scope == charm.ScopeGlobal {
-		prr.ru0, prr.rru0 = addRU(c, rsvc, rel, nil)
-		prr.ru1, prr.rru1 = addRU(c, rsvc, rel, nil)
+	prr := &ProReqRelation{rel: rel, psvc: proApp, rsvc: reqApp}
+	prr.pu0, prr.pru0 = addRU(c, proApp, rel, nil)
+	prr.pu1, prr.pru1 = addRU(c, proApp, rel, nil)
+	if eps[0].Scope == charm.ScopeGlobal {
+		prr.ru0, prr.rru0 = addRU(c, reqApp, rel, nil)
+		prr.ru1, prr.rru1 = addRU(c, reqApp, rel, nil)
 	} else {
-		prr.ru0, prr.rru0 = addRU(c, rsvc, rel, prr.pu0)
-		prr.ru1, prr.rru1 = addRU(c, rsvc, rel, prr.pu1)
+		prr.ru0, prr.rru0 = addRU(c, reqApp, rel, prr.pu0)
+		prr.ru1, prr.rru1 = addRU(c, reqApp, rel, prr.pu1)
 	}
 	return prr
 }
@@ -900,7 +1128,6 @@ func addRU(c *gc.C, svc *state.Application, rel *state.Relation, principal *stat
 		}
 		c.Assert(u, gc.NotNil)
 	}
-	preventUnitDestroyRemove(c, u)
 	ru, err := rel.Unit(u)
 	c.Assert(err, jc.ErrorIsNil)
 	return u, ru
@@ -911,15 +1138,19 @@ type RemoteProReqRelation struct {
 	psvc                   *state.RemoteApplication
 	rsvc                   *state.Application
 	pru0, pru1, rru0, rru1 *state.RelationUnit
+	ru0, ru1               *state.Unit
 }
 
-func NewRemoteProReqRelation(c *gc.C, s *ConnSuite) *RemoteProReqRelation {
-	psvc, err := s.State.AddRemoteApplication("mysql", "local:/u/me/mysql", []charm.Relation{{
-		Interface: "mysql",
-		Name:      "server",
-		Role:      charm.RoleProvider,
-		Scope:     charm.ScopeGlobal,
-	}})
+func newRemoteProReqRelation(c *gc.C, s *ConnSuite) *RemoteProReqRelation {
+	psvc, err := s.State.AddRemoteApplication(state.AddRemoteApplicationParams{
+		Name:        "mysql",
+		SourceModel: coretesting.ModelTag,
+		Endpoints: []charm.Relation{{
+			Interface: "mysql",
+			Name:      "server",
+			Role:      charm.RoleProvider,
+			Scope:     charm.ScopeGlobal,
+		}}})
 	c.Assert(err, jc.ErrorIsNil)
 	rsvc := s.AddTestingService(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
 
@@ -931,8 +1162,8 @@ func NewRemoteProReqRelation(c *gc.C, s *ConnSuite) *RemoteProReqRelation {
 	prr := &RemoteProReqRelation{rel: rel, psvc: psvc, rsvc: rsvc}
 	prr.pru0 = addRemoteRU(c, rel, "mysql/0")
 	prr.pru1 = addRemoteRU(c, rel, "mysql/1")
-	_, prr.rru0 = addRU(c, rsvc, rel, nil)
-	_, prr.rru1 = addRU(c, rsvc, rel, nil)
+	prr.ru0, prr.rru0 = addRU(c, rsvc, rel, nil)
+	prr.ru1, prr.rru1 = addRU(c, rsvc, rel, nil)
 	return prr
 }
 
@@ -1361,13 +1592,13 @@ func (s *WatchScopeSuite) TestProviderRequirerContainer(c *gc.C) {
 	// connections are in place.
 }
 
-type WatchCounterpartEndpointUnitsSuite struct {
+type WatchUnitsSuite struct {
 	ConnSuite
 }
 
-var _ = gc.Suite(&WatchCounterpartEndpointUnitsSuite{})
+var _ = gc.Suite(&WatchUnitsSuite{})
 
-func (s *WatchCounterpartEndpointUnitsSuite) TestProviderRequirerGlobal(c *gc.C) {
+func (s *WatchUnitsSuite) TestProviderRequirerGlobal(c *gc.C) {
 	// Create a pair of services and a relation between them.
 	mysql := s.AddTestingService(c, "mysql", s.AddTestingCharm(c, "mysql"))
 	mysqlEP, err := mysql.Endpoint("server")
@@ -1389,7 +1620,7 @@ func (s *WatchCounterpartEndpointUnitsSuite) TestProviderRequirerGlobal(c *gc.C)
 	mysql0 := addUnit(mysql)
 	wordpress0 := addUnit(wordpress)
 
-	wordpressWatcher, err := rel.WatchCounterpartEndpointUnits("mysql")
+	wordpressWatcher, err := rel.WatchUnits("wordpress")
 	c.Assert(err, jc.ErrorIsNil)
 	defer testing.AssertStop(c, wordpressWatcher)
 	wordpressWatcherC := testing.NewRelationUnitsWatcherC(c, s.State, wordpressWatcher)
@@ -1402,7 +1633,7 @@ func (s *WatchCounterpartEndpointUnitsSuite) TestProviderRequirerGlobal(c *gc.C)
 	c.Assert(err, jc.ErrorIsNil)
 	changeSettings(c, mysql0)
 
-	mysqlWatcher, err := rel.WatchCounterpartEndpointUnits("wordpress")
+	mysqlWatcher, err := rel.WatchUnits("mysql")
 	c.Assert(err, jc.ErrorIsNil)
 	defer testing.AssertStop(c, mysqlWatcher)
 	mysqlWatcherC := testing.NewRelationUnitsWatcherC(c, s.State, mysqlWatcher)
@@ -1419,7 +1650,7 @@ func (s *WatchCounterpartEndpointUnitsSuite) TestProviderRequirerGlobal(c *gc.C)
 	mysqlWatcherC.AssertNoChange()
 }
 
-func (s *WatchCounterpartEndpointUnitsSuite) TestProviderRequirerContainer(c *gc.C) {
+func (s *WatchUnitsSuite) TestProviderRequirerContainer(c *gc.C) {
 	// Create a pair of services and a relation between them.
 	mysql := s.AddTestingService(c, "mysql", s.AddTestingCharm(c, "mysql"))
 	mysqlEP, err := mysql.Endpoint("juju-info")
@@ -1430,9 +1661,9 @@ func (s *WatchCounterpartEndpointUnitsSuite) TestProviderRequirerContainer(c *gc
 	rel, err := s.State.AddRelation(mysqlEP, loggingEP)
 	c.Assert(err, jc.ErrorIsNil)
 
-	_, err = rel.WatchCounterpartEndpointUnits("mysql")
+	_, err = rel.WatchUnits("mysql")
 	c.Assert(err, gc.ErrorMatches, `"juju-info" endpoint is not globally scoped`)
-	_, err = rel.WatchCounterpartEndpointUnits("logging")
+	_, err = rel.WatchUnits("logging")
 	c.Assert(err, gc.ErrorMatches, `"info" endpoint is not globally scoped`)
 }
 
