@@ -12,14 +12,13 @@ import (
 	"sort"
 	"strconv"
 	"strings"
-	"sync"
-	"time"
 
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
 	jujutxn "github.com/juju/txn"
 	"github.com/juju/utils"
 	"github.com/juju/utils/clock"
+	"github.com/juju/utils/clock/monotonic"
 	"github.com/juju/utils/featureflag"
 	"github.com/juju/utils/os"
 	"github.com/juju/utils/series"
@@ -44,7 +43,8 @@ import (
 	"github.com/juju/juju/state/cloudimagemetadata"
 	stateaudit "github.com/juju/juju/state/internal/audit"
 	statelease "github.com/juju/juju/state/lease"
-	"github.com/juju/juju/state/workers"
+	"github.com/juju/juju/state/presence"
+	"github.com/juju/juju/state/watcher"
 	"github.com/juju/juju/status"
 	jujuversion "github.com/juju/juju/version"
 )
@@ -78,15 +78,16 @@ type providerIdDoc struct {
 // State represents the state of an model
 // managed by juju.
 type State struct {
-	clock              clock.Clock
-	modelTag           names.ModelTag
-	controllerModelTag names.ModelTag
-	controllerTag      names.ControllerTag
-	mongoInfo          *mongo.MongoInfo
-	session            *mgo.Session
-	database           Database
-	policy             Policy
-	newPolicy          NewPolicyFunc
+	clock                  clock.Clock
+	modelTag               names.ModelTag
+	controllerModelTag     names.ModelTag
+	controllerTag          names.ControllerTag
+	mongoInfo              *mongo.MongoInfo
+	session                *mgo.Session
+	database               Database
+	policy                 Policy
+	newPolicy              NewPolicyFunc
+	runTransactionObserver RunTransactionObserverFunc
 
 	// cloudName is the name of the cloud on which the model
 	// represented by this state runs.
@@ -101,16 +102,7 @@ type State struct {
 	// available by starting new ones as they fail. It doesn't do
 	// that yet, but having a type that collects them together is the
 	// first step.
-	//
-	// note that the allManager stuff below probably ought to be
-	// folded in as well, but that feels like its own task.
-	workers workers.Workers
-
-	// mu guards allManager, allModelManager & allModelWatcherBacking
-	mu                     sync.Mutex
-	allManager             *storeManager
-	allModelManager        *storeManager
-	allModelWatcherBacking Backing
+	workers *workers
 
 	// TODO(anastasiamac 2015-07-16) As state gets broken up, remove this.
 	CloudImageMetadataStorage cloudimagemetadata.Storage
@@ -147,6 +139,7 @@ func (st *State) IsController() bool {
 func (st *State) ControllerUUID() string {
 	return st.controllerTag.Id()
 }
+
 func (st *State) ControllerTag() names.ControllerTag {
 	return st.controllerTag
 }
@@ -222,6 +215,13 @@ func (st *State) removeAllModelDocs(modelAssertion bson.D) error {
 			}
 		}
 	}
+	// Logs and presence are in separate databases so don't get caught by that
+	// loop.
+	removeModelLogs(st.MongoSession(), modelUUID)
+	err := presence.RemovePresenceForModel(st.getPresenceCollection(), st.modelTag)
+	if err != nil {
+		return errors.Trace(err)
+	}
 
 	// Remove all user permissions for the model.
 	permPattern := bson.M{
@@ -237,15 +237,14 @@ func (st *State) removeAllModelDocs(modelAssertion bson.D) error {
 	}
 
 	// Now remove remove the model.
-	env, err := st.Model()
+	model, err := st.Model()
 	if err != nil {
 		return errors.Trace(err)
 	}
-	id := userModelNameIndex(env.Owner().Id(), env.Name())
 	ops = []txn.Op{{
 		// Cleanup the owner:envName unique key.
 		C:      usermodelnameC,
-		Id:     id,
+		Id:     model.uniqueIndexID(),
 		Remove: true,
 	}, {
 		C:      modelEntityRefsC,
@@ -266,7 +265,7 @@ func (st *State) removeAllModelDocs(modelAssertion bson.D) error {
 // removeAllInCollectionRaw removes all the documents from the given
 // named collection.
 func (st *State) removeAllInCollectionRaw(name string) error {
-	coll, closer := st.getCollection(name)
+	coll, closer := st.db().GetCollection(name)
 	defer closer()
 	_, err := coll.Writeable().RemoveAll(nil)
 	return errors.Trace(err)
@@ -281,7 +280,7 @@ func (st *State) removeAllInCollectionOps(name string) ([]txn.Op, error) {
 // removeInCollectionOps generates operations to remove all documents
 // from the named collection matching a specific selector.
 func (st *State) removeInCollectionOps(name string, sel interface{}) ([]txn.Op, error) {
-	coll, closer := st.getCollection(name)
+	coll, closer := st.db().GetCollection(name)
 	defer closer()
 
 	var ids []bson.M
@@ -306,6 +305,7 @@ func (st *State) ForModel(modelTag names.ModelTag) (*State, error) {
 	session := st.session.Copy()
 	newSt, err := newState(
 		modelTag, st.controllerModelTag, session, st.mongoInfo, st.newPolicy, st.clock,
+		st.runTransactionObserver,
 	)
 	if err != nil {
 		return nil, errors.Trace(err)
@@ -354,18 +354,9 @@ func (st *State) start(controllerTag names.ControllerTag) (err error) {
 	// now we've set up leaseClientId, we can use workersFactory
 
 	logger.Infof("starting standard state workers")
-	factory := workersFactory{
-		st:    st,
-		clock: st.clock,
-	}
-	workers, err := workers.NewRestartWorkers(workers.RestartConfig{
-		Factory: factory,
-		Logger:  loggo.GetLogger(logger.Name() + ".workers"),
-		Clock:   st.clock,
-		Delay:   time.Second,
-	})
+	workers, err := newWorkers(st)
 	if err != nil {
-		return errors.Annotatef(err, "cannot create standard state workers")
+		return errors.Trace(err)
 	}
 	st.workers = workers
 
@@ -416,11 +407,12 @@ func (st *State) ApplicationLeaders() (map[string]string, error) {
 
 func (st *State) getLeadershipLeaseClient() (lease.Client, error) {
 	client, err := statelease.NewClient(statelease.ClientConfig{
-		Id:         st.leaseClientId,
-		Namespace:  applicationLeadershipNamespace,
-		Collection: leasesC,
-		Mongo:      &environMongo{st},
-		Clock:      st.clock,
+		Id:           st.leaseClientId,
+		Namespace:    applicationLeadershipNamespace,
+		Collection:   leasesC,
+		Mongo:        &environMongo{st},
+		Clock:        st.clock,
+		MonotonicNow: monotonic.Now,
 	})
 	if err != nil {
 		return nil, errors.Annotatef(err, "cannot create leadership lease client")
@@ -430,11 +422,12 @@ func (st *State) getLeadershipLeaseClient() (lease.Client, error) {
 
 func (st *State) getSingularLeaseClient() (lease.Client, error) {
 	client, err := statelease.NewClient(statelease.ClientConfig{
-		Id:         st.leaseClientId,
-		Namespace:  singularControllerNamespace,
-		Collection: leasesC,
-		Mongo:      &environMongo{st},
-		Clock:      st.clock,
+		Id:           st.leaseClientId,
+		Namespace:    singularControllerNamespace,
+		Collection:   leasesC,
+		Mongo:        &environMongo{st},
+		Clock:        st.clock,
+		MonotonicNow: monotonic.Now,
 	})
 	if err != nil {
 		return nil, errors.Annotatef(err, "cannot create singular lease client")
@@ -469,7 +462,7 @@ func (st *State) EnsureModelRemoved() error {
 		if info.global {
 			continue
 		}
-		coll, closer := st.getCollection(name)
+		coll, closer := st.db().GetCollection(name)
 		defer closer()
 		n, err := coll.Find(nil).Count()
 		if err != nil {
@@ -501,6 +494,12 @@ func (st *State) getPresenceCollection() *mgo.Collection {
 	return st.session.DB(presenceDB).C(presenceC)
 }
 
+// getPingBatcher returns the implementation of how we serialize Ping requests
+// for agents to the database.
+func (st *State) getPingBatcher() *presence.PingBatcher {
+	return st.workers.pingBatcherWorker()
+}
+
 // getTxnLogCollection returns the raw mongodb txns collection, which is
 // needed to interact with the state/watcher package.
 func (st *State) getTxnLogCollection() *mgo.Collection {
@@ -515,6 +514,18 @@ func (st *State) newDB() (Database, func()) {
 	return st.database.Copy()
 }
 
+// db returns the Database instance used by the State. It is part of
+// the modelBackend interface.
+func (st *State) db() Database {
+	return st.database
+}
+
+// txnLogWatcher returns the TxnLogWatcher for the State. It is part
+// of the modelBackend interface.
+func (st *State) txnLogWatcher() *watcher.Watcher {
+	return st.workers.txnLogWatcher()
+}
+
 // Ping probes the state's database connection to ensure
 // that it is still alive.
 func (st *State) Ping() error {
@@ -538,23 +549,19 @@ func (st *State) MongoSession() *mgo.Session {
 	return st.session
 }
 
-func (st *State) Watch() *Multiwatcher {
-	st.mu.Lock()
-	if st.allManager == nil {
-		st.allManager = newStoreManager(newAllWatcherStateBacking(st))
-	}
-	st.mu.Unlock()
-	return NewMultiwatcher(st.allManager)
+// WatchParams defines config to control which
+// entites are included when watching a model.
+type WatchParams struct {
+	// IncludeOffers controls whether application offers should be watched.
+	IncludeOffers bool
 }
 
-func (st *State) WatchAllModels() *Multiwatcher {
-	st.mu.Lock()
-	if st.allModelManager == nil {
-		st.allModelWatcherBacking = NewAllModelWatcherStateBacking(st)
-		st.allModelManager = newStoreManager(st.allModelWatcherBacking)
-	}
-	st.mu.Unlock()
-	return NewMultiwatcher(st.allModelManager)
+func (st *State) Watch(params WatchParams) *Multiwatcher {
+	return NewMultiwatcher(st.workers.allManager(params))
+}
+
+func (st *State) WatchAllModels(pool *StatePool) *Multiwatcher {
+	return NewMultiwatcher(st.workers.allModelManager(pool))
 }
 
 // versionInconsistentError indicates one or more agents have a
@@ -602,7 +609,7 @@ func (st *State) checkCanUpgrade(currentVersion, newVersion string) error {
 	}}}
 	var agentTags []string
 	for _, name := range []string{machinesC, unitsC} {
-		collection, closer := st.getCollection(name)
+		collection, closer := st.db().GetCollection(name)
 		defer closer()
 		var doc struct {
 			DocID string `bson:"_id"`
@@ -643,7 +650,7 @@ func IsUpgradeInProgressError(err error) bool {
 // given version, only if the model is in a stable state (all agents are
 // running the current version). If this is a hosted model, newVersion
 // cannot be higher than the controller version.
-func (st *State) SetModelAgentVersion(newVersion version.Number) (err error) {
+func (st *State) SetModelAgentVersion(newVersion version.Number, ignoreAgentVersions bool) (err error) {
 	if newVersion.Compare(jujuversion.Current) > 0 && !st.IsController() {
 		return errors.Errorf("a hosted model cannot have a higher version than the server model: %s > %s",
 			newVersion.String(),
@@ -652,7 +659,7 @@ func (st *State) SetModelAgentVersion(newVersion version.Number) (err error) {
 	}
 
 	buildTxn := func(attempt int) ([]txn.Op, error) {
-		settings, err := readSettings(st, settingsC, modelGlobalKey)
+		settings, err := readSettings(st.db(), settingsC, modelGlobalKey)
 		if err != nil {
 			return nil, errors.Trace(err)
 		}
@@ -669,8 +676,10 @@ func (st *State) SetModelAgentVersion(newVersion version.Number) (err error) {
 			return nil, jujutxn.ErrNoOperations
 		}
 
-		if err := st.checkCanUpgrade(currentVersion, newVersion.String()); err != nil {
-			return nil, errors.Trace(err)
+		if !ignoreAgentVersions {
+			if err := st.checkCanUpgrade(currentVersion, newVersion.String()); err != nil {
+				return nil, errors.Trace(err)
+			}
 		}
 
 		ops := []txn.Op{
@@ -738,7 +747,7 @@ func (st *State) allMachines(machinesCollection mongo.Collection) ([]*Machine, e
 // AllMachines returns all machines in the model
 // ordered by id.
 func (st *State) AllMachines() ([]*Machine, error) {
-	machinesCollection, closer := st.getCollection(machinesC)
+	machinesCollection, closer := st.db().GetCollection(machinesC)
 	defer closer()
 	return st.allMachines(machinesCollection)
 }
@@ -746,7 +755,7 @@ func (st *State) AllMachines() ([]*Machine, error) {
 // AllMachinesFor returns all machines for the model represented
 // by the given modeluuid
 func (st *State) AllMachinesFor(modelUUID string) ([]*Machine, error) {
-	machinesCollection, closer := st.getCollectionFor(modelUUID, machinesC)
+	machinesCollection, closer := st.db().GetCollectionFor(modelUUID, machinesC)
 	defer closer()
 	return st.allMachines(machinesCollection)
 }
@@ -811,7 +820,7 @@ func (st *State) Machine(id string) (*Machine, error) {
 }
 
 func (st *State) getMachineDoc(id string) (*machineDoc, error) {
-	machinesCollection, closer := st.getCollection(machinesC)
+	machinesCollection, closer := st.db().GetCollection(machinesC)
 	defer closer()
 
 	var err error
@@ -985,6 +994,7 @@ type AddApplicationArgs struct {
 // they will be created automatically.
 func (st *State) AddApplication(args AddApplicationArgs) (_ *Application, err error) {
 	defer errors.DeferredAnnotatef(&err, "cannot add application %q", args.Name)
+
 	// Sanity checks.
 	if !names.IsValidApplication(args.Name) {
 		return nil, errors.Errorf("invalid name")
@@ -1138,6 +1148,11 @@ func (st *State) AddApplication(args AddApplicationArgs) (_ *Application, err er
 		NeverSet: true,
 	}
 
+	// When creating the settings, we ignore nils.  In other circumstances, nil
+	// means to delete the value (reset to default), so creating with nil should
+	// mean to use the default, i.e. don't set the value.
+	removeNils(args.Settings)
+
 	buildTxn := func(attempt int) ([]txn.Op, error) {
 		// If we've tried once already and failed, check that
 		// model may have been destroyed.
@@ -1166,7 +1181,7 @@ func (st *State) AddApplication(args AddApplicationArgs) (_ *Application, err er
 			assertModelActiveOp(st.ModelUUID()),
 			endpointBindingsOp,
 		}
-		addOps, err := addApplicationOps(st, addApplicationOpsArgs{
+		addOps, err := addApplicationOps(st, app, addApplicationOpsArgs{
 			applicationDoc: appDoc,
 			statusDoc:      statusDoc,
 			constraints:    args.Constraints,
@@ -1217,7 +1232,7 @@ func (st *State) AddApplication(args AddApplicationArgs) (_ *Application, err er
 		return ops, nil
 	}
 	// At the last moment before inserting the application, prime status history.
-	probablyUpdateStatusHistory(st, app.globalKey(), statusDoc)
+	probablyUpdateStatusHistory(st.db(), app.globalKey(), statusDoc)
 
 	if err = st.run(buildTxn); err == nil {
 		// Refresh to pick the txn-revno.
@@ -1229,6 +1244,15 @@ func (st *State) AddApplication(args AddApplicationArgs) (_ *Application, err er
 	return nil, errors.Trace(err)
 }
 
+// removeNils removes any keys with nil values from the given map.
+func removeNils(m map[string]interface{}) {
+	for k, v := range m {
+		if v == nil {
+			delete(m, k)
+		}
+	}
+}
+
 // assignUnitOps returns the db ops to save unit assignment for use by the
 // UnitAssigner worker.
 func assignUnitOps(unitName string, placement instance.Placement) []txn.Op {
@@ -1268,7 +1292,7 @@ func (st *State) AllUnitAssignments() ([]UnitAssignment, error) {
 }
 
 func (st *State) unitAssignments(query bson.D) ([]UnitAssignment, error) {
-	col, close := st.getCollection(assignUnitC)
+	col, close := st.db().GetCollection(assignUnitC)
 	defer close()
 
 	var docs []assignUnitDoc
@@ -1416,7 +1440,7 @@ func (st *State) addMachineWithPlacement(unit *Unit, placement *instance.Placeme
 
 // Application returns a application state by name.
 func (st *State) Application(name string) (_ *Application, err error) {
-	applications, closer := st.getCollection(applicationsC)
+	applications, closer := st.db().GetCollection(applicationsC)
 	defer closer()
 
 	if !names.IsValidApplication(name) {
@@ -1435,7 +1459,7 @@ func (st *State) Application(name string) (_ *Application, err error) {
 
 // AllApplications returns all deployed applications in the model.
 func (st *State) AllApplications() (applications []*Application, err error) {
-	applicationsCollection, closer := st.getCollection(applicationsC)
+	applicationsCollection, closer := st.db().GetCollection(applicationsC)
 	defer closer()
 
 	sdocs := []applicationDoc{}
@@ -1548,13 +1572,23 @@ func containerScopeOk(st *State, ep1, ep2 Endpoint) (bool, error) {
 }
 
 func applicationByName(st *State, name string) (ApplicationEntity, error) {
-	s, err := st.RemoteApplication(name)
+	app, err := st.Application(name)
 	if err == nil {
-		return s, nil
+		return app, nil
 	} else if err != nil && !errors.IsNotFound(err) {
 		return nil, err
+	} else if !featureflag.Enabled(feature.CrossModelRelations) {
+		return nil, err
 	}
-	return st.Application(name)
+	remoteApp, remoteErr := st.RemoteApplication(name)
+	if errors.IsNotFound(remoteErr) {
+		// We can't find either an application or a remote application
+		// by that name. Report the missing application, since that's
+		// probably what was intended (and still indicates the problem
+		// for remote applications).
+		return nil, err
+	}
+	return remoteApp, remoteErr
 }
 
 // endpoints returns all endpoints that could be intended by the
@@ -1649,7 +1683,7 @@ func (st *State) AddRelation(eps ...Endpoint) (r *Relation, err error) {
 		if exists, err := isNotDead(st, relationsC, key); err != nil {
 			return nil, errors.Trace(err)
 		} else if exists {
-			return nil, errors.Errorf("relation already exists")
+			return nil, errors.AlreadyExistsf("relation %v", key)
 		}
 		// Collect per-application operations, checking sanity as we go.
 		var ops []txn.Op
@@ -1749,7 +1783,7 @@ func (st *State) EndpointsRelation(endpoints ...Endpoint) (*Relation, error) {
 // KeyRelation returns the existing relation with the given key (which can
 // be derived unambiguously from the relation's endpoints).
 func (st *State) KeyRelation(key string) (*Relation, error) {
-	relations, closer := st.getCollection(relationsC)
+	relations, closer := st.db().GetCollection(relationsC)
 	defer closer()
 
 	doc := relationDoc{}
@@ -1765,7 +1799,7 @@ func (st *State) KeyRelation(key string) (*Relation, error) {
 
 // Relation returns the existing relation with the given id.
 func (st *State) Relation(id int) (*Relation, error) {
-	relations, closer := st.getCollection(relationsC)
+	relations, closer := st.db().GetCollection(relationsC)
 	defer closer()
 
 	doc := relationDoc{}
@@ -1781,7 +1815,7 @@ func (st *State) Relation(id int) (*Relation, error) {
 
 // AllRelations returns all relations in the model ordered by id.
 func (st *State) AllRelations() (relations []*Relation, err error) {
-	relationsCollection, closer := st.getCollection(relationsC)
+	relationsCollection, closer := st.db().GetCollection(relationsC)
 	defer closer()
 
 	docs := relationDocSlice{}
@@ -1809,7 +1843,7 @@ func (st *State) Unit(name string) (*Unit, error) {
 	if !names.IsValidUnit(name) {
 		return nil, errors.Errorf("%q is not a valid unit name", name)
 	}
-	units, closer := st.getCollection(unitsC)
+	units, closer := st.db().GetCollection(unitsC)
 	defer closer()
 
 	doc := unitDoc{}
@@ -1872,8 +1906,9 @@ func (st *State) AssignUnit(u *Unit, policy AssignmentPolicy) (err error) {
 // StartSync forces watchers to resynchronize their state with the
 // database immediately. This will happen periodically automatically.
 func (st *State) StartSync() {
-	st.workers.TxnLogWatcher().StartSync()
-	st.workers.PresenceWatcher().Sync()
+	st.workers.txnLogWatcher().StartSync()
+	st.workers.pingBatcherWorker().Sync()
+	st.workers.presenceWatcher().Sync()
 }
 
 // SetAdminMongoPassword sets the administrative password
@@ -1973,7 +2008,7 @@ const stateServingInfoKey = "stateServingInfo"
 
 // StateServingInfo returns information for running a controller machine
 func (st *State) StateServingInfo() (StateServingInfo, error) {
-	controllers, closer := st.getCollection(controllersC)
+	controllers, closer := st.db().GetCollection(controllersC)
 	defer closer()
 
 	var info StateServingInfo
@@ -2119,7 +2154,7 @@ func (st *State) networkEntityGlobalKey(globalKey string, providerId network.Id)
 // audit.AuditEntry instances to the database.
 func (st *State) PutAuditEntryFn() func(audit.AuditEntry) error {
 	insert := func(collectionName string, docs ...interface{}) error {
-		collection, closeCollection := st.getCollection(collectionName)
+		collection, closeCollection := st.db().GetCollection(collectionName)
 		defer closeCollection()
 
 		writeableCollection := collection.Writeable()
@@ -2129,6 +2164,51 @@ func (st *State) PutAuditEntryFn() func(audit.AuditEntry) error {
 	return stateaudit.PutAuditEntryFn(auditingC, insert)
 }
 
+// SetSLA sets the SLA on the current connected model.
+func (st *State) SetSLA(level, owner string, credentials []byte) error {
+	model, err := st.Model()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	return model.SetSLA(level, owner, credentials)
+}
+
+// SetModelMeterStatus sets the meter status for the current connected model.
+func (st *State) SetModelMeterStatus(status, info string) error {
+	model, err := st.Model()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	return model.SetMeterStatus(status, info)
+}
+
+// ModelMeterStatus returns the meter status for the current connected model.
+func (st *State) ModelMeterStatus() (MeterStatus, error) {
+	model, err := st.Model()
+	if err != nil {
+		return MeterStatus{MeterNotAvailable, ""}, errors.Trace(err)
+	}
+	return model.MeterStatus(), nil
+}
+
+// SLALevel returns the SLA level of the current connected model.
+func (st *State) SLALevel() (string, error) {
+	model, err := st.Model()
+	if err != nil {
+		return "", errors.Trace(err)
+	}
+	return model.SLALevel(), nil
+}
+
+// SLACredential returns the SLA credential of the current connected model.
+func (st *State) SLACredential() ([]byte, error) {
+	model, err := st.Model()
+	if err != nil {
+		return []byte{}, errors.Trace(err)
+	}
+	return model.SLACredential(), nil
+}
+
 var tagPrefix = map[byte]string{
 	'm': names.MachineTagKind + "-",
 	'a': names.ApplicationTagKind + "-",
@@ -2152,13 +2232,14 @@ func tagForGlobalKey(key string) (string, bool) {
 // to set the internal clock for the State instance. It is named such
 // that it should be obvious if it is ever called from a non-test package.
 func (st *State) SetClockForTesting(clock clock.Clock) error {
-	st.clock = clock
 	// Need to restart the lease workers so they get the new clock.
+	// Stop them first so they don't try to use it when we're setting it.
 	st.workers.Kill()
 	err := st.workers.Wait()
 	if err != nil {
 		return errors.Trace(err)
 	}
+	st.clock = clock
 	err = st.start(st.controllerTag)
 	if err != nil {
 		return errors.Trace(err)
