@@ -4,6 +4,7 @@
 package state_test
 
 import (
+	"fmt"
 	"strconv"
 	"time" // Only used for time types.
 
@@ -12,6 +13,7 @@ import (
 	jujutxn "github.com/juju/txn"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/charm.v6-unstable"
+	worker "gopkg.in/juju/worker.v1"
 
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/network"
@@ -19,7 +21,6 @@ import (
 	"github.com/juju/juju/state/testing"
 	"github.com/juju/juju/status"
 	coretesting "github.com/juju/juju/testing"
-	"github.com/juju/juju/worker"
 )
 
 const (
@@ -179,9 +180,12 @@ func (s *UnitSuite) addSubordinateUnit(c *gc.C) *state.Unit {
 }
 
 func (s *UnitSuite) setAssignedMachineAddresses(c *gc.C, u *state.Unit) {
-	err := u.AssignToNewMachine()
-	c.Assert(err, jc.ErrorIsNil)
 	mid, err := u.AssignedMachineId()
+	if errors.IsNotAssigned(err) {
+		err = u.AssignToNewMachine()
+		c.Assert(err, jc.ErrorIsNil)
+		mid, err = u.AssignedMachineId()
+	}
 	c.Assert(err, jc.ErrorIsNil)
 	machine, err := s.State.Machine(mid)
 	c.Assert(err, jc.ErrorIsNil)
@@ -683,13 +687,15 @@ func (s *UnitSuite) TestSetCharmURLRetriesWithDifferentURL(c *gc.C) {
 
 func (s *UnitSuite) TestDestroySetStatusRetry(c *gc.C) {
 	defer state.SetRetryHooks(c, s.State, func() {
+		err := s.unit.AssignToNewMachine()
+		c.Assert(err, jc.ErrorIsNil)
 		now := coretesting.NonZeroTime()
 		sInfo := status.StatusInfo{
 			Status:  status.Idle,
 			Message: "",
 			Since:   &now,
 		}
-		err := s.unit.SetAgentStatus(sInfo)
+		err = s.unit.SetAgentStatus(sInfo)
 		c.Assert(err, jc.ErrorIsNil)
 	}, func() {
 		assertLife(c, s.unit, state.Dying)
@@ -766,6 +772,36 @@ func (s *UnitSuite) TestDestroyUnassignRetry(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 }
 
+func (s *UnitSuite) TestDestroyAssignErrorRetry(c *gc.C) {
+	now := coretesting.NonZeroTime()
+	sInfo := status.StatusInfo{
+		Status:  status.Error,
+		Message: "failed to assign",
+		Since:   &now,
+	}
+	err := s.unit.SetAgentStatus(sInfo)
+	c.Assert(err, jc.ErrorIsNil)
+	_, err = s.unit.AssignedMachineId()
+	c.Assert(err, jc.Satisfies, errors.IsNotAssigned)
+
+	defer state.SetRetryHooks(c, s.State, func() {
+		err := s.unit.AssignToNewMachine()
+		c.Assert(err, jc.ErrorIsNil)
+		now := coretesting.NonZeroTime()
+		sInfo := status.StatusInfo{
+			Status:  status.Idle,
+			Message: "",
+			Since:   &now,
+		}
+		err = s.unit.SetAgentStatus(sInfo)
+		c.Assert(err, jc.ErrorIsNil)
+	}, func() {
+		assertLife(c, s.unit, state.Dying)
+	}).Check()
+	err = s.unit.Destroy()
+	c.Assert(err, jc.ErrorIsNil)
+}
+
 func (s *UnitSuite) TestShortCircuitDestroyUnit(c *gc.C) {
 	// A unit that has not set any status is removed directly.
 	err := s.unit.Destroy()
@@ -774,11 +810,46 @@ func (s *UnitSuite) TestShortCircuitDestroyUnit(c *gc.C) {
 	assertRemoved(c, s.unit)
 }
 
+func (s *UnitSuite) TestShortCircuitDestroyUnitNotAssigned(c *gc.C) {
+	// A unit that has not been assigned is removed directly.
+	now := coretesting.NonZeroTime()
+	err := s.unit.SetAgentStatus(status.StatusInfo{
+		Status:  status.Error,
+		Message: "cannot assign",
+		Since:   &now,
+	})
+	c.Assert(err, jc.ErrorIsNil)
+	err = s.unit.Destroy()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(s.unit.Life(), gc.Equals, state.Dying)
+	assertRemoved(c, s.unit)
+}
+
+func (s *UnitSuite) TestCannotShortCircuitDestroyAssignedUnit(c *gc.C) {
+	// This test is similar to TestShortCircuitDestroyUnitNotAssigned but
+	// the unit is assigned to a machine.
+	err := s.unit.AssignToNewMachine()
+	c.Assert(err, jc.ErrorIsNil)
+	now := coretesting.NonZeroTime()
+	err = s.unit.SetAgentStatus(status.StatusInfo{
+		Status:  status.Error,
+		Message: "some error",
+		Since:   &now,
+	})
+	c.Assert(err, jc.ErrorIsNil)
+	err = s.unit.Destroy()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(s.unit.Life(), gc.Equals, state.Dying)
+	assertLife(c, s.unit, state.Dying)
+}
+
 func (s *UnitSuite) TestCannotShortCircuitDestroyWithSubordinates(c *gc.C) {
 	// A unit with subordinates is just set to Dying.
 	s.AddTestingService(c, "logging", s.AddTestingCharm(c, "logging"))
 	eps, err := s.State.InferEndpoints("logging", "wordpress")
 	c.Assert(err, jc.ErrorIsNil)
+	err = s.unit.AssignToNewMachine()
+	c.Assert(err, jc.ErrorIsNil)
 	rel, err := s.State.AddRelation(eps...)
 	c.Assert(err, jc.ErrorIsNil)
 	ru, err := rel.Unit(s.unit)
@@ -807,6 +878,8 @@ func (s *UnitSuite) TestCannotShortCircuitDestroyWithAgentStatus(c *gc.C) {
 		c.Logf("test %d: %s", i, test.status)
 		unit, err := s.service.AddUnit()
 		c.Assert(err, jc.ErrorIsNil)
+		err = unit.AssignToNewMachine()
+		c.Assert(err, jc.ErrorIsNil)
 		now := coretesting.NonZeroTime()
 		sInfo := status.StatusInfo{
 			Status:  test.status,
@@ -839,6 +912,54 @@ func (s *UnitSuite) TestShortCircuitDestroyWithProvisionedMachine(c *gc.C) {
 	assertRemoved(c, s.unit)
 }
 
+func (s *UnitSuite) TestDestroyRemovesStatusHistory(c *gc.C) {
+	err := s.unit.AssignToNewMachine()
+	c.Assert(err, jc.ErrorIsNil)
+	now := coretesting.NonZeroTime()
+	for i := 0; i < 10; i++ {
+		info := status.StatusInfo{
+			Status:  status.Executing,
+			Message: fmt.Sprintf("status %d", i),
+			Since:   &now,
+		}
+		err := s.unit.SetAgentStatus(info)
+		c.Assert(err, jc.ErrorIsNil)
+		info.Status = status.Active
+		err = s.unit.SetStatus(info)
+		c.Assert(err, jc.ErrorIsNil)
+
+		err = s.unit.SetWorkloadVersion(fmt.Sprintf("v.%d", i))
+	}
+
+	filter := status.StatusHistoryFilter{Size: 100}
+	agentInfo, err := s.unit.AgentHistory().StatusHistory(filter)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(len(agentInfo), jc.GreaterThan, 9)
+
+	workloadInfo, err := s.unit.StatusHistory(filter)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(len(workloadInfo), jc.GreaterThan, 9)
+
+	versionInfo, err := s.unit.WorkloadVersionHistory().StatusHistory(filter)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(len(versionInfo), jc.GreaterThan, 9)
+
+	err = s.unit.Destroy()
+	c.Assert(err, jc.ErrorIsNil)
+
+	agentInfo, err = s.unit.AgentHistory().StatusHistory(filter)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(agentInfo, gc.HasLen, 0)
+
+	workloadInfo, err = s.unit.StatusHistory(filter)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(workloadInfo, gc.HasLen, 0)
+
+	versionInfo, err = s.unit.WorkloadVersionHistory().StatusHistory(filter)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(versionInfo, gc.HasLen, 0)
+}
+
 func assertLife(c *gc.C, entity state.Living, life state.Life) {
 	c.Assert(entity.Refresh(), gc.IsNil)
 	c.Assert(entity.Life(), gc.Equals, life)
@@ -853,7 +974,11 @@ func assertRemoved(c *gc.C, entity state.Living) {
 		err = entity.EnsureDead()
 		c.Assert(err, jc.ErrorIsNil)
 		err = entity.Remove()
-		c.Assert(err, jc.ErrorIsNil)
+		if err != nil {
+			c.Assert(err, gc.ErrorMatches, ".*already removed.*")
+		}
+		err = entity.Refresh()
+		c.Assert(err, jc.Satisfies, errors.IsNotFound)
 	}
 }
 
@@ -1602,7 +1727,6 @@ func (s *UnitSuite) TestWatchSubordinates(c *gc.C) {
 }
 
 func (s *UnitSuite) TestWatchUnit(c *gc.C) {
-	preventUnitDestroyRemove(c, s.unit)
 	w := s.unit.Watch()
 	defer testing.AssertStop(c, w)
 
@@ -1619,6 +1743,7 @@ func (s *UnitSuite) TestWatchUnit(c *gc.C) {
 	// Make two changes, check one event.
 	err = unit.SetPassword("arble-farble-dying-yarble")
 	c.Assert(err, jc.ErrorIsNil)
+	preventUnitDestroyRemove(c, unit)
 	err = unit.Destroy()
 	c.Assert(err, jc.ErrorIsNil)
 	wc.AssertOneChange()
@@ -1685,17 +1810,19 @@ snapshot:
 		},
 		{
 			actionName: "juju-run",
-			errString:  `validation failed: (root) : "command" property is missing and required, given {}; (root) : "timeout" property is missing and required, given {}`,
+			errString:  `validation failed: \(root\) : "command" property is missing and required, given \{\}; \(root\) : "timeout" property is missing and required, given \{\}`,
 		},
 		{
 			actionName:   "juju-run",
 			givenPayload: map[string]interface{}{"command": "allyourbasearebelongtous"},
-			errString:    `validation failed: (root) : "timeout" property is missing and required, given {"command":"allyourbasearebelongtous"}`,
+			errString:    `validation failed: \(root\) : "timeout" property is missing and required, given \{"command":"allyourbasearebelongtous"\}`,
 		},
 		{
 			actionName:   "juju-run",
 			givenPayload: map[string]interface{}{"timeout": 5 * time.Second},
-			errString:    `validation failed: (root) : "command" property is missing and required, given {"timeout":5e+09}`,
+			// Note: in Go 1.8 the representation of large numbers in JSON changed
+			// to use integer rather than exponential notation, hence the pattern.
+			errString: `validation failed: \(root\) : "command" property is missing and required, given \{"timeout":5.*\}`,
 		},
 		{
 			actionName:      "juju-run",
@@ -1712,8 +1839,7 @@ snapshot:
 		c.Logf("running test %d", i)
 		action, err := unit1.AddAction(t.actionName, t.givenPayload)
 		if t.errString != "" {
-			c.Assert(err.Error(), gc.Equals, t.errString)
-			continue
+			c.Assert(err, gc.ErrorMatches, t.errString)
 		} else {
 			c.Assert(err, jc.ErrorIsNil)
 			c.Assert(action.Parameters(), jc.DeepEquals, t.expectedPayload)
