@@ -22,7 +22,7 @@ import (
 // Volume describes a volume (disk, logical volume, etc.) in the model.
 type Volume interface {
 	GlobalEntity
-	LifeBinder
+	Lifer
 	status.StatusGetter
 	status.StatusSetter
 
@@ -88,9 +88,14 @@ type volumeDoc struct {
 	Life            Life          `bson:"life"`
 	StorageId       string        `bson:"storageid,omitempty"`
 	AttachmentCount int           `bson:"attachmentcount"`
-	Binding         string        `bson:"binding,omitempty"`
 	Info            *VolumeInfo   `bson:"info,omitempty"`
 	Params          *VolumeParams `bson:"params,omitempty"`
+
+	// MachineId is the ID of the machine that a non-detachable
+	// volume is initially attached to. We use this to identify
+	// the volume as being non-detachable, and to determine
+	// which volumes must be removed along with said machine.
+	MachineId string `bson:"machineid,omitempty"`
 }
 
 // volumeAttachmentDoc records information about a volume attachment.
@@ -111,10 +116,6 @@ type VolumeParams struct {
 	// that the volume is to be assigned to.
 	storage names.StorageTag
 
-	// binding, if non-nil, is the tag of the entity to which
-	// the volume's lifecycle will be bound.
-	binding names.Tag
-
 	Pool string `bson:"pool"`
 	Size uint64 `bson:"size"`
 }
@@ -122,6 +123,7 @@ type VolumeParams struct {
 // VolumeInfo describes information about a volume.
 type VolumeInfo struct {
 	HardwareId string `bson:"hardwareid,omitempty"`
+	WWN        string `bson:"wwn,omitempty"`
 	Size       uint64 `bson:"size"`
 	Pool       string `bson:"pool"`
 	VolumeId   string `bson:"volumeid"`
@@ -144,22 +146,6 @@ type VolumeAttachmentParams struct {
 
 // validate validates the contents of the volume document.
 func (v *volume) validate() error {
-	if v.doc.Binding != "" {
-		tag, err := names.ParseTag(v.doc.Binding)
-		if err != nil {
-			return errors.Annotate(err, "parsing binding")
-		}
-		switch tag.(type) {
-		case names.ModelTag:
-			// TODO(axw) support binding to model
-			return errors.NotSupportedf("binding to model")
-		case names.MachineTag:
-		case names.FilesystemTag:
-		case names.StorageTag:
-		default:
-			return errors.Errorf("invalid binding: %v", v.doc.Binding)
-		}
-	}
 	return nil
 }
 
@@ -183,34 +169,6 @@ func (v *volume) Life() Life {
 	return v.doc.Life
 }
 
-// LifeBinding is required to implement LifeBinder.
-//
-// Below is the set of possible entity types that a volume may be bound
-// to, and a description of the effects of doing so:
-//
-//   Machine:     If the volume is bound to a machine, then the volume
-//                will be destroyed when it is detached from the
-//                machine. It is not permitted for a volume to be
-//                attached to multiple machines while it is bound to a
-//                machine.
-//   Storage:     If the volume is bound to a storage instance, then
-//                the volume will be destroyed when the storage insance
-//                is removed from state.
-//   Filesystem:  If the volume is bound to a filesystem, i.e. the
-//                volume backs that filesystem, then it will be
-//                destroyed when the filesystem is removed from state.
-//   Model: If the volume is bound to the model, then the
-//                volume must be destroyed prior to the model
-//                being destroyed.
-func (v *volume) LifeBinding() names.Tag {
-	if v.doc.Binding == "" {
-		return nil
-	}
-	// Tag is validated in volume.validate.
-	tag, _ := names.ParseTag(v.doc.Binding)
-	return tag
-}
-
 // StorageInstance is required to implement Volume.
 func (v *volume) StorageInstance() (names.StorageTag, error) {
 	if v.doc.StorageId == "" {
@@ -284,7 +242,7 @@ func (st *State) Volume(tag names.VolumeTag) (Volume, error) {
 }
 
 func (st *State) volumes(query interface{}) ([]*volume, error) {
-	coll, cleanup := st.getCollection(volumesC)
+	coll, cleanup := st.db().GetCollection(volumesC)
 	defer cleanup()
 
 	var docs []volumeDoc
@@ -345,7 +303,7 @@ func (st *State) StorageInstanceVolume(tag names.StorageTag) (Volume, error) {
 // VolumeAttachment returns the VolumeAttachment corresponding to
 // the specified volume and machine.
 func (st *State) VolumeAttachment(machine names.MachineTag, volume names.VolumeTag) (VolumeAttachment, error) {
-	coll, cleanup := st.getCollection(volumeAttachmentsC)
+	coll, cleanup := st.db().GetCollection(volumeAttachmentsC)
 	defer cleanup()
 
 	var att volumeAttachment
@@ -379,7 +337,7 @@ func (st *State) VolumeAttachments(volume names.VolumeTag) ([]VolumeAttachment,
 }
 
 func (st *State) volumeAttachments(query bson.D) ([]VolumeAttachment, error) {
-	coll, cleanup := st.getCollection(volumeAttachmentsC)
+	coll, cleanup := st.db().GetCollection(volumeAttachmentsC)
 	defer cleanup()
 
 	var docs []volumeAttachmentDoc
@@ -406,85 +364,114 @@ func IsContainsFilesystem(err error) bool {
 }
 
 // removeMachineVolumesOps returns txn.Ops to remove non-persistent volumes
-// attached to the specified machine. This is used when the given machine is
-// being removed from state.
-func (st *State) removeMachineVolumesOps(machine names.MachineTag) ([]txn.Op, error) {
-	attachments, err := st.MachineVolumeAttachments(machine)
+// bound or attached to the specified machine. This is used when the given
+// machine is being removed from state.
+func (st *State) removeMachineVolumesOps(m *Machine) ([]txn.Op, error) {
+	// A machine cannot transition to Dead if it has any detachable storage
+	// attached, so any attachments are for machine-bound storage.
+	//
+	// Even if a volume is "non-detachable", there still exist volume
+	// attachments, and they may be removed independently of the volume.
+	// For example, the user may request that the volume be destroyed.
+	// This will cause the volume to become Dying, and the attachment to
+	// be Dying, then Dead, and finally removed. Only once the attachment
+	// is removed will the volume transition to Dead and then be removed.
+	// Therefore, there may be volumes that are bound, but not attached,
+	// to the machine.
+
+	machineVolumes, err := st.volumes(bson.D{{"machineid", m.Id()}})
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
-	ops := make([]txn.Op, 0, len(attachments))
-	for _, a := range attachments {
-		volumeTag := a.Volume()
-		// When removing the machine, there should only remain
-		// non-persistent storage. This will be implicitly
-		// removed when the machine is removed, so we do not
-		// use removeVolumeAttachmentOps or removeVolumeOps,
-		// which track and update related documents.
+	ops := make([]txn.Op, 0, 2*len(machineVolumes)+len(m.doc.Volumes))
+	for _, volumeId := range m.doc.Volumes {
 		ops = append(ops, txn.Op{
 			C:      volumeAttachmentsC,
-			Id:     volumeAttachmentId(machine.Id(), volumeTag.Id()),
+			Id:     volumeAttachmentId(m.Id(), volumeId),
 			Assert: txn.DocExists,
 			Remove: true,
 		})
-		canRemove, err := isVolumeInherentlyMachineBound(st, volumeTag)
-		if err != nil {
-			return nil, errors.Trace(err)
+	}
+	for _, v := range machineVolumes {
+		volumeId := v.Tag().Id()
+		if v.doc.StorageId != "" {
+			// The volume is assigned to a storage instance;
+			// make sure we also remove the storage instance.
+			// There should be no storage attachments remaining,
+			// as the units must have been removed before the
+			// machine can be; and the storage attachments must
+			// have been removed before the unit can be.
+			ops = append(ops,
+				txn.Op{
+					C:      storageInstancesC,
+					Id:     v.doc.StorageId,
+					Assert: txn.DocExists,
+					Remove: true,
+				},
+			)
 		}
-		if !canRemove {
-			return nil, errors.Errorf("machine has non-machine bound volume %v", volumeTag.Id())
-		}
-		ops = append(ops, txn.Op{
-			C:      volumesC,
-			Id:     volumeTag.Id(),
-			Assert: txn.DocExists,
-			Remove: true,
-		})
+		ops = append(ops,
+			txn.Op{
+				C:      volumesC,
+				Id:     volumeId,
+				Assert: txn.DocExists,
+				Remove: true,
+			},
+			removeModelVolumeRefOp(st, volumeId),
+		)
 	}
 	return ops, nil
 }
 
-// isVolumeInherentlyMachineBound reports whether or not the volume with the
-// specified tag is inherently bound to the lifetime of the machine, and will
-// be removed along with it, leaving no resources dangling.
-func isVolumeInherentlyMachineBound(st *State, tag names.VolumeTag) (bool, error) {
-	volume, err := st.Volume(tag)
+// isDetachableVolumeTag reports whether or not the volume with the specified
+// tag is detachable.
+func isDetachableVolumeTag(st *State, tag names.VolumeTag) (bool, error) {
+	volume, err := st.volumeByTag(tag)
 	if err != nil {
 		return false, errors.Trace(err)
 	}
-	volumeInfo, err := volume.Info()
-	if errors.IsNotProvisioned(err) {
-		params, _ := volume.Params()
-		_, provider, err := poolStorageProvider(st, params.Pool)
-		if err != nil {
-			return false, errors.Trace(err)
-		}
-		if provider.Scope() == storage.ScopeMachine {
-			// Any storage created by a machine must be destroyed
-			// along with the machine.
-			return true, nil
-		}
-		if provider.Dynamic() {
-			// We don't know ahead of time whether the storage
-			// will be Persistent, so we assume it will be, and
-			// rely on the environment-level storage provisioner
-			// to clean up.
-			return false, nil
-		}
-		// Volume is static, so even if it is provisioned, it will
-		// be tied to the machine.
-		return true, nil
-	} else if err != nil {
+	return volume.detachable(), nil
+}
+
+// detachable reports whether or not the volume is detachable.
+func (v *volume) detachable() bool {
+	return v.doc.MachineId == ""
+}
+
+// isDetachableVolumePool reports whether or not the given storage
+// pool will create a volume that is not inherently bound to a machine,
+// and therefore can be detached.
+func isDetachableVolumePool(st *State, pool string) (bool, error) {
+	_, provider, err := poolStorageProvider(st, pool)
+	if err != nil {
 		return false, errors.Trace(err)
 	}
-	// If volume does not outlive machine it can be removed.
-	return !volumeInfo.Persistent, nil
+	if provider.Scope() == storage.ScopeMachine {
+		// Any storage created by a machine cannot be detached from
+		// the machine, and must be destroyed along with it.
+		return false, nil
+	}
+	if provider.Dynamic() {
+		// NOTE(axw) In theory, we don't know ahead of time
+		// whether the storage will be Persistent, as the model
+		// allows for a dynamic storage provider to create non-
+		// persistent storage. None of the storage providers do
+		// this, so we assume it will be persistent for now.
+		//
+		// TODO(axw) get rid of the Persistent field from Volume
+		// and Filesystem. We only need to care whether the
+		// storage is dynamic and model-scoped.
+		return true, nil
+	}
+	// Volume is static, so it will be tied to the machine.
+	return false, nil
 }
 
 // DetachVolume marks the volume attachment identified by the specified machine
 // and volume tags as Dying, if it is Alive. DetachVolume will fail with a
 // IsContainsFilesystem error if the volume contains an attached filesystem; the
-// filesystem attachment must be removed first.
+// filesystem attachment must be removed first. DetachVolume will fail for
+// inherently machine-bound volumes.
 func (st *State) DetachVolume(machine names.MachineTag, volume names.VolumeTag) (err error) {
 	defer errors.DeferredAnnotatef(&err, "detaching volume %s from machine %s", volume.Id(), machine.Id())
 	// If the volume is backing a filesystem, the volume cannot be detached
@@ -502,6 +489,13 @@ func (st *State) DetachVolume(machine names.MachineTag, volume names.VolumeTag)
 		if va.Life() != Alive {
 			return nil, jujutxn.ErrNoOperations
 		}
+		detachable, err := isDetachableVolumeTag(st, volume)
+		if err != nil {
+			return nil, errors.Trace(err)
+		}
+		if !detachable {
+			return nil, errors.New("volume is not detachable")
+		}
 		return detachVolumeOps(machine, volume), nil
 	}
 	return st.run(buildTxn)
@@ -554,8 +548,7 @@ func (st *State) RemoveVolumeAttachment(machine names.MachineTag, volume names.V
 func removeVolumeAttachmentOps(m names.MachineTag, v *volume) []txn.Op {
 	decrefVolumeOp := machineStorageDecrefOp(
 		volumesC, v.doc.Name,
-		v.doc.AttachmentCount, v.doc.Life,
-		m, v.doc.Binding,
+		v.doc.AttachmentCount, v.doc.Life, m,
 	)
 	return []txn.Op{{
 		C:      volumeAttachmentsC,
@@ -578,7 +571,6 @@ func machineStorageDecrefOp(
 	collection, id string,
 	attachmentCount int, life Life,
 	machine names.MachineTag,
-	binding string,
 ) txn.Op {
 	op := txn.Op{
 		C:  collection,
@@ -612,24 +604,15 @@ func machineStorageDecrefOp(
 		}
 	} else {
 		// The volume is still Alive: decref, retrying if the
-		// volume is destroyed concurrently or the binding changes.
-		// If the volume is bound to the machine, advance it to
-		// Dead; binding storage to a machine and attaching the
-		// storage to multiple machines will be mutually exclusive.
+		// volume is destroyed concurrently.
 		//
-		// Otherwise, when DestroyVolume is called, the volume will
-		// be marked Dead if it has no attachments.
+		// Otherwise, when DestroyVolume is called, the volume
+		// will be marked Dead if it has no attachments.
 		update := bson.D{
 			{"$inc", bson.D{{"attachmentcount", -1}}},
 		}
-		if binding == machine.String() {
-			update = append(update, bson.DocElem{
-				"$set", bson.D{{"life", Dead}},
-			})
-		}
 		op.Assert = bson.D{
 			{"life", Alive},
-			{"binding", binding},
 			{"attachmentcount", bson.D{{"$gt", 0}}},
 		}
 		op.Update = update
@@ -650,37 +633,75 @@ func (st *State) DestroyVolume(tag names.VolumeTag) (err error) {
 	}
 	buildTxn := func(attempt int) ([]txn.Op, error) {
 		volume, err := st.volumeByTag(tag)
-		if errors.IsNotFound(err) {
+		if errors.IsNotFound(err) && attempt > 0 {
+			// On the first attempt, we expect it to exist.
 			return nil, jujutxn.ErrNoOperations
 		} else if err != nil {
 			return nil, errors.Trace(err)
 		}
+		if volume.doc.StorageId != "" {
+			return nil, errors.Errorf(
+				"volume is assigned to %s",
+				names.ReadableString(names.NewStorageTag(volume.doc.StorageId)),
+			)
+		}
 		if volume.Life() != Alive {
 			return nil, jujutxn.ErrNoOperations
 		}
-		return destroyVolumeOps(st, volume), nil
+		hasNoStorageAssignment := bson.D{{"$or", []bson.D{
+			{{"storageid", ""}},
+			{{"storageid", bson.D{{"$exists", false}}}},
+		}}}
+		return destroyVolumeOps(st, volume, hasNoStorageAssignment)
 	}
 	return st.run(buildTxn)
 }
 
-func destroyVolumeOps(st *State, v *volume) []txn.Op {
+func destroyVolumeOps(st *State, v *volume, extraAssert bson.D) ([]txn.Op, error) {
+	baseAssert := append(isAliveDoc, extraAssert...)
 	if v.doc.AttachmentCount == 0 {
 		hasNoAttachments := bson.D{{"attachmentcount", 0}}
 		return []txn.Op{{
 			C:      volumesC,
 			Id:     v.doc.Name,
-			Assert: append(hasNoAttachments, isAliveDoc...),
+			Assert: append(hasNoAttachments, baseAssert...),
 			Update: bson.D{{"$set", bson.D{{"life", Dead}}}},
-		}}
+		}}, nil
 	}
-	cleanupOp := newCleanupOp(cleanupAttachmentsForDyingVolume, v.doc.Name)
 	hasAttachments := bson.D{{"attachmentcount", bson.D{{"$gt", 0}}}}
-	return []txn.Op{{
+	ops := []txn.Op{{
 		C:      volumesC,
 		Id:     v.doc.Name,
-		Assert: append(hasAttachments, isAliveDoc...),
+		Assert: append(hasAttachments, baseAssert...),
 		Update: bson.D{{"$set", bson.D{{"life", Dying}}}},
-	}, cleanupOp}
+	}}
+	if !v.detachable() {
+		// This volume cannot be directly detached, so we do not
+		// issue a cleanup. Since there can (should!) be only one
+		// attachment for the lifetime of the filesystem, we can
+		// look it up and destroy it along with the filesystem.
+		attachments, err := st.VolumeAttachments(v.VolumeTag())
+		if err != nil {
+			return nil, errors.Trace(err)
+		}
+		if len(attachments) != 1 {
+			return nil, errors.Errorf(
+				"expected 1 attachment, found %d",
+				len(attachments),
+			)
+		}
+		detachOps := detachVolumeOps(
+			attachments[0].Machine(),
+			v.VolumeTag(),
+		)
+		ops = append(ops, detachOps...)
+	} else {
+		ops = append(ops, newCleanupOp(
+			cleanupAttachmentsForDyingVolume,
+			v.doc.Name,
+		))
+	}
+	return ops, nil
 }
 
 // RemoveVolume removes the volume from state. RemoveVolume will fail if
@@ -704,6 +725,7 @@ func (st *State) RemoveVolume(tag names.VolumeTag) (err error) {
 				Assert: txn.DocExists,
 				Remove: true,
 			},
+			removeModelVolumeRefOp(st, tag.Id()),
 			removeStatusOp(st, volumeGlobalKey(tag.Id())),
 		}, nil
 	}
@@ -731,13 +753,15 @@ func newVolumeName(st *State, machineId string) (string, error) {
 // provider is machine-scoped, then the volume will be scoped to that
 // machine.
 func (st *State) addVolumeOps(params VolumeParams, machineId string) ([]txn.Op, names.VolumeTag, error) {
-	if params.binding == nil {
-		params.binding = names.NewMachineTag(machineId)
-	}
-	params, err := st.volumeParamsWithDefaults(params)
+	params, err := st.volumeParamsWithDefaults(params, machineId)
 	if err != nil {
 		return nil, names.VolumeTag{}, errors.Trace(err)
 	}
+	detachable, err := isDetachableVolumePool(st, params.Pool)
+	if err != nil {
+		return nil, names.VolumeTag{}, errors.Trace(err)
+	}
+	origMachineId := machineId
 	machineId, err = st.validateVolumeParams(params, machineId)
 	if err != nil {
 		return nil, names.VolumeTag{}, errors.Annotate(err, "validating volume params")
@@ -753,11 +777,13 @@ func (st *State) addVolumeOps(params VolumeParams, machineId string) ([]txn.Op,
 	doc := volumeDoc{
 		Name:      name,
 		StorageId: params.storage.Id(),
-		Binding:   params.binding.String(),
 		Params:    &params,
 		// Every volume is created with one attachment.
 		AttachmentCount: 1,
 	}
+	if !detachable {
+		doc.MachineId = origMachineId
+	}
 	return st.newVolumeOps(doc, status), names.NewVolumeTag(name), nil
 }
 
@@ -770,27 +796,27 @@ func (st *State) newVolumeOps(doc volumeDoc, status statusDoc) []txn.Op {
 			Assert: txn.DocMissing,
 			Insert: &doc,
 		},
+		addModelVolumeRefOp(st, doc.Name),
 	}
 }
 
-func (st *State) volumeParamsWithDefaults(params VolumeParams) (VolumeParams, error) {
-	if params.Pool != "" {
-		return params, nil
+func (st *State) volumeParamsWithDefaults(params VolumeParams, machineId string) (VolumeParams, error) {
+	if params.Pool == "" {
+		modelConfig, err := st.ModelConfig()
+		if err != nil {
+			return VolumeParams{}, errors.Trace(err)
+		}
+		cons := StorageConstraints{
+			Pool:  params.Pool,
+			Size:  params.Size,
+			Count: 1,
+		}
+		poolName, err := defaultStoragePool(modelConfig, storage.StorageKindBlock, cons)
+		if err != nil {
+			return VolumeParams{}, errors.Annotate(err, "getting default block storage pool")
+		}
+		params.Pool = poolName
 	}
-	envConfig, err := st.ModelConfig()
-	if err != nil {
-		return VolumeParams{}, errors.Trace(err)
-	}
-	cons := StorageConstraints{
-		Pool:  params.Pool,
-		Size:  params.Size,
-		Count: 1,
-	}
-	poolName, err := defaultStoragePool(envConfig, storage.StorageKindBlock, cons)
-	if err != nil {
-		return VolumeParams{}, errors.Annotate(err, "getting default block storage pool")
-	}
-	params.Pool = poolName
 	return params, nil
 }
 
@@ -1033,7 +1059,7 @@ func volumeGlobalKey(name string) string {
 
 // VolumeStatus returns the status of the specified volume.
 func (st *State) VolumeStatus(tag names.VolumeTag) (status.StatusInfo, error) {
-	return getStatus(st, volumeGlobalKey(tag.Id()), "volume")
+	return getStatus(st.db(), volumeGlobalKey(tag.Id()), "volume")
 }
 
 // SetVolumeStatus sets the status of the specified volume.
@@ -1059,12 +1085,12 @@ func (st *State) SetVolumeStatus(tag names.VolumeTag, volumeStatus status.Status
 	default:
 		return errors.Errorf("cannot set invalid status %q", volumeStatus)
 	}
-	return setStatus(st, setStatusParams{
+	return setStatus(st.db(), setStatusParams{
 		badge:     "volume",
 		globalKey: volumeGlobalKey(tag.Id()),
 		status:    volumeStatus,
 		message:   info,
 		rawData:   data,
-		updated:   updated,
+		updated:   timeOrNow(updated, st.clock),
 	})
 }
