@@ -94,6 +94,13 @@ func SkipIfI386(c *gc.C, bugID string) {
 	}
 }
 
+// SkipIfS390X skips the test if the arch is S390X.
+func SkipIfS390X(c *gc.C, bugID string) {
+	if arch.NormaliseArch(runtime.GOARCH) == arch.S390X {
+		c.Skip(fmt.Sprintf("Test disabled on S390X until fixed - see bug %s", bugID))
+	}
+}
+
 // SkipIfWindowsBug skips the test if the OS is Windows.
 func SkipIfWindowsBug(c *gc.C, bugID string) {
 	if runtime.GOOS == "windows" {
@@ -101,6 +108,11 @@ func SkipIfWindowsBug(c *gc.C, bugID string) {
 	}
 }
 
+// SkipFlaky skips the test if there is an open bug for intermittent test failures
+func SkipFlaky(c *gc.C, bugID string) {
+	c.Skip(fmt.Sprintf("Test disabled until flakiness is fixed - see bug %s", bugID))
+}
+
 // SetInitialFeatureFlags sets the feature flags to be in effect for
 // the next call to SetUpTest.
 func (s *JujuOSEnvSuite) SetInitialFeatureFlags(flags ...string) {
@@ -238,6 +250,10 @@ func GetPackageManager() (s PackageManagerStruct, err error) {
 		s.PackageManager = "yum"
 		s.PackageQuery = "yum"
 		s.RepositoryManager = "yum-config-manager --add-repo"
+	case jujuos.OpenSUSE:
+		s.PackageManager = "zypper"
+		s.PackageQuery = "zypper"
+		s.RepositoryManager = "zypper addrepo"
 	case jujuos.Ubuntu:
 		s.PackageManager = "apt-get"
 		s.PackageQuery = "dpkg-query"
