@@ -14,8 +14,10 @@ import (
 	"github.com/juju/utils"
 	"github.com/juju/utils/arch"
 	"github.com/juju/utils/series"
+	"github.com/juju/version"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/charm.v6-unstable"
+	charmresource "gopkg.in/juju/charm.v6-unstable/resource"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/constraints"
@@ -29,7 +31,6 @@ import (
 	"github.com/juju/juju/testcharms"
 	"github.com/juju/juju/testing"
 	jujuversion "github.com/juju/juju/version"
-	"github.com/juju/version"
 )
 
 const (
@@ -128,6 +129,7 @@ type ModelParams struct {
 	CloudRegion             string
 	CloudCredential         names.CloudCredentialTag
 	StorageProviderRegistry storage.ProviderRegistry
+	EnvironVersion          int
 }
 
 type SpaceParams struct {
@@ -294,7 +296,7 @@ func (factory *Factory) MakeMachineNested(c *gc.C, parentId string, params *Mach
 	current := version.Binary{
 		Number: jujuversion.Current,
 		Arch:   arch.HostArch(),
-		Series: series.HostSeries(),
+		Series: series.MustHostSeries(),
 	}
 	err = m.SetAgentVersion(current)
 	c.Assert(err, jc.ErrorIsNil)
@@ -345,6 +347,10 @@ func (factory *Factory) makeMachineReturningPassword(c *gc.C, params *MachinePar
 		Filesystems: params.Filesystems,
 		Constraints: params.Constraints,
 	}
+
+	if params.Characteristics != nil {
+		machineTemplate.HardwareCharacteristics = *params.Characteristics
+	}
 	machine, err := factory.st.AddOneMachine(machineTemplate)
 	c.Assert(err, jc.ErrorIsNil)
 	if setProvisioned {
@@ -360,7 +366,7 @@ func (factory *Factory) makeMachineReturningPassword(c *gc.C, params *MachinePar
 	current := version.Binary{
 		Number: jujuversion.Current,
 		Arch:   arch.HostArch(),
-		Series: series.HostSeries(),
+		Series: series.MustHostSeries(),
 	}
 	err = machine.SetAgentVersion(current)
 	c.Assert(err, jc.ErrorIsNil)
@@ -420,12 +426,27 @@ func (factory *Factory) MakeApplication(c *gc.C, params *ApplicationParams) *sta
 	if params.Name == "" {
 		params.Name = params.Charm.Meta().Name
 	}
+
+	rSt, err := factory.st.Resources()
+	c.Assert(err, jc.ErrorIsNil)
+
+	resourceMap := make(map[string]string)
+	for name, res := range params.Charm.Meta().Resources {
+		pendingID, err := rSt.AddPendingResource(params.Name, "", charmresource.Resource{
+			Meta:   res,
+			Origin: charmresource.OriginUpload,
+		})
+		c.Assert(err, jc.ErrorIsNil)
+		resourceMap[name] = pendingID
+	}
+
 	application, err := factory.st.AddApplication(state.AddApplicationArgs{
 		Name:        params.Name,
 		Charm:       params.Charm,
 		Settings:    charm.Settings(params.Settings),
 		Storage:     params.Storage,
 		Constraints: params.Constraints,
+		Resources:   resourceMap,
 	})
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -629,6 +650,7 @@ func (factory *Factory) MakeModel(c *gc.C, params *ModelParams) *state.State {
 		Config:          cfg,
 		Owner:           params.Owner.(names.UserTag),
 		StorageProviderRegistry: params.StorageProviderRegistry,
+		EnvironVersion:          params.EnvironVersion,
 	})
 	c.Assert(err, jc.ErrorIsNil)
 	return st
