@@ -4,13 +4,9 @@
 package tools
 
 import (
-	"github.com/juju/loggo"
-
-	"github.com/juju/juju/version"
+	"github.com/juju/version"
 )
 
-var logger = loggo.GetLogger("juju.tools")
-
 // Tools represents the location and version of a tools tarball.
 type Tools struct {
 	Version version.Binary `json:"version"`
@@ -19,7 +15,10 @@ type Tools struct {
 	Size    int64          `json:"size"`
 }
 
-// HasTools instances can be asked for a tools list.
-type HasTools interface {
-	Tools(series string) List
+// GUI represents the location and version of a GUI release archive.
+type GUIArchive struct {
+	Version version.Number `json:"version"`
+	URL     string         `json:"url"`
+	SHA256  string         `json:"sha256,omitempty"`
+	Size    int64          `json:"size"`
 }
