@@ -48,7 +48,7 @@ EOF`)
 
 	devices, err := diskmanager.ListBlockDevices()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(devices, jc.SameContents, []storage.BlockDevice{{
+	c.Assert(devices, jc.DeepEquals, []storage.BlockDevice{{
 		DeviceName: "sda",
 		Size:       228936,
 	}, {
@@ -73,6 +73,14 @@ EOF`)
 	}})
 }
 
+func (s *ListBlockDevicesSuite) TestListBlockDevicesWWN(c *gc.C) {
+	// If ID_WWN is found, then we should get
+	// a WWN value.
+	s.testListBlockDevicesExtended(c, `
+ID_WWN=foo
+`, storage.BlockDevice{WWN: "foo"})
+}
+
 func (s *ListBlockDevicesSuite) TestListBlockDevicesBusAddress(c *gc.C) {
 	// If ID_BUS is scsi, then we should get a
 	// BusAddress value.
@@ -91,6 +99,16 @@ ID_SERIAL=0980978987987
 `, storage.BlockDevice{HardwareId: "ata-0980978987987"})
 }
 
+func (s *ListBlockDevicesSuite) TestListBlockDevicesDeviceLinks(c *gc.C) {
+	// Values from DEVLINKS should be split by space, and entered into
+	// DeviceLinks verbatim.
+	s.testListBlockDevicesExtended(c, `
+DEVLINKS=/dev/disk/by-id/abc /dev/disk/by-id/def
+`, storage.BlockDevice{
+		DeviceLinks: []string{"/dev/disk/by-id/abc", "/dev/disk/by-id/def"},
+	})
+}
+
 func (s *ListBlockDevicesSuite) TestListBlockDevicesAll(c *gc.C) {
 	s.testListBlockDevicesExtended(c, `
 DEVPATH=/a/b/c/d/1:2:3:4/block/sda
@@ -158,7 +176,7 @@ EOF`)
 	// to prevent it from being used, but no error will be returned.
 	devices, err := diskmanager.ListBlockDevices()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(devices, jc.SameContents, []storage.BlockDevice{{
+	c.Assert(devices, jc.DeepEquals, []storage.BlockDevice{{
 		DeviceName: "sda",
 		Size:       228936,
 		InUse:      true,
@@ -176,7 +194,7 @@ EOF`)
 
 	devices, err := diskmanager.ListBlockDevices()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(devices, jc.SameContents, []storage.BlockDevice{{
+	c.Assert(devices, jc.DeepEquals, []storage.BlockDevice{{
 		DeviceName: "sda",
 		Size:       0,
 	}, {
@@ -212,7 +230,7 @@ EOF`)
 
 	devices, err := diskmanager.ListBlockDevices()
 	c.Assert(err, gc.IsNil)
-	c.Assert(devices, jc.SameContents, []storage.BlockDevice{{
+	c.Assert(devices, jc.DeepEquals, []storage.BlockDevice{{
 		DeviceName: "sda",
 		Size:       228936,
 	}, {
