@@ -7,35 +7,68 @@ import (
 	"reflect"
 	"time"
 
+	"github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
+	"github.com/juju/utils/clock"
 	gc "gopkg.in/check.v1"
+	"gopkg.in/juju/charm.v6-unstable"
+	"gopkg.in/juju/worker.v1"
 
 	"github.com/juju/juju/api"
 	apifirewaller "github.com/juju/juju/api/firewaller"
+	"github.com/juju/juju/api/remotefirewaller"
+	"github.com/juju/juju/api/remoterelations"
 	"github.com/juju/juju/environs/config"
+	"github.com/juju/juju/feature"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju"
-	"github.com/juju/juju/juju/testing"
+	jujutesting "github.com/juju/juju/juju/testing"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/provider/dummy"
 	"github.com/juju/juju/state"
 	statetesting "github.com/juju/juju/state/testing"
 	coretesting "github.com/juju/juju/testing"
-	"github.com/juju/juju/worker"
+	"github.com/juju/juju/testing/factory"
 	"github.com/juju/juju/worker/firewaller"
-	"github.com/juju/juju/worker/workertest"
 )
 
 // firewallerBaseSuite implements common functionality for embedding
 // into each of the other per-mode suites.
 type firewallerBaseSuite struct {
-	testing.JujuConnSuite
-	op    <-chan dummy.Operation
-	charm *state.Charm
+	jujutesting.JujuConnSuite
+	testing.OsEnvSuite
+	op                 <-chan dummy.Operation
+	charm              *state.Charm
+	controllerMachine  *state.Machine
+	controllerPassword string
 
-	st         api.Connection
-	firewaller *apifirewaller.State
+	st               api.Connection
+	firewaller       *apifirewaller.State
+	remoteRelations  *remoterelations.Client
+	remotefirewaller *remotefirewaller.Client
+	mockClock        *mockClock
+}
+
+func (s *firewallerBaseSuite) SetUpSuite(c *gc.C) {
+	s.OsEnvSuite.SetUpSuite(c)
+	s.JujuConnSuite.SetUpSuite(c)
+}
+
+func (s *firewallerBaseSuite) TearDownSuite(c *gc.C) {
+	s.JujuConnSuite.TearDownSuite(c)
+	s.OsEnvSuite.TearDownSuite(c)
+}
+
+func (s *firewallerBaseSuite) SetUpTest(c *gc.C) {
+	s.SetInitialFeatureFlags(feature.CrossModelRelations)
+	s.OsEnvSuite.SetUpTest(c)
+	s.JujuConnSuite.SetUpTest(c)
+}
+
+func (s *firewallerBaseSuite) TearDownTest(c *gc.C) {
+	s.JujuConnSuite.TearDownTest(c)
+	s.OsEnvSuite.TearDownTest(c)
 }
 
 var _ worker.Worker = (*firewaller.Firewaller)(nil)
@@ -48,35 +81,38 @@ func (s *firewallerBaseSuite) setUpTest(c *gc.C, firewallMode string) {
 	s.charm = s.AddTestingCharm(c, "dummy")
 
 	// Create a manager machine and login to the API.
-	machine, err := s.State.AddMachine("quantal", state.JobManageModel)
+	var err error
+	s.controllerMachine, err = s.State.AddMachine("quantal", state.JobManageModel)
 	c.Assert(err, jc.ErrorIsNil)
-	password, err := utils.RandomPassword()
+	s.controllerPassword, err = utils.RandomPassword()
 	c.Assert(err, jc.ErrorIsNil)
-	err = machine.SetPassword(password)
+	err = s.controllerMachine.SetPassword(s.controllerPassword)
 	c.Assert(err, jc.ErrorIsNil)
-	err = machine.SetProvisioned("i-manager", "fake_nonce", nil)
+	err = s.controllerMachine.SetProvisioned("i-manager", "fake_nonce", nil)
 	c.Assert(err, jc.ErrorIsNil)
-	s.st = s.OpenAPIAsMachine(c, machine.Tag(), password, "fake_nonce")
+	s.st = s.OpenAPIAsMachine(c, s.controllerMachine.Tag(), s.controllerPassword, "fake_nonce")
 	c.Assert(s.st, gc.NotNil)
 
-	// Create the firewaller API facade.
+	// Create the API facades.
 	s.firewaller = apifirewaller.NewState(s.st)
 	c.Assert(s.firewaller, gc.NotNil)
+	s.remoteRelations = remoterelations.NewClient(s.st)
+	c.Assert(s.remoteRelations, gc.NotNil)
 }
 
 // assertPorts retrieves the open ports of the instance and compares them
 // to the expected.
-func (s *firewallerBaseSuite) assertPorts(c *gc.C, inst instance.Instance, machineId string, expected []network.PortRange) {
+func (s *firewallerBaseSuite) assertPorts(c *gc.C, inst instance.Instance, machineId string, expected []network.IngressRule) {
 	s.BackingState.StartSync()
 	start := time.Now()
 	for {
-		got, err := inst.Ports(machineId)
+		got, err := inst.IngressRules(machineId)
 		if err != nil {
 			c.Fatal(err)
 			return
 		}
-		network.SortPortRanges(got)
-		network.SortPortRanges(expected)
+		network.SortIngressRules(got)
+		network.SortIngressRules(expected)
 		if reflect.DeepEqual(got, expected) {
 			c.Succeed()
 			return
@@ -91,17 +127,17 @@ func (s *firewallerBaseSuite) assertPorts(c *gc.C, inst instance.Instance, machi
 
 // assertEnvironPorts retrieves the open ports of environment and compares them
 // to the expected.
-func (s *firewallerBaseSuite) assertEnvironPorts(c *gc.C, expected []network.PortRange) {
+func (s *firewallerBaseSuite) assertEnvironPorts(c *gc.C, expected []network.IngressRule) {
 	s.BackingState.StartSync()
 	start := time.Now()
 	for {
-		got, err := s.Environ.Ports()
+		got, err := s.Environ.IngressRules()
 		if err != nil {
 			c.Fatal(err)
 			return
 		}
-		network.SortPortRanges(got)
-		network.SortPortRanges(expected)
+		network.SortIngressRules(got)
+		network.SortIngressRules(expected)
 		if reflect.DeepEqual(got, expected) {
 			c.Succeed()
 			return
@@ -127,7 +163,7 @@ func (s *firewallerBaseSuite) addUnit(c *gc.C, app *state.Application) (*state.U
 
 // startInstance starts a new instance for the given machine.
 func (s *firewallerBaseSuite) startInstance(c *gc.C, m *state.Machine) instance.Instance {
-	inst, hc := testing.AssertStartInstance(c, s.Environ, s.ControllerConfig.ControllerUUID(), m.Id())
+	inst, hc := jujutesting.AssertStartInstance(c, s.Environ, s.ControllerConfig.ControllerUUID(), m.Id())
 	err := m.SetProvisioned(inst.Id(), "fake_nonce", hc)
 	c.Assert(err, jc.ErrorIsNil)
 	return inst
@@ -147,22 +183,51 @@ func (s *InstanceModeSuite) TearDownTest(c *gc.C) {
 	s.firewallerBaseSuite.JujuConnSuite.TearDownTest(c)
 }
 
-func (s *InstanceModeSuite) TestStartStop(c *gc.C) {
-	fw, err := firewaller.NewFirewaller(s.firewaller)
+// mockClock will panic if anything but After is called
+type mockClock struct {
+	clock.Clock
+	wait time.Duration
+	c    *gc.C
+}
+
+func (m *mockClock) After(duration time.Duration) <-chan time.Time {
+	m.wait = duration
+	return time.After(time.Millisecond)
+}
+
+func (s *InstanceModeSuite) newFirewaller(c *gc.C) worker.Worker {
+	s.mockClock = &mockClock{c: c}
+	cfg := firewaller.Config{
+		ModelUUID:          s.State.ModelUUID(),
+		Mode:               config.FwInstance,
+		EnvironFirewaller:  s.Environ,
+		EnvironInstances:   s.Environ,
+		FirewallerAPI:      s.firewaller,
+		RemoteRelationsApi: s.remoteRelations,
+		NewRemoteFirewallerAPIFunc: func(modelUUID string) (firewaller.RemoteFirewallerAPICloser, error) {
+			return s.remotefirewaller, nil
+		},
+		Clock: s.mockClock,
+	}
+	fw, err := firewaller.NewFirewaller(cfg)
 	c.Assert(err, jc.ErrorIsNil)
+	return fw
+}
+
+func (s *InstanceModeSuite) TestStartStop(c *gc.C) {
+	fw := s.newFirewaller(c)
 	statetesting.AssertKillAndWait(c, fw)
 }
 
-func (s *InstanceModeSuite) TestNotExposedService(c *gc.C) {
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+func (s *InstanceModeSuite) TestNotExposedApplication(c *gc.C) {
+	fw := s.newFirewaller(c)
 	defer statetesting.AssertKillAndWait(c, fw)
 
 	app := s.AddTestingService(c, "wordpress", s.charm)
 	u, m := s.addUnit(c, app)
 	inst := s.startInstance(c, m)
 
-	err = u.OpenPort("tcp", 80)
+	err := u.OpenPort("tcp", 80)
 	c.Assert(err, jc.ErrorIsNil)
 	err = u.OpenPort("tcp", 8080)
 	c.Assert(err, jc.ErrorIsNil)
@@ -175,14 +240,13 @@ func (s *InstanceModeSuite) TestNotExposedService(c *gc.C) {
 	s.assertPorts(c, inst, m.Id(), nil)
 }
 
-func (s *InstanceModeSuite) TestExposedService(c *gc.C) {
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+func (s *InstanceModeSuite) TestExposedApplication(c *gc.C) {
+	fw := s.newFirewaller(c)
 	defer statetesting.AssertKillAndWait(c, fw)
 
 	app := s.AddTestingService(c, "wordpress", s.charm)
 
-	err = app.SetExposed()
+	err := app.SetExposed()
 	c.Assert(err, jc.ErrorIsNil)
 	u, m := s.addUnit(c, app)
 	inst := s.startInstance(c, m)
@@ -192,21 +256,25 @@ func (s *InstanceModeSuite) TestExposedService(c *gc.C) {
 	err = u.OpenPort("tcp", 8080)
 	c.Assert(err, jc.ErrorIsNil)
 
-	s.assertPorts(c, inst, m.Id(), []network.PortRange{{80, 90, "tcp"}, {8080, 8080, "tcp"}})
+	s.assertPorts(c, inst, m.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 90, "0.0.0.0/0"),
+		network.MustNewIngressRule("tcp", 8080, 8080, "0.0.0.0/0"),
+	})
 
 	err = u.ClosePorts("tcp", 80, 90)
 	c.Assert(err, jc.ErrorIsNil)
 
-	s.assertPorts(c, inst, m.Id(), []network.PortRange{{8080, 8080, "tcp"}})
+	s.assertPorts(c, inst, m.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 8080, 8080, "0.0.0.0/0"),
+	})
 }
 
-func (s *InstanceModeSuite) TestMultipleExposedServices(c *gc.C) {
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+func (s *InstanceModeSuite) TestMultipleExposedApplications(c *gc.C) {
+	fw := s.newFirewaller(c)
 	defer statetesting.AssertKillAndWait(c, fw)
 
 	app1 := s.AddTestingService(c, "wordpress", s.charm)
-	err = app1.SetExposed()
+	err := app1.SetExposed()
 	c.Assert(err, jc.ErrorIsNil)
 
 	u1, m1 := s.addUnit(c, app1)
@@ -226,25 +294,31 @@ func (s *InstanceModeSuite) TestMultipleExposedServices(c *gc.C) {
 	err = u2.OpenPort("tcp", 3306)
 	c.Assert(err, jc.ErrorIsNil)
 
-	s.assertPorts(c, inst1, m1.Id(), []network.PortRange{{80, 80, "tcp"}, {8080, 8080, "tcp"}})
-	s.assertPorts(c, inst2, m2.Id(), []network.PortRange{{3306, 3306, "tcp"}})
+	s.assertPorts(c, inst1, m1.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+		network.MustNewIngressRule("tcp", 8080, 8080, "0.0.0.0/0"),
+	})
+	s.assertPorts(c, inst2, m2.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 3306, 3306, "0.0.0.0/0"),
+	})
 
 	err = u1.ClosePort("tcp", 80)
 	c.Assert(err, jc.ErrorIsNil)
 	err = u2.ClosePort("tcp", 3306)
 	c.Assert(err, jc.ErrorIsNil)
 
-	s.assertPorts(c, inst1, m1.Id(), []network.PortRange{{8080, 8080, "tcp"}})
+	s.assertPorts(c, inst1, m1.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 8080, 8080, "0.0.0.0/0"),
+	})
 	s.assertPorts(c, inst2, m2.Id(), nil)
 }
 
 func (s *InstanceModeSuite) TestMachineWithoutInstanceId(c *gc.C) {
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+	fw := s.newFirewaller(c)
 	defer statetesting.AssertKillAndWait(c, fw)
 
 	app := s.AddTestingService(c, "wordpress", s.charm)
-	err = app.SetExposed()
+	err := app.SetExposed()
 	c.Assert(err, jc.ErrorIsNil)
 	// add a unit but don't start its instance yet.
 	u1, m1 := s.addUnit(c, app)
@@ -255,21 +329,24 @@ func (s *InstanceModeSuite) TestMachineWithoutInstanceId(c *gc.C) {
 	inst2 := s.startInstance(c, m2)
 	err = u2.OpenPort("tcp", 80)
 	c.Assert(err, jc.ErrorIsNil)
-	s.assertPorts(c, inst2, m2.Id(), []network.PortRange{{80, 80, "tcp"}})
+	s.assertPorts(c, inst2, m2.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+	})
 
 	inst1 := s.startInstance(c, m1)
 	err = u1.OpenPort("tcp", 8080)
 	c.Assert(err, jc.ErrorIsNil)
-	s.assertPorts(c, inst1, m1.Id(), []network.PortRange{{8080, 8080, "tcp"}})
+	s.assertPorts(c, inst1, m1.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 8080, 8080, "0.0.0.0/0"),
+	})
 }
 
 func (s *InstanceModeSuite) TestMultipleUnits(c *gc.C) {
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+	fw := s.newFirewaller(c)
 	defer statetesting.AssertKillAndWait(c, fw)
 
 	app := s.AddTestingService(c, "wordpress", s.charm)
-	err = app.SetExposed()
+	err := app.SetExposed()
 	c.Assert(err, jc.ErrorIsNil)
 
 	u1, m1 := s.addUnit(c, app)
@@ -282,8 +359,12 @@ func (s *InstanceModeSuite) TestMultipleUnits(c *gc.C) {
 	err = u2.OpenPort("tcp", 80)
 	c.Assert(err, jc.ErrorIsNil)
 
-	s.assertPorts(c, inst1, m1.Id(), []network.PortRange{{80, 80, "tcp"}})
-	s.assertPorts(c, inst2, m2.Id(), []network.PortRange{{80, 80, "tcp"}})
+	s.assertPorts(c, inst1, m1.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+	})
+	s.assertPorts(c, inst2, m2.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+	})
 
 	err = u1.ClosePort("tcp", 80)
 	c.Assert(err, jc.ErrorIsNil)
@@ -310,11 +391,13 @@ func (s *InstanceModeSuite) TestStartWithState(c *gc.C) {
 	s.assertPorts(c, inst, m.Id(), nil)
 
 	// Starting the firewaller opens the ports.
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+	fw := s.newFirewaller(c)
 	defer statetesting.AssertKillAndWait(c, fw)
 
-	s.assertPorts(c, inst, m.Id(), []network.PortRange{{80, 80, "tcp"}, {8080, 8080, "tcp"}})
+	s.assertPorts(c, inst, m.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+		network.MustNewIngressRule("tcp", 8080, 8080, "0.0.0.0/0"),
+	})
 
 	err = app.SetExposed()
 	c.Assert(err, jc.ErrorIsNil)
@@ -330,8 +413,7 @@ func (s *InstanceModeSuite) TestStartWithPartialState(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Starting the firewaller, no open ports.
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+	fw := s.newFirewaller(c)
 	defer statetesting.AssertKillAndWait(c, fw)
 
 	s.assertPorts(c, inst, m.Id(), nil)
@@ -344,10 +426,12 @@ func (s *InstanceModeSuite) TestStartWithPartialState(c *gc.C) {
 	err = u.OpenPort("tcp", 80)
 	c.Assert(err, jc.ErrorIsNil)
 
-	s.assertPorts(c, inst, m.Id(), []network.PortRange{{80, 80, "tcp"}})
+	s.assertPorts(c, inst, m.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+	})
 }
 
-func (s *InstanceModeSuite) TestStartWithUnexposedService(c *gc.C) {
+func (s *InstanceModeSuite) TestStartWithUnexposedApplication(c *gc.C) {
 	m, err := s.State.AddMachine("quantal", state.JobHostUnits)
 	c.Assert(err, jc.ErrorIsNil)
 	inst := s.startInstance(c, m)
@@ -361,8 +445,7 @@ func (s *InstanceModeSuite) TestStartWithUnexposedService(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Starting the firewaller, no open ports.
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+	fw := s.newFirewaller(c)
 	defer statetesting.AssertKillAndWait(c, fw)
 
 	s.assertPorts(c, inst, m.Id(), nil)
@@ -370,19 +453,20 @@ func (s *InstanceModeSuite) TestStartWithUnexposedService(c *gc.C) {
 	// Expose service.
 	err = app.SetExposed()
 	c.Assert(err, jc.ErrorIsNil)
-	s.assertPorts(c, inst, m.Id(), []network.PortRange{{80, 80, "tcp"}})
+	s.assertPorts(c, inst, m.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+	})
 }
 
-func (s *InstanceModeSuite) TestSetClearExposedService(c *gc.C) {
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+func (s *InstanceModeSuite) TestSetClearExposedApplication(c *gc.C) {
+	fw := s.newFirewaller(c)
 	defer statetesting.AssertKillAndWait(c, fw)
 
 	app := s.AddTestingService(c, "wordpress", s.charm)
 
 	u, m := s.addUnit(c, app)
 	inst := s.startInstance(c, m)
-	err = u.OpenPort("tcp", 80)
+	err := u.OpenPort("tcp", 80)
 	c.Assert(err, jc.ErrorIsNil)
 	err = u.OpenPort("tcp", 8080)
 	c.Assert(err, jc.ErrorIsNil)
@@ -394,7 +478,10 @@ func (s *InstanceModeSuite) TestSetClearExposedService(c *gc.C) {
 	err = app.SetExposed()
 	c.Assert(err, jc.ErrorIsNil)
 
-	s.assertPorts(c, inst, m.Id(), []network.PortRange{{80, 80, "tcp"}, {8080, 8080, "tcp"}})
+	s.assertPorts(c, inst, m.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+		network.MustNewIngressRule("tcp", 8080, 8080, "0.0.0.0/0"),
+	})
 
 	// ClearExposed closes the ports again.
 	err = app.ClearExposed()
@@ -404,12 +491,11 @@ func (s *InstanceModeSuite) TestSetClearExposedService(c *gc.C) {
 }
 
 func (s *InstanceModeSuite) TestRemoveUnit(c *gc.C) {
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+	fw := s.newFirewaller(c)
 	defer statetesting.AssertKillAndWait(c, fw)
 
 	app := s.AddTestingService(c, "wordpress", s.charm)
-	err = app.SetExposed()
+	err := app.SetExposed()
 	c.Assert(err, jc.ErrorIsNil)
 
 	u1, m1 := s.addUnit(c, app)
@@ -422,8 +508,12 @@ func (s *InstanceModeSuite) TestRemoveUnit(c *gc.C) {
 	err = u2.OpenPort("tcp", 80)
 	c.Assert(err, jc.ErrorIsNil)
 
-	s.assertPorts(c, inst1, m1.Id(), []network.PortRange{{80, 80, "tcp"}})
-	s.assertPorts(c, inst2, m2.Id(), []network.PortRange{{80, 80, "tcp"}})
+	s.assertPorts(c, inst1, m1.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+	})
+	s.assertPorts(c, inst2, m2.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+	})
 
 	// Remove unit.
 	err = u1.EnsureDead()
@@ -432,16 +522,17 @@ func (s *InstanceModeSuite) TestRemoveUnit(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 
 	s.assertPorts(c, inst1, m1.Id(), nil)
-	s.assertPorts(c, inst2, m2.Id(), []network.PortRange{{80, 80, "tcp"}})
+	s.assertPorts(c, inst2, m2.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+	})
 }
 
-func (s *InstanceModeSuite) TestRemoveService(c *gc.C) {
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+func (s *InstanceModeSuite) TestRemoveApplication(c *gc.C) {
+	fw := s.newFirewaller(c)
 	defer statetesting.AssertKillAndWait(c, fw)
 
 	app := s.AddTestingService(c, "wordpress", s.charm)
-	err = app.SetExposed()
+	err := app.SetExposed()
 	c.Assert(err, jc.ErrorIsNil)
 
 	u, m := s.addUnit(c, app)
@@ -449,9 +540,11 @@ func (s *InstanceModeSuite) TestRemoveService(c *gc.C) {
 	err = u.OpenPort("tcp", 80)
 	c.Assert(err, jc.ErrorIsNil)
 
-	s.assertPorts(c, inst, m.Id(), []network.PortRange{{80, 80, "tcp"}})
+	s.assertPorts(c, inst, m.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+	})
 
-	// Remove service.
+	// Remove application.
 	err = u.EnsureDead()
 	c.Assert(err, jc.ErrorIsNil)
 	err = u.Remove()
@@ -461,13 +554,12 @@ func (s *InstanceModeSuite) TestRemoveService(c *gc.C) {
 	s.assertPorts(c, inst, m.Id(), nil)
 }
 
-func (s *InstanceModeSuite) TestRemoveMultipleServices(c *gc.C) {
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+func (s *InstanceModeSuite) TestRemoveMultipleApplications(c *gc.C) {
+	fw := s.newFirewaller(c)
 	defer statetesting.AssertKillAndWait(c, fw)
 
 	app1 := s.AddTestingService(c, "wordpress", s.charm)
-	err = app1.SetExposed()
+	err := app1.SetExposed()
 	c.Assert(err, jc.ErrorIsNil)
 
 	u1, m1 := s.addUnit(c, app1)
@@ -484,10 +576,14 @@ func (s *InstanceModeSuite) TestRemoveMultipleServices(c *gc.C) {
 	err = u2.OpenPort("tcp", 3306)
 	c.Assert(err, jc.ErrorIsNil)
 
-	s.assertPorts(c, inst1, m1.Id(), []network.PortRange{{80, 80, "tcp"}})
-	s.assertPorts(c, inst2, m2.Id(), []network.PortRange{{3306, 3306, "tcp"}})
+	s.assertPorts(c, inst1, m1.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+	})
+	s.assertPorts(c, inst2, m2.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 3306, 3306, "0.0.0.0/0"),
+	})
 
-	// Remove services.
+	// Remove applications.
 	err = u2.EnsureDead()
 	c.Assert(err, jc.ErrorIsNil)
 	err = u2.Remove()
@@ -507,12 +603,11 @@ func (s *InstanceModeSuite) TestRemoveMultipleServices(c *gc.C) {
 }
 
 func (s *InstanceModeSuite) TestDeadMachine(c *gc.C) {
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+	fw := s.newFirewaller(c)
 	defer statetesting.AssertKillAndWait(c, fw)
 
 	app := s.AddTestingService(c, "wordpress", s.charm)
-	err = app.SetExposed()
+	err := app.SetExposed()
 	c.Assert(err, jc.ErrorIsNil)
 
 	u, m := s.addUnit(c, app)
@@ -520,9 +615,11 @@ func (s *InstanceModeSuite) TestDeadMachine(c *gc.C) {
 	err = u.OpenPort("tcp", 80)
 	c.Assert(err, jc.ErrorIsNil)
 
-	s.assertPorts(c, inst, m.Id(), []network.PortRange{{80, 80, "tcp"}})
+	s.assertPorts(c, inst, m.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+	})
 
-	// Remove unit and service, also tested without. Has no effect.
+	// Remove unit and application, also tested without. Has no effect.
 	err = u.EnsureDead()
 	c.Assert(err, jc.ErrorIsNil)
 	err = u.Remove()
@@ -540,12 +637,11 @@ func (s *InstanceModeSuite) TestDeadMachine(c *gc.C) {
 }
 
 func (s *InstanceModeSuite) TestRemoveMachine(c *gc.C) {
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+	fw := s.newFirewaller(c)
 	defer statetesting.AssertKillAndWait(c, fw)
 
 	app := s.AddTestingService(c, "wordpress", s.charm)
-	err = app.SetExposed()
+	err := app.SetExposed()
 	c.Assert(err, jc.ErrorIsNil)
 
 	u, m := s.addUnit(c, app)
@@ -553,7 +649,9 @@ func (s *InstanceModeSuite) TestRemoveMachine(c *gc.C) {
 	err = u.OpenPort("tcp", 80)
 	c.Assert(err, jc.ErrorIsNil)
 
-	s.assertPorts(c, inst, m.Id(), []network.PortRange{{80, 80, "tcp"}})
+	s.assertPorts(c, inst, m.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+	})
 
 	// Remove unit.
 	err = u.EnsureDead()
@@ -588,8 +686,7 @@ func (s *InstanceModeSuite) TestStartWithStateOpenPortsBroken(c *gc.C) {
 
 	// Starting the firewaller should attempt to open the ports,
 	// and fail due to the method being broken.
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+	fw := s.newFirewaller(c)
 
 	errc := make(chan error, 1)
 	go func() { errc <- fw.Wait() }()
@@ -605,6 +702,103 @@ func (s *InstanceModeSuite) TestStartWithStateOpenPortsBroken(c *gc.C) {
 	}
 }
 
+func (s *InstanceModeSuite) assertRemoteRelation(c *gc.C, expectedCIDRS []string) {
+	// Set up another model to host one side of a remote relation.
+	otherState := s.Factory.MakeModel(c, &factory.ModelParams{Name: "other"})
+	defer otherState.Close()
+
+	// Create the consuming side.
+	otherFactory := factory.NewFactory(otherState)
+	ch := otherFactory.MakeCharm(c, &factory.CharmParams{Name: "wordpress"})
+	wp := otherFactory.MakeApplication(c, &factory.ApplicationParams{Name: "wordpress", Charm: ch})
+
+	// Create an api connection to the other model.
+	apiInfo := s.APIInfo(c)
+	apiInfo.ModelTag = otherState.ModelTag()
+	apiInfo.Tag = s.controllerMachine.Tag()
+	apiInfo.Password = s.controllerPassword
+	apiInfo.Nonce = "fake_nonce"
+	apiCaller, err := api.Open(apiInfo, api.DialOpts{})
+	c.Assert(err, jc.ErrorIsNil)
+	defer apiCaller.Close()
+
+	// Use the other model api connection to create the remote firewaller facade
+	s.remotefirewaller = remotefirewaller.NewClient(apiCaller)
+	c.Assert(s.remotefirewaller, gc.NotNil)
+
+	// Create the firewaller facade on the offering model.
+	fw := s.newFirewaller(c)
+	defer statetesting.AssertKillAndWait(c, fw)
+
+	// Set up the offering model - create the remote app and relation.
+	_, err = s.State.AddRemoteApplication(state.AddRemoteApplicationParams{
+		Name: "wordpress", SourceModel: otherState.ModelTag(),
+		Endpoints: []charm.Relation{{Name: "database", Interface: "mysql", Role: "requirer", Scope: "global"}},
+	})
+	c.Assert(err, jc.ErrorIsNil)
+	mysql := s.AddTestingService(c, "mysql", s.AddTestingCharm(c, "mysql"))
+
+	eps, err := s.State.InferEndpoints("wordpress", "mysql")
+	c.Assert(err, jc.ErrorIsNil)
+	rel, err := s.State.AddRelation(eps...)
+	c.Assert(err, jc.ErrorIsNil)
+
+	u, m := s.addUnit(c, mysql)
+	err = u.OpenPort("tcp", 3306)
+	c.Assert(err, jc.ErrorIsNil)
+	inst := s.startInstance(c, m)
+
+	// Export the relation so the firewaller knows it's ready to be processed.
+	re := s.State.RemoteEntities()
+	token, err := re.ExportLocalEntity(rel.Tag())
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Set up the remote relation in the consuming model.
+	_, err = otherState.AddRemoteApplication(state.AddRemoteApplicationParams{
+		Name: "mysql", SourceModel: otherState.ModelTag(),
+		Endpoints: []charm.Relation{{Name: "database", Interface: "mysql", Role: "provider", Scope: "global"}},
+	})
+	eps, err = otherState.InferEndpoints("wordpress", "mysql")
+	c.Assert(err, jc.ErrorIsNil)
+	otherRel, err := otherState.AddRelation(eps...)
+	c.Assert(err, jc.ErrorIsNil)
+	re = otherState.RemoteEntities()
+	err = re.ImportRemoteEntity(s.State.ModelTag(), otherRel.Tag(), token)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// We should not have opened any ports yet - no unit has entered scope.
+	s.assertPorts(c, inst, m.Id(), nil)
+
+	// Add a public address to the consuming unit so the firewaller can use it.
+	wpm := otherFactory.MakeMachine(c, &factory.MachineParams{
+		Addresses: []network.Address{network.NewAddress("10.0.0.4")},
+	})
+	wpu := otherFactory.MakeUnit(c, &factory.UnitParams{Application: wp, Machine: wpm})
+	c.Assert(err, jc.ErrorIsNil)
+	relUnit, err := otherRel.Unit(wpu)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Add a unit on the consuming app and have it enter the relation scope.
+	// This will trigger the firewaller.
+	err = relUnit.EnterScope(map[string]interface{}{})
+	c.Assert(err, jc.ErrorIsNil)
+	s.assertPorts(c, inst, m.Id(), []network.IngressRule{
+		network.MustNewIngressRule("tcp", 3306, 3306, expectedCIDRS...),
+	})
+
+	// Check the relation ready poll time is as expected.
+	c.Assert(s.mockClock.wait, gc.Equals, 3*time.Second)
+
+	// Ports should be closed when unit leaves scope.
+	err = relUnit.LeaveScope()
+	c.Assert(err, jc.ErrorIsNil)
+	s.assertPorts(c, inst, m.Id(), nil)
+}
+
+func (s *InstanceModeSuite) TestRemoteRelation(c *gc.C) {
+	s.assertRemoteRelation(c, []string{"10.0.0.4/32"})
+}
+
 type GlobalModeSuite struct {
 	firewallerBaseSuite
 }
@@ -619,20 +813,35 @@ func (s *GlobalModeSuite) TearDownTest(c *gc.C) {
 	s.firewallerBaseSuite.JujuConnSuite.TearDownTest(c)
 }
 
-func (s *GlobalModeSuite) TestStartStop(c *gc.C) {
-	fw, err := firewaller.NewFirewaller(s.firewaller)
+func (s *GlobalModeSuite) newFirewaller(c *gc.C) worker.Worker {
+	cfg := firewaller.Config{
+		ModelUUID:          s.State.ModelUUID(),
+		Mode:               config.FwGlobal,
+		EnvironFirewaller:  s.Environ,
+		EnvironInstances:   s.Environ,
+		FirewallerAPI:      s.firewaller,
+		RemoteRelationsApi: s.remoteRelations,
+		NewRemoteFirewallerAPIFunc: func(modelUUID string) (firewaller.RemoteFirewallerAPICloser, error) {
+			return s.remotefirewaller, nil
+		},
+	}
+	fw, err := firewaller.NewFirewaller(cfg)
 	c.Assert(err, jc.ErrorIsNil)
+	return fw
+}
+
+func (s *GlobalModeSuite) TestStartStop(c *gc.C) {
+	fw := s.newFirewaller(c)
 	statetesting.AssertKillAndWait(c, fw)
 }
 
 func (s *GlobalModeSuite) TestGlobalMode(c *gc.C) {
 	// Start firewaller and open ports.
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+	fw := s.newFirewaller(c)
 	defer statetesting.AssertKillAndWait(c, fw)
 
 	app1 := s.AddTestingService(c, "wordpress", s.charm)
-	err = app1.SetExposed()
+	err := app1.SetExposed()
 	c.Assert(err, jc.ErrorIsNil)
 
 	u1, m1 := s.addUnit(c, app1)
@@ -652,17 +861,25 @@ func (s *GlobalModeSuite) TestGlobalMode(c *gc.C) {
 	err = u2.OpenPorts("tcp", 80, 90)
 	c.Assert(err, jc.ErrorIsNil)
 
-	s.assertEnvironPorts(c, []network.PortRange{{80, 90, "tcp"}, {8080, 8080, "tcp"}})
+	s.assertEnvironPorts(c, []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 90, "0.0.0.0/0"),
+		network.MustNewIngressRule("tcp", 8080, 8080, "0.0.0.0/0"),
+	})
 
 	// Closing a port opened by a different unit won't touch the environment.
 	err = u1.ClosePorts("tcp", 80, 90)
 	c.Assert(err, jc.ErrorIsNil)
-	s.assertEnvironPorts(c, []network.PortRange{{80, 90, "tcp"}, {8080, 8080, "tcp"}})
+	s.assertEnvironPorts(c, []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 90, "0.0.0.0/0"),
+		network.MustNewIngressRule("tcp", 8080, 8080, "0.0.0.0/0"),
+	})
 
 	// Closing a port used just once changes the environment.
 	err = u1.ClosePort("tcp", 8080)
 	c.Assert(err, jc.ErrorIsNil)
-	s.assertEnvironPorts(c, []network.PortRange{{80, 90, "tcp"}})
+	s.assertEnvironPorts(c, []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 90, "0.0.0.0/0"),
+	})
 
 	// Closing the last port also modifies the environment.
 	err = u2.ClosePorts("tcp", 80, 90)
@@ -670,7 +887,7 @@ func (s *GlobalModeSuite) TestGlobalMode(c *gc.C) {
 	s.assertEnvironPorts(c, nil)
 }
 
-func (s *GlobalModeSuite) TestStartWithUnexposedService(c *gc.C) {
+func (s *GlobalModeSuite) TestStartWithUnexposedApplication(c *gc.C) {
 	m, err := s.State.AddMachine("quantal", state.JobHostUnits)
 	c.Assert(err, jc.ErrorIsNil)
 	s.startInstance(c, m)
@@ -684,25 +901,25 @@ func (s *GlobalModeSuite) TestStartWithUnexposedService(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Starting the firewaller, no open ports.
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+	fw := s.newFirewaller(c)
 	defer statetesting.AssertKillAndWait(c, fw)
 
 	s.assertEnvironPorts(c, nil)
 
-	// Expose service.
+	// Expose application.
 	err = app.SetExposed()
 	c.Assert(err, jc.ErrorIsNil)
-	s.assertEnvironPorts(c, []network.PortRange{{80, 80, "tcp"}})
+	s.assertEnvironPorts(c, []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+	})
 }
 
 func (s *GlobalModeSuite) TestRestart(c *gc.C) {
 	// Start firewaller and open ports.
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+	fw := s.newFirewaller(c)
 
 	app := s.AddTestingService(c, "wordpress", s.charm)
-	err = app.SetExposed()
+	err := app.SetExposed()
 	c.Assert(err, jc.ErrorIsNil)
 
 	u, m := s.addUnit(c, app)
@@ -712,7 +929,10 @@ func (s *GlobalModeSuite) TestRestart(c *gc.C) {
 	err = u.OpenPort("tcp", 8080)
 	c.Assert(err, jc.ErrorIsNil)
 
-	s.assertEnvironPorts(c, []network.PortRange{{80, 90, "tcp"}, {8080, 8080, "tcp"}})
+	s.assertEnvironPorts(c, []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 90, "0.0.0.0/0"),
+		network.MustNewIngressRule("tcp", 8080, 8080, "0.0.0.0/0"),
+	})
 
 	// Stop firewaller and close one and open a different port.
 	err = worker.Stop(fw)
@@ -724,20 +944,21 @@ func (s *GlobalModeSuite) TestRestart(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Start firewaller and check port.
-	fw, err = firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+	fw = s.newFirewaller(c)
 	defer statetesting.AssertKillAndWait(c, fw)
 
-	s.assertEnvironPorts(c, []network.PortRange{{80, 90, "tcp"}, {8888, 8888, "tcp"}})
+	s.assertEnvironPorts(c, []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 90, "0.0.0.0/0"),
+		network.MustNewIngressRule("tcp", 8888, 8888, "0.0.0.0/0"),
+	})
 }
 
-func (s *GlobalModeSuite) TestRestartUnexposedService(c *gc.C) {
+func (s *GlobalModeSuite) TestRestartUnexposedApplication(c *gc.C) {
 	// Start firewaller and open ports.
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+	fw := s.newFirewaller(c)
 
 	app := s.AddTestingService(c, "wordpress", s.charm)
-	err = app.SetExposed()
+	err := app.SetExposed()
 	c.Assert(err, jc.ErrorIsNil)
 
 	u, m := s.addUnit(c, app)
@@ -747,9 +968,12 @@ func (s *GlobalModeSuite) TestRestartUnexposedService(c *gc.C) {
 	err = u.OpenPort("tcp", 8080)
 	c.Assert(err, jc.ErrorIsNil)
 
-	s.assertEnvironPorts(c, []network.PortRange{{80, 80, "tcp"}, {8080, 8080, "tcp"}})
+	s.assertEnvironPorts(c, []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+		network.MustNewIngressRule("tcp", 8080, 8080, "0.0.0.0/0"),
+	})
 
-	// Stop firewaller and clear exposed flag on service.
+	// Stop firewaller and clear exposed flag on application.
 	err = worker.Stop(fw)
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -757,8 +981,7 @@ func (s *GlobalModeSuite) TestRestartUnexposedService(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Start firewaller and check port.
-	fw, err = firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+	fw = s.newFirewaller(c)
 	defer statetesting.AssertKillAndWait(c, fw)
 
 	s.assertEnvironPorts(c, nil)
@@ -766,11 +989,10 @@ func (s *GlobalModeSuite) TestRestartUnexposedService(c *gc.C) {
 
 func (s *GlobalModeSuite) TestRestartPortCount(c *gc.C) {
 	// Start firewaller and open ports.
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+	fw := s.newFirewaller(c)
 
 	app1 := s.AddTestingService(c, "wordpress", s.charm)
-	err = app1.SetExposed()
+	err := app1.SetExposed()
 	c.Assert(err, jc.ErrorIsNil)
 
 	u1, m1 := s.addUnit(c, app1)
@@ -780,9 +1002,12 @@ func (s *GlobalModeSuite) TestRestartPortCount(c *gc.C) {
 	err = u1.OpenPort("tcp", 8080)
 	c.Assert(err, jc.ErrorIsNil)
 
-	s.assertEnvironPorts(c, []network.PortRange{{80, 80, "tcp"}, {8080, 8080, "tcp"}})
+	s.assertEnvironPorts(c, []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+		network.MustNewIngressRule("tcp", 8080, 8080, "0.0.0.0/0"),
+	})
 
-	// Stop firewaller and add another service using the port.
+	// Stop firewaller and add another application using the port.
 	err = worker.Stop(fw)
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -796,21 +1021,28 @@ func (s *GlobalModeSuite) TestRestartPortCount(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Start firewaller and check port.
-	fw, err = firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
+	fw = s.newFirewaller(c)
 	defer statetesting.AssertKillAndWait(c, fw)
 
-	s.assertEnvironPorts(c, []network.PortRange{{80, 80, "tcp"}, {8080, 8080, "tcp"}})
+	s.assertEnvironPorts(c, []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+		network.MustNewIngressRule("tcp", 8080, 8080, "0.0.0.0/0"),
+	})
 
 	// Closing a port opened by a different unit won't touch the environment.
 	err = u1.ClosePort("tcp", 80)
 	c.Assert(err, jc.ErrorIsNil)
-	s.assertEnvironPorts(c, []network.PortRange{{80, 80, "tcp"}, {8080, 8080, "tcp"}})
+	s.assertEnvironPorts(c, []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+		network.MustNewIngressRule("tcp", 8080, 8080, "0.0.0.0/0"),
+	})
 
 	// Closing a port used just once changes the environment.
 	err = u1.ClosePort("tcp", 8080)
 	c.Assert(err, jc.ErrorIsNil)
-	s.assertEnvironPorts(c, []network.PortRange{{80, 80, "tcp"}})
+	s.assertEnvironPorts(c, []network.IngressRule{
+		network.MustNewIngressRule("tcp", 80, 80, "0.0.0.0/0"),
+	})
 
 	// Closing the last port also modifies the environment.
 	err = u2.ClosePort("tcp", 80)
@@ -829,8 +1061,17 @@ func (s *NoneModeSuite) SetUpTest(c *gc.C) {
 }
 
 func (s *NoneModeSuite) TestStopImmediately(c *gc.C) {
-	fw, err := firewaller.NewFirewaller(s.firewaller)
-	c.Assert(err, jc.ErrorIsNil)
-	err = workertest.CheckKilled(c, fw)
-	c.Check(err, jc.ErrorIsNil)
+	cfg := firewaller.Config{
+		ModelUUID:          s.State.ModelUUID(),
+		Mode:               config.FwNone,
+		EnvironFirewaller:  s.Environ,
+		EnvironInstances:   s.Environ,
+		FirewallerAPI:      s.firewaller,
+		RemoteRelationsApi: s.remoteRelations,
+		NewRemoteFirewallerAPIFunc: func(modelUUID string) (firewaller.RemoteFirewallerAPICloser, error) {
+			return s.remotefirewaller, nil
+		},
+	}
+	_, err := firewaller.NewFirewaller(cfg)
+	c.Assert(err, gc.ErrorMatches, `invalid firewall-mode "none"`)
 }
