@@ -8,13 +8,14 @@ import (
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
 	"gopkg.in/juju/names.v2"
+	"gopkg.in/juju/worker.v1"
 
-	"github.com/juju/juju/apiserver/common/networkingcommon"
+	"github.com/juju/juju/api/common"
 	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/status"
 	"github.com/juju/juju/watcher"
-	"github.com/juju/juju/worker"
+	jworker "github.com/juju/juju/worker"
 )
 
 var logger = loggo.GetLogger("juju.worker.machiner")
@@ -74,20 +75,20 @@ var NewMachiner = func(cfg Config) (worker.Worker, error) {
 	return w, nil
 }
 
-var getObservedNetworkConfig = networkingcommon.GetObservedNetworkConfig
+var getObservedNetworkConfig = common.GetObservedNetworkConfig
 
 func (mr *Machiner) SetUp() (watcher.NotifyWatcher, error) {
 	// Find which machine we're responsible for.
 	m, err := mr.config.MachineAccessor.Machine(mr.config.Tag)
 	if params.IsCodeNotFoundOrCodeUnauthorized(err) {
-		return nil, worker.ErrTerminateAgent
+		return nil, jworker.ErrTerminateAgent
 	} else if err != nil {
 		return nil, errors.Trace(err)
 	}
 	mr.machine = m
 
 	if mr.config.ClearMachineAddressesOnStart {
-		logger.Debugf("machine addresses ignored on start - resetting machine addresses")
+		logger.Debugf("machiner configured to reset machine %q addresses to empty", mr.config.Tag)
 		if err := m.SetMachineAddresses(nil); err != nil {
 			return nil, errors.Annotate(err, "reseting machine addresses")
 		}
@@ -139,7 +140,7 @@ func setMachineAddresses(tag names.MachineTag, m Machine) error {
 	}
 	// Filter out any LXC or LXD bridge addresses.
 	hostAddresses = network.FilterBridgeAddresses(hostAddresses)
-	logger.Infof("setting addresses for %v to %q", tag, hostAddresses)
+	logger.Infof("setting addresses for %q to %v", tag, hostAddresses)
 	return m.SetMachineAddresses(hostAddresses)
 }
 
@@ -150,14 +151,14 @@ func (mr *Machiner) Handle(_ <-chan struct{}) error {
 		// call NotifyMachineDead directly after setting the machine Dead in
 		// the first place. We're not doing that to be cautious: the machine
 		// could be missing from state due to invalid global state.
-		return worker.ErrTerminateAgent
+		return jworker.ErrTerminateAgent
 	} else if err != nil {
 		return err
 	}
 
 	life := mr.machine.Life()
 	if life == params.Alive {
-		observedConfig, err := getObservedNetworkConfig(networkingcommon.DefaultNetworkConfigSource())
+		observedConfig, err := getObservedNetworkConfig(common.DefaultNetworkConfigSource())
 		if err != nil {
 			return errors.Annotate(err, "cannot discover observed network config")
 		} else if len(observedConfig) == 0 {
@@ -168,7 +169,7 @@ func (mr *Machiner) Handle(_ <-chan struct{}) error {
 				return errors.Annotate(err, "cannot update observed network config")
 			}
 		}
-		logger.Debugf("observed network config updated")
+		logger.Debugf("observed network config updated for %q to %v", mr.config.Tag, observedConfig)
 
 		return nil
 	}
@@ -201,7 +202,7 @@ func (mr *Machiner) Handle(_ <-chan struct{}) error {
 			return errors.Annotate(err, "reporting machine death")
 		}
 	}
-	return worker.ErrTerminateAgent
+	return jworker.ErrTerminateAgent
 }
 
 func (mr *Machiner) TearDown() error {
