@@ -5,22 +5,29 @@ package migrationmaster
 
 import (
 	"fmt"
+	"io"
 	"strings"
 	"time"
 
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
 	"github.com/juju/utils/clock"
+	"github.com/juju/version"
+	"gopkg.in/juju/charm.v6-unstable"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/api"
+	"github.com/juju/juju/api/common"
 	"github.com/juju/juju/api/migrationtarget"
 	"github.com/juju/juju/apiserver/params"
 	coremigration "github.com/juju/juju/core/migration"
 	"github.com/juju/juju/migration"
+	"github.com/juju/juju/resource"
+	"github.com/juju/juju/tools"
 	"github.com/juju/juju/watcher"
 	"github.com/juju/juju/worker/catacomb"
 	"github.com/juju/juju/worker/fortress"
+	"github.com/juju/juju/wrench"
 )
 
 var (
@@ -33,6 +40,11 @@ var (
 	// server. The migrationmaster should not be restarted again in
 	// this case.
 	ErrMigrated = errors.New("model has migrated")
+
+	// utcZero matches the deserialised zero times coming back from
+	// MigrationTarget.LatestLogTime, because they have a non-nil
+	// location.
+	utcZero = time.Time{}.In(time.UTC)
 )
 
 const (
@@ -41,10 +53,11 @@ const (
 	// phase.
 	maxMinionWait = 15 * time.Minute
 
-	// minionWaitLogInterval is the time between progress update
-	// messages, while the migrationmaster is waiting for reports from
-	// minions.
-	minionWaitLogInterval = 30 * time.Second
+	// progressUpdateInterval is the time between progress update
+	// messages. It's used while the migrationmaster is waiting for
+	// reports from minions and while it's transferring log messages
+	// to the newly-migrated model.
+	progressUpdateInterval = 30 * time.Second
 )
 
 // Facade exposes controller functionality to a Worker.
@@ -76,6 +89,9 @@ type Facade interface {
 	// associated with the API connection.
 	Export() (coremigration.SerializedModel, error)
 
+	// OpenResource downloads a single resource for an application.
+	OpenResource(string, string) (io.ReadCloser, error)
+
 	// Reap removes all documents of the model associated with the API
 	// connection.
 	Reap() error
@@ -87,6 +103,12 @@ type Facade interface {
 	// MinionReports returns details of the reports made by migration
 	// minions to the controller for the current migration phase.
 	MinionReports() (coremigration.MinionReports, error)
+
+	// StreamModelLog takes a starting time and returns a channel that
+	// will yield the logs on or after that time - these are the logs
+	// that need to be transferred to the target after the migration
+	// is successful.
+	StreamModelLog(time.Time) (<-chan common.LogMessage, error)
 }
 
 // Config defines the operation of a Worker.
@@ -140,7 +162,7 @@ func New(config Config) (*Worker, error) {
 	// controller logged against the model. Until then, distinguish
 	// the logs from different migrationmaster insteads using the
 	// model UUID suffix.
-	loggerName := "juju.worker.migrationmaster:" + config.ModelUUID[len(config.ModelUUID)-6:]
+	loggerName := "juju.worker.migrationmaster." + config.ModelUUID[len(config.ModelUUID)-6:]
 	logger := loggo.GetLogger(loggerName)
 
 	w := &Worker{
@@ -188,11 +210,6 @@ func (w *Worker) run() error {
 		return errors.Trace(err)
 	}
 
-	if status.ExternalControl {
-		err := w.waitForMigrationEnd()
-		return errors.Trace(err)
-	}
-
 	phase := status.Phase
 
 	for {
@@ -207,7 +224,7 @@ func (w *Worker) run() error {
 		case coremigration.SUCCESS:
 			phase, err = w.doSUCCESS(status)
 		case coremigration.LOGTRANSFER:
-			phase, err = w.doLOGTRANSFER()
+			phase, err = w.doLOGTRANSFER(status.TargetInfo, status.ModelUUID)
 		case coremigration.REAP:
 			phase, err = w.doREAP()
 		case coremigration.ABORT:
@@ -258,10 +275,6 @@ func (w *Worker) setInfoStatus(s string, a ...interface{}) {
 	w.setStatusAndLog(w.logger.Infof, s, a...)
 }
 
-func (w *Worker) setWarningStatus(s string, a ...interface{}) {
-	w.setStatusAndLog(w.logger.Warningf, s, a...)
-}
-
 func (w *Worker) setErrorStatus(s string, a ...interface{}) {
 	w.setStatusAndLog(w.logger.Errorf, s, a...)
 }
@@ -283,8 +296,10 @@ func (w *Worker) setStatus(message string) error {
 }
 
 func (w *Worker) doQUIESCE(status coremigration.MigrationStatus) (coremigration.Phase, error) {
-	err := w.prechecks(status)
-	if err != nil {
+	// Run prechecks before waiting for minions to report back. This
+	// short-circuits the long timeout in the case of an agent being
+	// down.
+	if err := w.prechecks(status); err != nil {
 		w.setErrorStatus(err.Error())
 		return coremigration.ABORT, nil
 	}
@@ -297,6 +312,12 @@ func (w *Worker) doQUIESCE(status coremigration.MigrationStatus) (coremigration.
 		return coremigration.ABORT, nil
 	}
 
+	// Now that the model is stable, run the prechecks again.
+	if err := w.prechecks(status); err != nil {
+		w.setErrorStatus(err.Error())
+		return coremigration.ABORT, nil
+	}
+
 	return coremigration.IMPORT, nil
 }
 
@@ -337,6 +358,36 @@ func (w *Worker) doIMPORT(targetInfo coremigration.TargetInfo, modelUUID string)
 	return coremigration.VALIDATION, nil
 }
 
+type uploadWrapper struct {
+	client    *migrationtarget.Client
+	modelUUID string
+}
+
+// UploadTools prepends the model UUID to the args passed to the migration client.
+func (w *uploadWrapper) UploadTools(r io.ReadSeeker, vers version.Binary, additionalSeries ...string) (tools.List, error) {
+	return w.client.UploadTools(w.modelUUID, r, vers, additionalSeries...)
+}
+
+// UploadCharm prepends the model UUID to the args passed to the migration client.
+func (w *uploadWrapper) UploadCharm(curl *charm.URL, content io.ReadSeeker) (*charm.URL, error) {
+	return w.client.UploadCharm(w.modelUUID, curl, content)
+}
+
+// UploadResource prepends the model UUID to the args passed to the migration client.
+func (w *uploadWrapper) UploadResource(res resource.Resource, content io.ReadSeeker) error {
+	return w.client.UploadResource(w.modelUUID, res, content)
+}
+
+// SetPlaceholderResource prepends the model UUID to the args passed to the migration client.
+func (w *uploadWrapper) SetPlaceholderResource(res resource.Resource) error {
+	return w.client.SetPlaceholderResource(w.modelUUID, res)
+}
+
+// SetUnitResource prepends the model UUID to the args passed to the migration client.
+func (w *uploadWrapper) SetUnitResource(unitName string, res resource.Resource) error {
+	return w.client.SetUnitResource(w.modelUUID, unitName, res)
+}
+
 func (w *Worker) transferModel(targetInfo coremigration.TargetInfo, modelUUID string) error {
 	w.setInfoStatus("exporting model")
 	serialized, err := w.config.Facade.Export()
@@ -357,21 +408,21 @@ func (w *Worker) transferModel(targetInfo coremigration.TargetInfo, modelUUID st
 	}
 
 	w.setInfoStatus("uploading model binaries into target controller")
-	targetModelConn, err := w.openAPIConnForModel(targetInfo, modelUUID)
-	if err != nil {
-		return errors.Annotate(err, "failed to open connection to target model")
-	}
-	defer targetModelConn.Close()
-	targetModelClient := targetModelConn.Client()
+	wrapper := &uploadWrapper{targetClient, modelUUID}
 	err = w.config.UploadBinaries(migration.UploadBinariesConfig{
 		Charms:          serialized.Charms,
 		CharmDownloader: w.config.CharmDownloader,
-		CharmUploader:   targetModelClient,
+		CharmUploader:   wrapper,
+
 		Tools:           serialized.Tools,
 		ToolsDownloader: w.config.ToolsDownloader,
-		ToolsUploader:   targetModelClient,
+		ToolsUploader:   wrapper,
+
+		Resources:          serialized.Resources,
+		ResourceDownloader: w.config.Facade,
+		ResourceUploader:   wrapper,
 	})
-	return errors.Annotate(err, "failed migration binaries")
+	return errors.Annotate(err, "failed to migrate binaries")
 }
 
 func (w *Worker) doVALIDATION(status coremigration.MigrationStatus) (coremigration.Phase, error) {
@@ -384,9 +435,25 @@ func (w *Worker) doVALIDATION(status coremigration.MigrationStatus) (coremigrati
 		return coremigration.ABORT, nil
 	}
 
+	client, closer, err := w.openTargetAPI(status.TargetInfo)
+	if err != nil {
+		return coremigration.UNKNOWN, errors.Trace(err)
+	}
+	defer closer()
+
+	// Check that the provider and target controller agree about what
+	// machines belong to the migrated model.
+	ok, err = w.checkTargetMachines(client, status.ModelUUID)
+	if err != nil {
+		return coremigration.UNKNOWN, errors.Trace(err)
+	}
+	if !ok {
+		return coremigration.ABORT, nil
+	}
+
 	// Once all agents have validated, activate the model in the
 	// target controller.
-	err = w.activateModel(status.TargetInfo, status.ModelUUID)
+	err = w.activateModel(client, status.ModelUUID)
 	if err != nil {
 		w.setErrorStatus("model activation failed, %v", err)
 		return coremigration.ABORT, nil
@@ -394,17 +461,29 @@ func (w *Worker) doVALIDATION(status coremigration.MigrationStatus) (coremigrati
 	return coremigration.SUCCESS, nil
 }
 
-func (w *Worker) activateModel(targetInfo coremigration.TargetInfo, modelUUID string) error {
-	w.setInfoStatus("activating model in target controller")
-	conn, err := w.openAPIConn(targetInfo)
+func (w *Worker) checkTargetMachines(targetClient *migrationtarget.Client, modelUUID string) (bool, error) {
+	w.setInfoStatus("checking machines in migrated model")
+	results, err := targetClient.CheckMachines(modelUUID)
 	if err != nil {
-		return errors.Trace(err)
+		return false, errors.Trace(err)
 	}
-	defer conn.Close()
+	if len(results) > 0 {
+		for _, resultErr := range results {
+			w.logger.Errorf(resultErr.Error())
+		}
+		plural := "s"
+		if len(results) == 1 {
+			plural = ""
+		}
+		w.setErrorStatus("machine sanity check failed, %d error%s found", len(results), plural)
+		return false, nil
+	}
+	return true, nil
+}
 
-	targetClient := migrationtarget.NewClient(conn)
-	err = targetClient.Activate(modelUUID)
-	return errors.Trace(err)
+func (w *Worker) activateModel(targetClient *migrationtarget.Client, modelUUID string) error {
+	w.setInfoStatus("activating model in target controller")
+	return errors.Trace(targetClient.Activate(modelUUID))
 }
 
 func (w *Worker) doSUCCESS(status coremigration.MigrationStatus) (coremigration.Phase, error) {
@@ -412,23 +491,122 @@ func (w *Worker) doSUCCESS(status coremigration.MigrationStatus) (coremigration.
 	if err != nil {
 		return coremigration.UNKNOWN, errors.Trace(err)
 	}
+	err = w.transferResources(status.TargetInfo, status.ModelUUID)
+	if err != nil {
+		return coremigration.UNKNOWN, errors.Trace(err)
+	}
 	// There's no turning back from SUCCESS - any problems should have
 	// been picked up in VALIDATION. After the minion wait in the
 	// SUCCESS phase, the migration can only proceed to LOGTRANSFER.
 	return coremigration.LOGTRANSFER, nil
 }
 
-func (w *Worker) doLOGTRANSFER() (coremigration.Phase, error) {
-	// TODO(mjs) - To be implemented.
-	// w.setInfoStatus("successful: transferring logs to target controller")
+func (w *Worker) transferResources(targetInfo coremigration.TargetInfo, modelUUID string) error {
+	w.setInfoStatus("transferring ownership of cloud resources to target controller")
+	conn, err := w.openAPIConn(targetInfo)
+	if err != nil {
+		return errors.Trace(err)
+	}
+	defer conn.Close()
+
+	targetClient := migrationtarget.NewClient(conn)
+	err = targetClient.AdoptResources(modelUUID)
+	return errors.Trace(err)
+}
+
+func (w *Worker) doLOGTRANSFER(targetInfo coremigration.TargetInfo, modelUUID string) (coremigration.Phase, error) {
+	err := w.transferLogs(targetInfo, modelUUID)
+	if err != nil {
+		return coremigration.UNKNOWN, errors.Trace(err)
+	}
 	return coremigration.REAP, nil
 }
 
+func (w *Worker) transferLogs(targetInfo coremigration.TargetInfo, modelUUID string) error {
+	sent := 0
+	reportProgress := func(finished bool, sent int) {
+		verb := "transferring"
+		if finished {
+			verb = "transferred"
+		}
+		w.setInfoStatus("successful, %s logs to target controller (%d sent)", verb, sent)
+	}
+	reportProgress(false, sent)
+
+	conn, err := w.openAPIConn(targetInfo)
+	if err != nil {
+		return errors.Annotate(err, "connecting to target API")
+	}
+	targetClient := migrationtarget.NewClient(conn)
+	latestLogTime, err := targetClient.LatestLogTime(modelUUID)
+	if err != nil {
+		return errors.Annotate(err, "getting log start time")
+	}
+
+	if latestLogTime != utcZero {
+		w.logger.Debugf("log transfer was interrupted - restarting from %s", latestLogTime)
+	}
+
+	throwWrench := latestLogTime == utcZero && wrench.IsActive("migrationmaster", "die-after-500-log-messages")
+
+	logSource, err := w.config.Facade.StreamModelLog(latestLogTime)
+	if err != nil {
+		return errors.Annotate(err, "opening source log stream")
+	}
+
+	logTarget, err := targetClient.OpenLogTransferStream(modelUUID)
+	if err != nil {
+		return errors.Annotate(err, "opening target log stream")
+	}
+	defer logTarget.Close()
+
+	clk := w.config.Clock
+	logProgress := clk.After(progressUpdateInterval)
+
+	for {
+		select {
+		case <-w.catacomb.Dying():
+			return w.catacomb.ErrDying()
+		case msg, ok := <-logSource:
+			if !ok {
+				// The channel's been closed, we're finished!
+				reportProgress(true, sent)
+				return nil
+			}
+			err := logTarget.WriteJSON(params.LogRecord{
+				Entity:   msg.Entity,
+				Time:     msg.Timestamp,
+				Module:   msg.Module,
+				Location: msg.Location,
+				Level:    msg.Severity,
+				Message:  msg.Message,
+			})
+			if err != nil {
+				return errors.Trace(err)
+			}
+			sent++
+
+			if throwWrench && sent == 500 {
+				// Simulate a connection drop to test restartability.
+				return errors.New("wrench in the works")
+			}
+		case <-logProgress:
+			reportProgress(false, sent)
+			logProgress = clk.After(progressUpdateInterval)
+		}
+	}
+}
+
 func (w *Worker) doREAP() (coremigration.Phase, error) {
 	w.setInfoStatus("successful, removing model from source controller")
+	// NOTE(babbageclunk): Calling Reap will set the migration phase
+	// to DONE if successful - this avoids a race where this worker is
+	// killed by the model going away before it can update the phase
+	// itself.
 	err := w.config.Facade.Reap()
 	if err != nil {
-		return coremigration.REAPFAILED, errors.Trace(err)
+		w.setErrorStatus("removing exported model failed: %s", err.Error())
+		return coremigration.REAPFAILED, nil
 	}
 	return coremigration.DONE, nil
 }
@@ -438,7 +616,7 @@ func (w *Worker) doABORT(targetInfo coremigration.TargetInfo, modelUUID string)
 	if err := w.removeImportedModel(targetInfo, modelUUID); err != nil {
 		// This isn't fatal. Removing the imported model is a best
 		// efforts attempt so just report the error and proceed.
-		w.setWarningStatus("failed to remove model from target controller, %v", err)
+		w.logger.Warningf("failed to remove model from target controller, %v", err)
 	}
 	return coremigration.ABORTDONE, nil
 }
@@ -556,7 +734,7 @@ func (w *Worker) waitForMinions(
 		return false, errors.Trace(err)
 	}
 
-	logProgress := clk.After(minionWaitLogInterval)
+	logProgress := clk.After(progressUpdateInterval)
 
 	var reports coremigration.MinionReports
 	for {
@@ -600,7 +778,7 @@ func (w *Worker) waitForMinions(
 
 		case <-logProgress:
 			w.setInfoStatus("%s, %s", infoPrefix, formatMinionWaitUpdate(reports))
-			logProgress = clk.After(minionWaitLogInterval)
+			logProgress = clk.After(progressUpdateInterval)
 		}
 	}
 }
@@ -671,6 +849,14 @@ func formatMinionWaitUpdate(reports coremigration.MinionReports) string {
 	return msg
 }
 
+func (w *Worker) openTargetAPI(targetInfo coremigration.TargetInfo) (*migrationtarget.Client, func() error, error) {
+	conn, err := w.openAPIConn(targetInfo)
+	if err != nil {
+		return nil, nil, errors.Trace(err)
+	}
+	return migrationtarget.NewClient(conn), conn.Close, nil
+}
+
 func (w *Worker) openAPIConn(targetInfo coremigration.TargetInfo) (api.Connection, error) {
 	return w.openAPIConnForModel(targetInfo, "")
 }
