@@ -20,9 +20,12 @@ import (
 	"github.com/juju/version"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/names.v2"
+	"gopkg.in/juju/worker.v1"
 
 	"github.com/juju/juju/agent"
+	"github.com/juju/juju/api/common"
 	apiprovisioner "github.com/juju/juju/api/provisioner"
+	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/container"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/instance"
@@ -31,7 +34,7 @@ import (
 	"github.com/juju/juju/tools"
 	jujuversion "github.com/juju/juju/version"
 	"github.com/juju/juju/watcher"
-	"github.com/juju/juju/worker"
+	jworker "github.com/juju/juju/worker"
 	"github.com/juju/juju/worker/provisioner"
 )
 
@@ -86,8 +89,12 @@ func (s *ContainerSetupSuite) TearDownTest(c *gc.C) {
 	s.CommonProvisionerSuite.TearDownTest(c)
 }
 
-func (s *ContainerSetupSuite) setupContainerWorker(c *gc.C, tag names.MachineTag) (watcher.StringsHandler, worker.Runner) {
-	runner := worker.NewRunner(allFatal, noImportance, worker.RestartDelay)
+func (s *ContainerSetupSuite) setupContainerWorker(c *gc.C, tag names.MachineTag) (watcher.StringsHandler, *worker.Runner) {
+	runner := worker.NewRunner(worker.RunnerParams{
+		IsFatal:       allFatal,
+		MoreImportant: noImportance,
+		RestartDelay:  jworker.RestartDelay,
+	})
 	pr := apiprovisioner.NewState(s.st)
 	machine, err := pr.Machine(tag)
 	c.Assert(err, jc.ErrorIsNil)
@@ -116,7 +123,7 @@ func (s *ContainerSetupSuite) setupContainerWorker(c *gc.C, tag names.MachineTag
 
 func (s *ContainerSetupSuite) createContainer(c *gc.C, host *state.Machine, ctype instance.ContainerType) {
 	inst := s.checkStartInstance(c, host)
-	s.setupContainerWorker(c, host.Tag().(names.MachineTag))
+	s.setupContainerWorker(c, host.MachineTag())
 
 	// make a container on the host machine
 	template := state.MachineTemplate{
@@ -140,9 +147,12 @@ func (s *ContainerSetupSuite) createContainer(c *gc.C, host *state.Machine, ctyp
 func (s *ContainerSetupSuite) assertContainerProvisionerStarted(
 	c *gc.C, host *state.Machine, ctype instance.ContainerType) {
 
+	s.PatchValue(provisioner.GetObservedNetworkConfig, func(_ common.NetworkConfigSource) ([]params.NetworkConfig, error) {
+		return nil, nil
+	})
 	// A stub worker callback to record what happens.
 	var provisionerStarted uint32
-	startProvisionerWorker := func(runner worker.Runner, containerType instance.ContainerType,
+	startProvisionerWorker := func(runner *worker.Runner, containerType instance.ContainerType,
 		pr *apiprovisioner.State, cfg agent.Config, broker environs.InstanceBroker,
 		toolsFinder provisioner.ToolsFinder) error {
 		c.Assert(containerType, gc.Equals, ctype)
@@ -153,17 +163,15 @@ func (s *ContainerSetupSuite) assertContainerProvisionerStarted(
 	s.PatchValue(&provisioner.StartProvisioner, startProvisionerWorker)
 
 	s.createContainer(c, host, ctype)
-	// Consume the apt command used to initialise the container.
-	select {
-	case <-s.aptCmdChan:
-	case <-time.After(coretesting.LongWait):
-		c.Fatalf("took too long to get command from channel")
-	}
+
 	// the container worker should have created the provisioner
 	c.Assert(atomic.LoadUint32(&provisionerStarted) > 0, jc.IsTrue)
 }
 
 func (s *ContainerSetupSuite) TestContainerProvisionerStarted(c *gc.C) {
+	s.PatchValue(provisioner.GetContainerInitialiser, func(instance.ContainerType, string) container.Initialiser {
+		return fakeContainerInitialiser{}
+	})
 	// Specifically ignore LXD here, if present in instance.ContainerTypes.
 	containerTypes := []instance.ContainerType{instance.KVM}
 	for _, ctype := range containerTypes {
@@ -179,7 +187,7 @@ func (s *ContainerSetupSuite) TestContainerProvisionerStarted(c *gc.C) {
 		current := version.Binary{
 			Number: jujuversion.Current,
 			Arch:   arch.HostArch(),
-			Series: series.HostSeries(),
+			Series: series.MustHostSeries(),
 		}
 		err = m.SetAgentVersion(current)
 		c.Assert(err, jc.ErrorIsNil)
@@ -187,15 +195,27 @@ func (s *ContainerSetupSuite) TestContainerProvisionerStarted(c *gc.C) {
 	}
 }
 
-func (s *ContainerSetupSuite) TestKvmContainerUsesHostArch(c *gc.C) {
+func (s *ContainerSetupSuite) TestKvmContainerUsesTargetArch(c *gc.C) {
 	// KVM should do what it's told, and use the architecture in
 	// constraints.
 	s.PatchValue(&arch.HostArch, func() string { return arch.PPC64EL })
+	s.PatchValue(provisioner.GetContainerInitialiser, func(instance.ContainerType, string) container.Initialiser {
+		return fakeContainerInitialiser{}
+	})
 	s.testContainerConstraintsArch(c, instance.KVM, arch.AMD64)
 }
 
+type fakeContainerInitialiser struct{}
+
+func (_ fakeContainerInitialiser) Initialise() error {
+	return nil
+}
+
 func (s *ContainerSetupSuite) testContainerConstraintsArch(c *gc.C, containerType instance.ContainerType, expectArch string) {
 	var called uint32
+	s.PatchValue(provisioner.GetObservedNetworkConfig, func(_ common.NetworkConfigSource) ([]params.NetworkConfig, error) {
+		return nil, nil
+	})
 	s.PatchValue(provisioner.GetToolsFinder, func(*apiprovisioner.State) provisioner.ToolsFinder {
 		return toolsFinderFunc(func(v version.Number, series string, arch string) (tools.List, error) {
 			atomic.StoreUint32(&called, 1)
@@ -209,10 +229,10 @@ func (s *ContainerSetupSuite) testContainerConstraintsArch(c *gc.C, containerTyp
 		})
 	})
 
-	s.PatchValue(&provisioner.StartProvisioner, func(runner worker.Runner, containerType instance.ContainerType,
+	s.PatchValue(&provisioner.StartProvisioner, func(runner *worker.Runner, containerType instance.ContainerType,
 		pr *apiprovisioner.State, cfg agent.Config, broker environs.InstanceBroker,
 		toolsFinder provisioner.ToolsFinder) error {
-		toolsFinder.FindTools(jujuversion.Current, series.HostSeries(), arch.AMD64)
+		toolsFinder.FindTools(jujuversion.Current, series.MustHostSeries(), arch.AMD64)
 		return nil
 	})
 
@@ -228,17 +248,13 @@ func (s *ContainerSetupSuite) testContainerConstraintsArch(c *gc.C, containerTyp
 	current := version.Binary{
 		Number: jujuversion.Current,
 		Arch:   arch.HostArch(),
-		Series: series.HostSeries(),
+		Series: series.MustHostSeries(),
 	}
 	err = m.SetAgentVersion(current)
 	c.Assert(err, jc.ErrorIsNil)
 
 	s.createContainer(c, m, containerType)
-	select {
-	case <-s.aptCmdChan:
-	case <-time.After(coretesting.LongWait):
-		c.Fatalf("took too long to get command from channel")
-	}
+
 	c.Assert(atomic.LoadUint32(&called) > 0, jc.IsTrue)
 }
 
@@ -256,14 +272,14 @@ type ContainerInstance struct {
 
 func (s *ContainerSetupSuite) assertContainerInitialised(c *gc.C, cont ContainerInstance) {
 	// A noop worker callback.
-	startProvisionerWorker := func(runner worker.Runner, containerType instance.ContainerType,
+	startProvisionerWorker := func(runner *worker.Runner, containerType instance.ContainerType,
 		pr *apiprovisioner.State, cfg agent.Config, broker environs.InstanceBroker,
 		toolsFinder provisioner.ToolsFinder) error {
 		return nil
 	}
 	s.PatchValue(&provisioner.StartProvisioner, startProvisionerWorker)
 
-	current_os, err := series.GetOSFromSeries(series.HostSeries())
+	current_os, err := series.GetOSFromSeries(series.MustHostSeries())
 	c.Assert(err, jc.ErrorIsNil)
 
 	var ser string
@@ -273,6 +289,10 @@ func (s *ContainerSetupSuite) assertContainerInitialised(c *gc.C, cont Container
 		ser = "centos7"
 		expected_initial = []string{
 			"yum", "--assumeyes", "--debuglevel=1", "install"}
+	case jujuos.OpenSUSE:
+		ser = "opensuseleap"
+		expected_initial = []string{
+			"zypper", " --quiet", "--non-interactive-include-reboot-patches", "install"}
 	default:
 		ser = "precise"
 		expected_initial = []string{
@@ -293,7 +313,7 @@ func (s *ContainerSetupSuite) assertContainerInitialised(c *gc.C, cont Container
 	current := version.Binary{
 		Number: jujuversion.Current,
 		Arch:   arch.HostArch(),
-		Series: series.HostSeries(),
+		Series: series.MustHostSeries(),
 	}
 	err = m.SetAgentVersion(current)
 	c.Assert(err, jc.ErrorIsNil)
@@ -320,7 +340,7 @@ func (s *ContainerSetupSuite) TestContainerInitialised(c *gc.C) {
 	}
 }
 
-func (s *ContainerSetupSuite) TestContainerInitLockError(c *gc.C) {
+func (s *ContainerSetupSuite) TestContainerInitInstDataError(c *gc.C) {
 	spec := mutex.Spec{
 		Name:  s.lockName,
 		Clock: clock.WallClock,
@@ -339,12 +359,12 @@ func (s *ContainerSetupSuite) TestContainerInitLockError(c *gc.C) {
 	current := version.Binary{
 		Number: jujuversion.Current,
 		Arch:   arch.HostArch(),
-		Series: series.HostSeries(),
+		Series: series.MustHostSeries(),
 	}
 	err = m.SetAgentVersion(current)
 	c.Assert(err, jc.ErrorIsNil)
 
-	handler, runner := s.setupContainerWorker(c, m.Tag().(names.MachineTag))
+	handler, runner := s.setupContainerWorker(c, m.MachineTag())
 	runner.Kill()
 	err = runner.Wait()
 	c.Assert(err, jc.ErrorIsNil)
@@ -354,7 +374,7 @@ func (s *ContainerSetupSuite) TestContainerInitLockError(c *gc.C) {
 	abort := make(chan struct{})
 	close(abort)
 	err = handler.Handle(abort, []string{"0/lxd/0"})
-	c.Assert(err, gc.ErrorMatches, ".*failed to acquire initialization lock:.*")
+	c.Assert(err, gc.ErrorMatches, ".*initialising container infrastructure on host machine: instance data for machine.*not found")
 
 }
 
@@ -365,11 +385,17 @@ func (t toolsFinderFunc) FindTools(v version.Number, series string, arch string)
 }
 
 func getContainerInstance() (cont []ContainerInstance, err error) {
+	pkgs := [][]string{
+		{"qemu-kvm"},
+		{"qemu-utils"},
+		{"genisoimage"},
+		{"libvirt-bin"},
+	}
+	if arch.HostArch() == arch.ARM64 {
+		pkgs = append([][]string{{"qemu-efi"}}, pkgs...)
+	}
 	cont = []ContainerInstance{
-		{instance.KVM, [][]string{
-			{"uvtool-libvirt"},
-			{"uvtool"},
-		}},
+		{instance.KVM, pkgs},
 	}
 	return cont, nil
 }
