@@ -6,27 +6,36 @@ package provisioner
 import (
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
+	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/agent"
 	"github.com/juju/juju/cloudconfig/instancecfg"
 	"github.com/juju/juju/container"
-	"github.com/juju/juju/container/kvm"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/instance"
+	"github.com/juju/juju/network"
 )
 
 var kvmLogger = loggo.GetLogger("juju.provisioner.kvm")
 
-func NewKvmBroker(
+// NewKVMBroker creates a Broker that can be used to start KVM guests in a
+// similar fashion to normal StartInstance requests.
+// prepareHost is a callback that will be called when a new container is about
+// to be started. It provides the intersection point where the host can update
+// itself to be ready for whatever changes are necessary to have a functioning
+// container. (such as bridging host devices.)
+// manager is the infrastructure to actually launch the container.
+// agentConfig is currently only used to find out the 'default' bridge to use
+// when a specific network device is not specified in StartInstanceParams. This
+// should be deprecated. And hopefully removed in the future.
+func NewKVMBroker(
+	prepareHost PrepareHostFunc,
 	api APICalls,
+	manager container.Manager,
 	agentConfig agent.Config,
-	managerConfig container.ManagerConfig,
 ) (environs.InstanceBroker, error) {
-	manager, err := kvm.NewContainerManager(managerConfig)
-	if err != nil {
-		return nil, err
-	}
 	return &kvmBroker{
+		prepareHost: prepareHost,
 		manager:     manager,
 		api:         api,
 		agentConfig: agentConfig,
@@ -34,6 +43,7 @@ func NewKvmBroker(
 }
 
 type kvmBroker struct {
+	prepareHost PrepareHostFunc
 	manager     container.Manager
 	api         APICalls
 	agentConfig agent.Config
@@ -42,15 +52,15 @@ type kvmBroker struct {
 // StartInstance is specified in the Broker interface.
 func (broker *kvmBroker) StartInstance(args environs.StartInstanceParams) (*environs.StartInstanceResult, error) {
 	// TODO: refactor common code out of the container brokers.
-	machineId := args.InstanceConfig.MachineId
-	kvmLogger.Infof("starting kvm container for machineId: %s", machineId)
+	containerMachineID := args.InstanceConfig.MachineId
+	kvmLogger.Infof("starting kvm container for containerMachineID: %s", containerMachineID)
 
 	// TODO: Default to using the host network until we can configure.  Yes,
 	// this is using the LxcBridge value, we should put it in the api call for
 	// container config.
 	bridgeDevice := broker.agentConfig.Value(agent.LxcBridge)
 	if bridgeDevice == "" {
-		bridgeDevice = container.DefaultKvmBridge
+		bridgeDevice = network.DefaultKVMBridge
 	}
 
 	config, err := broker.api.ContainerConfig()
@@ -59,28 +69,32 @@ func (broker *kvmBroker) StartInstance(args environs.StartInstanceParams) (*envi
 		return nil, err
 	}
 
-	preparedInfo, err := prepareOrGetContainerInterfaceInfo(
-		broker.api,
-		machineId,
-		bridgeDevice,
-		true, // allocate if possible, do not maintain existing.
-		args.NetworkInfo,
-		kvmLogger,
-	)
-	if err != nil {
-		// It's not fatal (yet) if we couldn't pre-allocate addresses for the
-		// container.
-		logger.Warningf("failed to prepare container %q network config: %v", machineId, err)
-	} else {
-		args.NetworkInfo = preparedInfo
-	}
-
-	network := container.BridgeNetworkConfig(bridgeDevice, 0, args.NetworkInfo)
-	interfaces, err := finishNetworkConfig(bridgeDevice, args.NetworkInfo)
+	err = broker.prepareHost(names.NewMachineTag(containerMachineID), kvmLogger)
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
-	network.Interfaces = interfaces
+
+	preparedInfo, err := prepareOrGetContainerInterfaceInfo(
+		broker.api,
+		containerMachineID,
+		true, // allocate if possible, do not maintain existing.
+		kvmLogger,
+	)
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+
+	// Something to fallback to if there are no devices given in args.NetworkInfo
+	// TODO(jam): 2017-02-07, this feels like something that should never need
+	// to be invoked, because either StartInstance or
+	// prepareOrGetContainerInterfaceInfo should always return a value. The
+	// test suite currently doesn't think so, and I'm hesitant to munge it too
+	// much.
+	interfaces, err := finishNetworkConfig(bridgeDevice, preparedInfo)
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	network := container.BridgeNetworkConfig(bridgeDevice, 0, interfaces)
 
 	// The provisioner worker will provide all tools it knows about
 	// (after applying explicitly specified constraints), which may
@@ -126,7 +140,7 @@ func (broker *kvmBroker) StartInstance(args environs.StartInstanceParams) (*envi
 		kvmLogger.Errorf("failed to start container: %v", err)
 		return nil, err
 	}
-	kvmLogger.Infof("started kvm container for machineId: %s, %s, %s", machineId, inst.Id(), hardware.String())
+	kvmLogger.Infof("started kvm container for containerMachineID: %s, %s, %s", containerMachineID, inst.Id(), hardware.String())
 	return &environs.StartInstanceResult{
 		Instance:    inst,
 		Hardware:    hardware,
@@ -140,19 +154,11 @@ func (broker *kvmBroker) StartInstance(args environs.StartInstanceParams) (*envi
 func (broker *kvmBroker) MaintainInstance(args environs.StartInstanceParams) error {
 	machineID := args.InstanceConfig.MachineId
 
-	// Default to using the host network until we can configure.
-	bridgeDevice := broker.agentConfig.Value(agent.LxcBridge)
-	if bridgeDevice == "" {
-		bridgeDevice = container.DefaultKvmBridge
-	}
-
 	// There's no InterfaceInfo we expect to get below.
 	_, err := prepareOrGetContainerInterfaceInfo(
 		broker.api,
 		machineID,
-		bridgeDevice,
 		false, // maintain, do not allocate.
-		args.NetworkInfo,
 		kvmLogger,
 	)
 	return err
