@@ -10,6 +10,7 @@ import (
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
 	"gopkg.in/juju/names.v2"
+	worker "gopkg.in/juju/worker.v1"
 
 	"github.com/juju/juju/agent"
 	apiprovisioner "github.com/juju/juju/api/provisioner"
@@ -18,7 +19,6 @@ import (
 	"github.com/juju/juju/environs/config"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/watcher"
-	"github.com/juju/juju/worker"
 	"github.com/juju/juju/worker/catacomb"
 )
 
@@ -30,7 +30,7 @@ var _ Provisioner = (*containerProvisioner)(nil)
 
 var (
 	retryStrategyDelay = 10 * time.Second
-	retryStrategyCount = 3
+	retryStrategyCount = 10
 )
 
 // Provisioner represents a running provisioner worker.
