@@ -9,7 +9,9 @@ import (
 
 	"github.com/juju/errors"
 	"github.com/juju/utils"
+	"github.com/juju/version"
 	"gopkg.in/juju/names.v2"
+	worker "gopkg.in/juju/worker.v1"
 
 	apiprovisioner "github.com/juju/juju/api/provisioner"
 	"github.com/juju/juju/apiserver/common/networkingcommon"
@@ -29,12 +31,9 @@ import (
 	"github.com/juju/juju/status"
 	"github.com/juju/juju/storage"
 	coretools "github.com/juju/juju/tools"
-	jujuversion "github.com/juju/juju/version"
 	"github.com/juju/juju/watcher"
-	"github.com/juju/juju/worker"
 	"github.com/juju/juju/worker/catacomb"
 	"github.com/juju/juju/wrench"
-	"github.com/juju/version"
 )
 
 type ProvisionerTask interface {
@@ -208,6 +207,10 @@ func (task *provisionerTask) processMachinesWithTransientErrors() error {
 			logger.Errorf("cannot reset status of machine %q: %v", statusResult.Id, err)
 			continue
 		}
+		if err := machine.SetInstanceStatus(status.Provisioning, "", nil); err != nil {
+			logger.Errorf("cannot reset instance status of machine %q: %v", statusResult.Id, err)
+			continue
+		}
 		task.machines[machine.Tag().String()] = machine
 		pending = append(pending, machine)
 	}
@@ -229,7 +232,7 @@ func (task *provisionerTask) processMachines(ids []string) error {
 	}
 
 	// Stop all machines that are dead
-	stopping := task.instancesForMachines(dead)
+	stopping := task.instancesForDeadMachines(dead)
 
 	// Find running instances that have no machines associated
 	unknown, err := task.findUnknownInstances(stopping)
@@ -358,6 +361,7 @@ type ClassifiableMachine interface {
 	InstanceId() (instance.Id, error)
 	EnsureDead() error
 	Status() (status.Status, string, error)
+	InstanceStatus() (status.Status, string, error)
 	Id() string
 }
 
@@ -401,6 +405,15 @@ func classifyMachine(machine ClassifiableMachine) (
 			logger.Infof("found machine pending provisioning id:%s, details:%v", machine.Id(), machine)
 			return Pending, nil
 		}
+		instanceStatus, _, err := machine.InstanceStatus()
+		if err != nil {
+			logger.Infof("cannot read instance status id:%s, details:%v, err:%v", machine.Id(), machine, err)
+			return None, nil
+		}
+		if instanceStatus == status.Provisioning {
+			logger.Infof("found machine provisioning id:%s, details:%v", machine.Id(), machine)
+			return Pending, nil
+		}
 		return None, nil
 	}
 	logger.Infof("machine %s already started as instance %q", machine.Id(), instId)
@@ -442,14 +455,19 @@ func (task *provisionerTask) findUnknownInstances(stopping []instance.Instance)
 	return unknown, nil
 }
 
-// instancesForMachines returns a list of instance.Instance that represent
-// the list of machines running in the provider. Missing machines are
+// instancesForDeadMachines returns a list of instance.Instance that represent
+// the list of dead machines running in the provider. Missing machines are
 // omitted from the list.
-func (task *provisionerTask) instancesForMachines(machines []*apiprovisioner.Machine) []instance.Instance {
+func (task *provisionerTask) instancesForDeadMachines(deadMachines []*apiprovisioner.Machine) []instance.Instance {
 	var instances []instance.Instance
-	for _, machine := range machines {
+	for _, machine := range deadMachines {
 		instId, err := machine.InstanceId()
 		if err == nil {
+			keep, _ := machine.KeepInstance()
+			if keep {
+				logger.Debugf("machine %v is dead but keep-instance is true", instId)
+				continue
+			}
 			instance, found := task.instances[instId]
 			// If the instance is not found we can't stop it.
 			if found {
@@ -638,6 +656,12 @@ func (task *provisionerTask) maintainMachines(machines []*apiprovisioner.Machine
 
 func (task *provisionerTask) startMachines(machines []*apiprovisioner.Machine) error {
 	for _, m := range machines {
+		// Make sure we shouldn't be stopping before we start the next machine
+		select {
+		case <-task.catacomb.Dying():
+			return task.catacomb.ErrDying()
+		default:
+		}
 
 		pInfo, err := m.ProvisioningInfo()
 		if err != nil {
@@ -656,8 +680,12 @@ func (task *provisionerTask) startMachines(machines []*apiprovisioner.Machine) e
 			arch = *pInfo.Constraints.Arch
 		}
 
+		v, err := m.ModelAgentVersion()
+		if err != nil {
+			return errors.Trace(err)
+		}
 		possibleTools, err := task.toolsFinder.FindTools(
-			jujuversion.Current,
+			*v,
 			pInfo.Series,
 			arch,
 		)
@@ -685,9 +713,9 @@ func (task *provisionerTask) startMachines(machines []*apiprovisioner.Machine) e
 
 func (task *provisionerTask) setErrorStatus(message string, machine *apiprovisioner.Machine, err error) error {
 	logger.Errorf(message, machine, err)
-	if err1 := machine.SetStatus(status.Error, err.Error(), nil); err1 != nil {
+	if err := machine.SetInstanceStatus(status.ProvisioningError, err.Error(), nil); err != nil {
 		// Something is wrong with this machine, better report it back.
-		return errors.Annotatef(err1, "cannot set error status for machine %q", machine)
+		return errors.Annotatef(err, "cannot set error status for machine %q", machine)
 	}
 	return nil
 }
@@ -698,6 +726,10 @@ func (task *provisionerTask) startMachine(
 	startInstanceParams environs.StartInstanceParams,
 ) error {
 	var result *environs.StartInstanceResult
+	// TODO (jam): 2017-01-19 Should we be setting this earlier in the cycle?
+	if err := machine.SetInstanceStatus(status.Provisioning, "starting", nil); err != nil {
+		logger.Errorf("%v", err)
+	}
 	for attemptsLeft := task.retryStartInstanceStrategy.retryCount; attemptsLeft >= 0; attemptsLeft-- {
 		attemptResult, err := task.broker.StartInstance(startInstanceParams)
 		if err == nil {
@@ -710,12 +742,12 @@ func (task *provisionerTask) startMachine(
 			return task.setErrorStatus("cannot start instance for machine %q: %v", machine, err)
 		}
 
-		logger.Warningf("%v", errors.Annotate(err, "starting instance"))
-		retryMsg := fmt.Sprintf("will retry to start instance in %v", task.retryStartInstanceStrategy.retryDelay)
-		if err2 := machine.SetStatus(status.Pending, retryMsg, nil); err2 != nil {
+		retryMsg := fmt.Sprintf("failed to start instance (%s), retrying in %v (%d more attempts)",
+			err.Error(), task.retryStartInstanceStrategy.retryDelay, attemptsLeft)
+		logger.Warningf(retryMsg)
+		if err2 := machine.SetInstanceStatus(status.Provisioning, retryMsg, nil); err2 != nil {
 			logger.Errorf("%v", err2)
 		}
-		logger.Infof(retryMsg)
 
 		select {
 		case <-task.catacomb.Dying():
@@ -788,6 +820,8 @@ func volumesToAPIserver(volumes []storage.Volume) []params.Volume {
 			params.VolumeInfo{
 				v.VolumeId,
 				v.HardwareId,
+				v.WWN,
+				"", // pool
 				v.Size,
 				v.Persistent,
 			},
