@@ -17,6 +17,7 @@ import (
 	"github.com/juju/version"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/names.v2"
+	worker "gopkg.in/juju/worker.v1"
 
 	"github.com/juju/juju/agent"
 	"github.com/juju/juju/api"
@@ -45,7 +46,6 @@ import (
 	coretesting "github.com/juju/juju/testing"
 	coretools "github.com/juju/juju/tools"
 	jujuversion "github.com/juju/juju/version"
-	"github.com/juju/juju/worker"
 	"github.com/juju/juju/worker/provisioner"
 )
 
@@ -69,7 +69,7 @@ func (s *CommonProvisionerSuite) assertProvisionerObservesConfigChanges(c *gc.C,
 	attrs := map[string]interface{}{
 		config.ProvisionerHarvestModeKey: config.HarvestAll.String(),
 	}
-	err := s.State.UpdateModelConfig(attrs, nil, nil)
+	err := s.State.UpdateModelConfig(attrs, nil)
 	c.Assert(err, jc.ErrorIsNil)
 
 	s.BackingState.StartSync()
@@ -160,7 +160,7 @@ func (s *CommonProvisionerSuite) SetUpTest(c *gc.C) {
 	current := version.Binary{
 		Number: jujuversion.Current,
 		Arch:   arch.HostArch(),
-		Series: series.HostSeries(),
+		Series: series.MustHostSeries(),
 	}
 	err = machine.SetAgentVersion(current)
 	c.Assert(err, jc.ErrorIsNil)
@@ -469,16 +469,21 @@ func (s *ProvisionerSuite) TestPossibleTools(c *gc.C) {
 	s.PatchValue(&tools.DefaultBaseURL, storageDir)
 	stor, err := filestorage.NewFileStorageWriter(storageDir)
 	c.Assert(err, jc.ErrorIsNil)
+	currentVersion := version.MustParseBinary("1.2.3-quantal-amd64")
+
+	// The current version is determined by the current model's agent
+	// version when locating tools to provision an added unit
+	attrs := map[string]interface{}{
+		config.AgentVersionKey: currentVersion.Number.String(),
+	}
+	err = s.State.UpdateModelConfig(attrs, nil)
+	c.Assert(err, jc.ErrorIsNil)
 
-	// Set a current version that does not match the
-	// agent-version in the environ config.
-	currentVersion := version.MustParseBinary("1.2.3-quantal-arm64")
 	s.PatchValue(&arch.HostArch, func() string { return currentVersion.Arch })
-	s.PatchValue(&series.HostSeries, func() string { return currentVersion.Series })
-	s.PatchValue(&jujuversion.Current, currentVersion.Number)
+	s.PatchValue(&series.MustHostSeries, func() string { return currentVersion.Series })
 
 	// Upload some plausible matches, and some that should be filtered out.
-	compatibleVersion := version.MustParseBinary("1.2.3-quantal-amd64")
+	compatibleVersion := version.MustParseBinary("1.2.3-quantal-arm64")
 	ignoreVersion1 := version.MustParseBinary("1.2.4-quantal-arm64")
 	ignoreVersion2 := version.MustParseBinary("1.2.3-precise-arm64")
 	availableVersions := []version.Binary{
@@ -522,19 +527,12 @@ func (s *ProvisionerSuite) TestProvisionerSetsErrorStatusWhenNoToolsAreAvailable
 	c.Assert(err, jc.ErrorIsNil)
 	s.checkNoOperations(c)
 
-	t0 := time.Now()
-	for time.Since(t0) < coretesting.LongWait {
-		// And check the machine status is set to error.
-		statusInfo, err := m.Status()
-		c.Assert(err, jc.ErrorIsNil)
-		if statusInfo.Status == status.Pending {
-			time.Sleep(coretesting.ShortWait)
-			continue
-		}
-		c.Assert(statusInfo.Status, gc.Equals, status.Error)
-		c.Assert(statusInfo.Message, gc.Equals, "no matching tools available")
-		break
-	}
+	// Ensure machine error status was set, and the error matches
+	agentStatus, instanceStatus := s.waitUntilMachineNotPending(c, m)
+	c.Check(agentStatus.Status, gc.Equals, status.Error)
+	c.Check(agentStatus.Message, gc.Equals, "no matching agent binaries available")
+	c.Check(instanceStatus.Status, gc.Equals, status.ProvisioningError)
+	c.Check(instanceStatus.Message, gc.Equals, "no matching agent binaries available")
 
 	// Restart the PA to make sure the machine is skipped again.
 	stop(c, p)
@@ -543,6 +541,31 @@ func (s *ProvisionerSuite) TestProvisionerSetsErrorStatusWhenNoToolsAreAvailable
 	s.checkNoOperations(c)
 }
 
+func (s *ProvisionerSuite) waitUntilMachineNotPending(c *gc.C, m *state.Machine) (status.StatusInfo, status.StatusInfo) {
+	t0 := time.Now()
+	for time.Since(t0) < coretesting.LongWait {
+		agentStatusInfo, err := m.Status()
+		c.Assert(err, jc.ErrorIsNil)
+		if agentStatusInfo.Status == status.Pending {
+			time.Sleep(coretesting.ShortWait)
+			continue
+		}
+		instanceStatusInfo, err := m.InstanceStatus()
+		c.Assert(err, jc.ErrorIsNil)
+		// officially InstanceStatus is only supposed to be Provisioning, but
+		// all current Providers have their unknown state as Pending.
+		if instanceStatusInfo.Status == status.Provisioning ||
+			instanceStatusInfo.Status == status.Pending {
+			time.Sleep(coretesting.ShortWait)
+			continue
+		}
+		return agentStatusInfo, instanceStatusInfo
+	}
+	c.Fatalf("machine %q stayed in pending", m.Id())
+	// Satisfy Go, Fatal should be a panic anyway
+	return status.StatusInfo{}, status.StatusInfo{}
+}
+
 func (s *ProvisionerSuite) TestProvisionerFailedStartInstanceWithInjectedCreationError(c *gc.C) {
 	// Set the retry delay to 0, and retry count to 2 to keep tests short
 	s.PatchValue(provisioner.RetryStrategyDelay, 0*time.Second)
@@ -569,21 +592,12 @@ func (s *ProvisionerSuite) TestProvisionerFailedStartInstanceWithInjectedCreatio
 	c.Assert(err, jc.ErrorIsNil)
 	s.checkNoOperations(c)
 
-	t0 := time.Now()
-	for time.Since(t0) < coretesting.LongWait {
-		// And check the machine status is set to error.
-		statusInfo, err := m.Status()
-		c.Assert(err, jc.ErrorIsNil)
-		if statusInfo.Status == status.Pending {
-			time.Sleep(coretesting.ShortWait)
-			continue
-		}
-		c.Assert(statusInfo.Status, gc.Equals, status.Error)
-		// check that the status matches the error message
-		c.Assert(statusInfo.Message, gc.Equals, destroyError.Error())
-		return
-	}
-	c.Fatal("Test took too long to complete")
+	agentStatus, instanceStatus := s.waitUntilMachineNotPending(c, m)
+	// check that the status matches the error message
+	c.Check(agentStatus.Status, gc.Equals, status.Error)
+	c.Check(agentStatus.Message, gc.Equals, destroyError.Error())
+	c.Check(instanceStatus.Status, gc.Equals, status.ProvisioningError)
+	c.Check(instanceStatus.Message, gc.Equals, destroyError.Error())
 }
 
 func (s *ProvisionerSuite) TestProvisionerSucceedStartInstanceWithInjectedRetryableCreationError(c *gc.C) {
@@ -635,7 +649,13 @@ func (s *ProvisionerSuite) TestProvisionerStopRetryingIfDying(c *gc.C) {
 	stop(c, p)
 	statusInfo, err := m.Status()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(statusInfo.Status, gc.Equals, status.Pending)
+	c.Check(statusInfo.Status, gc.Equals, status.Pending)
+	statusInfo, err = m.InstanceStatus()
+	c.Assert(err, jc.ErrorIsNil)
+	if statusInfo.Status != status.Pending && statusInfo.Status != status.Provisioning {
+		c.Errorf("statusInfo.Status was %q not one of %q or %q",
+			statusInfo.Status, status.Pending, status.Provisioning)
+	}
 	s.checkNoOperations(c)
 }
 
@@ -725,6 +745,10 @@ func (m *MockMachine) Status() (status.Status, string, error) {
 	return m.status, "", m.statusErr
 }
 
+func (m *MockMachine) InstanceStatus() (status.Status, string, error) {
+	return m.status, "", m.statusErr
+}
+
 func (m *MockMachine) Id() string {
 	return m.id
 }
@@ -906,19 +930,12 @@ func (s *ProvisionerSuite) testProvisioningFailsAndSetsErrorStatusForConstraints
 	// Expect StartInstance to fail.
 	s.checkNoOperations(c)
 
-	// Ensure machine error status was set.
-	t0 := time.Now()
-	for time.Since(t0) < coretesting.LongWait {
-		statusInfo, err := machine.Status()
-		c.Assert(err, jc.ErrorIsNil)
-		if statusInfo.Status == status.Pending {
-			time.Sleep(coretesting.ShortWait)
-			continue
-		}
-		c.Assert(statusInfo.Status, gc.Equals, status.Error)
-		c.Assert(statusInfo.Message, gc.Equals, expectedErrorStatus)
-		break
-	}
+	// Ensure machine error status was set, and the error matches
+	agentStatus, instanceStatus := s.waitUntilMachineNotPending(c, machine)
+	c.Check(agentStatus.Status, gc.Equals, status.Error)
+	c.Check(agentStatus.Message, gc.Equals, expectedErrorStatus)
+	c.Check(instanceStatus.Status, gc.Equals, status.ProvisioningError)
+	c.Check(instanceStatus.Message, gc.Equals, expectedErrorStatus)
 
 	// Make sure the task didn't stop with an error
 	died := make(chan error)
@@ -1241,6 +1258,36 @@ func (s *ProvisionerSuite) TestHarvestAllReapsAllTheThings(c *gc.C) {
 	s.waitForRemovalMark(c, m0)
 }
 
+func (s *ProvisionerSuite) TestStopInstancesIgnoresMachinesWithKeep(c *gc.C) {
+
+	task := s.newProvisionerTask(c,
+		config.HarvestAll,
+		s.Environ,
+		s.provisioner,
+		mockToolsFinder{},
+	)
+	defer stop(c, task)
+
+	// Create two machines, one with keep-instance=true.
+	m0, err := s.addMachine()
+	c.Assert(err, jc.ErrorIsNil)
+	i0 := s.checkStartInstance(c, m0)
+	m1, err := s.addMachine()
+	c.Assert(err, jc.ErrorIsNil)
+	i1 := s.checkStartInstance(c, m1)
+	err = m1.SetKeepInstance(true)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Mark them both as dead.
+	c.Assert(m0.EnsureDead(), gc.IsNil)
+	c.Assert(m1.EnsureDead(), gc.IsNil)
+
+	// Only one of the machines is stopped.
+	s.checkStopSomeInstances(c, []instance.Instance{i0}, []instance.Instance{i1})
+	s.waitForRemovalMark(c, m0)
+	s.waitForRemovalMark(c, m1)
+}
+
 func (s *ProvisionerSuite) TestProvisionerRetriesTransientErrors(c *gc.C) {
 	s.PatchValue(&apiserverprovisioner.ErrorRetryWaitDelay, 5*time.Millisecond)
 	e := &mockBroker{Environ: s.Environ, retryCount: make(map[string]int)}
@@ -1272,12 +1319,12 @@ func (s *ProvisionerSuite) TestProvisionerRetriesTransientErrors(c *gc.C) {
 			case <-time.After(coretesting.ShortWait):
 				now := time.Now()
 				sInfo := status.StatusInfo{
-					Status:  status.Error,
+					Status:  status.ProvisioningError,
 					Message: "info",
 					Data:    map[string]interface{}{"transient": true},
 					Since:   &now,
 				}
-				err := m3.SetStatus(sInfo)
+				err := m3.SetInstanceStatus(sInfo)
 				c.Assert(err, jc.ErrorIsNil)
 			}
 		}
@@ -1286,9 +1333,9 @@ func (s *ProvisionerSuite) TestProvisionerRetriesTransientErrors(c *gc.C) {
 	close(thatsAllFolks)
 
 	// Machine 4 is never provisioned.
-	statusInfo, err := m4.Status()
+	statusInfo, err := m4.InstanceStatus()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(statusInfo.Status, gc.Equals, status.Error)
+	c.Assert(statusInfo.Status, gc.Equals, status.ProvisioningError)
 	_, err = m4.InstanceId()
 	c.Assert(err, jc.Satisfies, errors.IsNotProvisioned)
 }
