@@ -4,17 +4,15 @@
 package operation_test
 
 import (
-	"time"
-
 	"github.com/juju/errors"
 	"github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v5/hooks"
+	"gopkg.in/juju/charm.v6-unstable/hooks"
 
 	"github.com/juju/juju/worker/uniter/hook"
 	"github.com/juju/juju/worker/uniter/operation"
-	"github.com/juju/juju/worker/uniter/runner"
+	"github.com/juju/juju/worker/uniter/runner/context"
 	"github.com/juju/juju/worker/uniter/runner/jujuc"
 )
 
@@ -26,42 +24,20 @@ var _ = gc.Suite(&RunHookSuite{})
 
 type newHook func(operation.Factory, hook.Info) (operation.Operation, error)
 
-func (s *RunHookSuite) testClearResolvedFlagError(c *gc.C, newHook newHook) {
-	callbacks := &PrepareHookCallbacks{
-		MockClearResolvedFlag: &MockNoArgs{err: errors.New("biff")},
-	}
-	factory := operation.NewFactory(nil, nil, callbacks, nil, nil)
-	op, err := newHook(factory, hook.Info{Kind: hooks.ConfigChanged})
-	c.Assert(err, jc.ErrorIsNil)
-
-	newState, err := op.Prepare(operation.State{})
-	c.Check(newState, gc.IsNil)
-	c.Check(callbacks.MockClearResolvedFlag.called, jc.IsTrue)
-	c.Check(err, gc.ErrorMatches, "biff")
-}
-
-func (s *RunHookSuite) TestClearResolvedFlagError_Retry(c *gc.C) {
-	s.testClearResolvedFlagError(c, (operation.Factory).NewRetryHook)
-}
-
-func (s *RunHookSuite) TestClearResolvedFlagError_Skip(c *gc.C) {
-	s.testClearResolvedFlagError(c, (operation.Factory).NewSkipHook)
-}
-
 func (s *RunHookSuite) testPrepareHookError(
 	c *gc.C, newHook newHook, expectClearResolvedFlag, expectSkip bool,
 ) {
 	callbacks := &PrepareHookCallbacks{
-		MockPrepareHook:       &MockPrepareHook{err: errors.New("pow")},
-		MockClearResolvedFlag: &MockNoArgs{},
+		MockPrepareHook: &MockPrepareHook{err: errors.New("pow")},
 	}
-	factory := operation.NewFactory(nil, nil, callbacks, nil, nil)
+	factory := operation.NewFactory(operation.FactoryParams{
+		Callbacks: callbacks,
+	})
 	op, err := newHook(factory, hook.Info{Kind: hooks.ConfigChanged})
 	c.Assert(err, jc.ErrorIsNil)
 
 	newState, err := op.Prepare(operation.State{})
 	c.Check(newState, gc.IsNil)
-	c.Check(callbacks.MockClearResolvedFlag.called, gc.Equals, expectClearResolvedFlag)
 	if expectSkip {
 		c.Check(err, gc.Equals, operation.ErrSkipExecute)
 		c.Check(callbacks.MockPrepareHook.gotHook, gc.IsNil)
@@ -73,12 +49,63 @@ func (s *RunHookSuite) testPrepareHookError(
 	})
 }
 
-func (s *RunHookSuite) TestPrepareHookError_Run(c *gc.C) {
-	s.testPrepareHookError(c, (operation.Factory).NewRunHook, false, false)
+func (s *RunHookSuite) TestPrepareHookCtxCalled(c *gc.C) {
+	ctx := &MockContext{}
+	callbacks := &PrepareHookCallbacks{
+		MockPrepareHook: &MockPrepareHook{},
+	}
+	runnerFactory := &MockRunnerFactory{
+		MockNewHookRunner: &MockNewHookRunner{
+			runner: &MockRunner{
+				context: ctx,
+			},
+		},
+	}
+	factory := operation.NewFactory(operation.FactoryParams{
+		RunnerFactory: runnerFactory,
+		Callbacks:     callbacks,
+	})
+
+	op, err := factory.NewRunHook(hook.Info{Kind: hooks.ConfigChanged})
+	c.Assert(err, jc.ErrorIsNil)
+
+	newState, err := op.Prepare(operation.State{})
+	c.Check(newState, gc.NotNil)
+	c.Assert(err, jc.ErrorIsNil)
+
+	ctx.CheckCall(c, 0, "Prepare")
 }
 
-func (s *RunHookSuite) TestPrepareHookError_Retry(c *gc.C) {
-	s.testPrepareHookError(c, (operation.Factory).NewRetryHook, true, false)
+func (s *RunHookSuite) TestPrepareHookCtxError(c *gc.C) {
+	ctx := &MockContext{}
+	ctx.SetErrors(errors.New("ctx prepare error"))
+	callbacks := &PrepareHookCallbacks{
+		MockPrepareHook: &MockPrepareHook{},
+	}
+	runnerFactory := &MockRunnerFactory{
+		MockNewHookRunner: &MockNewHookRunner{
+			runner: &MockRunner{
+				context: ctx,
+			},
+		},
+	}
+	factory := operation.NewFactory(operation.FactoryParams{
+		RunnerFactory: runnerFactory,
+		Callbacks:     callbacks,
+	})
+
+	op, err := factory.NewRunHook(hook.Info{Kind: hooks.ConfigChanged})
+	c.Assert(err, jc.ErrorIsNil)
+
+	newState, err := op.Prepare(operation.State{})
+	c.Check(newState, gc.IsNil)
+	c.Assert(err, gc.ErrorMatches, `ctx prepare error`)
+
+	ctx.CheckCall(c, 0, "Prepare")
+}
+
+func (s *RunHookSuite) TestPrepareHookError_Run(c *gc.C) {
+	s.testPrepareHookError(c, (operation.Factory).NewRunHook, false, false)
 }
 
 func (s *RunHookSuite) TestPrepareHookError_Skip(c *gc.C) {
@@ -90,7 +117,10 @@ func (s *RunHookSuite) testPrepareRunnerError(c *gc.C, newHook newHook) {
 	runnerFactory := &MockRunnerFactory{
 		MockNewHookRunner: &MockNewHookRunner{err: errors.New("splat")},
 	}
-	factory := operation.NewFactory(nil, runnerFactory, callbacks, nil, nil)
+	factory := operation.NewFactory(operation.FactoryParams{
+		RunnerFactory: runnerFactory,
+		Callbacks:     callbacks,
+	})
 	op, err := newHook(factory, hook.Info{Kind: hooks.ConfigChanged})
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -106,16 +136,15 @@ func (s *RunHookSuite) TestPrepareRunnerError_Run(c *gc.C) {
 	s.testPrepareRunnerError(c, (operation.Factory).NewRunHook)
 }
 
-func (s *RunHookSuite) TestPrepareRunnerError_Retry(c *gc.C) {
-	s.testPrepareRunnerError(c, (operation.Factory).NewRetryHook)
-}
-
 func (s *RunHookSuite) testPrepareSuccess(
 	c *gc.C, newHook newHook, before, after operation.State,
 ) {
 	runnerFactory := NewRunHookRunnerFactory(errors.New("should not call"))
 	callbacks := NewPrepareHookCallbacks()
-	factory := operation.NewFactory(nil, runnerFactory, callbacks, nil, nil)
+	factory := operation.NewFactory(operation.FactoryParams{
+		RunnerFactory: runnerFactory,
+		Callbacks:     callbacks,
+	})
 	op, err := newHook(factory, hook.Info{Kind: hooks.ConfigChanged})
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -125,41 +154,28 @@ func (s *RunHookSuite) testPrepareSuccess(
 }
 
 func (s *RunHookSuite) TestPrepareSuccess_BlankSlate(c *gc.C) {
-	for i, newHook := range []newHook{
+	s.testPrepareSuccess(c,
 		(operation.Factory).NewRunHook,
-		(operation.Factory).NewRetryHook,
-	} {
-		c.Logf("variant %d", i)
-		s.testPrepareSuccess(c,
-			newHook,
-			operation.State{},
-			operation.State{
-				Kind: operation.RunHook,
-				Step: operation.Pending,
-				Hook: &hook.Info{Kind: hooks.ConfigChanged},
-			},
-		)
-	}
+		operation.State{},
+		operation.State{
+			Kind: operation.RunHook,
+			Step: operation.Pending,
+			Hook: &hook.Info{Kind: hooks.ConfigChanged},
+		},
+	)
 }
 
 func (s *RunHookSuite) TestPrepareSuccess_Preserve(c *gc.C) {
-	for i, newHook := range []newHook{
+	s.testPrepareSuccess(c,
 		(operation.Factory).NewRunHook,
-		(operation.Factory).NewRetryHook,
-	} {
-		c.Logf("variant %d", i)
-		s.testPrepareSuccess(c,
-			newHook,
-			overwriteState,
-			operation.State{
-				Started:            true,
-				CollectMetricsTime: 1234567,
-				Kind:               operation.RunHook,
-				Step:               operation.Pending,
-				Hook:               &hook.Info{Kind: hooks.ConfigChanged},
-			},
-		)
-	}
+		overwriteState,
+		operation.State{
+			Started: true,
+			Kind:    operation.RunHook,
+			Step:    operation.Pending,
+			Hook:    &hook.Info{Kind: hooks.ConfigChanged},
+		},
+	)
 }
 
 func (s *RunHookSuite) getExecuteRunnerTest(c *gc.C, newHook newHook, kind hooks.Kind, runErr error) (operation.Operation, *ExecuteHookCallbacks, *MockRunnerFactory) {
@@ -169,17 +185,20 @@ func (s *RunHookSuite) getExecuteRunnerTest(c *gc.C, newHook newHook, kind hooks
 		MockNotifyHookCompleted: &MockNotify{},
 		MockNotifyHookFailed:    &MockNotify{},
 	}
-	factory := operation.NewFactory(nil, runnerFactory, callbacks, nil, nil)
+	factory := operation.NewFactory(operation.FactoryParams{
+		RunnerFactory: runnerFactory,
+		Callbacks:     callbacks,
+	})
 	op, err := newHook(factory, hook.Info{Kind: kind})
 	c.Assert(err, jc.ErrorIsNil)
 	return op, callbacks, runnerFactory
 }
 
-func (s *RunHookSuite) testExecuteMissingHookError(c *gc.C, newHook newHook) {
-	runErr := runner.NewMissingHookError("blah-blah")
+func (s *RunHookSuite) TestExecuteMissingHookError(c *gc.C) {
+	runErr := context.NewMissingHookError("blah-blah")
 	for _, kind := range hooks.UnitHooks() {
 		c.Logf("hook %v", kind)
-		op, callbacks, runnerFactory := s.getExecuteRunnerTest(c, newHook, kind, runErr)
+		op, callbacks, runnerFactory := s.getExecuteRunnerTest(c, (operation.Factory).NewRunHook, kind, runErr)
 		_, err := op.Prepare(operation.State{})
 		c.Assert(err, jc.ErrorIsNil)
 
@@ -200,17 +219,9 @@ func (s *RunHookSuite) testExecuteMissingHookError(c *gc.C, newHook newHook) {
 	}
 }
 
-func (s *RunHookSuite) TestExecuteMissingHookError_Run(c *gc.C) {
-	s.testExecuteMissingHookError(c, (operation.Factory).NewRunHook)
-}
-
-func (s *RunHookSuite) TestExecuteMissingHookError_Retry(c *gc.C) {
-	s.testExecuteMissingHookError(c, (operation.Factory).NewRetryHook)
-}
-
-func (s *RunHookSuite) testExecuteRequeueRebootError(c *gc.C, newHook newHook) {
-	runErr := runner.ErrRequeueAndReboot
-	op, callbacks, runnerFactory := s.getExecuteRunnerTest(c, newHook, hooks.ConfigChanged, runErr)
+func (s *RunHookSuite) TestExecuteRequeueRebootError(c *gc.C) {
+	runErr := context.ErrRequeueAndReboot
+	op, callbacks, runnerFactory := s.getExecuteRunnerTest(c, (operation.Factory).NewRunHook, hooks.ConfigChanged, runErr)
 	_, err := op.Prepare(operation.State{})
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -227,17 +238,9 @@ func (s *RunHookSuite) testExecuteRequeueRebootError(c *gc.C, newHook newHook) {
 	c.Assert(callbacks.MockNotifyHookFailed.gotName, gc.IsNil)
 }
 
-func (s *RunHookSuite) TestExecuteRequeueRebootError_Run(c *gc.C) {
-	s.testExecuteRequeueRebootError(c, (operation.Factory).NewRunHook)
-}
-
-func (s *RunHookSuite) TestExecuteRequeueRebootError_Retry(c *gc.C) {
-	s.testExecuteRequeueRebootError(c, (operation.Factory).NewRetryHook)
-}
-
-func (s *RunHookSuite) testExecuteRebootError(c *gc.C, newHook newHook) {
-	runErr := runner.ErrReboot
-	op, callbacks, runnerFactory := s.getExecuteRunnerTest(c, newHook, hooks.ConfigChanged, runErr)
+func (s *RunHookSuite) TestExecuteRebootError(c *gc.C) {
+	runErr := context.ErrReboot
+	op, callbacks, runnerFactory := s.getExecuteRunnerTest(c, (operation.Factory).NewRunHook, hooks.ConfigChanged, runErr)
 	_, err := op.Prepare(operation.State{})
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -254,17 +257,9 @@ func (s *RunHookSuite) testExecuteRebootError(c *gc.C, newHook newHook) {
 	c.Assert(callbacks.MockNotifyHookFailed.gotName, gc.IsNil)
 }
 
-func (s *RunHookSuite) TestExecuteRebootError_Run(c *gc.C) {
-	s.testExecuteRebootError(c, (operation.Factory).NewRunHook)
-}
-
-func (s *RunHookSuite) TestExecuteRebootError_Retry(c *gc.C) {
-	s.testExecuteRebootError(c, (operation.Factory).NewRetryHook)
-}
-
-func (s *RunHookSuite) testExecuteOtherError(c *gc.C, newHook newHook) {
+func (s *RunHookSuite) TestExecuteOtherError(c *gc.C) {
 	runErr := errors.New("graaargh")
-	op, callbacks, runnerFactory := s.getExecuteRunnerTest(c, newHook, hooks.ConfigChanged, runErr)
+	op, callbacks, runnerFactory := s.getExecuteRunnerTest(c, (operation.Factory).NewRunHook, hooks.ConfigChanged, runErr)
 	_, err := op.Prepare(operation.State{})
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -277,18 +272,48 @@ func (s *RunHookSuite) testExecuteOtherError(c *gc.C, newHook newHook) {
 	c.Assert(callbacks.MockNotifyHookCompleted.gotName, gc.IsNil)
 }
 
-func (s *RunHookSuite) TestExecuteOtherError_Run(c *gc.C) {
-	s.testExecuteOtherError(c, (operation.Factory).NewRunHook)
+func (s *RunHookSuite) TestInstallHookPreservesStatus(c *gc.C) {
+	op, callbacks, f := s.getExecuteRunnerTest(c, (operation.Factory).NewRunHook, hooks.Install, nil)
+	err := f.MockNewHookRunner.runner.Context().SetUnitStatus(jujuc.StatusInfo{Status: "blocked", Info: "no database"})
+	c.Assert(err, jc.ErrorIsNil)
+	st := operation.State{
+		StatusSet: true,
+	}
+	midState, err := op.Prepare(st)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(midState, gc.NotNil)
+
+	_, err = op.Execute(*midState)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(callbacks.executingMessage, gc.Equals, "running some-hook-name hook")
+	status, err := f.MockNewHookRunner.runner.Context().UnitStatus()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(status.Status, gc.Equals, "blocked")
+	c.Assert(status.Info, gc.Equals, "no database")
 }
 
-func (s *RunHookSuite) TestExecuteOtherError_Retry(c *gc.C) {
-	s.testExecuteOtherError(c, (operation.Factory).NewRetryHook)
+func (s *RunHookSuite) TestInstallHookWHenNoStatusSet(c *gc.C) {
+	op, callbacks, f := s.getExecuteRunnerTest(c, (operation.Factory).NewRunHook, hooks.Install, nil)
+	st := operation.State{
+		StatusSet: false,
+	}
+	midState, err := op.Prepare(st)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(midState, gc.NotNil)
+
+	_, err = op.Execute(*midState)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(callbacks.executingMessage, gc.Equals, "running some-hook-name hook")
+	status, err := f.MockNewHookRunner.runner.Context().UnitStatus()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(status.Status, gc.Equals, "maintenance")
+	c.Assert(status.Info, gc.Equals, "installing charm software")
 }
 
 func (s *RunHookSuite) testExecuteSuccess(
-	c *gc.C, newHook newHook, before, after operation.State, setStatusCalled bool,
+	c *gc.C, before, after operation.State, setStatusCalled bool,
 ) {
-	op, callbacks, f := s.getExecuteRunnerTest(c, newHook, hooks.ConfigChanged, nil)
+	op, callbacks, f := s.getExecuteRunnerTest(c, (operation.Factory).NewRunHook, hooks.ConfigChanged, nil)
 	f.MockNewHookRunner.runner.MockRunHook.setStatusCalled = setStatusCalled
 	midState, err := op.Prepare(before)
 	c.Assert(err, jc.ErrorIsNil)
@@ -301,51 +326,36 @@ func (s *RunHookSuite) testExecuteSuccess(
 }
 
 func (s *RunHookSuite) TestExecuteSuccess_BlankSlate(c *gc.C) {
-	for i, newHook := range []newHook{
-		(operation.Factory).NewRunHook,
-		(operation.Factory).NewRetryHook,
-	} {
-		c.Logf("variant %d", i)
-		s.testExecuteSuccess(c,
-			newHook,
-			operation.State{},
-			operation.State{
-				Kind:      operation.RunHook,
-				Step:      operation.Done,
-				Hook:      &hook.Info{Kind: hooks.ConfigChanged},
-				StatusSet: true,
-			},
-			true,
-		)
-	}
+	s.testExecuteSuccess(c,
+		operation.State{},
+		operation.State{
+			Kind:      operation.RunHook,
+			Step:      operation.Done,
+			Hook:      &hook.Info{Kind: hooks.ConfigChanged},
+			StatusSet: true,
+		},
+		true,
+	)
 }
 
 func (s *RunHookSuite) TestExecuteSuccess_Preserve(c *gc.C) {
-	for i, newHook := range []newHook{
-		(operation.Factory).NewRunHook,
-		(operation.Factory).NewRetryHook,
-	} {
-		c.Logf("variant %d", i)
-		s.testExecuteSuccess(c,
-			newHook,
-			overwriteState,
-			operation.State{
-				Started:            true,
-				CollectMetricsTime: 1234567,
-				Kind:               operation.RunHook,
-				Step:               operation.Done,
-				Hook:               &hook.Info{Kind: hooks.ConfigChanged},
-				StatusSet:          true,
-			},
-			true,
-		)
-	}
+	s.testExecuteSuccess(c,
+		overwriteState,
+		operation.State{
+			Started:   true,
+			Kind:      operation.RunHook,
+			Step:      operation.Done,
+			Hook:      &hook.Info{Kind: hooks.ConfigChanged},
+			StatusSet: true,
+		},
+		true,
+	)
 }
 
 func (s *RunHookSuite) testExecuteThenCharmStatus(
-	c *gc.C, newHook newHook, before, after operation.State, kind hooks.Kind, setStatusCalled bool,
+	c *gc.C, before, after operation.State, kind hooks.Kind, setStatusCalled bool,
 ) {
-	op, _, f := s.getExecuteRunnerTest(c, newHook, kind, nil)
+	op, _, f := s.getExecuteRunnerTest(c, (operation.Factory).NewRunHook, kind, nil)
 	f.MockNewHookRunner.runner.MockRunHook.setStatusCalled = setStatusCalled
 	midState, err := op.Prepare(before)
 	c.Assert(err, jc.ErrorIsNil)
@@ -415,37 +425,23 @@ func (s *RunHookSuite) testBeforeHookExecute(c *gc.C, newHook newHook, kind hook
 func (s *RunHookSuite) TestBeforeHookStatus(c *gc.C) {
 	for _, kind := range hooks.UnitHooks() {
 		c.Logf("hook %v", kind)
-		for i, newHook := range []newHook{
-			(operation.Factory).NewRunHook,
-			(operation.Factory).NewRetryHook,
-		} {
-			c.Logf("variant %d", i)
-			s.testBeforeHookExecute(c, newHook, kind)
-		}
+		s.testBeforeHookExecute(c, (operation.Factory).NewRunHook, kind)
 	}
 }
 
 func (s *RunHookSuite) testExecuteHookWithSetStatus(c *gc.C, kind hooks.Kind, setStatusCalled bool) {
-	for i, newHook := range []newHook{
-		(operation.Factory).NewRunHook,
-		(operation.Factory).NewRetryHook,
-	} {
-		c.Logf("variant %d", i)
-		s.testExecuteThenCharmStatus(c,
-			newHook,
-			overwriteState,
-			operation.State{
-				Started:            true,
-				CollectMetricsTime: 1234567,
-				Kind:               operation.RunHook,
-				Step:               operation.Done,
-				Hook:               &hook.Info{Kind: kind},
-				StatusSet:          setStatusCalled,
-			},
-			kind,
-			setStatusCalled,
-		)
-	}
+	s.testExecuteThenCharmStatus(c,
+		overwriteState,
+		operation.State{
+			Started:   true,
+			Kind:      operation.RunHook,
+			Step:      operation.Done,
+			Hook:      &hook.Info{Kind: kind},
+			StatusSet: setStatusCalled,
+		},
+		kind,
+		setStatusCalled,
+	)
 }
 
 func (s *RunHookSuite) TestExecuteHookWithSetStatus(c *gc.C) {
@@ -460,7 +456,9 @@ func (s *RunHookSuite) testCommitError(c *gc.C, newHook newHook) {
 	callbacks := &CommitHookCallbacks{
 		MockCommitHook: &MockCommitHook{nil, errors.New("pow")},
 	}
-	factory := operation.NewFactory(nil, nil, callbacks, nil, nil)
+	factory := operation.NewFactory(operation.FactoryParams{
+		Callbacks: callbacks,
+	})
 	op, err := newHook(factory, hook.Info{Kind: hooks.ConfigChanged})
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -473,10 +471,6 @@ func (s *RunHookSuite) TestCommitError_Run(c *gc.C) {
 	s.testCommitError(c, (operation.Factory).NewRunHook)
 }
 
-func (s *RunHookSuite) TestCommitError_Retry(c *gc.C) {
-	s.testCommitError(c, (operation.Factory).NewRetryHook)
-}
-
 func (s *RunHookSuite) TestCommitError_Skip(c *gc.C) {
 	s.testCommitError(c, (operation.Factory).NewSkipHook)
 }
@@ -485,7 +479,9 @@ func (s *RunHookSuite) testCommitSuccess(c *gc.C, newHook newHook, hookInfo hook
 	callbacks := &CommitHookCallbacks{
 		MockCommitHook: &MockCommitHook{},
 	}
-	factory := operation.NewFactory(nil, nil, callbacks, nil, nil)
+	factory := operation.NewFactory(operation.FactoryParams{
+		Callbacks: callbacks,
+	})
 	op, err := newHook(factory, hookInfo)
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -497,7 +493,6 @@ func (s *RunHookSuite) testCommitSuccess(c *gc.C, newHook newHook, hookInfo hook
 func (s *RunHookSuite) TestCommitSuccess_ConfigChanged_QueueStartHook(c *gc.C) {
 	for i, newHook := range []newHook{
 		(operation.Factory).NewRunHook,
-		(operation.Factory).NewRetryHook,
 		(operation.Factory).NewSkipHook,
 	} {
 		c.Logf("variant %d", i)
@@ -517,7 +512,6 @@ func (s *RunHookSuite) TestCommitSuccess_ConfigChanged_QueueStartHook(c *gc.C) {
 func (s *RunHookSuite) TestCommitSuccess_ConfigChanged_Preserve(c *gc.C) {
 	for i, newHook := range []newHook{
 		(operation.Factory).NewRunHook,
-		(operation.Factory).NewRetryHook,
 		(operation.Factory).NewSkipHook,
 	} {
 		c.Logf("variant %d", i)
@@ -526,10 +520,9 @@ func (s *RunHookSuite) TestCommitSuccess_ConfigChanged_Preserve(c *gc.C) {
 			hook.Info{Kind: hooks.ConfigChanged},
 			overwriteState,
 			operation.State{
-				Started:            true,
-				CollectMetricsTime: 1234567,
-				Kind:               operation.Continue,
-				Step:               operation.Pending,
+				Started: true,
+				Kind:    operation.Continue,
+				Step:    operation.Pending,
 			},
 		)
 	}
@@ -538,7 +531,6 @@ func (s *RunHookSuite) TestCommitSuccess_ConfigChanged_Preserve(c *gc.C) {
 func (s *RunHookSuite) TestCommitSuccess_Start_SetStarted(c *gc.C) {
 	for i, newHook := range []newHook{
 		(operation.Factory).NewRunHook,
-		(operation.Factory).NewRetryHook,
 		(operation.Factory).NewSkipHook,
 	} {
 		c.Logf("variant %d", i)
@@ -558,7 +550,6 @@ func (s *RunHookSuite) TestCommitSuccess_Start_SetStarted(c *gc.C) {
 func (s *RunHookSuite) TestCommitSuccess_Start_Preserve(c *gc.C) {
 	for i, newHook := range []newHook{
 		(operation.Factory).NewRunHook,
-		(operation.Factory).NewRetryHook,
 		(operation.Factory).NewSkipHook,
 	} {
 		c.Logf("variant %d", i)
@@ -567,10 +558,9 @@ func (s *RunHookSuite) TestCommitSuccess_Start_Preserve(c *gc.C) {
 			hook.Info{Kind: hooks.Start},
 			overwriteState,
 			operation.State{
-				Started:            true,
-				CollectMetricsTime: 1234567,
-				Kind:               operation.Continue,
-				Step:               operation.Pending,
+				Started: true,
+				Kind:    operation.Continue,
+				Step:    operation.Pending,
 			},
 		)
 	}
@@ -579,7 +569,6 @@ func (s *RunHookSuite) TestCommitSuccess_Start_Preserve(c *gc.C) {
 func (s *RunHookSuite) testQueueHook_BlankSlate(c *gc.C, cause hooks.Kind) {
 	for i, newHook := range []newHook{
 		(operation.Factory).NewRunHook,
-		(operation.Factory).NewRetryHook,
 		(operation.Factory).NewSkipHook,
 	} {
 		c.Logf("variant %d", i)
@@ -607,7 +596,6 @@ func (s *RunHookSuite) testQueueHook_BlankSlate(c *gc.C, cause hooks.Kind) {
 func (s *RunHookSuite) testQueueHook_Preserve(c *gc.C, cause hooks.Kind) {
 	for i, newHook := range []newHook{
 		(operation.Factory).NewRunHook,
-		(operation.Factory).NewRetryHook,
 		(operation.Factory).NewSkipHook,
 	} {
 		c.Logf("variant %d", i)
@@ -623,12 +611,11 @@ func (s *RunHookSuite) testQueueHook_Preserve(c *gc.C, cause hooks.Kind) {
 			hook.Info{Kind: cause},
 			overwriteState,
 			operation.State{
-				Kind:               operation.RunHook,
-				Step:               operation.Queued,
-				Started:            true,
-				Stopped:            cause == hooks.Stop,
-				Hook:               hi,
-				CollectMetricsTime: 1234567,
+				Kind:    operation.RunHook,
+				Step:    operation.Queued,
+				Started: true,
+				Stopped: cause == hooks.Stop,
+				Hook:    hi,
 			},
 		)
 	}
@@ -645,7 +632,6 @@ func (s *RunHookSuite) TestQueueHook_UpgradeCharm_Preserve(c *gc.C) {
 func (s *RunHookSuite) testQueueNothing_BlankSlate(c *gc.C, hookInfo hook.Info) {
 	for i, newHook := range []newHook{
 		(operation.Factory).NewRunHook,
-		(operation.Factory).NewRetryHook,
 		(operation.Factory).NewSkipHook,
 	} {
 		c.Logf("variant %d", i)
@@ -654,9 +640,10 @@ func (s *RunHookSuite) testQueueNothing_BlankSlate(c *gc.C, hookInfo hook.Info)
 			hookInfo,
 			operation.State{},
 			operation.State{
-				Kind:    operation.Continue,
-				Step:    operation.Pending,
-				Stopped: hookInfo.Kind == hooks.Stop,
+				Installed: hookInfo.Kind == hooks.Install,
+				Kind:      operation.Continue,
+				Step:      operation.Pending,
+				Stopped:   hookInfo.Kind == hooks.Stop,
 			},
 		)
 	}
@@ -665,7 +652,6 @@ func (s *RunHookSuite) testQueueNothing_BlankSlate(c *gc.C, hookInfo hook.Info)
 func (s *RunHookSuite) testQueueNothing_Preserve(c *gc.C, hookInfo hook.Info) {
 	for i, newHook := range []newHook{
 		(operation.Factory).NewRunHook,
-		(operation.Factory).NewRetryHook,
 		(operation.Factory).NewSkipHook,
 	} {
 		c.Logf("variant %d", i)
@@ -674,11 +660,11 @@ func (s *RunHookSuite) testQueueNothing_Preserve(c *gc.C, hookInfo hook.Info) {
 			hookInfo,
 			overwriteState,
 			operation.State{
-				Kind:               operation.Continue,
-				Step:               operation.Pending,
-				Started:            true,
-				Stopped:            hookInfo.Kind == hooks.Stop,
-				CollectMetricsTime: 1234567,
+				Kind:      operation.Continue,
+				Step:      operation.Pending,
+				Installed: hookInfo.Kind == hooks.Install,
+				Started:   true,
+				Stopped:   hookInfo.Kind == hooks.Stop,
 			},
 		)
 	}
@@ -762,47 +748,8 @@ func (s *RunHookSuite) TestQueueNothing_RelationBroken_Preserve(c *gc.C) {
 	})
 }
 
-func (s *RunHookSuite) testCommitSuccess_CollectMetricsTime(c *gc.C, newHook newHook) {
-	callbacks := &CommitHookCallbacks{
-		MockCommitHook: &MockCommitHook{},
-	}
-	factory := operation.NewFactory(nil, nil, callbacks, nil, nil)
-	op, err := newHook(factory, hook.Info{Kind: hooks.CollectMetrics})
-	c.Assert(err, jc.ErrorIsNil)
-
-	nowBefore := time.Now().Unix()
-	newState, err := op.Commit(overwriteState)
-	c.Assert(err, jc.ErrorIsNil)
-
-	nowAfter := time.Now().Unix()
-	nowWritten := newState.CollectMetricsTime
-	c.Logf("%d <= %d <= %d", nowBefore, nowWritten, nowAfter)
-	c.Check(nowBefore <= nowWritten, jc.IsTrue)
-	c.Check(nowWritten <= nowAfter, jc.IsTrue)
-
-	// Check the other fields match.
-	newState.CollectMetricsTime = 0
-	c.Check(newState, gc.DeepEquals, &operation.State{
-		Started: true,
-		Kind:    operation.Continue,
-		Step:    operation.Pending,
-	})
-}
-
-func (s *RunHookSuite) TestCommitSuccess_CollectMetricsTime_Run(c *gc.C) {
-	s.testCommitSuccess_CollectMetricsTime(c, (operation.Factory).NewRunHook)
-}
-
-func (s *RunHookSuite) TestCommitSuccess_CollectMetricsTime_Retry(c *gc.C) {
-	s.testCommitSuccess_CollectMetricsTime(c, (operation.Factory).NewRetryHook)
-}
-
-func (s *RunHookSuite) TestCommitSuccess_CollectMetricsTime_Skip(c *gc.C) {
-	s.testCommitSuccess_CollectMetricsTime(c, (operation.Factory).NewSkipHook)
-}
-
 func (s *RunHookSuite) testNeedsGlobalMachineLock(c *gc.C, newHook newHook, expected bool) {
-	factory := operation.NewFactory(nil, nil, nil, nil, nil)
+	factory := operation.NewFactory(operation.FactoryParams{})
 	op, err := newHook(factory, hook.Info{Kind: hooks.ConfigChanged})
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(op.NeedsGlobalMachineLock(), gc.Equals, expected)
@@ -812,10 +759,6 @@ func (s *RunHookSuite) TestNeedsGlobalMachineLock_Run(c *gc.C) {
 	s.testNeedsGlobalMachineLock(c, (operation.Factory).NewRunHook, true)
 }
 
-func (s *RunHookSuite) TestNeedsGlobalMachineLock_Retry(c *gc.C) {
-	s.testNeedsGlobalMachineLock(c, (operation.Factory).NewRetryHook, true)
-}
-
 func (s *RunHookSuite) TestNeedsGlobalMachineLock_Skip(c *gc.C) {
 	s.testNeedsGlobalMachineLock(c, (operation.Factory).NewSkipHook, false)
 }
