@@ -106,6 +106,9 @@ type HookContext struct {
 	// LeadershipContext supplies several jujuc.Context methods.
 	LeadershipContext
 
+	// principal is the unitName of the principal charm.
+	principal string
+
 	// privateAddress is the cached value of the unit's private
 	// address.
 	privateAddress string
@@ -207,6 +210,9 @@ type HookContext struct {
 
 	componentDir   func(string) string
 	componentFuncs map[string]ComponentFunc
+
+	//  slaLevel contains the current SLA level.
+	slaLevel string
 }
 
 // Component implements jujuc.Context.
@@ -579,7 +585,9 @@ func (context *HookContext) HookVars(paths Paths) ([]string, error) {
 		"JUJU_API_ADDRESSES="+strings.Join(context.apiAddrs, " "),
 		"JUJU_METER_STATUS="+context.meterStatus.code,
 		"JUJU_METER_INFO="+context.meterStatus.info,
+		"JUJU_SLA="+context.slaLevel,
 		"JUJU_MACHINE_ID="+context.assignedMachineTag.Id(),
+		"JUJU_PRINCIPAL_UNIT="+context.principal,
 		"JUJU_AVAILABILITY_ZONE="+context.availabilityzone,
 	)
 	if r, err := context.HookRelation(); err == nil {
@@ -828,3 +836,8 @@ func (ctx *HookContext) SetUnitWorkloadVersion(version string) error {
 	}
 	return result.OneError()
 }
+
+// NetworkInfo returns the network info for the given bindingNames.
+func (ctx *HookContext) NetworkInfo(bindingNames []string) (map[string]params.NetworkInfoResult, error) {
+	return ctx.unit.NetworkInfo(bindingNames)
+}
