@@ -738,9 +738,9 @@ resources:
 
 func (s *UniterSuite) TestUniterUpgradeOverwrite(c *gc.C) {
 	//TODO(bogdanteleaga): Fix this on windows
-	if runtime.GOOS == "windows" {
-		c.Skip("bug 1403084: currently does not work on windows")
-	}
+	coretesting.SkipIfWindowsBug(c, "lp:1403084")
+	//TODO(hml): Fix this on S390X, intermittent there.
+	coretesting.SkipIfS390X(c, "lp:1534637")
 	makeTest := func(description string, content, extraChecks ft.Entries) uniterTest {
 		return ut(description,
 			createCharm{
@@ -1833,16 +1833,25 @@ func (s *UniterSuite) TestStorage(c *gc.C) {
 			err := f.Close()
 			c.Assert(err, jc.ErrorIsNil)
 		}()
-		_, err = io.WriteString(f, "storage:\n  wp-content:\n    type: filesystem\n")
+		_, err = io.WriteString(f, `
+storage:
+  wp-content:
+    type: filesystem
+    multiple:
+      range: 0-
+`[1:])
 		c.Assert(err, jc.ErrorIsNil)
 	}
+	storageConstraints := map[string]state.StorageConstraints{
+		"wp-content": {Count: 1},
+	}
 	s.runUniterTests(c, []uniterTest{
 		ut(
 			"test that storage-attached is called",
 			createCharm{customize: appendStorageMetadata},
 			serveCharm{},
 			ensureStateWorker{},
-			createServiceAndUnit{},
+			createServiceAndUnit{storage: storageConstraints},
 			provisionStorage{},
 			startUniter{},
 			waitAddresses{},
@@ -1853,7 +1862,7 @@ func (s *UniterSuite) TestStorage(c *gc.C) {
 			createCharm{customize: appendStorageMetadata},
 			serveCharm{},
 			ensureStateWorker{},
-			createServiceAndUnit{},
+			createServiceAndUnit{storage: storageConstraints},
 			provisionStorage{},
 			startUniter{},
 			waitAddresses{},
@@ -1870,7 +1879,7 @@ func (s *UniterSuite) TestStorage(c *gc.C) {
 			createCharm{customize: appendStorageMetadata},
 			serveCharm{},
 			ensureStateWorker{},
-			createServiceAndUnit{},
+			createServiceAndUnit{storage: storageConstraints},
 			// provision and destroy the storage before the uniter starts,
 			// to ensure it never sees the storage as attached
 			provisionStorage{},
@@ -1887,7 +1896,7 @@ func (s *UniterSuite) TestStorage(c *gc.C) {
 			createCharm{customize: appendStorageMetadata},
 			serveCharm{},
 			ensureStateWorker{},
-			createServiceAndUnit{},
+			createServiceAndUnit{storage: storageConstraints},
 			startUniter{},
 			// no hooks should be run, as storage isn't provisioned
 			waitHooks{},
@@ -1899,7 +1908,7 @@ func (s *UniterSuite) TestStorage(c *gc.C) {
 			createCharm{customize: appendStorageMetadata},
 			serveCharm{},
 			ensureStateWorker{},
-			createServiceAndUnit{},
+			createServiceAndUnit{storage: storageConstraints},
 			unitDying,
 			startUniter{},
 			// no hooks should be run, and unit agent should terminate
@@ -1913,6 +1922,8 @@ func (s *UniterSuite) TestStorage(c *gc.C) {
 	})
 }
 
+var mockExecutorErr = errors.New("some error occurred")
+
 type mockExecutor struct {
 	operation.Executor
 }
@@ -1923,7 +1934,7 @@ func (m *mockExecutor) Run(op operation.Operation) error {
 		return m.Executor.Run(op)
 	}
 	// but hooks should error
-	return errors.New("some error occurred")
+	return mockExecutorErr
 }
 
 func (s *UniterSuite) TestOperationErrorReported(c *gc.C) {
@@ -1946,3 +1957,31 @@ func (s *UniterSuite) TestOperationErrorReported(c *gc.C) {
 		),
 	})
 }
+
+func (s *UniterSuite) TestTranslateResolverError(c *gc.C) {
+	executorFunc := func(stateFilePath string, getInstallCharm func() (*corecharm.URL, error), acquireLock func() (mutex.Releaser, error)) (operation.Executor, error) {
+		e, err := operation.NewExecutor(stateFilePath, getInstallCharm, acquireLock)
+		c.Assert(err, jc.ErrorIsNil)
+		return &mockExecutor{e}, nil
+	}
+	translateResolverErr := func(in error) error {
+		c.Check(errors.Cause(in), gc.Equals, mockExecutorErr)
+		return errors.New("some other error")
+	}
+	s.runUniterTests(c, []uniterTest{
+		ut(
+			"resolver errors are translated",
+			createCharm{},
+			serveCharm{},
+			createUniter{
+				executorFunc:         executorFunc,
+				translateResolverErr: translateResolverErr,
+			},
+			waitUnitAgent{
+				status: status.Failed,
+				info:   "resolver loop error",
+			},
+			expectError{".*some other error.*"},
+		),
+	})
+}
