@@ -28,6 +28,7 @@ import (
 	gc "gopkg.in/check.v1"
 	corecharm "gopkg.in/juju/charm.v6-unstable"
 	"gopkg.in/juju/names.v2"
+	"gopkg.in/juju/worker.v1"
 	goyaml "gopkg.in/yaml.v2"
 
 	"github.com/juju/juju/api"
@@ -43,10 +44,11 @@ import (
 	"github.com/juju/juju/status"
 	"github.com/juju/juju/testcharms"
 	coretesting "github.com/juju/juju/testing"
-	"github.com/juju/juju/worker"
+	jworker "github.com/juju/juju/worker"
 	"github.com/juju/juju/worker/fortress"
 	"github.com/juju/juju/worker/uniter"
 	"github.com/juju/juju/worker/uniter/operation"
+	"github.com/juju/juju/worker/uniter/remotestate"
 )
 
 // worstCase is used for timeouts when timing out
@@ -133,6 +135,16 @@ func (ctx *context) run(c *gc.C, steps []stepper) {
 		if ctx.uniter != nil {
 			err := worker.Stop(ctx.uniter)
 			if ctx.err == "" {
+				if errors.Cause(err) == mutex.ErrCancelled {
+					// This can happen if the uniter lock acquire was
+					// temporarily blocked by test code holding the
+					// lock (like in waitHooks). The acquire call is
+					// delaying but then gets cancelled, and that
+					// error bubbles up to here.
+					// lp:1635664
+					c.Logf("ignoring lock acquire cancelled by stop")
+					return
+				}
 				c.Assert(err, jc.ErrorIsNil)
 			} else {
 				c.Assert(err, gc.ErrorMatches, ctx.err)
@@ -282,9 +294,6 @@ func (s ensureStateWorker) step(c *gc.C, ctx *context) {
 	if err != nil || len(addresses) == 0 {
 		addControllerMachine(c, ctx.st)
 	}
-	addresses, err = ctx.st.APIAddressesFromMachines()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(addresses, gc.HasLen, 1)
 }
 
 func addControllerMachine(c *gc.C, st *state.State) {
@@ -396,6 +405,7 @@ func (s serveCharm) step(c *gc.C, ctx *context) {
 
 type createServiceAndUnit struct {
 	serviceName string
+	storage     map[string]state.StorageConstraints
 }
 
 func (csau createServiceAndUnit) step(c *gc.C, ctx *context) {
@@ -404,7 +414,7 @@ func (csau createServiceAndUnit) step(c *gc.C, ctx *context) {
 	}
 	sch, err := ctx.st.Charm(curl(0))
 	c.Assert(err, jc.ErrorIsNil)
-	svc := ctx.s.AddTestingService(c, csau.serviceName, sch)
+	svc := ctx.s.AddTestingServiceWithStorage(c, csau.serviceName, sch, csau.storage)
 	unit, err := svc.AddUnit()
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -417,8 +427,9 @@ func (csau createServiceAndUnit) step(c *gc.C, ctx *context) {
 }
 
 type createUniter struct {
-	minion       bool
-	executorFunc uniter.NewExecutorFunc
+	minion               bool
+	executorFunc         uniter.NewExecutorFunc
+	translateResolverErr func(error) error
 }
 
 func (s createUniter) step(c *gc.C, ctx *context) {
@@ -427,7 +438,10 @@ func (s createUniter) step(c *gc.C, ctx *context) {
 	if s.minion {
 		step(c, ctx, forceMinion{})
 	}
-	step(c, ctx, startUniter{newExecutorFunc: s.executorFunc})
+	step(c, ctx, startUniter{
+		newExecutorFunc:      s.executorFunc,
+		translateResolverErr: s.translateResolverErr,
+	})
 	step(c, ctx, waitAddresses{})
 }
 
@@ -464,8 +478,9 @@ func hookExecutionLockName() string {
 }
 
 type startUniter struct {
-	unitTag         string
-	newExecutorFunc uniter.NewExecutorFunc
+	unitTag              string
+	newExecutorFunc      uniter.NewExecutorFunc
+	translateResolverErr func(error) error
 }
 
 func (s startUniter) step(c *gc.C, ctx *context) {
@@ -496,8 +511,9 @@ func (s startUniter) step(c *gc.C, ctx *context) {
 		DataDir:              ctx.dataDir,
 		Downloader:           downloader,
 		MachineLockName:      hookExecutionLockName(),
-		UpdateStatusSignal:   ctx.updateStatusHookTicker.ReturnTimer,
+		UpdateStatusSignal:   ctx.updateStatusHookTicker.ReturnTimer(),
 		NewOperationExecutor: operationExecutor,
+		TranslateResolverErr: s.translateResolverErr,
 		Observer:             ctx,
 		// TODO(axw) 2015-11-02 #1512191
 		// update tests that rely on timing to advance clock
@@ -527,11 +543,11 @@ func (s waitUniterDead) step(c *gc.C, ctx *context) {
 	// mimics the behaviour of the unit agent and verifies that the UA will,
 	// eventually, see the correct error and respond appropriately.
 	err := s.waitDead(c, ctx)
-	if err != worker.ErrTerminateAgent {
+	if err != jworker.ErrTerminateAgent {
 		step(c, ctx, startUniter{})
 		err = s.waitDead(c, ctx)
 	}
-	c.Assert(err, gc.Equals, worker.ErrTerminateAgent)
+	c.Assert(err, gc.Equals, jworker.ErrTerminateAgent)
 	err = ctx.unit.Refresh()
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(ctx.unit.Life(), gc.Equals, state.Dead)
@@ -1440,7 +1456,7 @@ func (s setProxySettings) step(c *gc.C, ctx *context) {
 		"ftp-proxy":   s.Ftp,
 		"no-proxy":    s.NoProxy,
 	}
-	err := ctx.st.UpdateModelConfig(attrs, nil, nil)
+	err := ctx.st.UpdateModelConfig(attrs, nil)
 	c.Assert(err, jc.ErrorIsNil)
 }
 
@@ -1752,7 +1768,7 @@ func (s destroyStorageAttachment) step(c *gc.C, ctx *context) {
 	storageAttachments, err := ctx.st.UnitStorageAttachments(ctx.unit.UnitTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(storageAttachments, gc.HasLen, 1)
-	err = ctx.st.DestroyStorageAttachment(
+	err = ctx.st.DetachStorage(
 		storageAttachments[0].StorageInstance(),
 		ctx.unit.UnitTag(),
 	)
@@ -1791,9 +1807,19 @@ func (t *manualTicker) Tick() error {
 	return nil
 }
 
+type dummyWaiter struct {
+	c chan time.Time
+}
+
+func (w dummyWaiter) After() <-chan time.Time {
+	return w.c
+}
+
 // ReturnTimer can be used to replace the update status signal generator.
-func (t *manualTicker) ReturnTimer() <-chan time.Time {
-	return t.c
+func (t *manualTicker) ReturnTimer() remotestate.UpdateStatusTimerFunc {
+	return func(_ time.Duration) remotestate.Waiter {
+		return dummyWaiter{t.c}
+	}
 }
 
 func newManualTicker() *manualTicker {
