@@ -72,7 +72,7 @@ func (s *UpgraderSuite) SetUpTest(c *gc.C) {
 
 func (s *UpgraderSuite) patchVersion(v version.Binary) {
 	s.PatchValue(&arch.HostArch, func() string { return v.Arch })
-	s.PatchValue(&series.HostSeries, func() string { return v.Series })
+	s.PatchValue(&series.MustHostSeries, func() string { return v.Series })
 	s.PatchValue(&jujuversion.Current, v.Number)
 }
 
