@@ -15,6 +15,7 @@ package txn
 import (
 	stderrors "errors"
 	"strings"
+	"time"
 
 	"github.com/juju/loggo"
 	"gopkg.in/mgo.v2"
@@ -54,6 +55,28 @@ var (
 // TransactionSource defines a function that can return transaction operations to run.
 type TransactionSource func(attempt int) ([]txn.Op, error)
 
+// PruneOptions controls when we will trigger a database prune.
+type PruneOptions struct {
+
+	// PruneFactor will trigger a prune when the current count of
+	// transactions in the database is greater than old*PruneFactor
+	PruneFactor float32
+
+	// MinNewTransactions will skip a prune even if pruneFactor is true
+	// if there are less than MinNewTransactions that might be cleaned up.
+	MinNewTransactions int
+
+	// MaxNewTransactions will force a prune if it sees more than
+	// MaxNewTransactions since the last run.
+	MaxNewTransactions int
+
+	// MaxTime sets a threshold for 'completed' transactions. Transactions
+	// will be considered completed only if they are both older than
+	// MaxTime and have a status of Completed or Aborted. Passing the
+	// zero Time will cause us to only filter on the Status field.
+	MaxTime time.Time
+}
+
 // Runner instances applies operations to collections in a database.
 type Runner interface {
 	// RunTransaction applies the specified transaction operations to a database.
@@ -75,7 +98,7 @@ type Runner interface {
 	//
 	//   txn_count >= pruneFactor * txn_count_at_last_prune
 	//
-	MaybePruneTransactions(pruneFactor float32) error
+	MaybePruneTransactions(pruneOpts PruneOptions) error
 }
 
 type txnRunner interface {
@@ -88,6 +111,7 @@ type transactionRunner struct {
 	transactionCollectionName string
 	changeLogName             string
 	testHooks                 chan ([]TestHook)
+	runTransactionObserver    func([]txn.Op, error)
 
 	newRunner func() txnRunner
 }
@@ -109,6 +133,11 @@ type RunnerParams struct {
 	// ChangeLogName is the mgo transaction runner change log,
 	// defaults to "txns.log" if unspecified.
 	ChangeLogName string
+
+	// RunTransactionObserver, if non-nil, will be called when
+	// a Run or RunTransaction call has completed. It will be
+	// passed the txn.Ops and the error result.
+	RunTransactionObserver func([]txn.Op, error)
 }
 
 // NewRunner returns a Runner which runs transactions for the database specified in params.
@@ -119,6 +148,7 @@ func NewRunner(params RunnerParams) Runner {
 		db: params.Database,
 		transactionCollectionName: params.TransactionCollectionName,
 		changeLogName:             params.ChangeLogName,
+		runTransactionObserver:    params.RunTransactionObserver,
 	}
 	if txnRunner.transactionCollectionName == "" {
 		txnRunner.transactionCollectionName = defaultTxnCollectionName
@@ -152,6 +182,10 @@ func (tr *transactionRunner) Run(transactions TransactionSource) error {
 		if err != nil {
 			return err
 		}
+		if len(ops) == 0 {
+			// Treat this the same as ErrNoOperations but don't suppress other errors.
+			return nil
+		}
 		if err := tr.RunTransaction(ops); err == nil {
 			return nil
 		} else if err != txn.ErrAborted {
@@ -193,7 +227,11 @@ func (tr *transactionRunner) RunTransaction(ops []txn.Op) error {
 		}
 	}
 	runner := tr.newRunner()
-	return runner.Run(ops, "", nil)
+	err := runner.Run(ops, "", nil)
+	if tr.runTransactionObserver != nil {
+		tr.runTransactionObserver(ops, err)
+	}
+	return err
 }
 
 // ResumeTransactions is defined on Runner.
@@ -203,8 +241,8 @@ func (tr *transactionRunner) ResumeTransactions() error {
 }
 
 // MaybePruneTransactions is defined on Runner.
-func (tr *transactionRunner) MaybePruneTransactions(pruneFactor float32) error {
-	return maybePrune(tr.db, tr.transactionCollectionName, pruneFactor)
+func (tr *transactionRunner) MaybePruneTransactions(pruneOpts PruneOptions) error {
+	return maybePrune(tr.db, tr.transactionCollectionName, pruneOpts)
 }
 
 // TestHook holds a pair of functions to be called before and after a
