@@ -31,7 +31,10 @@ import (
 
 	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/logging"
+	"github.com/lxc/lxd/shared/osarch"
+	"github.com/lxc/lxd/shared/version"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
@@ -77,8 +80,6 @@ type Daemon struct {
 	shutdownChan        chan bool
 	resetAutoUpdateChan chan bool
 
-	Storage storage
-
 	TCPSocket  *Socket
 	UnixSocket *Socket
 
@@ -87,9 +88,6 @@ type Daemon struct {
 	MockMode  bool
 	SetupMode bool
 
-	imagesDownloading     map[string]chan bool
-	imagesDownloadingLock sync.RWMutex
-
 	tlsConfig *tls.Config
 
 	proxy func(req *http.Request) (*url.URL, error)
@@ -107,10 +105,10 @@ type Command struct {
 	patch         func(d *Daemon, r *http.Request) Response
 }
 
-func (d *Daemon) httpGetSync(url string, certificate string) (*lxd.Response, error) {
+func (d *Daemon) httpClient(certificate string) (*http.Client, error) {
 	var err error
-
 	var cert *x509.Certificate
+
 	if certificate != "" {
 		certBlock, _ := pem.Decode([]byte(certificate))
 		if certBlock == nil {
@@ -139,12 +137,31 @@ func (d *Daemon) httpGetSync(url string, certificate string) (*lxd.Response, err
 		Transport: tr,
 	}
 
+	// Setup redirect policy
+	myhttp.CheckRedirect = func(req *http.Request, via []*http.Request) error {
+		// Replicate the headers
+		req.Header = via[len(via)-1].Header
+
+		return nil
+	}
+
+	return &myhttp, nil
+}
+
+func (d *Daemon) httpGetSync(url string, certificate string) (*api.Response, error) {
+	var err error
+
 	req, err := http.NewRequest("GET", url, nil)
 	if err != nil {
 		return nil, err
 	}
 
-	req.Header.Set("User-Agent", shared.UserAgent)
+	req.Header.Set("User-Agent", version.UserAgent)
+
+	myhttp, err := d.httpClient(certificate)
+	if err != nil {
+		return nil, err
+	}
 
 	r, err := myhttp.Do(req)
 	if err != nil {
@@ -156,7 +173,7 @@ func (d *Daemon) httpGetSync(url string, certificate string) (*lxd.Response, err
 		return nil, err
 	}
 
-	if resp.Type != lxd.Sync {
+	if resp.Type != api.SyncResponse {
 		return nil, fmt.Errorf("unexpected non-sync response")
 	}
 
@@ -166,40 +183,17 @@ func (d *Daemon) httpGetSync(url string, certificate string) (*lxd.Response, err
 func (d *Daemon) httpGetFile(url string, certificate string) (*http.Response, error) {
 	var err error
 
-	var cert *x509.Certificate
-	if certificate != "" {
-		certBlock, _ := pem.Decode([]byte(certificate))
-		if certBlock == nil {
-			return nil, fmt.Errorf("Invalid certificate")
-		}
-
-		cert, err = x509.ParseCertificate(certBlock.Bytes)
-		if err != nil {
-			return nil, err
-		}
-	}
-
-	tlsConfig, err := shared.GetTLSConfig("", "", "", cert)
+	myhttp, err := d.httpClient(certificate)
 	if err != nil {
 		return nil, err
 	}
 
-	tr := &http.Transport{
-		TLSClientConfig:   tlsConfig,
-		Dial:              shared.RFC3493Dialer,
-		Proxy:             d.proxy,
-		DisableKeepAlives: true,
-	}
-	myhttp := http.Client{
-		Transport: tr,
-	}
-
 	req, err := http.NewRequest("GET", url, nil)
 	if err != nil {
 		return nil, err
 	}
 
-	req.Header.Set("User-Agent", shared.UserAgent)
+	req.Header.Set("User-Agent", version.UserAgent)
 
 	raw, err := myhttp.Do(req)
 	if err != nil {
@@ -207,7 +201,7 @@ func (d *Daemon) httpGetFile(url string, certificate string) (*http.Response, er
 	}
 
 	if raw.StatusCode != 200 {
-		_, err := lxd.HoistResponse(raw, lxd.Error)
+		_, err := lxd.HoistResponse(raw, api.ErrorResponse)
 		if err != nil {
 			return nil, err
 		}
@@ -362,37 +356,53 @@ func (d *Daemon) createCmd(version string, c Command) {
 }
 
 func (d *Daemon) SetupStorageDriver() error {
-	var err error
-
-	lvmVgName := daemonConfig["storage.lvm_vg_name"].Get()
-	zfsPoolName := daemonConfig["storage.zfs_pool_name"].Get()
-
-	if lvmVgName != "" {
-		d.Storage, err = newStorage(d, storageTypeLvm)
-		if err != nil {
-			shared.LogErrorf("Could not initialize storage type LVM: %s - falling back to dir", err)
-		} else {
+	pools, err := dbStoragePools(d.db)
+	if err != nil {
+		if err == NoSuchObjectError {
+			shared.LogDebugf("No existing storage pools detected.")
 			return nil
 		}
-	} else if zfsPoolName != "" {
-		d.Storage, err = newStorage(d, storageTypeZfs)
+		shared.LogDebugf("Failed to retrieve existing storage pools.")
+		return err
+	}
+
+	for _, pool := range pools {
+		shared.LogDebugf("Initializing and checking storage pool \"%s\".", pool)
+		ps, err := storagePoolInit(d, pool)
 		if err != nil {
-			shared.LogErrorf("Could not initialize storage type ZFS: %s - falling back to dir", err)
-		} else {
-			return nil
+			shared.LogErrorf("Error initializing storage pool \"%s\": %s. Correct functionality of the storage pool cannot be guaranteed.", pool, err)
+			continue
 		}
-	} else if d.BackingFs == "btrfs" {
-		d.Storage, err = newStorage(d, storageTypeBtrfs)
+
+		err = ps.StoragePoolCheck()
 		if err != nil {
-			shared.LogErrorf("Could not initialize storage type btrfs: %s - falling back to dir", err)
-		} else {
-			return nil
+			shared.LogErrorf("Error checking storage pool \"%s\": %s. Correct functionality of the storage pool cannot be guaranteed.", pool, err)
 		}
 	}
 
-	d.Storage, err = newStorage(d, storageTypeDir)
+	// Get a list of all storage drivers currently in use
+	// on this LXD instance. Only do this when we do not already have done
+	// this once to avoid unnecessarily querying the db. All subsequent
+	// updates of the cache will be done when we create or delete storage
+	// pools in the db. Since this is a rare event, this cache
+	// implementation is a classic frequent-read, rare-update case so
+	// copy-on-write semantics without locking in the read case seems
+	// appropriate. (Should be cheaper then querying the db all the time,
+	// especially if we keep adding more storage drivers.)
+	if !storagePoolDriversCacheInitialized {
+		tmp, err := dbStoragePoolsGetDrivers(d.db)
+		if err != nil && err != NoSuchObjectError {
+			return nil
+		}
 
-	return err
+		storagePoolDriversCacheLock.Lock()
+		storagePoolDriversCacheVal.Store(tmp)
+		storagePoolDriversCacheLock.Unlock()
+
+		storagePoolDriversCacheInitialized = true
+	}
+
+	return nil
 }
 
 // have we setup shared mounts?
@@ -400,35 +410,28 @@ var sharedMounted bool
 var sharedMountsLock sync.Mutex
 
 func setupSharedMounts() error {
+	// Check if we already went through this
 	if sharedMounted {
 		return nil
 	}
 
+	// Get a lock to prevent races
 	sharedMountsLock.Lock()
 	defer sharedMountsLock.Unlock()
 
-	if sharedMounted {
-		return nil
-	}
-
+	// Check if already setup
 	path := shared.VarPath("shmounts")
-
-	isShared, err := shared.IsOnSharedMount(path)
-	if err != nil {
-		return err
-	}
-
-	if isShared {
-		// / may already be ms-shared, or shmounts may have
-		// been mounted by a previous lxd run
+	if shared.IsMountPoint(path) {
 		sharedMounted = true
 		return nil
 	}
 
-	if err := syscall.Mount(path, path, "none", syscall.MS_BIND, ""); err != nil {
+	// Mount a new tmpfs
+	if err := syscall.Mount("tmpfs", path, "tmpfs", 0, "size=100k,mode=0711"); err != nil {
 		return err
 	}
 
+	// Mark as MS_SHARED and MS_REC
 	var flags uintptr = syscall.MS_SHARED | syscall.MS_REC
 	if err := syscall.Mount(path, path, "none", flags, ""); err != nil {
 		return err
@@ -534,7 +537,7 @@ func (d *Daemon) UpdateHTTPsPort(newAddress string) error {
 			return fmt.Errorf("cannot listen on https socket: %v", err)
 		}
 
-		d.tomb.Go(func() error { return http.Serve(tcpl, d.mux) })
+		d.tomb.Go(func() error { return http.Serve(tcpl, &lxdHttpServer{d.mux, d}) })
 		d.TCPSocket = &Socket{Socket: tcpl, CloseOnExit: true}
 	}
 
@@ -554,8 +557,6 @@ func haveMacAdmin() bool {
 
 func (d *Daemon) Init() error {
 	/* Initialize some variables */
-	d.imagesDownloading = map[string]chan bool{}
-
 	d.readyChan = make(chan bool)
 	d.shutdownChan = make(chan bool)
 
@@ -576,13 +577,13 @@ func (d *Daemon) Init() error {
 
 	/* Print welcome message */
 	if d.MockMode {
-		shared.LogInfo("LXD is starting in mock mode",
+		shared.LogInfo(fmt.Sprintf("LXD %s is starting in mock mode", version.Version),
 			log.Ctx{"path": shared.VarPath("")})
 	} else if d.SetupMode {
-		shared.LogInfo("LXD is starting in setup mode",
+		shared.LogInfo(fmt.Sprintf("LXD %s is starting in setup mode", version.Version),
 			log.Ctx{"path": shared.VarPath("")})
 	} else {
-		shared.LogInfo("LXD is starting in normal mode",
+		shared.LogInfo(fmt.Sprintf("LXD %s is starting in normal mode", version.Version),
 			log.Ctx{"path": shared.VarPath("")})
 	}
 
@@ -724,18 +725,18 @@ func (d *Daemon) Init() error {
 	/* Get the list of supported architectures */
 	var architectures = []int{}
 
-	architectureName, err := shared.ArchitectureGetLocal()
+	architectureName, err := osarch.ArchitectureGetLocal()
 	if err != nil {
 		return err
 	}
 
-	architecture, err := shared.ArchitectureId(architectureName)
+	architecture, err := osarch.ArchitectureId(architectureName)
 	if err != nil {
 		return err
 	}
 	architectures = append(architectures, architecture)
 
-	personalities, err := shared.ArchitecturePersonalities(architecture)
+	personalities, err := osarch.ArchitecturePersonalities(architecture)
 	if err != nil {
 		return err
 	}
@@ -748,6 +749,12 @@ func (d *Daemon) Init() error {
 	d.lxcpath = shared.VarPath("containers")
 
 	/* Make sure all our directories are available */
+	if err := os.MkdirAll(shared.VarPath(), 0711); err != nil {
+		return err
+	}
+	if err := os.MkdirAll(shared.CachePath(), 0700); err != nil {
+		return err
+	}
 	if err := os.MkdirAll(shared.VarPath("containers"), 0711); err != nil {
 		return err
 	}
@@ -772,6 +779,15 @@ func (d *Daemon) Init() error {
 	if err := os.MkdirAll(shared.VarPath("snapshots"), 0700); err != nil {
 		return err
 	}
+	if err := os.MkdirAll(shared.VarPath("networks"), 0711); err != nil {
+		return err
+	}
+	if err := os.MkdirAll(shared.VarPath("disks"), 0700); err != nil {
+		return err
+	}
+	if err := os.MkdirAll(shared.VarPath("storage-pools"), 0711); err != nil {
+		return err
+	}
 
 	/* Detect the filesystem */
 	d.BackingFs, err = filesystemDetect(d.lxcpath)
@@ -804,10 +820,10 @@ func (d *Daemon) Init() error {
 	}
 
 	if !d.MockMode {
-		/* Setup the storage driver */
+		/* Read the storage pools */
 		err = d.SetupStorageDriver()
 		if err != nil {
-			return fmt.Errorf("Failed to setup storage: %s", err)
+			return err
 		}
 
 		/* Apply all patches */
@@ -821,6 +837,12 @@ func (d *Daemon) Init() error {
 		if err != nil {
 			return err
 		}
+
+		/* Restore simplestreams cache */
+		err = imageLoadStreamCache(d)
+		if err != nil {
+			return err
+		}
 	}
 
 	/* Log expiry */
@@ -846,6 +868,17 @@ func (d *Daemon) Init() error {
 		daemonConfig["core.proxy_ignore_hosts"].Get(),
 	)
 
+	/* Setup some mounts (nice to have) */
+	if !d.MockMode {
+		// Attempt to mount the shmounts tmpfs
+		setupSharedMounts()
+
+		// Attempt to Mount the devlxd tmpfs
+		if !shared.IsMountPoint(shared.VarPath("devlxd")) {
+			syscall.Mount("tmpfs", shared.VarPath("devlxd"), "tmpfs", 0, "size=100k,mode=0755")
+		}
+	}
+
 	/* Setup /dev/lxd */
 	shared.LogInfof("Starting /dev/lxd handler")
 	d.devlxd, err = createAndBindDevLxd()
@@ -869,11 +902,10 @@ func (d *Daemon) Init() error {
 		}
 
 		tlsConfig := &tls.Config{
-			InsecureSkipVerify: true,
-			ClientAuth:         tls.RequestClientCert,
-			Certificates:       []tls.Certificate{cert},
-			MinVersion:         tls.VersionTLS12,
-			MaxVersion:         tls.VersionTLS12,
+			ClientAuth:   tls.RequestClientCert,
+			Certificates: []tls.Certificate{cert},
+			MinVersion:   tls.VersionTLS12,
+			MaxVersion:   tls.VersionTLS12,
 			CipherSuites: []uint16{
 				tls.TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256,
 				tls.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA},
@@ -1060,8 +1092,13 @@ func (d *Daemon) Ready() error {
 	/* Auto-update images */
 	d.resetAutoUpdateChan = make(chan bool)
 	go func() {
-		autoUpdateImages(d)
+		// Initial image sync
+		interval := daemonConfig["images.auto_update_interval"].GetInt64()
+		if interval > 0 {
+			autoUpdateImages(d)
+		}
 
+		// Background image sync
 		for {
 			interval := daemonConfig["images.auto_update_interval"].GetInt64()
 			if interval > 0 {
@@ -1084,7 +1121,7 @@ func (d *Daemon) Ready() error {
 	}()
 
 	/* Restore containers */
-	go containersRestart(d)
+	containersRestart(d)
 
 	/* Re-balance in case things changed while LXD was down */
 	deviceTaskBalance(d)
@@ -1149,22 +1186,27 @@ func (d *Daemon) Stop() error {
 		}
 	}
 
-	if n, err := d.numRunningContainers(); err != nil || n == 0 {
-		shared.LogInfof("Unmounting shmounts")
+	shared.LogInfof("Stopping /dev/lxd handler")
+	d.devlxd.Close()
+	shared.LogInfof("Stopped /dev/lxd handler")
 
+	if n, err := d.numRunningContainers(); err != nil || n == 0 {
+		shared.LogInfof("Unmounting temporary filesystems")
+
+		syscall.Unmount(shared.VarPath("devlxd"), syscall.MNT_DETACH)
 		syscall.Unmount(shared.VarPath("shmounts"), syscall.MNT_DETACH)
 
-		shared.LogInfof("Done unmounting shmounts")
+		shared.LogInfof("Done unmounting temporary filesystems")
 	} else {
-		shared.LogDebugf("Not unmounting shmounts (containers are still running)")
+		shared.LogDebugf("Not unmounting temporary filesystems (containers are still running)")
 	}
 
 	shared.LogInfof("Closing the database")
 	d.db.Close()
 
-	shared.LogInfof("Stopping /dev/lxd handler")
-	d.devlxd.Close()
-	shared.LogInfof("Stopped /dev/lxd handler")
+	shared.LogInfof("Saving simplestreams cache")
+	imageSaveStreamCache()
+	shared.LogInfof("Saved simplestreams cache")
 
 	if d.MockMode || forceStop {
 		return nil
