@@ -2,7 +2,6 @@ package txn
 
 import (
 	"fmt"
-	"sort"
 
 	"gopkg.in/mgo.v2"
 	"gopkg.in/mgo.v2/bson"
@@ -225,10 +224,9 @@ func (f *flusher) prepare(t *transaction, force bool) (revnos []int64, err error
 	}
 	f.debugf("Preparing %s", t)
 
-	// Iterate in a stable way across all runners. This isn't
-	// strictly required, but reduces the chances of cycles.
+	// dkeys being sorted means stable iteration across all runners. This
+	// isn't strictly required, but reduces the chances of cycles.
 	dkeys := t.docKeys()
-	sort.Sort(dkeys)
 
 	revno := make(map[docKey]int64)
 	info := txnInfo{}
@@ -316,7 +314,7 @@ NextDoc:
 			}
 		}
 
-		// The stash wasn't valid and tt got overwriten. Try again.
+		// The stash wasn't valid and tt got overwritten. Try again.
 		f.unstashToken(tt, dkey)
 		goto RetryDoc
 	}
@@ -380,10 +378,10 @@ func (f *flusher) rescan(t *transaction, force bool) (revnos []int64, err error)
 		panic(fmt.Errorf("rescanning transaction in invalid state: %q", t.State))
 	}
 
-	// Iterate in a stable way across all runners. This isn't
-	// strictly required, but reduces the chances of cycles.
+	// dkeys being sorted means stable iteration across all
+	// runners. This isn't strictly required, but reduces the chances
+	// of cycles.
 	dkeys := t.docKeys()
-	sort.Sort(dkeys)
 
 	tt := t.token()
 	if !force {
@@ -397,12 +395,15 @@ func (f *flusher) rescan(t *transaction, force bool) (revnos []int64, err error)
 	revno := make(map[docKey]int64)
 	info := txnInfo{}
 	for _, dkey := range dkeys {
-		retry := 0
+		const retries = 3
+		retry := -1
 
 	RetryDoc:
+		retry++
 		c := f.tc.Database.C(dkey.C)
 		if err := c.FindId(dkey.Id).Select(txnFields).One(&info); err == mgo.ErrNotFound {
 			// Document is missing. Look in stash.
+			chaos("")
 			if err := f.sc.FindId(dkey).One(&info); err == mgo.ErrNotFound {
 				// Stash also doesn't exist. Maybe someone applied it.
 				if err := f.reload(t); err != nil {
@@ -411,8 +412,7 @@ func (f *flusher) rescan(t *transaction, force bool) (revnos []int64, err error)
 					return t.Revnos, err
 				}
 				// Not applying either.
-				retry++
-				if retry < 3 {
+				if retry < retries {
 					// Retry since there might be an insert/remove race.
 					goto RetryDoc
 				}
@@ -453,13 +453,28 @@ func (f *flusher) rescan(t *transaction, force bool) (revnos []int64, err error)
 		}
 		f.queue[dkey] = info.Queue
 		if !found {
-			// Previously set txn-queue was popped by someone.
-			// Transaction is being/has been applied elsewhere.
+			// Rescanned transaction id was not in the queue. This could mean one
+			// of three things:
+			//  1) The transaction was applied and popped by someone else. This is
+			//     the common case.
+			//  2) We've read an out-of-date queue from the stash. This can happen
+			//     when someone else was paused for a long while preparing another
+			//     transaction for this document, and improperly upserted to the
+			//     stash when unpaused (after someone else inserted the document).
+			//     This is rare but possible.
+			//  3) There's an actual bug somewhere, or outside interference. Worst
+			//     possible case.
 			f.debugf("Rescanned document %v misses %s in queue: %v", dkey, tt, info.Queue)
 			err := f.reload(t)
 			if t.State == tpreparing || t.State == tprepared {
-				panic("rescanned document misses transaction in queue")
+				if retry < retries {
+					// Case 2.
+					goto RetryDoc
+				}
+				// Case 3.
+				return nil, fmt.Errorf("cannot find transaction %s in queue for document %v", t, dkey)
 			}
+			// Case 1.
 			return t.Revnos, err
 		}
 	}
@@ -676,7 +691,7 @@ func (f *flusher) checkpoint(t *transaction, revnos []int64) error {
 		f.debugf("Ready to apply %s. Saving revnos %v: LOST RACE", t, debugRevnos)
 		return f.reload(t)
 	}
-	return nil
+	return err
 }
 
 func (f *flusher) apply(t *transaction, pull map[bson.ObjectId]*transaction) error {
